@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id:  qt/biff.cpp   3.0.5   edited Oct 12 2001 $
+** $Id: qt/biff.cpp   3.2.3   edited Jul 2 11:38 $
 **
 ** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
 **
@@ -20,7 +20,7 @@
 
 
 Biff::Biff( QWidget *parent, const char *name )
-    : QWidget( parent, name, WType_Modal )
+    : QWidget( parent, name, WShowModal | WType_Dialog )
 {
     QFileInfo fi = QString(getenv( "MAIL" ));
     if ( !fi.exists() ) {
