@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id:  qt/buttongroups.cpp   3.0.5   edited Jun 4 09:54 $
+** $Id: qt/buttongroups.cpp   3.2.3   edited May 13 09:08 $
 **
 ** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
 **
@@ -10,6 +10,7 @@
 
 #include "buttongroups.h"
 
+#include <qpopupmenu.h>
 #include <qbuttongroup.h>
 #include <qlayout.h>
 #include <qradiobutton.h>
@@ -34,7 +35,7 @@ ButtonsGroups::ButtonsGroups( QWidget *parent, const char *name )
     // ------- first group
 
     // Create an exclusive button group
-    QButtonGroup *bgrp1 = new QButtonGroup( 1, QGroupBox::Horizontal, "Button Group &1 (exclusive)", this);
+    QButtonGroup *bgrp1 = new QButtonGroup( 1, QGroupBox::Horizontal, "Button Group 1 (exclusive)", this);
     box1->addWidget( bgrp1 );
     bgrp1->setExclusive( TRUE );
 
@@ -47,7 +48,7 @@ ButtonsGroups::ButtonsGroups( QWidget *parent, const char *name )
     // ------- second group
 
     // Create a non-exclusive buttongroup
-    QButtonGroup *bgrp2 = new QButtonGroup( 1, QGroupBox::Horizontal, "Button Group &2 (non-exclusive)", this );
+    QButtonGroup *bgrp2 = new QButtonGroup( 1, QGroupBox::Horizontal, "Button Group 2 (non-exclusive)", this );
     box1->addWidget( bgrp2 );
     bgrp2->setExclusive( FALSE );
 
@@ -62,7 +63,7 @@ ButtonsGroups::ButtonsGroups( QWidget *parent, const char *name )
     // ------------ third group
 
     // create a buttongroup which is exclusive for radiobuttons and non-exclusive for all other buttons
-    QButtonGroup *bgrp3 = new QButtonGroup( 1, QGroupBox::Horizontal, "Button Group &3 (Radiobutton-exclusive)", this );
+    QButtonGroup *bgrp3 = new QButtonGroup( 1, QGroupBox::Horizontal, "Button Group 3 (Radiobutton-exclusive)", this );
     box2->addWidget( bgrp3 );
     bgrp3->setRadioButtonExclusive( TRUE );
 
@@ -81,20 +82,29 @@ ButtonsGroups::ButtonsGroups( QWidget *parent, const char *name )
     // ------------ fourth group
 
     // create a groupbox which layouts its childs in a columns
-    QGroupBox *bgrp4 = new QButtonGroup( 1, QGroupBox::Horizontal, "Groupbox with &normal buttons", this );
+    QGroupBox *bgrp4 = new QButtonGroup( 1, QGroupBox::Horizontal, "Groupbox with normal buttons", this );
     box2->addWidget( bgrp4 );
 
-    // insert three pushbuttons...
-    (void)new QPushButton( "&Push Button", bgrp4 );
-    QPushButton *tb2 = new QPushButton( "&Toggle Button", bgrp4 );
-    QPushButton *tb3 = new QPushButton( "&Flat Button", bgrp4 );
+    // insert four pushbuttons...
+    (void)new QPushButton( "&Push Button", bgrp4, "push" );
 
-    // ... and make the second one a toggle button
+    // now make the second one a toggle button
+    QPushButton *tb2 = new QPushButton( "&Toggle Button", bgrp4, "toggle" );
     tb2->setToggleButton( TRUE );
     tb2->setOn( TRUE );
 
     // ... and make the third one a flat button
+    QPushButton *tb3 = new QPushButton( "&Flat Button", bgrp4, "flat" );
     tb3->setFlat(TRUE);
+
+    // .. and the fourth a button with a menu
+    QPushButton *tb4 = new QPushButton( "Popup Button", bgrp4, "popup" );
+    QPopupMenu *menu = new QPopupMenu(tb4);
+    menu->insertItem("Item1", 0);
+    menu->insertItem("Item2", 1);
+    menu->insertItem("Item3", 2);
+    menu->insertItem("Item4", 3);
+    tb4->setPopup(menu);
 }
 
 /*
