@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id:  qt/frame.h   3.0.3   edited Oct 31 22:39 $
+** $Id: qt/frame.h   3.2.3   edited May 13 09:08 $
 **
 ** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
 **
@@ -9,31 +9,38 @@
 *****************************************************************************/
 
 #include <qmainwindow.h>
+#include <qintdict.h>
+#include "categoryinterface.h"
 
+class QToolBox;
 class QStyle;
-class QListBox;
-class QListBoxItem;
 class QWidgetStack;
 
-class Frame : public QMainWindow {
+class Frame : public QMainWindow
+{
     Q_OBJECT
+
 public:
     Frame( QWidget *parent=0, const char *name=0 );
-
-    void addCategory( QWidget *w, const QPixmap &p, const QString &n );
-    void addCategory( QWidget *w, const QPixmap &p1, const QPixmap &p2, const QString &n );
+    void setCategories( const QPtrList<CategoryInterface> &l );
 
     static void updateTranslators();
 
+    QWidgetStack *widgetStack() const { return stack; }
+
 private slots:
     void setStyle( const QString& );
-    void clickedCategory( QListBoxItem * );
+
+protected:
+    bool event( QEvent *e );
 
 private:
-    QWidget *createCategory( const QString& );
-    int idEnglish;
+    QWidget *createCategoryPage( CategoryInterface *c );
 
-    QString title;
-    QListBox *categories;
+private:
+    QToolBox *toolBox;
     QWidgetStack *stack;
+    QIntDict<QWidget> categoryPages;
+    QPtrList<CategoryInterface> categories;
+
 };
