@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id:  qt/dirview.h   3.0.5   edited Oct 12 2001 $
+** $Id: qt/dirview.h   3.2.3   edited May 13 09:08 $
 **
 ** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
 **
@@ -30,6 +30,9 @@ public:
 	: QListViewItem( parent, s1, s2 ), pix( 0 ) {}
 
     const QPixmap *pixmap( int i ) const;
+#if !defined(Q_NO_USING_KEYWORD)
+    using QListViewItem::setPixmap;
+#endif
     void setPixmap( QPixmap *p );
 
 private:
@@ -53,6 +56,9 @@ public:
     void setup();
 
     const QPixmap *pixmap( int i ) const;
+#if !defined(Q_NO_USING_KEYWORD)
+    using QListViewItem::setPixmap;
+#endif
     void setPixmap( QPixmap *p );
 
 private:
