@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id:  qt/dropsite.cpp   3.0.5   edited Oct 12 2001 $
+** $Id: qt/dropsite.cpp   3.2.3   edited May 30 03:10 $
 **
 ** Drop site example implementation
 **
@@ -33,10 +33,11 @@ DropSite::~DropSite()
 }
 
 
-void DropSite::dragMoveEvent( QDragMoveEvent * /*e*/ )
+void DropSite::dragMoveEvent( QDragMoveEvent *e )
 {
     // Check if you want the drag at e->pos()...
-    // Give the user some feedback...
+    // Give the user some feedback - only copy is possible
+    e->acceptAction( e->action() == QDropEvent::Copy );
 }
 
 
@@ -54,11 +55,12 @@ void DropSite::dragEnterEvent( QDragEnterEvent *e )
 
     // Give the user some feedback...
     QString t;
-    for( int i=0; e->format( i ); i++ ) {
-	if ( *(e->format( i )) ) {
+    const char *f;
+    for( int i=0; (f=e->format( i )); i++ ) {
+	if ( *(f) ) {
 	    if ( !t.isEmpty() )
 		t += "\n";
-	    t += e->format( i );
+	    t += f;
 	}
     }
     emit message( t );
@@ -78,21 +80,6 @@ void DropSite::dropEvent( QDropEvent * e )
     setBackgroundColor(lightGray);
 
     // Try to decode to the data you understand...
-
-    QString str;
-    if ( QTextDrag::decode( e, str ) ) {
-	setText( str );
-	setMinimumSize( minimumSize().expandedTo( sizeHint() ) );
-	return;
-    }
-
-    QPixmap pm;
-    if ( QImageDrag::decode( e, pm ) ) {
-	setPixmap( pm );
-	setMinimumSize( minimumSize().expandedTo( sizeHint() ) );
-	return;
-    }
-
     QStrList strings;
     if ( QUriDrag::decode( e, strings ) ) {
 	QString m("Full URLs:\n");
@@ -109,6 +96,20 @@ void DropSite::dropEvent( QDropEvent * e )
 	return;
     }
 
+    QString str;
+    if ( QTextDrag::decode( e, str ) ) {
+	setText( str );
+	setMinimumSize( minimumSize().expandedTo( sizeHint() ) );
+	return;
+    }
+
+    QPixmap pm;
+    if ( QImageDrag::decode( e, pm ) ) {
+	setPixmap( pm );
+	setMinimumSize( minimumSize().expandedTo( sizeHint() ) );
+	return;
+    }
+
     if ( SecretDrag::decode( e, str ) ) {
 	setText( str );
 	setMinimumSize( minimumSize().expandedTo( sizeHint() ) );
