@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id:  qt/drawdemo.cpp   3.0.5   edited Oct 12 2001 $
+** $Id: qt/drawdemo.cpp   3.2.3   edited Jun 18 16:07 $
 **
 ** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
 **
@@ -268,7 +268,9 @@ void DrawView::drawIt( QPainter *p )
 void DrawView::printIt()
 {
     if ( printer->setup( this ) ) {
-	QPainter paint( printer );
+	QPainter paint;
+	if( !paint.begin( printer ) )               
+	    return;
         drawIt( &paint );
     }
 }
