@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id:  qt/qfileiconview.cpp   3.0.5   edited Oct 12 2001 $
+** $Id: qt/qfileiconview.cpp   3.2.3   edited Oct 28 15:40 $
 **
 ** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
 **
@@ -416,8 +416,8 @@ void QtFileIconViewItem::dropped( QDropEvent *e, const QValueList<QIconDragItem>
 	return;
     }
 
-    QStrList lst;
-    QUriDrag::decode( e, lst );
+    QStringList lst;
+    QUriDrag::decodeLocalFiles( e, lst );
 
     QString str;
     if ( e->action() == QDropEvent::Copy )
@@ -425,10 +425,10 @@ void QtFileIconViewItem::dropped( QDropEvent *e, const QValueList<QIconDragItem>
     else
 	str = "Move\n\n";
     for ( uint i = 0; i < lst.count(); ++i )
-	str += QString( "   %1\n" ).arg( lst.at( i ) );
+	str += QString( "   %1\n" ).arg( lst[i] );
     str += QString( "\n"
-		    "To\n\n"
-		    "	%1" ).arg( filename() );
+	"To\n\n"
+	"	%1" ).arg( filename() );
 
     QMessageBox::information( iconView(), e->action() == QDropEvent::Copy ? "Copy" : "Move" , str, "Not Implemented" );
     if ( e->action() == QDropEvent::Move )
@@ -694,8 +694,8 @@ void QtFileIconView::slotDropped( QDropEvent *e, const QValueList<QIconDragItem>
 	return;
     }
 
-    QStrList lst;
-    QUriDrag::decode( e, lst );
+    QStringList lst;
+    QUriDrag::decodeLocalFiles( e, lst );
 
     QString str;
     if ( e->action() == QDropEvent::Copy )
@@ -703,14 +703,14 @@ void QtFileIconView::slotDropped( QDropEvent *e, const QValueList<QIconDragItem>
     else
 	str = "Move\n\n";
     for ( uint i = 0; i < lst.count(); ++i )
-	str += QString( "   %1\n" ).arg( lst.at( i ) );
+	str += QString( "   %1\n" ).arg( QDir::convertSeparators(lst[i]) );
     str += QString( "\n"
-		    "To\n\n"
-		    "	%1" ).arg( viewDir.absPath() );
+	"To\n\n"
+	"	%1" ).arg( viewDir.absPath() );
 
     QMessageBox::information( this, e->action() == QDropEvent::Copy ? "Copy" : "Move" , str, "Not Implemented" );
     if ( e->action() == QDropEvent::Move )
-	QMessageBox::information( this, "Remove" , str, "Not Implemented" );
+	QMessageBox::information( this, "Remove" , QDir::convertSeparators(lst.join("\n")), "Not Implemented" );
     e->acceptAction();
     openItem = 0;
 }
