@@ -1,7 +1,7 @@
 /****************************************************************************
-** $Id:  qt/helpwindow.cpp   3.0.5   edited May 27 19:35 $
+** $Id: qt/helpwindow.cpp   3.2.3   edited Jul 16 13:14 $
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2002 Trolltech AS.  All rights reserved.
 **
 ** This file is part of an example program for Qt.  This example
 ** program may be used, distributed and modified without limitation.
@@ -39,7 +39,7 @@
 HelpWindow::HelpWindow( const QString& home_, const QString& _path,
 			QWidget* parent, const char *name )
     : QMainWindow( parent, name, WDestructiveClose ),
-      pathCombo( 0 ), selectedURL()
+      pathCombo( 0 )
 {
     readHistory();
     readBookmarks();
@@ -48,8 +48,8 @@ HelpWindow::HelpWindow( const QString& home_, const QString& _path,
 
     browser->mimeSourceFactory()->setFilePath( _path );
     browser->setFrameStyle( QFrame::Panel | QFrame::Sunken );
-    connect( browser, SIGNAL( textChanged() ),
-	     this, SLOT( textChanged() ) );
+    connect( browser, SIGNAL( sourceChanged(const QString& ) ),
+	     this, SLOT( sourceChanged( const QString&) ) );
 
     setCentralWidget( browser );
 
@@ -84,8 +84,8 @@ HelpWindow::HelpWindow( const QString& home_, const QString& _path,
     go->insertItem( icon_home, tr("&Home"), browser, SLOT( home() ) );
 
     QPopupMenu* help = new QPopupMenu( this );
-    help->insertItem( tr("&About ..."), this, SLOT( about() ) );
-    help->insertItem( tr("About &Qt ..."), this, SLOT( aboutQt() ) );
+    help->insertItem( tr("&About"), this, SLOT( about() ) );
+    help->insertItem( tr("About &Qt"), this, SLOT( aboutQt() ) );
 
     hist = new QPopupMenu( this );
     QStringList::Iterator it = history.begin();
@@ -158,40 +158,34 @@ void HelpWindow::setForwardAvailable( bool b)
 }
 
 
-void HelpWindow::textChanged()
+void HelpWindow::sourceChanged( const QString& url )
 {
     if ( browser->documentTitle().isNull() )
-	setCaption( "Qt Example - Helpviewer - " + browser->context() );
+	setCaption( "Qt Example - Helpviewer - " + url );
     else
 	setCaption( "Qt Example - Helpviewer - " + browser->documentTitle() ) ;
 
-    selectedURL = browser->context();
-
-    if ( !selectedURL.isEmpty() && pathCombo ) {
+    if ( !url.isEmpty() && pathCombo ) {
 	bool exists = FALSE;
 	int i;
 	for ( i = 0; i < pathCombo->count(); ++i ) {
-	    if ( pathCombo->text( i ) == selectedURL ) {
+	    if ( pathCombo->text( i ) == url ) {
 		exists = TRUE;
 		break;
 	    }
 	}
 	if ( !exists ) {
-	    pathCombo->insertItem( selectedURL, 0 );
+	    pathCombo->insertItem( url, 0 );
 	    pathCombo->setCurrentItem( 0 );
-	    mHistory[ hist->insertItem( selectedURL ) ] = selectedURL;
+	    mHistory[ hist->insertItem( url ) ] = url;
 	} else
 	    pathCombo->setCurrentItem( i );
-	selectedURL = QString::null;
     }
 }
 
 HelpWindow::~HelpWindow()
 {
-    history.clear();
-    QMap<int, QString>::Iterator it = mHistory.begin();
-    for ( ; it != mHistory.end(); ++it )
-	history.append( *it );
+    history =  mHistory.values();
 
     QFile f( QDir::currentDirPath() + "/.history" );
     f.open( IO_WriteOnly );
@@ -199,10 +193,7 @@ HelpWindow::~HelpWindow()
     s << history;
     f.close();
 
-    bookmarks.clear();
-    QMap<int, QString>::Iterator it2 = mBookmarks.begin();
-    for ( ; it2 != mBookmarks.end(); ++it2 )
-	bookmarks.append( *it2 );
+    bookmarks = mBookmarks.values();
 
     QFile f2( QDir::currentDirPath() + "/.bookmarks" );
     f2.open( IO_WriteOnly );
@@ -216,7 +207,7 @@ void HelpWindow::about()
     QMessageBox::about( this, "HelpViewer Example",
 			"<p>This example implements a simple HTML help viewer "
 			"using Qt's rich text capabilities</p>"
-			"<p>It's just about 100 lines of C++ code, so don't expect too much :-)</p>"
+			"<p>It's just about 400 lines of C++ code, so don't expect too much :-)</p>"
 			);
 }
 
@@ -243,22 +234,21 @@ void HelpWindow::newWindow()
 void HelpWindow::print()
 {
 #ifndef QT_NO_PRINTER
-    QPrinter printer;
+    QPrinter printer( QPrinter::HighResolution );
     printer.setFullPage(TRUE);
     if ( printer.setup( this ) ) {
 	QPainter p( &printer );
+	if( !p.isActive() ) // starting printing failed
+	    return;
 	QPaintDeviceMetrics metrics(p.device());
-	int dpix = metrics.logicalDpiX();
 	int dpiy = metrics.logicalDpiY();
-	const int margin = 72; // pt
-	QRect body(margin*dpix/72, margin*dpiy/72,
-		   metrics.width()-margin*dpix/72*2,
- 		   metrics.height()-margin*dpiy/72*2 );
-	QFont font( "times", 10 );
-	QSimpleRichText richText( browser->text(), font,
-				  browser->context(), 
+	int margin = (int) ( (2/2.54)*dpiy ); // 2 cm margins
+	QRect body( margin, margin, metrics.width() - 2*margin, metrics.height() - 2*margin );
+	QSimpleRichText richText( browser->text(),
+				  QFont(),
+				  browser->context(),
 				  browser->styleSheet(),
-				  browser->mimeSourceFactory(), 
+				  browser->mimeSourceFactory(),
 				  body.height() );
 	richText.setWidth( &p, body.width() );
 	QRect view( body );
@@ -267,10 +257,9 @@ void HelpWindow::print()
 	    richText.draw( &p, body.left(), body.top(), view, colorGroup() );
 	    view.moveBy( 0, body.height() );
 	    p.translate( 0 , -body.height() );
-	    p.setFont( font );
 	    p.drawText( view.right() - p.fontMetrics().width( QString::number(page) ),
 			view.bottom() + p.fontMetrics().ascent() + 5, QString::number(page) );
-	    if ( view.top()  >= body.top() + richText.height() )
+	    if ( view.top()  >= richText.height() )
 		break;
 	    printer.newPage();
 	    page++;
@@ -282,15 +271,7 @@ void HelpWindow::print()
 void HelpWindow::pathSelected( const QString &_path )
 {
     browser->setSource( _path );
-    QMap<int, QString>::Iterator it = mHistory.begin();
-    bool exists = FALSE;
-    for ( ; it != mHistory.end(); ++it ) {
-	if ( *it == _path ) {
-	    exists = TRUE;
-	    break;
-	}
-    }
-    if ( !exists )
+    if ( mHistory.values().contains(_path) )
 	mHistory[ hist->insertItem( _path ) ] = _path;
 }
 
