@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id:  qt/helpwindow.h   3.0.2   edited Oct 12 12:18 $
+** $Id: qt/helpwindow.h   3.2.3   edited May 13 09:08 $
 **
 ** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
 **
@@ -31,7 +31,7 @@ private slots:
     void setBackwardAvailable( bool );
     void setForwardAvailable( bool );
 
-    void textChanged();
+    void sourceChanged( const QString& );
     void about();
     void aboutQt();
     void openFile();
@@ -42,19 +42,18 @@ private slots:
     void histChosen( int );
     void bookmChosen( int );
     void addBookmark();
-    
+
 private:
     void readHistory();
     void readBookmarks();
-    
+
     QTextBrowser* browser;
     QComboBox *pathCombo;
     int backwardId, forwardId;
-    QString selectedURL;
     QStringList history, bookmarks;
     QMap<int, QString> mHistory, mBookmarks;
     QPopupMenu *hist, *bookm;
-    
+
 };
 
 
