@@ -1,7 +1,7 @@
 /****************************************************************************
-** $Id:  qt/lineedits.cpp   3.0.5   edited May 7 17:30 $
+** $Id: qt/lineedits.cpp   3.2.3   edited Jun 17 11:12 $
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2003 Trolltech AS.  All rights reserved.
 **
 ** This file is part of an example program for Qt.  This example
 ** program may be used, distributed and modified without limitation.
@@ -30,7 +30,7 @@ LineEdits::LineEdits( QWidget *parent, const char *name )
     setMargin( 10 );
 
     QVBoxLayout* box = new QVBoxLayout( layout() );
-    
+
     QHBoxLayout *row1 = new QHBoxLayout( box );
     row1->setMargin( 5 );
 
@@ -41,9 +41,9 @@ LineEdits::LineEdits( QWidget *parent, const char *name )
     // Create a Combobox with three items...
     combo1 = new QComboBox( FALSE, this );
     row1->addWidget( combo1 );
-    combo1->insertItem( "Normal", -1 );
-    combo1->insertItem( "Password", -1 );
-    combo1->insertItem( "No Echo", -1 );
+    combo1->insertItem( "Normal" );
+    combo1->insertItem( "Password" );
+    combo1->insertItem( "No Echo" );
     // ...and connect the activated() SIGNAL with the slotEchoChanged() SLOT to be able
     // to react when an item is selected
     connect( combo1, SIGNAL( activated( int ) ), this, SLOT( slotEchoChanged( int ) ) );
@@ -63,9 +63,9 @@ LineEdits::LineEdits( QWidget *parent, const char *name )
     // A second Combobox with again three items...
     combo2 = new QComboBox( FALSE, this );
     row2->addWidget( combo2 );
-    combo2->insertItem( "No Validator", -1 );
-    combo2->insertItem( "Integer Validator", -1 );
-    combo2->insertItem( "Double Validator", -1 );
+    combo2->insertItem( "No Validator" );
+    combo2->insertItem( "Integer Validator" );
+    combo2->insertItem( "Double Validator" );
     // ...and again the activated() SIGNAL gets connected with a SLOT
     connect( combo2, SIGNAL( activated( int ) ), this, SLOT( slotValidatorChanged( int ) ) );
 
@@ -84,9 +84,9 @@ LineEdits::LineEdits( QWidget *parent, const char *name )
     // A combo box for setting alignment
     combo3 = new QComboBox( FALSE, this );
     row3->addWidget( combo3 );
-    combo3->insertItem( "Left", -1 );
-    combo3->insertItem( "Centered", -1 );
-    combo3->insertItem( "Right", -1 );
+    combo3->insertItem( "Left" );
+    combo3->insertItem( "Centered" );
+    combo3->insertItem( "Right" );
     // ...and again the activated() SIGNAL gets connected with a SLOT
     connect( combo3, SIGNAL( activated( int ) ), this, SLOT( slotAlignmentChanged( int ) ) );
 
@@ -94,25 +94,49 @@ LineEdits::LineEdits( QWidget *parent, const char *name )
     lined3 = new QLineEdit( this );
     box->addWidget( lined3 );
 
-    // last widget used for layouting
-    QHBox *row4 = new QHBox( this );
-    box->addWidget( row4 );
+    // exactly the same for the fourth
+    QHBoxLayout *row4 = new QHBoxLayout( box );
     row4->setMargin( 5 );
 
-    // last label
-    (void)new QLabel( "Read-Only: ", row4 );
+    // we need a label for this too
+    label = new QLabel( "Input mask: ", this );
+    row4->addWidget( label );
 
-    // A combo box for setting alignment
-    combo4 = new QComboBox( FALSE, row4 );
-    combo4->insertItem( "False", -1 );
-    combo4->insertItem( "True", -1 );
-    // ...and again the activated() SIGNAL gets connected with a SLOT
-    connect( combo4, SIGNAL( activated( int ) ), this, SLOT( slotReadOnlyChanged( int ) ) );
+    // A combo box for choosing an input mask
+    combo4 = new QComboBox( FALSE, this );
+    row4->addWidget( combo4 );
+    combo4->insertItem( "No mask" );
+    combo4->insertItem( "Phone number" );
+    combo4->insertItem( "ISO date" );
+    combo4->insertItem( "License key" );
 
-    // and the last lineedit
+    // ...this time we use the activated( const QString & ) signal
+    connect( combo4, SIGNAL( activated( int ) ),
+	     this, SLOT( slotInputMaskChanged( int ) ) );
+
+    // and the fourth lineedit
     lined4 = new QLineEdit( this );
     box->addWidget( lined4 );
 
+    // last widget used for layouting
+    QHBox *row5 = new QHBox( this );
+    box->addWidget( row5 );
+    row5->setMargin( 5 );
+
+    // last label
+    (void)new QLabel( "Read-Only: ", row5 );
+
+    // A combo box for setting alignment
+    combo5 = new QComboBox( FALSE, row5 );
+    combo5->insertItem( "False" );
+    combo5->insertItem( "True" );
+    // ...and again the activated() SIGNAL gets connected with a SLOT
+    connect( combo5, SIGNAL( activated( int ) ), this, SLOT( slotReadOnlyChanged( int ) ) );
+
+    // and the last lineedit
+    lined5 = new QLineEdit( this );
+    box->addWidget( lined5 );
+
     // give the first LineEdit the focus at the beginning
     lined1->setFocus();
 }
@@ -197,6 +221,34 @@ void LineEdits::slotAlignmentChanged( int i )
     lined3->setFocus();
 }
 
+/*
+ * SLOT slotInputMaskChanged( const QString &mask )
+ *
+ * i contains the number of the item which the user has been chosen in
+ * the third Combobox.  According to this value, we set an input mask on
+ * third LineEdit.
+ */
+
+void LineEdits::slotInputMaskChanged( int i )
+{
+    switch( i ) {
+    case 0:
+	lined4->setInputMask( QString::null );
+	break;
+    case 1:
+	lined4->setInputMask( "+99 99 99 99 99;_" );
+	break;
+    case 2:
+	lined4->setInputMask( "0000-00-00" );
+	lined4->setText( "00000000" );
+	lined4->setCursorPosition( 0 );
+	break;
+    case 3:
+	lined4->setInputMask( ">AAAAA-AAAAA-AAAAA-AAAAA-AAAAA;#" );
+	break;
+    }
+    lined4->setFocus();
+}
 
 /*
  * SLOT slotReadOnlyChanged( int i )
@@ -209,12 +261,13 @@ void LineEdits::slotReadOnlyChanged( int i )
 {
     switch ( i ) {
     case 0:
-	lined4->setReadOnly( FALSE );
+	lined5->setReadOnly( FALSE );
         break;
     case 1:
-	lined4->setReadOnly( TRUE );
+	lined5->setReadOnly( TRUE );
         break;
     }
 
-    lined4->setFocus();
+    lined5->setFocus();
 }
+
