@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id:  qt/smtp.cpp   3.0.5   edited Apr 18 14:18 $
+** $Id: qt/smtp.cpp   3.2.3   edited May 13 09:08 $
 **
 ** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
 **
@@ -37,9 +37,9 @@ Smtp::Smtp( const QString &from, const QString &to,
 	      QString::fromLatin1( "\nTo: " ) + to +
 	      QString::fromLatin1( "\nSubject: " ) + subject +
 	      QString::fromLatin1( "\n\n" ) + body + "\n";
-    message.replace( QRegExp( QString::fromLatin1( "\n" ) ),
+    message.replace( QString::fromLatin1( "\n" ),
 		     QString::fromLatin1( "\r\n" ) );
-    message.replace( QRegExp( QString::fromLatin1( "\r\n.\r\n" ) ),
+    message.replace( QString::fromLatin1( "\r\n.\r\n" ),
 		     QString::fromLatin1( "\r\n..\r\n" ) );
 
     this->from = from;
@@ -59,8 +59,11 @@ Smtp::~Smtp()
 void Smtp::dnsLookupHelper()
 {
     QValueList<QDns::MailServer> s = mxLookup->mailServers();
-    if ( s.isEmpty() && mxLookup->isWorking() )
+    if ( s.isEmpty() ) {
+	if ( !mxLookup->isWorking() )
+	    emit status( tr( "Error in MX record lookup" ) );
 	return;
+    }
 
     emit status( tr( "Connecting to %1" ).arg( s.first().name ) );
 
@@ -74,7 +77,6 @@ void Smtp::connected()
     emit status( tr( "Connected to %1" ).arg( socket->peerName() ) );
 }
 
-
 void Smtp::readyRead()
 {
     // SMTP is line-oriented
@@ -111,7 +113,7 @@ void Smtp::readyRead()
 	state = Close;
 	emit status( tr( "Message sent" ) );
     } else if ( state == Close ) {
-	delete this;
+	deleteLater();
 	return;
     } else {
 	// something broke.
