@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id:  qt/qfd.cpp   3.0.5   edited Oct 12 2001 $
+** $Id: qt/qfd.cpp   3.2.3   edited May 13 09:08 $
 **
 ** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
 **
@@ -19,9 +19,6 @@
 
 int main(int argc, char** argv)
 {
-    // Use an interesting font
-    QApplication::setFont(QFont("unifont",16));
-
     QApplication app(argc,argv);
 
     FontDisplayer m;
