@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id:  qt/qvaluelistiterator.cpp   3.0.2   edited Oct 12 12:18 $
+** $Id: qt/qvaluelistiterator.cpp   3.2.3   edited May 13 09:08 $
 **
 ** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
 **
@@ -25,6 +25,10 @@ public:
 
     int salary() const { return s; }
     void setSalary( int salary ) { s = salary; }
+
+    // this is here to support very old compilers
+    Q_DUMMY_COMPARISON_OPERATOR( Employee )
+
 private:
     QString n;
     int s;
@@ -43,7 +47,7 @@ int main( int, char** )
     Employee joe( "Joe", 50000 );
     list.append( joe );
     joe.setSalary( 4000 );
-	
+
     EmployeeList::ConstIterator it = list.begin();
     while( it != list.end() ) {
 	printf( "%s earns %d\n", (*it).name().latin1(), (*it).salary() );
