@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id:  qt/statistics.cpp   3.0.5   edited Feb 18 11:55 $
+** $Id: qt/statistics.cpp   3.2.3   edited May 27 19:43 $
 **
 ** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
 **
@@ -49,14 +49,14 @@ void Table::initTable()
     // read all the Qt source and header files into a list
     QStringList all;
     int i = 0;
-    QString qtdir = getenv( "QTDIR" );
+    QString srcdir( "../../../src/" );
     while ( dirs[ i ] ) {
-	QDir dir( qtdir + "/src/" + dirs[ i ] );
+	QDir dir( srcdir + dirs[ i ] );
 	QStringList lst = dir.entryList( "*.cpp; *.h" );
 	for ( QStringList::Iterator it = lst.begin(); it != lst.end(); ++it ) {
 	    if ( ( *it ).contains( "moc" ) )
 		continue;
-	    all << QString( dirs[ i ] ) + "/" + *it;
+	    all << (QString( dirs[ i ] ) + "/" + *it);
 	}
 	++i;
     }
@@ -69,8 +69,8 @@ void Table::initTable()
     // insert the data into the table
     for ( QStringList::Iterator it = all.begin(); it != all.end(); ++it ) {
 	setText( i, 0, *it );
-	QFile f( qtdir + "/src/" + *it );
-	setText( i, 1, QString::number( f.size() ) );
+	QFile f( srcdir + *it );
+	setText( i, 1, QString::number( (ulong)f.size() ) );
 	ComboItem *ci = new ComboItem( this, QTableItem::WhenCurrent );
 	setItem( i++, 2, ci );
 	sum += f.size();
@@ -157,7 +157,7 @@ void ComboItem::setContentFromEditor( QWidget *w )
 	setText( ( (QComboBox*)w )->currentText() );
     else
 	QTableItem::setContentFromEditor( w );
-}	
+}
 
 void ComboItem::setText( const QString &s )
 {
@@ -167,6 +167,6 @@ void ComboItem::setText( const QString &s )
 	    cb->setCurrentItem( 1 );
 	else
 	    cb->setCurrentItem( 0 );
-    }	
+    }
     QTableItem::setText( s );
 }
