@@ -1,7 +1,7 @@
 /****************************************************************************
-** $Id:  qt/widgets.cpp   3.0.5   edited May 7 17:36 $
+** $Id: qt/widgets.cpp   3.2.3   edited May 26 16:16 $
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2003 Trolltech AS.  All rights reserved.
 **
 ** This file is part of an example program for Qt.  This example
 ** program may be used, distributed and modified without limitation.
@@ -44,7 +44,7 @@
 #include <qsplitter.h>
 #include <qlistview.h>
 #include <qheader.h>
-#include <qtextview.h>
+#include <qtextbrowser.h>
 #include <qfiledialog.h>
 #include <qaccel.h>
 #include <qmetaobject.h>
@@ -65,16 +65,6 @@
 #include "../application/filesave.xpm"
 #include "../application/fileprint.xpm"
 
-class MyLabel : public QLabel
-{
-public:
-    MyLabel( QWidget *parent = 0, const char *name = 0 )
-	: QLabel( parent, name ) { }
-
-    virtual QSize minimumSizeHint() const {
-	return QSize( 1, 1 );
-    }
-};
 
 class MyWhatsThis : public QWhatsThis
 {
@@ -189,7 +179,7 @@ WidgetView::WidgetView( QWidget *parent, const char *name )
     int id;
     id = popup->insertItem( "&New" );
     popup->setItemEnabled( id, FALSE );
-    id = popup->insertItem( openIcon, "&Open", this, SLOT( open() ) );
+    id = popup->insertItem( openIcon, "&Open...", this, SLOT( open() ) );
 
     popup->insertSeparator();
     popup->insertItem( "&Quit", qApp, SLOT(quit()), CTRL+Key_Q );
@@ -369,11 +359,13 @@ WidgetView::WidgetView( QWidget *parent, const char *name )
 
     // Create an editable combo box
     QComboBox *edCombo = new QComboBox( TRUE, central, "edComboBox" );
-    edCombo->insertItem( "Permutable" );
-    edCombo->insertItem( "Malleable" );
-    edCombo->insertItem( "Adaptable" );
-    edCombo->insertItem( "Alterable" );
-    edCombo->insertItem( "Inconstant" );
+    QListBox *edComboLst = new QListBox(this);
+    edCombo->setListBox(edComboLst);
+    edComboLst->insertItem( "Permutable" );
+    edComboLst->insertItem( "Malleable" );
+    edComboLst->insertItem( "Adaptable" );
+    edComboLst->insertItem( "Alterable" );
+    edComboLst->insertItem( "Inconstant" );
     vbox->addWidget( edCombo );
     connect( edCombo, SIGNAL(activated(const QString&)),
 	     this, SLOT(edComboBoxItemActivated(const QString&)) );
@@ -421,13 +413,13 @@ WidgetView::WidgetView( QWidget *parent, const char *name )
     mleText += "Japanese: ";
     mleText += QChar((ushort)0x6a38); // Kanji
     mleText += "\n";
-    mleText += "Russian:";
+    mleText += "Russian: ";
     mleText += QChar((ushort)0x042e); // Cyrillic
     mleText += "\n";
-    mleText += "Norwegian:";
+    mleText += "Norwegian: ";
     mleText += QChar((ushort)0x00d8); // Norwegian
     mleText += "\n";
-    mleText += "Unicode (black square):";
+    mleText += "Unicode (black square): ";
     mleText += QChar((ushort)0x25A0); // BLACK SQUARE
     mleText += "\n";
 #endif
@@ -510,18 +502,19 @@ WidgetView::WidgetView( QWidget *parent, const char *name )
     QWhatsThis::add( lv, "This is also a <b>QListView</b>, with "
 		     "interactive items." );
 
-     QTextView *qmlv =  new QTextView( "<hr><h1>QTextView</h1>"
-		      "<p>Qt supports formatted rich text, such "
-		      "as the heading above, <em>emphasized</em> and "
-		      "<b>bold</b> text, via an XML subset.</p> "
-		      "<p>Style sheets are supported.</p>",
- 				  "", split );
-    qmlv->setFont(QFont("Charter",11));
-    qmlv->setFrameStyle( QFrame::WinPanel | QFrame::Sunken );
+    QTextBrowser *browser =  new QTextBrowser( split );
+    browser->setText( "<h1>QTextBrowser</h1>"
+		   "<p>Qt supports formatted rich text, such "
+		   "as the heading above, <em>emphasized</em> and "
+		   "<b>bold</b> text, via an XML subset.</p> "
+		   "<p><a href=\"nogo://some.where.com\">Hypertext navigation</a> and style sheets are supported.</p>", "" );
+    browser->setFont(QFont("Charter",11));
+    browser->setFrameStyle( QFrame::WinPanel | QFrame::Sunken );
+    connect( browser, SIGNAL(linkClicked(const QString&)), browser, SLOT(setText(const QString&)) );
 
     // Create an label and a message in the status bar
     // The message is updated when buttons are clicked etc.
-    msg = new MyLabel( statusBar(), "message" );
+    msg = new QLabel( statusBar(), "message" );
     msg->setAlignment( AlignCenter );
     QFont boldfont; boldfont.setWeight(QFont::Bold);
     msg->setFont( boldfont );
