@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id:  qt/structureparser.cpp   3.0.2   edited Oct 12 12:18 $
+** $Id: qt/structureparser.cpp   3.2.3   edited Jun 12 17:58 $
 **
 ** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
 **
@@ -30,6 +30,6 @@ bool StructureParser::startElement( const QString&, const QString&,
 
 bool StructureParser::endElement( const QString&, const QString&, const QString& )
 {
-    indent.remove( 0, 4 );
+    indent.remove( (uint)0, 4 );
     return TRUE;
 }
