@@ -1,3 +1,3 @@
-#error "/home/reggie/tmp/qt-3.0-reggie-5401/qt/extensions/nsplugin/src/npunix.c must be provided by Netscape"
+#error "/home/espenr/tmp/qt-3.2-espenr-29145/qt/extensions/nsplugin/src/npunix.c must be provided by Netscape"
 #error "This file is just a placeholder. Please see the documentation"
 #error "to learn how to obtain the real file"
