@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id:  qt/qwidgetresizehandler_p.h   3.0.5   edited Oct 30 2001 $
+** $Id: qt/qwidgetresizehandler_p.h   3.2.3   edited May 13 09:08 $
 **
 ** Definition of the QWidgetResizeHandler class
 **
 ** Created : 001010
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2002 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the workspace module of the Qt GUI Toolkit.
 **
@@ -35,8 +35,20 @@
 **
 **********************************************************************/
 
-#ifndef QWIDGETRESIZEHANDLER_H
-#define QWIDGETRESIZEHANDLER_H
+#ifndef QWIDGETRESIZEHANDLER_P_H
+#define QWIDGETRESIZEHANDLER_P_H
+
+//
+//  W A R N I N G
+//  -------------
+//
+// This file is not part of the Qt API.  This header file may
+// change from version to version without notice, or even be
+// removed.
+//
+// We mean it.
+//
+//
 
 #ifndef QT_H
 #include "qobject.h"
@@ -50,9 +62,17 @@ class Q_EXPORT QWidgetResizeHandler : public QObject
     Q_OBJECT
 
 public:
+    enum Action {
+	Move	= 0x01,
+	Resize	= 0x02,
+	Any	= Move|Resize
+    };
+
     QWidgetResizeHandler( QWidget *parent, QWidget *cw = 0, const char *name = 0 );
-    void setActive( bool b ) { active = b; if ( !active ) setMouseCursor( Nowhere ); }
-    bool isActive() const { return active; }
+    void setActive( bool b ) { setActive( Any, b ); }
+    void setActive( Action ac, bool b );
+    bool isActive() const { return isActive( Any ); }
+    bool isActive( Action ac ) const;
     void setMovingEnabled( bool b ) { moving = b; }
     bool isMovingEnabled() const { return moving; }
 
@@ -89,9 +109,10 @@ private:
     int range;
     uint buttonDown	    :1;
     uint moveResizeMode	    :1;
-    uint active		    :1;
+    uint activeForResize    :1;
     uint sizeprotect	    :1;
     uint moving		    :1; 
+    uint activeForMove	    :1;
 
     void setMouseCursor( MousePosition m );
     bool isMove() const {
