@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id:  qt/qfontinfo.h   3.0.5   edited Oct 12 2001 $
+** $Id: qt/qfontinfo.h   3.2.3   edited May 13 09:08 $
 **
 ** Definition of QFontInfo class
 **
@@ -47,6 +47,7 @@ class Q_EXPORT QFontInfo
 {
 public:
     QFontInfo( const QFont & );
+    QFontInfo( const QFont &, QFont::Script );
     QFontInfo( const QFontInfo & );
     ~QFontInfo();
 
@@ -59,6 +60,7 @@ public:
     int			weight()	const;
     bool		bold()		const;
     bool		underline()	const;
+    bool                overline()      const;
     bool		strikeOut()	const;
     bool		fixedPitch()	const;
     QFont::StyleHint	styleHint()	const;
@@ -69,18 +71,10 @@ public:
 
 private:
     QFontInfo( const QPainter * );
-    static void reset( const QPainter * );
 
     QFontPrivate *d;
     QPainter *painter;
-    int flags;
-
-    bool    underlineFlag()  const { return (flags & 0x1) != 0; }
-    bool    strikeOutFlag()  const { return (flags & 0x2) != 0; }
-    bool    exactMatchFlag() const { return (flags & 0x4) != 0; }
-    void    setUnderlineFlag()	   { flags |= 0x1; }
-    void    setStrikeOutFlag()	   { flags |= 0x2; }
-    void    setExactMatchFlag()	   { flags |= 0x4; }
+    int fscript;
 
     friend class QWidget;
     friend class QPainter;
