@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id:  qt/qiodevice.h   3.0.5   edited Oct 12 2001 $
+** $Id: qt/qiodevice.h   3.2.3   edited Jun 1 18:37 $
 **
 ** Definition of QIODevice class
 **
 ** Created : 940913
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2002 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the tools module of the Qt GUI Toolkit.
 **
@@ -39,13 +39,6 @@
 #define QIODEVICE_H
 
 #ifndef QT_H
-#ifdef QT_LARGE_FILE_SUPPORT
-// ### Should be included first. This is a problem in the current
-// ### "qplatformdefs.h" strategy which is OK for source files but
-// ### not for header files. Do we need defines back in qmake.conf?
-#define _FILE_OFFSET_BITS 64
-#include "qplatformdefs.h"
-#endif
 #include "qglobal.h"
 #include "qcstring.h"
 #endif // QT_H
@@ -78,7 +71,6 @@
 #define IO_Open			0x1000		// device is open
 #define IO_StateMask		0xf000
 
-
 // IO device status
 
 #define IO_Ok			0
@@ -92,11 +84,12 @@
 #define IO_TimeOutError		7		// time out
 #define IO_UnspecifiedError	8		// unspecified error
 
-class Q_EXPORT QIODevice			// IO device class
+
+class Q_EXPORT QIODevice
 {
 public:
-#ifdef QT_LARGE_FILE_SUPPORT
-    typedef off_t Offset;
+#if defined(QT_ABI_QT4)
+    typedef QtOffset Offset;
 #else
     typedef Q_ULONG Offset;
 #endif
