@@ -1,13 +1,11 @@
 /****************************************************************************
-** $Id:  qt/metrowerks_xml.cpp   3.0.5   edited May 21 13:38 $
+** $Id: qt/metrowerks_xml.cpp   3.2.3   edited May 30 10:01 $
 **
-** Definition of ________ class.
+** Implementation of MetrowerksMakefileGenerator class.
 **
-** Created : 970521
+** Copyright (C) 1992-2003 Trolltech AS.  All rights reserved.
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
-**
-** This file is part of the network module of the Qt GUI Toolkit.
+** This file is part of qmake.
 **
 ** This file may be distributed under the terms of the Q Public License
 ** as defined by Trolltech AS of Norway and appearing in the file
@@ -42,13 +40,12 @@
 #include <qregexp.h>
 #include <stdlib.h>
 #include <time.h>
-#ifdef Q_OS_MAC
+#if !defined(QWS) && defined(Q_OS_MAC)
 #include <Carbon/Carbon.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #endif
 
-
 MetrowerksMakefileGenerator::MetrowerksMakefileGenerator(QMakeProject *p) : MakefileGenerator(p), init_flag(FALSE)
 {
 
@@ -90,13 +87,13 @@ MetrowerksMakefileGenerator::writeMakeParts(QTextStream &t)
 	extra_objs += project->variables()["QMAKE_LIBS"];
     for(QStringList::Iterator val_it = extra_objs.begin(); 
 	val_it != extra_objs.end(); ++val_it) {
-	if((*val_it).left(2) == "-L") {
+	if((*val_it).startsWith("-L")) {
 	    QString dir((*val_it).right((*val_it).length() - 2));
 	    fixEnvVariables(dir);
 	    if(project->variables()["DEPENDPATH"].findIndex(dir) == -1 &&
 	       project->variables()["INCLUDEPATH"].findIndex(dir) == -1)
 		project->variables()["INCLUDEPATH"].append(dir);
-	} else if((*val_it).left(2) == "-l") {
+	} else if((*val_it).startsWith("-l")) {
 	    QString lib("lib" + (*val_it).right((*val_it).length() - 2)  + "." + 
 			project->first("QMAKE_EXTENSION_SHLIB"));
 	    if(project->variables()["LIBRARIES"].findIndex(lib) == -1)
@@ -225,7 +222,7 @@ MetrowerksMakefileGenerator::writeMakeParts(QTextStream &t)
 				    debug = FALSE;
 				} else {
 				    for(QStringList::Iterator hit = Option::h_ext.begin(); hit != Option::h_ext.end(); ++hit) {
-					if((*it).right((*hit).length()) == (*hit)) { 
+					if((*it).endsWith((*hit))) { 
 					    debug = FALSE;
 					    break;
 					}
@@ -355,7 +352,7 @@ MetrowerksMakefileGenerator::writeMakeParts(QTextStream &t)
 		}
 		for(QStringList::Iterator it = list.begin(); it != list.end(); ++it) {
 		    QString p = (*it), v, recursive = "false", framework = "false";
-		    if(p.left(11) == "recursive--") {
+		    if(p.startsWith("recursive--")) {
 			p = p.right(p.length() - 11);
 			recursive = "true";
 		    } 
@@ -379,7 +376,7 @@ MetrowerksMakefileGenerator::writeMakeParts(QTextStream &t)
 	    } else if(variable == "CODEWARRIOR_WARNING" || variable == "!CODEWARRIOR_WARNING") {
 		bool b = ((!project->isActiveConfig("warn_off")) && 
 			  project->isActiveConfig("warn_on"));
-		if(variable.left(1) == "!")
+		if(variable.startsWith("!"))
 		    b = !b;
 		t << (int)b;
 	    } else if(variable == "CODEWARRIOR_TEMPLATE") {
@@ -542,8 +539,8 @@ MetrowerksMakefileGenerator::init()
 	project->variables()["VERSION"].append("1.0." + 
 					       (project->isEmpty("VER_PAT") ? QString("0") : 
 						project->first("VER_PAT")) );
-    QStringList ver = QStringList::split('.',  //make sure there are three
-					 project->first("VERSION")) << "0" << "0";
+    QStringList ver = QStringList::split('.', project->first("VERSION"));
+    ver << "0" << "0"; //make sure there are three
     project->variables()["VER_MAJ"].append(ver[0]);
     project->variables()["VER_MIN"].append(ver[1]);
     project->variables()["VER_PAT"].append(ver[2]);
@@ -607,7 +604,7 @@ MetrowerksMakefileGenerator::init()
 
 
 QString
-MetrowerksMakefileGenerator::findTemplate(QString file)
+MetrowerksMakefileGenerator::findTemplate(const QString &file)
 {
     QString ret;
     if(!QFile::exists(ret = file) && 
@@ -621,7 +618,7 @@ MetrowerksMakefileGenerator::findTemplate(QString file)
 bool
 MetrowerksMakefileGenerator::createFork(const QString &f)
 {
-#ifdef Q_OS_MACX
+#if !defined(QWS) && defined(Q_OS_MACX)
     FSRef fref;
     FSSpec fileSpec;
     if(QFile::exists(f)) {
@@ -664,7 +661,7 @@ MetrowerksMakefileGenerator::fixifyToMacPath(QString &p, QString &v, bool )
     static QString st_volume;
     if(st_volume.isEmpty()) {
 	st_volume = var("QMAKE_VOLUMENAME");
-#ifdef Q_OS_MAC
+#if !defined(QWS) && defined(Q_OS_MACX)
 	if(st_volume.isEmpty()) {
 	    uchar foo[512];
 	    HVolumeParam pb;
@@ -683,14 +680,14 @@ MetrowerksMakefileGenerator::fixifyToMacPath(QString &p, QString &v, bool )
     QString volume = st_volume;
 
     fixEnvVariables(p);
-    if(p.left(1) == "\"" && p.right(1) == "\"")
+    if(p.startsWith("\"") && p.endsWith("\""))
 	p = p.mid(1, p.length() - 2);
     if(p.isEmpty()) 
 	return FALSE;
-    if(p.right(1) != "/")
+    if(!p.endsWith("/"))
 	p += "/";
     if(QDir::isRelativePath(p)) {
-	if(p.left(1) == "{") {
+	if(p.startsWith("{")) {
 	    int eoc = p.find('}');
 	    if(eoc == -1)
 		return FALSE;
@@ -706,7 +703,7 @@ MetrowerksMakefileGenerator::fixifyToMacPath(QString &p, QString &v, bool )
     p = QDir::cleanDirPath(p);
     if(!volume.isEmpty()) 
 	v = volume;
-    p.replace(QRegExp("/"), ":");
+    p.replace("/", ":");
     if(p.right(1) != ":")
 	p += ':';
     return TRUE;
@@ -724,18 +721,18 @@ MetrowerksMakefileGenerator::processPrlFiles()
 	    QStringList &l = project->variables()[lflags[i]];
 	    for(QStringList::Iterator it = l.begin(); it != l.end(); ++it) {
 		QString opt = (*it);
-		if(opt.left(1) == "-") {
-		    if(opt.left(2) == "-L") {
+		if(opt.startsWith("-")) {
+		    if(opt.startsWith("-L")) {
 			QString r = opt.right(opt.length() - 2), l = r;
 			fixEnvVariables(l);
-			libdirs.append(new MakefileDependDir(r.replace(QRegExp("\""),""),
-							     l.replace(QRegExp("\""),"")));
+			libdirs.append(new MakefileDependDir(r.replace( "\"", ""),
+							     l.replace( "\"", "")));
 		    } else if(opt.left(2) == "-l") {
 			QString lib = opt.right(opt.length() - 2), prl;
 			for(MakefileDependDir *mdd = libdirs.first(); mdd; mdd = libdirs.next() ) {
-			    prl = mdd->local_dir + Option::dir_sep + "lib" + lib + Option::prl_ext;
+			    prl = mdd->local_dir + Option::dir_sep + "lib" + lib;
 			    if(processPrlFile(prl)) {
-				if(prl.left(mdd->local_dir.length()) == mdd->local_dir)
+				if(prl.startsWith(mdd->local_dir))
 				    prl.replace(0, mdd->local_dir.length(), mdd->real_dir);
 				QRegExp reg("^.*lib(" + lib + "[^.]*)\\." + 
 					    project->first("QMAKE_EXTENSION_SHLIB") + "$");
@@ -751,7 +748,7 @@ MetrowerksMakefileGenerator::processPrlFiles()
 			++it;
 			opt = (*it);
 			QString prl = "/System/Library/Frameworks/" + opt +
-				      ".framework/" + opt + Option::prl_ext;
+				      ".framework/" + opt;
 			if(processPrlFile(prl))
 			    ret = TRUE;
 		    }
@@ -779,10 +776,10 @@ MetrowerksMakefileGenerator::processPrlVariable(const QString &var, const QStrin
 	QStringList &out = project->variables()["QMAKE_LIBS"];
 	for(QStringList::ConstIterator it = l.begin(); it != l.end(); ++it) {
 	    bool append = TRUE;
-	    if((*it).left(1) == "-") {
-		if((*it).left(2) == "-l" || (*it).left(2) == "-L") {
+	    if((*it).startsWith("-")) {
+		if((*it).startsWith("-l") || (*it).startsWith("-L")) {
 		    append = out.findIndex((*it)) == -1;
-		} else if((*it).left(2) == "-framework") {
+		} else if((*it).startsWith("-framework")) {
 		    ++it;
 		    for(QStringList::ConstIterator outit = out.begin(); 
 			outit != out.end(); ++it) {
@@ -805,3 +802,18 @@ MetrowerksMakefileGenerator::processPrlVariable(const QString &var, const QStrin
 	MakefileGenerator::processPrlVariable(var, l);
     }
 }
+
+
+bool
+MetrowerksMakefileGenerator::openOutput(QFile &file) const
+{
+    QString outdir;
+    if(!file.name().isEmpty()) {
+	QFileInfo fi(file);
+	if(fi.isDir())
+	    outdir = file.name() + QDir::separator();
+    }
+    if(!outdir.isEmpty() || file.name().isEmpty()) 
+	file.setName(outdir + project->first("TARGET") + ".xml");
+    return MakefileGenerator::openOutput(file);
+}
