@@ -1,13 +1,11 @@
 /****************************************************************************
-** $Id:  qt/projectgenerator.h   3.0.5   edited Jan 14 23:35 $
+** $Id: qt/projectgenerator.h   3.2.3   edited May 30 09:55 $
 **
-** Definition of ________ class.
+** Definition of ProjectGenerator class.
 **
-** Created : 970521
+** Copyright (C) 1992-2003 Trolltech AS.  All rights reserved.
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
-**
-** This file is part of the network module of the Qt GUI Toolkit.
+** This file is part of qmake.
 **
 ** This file may be distributed under the terms of the Q Public License
 ** as defined by Trolltech AS of Norway and appearing in the file
@@ -51,7 +49,7 @@ protected:
 public:
     ProjectGenerator(QMakeProject *p);
     ~ProjectGenerator();
-    virtual QString defaultMakefile() const;
+    virtual bool openOutput(QFile &) const;
 };
 
 inline ProjectGenerator::~ProjectGenerator()
