@@ -1,13 +1,11 @@
 /****************************************************************************
-** $Id:  qt/unixmake.h   3.0.5   edited Jan 16 07:40 $
+** $Id: qt/unixmake.h   3.2.3   edited Aug 14 02:39 $
 **
-** Definition of ________ class.
+** Definition of UnixMakefileGenerator class.
 **
-** Created : 970521
+** Copyright (C) 1992-2003 Trolltech AS.  All rights reserved.
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
-**
-** This file is part of the network module of the Qt GUI Toolkit.
+** This file is part of qmake.
 **
 ** This file may be distributed under the terms of the Q Public License
 ** as defined by Trolltech AS of Norway and appearing in the file
@@ -34,6 +32,7 @@
 ** not clear to you.
 **
 **********************************************************************/
+
 #ifndef __UNIXMAKE_H__
 #define __UNIXMAKE_H__
 
@@ -43,6 +42,13 @@ class UnixMakefileGenerator : public MakefileGenerator
 {
     bool init_flag, include_deps;
     bool writeMakefile(QTextStream &);
+    void writeExtraVariables(QTextStream &);
+    QString libtoolFileName();
+    void writeLibtoolFile();     // for libtool
+    QString pkgConfigFileName();
+    void writePkgConfigFile();   // for pkg-config
+    QStringList combineSetLFlags(const QStringList &list1, const QStringList &list2);
+    void writePrlFile(QTextStream &);
 
 public:
     UnixMakefileGenerator(QMakeProject *p);
@@ -54,6 +60,8 @@ protected:
     virtual void processPrlVariable(const QString &, const QStringList &);
     virtual void processPrlFiles();
 
+    virtual bool findLibraries();
+    virtual QString findDependency(const QString &);
     virtual void init();
 
     void writeMakeParts(QTextStream &);
