@@ -1,13 +1,11 @@
 /****************************************************************************
-** $Id:  qt/msvc_nmake.h   3.0.5   edited Oct 12 2001 $
+** $Id: qt/msvc_nmake.h   3.2.3   edited May 30 12:56 $
 **
-** Definition of ________ class.
+** Definition of NmakeMakefileGenerator class.
 **
-** Created : 970521
+** Copyright (C) 1992-2003 Trolltech AS.  All rights reserved.
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
-**
-** This file is part of the network module of the Qt GUI Toolkit.
+** This file is part of qmake.
 **
 ** This file may be distributed under the terms of the Q Public License
 ** as defined by Trolltech AS of Norway and appearing in the file
@@ -34,8 +32,9 @@
 ** not clear to you.
 **
 **********************************************************************/
-#ifndef __NMAKEMAKE_H__
-#define __NMAKEMAKE_H__
+
+#ifndef __MSVC_NMAKE_H__
+#define __MSVC_NMAKE_H__
 
 #include "winmakefile.h"
 
@@ -56,4 +55,4 @@ public:
 inline NmakeMakefileGenerator::~NmakeMakefileGenerator()
 { }
 
-#endif /* __NMAKEMAKE_H__ */
+#endif /* __MSVC_NMAKE_H__ */
