@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id: qt/msvc_objectmodel.cpp   3.1.2   edited Dec 6 12:42 $
+** $Id: qt/msvc_objectmodel.cpp   3.2.3   edited Nov 5 09:12 $
 **
-** Definition of ________ class.
+** Implementation of VCProject class.
 **
-** Copyright (C) 2002 Trolltech AS.  All rights reserved.
+** Copyright (C) 2002-2003 Trolltech AS.  All rights reserved.
 **
-** This file is part of the network module of the Qt GUI Toolkit.
+** This file is part of qmake.
 **
 ** This file may be distributed under the terms of the Q Public License
 ** as defined by Trolltech AS of Norway and appearing in the file
@@ -281,7 +281,6 @@ struct XPair {
 
 // void streamSPair( QTextStream &strm, const char *n, const QString &s )
 
-
 // Streaming operators for property Pairs ---------------------------
 QTextStream &operator<<( QTextStream &strm, const TPair &prop )
 {
@@ -318,7 +317,7 @@ QTextStream &operator<<( QTextStream &strm, const LPair &prop )
 QTextStream &operator<<( QTextStream &strm, const SPair &prop )
 {
     if ( !prop.value.isEmpty() )
-	strm << prop.name << prop.value.latin1() << "\"";
+	strm << prop.name << QString(prop.value).remove("\"") << "\"";
     return strm;
 }
 
@@ -334,7 +333,7 @@ VCCLCompilerTool::VCCLCompilerTool()
     :	AssemblerOutput( asmListingNone ),
 	BasicRuntimeChecks( runtimeBasicCheckNone ),
 	BrowseInformation( brInfoNone ),
-	BufferSecurityCheck( unset ),
+	BufferSecurityCheck( _False ),
 	CallingConvention( callConventionDefault ),
 	CompileAs( compileAsDefault ),
 	CompileAsManaged( managedDefault ),
@@ -346,7 +345,7 @@ VCCLCompilerTool::VCCLCompilerTool()
 	EnableFiberSafeOptimizations( unset ),
 	EnableFunctionLevelLinking( unset ),
 	EnableIntrinsicFunctions( unset ),
-	ExceptionHandling( unset ),
+	ExceptionHandling( _False ),
 	ExpandAttributedSource( unset ),
 	FavorSizeOrSpeed( favorNone ),
 	ForceConformanceInForLoopScope( unset ),
@@ -354,11 +353,11 @@ VCCLCompilerTool::VCCLCompilerTool()
 	GlobalOptimizations( unset ),
 	IgnoreStandardIncludePath( unset ),
 	ImproveFloatingPointConsistency( unset ),
-	InlineFunctionExpansion( expandOnlyInline ),
+	InlineFunctionExpansion( expandDefault ),
 	KeepComments( unset ),
 	MinimalRebuild( unset ),
 	OmitFramePointers( unset ),
-	Optimization( optimizeDisabled ),
+	Optimization( optimizeCustom ),
 	OptimizeForProcessor( procOptimizeBlended ),
 	OptimizeForWindowsApplication( unset ),
 	RuntimeLibrary( rtMultiThreaded ),
@@ -371,7 +370,7 @@ VCCLCompilerTool::VCCLCompilerTool()
 	TreatWChar_tAsBuiltInType( unset ),
 	TurnOffAssemblyGeneration( unset ),
 	UndefineAllPreprocessorDefinitions( unset ),
-	UsePrecompiledHeader( pchGenerateAuto ),
+	UsePrecompiledHeader( pchNone ),
 	WarnAsError( unset ),
 	WarningLevel( warningLevel_0 ),
 	WholeProgramOptimization( unset )
@@ -413,19 +412,19 @@ QTextStream &operator<<( QTextStream &strm, const VCCLCompilerTool &tool )
     strm << TPair( _GlobalOptimizations, tool.GlobalOptimizations );
     strm << TPair( _IgnoreStandardIncludePath, tool.IgnoreStandardIncludePath );
     strm << TPair( _ImproveFloatingPointConsistency, tool.ImproveFloatingPointConsistency );
-    strm << EPair( _InlineFunctionExpansion, tool.InlineFunctionExpansion );
+    if ( tool.InlineFunctionExpansion != expandDefault ) strm << EPair( _InlineFunctionExpansion, tool.InlineFunctionExpansion );
     strm << TPair( _KeepComments, tool.KeepComments );
     strm << TPair( _MinimalRebuild, tool.MinimalRebuild );
     strm << SPair( _ObjectFile, tool.ObjectFile );
     strm << TPair( _OmitFramePointers, tool.OmitFramePointers );
-    strm << EPair( _Optimization, tool.Optimization );
+    if ( tool.Optimization != optimizeDefault ) strm << EPair( _Optimization, tool.Optimization );
     if ( tool.OptimizeForProcessor != procOptimizeBlended ) strm << EPair( _OptimizeForProcessor, tool.OptimizeForProcessor );
     strm << TPair( _OptimizeForWindowsApplication, tool.OptimizeForWindowsApplication );
     strm << SPair( _OutputFile, tool.OutputFile );
     strm << SPair( _PrecompiledHeaderFile, tool.PrecompiledHeaderFile );
     strm << SPair( _PrecompiledHeaderThrough, tool.PrecompiledHeaderThrough );
     strm << XPair( _PreprocessorDefinitions, tool.PreprocessorDefinitions );
-    strm << SPair( _ProgramDataBaseFileName, tool.ProgramDataBaseFileName );
+    strm << _ProgramDataBaseFileName << tool.ProgramDataBaseFileName.latin1() << "\"";
     strm << EPair( _RuntimeLibrary, tool.RuntimeLibrary );
     strm << TPair( _RuntimeTypeInfo, tool.RuntimeTypeInfo );
     strm << TPair( _ShowIncludes, tool.ShowIncludes );
@@ -484,12 +483,18 @@ bool VCCLCompilerTool::parseOption( const char* option )
 	break;
     case 'E':
 	if ( second == 'H' ) {
-	    if ( third == 'a' || third == 'c' || third == 's' ) {
+	    if ( third == 'a'
+	    	|| (third == 'c' && fourth != 's')
+	    	|| (third == 's' && fourth != 'c') ) {
 		// ExceptionHandling must be false, or it will override
 		// with an /EHsc option
 		ExceptionHandling = _False;
 		AdditionalOptions += option;
 		break;
+	    } else if ( (third == 'c' && fourth == 's')
+		     || (third == 's' && fourth == 'c') ) {
+		ExceptionHandling = _True;
+		AdditionalOptions += option;
 	    }
 	    found = FALSE; break;
 	}
@@ -591,6 +596,9 @@ bool VCCLCompilerTool::parseOption( const char* option )
 	    EnableFiberSafeOptimizations = _True;
 	    break;
 	case 'X':
+	    // ExceptionHandling == true will override with
+	    // an /EHsc option, which is correct with /GX
+	    ExceptionHandling = _True; // Fall-through
 	case 'Z':
 	case 'e':
 	case 'h':
@@ -957,7 +965,7 @@ VCLinkerTool::VCLinkerTool()
 	IgnoreImportLibrary( unset ),
 	LargeAddressAware( addrAwareDefault ),
 	LinkDLL( unset ),
-	LinkIncremental( linkIncrementalYes ),
+	LinkIncremental( linkIncrementalDefault ),
 	LinkTimeCodeGeneration( unset ),
 	MapExports( unset ),
 	MapLines( unset ),
@@ -1019,7 +1027,7 @@ QTextStream &operator<<( QTextStream &strm, const VCLinkerTool &tool )
     if ( tool.OptimizeForWindows98 != optWin98Default )	    strm << EPair( _OptimizeForWindows98, tool.OptimizeForWindows98 );
     if ( tool.OptimizeReferences != optReferencesDefault )  strm << EPair( _OptimizeReferences, tool.OptimizeReferences );
     strm << SPair( _OutputFile, tool.OutputFile );
-    strm << SPair( _ProgramDatabaseFile, tool.ProgramDatabaseFile );
+    strm << _ProgramDatabaseFile << tool.ProgramDatabaseFile << "\"";
     strm << TPair( _RegisterOutput, tool.RegisterOutput );
     strm << TPair( _ResourceOnlyDLL, tool.ResourceOnlyDLL );
     strm << TPair( _SetChecksum, tool.SetChecksum );
@@ -1870,7 +1878,7 @@ void VCFilter::generateUIC( QTextStream &strm, const QString& str ) const
     QString uiDir = Project->var( "UI_DIR" );
     QString uiHeaders;
     QString uiSources;
-
+    
     // Determining the paths for the output files.
     int slash = str.findRev( '\\' );
     QString pname = ( slash != -1 ) ? str.left( slash+1 ) : QString( ".\\" );
@@ -1895,6 +1903,9 @@ void VCFilter::generateUIC( QTextStream &strm, const QString& str ) const
     if( dot != -1 )
 	fname.truncate( dot );
 
+    if ( mocDir.isEmpty() )
+	mocDir = pname;
+
     strm << _begFileConfiguration;
     strm << _Name5;
     strm << Config->Name;
@@ -1906,11 +1917,11 @@ void VCFilter::generateUIC( QTextStream &strm, const QString& str ) const
     strm << _CommandLine6;
     strm << uicApp << " " << str << " -o " << uiHeaders << fname << ".h &amp;&amp; ";				// Create .h from .ui file
     strm << uicApp << " " << str << " -i " << fname << ".h -o " << uiSources << fname << ".cpp &amp;&amp; ";	// Create .cpp from .ui file
-    strm << mocApp << " " << uiHeaders << fname << ".h -o " << mocDir << "moc_" << fname << ".cpp\"";
+    strm << mocApp << " " << uiHeaders << fname << ".h -o " << mocDir << Option::h_moc_mod << fname << Option::h_moc_ext << "\"";
     strm << _AdditionalDependencies6;
     strm << mocApp << ";" << uicApp << "\"";
     strm << _Outputs6;
-    strm << uiHeaders << fname << ".h;" << uiSources << fname << ".cpp;" << mocDir << "moc_" << fname << ".cpp\"";
+    strm << uiHeaders << fname << ".h;" << uiSources << fname << ".cpp;" << mocDir << Option::h_moc_mod << fname << Option::h_moc_ext << "\"";
     strm << "/>";
     strm << _endFileConfiguration;
 }
