@@ -1,13 +1,13 @@
 /****************************************************************************
-** $Id:  qt/qtextcodecplugin.cpp   3.0.5   edited Apr 3 23:03 $
+** $Id: qt/qtextcodecplugin.cpp   3.2.3   edited May 13 09:08 $
 **
 ** Implementation of QTextCodecPlugin class
 **
 ** Created : 010920
 **
-** Copyright (C) 2001 Trolltech AS.  All rights reserved.
+** Copyright (C) 2001-2002 Trolltech AS.  All rights reserved.
 **
-** This file is part of the widgets module of the Qt GUI Toolkit.
+** This file is part of the tools module of the Qt GUI Toolkit.
 **
 ** This file may be distributed under the terms of the Q Public License
 ** as defined by Trolltech AS of Norway and appearing in the file
@@ -39,58 +39,62 @@
 #ifndef QT_NO_TEXTCODECPLUGIN
 #include "qtextcodecinterface_p.h"
 
-/*!   \class QTextCodecPlugin qtextcodecplugin.h
-  \brief The QTextCodecPlugin class provides an abstract base for custom QTextCodec plugins.
-  \ingroup plugins
+/*!
+    \class QTextCodecPlugin qtextcodecplugin.h
+    \brief The QTextCodecPlugin class provides an abstract base for custom QTextCodec plugins.
+    \reentrant
+    \ingroup plugins
 
-  The text codec plugin is a simple plugin interface that makes it easy to
-  create custom text codecs that can be loaded dynamically into
-  applications.
+    The text codec plugin is a simple plugin interface that makes it
+    easy to create custom text codecs that can be loaded dynamically
+    into applications.
 
-  Writing a text codec plugin is achieved by subclassing this
-  base class, reimplementing the pure virtual functions names(),
-  createForName(), mibEnums() and createForMib(), and exporting the
-  class with the \c Q_EXPORT_PLUGIN macro.  See the \link
-  plugins-howto.html Qt Plugins documentation \endlink for details.
-
-  See the \link http://www.iana.org/assignments/character-sets IANA
-  character-sets encoding file\endlink for more information on mime
-  names and mib enums.
+    Writing a text codec plugin is achieved by subclassing this base
+    class, reimplementing the pure virtual functions names(),
+    createForName(), mibEnums() and createForMib(), and exporting the
+    class with the \c Q_EXPORT_PLUGIN macro. See the \link
+    plugins-howto.html Qt Plugins documentation \endlink for details.
 
+    See the \link http://www.iana.org/assignments/character-sets IANA
+    character-sets encoding file\endlink for more information on mime
+    names and mib enums.
 */
 
-/*! \fn QStringList QTextCodecPlugin::names() const
+/*!
+    \fn QStringList QTextCodecPlugin::names() const
 
-  Returns the list of mime names this plugin supports.
+    Returns the list of mime names supported by this plugin.
 
-
-  \sa createForName()
+    \sa createForName()
 */
 
-/*! \fn QTextCodec *QTextCodecPlugin::createForName( const QString &name );
+/*!
+    \fn QTextCodec *QTextCodecPlugin::createForName( const QString &name );
 
-  Creates a QTextCodec object for the codec called \a name.
+    Creates a QTextCodec object for the codec called \a name.
 
-  \sa names()
+    \sa names()
 */
 
 
-/*! \fn QValueList<int> QTextCodecPlugin::mibEnums() const
+/*!
+    \fn QValueList<int> QTextCodecPlugin::mibEnums() const
 
-  Returns the list of mib enums this plugin supports.
+    Returns the list of mib enums supported by this plugin.
 
-
-  \sa createForMib()
+    \sa createForMib()
 */
 
-/*! \fn QTextCodec *QTextCodecPlugin::createForMib( int mib );
+/*!
+    \fn QTextCodec *QTextCodecPlugin::createForMib( int mib );
 
-  Creates a QTextCodec object for the mib enum \a mib.
+    Creates a QTextCodec object for the mib enum \a mib.
 
-  (See \link ftp://ftp.isi.edu/in-notes/iana/assignments/character-sets
-  the IANA character-sets encoding file\endlink for more information)
+    (See \link
+    ftp://ftp.isi.edu/in-notes/iana/assignments/character-sets the
+    IANA character-sets encoding file\endlink for more information)
 
-  \sa mibEnums()
+    \sa mibEnums()
 */
 
 
@@ -158,20 +162,19 @@ QTextCodec *QTextCodecPluginPrivate::createForName( const QString &name )
 
 
 /*!
-  Constructs a text codec plugin. This is invoked automatically by
-  the \c Q_EXPORT_PLUGIN macro.
- */
+    Constructs a text codec plugin. This is invoked automatically by
+    the \c Q_EXPORT_PLUGIN macro.
+*/
 QTextCodecPlugin::QTextCodecPlugin()
     : QGPlugin( d = new QTextCodecPluginPrivate( this ) )
 {
 }
 
 /*!
-  Destroys the text codec plugin.
-
-  You never have to call this explicitly. Qt destroys a plugin
-  automatically when it is no longer used.
+    Destroys the text codec plugin.
 
+    You never have to call this explicitly. Qt destroys a plugin
+    automatically when it is no longer used.
 */
 QTextCodecPlugin::~QTextCodecPlugin()
 {
