@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id$
+** $Id: qt/qdialog.cpp   3.2.3   edited Aug 24 09:39 $
 **
 ** Implementation of QDialog class
 **
@@ -57,126 +57,136 @@
 #endif
 
 /*!
-  \class QDialog qdialog.h
-  \brief The QDialog class is the base class of dialog windows.
+    \class QDialog
+    \brief The QDialog class is the base class of dialog windows.
 
-  \ingroup dialogs
-  \ingroup abstractwidgets
-  \mainclass
+    \ingroup dialogs
+    \ingroup abstractwidgets
+    \mainclass
 
-  A dialog window is a top-level window mostly used for short-term
-  tasks and brief communications with the user. QDialogs may be modal
-  or modeless. QDialogs support \link #extensibility
-  extensibility\endlink and can provide a \link #return return
-  value\endlink. They can have \link #default default buttons\endlink.
-  QDialogs can also have a QSizeGrip in their lower-right corner,
-  using setSizeGripEnabled().
+    A dialog window is a top-level window mostly used for short-term
+    tasks and brief communications with the user. QDialogs may be
+    modal or modeless. QDialogs support \link #extensibility
+    extensibility\endlink and can provide a \link #return return
+    value\endlink. They can have \link #default default
+    buttons\endlink. QDialogs can also have a QSizeGrip in their
+    lower-right corner, using setSizeGripEnabled().
 
-  Note that QDialog uses the parent widget slightly differently from
-  other classes in Qt. A dialog is always a top-level widget, but if
-  it has a parent, its default location is centered on top of the
-  parent. It will also share the parent's taskbar entry.
+    Note that QDialog uses the parent widget slightly differently from
+    other classes in Qt. A dialog is always a top-level widget, but if
+    it has a parent, its default location is centered on top of the
+    parent's top-level widget (if it is not top-level itself). It will
+    also share the parent's taskbar entry.
 
-  There are three kinds of dialog that are useful:
+    \target modal
+    \section1 Modal Dialogs
 
-  \list 1
+    A <b>modal</b> dialog is a dialog that blocks input to other
+    visible windows in the same application. Users must finish
+    interacting with the dialog and close it before they can access
+    any other window in the application. Dialogs that are used to
+    request a file name from the user or that are used to set
+    application preferences are usually modal.
 
-  \i A <b>modal</b> dialog is a dialog that blocks input to other visible
-  windows in the same application: users must finish interacting
-  with the dialog and close it before they can access any other window
-  in the application. Modal dialogs have their own local event loop.
-  Dialogs which are used to request a filename from the user or which
-  are used to set application preferences are usually modal. Call exec()
-  to display a modal dialog. When the user closes the dialog, exec()
-  will provide a useful \link #return return value\endlink, and the flow
-  of control will follow on from the exec() call at this time. Typically
-  we connect a default button, e.g. "OK", to the accept() slot and a
-  "Cancel" button to the reject() slot, to get the dialog to close and
-  return the appropriate value. Alternatively you can connect to the
-  done() slot, passing it \c Accepted or \c Rejected.
+    The most common way to display a modal dialog is to call its
+    exec() function. When the user closes the dialog, exec() will
+    provide a useful \link #return return value\endlink. Typically we
+    connect a default button, e.g. "OK", to the accept() slot and a
+    "Cancel" button to the reject() slot, to get the dialog to close
+    and return the appropriate value. Alternatively you can connect to
+    the done() slot, passing it \c Accepted or \c Rejected.
 
-  \i A <b>modeless</b> dialog is a dialog that operates independently of
-  other windows in the same application. Find and replace dialogs in
-  word-processors are often modeless to allow the user to interact with
-  both the application's main window and the dialog. Call show() to
-  display a modeless dialog. show() returns immediately so the flow of
-  control will continue in the calling code. In practice you will often
-  call show() and come to the end of the function in which show() is
-  called with control returning to the main event loop.
+    An alternative is to call setModal(TRUE), then show(). Unlike
+    exec(), show() returns control to the caller immediately. Calling
+    setModal(TRUE) is especially useful for progress dialogs, where
+    the user must have the ability to interact with the dialog, e.g.
+    to cancel a long running operation. If you use show() and
+    setModal(TRUE) together you must call
+    QApplication::processEvents() periodically during processing to
+    enable the user to interact with the dialog. (See \l
+    QProgressDialog.)
 
-  \i
-  \target semimodal
-  A "<b>semi-modal</b>" dialog is a modal dialog that returns control
-  to the caller immediately. Semi-modal dialogs do not have their own
-  event loop, so you will need to call QApplication::processEvents()
-  periodically to give the semi-modal dialog the opportunity to
-  process its events. A progress dialog (e.g. QProgressDialog) is an
-  example, where you only want the user to be able to interact with
-  the progress dialog, e.g. to cancel a long running operation, but
-  need to actually carry out the operation. Semi-modal dialogs are
-  displayed by setting the modal flag to TRUE and calling the \l
-  show() function.
-  \endlist
+    \target modeless
+    \section1 Modeless Dialogs
 
-  \target default
-  \section1 Default button
+    A <b>modeless</b> dialog is a dialog that operates
+    independently of other windows in the same application. Find and
+    replace dialogs in word-processors are often modeless to allow the
+    user to interact with both the application's main window and with
+    the dialog.
 
-  A dialog's "default" button is the button that's pressed when the user
-  presses Enter or Return. This button is used to signify that the user
-  accepts the dialog's settings and wishes to close the dialog. Use
-  QPushButton::setDefault(), QPushButton::isDefault() and
-  QPushButton::autoDefault() to set and control the dialog's default
-  button.
+    Modeless dialogs are displayed using show(), which returns control
+    to the caller immediately.
 
-  \target extensibility
-  \section1 Extensibility
+    \target default
+    \section1 Default button
 
-  Extensibility is the ability to show the dialog in two ways: a partial
-  dialog that shows the most commonly used options, and a full dialog
-  that shows all the options. Typically an extensible dialog
-  will initially appear as a partial dialog, but with a "More"
-  button. If the user clicks the "More" button, the full dialog will
-  appear. Extensibility is controlled with setExtension(),
-  setOrientation() and showExtension().
+    A dialog's \e default button is the button that's pressed when the
+    user presses Enter (Return). This button is used to signify that
+    the user accepts the dialog's settings and wants to close the
+    dialog. Use QPushButton::setDefault(), QPushButton::isDefault()
+    and QPushButton::autoDefault() to set and control the dialog's
+    default button.
 
-  \target return
-  \section1 Return value (modal dialogs)
+    \target escapekey
+    \section1 Escape Key
 
-  Modal dialogs are often used in situations where a return value is
-  required; for example to indicate whether the user pressed "OK" or
-  "Cancel". A dialog can be closed by calling the accept() or the
-  reject() slots, and exec() will return \c Accepted or \c Rejected as
-  appropriate. The exec() call returns the result of the dialog. The
-  result is also available from result() if the dialog has not been
-  destroyed. If the \c WDestructiveClose flag is set, then when exec()
-  returns the dialog is deleted.
+    If the user presses the Esc key in a dialog, QDialog::reject()
+    will be called. This will cause the window to close: the \link
+    QCloseEvent closeEvent \endlink cannot be \link
+    QCloseEvent::ignore() ignored \endlink.
 
-  \target examples
-  \section1 Examples
+    \target extensibility
+    \section1 Extensibility
 
-  A modal dialog.
+    Extensibility is the ability to show the dialog in two ways: a
+    partial dialog that shows the most commonly used options, and a
+    full dialog that shows all the options. Typically an extensible
+    dialog will initially appear as a partial dialog, but with a
+    "More" toggle button. If the user presses the "More" button down,
+    the full dialog will appear. The extension widget will be resized
+    to its sizeHint(). If orientation is \c Horizontal the extension
+    widget's height() will be expanded to the height() of the dialog.
+    If the orientation is \c Vertical the extension widget's width()
+    will be expanded to the width() of the dialog. Extensibility is
+    controlled with setExtension(), setOrientation() and
+    showExtension().
 
-  \quotefile network/networkprotocol/view.cpp
-  \skipto QFileDialog *dlg
-  \printuntil return
+    \target return
+    \section1 Return value (modal dialogs)
 
-  A modeless dialog. After the show() call, control returns to the main
-  event loop.
-  \quotefile life/main.cpp
-  \skipto argv
-  \printuntil QApplication
-  \skipto scale
-  \printline
-  \skipto LifeDialog
-  \printuntil show
-  \skipto exec
-  \printuntil }
+    Modal dialogs are often used in situations where a return value is
+    required, e.g. to indicate whether the user pressed "OK" or
+    "Cancel". A dialog can be closed by calling the accept() or the
+    reject() slots, and exec() will return \c Accepted or \c Rejected
+    as appropriate. The exec() call returns the result of the dialog.
+    The result is also available from result() if the dialog has not
+    been destroyed. If the \c WDestructiveClose flag is set, the
+    dialog is deleted after exec() returns.
 
-  See the \l QProgressDialog documentation for an example of a
-  semi-modal dialog.
+    \target examples
+    \section1 Examples
 
-  \sa QTabDialog QWidget QProgressDialog
-  \link guibooks.html#fowler GUI Design Handbook: Dialogs, Standard\endlink
+    A modal dialog.
+
+    \quotefile network/networkprotocol/view.cpp
+    \skipto QFileDialog *dlg
+    \printuntil return
+
+    A modeless dialog. After the show() call, control returns to the main
+    event loop.
+    \quotefile life/main.cpp
+    \skipto argv
+    \printuntil QApplication
+    \skipto scale
+    \printline
+    \skipto LifeDialog
+    \printuntil show
+    \skipto exec
+    \printuntil }
+
+    \sa QTabDialog QWidget QProgressDialog
+    \link guibooks.html#fowler GUI Design Handbook: Dialogs, Standard\endlink
 */
 
 /*! \enum QDialog::DialogCode
@@ -201,7 +211,7 @@ class QDialogPrivate : public Qt
 public:
 
     QDialogPrivate()
-	: mainDef(0), orientation(Horizontal),extension(0)
+	: mainDef(0), orientation(Horizontal),extension(0), doShowExtension(FALSE)
 #ifndef QT_NO_SIZEGRIP
 	,resizer(0)
 #endif
@@ -211,6 +221,7 @@ public:
     QPushButton* mainDef;
     Orientation orientation;
     QWidget* extension;
+    bool doShowExtension;
     QSize size, min, max;
 #ifndef QT_NO_SIZEGRIP
     QSizeGrip* resizer;
@@ -222,32 +233,28 @@ public:
 /*!
   Constructs a dialog called \a name, with parent \a parent.
 
-  If \a modal is FALSE (the default), the dialog is modeless and should
-  be displayed with show(). If \a modal is TRUE and the dialog is
-  displayed with exec(), the dialog is modal, i.e. blocks input to other
-  windows in this application. If \a modal is TRUE and the dialog is
-  displayed using show(), the dialog is semi-modal.
+  A dialog is always a top-level widget, but if it has a parent, its
+  default location is centered on top of the parent. It will also
+  share the parent's taskbar entry.
 
   The widget flags \a f are passed on to the QWidget constructor.
-
   If, for example, you don't want a What's This button in the titlebar
   of the dialog, pass WStyle_Customize | WStyle_NormalBorder |
   WStyle_Title | WStyle_SysMenu in \a f.
 
-  We recommend that you always pass a non-null parent.
+  \warning In Qt 3.2, the \a modal flag is obsolete. There is now a
+  setModal() function that can be used for obtaining a modal behavior
+  when calling show(). This is rarely needed, because modal dialogs
+  are usually invoked using exec(), which ignores the \a modal flag.
 
   \sa QWidget::setWFlags() Qt::WidgetFlags
 */
 
 QDialog::QDialog( QWidget *parent, const char *name, bool modal, WFlags f )
     : QWidget( parent, name,
-	       (modal ? (f|WShowModal) : f) | WType_Dialog )
+	       (modal ? (f|WShowModal) : f) | WType_Dialog ),
+      rescode(0), did_move(0), has_relpos(0), did_resize(0), in_loop(0)
 {
-    rescode = 0;
-    did_move = FALSE;
-    did_resize = FALSE;
-    has_relpos = FALSE;
-    in_loop = FALSE;
     d = new QDialogPrivate;
 }
 
@@ -317,28 +324,28 @@ void QDialog::hideDefault()
 
 
 /*!
-  \fn int  QDialog::result() const
+  \fn int QDialog::result() const
 
   Returns the modal dialog's result code, \c Accepted or \c Rejected.
 
   Do not call this function if the dialog was constructed with the \c
-  WDestructiveClose flag. (exec() returns the result code anyway.)
+  WDestructiveClose flag.
 */
 
 /*!
-  \fn void  QDialog::setResult( int i )
+  \fn void QDialog::setResult( int i )
 
   Sets the modal dialog's result code to \a i.
 */
 
 
 /*!
-    Executes a modal dialog. Control passes to the dialog until the user
-    closes it, at which point the local event loop finishes and the
-    function returns with the \l DialogCode result.
-    Users will not be able to interact with any other window in
-    the same application until they close this dialog. For a modeless or
-    semi-modal dialog use show().
+    Shows the dialog as a \link #modal modal \endlink dialog,
+    blocking until the user closes it. The function returns a \l
+    DialogCode result.
+
+    Users cannot interact with any other window in the same
+    application until they close the dialog.
 
   \sa show(), result()
 */
@@ -346,7 +353,7 @@ void QDialog::hideDefault()
 int QDialog::exec()
 {
     if ( in_loop ) {
-	qWarning( "QDialog::exec: Recursive call detected." );
+	qWarning( "QDialog::exec: Recursive call detected" );
 	return -1;
     }
 
@@ -368,18 +375,21 @@ int QDialog::exec()
     int res = result();
 
     if ( destructiveClose )
-      delete this;
+	delete this;
 
     return res;
 }
 
 
-/*! Hides the modal dialog and sets its result code to \a r. This
-  uses the local event loop to finish, and exec() to return \a r.
+/*! Closes the dialog and sets its result code to \a r. If this dialog
+  is shown with exec(), done() causes the local event loop to finish,
+  and exec() to return \a r.
 
-  If the dialog has the \c WDestructiveClose flag set, done() also
-  deletes the dialog. If the dialog is the applications's main widget,
-  the application terminates.
+  As with QWidget::close(), done() deletes the dialog if the \c
+  WDestructiveClose flag is set. If the dialog is the application's
+  main widget, the application terminates. If the dialog is the
+  last window closed, the QApplication::lastWindowClosed() signal is
+  emitted.
 
   \sa accept(), reject(), QApplication::mainWidget(), QApplication::quit()
 */
@@ -389,21 +399,39 @@ void QDialog::done( int r )
     hide();
     setResult( r );
 
-    // We cannot use close() here, as close() calls closeEvent() calls
-    // reject() calls close(). But we can at least keep the
-    // mainWidget() and WDestructiveClose semantics. There should not
-    // be much of a difference whether the users types Alt-F4 or
-    // Escape. Without that, destructive-close dialogs were more or
-    // less useless without subclassing.
-    if ( qApp->mainWidget() == this )
+    // emulate QWidget::close()
+    bool isMain = qApp->mainWidget() == this;
+    bool checkLastWindowClosed = isTopLevel() && !isPopup();
+    if ( checkLastWindowClosed
+	 && qApp->receivers(SIGNAL(lastWindowClosed())) ) {
+	/* if there is no non-withdrawn top level window left (except
+	   the desktop, popups, or dialogs with parents), we emit the
+	   lastWindowClosed signal */
+	QWidgetList *list   = qApp->topLevelWidgets();
+	QWidget     *widget = list->first();
+	while ( widget ) {
+	    if ( !widget->isHidden()
+		 && !widget->isDesktop()
+		 && !widget->isPopup()
+		 && (!widget->isDialog() || !widget->parentWidget()))
+		break;
+	    widget = list->next();
+	}
+	delete list;
+	if ( widget == 0 )
+	    emit qApp->lastWindowClosed();
+    }
+    if ( isMain )
 	qApp->quit();
-
-    if ( testWFlags(WDestructiveClose) )
+    else if ( testWFlags(WDestructiveClose) ) {
+	clearWFlags(WDestructiveClose);
 	deleteLater();
+    }
 }
 
 /*!
   Hides the modal dialog and sets the result code to \c Accepted.
+
   \sa reject() done()
 */
 
@@ -414,6 +442,7 @@ void QDialog::accept()
 
 /*!
   Hides the modal dialog and sets the result code to \c Rejected.
+
   \sa accept() done()
 */
 
@@ -439,12 +468,17 @@ void QDialog::contextMenuEvent( QContextMenuEvent *e )
     QWidget* w = childAt( e->pos(), TRUE );
     if ( !w )
 	return;
-    QString s = QWhatsThis::textFor( w, e->pos(), TRUE );
+    QString s;
+    while ( s.isEmpty() && w ) {
+	s = QWhatsThis::textFor( w, e->pos(), FALSE );
+	if ( s.isEmpty() )
+	    w = w->parentWidget(TRUE);
+    }
     if ( !s.isEmpty() ) {
 	QPopupMenu p(0,"qt_whats_this_menu");
 	p.insertItem( tr("What's This?"), 42 );
 	if ( p.exec( e->globalPos() ) >= 42 )
-	    QWhatsThis::display( s, w->mapToGlobal( w->rect().center() ) );
+	    QWhatsThis::display( s, w->mapToGlobal( w->rect().center() ), w );
     }
 #endif
 }
@@ -455,6 +489,11 @@ void QDialog::keyPressEvent( QKeyEvent *e )
     //   Calls reject() if Escape is pressed. Simulates a button
     //   click for the default button if Enter is pressed. Move focus
     //   for the arrow keys. Ignore the rest.
+#ifdef Q_OS_MAC
+    if(e->state() == ControlButton && e->key() == Key_Period) {
+	reject();
+    } else
+#endif
     if ( e->state() == 0 || ( e->state() & Keypad && e->key() == Key_Enter ) ) {
 	switch ( e->key() ) {
 	case Key_Enter:
@@ -518,8 +557,10 @@ void QDialog::closeEvent( QCloseEvent *e )
     if ( isModal() && QWhatsThis::inWhatsThisMode() )
 	QWhatsThis::leaveWhatsThisMode();
 #endif
-    e->accept();
-    reject();
+    if ( isShown() )
+	reject();
+    if ( isHidden() )
+	e->accept();
 }
 
 
@@ -527,24 +568,18 @@ void QDialog::closeEvent( QCloseEvent *e )
   Geometry management.
  *****************************************************************************/
 
+#if defined(Q_WS_X11)
+extern "C" { int XSetTransientForHint( Display *, unsigned long, unsigned long ); }
+#endif // Q_WS_X11
+
 /*!
-    Shows a modeless or semi-modal dialog. Control returns immediately
-    to the calling code.
+    Shows the dialog as a \link #modeless modeless \endlink dialog.
+    Control returns immediately to the calling code.
 
-    The dialog does not have a local event loop so you must call
-    QApplication::processEvents() periodically to give the dialog the
-    opportunity to process its events.
+    The dialog will be modal or modeless according to the value
+    of the \l modal property.
 
-    The dialog will be \link #semimodal semi-modal\endlink if the modal
-    flag was set to TRUE in the constructor.
-
-  \warning
-
-  In Qt 2.x, calling show() on a modal dialog enters a local event
-  loop, and works like exec(), but doesn't return the result code
-  exec() returns. Trolltech has always warned that doing this is unwise.
-
-  \sa exec()
+    \sa exec(), modal
 */
 
 void QDialog::show()
@@ -553,56 +588,64 @@ void QDialog::show()
 	return;
     if ( !did_resize )
 	adjustSize();
-    if ( has_relpos ) {
-	adjustPositionInternal( parentWidget() ? parentWidget() : qApp->mainWidget(), TRUE );
+    if ( has_relpos && !did_move ) {
+	adjustPositionInternal( parentWidget(), TRUE );
     } else if ( !did_move ) {
-	adjustPositionInternal( parentWidget() ? parentWidget() : qApp->mainWidget() );
+	adjustPositionInternal( parentWidget() );
     }
+
+#if defined(Q_WS_X11)
+    if ( !parentWidget() && testWFlags( WShowModal ) &&
+	 qApp->mainWidget() && qApp->mainWidget()->isVisible() ) {
+	// make sure the transient for hint is set properly for modal dialogs
+        XSetTransientForHint( x11Display(), winId(), qApp->mainWidget()->winId() );
+    }
+#endif // Q_WS_X11
+
     QWidget::show();
+    showExtension( d->doShowExtension );
 #ifndef QT_NO_PUSHBUTTON
     QWidget *fw = focusWidget();
     QFocusData *fd = focusData();
 
     /*
-      The following block is to handle a special case, and does not really follow
-      propper logic in concern of autoDefault and TAB order. However, it's here to
-      ease usage for the users. If a dialog has a default QPushButton, and first
-      widget in the TAB order also is a QPushButton, then we give focus to the
-      main default QPushButton. This simplifies code for the developers, and
-      actually catches most cases... If not, then they simply have to use
-      [widget*]->setFocus() them selfs...
+      The following block is to handle a special case, and does not
+      really follow propper logic in concern of autoDefault and TAB
+      order. However, it's here to ease usage for the users. If a
+      dialog has a default QPushButton, and first widget in the TAB
+      order also is a QPushButton, then we give focus to the main
+      default QPushButton. This simplifies code for the developers,
+      and actually catches most cases... If not, then they simply
+      have to use [widget*]->setFocus() themselves...
     */
-    if ( !fw ) {
+    if ( !fw || fw->focusPolicy() == NoFocus ) {
 	fd->home(); // Skip main form
 	QWidget *first = fd->next(); // Get first main widget
 	if ( d->mainDef &&
 	     first != d->mainDef &&
-	     first->inherits("QPushButton") )
+	     ::qt_cast<QPushButton*>(first) )
 	    d->mainDef->setFocus();
     }
-    // -- [End special case block] --
 
     if ( !d->mainDef && isTopLevel() ) {
-	if ( !fw ) {
+	if ( !fw || fw->focusPolicy() == NoFocus ) {
 	    focusNextPrevChild( TRUE );
 	    fw = focusWidget();
 	}
-        if ( fw ) {
+	if ( fw ) {
 	    fd = focusData();
 	    QWidget *home = fd->home();
 	    QWidget *candidate = home;
 	    Q_ASSERT( candidate == fw );
 	    do {
-                if ( candidate->inherits("QPushButton") ) {
-		    QPushButton *pb = (QPushButton *)candidate;
-		    if ( pb->autoDefault() ) {
-                        pb->setDefault( TRUE );
-                        break;
-		    }
-                }
-                candidate = fd->next();
+		QPushButton *pb = ::qt_cast<QPushButton*>(candidate);
+		if ( pb && pb->autoDefault() ) {
+		    pb->setDefault( TRUE );
+		    break;
+		}
+		candidate = fd->next();
 	    } while ( candidate != home );
-        }
+	}
     }
     if ( fw ) {
 	QFocusEvent e( QEvent::FocusIn );
@@ -617,31 +660,38 @@ void QDialog::show()
 #endif
 }
 
-
-/*!\internal
- */
+/*! \internal */
 void QDialog::adjustPosition( QWidget* w)
 {
     adjustPositionInternal( w );
 }
 
+
 void QDialog::adjustPositionInternal( QWidget*w, bool useRelPos)
 {
     /* need to make sure these events are already sent to be sure
        our information below is correct --sam */
-    QApplication::sendPostedEvents(this, QEvent::LayoutHint);
-    QApplication::sendPostedEvents(this, QEvent::Resize);
+    QApplication::sendPostedEvents( this, QEvent::LayoutHint );
+    QApplication::sendPostedEvents( this, QEvent::Resize );
+
+    // processing the events might call polish(), which is a nice place
+    // to restore geometries, so return if the dialog has been positioned
+    if ( did_move || has_relpos )
+	return;
 
     QPoint p( 0, 0 );
     int extraw = 0, extrah = 0, scrn = 0;
     if ( w )
 	w = w->topLevelWidget();
     QRect desk;
-    if ( w )
-	scrn =  QApplication::desktop()->screenNumber( w );
-    else
+    if ( w ) {
+	scrn = QApplication::desktop()->screenNumber( w );
+    } else if ( QApplication::desktop()->isVirtualDesktop() ) {
 	scrn = QApplication::desktop()->screenNumber( QCursor::pos() );
-    desk = QApplication::desktop()->screenGeometry( scrn );
+    } else {
+	scrn = QApplication::desktop()->screenNumber( this );
+    }
+    desk = QApplication::desktop()->availableGeometry( scrn );
 
     QWidgetList  *list = QApplication::topLevelWidgets();
     QWidgetListIt it( *list );
@@ -650,7 +700,7 @@ void QDialog::adjustPositionInternal( QWidget*w, bool useRelPos)
 	int framew, frameh;
 	QWidget * current = it.current();
 	++it;
-	if ( ! current->isVisibleTo( 0 ) )
+	if ( ! current->isVisible() )
 	    continue;
 
 	framew = current->geometry().x() - current->x();
@@ -663,10 +713,12 @@ void QDialog::adjustPositionInternal( QWidget*w, bool useRelPos)
 
     // sanity check for decoration frames. With embedding, we
     // might get extraordinary values
-    if ( extraw >= 10 || extrah >= 40 )
-	extraw = extrah = 0;
+    if ( extraw == 0 || extrah == 0 || extraw >= 10 || extrah >= 40 ) {
+	extrah = 40;
+	extraw = 10;
+    }
 
-    if ( useRelPos ) {
+    if ( useRelPos && w ) {
 	p = w->pos() + d->relPos;
     } else {
 	if ( w ) {
@@ -680,7 +732,7 @@ void QDialog::adjustPositionInternal( QWidget*w, bool useRelPos)
 	    p = QPoint( desk.x() + desk.width()/2, desk.y() + desk.height()/2 );
 	}
 #if defined( Q_OS_TEMP )
-	p = QPoint( 0, GetSystemMetric( SM_CYCAPTION ) );
+	p = QPoint( 0, GetSystemMetrics( SM_CYCAPTION ) );
 #endif
 
 	// p = origin of this
@@ -700,6 +752,7 @@ void QDialog::adjustPositionInternal( QWidget*w, bool useRelPos)
 	p.setY( desk.y() );
 
     move( p );
+    did_move = !useRelPos;
 }
 
 
@@ -714,10 +767,9 @@ void QDialog::hide()
 	QAccessible::updateAccessibility( this, 0, QAccessible::DialogEnd );
 #endif
 
-    if ( parentWidget() ) {
+    if ( parentWidget() && !did_move ) {
 	d->relPos = pos() - parentWidget()->topLevelWidget()->pos();
 	has_relpos = 1;
-	did_move = 0;
     }
 
     // Reimplemented to exit a modal when the dialog is hidden.
@@ -827,8 +879,7 @@ void QDialog::setExtension( QWidget* extension )
 
     if ( extension->parentWidget() != this )
 	extension->reparent( this, QPoint(0,0) );
-    else
-	extension->hide();
+    extension->hide();
 }
 
 /*!
@@ -850,20 +901,21 @@ QWidget* QDialog::extension() const
   This slot is usually connected to the \l QButton::toggled() signal
   of a QPushButton.
 
-  If the dialog is not visible, or has no extension, nothing happens.
+  A dialog with a visible extension is not resizeable.
 
   \sa show(), setExtension(), setOrientation()
  */
 void QDialog::showExtension( bool showIt )
 {
+    d->doShowExtension = showIt;
     if ( !d->extension )
 	return;
     if ( !testWState(WState_Visible) )
 	return;
+    if ( d->extension->isVisible() == showIt )
+	return;
 
     if ( showIt ) {
-	if ( d->extension->isVisible() )
-	    return;
 	d->size = size();
 	d->min = minimumSize();
 	d->max = maximumSize();
@@ -885,8 +937,6 @@ void QDialog::showExtension( bool showIt )
 	}
 	d->extension->show();
     } else {
-	if ( !d->extension->isVisible() )
-	    return;
 	d->extension->hide();
 	// workaround for CDE window manager that won't shrink with (-1,-1)
 	setMinimumSize( d->min.expandedTo( QSize( 1, 1 ) ) );
@@ -929,7 +979,30 @@ QSize QDialog::minimumSizeHint() const
     return QWidget::minimumSizeHint();
 }
 
+/*! \property QDialog::modal
+    \brief whether show() should pop up the dialog as modal or modeless
 
+    By default, this property is false and show() pops up the dialog as
+    modeless.
+
+    exec() ignores the value of this property and always pops up the
+    dialog as modal.
+
+    \sa show(), exec()
+*/
+
+void QDialog::setModal( bool modal )
+{
+    if ( modal )
+	setWFlags( WShowModal );
+    else
+	clearWFlags( WShowModal );
+}
+
+bool QDialog::isModal() const
+{
+    return testWFlags( WShowModal ) != 0;
+}
 
 bool QDialog::isSizeGripEnabled() const
 {
@@ -947,7 +1020,8 @@ void QDialog::setSizeGripEnabled(bool enabled)
     if ( !enabled != !d->resizer ) {
 	if ( enabled ) {
 	    d->resizer = new QSizeGrip( this, "QDialog::resizer" );
-	    d->resizer->adjustSize();
+	    // adjustSize() processes all events, which is suboptimal
+	    d->resizer->resize( d->resizer->sizeHint() );
 	    if ( QApplication::reverseLayout() )
 		d->resizer->move( rect().bottomLeft() -d->resizer->rect().bottomLeft() );
 	    else
