@@ -1,9 +1,9 @@
 /****************************************************************************
-** $Id:  qt/qerrormessage.cpp   3.0.5   edited Apr 22 22:09 $
+** $Id: qt/qerrormessage.cpp   3.2.3   edited May 26 12:44 $
 **
 ** Implementation of a nice qInstallMsgHandler() handler
 **
-** Created : 2000-05-27, after Kalle Dalheimer's birthday
+** Created : 000527, after Kalle Dalheimer's birthday
 **
 ** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
 **
@@ -105,6 +105,15 @@ after seeing each message.
 */
 
 static QErrorMessage * qtMessageHandler = 0;
+
+static void deleteStaticcQErrorMessage() // post-routine
+{
+    if ( qtMessageHandler ) {
+	delete qtMessageHandler;
+	qtMessageHandler = 0;
+    }
+}
+
 static bool metFatal = FALSE;
 
 void jump( QtMsgType t, const char * m )
@@ -212,8 +221,11 @@ QErrorMessage * QErrorMessage::qtHandler()
 {
     if ( !qtMessageHandler ) {
 	qtMessageHandler = new QErrorMessage( 0, "automatic message handler" );
+	qAddPostRoutine( deleteStaticcQErrorMessage ); // clean up
+#ifndef QT_NO_WIDGET_TOPEXTRA
 	if ( qApp->mainWidget() )
 	    qtMessageHandler->setCaption( qApp->mainWidget()->caption() );
+#endif
 	qInstallMsgHandler( jump );
     }
     return qtMessageHandler;
