@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id:  qt/qprogressdialog.cpp   3.0.5   edited Jun 7 12:09 $
+** $Id: qt/qprogressdialog.cpp   3.2.3   edited Jul 10 09:47 $
 **
 ** Implementation of QProgressDialog class
 **
@@ -130,10 +130,11 @@ public:
   There are two ways of using QProgressDialog: modal and modeless.
 
   Using a modal QProgressDialog is simpler for the programmer, but you
-  must call qApp->processEvents() to keep the event loop running
-  to ensure that the application doesn't freeze. Do the operation in a
-  loop, call \l setProgress() at intervals, and check for cancellation
-  with wasCancelled(). For example:
+  must call QApplication::processEvents() or
+  QEventLoop::processEvents(ExcludeUserInput) to keep the event loop
+  running to ensure that the application doesn't freeze. Do the
+  operation in a loop, call \l setProgress() at intervals, and check
+  for cancellation with wasCanceled(). For example:
 \code
 QProgressDialog progress( "Copying files...", "Abort Copy", numFiles,
                           this, "progress", TRUE );
@@ -141,7 +142,7 @@ for ( int i = 0; i < numFiles; i++ ) {
     progress.setProgress( i );
     qApp->processEvents();
 
-    if ( progress.wasCancelled() )
+    if ( progress.wasCanceled() )
         break;
     //... copy one file
 }
@@ -156,14 +157,14 @@ progress.setProgress( numFiles );
   is often an alternative to a modeless progress dialog.
 
   You need to have an event loop to be running, connect the
-  cancelled() signal to a slot that stops the operation, and call \l
+  canceled() signal to a slot that stops the operation, and call \l
   setProgress() at intervals. For example:
 \code
 Operation::Operation( QObject *parent = 0 )
     : QObject( parent ), steps( 0 )
 {
     pd = new QProgressDialog( "Operation in progress.", "Cancel", 100 );
-    connect( pd, SIGNAL(cancelled()), this, SLOT(cancel()) );
+    connect( pd, SIGNAL(canceled()), this, SLOT(cancel()) );
     t = new QTimer( this );
     connect( t, SIGNAL(timeout()), this, SLOT(perform()) );
     t->start( 0 );
@@ -232,8 +233,8 @@ QProgressBar *QProgressDialog::bar() const
   \i The total number of steps is 100.
   \endlist
 
-  The top level parent of the \a creator widget becomes the parent of
-  the dialog. The \a name, \a modal, and the widget flags, \a f, are
+  The \a creator argument is the widget to use as the dialog's parent.
+  The \a name, \a modal, and the widget flags, \a f, are
   passed to the QDialog::QDialog() constructor. If \a modal is FALSE (the
   default), you must have an event loop proceeding for any redrawing
   of the dialog to occur. If \a modal is TRUE, the dialog ensures that
@@ -245,7 +246,7 @@ QProgressBar *QProgressDialog::bar() const
 
 QProgressDialog::QProgressDialog( QWidget *creator, const char *name,
 				  bool modal, WFlags f )
-    : QDialog( creator?creator->topLevelWidget():0, name, modal, f)
+    : QDialog( creator, name, modal, f)
 {
     init( creator, QString::fromLatin1(""), tr("Cancel"), 100 );
 }
@@ -265,15 +266,13 @@ QProgressDialog::QProgressDialog( QWidget *creator, const char *name,
    processed call setProgress(1), setProgress(2), etc., finally
    calling setProgress(50) after examining the last file.
 
+   The \a creator argument is the widget to use as the dialog's parent.
    The \a name, \a modal, and widget flags, \a f, are passed to the
    QDialog::QDialog() constructor. If \a modal is FALSE (the default),
    you will must have an event loop proceeding for any redrawing of
    the dialog to occur. If \a modal is TRUE, the dialog ensures that
    events are processed when needed.
 
-   The \a creator argument is the widget to use as the dialog's parent.
-   If \a creator is not a top level widget the argument passed on to
-   the QDialog constructor will be 0.
 
   \sa setLabelText(), setLabel(), setCancelButtonText(), setCancelButton(),
   setTotalSteps()
@@ -284,7 +283,7 @@ QProgressDialog::QProgressDialog( const QString &labelText,
 				  int totalSteps,
 				  QWidget *creator, const char *name,
 				  bool modal, WFlags f )
-    : QDialog( creator?creator->topLevelWidget():0, name, modal, f)
+    : QDialog( creator, name, modal, f)
 {
     init( creator, labelText, cancelButtonText, totalSteps );
 }
@@ -312,19 +311,28 @@ void QProgressDialog::init( QWidget *creator,
     d->autoReset = TRUE;
     d->forceHide = FALSE;
     setCancelButtonText( canc );
-    connect( this, SIGNAL(cancelled()), this, SLOT(cancel()) );
+    connect( this, SIGNAL(canceled()), this, SIGNAL(cancelled()) );
+    connect( this, SIGNAL(canceled()), this, SLOT(cancel()) );
     forceTimer = new QTimer( this );
     connect( forceTimer, SIGNAL(timeout()), this, SLOT(forceShow()) );
     layout();
 }
 
 /*!
-  \fn void QProgressDialog::cancelled()
+  \fn void QProgressDialog::canceled()
 
   This signal is emitted when the cancel button is clicked.
   It is connected to the cancel() slot by default.
 
-  \sa wasCancelled()
+  \sa wasCanceled()
+*/
+
+/*!
+  \fn void QProgressDialog::cancelled()
+
+  \obsolete
+
+  Use canceled() instead.
 */
 
 
@@ -399,7 +407,7 @@ void QProgressDialog::setCancelButton( QPushButton *cancelButton )
 	} else {
 	    cancelButton->reparent( this, 0, QPoint(0,0), FALSE );
 	}
-	connect( d->cancel, SIGNAL(clicked()), this, SIGNAL(cancelled()) );
+	connect( d->cancel, SIGNAL(clicked()), this, SIGNAL(canceled()) );
 #ifndef QT_NO_ACCEL
 	QAccel *accel = new QAccel( this );
 	accel->connectItem( accel->insertItem(Key_Escape),
@@ -459,7 +467,16 @@ void QProgressDialog::setBar( QProgressBar *bar )
 
 /*!
   \property QProgressDialog::wasCancelled
-  \brief whether the dialog was cancelled
+  \brief whether the dialog was canceled
+
+  \obsolete
+
+  Use \l wasCanceled instead.
+*/
+
+/*!
+  \property QProgressDialog::wasCanceled
+  \brief whether the dialog was canceled
 
   \sa setProgress()
 */
@@ -514,7 +531,7 @@ void QProgressDialog::reset()
 }
 
 /*!
-  Resets the progress dialog. wasCancelled() becomes TRUE until
+  Resets the progress dialog. wasCanceled() becomes TRUE until
   the progress dialog is reset.
   The progress dialog becomes hidden.
 */
@@ -606,6 +623,21 @@ void QProgressDialog::setProgress( int progress )
 	reset();
 }
 
+/*!
+  \overload
+
+  Sets the current amount of progress to \a progress and the total number of
+  steps to \a totalSteps.
+
+  \sa setTotalSteps()
+*/
+
+void QProgressDialog::setProgress( int progress, int totalSteps )
+{
+    setTotalSteps( totalSteps );
+    setProgress( progress );
+}
+
 /*!
   Returns a size that fits the contents of the progress dialog.
   The progress dialog resizes itself as required, so you should not
@@ -682,14 +714,18 @@ void QProgressDialog::layout()
 }
 
 /*!
-  \property QProgressDialog::minimumDuration
-  \brief the time that the progress should run for before the dialog opens
+    \property QProgressDialog::minimumDuration
+    \brief the time that must pass before the dialog appears
 
-  The dialog will not appear if the expected duration of the
-  progressing task is less than the minimum duration.
+    If the expected duration of the task is less than the
+    minimumDuration, the dialog will not appear at all. This prevents
+    the dialog popping up for tasks that are quickly over. For tasks
+    that are expected to exceed the minimumDuration, the dialog will
+    pop up after the minimumDuration time or as soon as any progress
+    is set.
 
-  If set to 0, the dialog is always shown as soon as any progress
-  is set. The default is 4000 milliseconds.
+    If set to 0, the dialog is always shown as soon as any progress is
+    set. The default is 4000 milliseconds.
 */
 void QProgressDialog::setMinimumDuration( int ms )
 {
@@ -712,7 +748,7 @@ int QProgressDialog::minimumDuration() const
 
 void QProgressDialog::closeEvent( QCloseEvent *e )
 {
-    emit cancelled();
+    emit canceled();
     QDialog::closeEvent( e );
 }
 
