@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id:  qt/qtabdialog.cpp   3.0.5   edited Apr 5 21:56 $
+** $Id: qt/qtabdialog.cpp   3.2.3   edited Aug 13 16:32 $
 **
 ** Implementation of QTabDialog class
 **
@@ -870,14 +870,14 @@ void QTabDialog::setUpLayout()
     // top margin
     d->tll->addSpacing( topMargin );
 
-    QBoxLayout * tmp = new QBoxLayout( QBoxLayout::LeftToRight );
-     d->tll->addLayout( tmp, 1 );
-     tmp->addSpacing( leftMargin );
-     tmp->addWidget( d->tw, 1);
-     tmp->addSpacing( rightMargin + 2 );
+    QBoxLayout * tmp = new QHBoxLayout();
+    d->tll->addLayout( tmp, 1 );
+    tmp->addSpacing( leftMargin );
+    tmp->addWidget( d->tw, 1);
+    tmp->addSpacing( rightMargin + 2 );
 
     d->tll->addSpacing( aboveButtonsMargin + 2 );
-    QBoxLayout * buttonRow = new QBoxLayout( QBoxLayout::RightToLeft );
+    QBoxLayout * buttonRow = new QBoxLayout(QBoxLayout::RightToLeft);
     d->tll->addLayout( buttonRow, 0 );
     d->tll->addSpacing( bottomMargin );
 
@@ -885,26 +885,31 @@ void QTabDialog::setUpLayout()
     if ( d->cb ) {
 	buttonRow->addWidget( d->cb, 0 );
 	buttonRow->addSpacing( betweenButtonsMargin );
+	d->cb->raise();
     }
 
     if ( d->ab ) {
 	buttonRow->addWidget( d->ab, 0 );
 	buttonRow->addSpacing( betweenButtonsMargin );
+	d->ab->raise();
     }
 
     if ( d->db ) {
 	buttonRow->addWidget( d->db, 0 );
 	buttonRow->addSpacing( betweenButtonsMargin );
+	d->db->raise();
     }
 
     if ( d->hb ) {
 	buttonRow->addWidget( d->hb, 0 );
 	buttonRow->addSpacing( betweenButtonsMargin );
+	d->hb->raise();
     }
 
     if ( d->ok ) {
 	buttonRow->addWidget( d->ok, 0 );
 	buttonRow->addSpacing( betweenButtonsMargin );
+	d->ok->raise();
     }
 
     // add one custom widget here
