@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id$
+** $Id: qt/qwizard.cpp   3.2.3   edited May 24 15:31 $
 **
 ** Implementation of QWizard class.
 **
@@ -52,51 +52,53 @@
 /*! \file wizard/wizard.cpp */
 /*! \file wizard/wizard.h */
 
-/*! \class QWizard qwizard.h
+/*!
+    \class QWizard qwizard.h
+    \brief The QWizard class provides a framework for wizard dialogs.
 
-  \ingroup abstractwidgets
-  \ingroup organizers
-  \ingroup dialogs
-  \mainclass
+    \ingroup abstractwidgets
+    \ingroup organizers
+    \ingroup dialogs
+    \mainclass
 
-  \brief The QWizard class provides a framework for wizard dialogs.
+    A wizard is a special type of input dialog that consists of a
+    sequence of dialog pages. A wizard's purpose is to walk the user
+    through a process step by step. Wizards are useful for complex or
+    infrequently occurring tasks that people may find difficult to
+    learn or do.
 
-  A wizard is a special type of input dialog that consists of a
-  sequence of dialog pages. A wizard's purpose is to assist a user by
-  automating a task by walking the user through the process step
-  by step. Wizards are useful for complex or infrequently occurring
-  tasks that people may find difficult to learn or do.
+    QWizard provides page titles and displays Next, Back, Finish,
+    Cancel, and Help push buttons, as appropriate to the current
+    position in the page sequence. These buttons can be
+    enabled/disabled using setBackEnabled(), setNextEnabled(),
+    setFinishEnabled() and setHelpEnabled().
 
-  QWizard provides page titles and displays Next, Back, Finish,
-  Cancel, and Help push buttons, as appropriate to the current
-  position in the page sequence.
+    Create and populate dialog pages that inherit from QWidget and add
+    them to the wizard using addPage(). Use insertPage() to add a
+    dialog page at a certain position in the page sequence. Use
+    removePage() to remove a page from the page sequence.
 
-  Create and populate dialog pages that inherit from QWidget and add
-  them to the wizard using addPage(). Use insertPage() to add a dialog
-  page at a certain position in the page sequence. Use removePage() to
-  remove a page from the page sequence.
+    Use currentPage() to retrieve a pointer to the currently displayed
+    page. page() returns a pointer to the page at a certain position
+    in the page sequence.
 
-  Use currentPage() to retrieve a pointer to the currently displayed
-  page. page() returns a pointer to the page at a certain position in
-  the page sequence.
+    Use pageCount() to retrieve the total number of pages in the page
+    sequence. indexOf() will return the index of a page in the page
+    sequence.
 
-  Use pageCount() to retrieve the total number of pages in the page
-  sequence. indexOf() will return the index of a page in the page
-  sequence.
+    QWizard provides functionality to mark pages as appropriate (or
+    not) in the current context with setAppropriate(). The idea is
+    that a page may be irrelevant and should be skipped depending on
+    the data entered by the user on a preceding page.
 
-  QWizard provides functionality to mark pages as appropriate (or not)
-  in the current context with setAppropriate(). The idea is that a
-  page may be irrelevant and should be skipped depending on the data
-  entered by the user on a preceding page.
+    It is generally considered good design to provide a greater number
+    of simple pages with fewer choices rather than a smaller number of
+    complex pages.
 
-  It is generally considered good design to provide a greater number
-  of simple pages with fewer choices rather than a smaller number of
-  complex pages.
+    Example code is available here: \l wizard/wizard.cpp \l wizard/wizard.h
 
-  Example code is available here: \l wizard/wizard.cpp \l wizard/wizard.h
-
-  \img qwizard.png A QWizard page
-  \caption A QWizard page
+    \img qwizard.png A QWizard page
+    \caption A QWizard page
 
 */
 
@@ -151,10 +153,9 @@ public:
 };
 
 
-/*!  Constructs an empty wizard dialog.
-    The \a parent, \a name, \a modal and \a f arguments are passed to
-    the QDialog constructor.
-
+/*!
+    Constructs an empty wizard dialog. The \a parent, \a name, \a
+    modal and \a f arguments are passed to the QDialog constructor.
 */
 
 QWizard::QWizard( QWidget *parent, const char *name, bool modal,
@@ -180,11 +181,11 @@ QWizard::QWizard( QWidget *parent, const char *name, bool modal,
     d->hbar1 = 0;
     d->hbar2 = 0;
 
-    d->cancelButton->setText( tr( "Cancel" ) );
-    d->backButton->setText( tr( "< Back" ) );
-    d->nextButton->setText( tr( "Next >" ) );
-    d->finishButton->setText( tr( "Finish" ) );
-    d->helpButton->setText( tr( "Help" ) );
+    d->cancelButton->setText( tr( "&Cancel" ) );
+    d->backButton->setText( tr( "< &Back" ) );
+    d->nextButton->setText( tr( "&Next >" ) );
+    d->finishButton->setText( tr( "&Finish" ) );
+    d->helpButton->setText( tr( "&Help" ) );
 
     d->nextButton->setDefault( TRUE );
 
@@ -220,22 +221,27 @@ QWizard::~QWizard()
 }
 
 
-/*!  \reimp  */
+/*!
+    \reimp
+*/
 
 void QWizard::show()
 {
-    if ( d->current )
-	showPage( d->current->w );
-    else if ( pageCount() > 0 )
-	showPage( d->pages.at( 0 )->w );
-    else
-	showPage( 0 );
+    if ( !d->current ) {
+	// No page yet
+	if ( pageCount() > 0 )
+	    showPage( d->pages.at( 0 )->w );
+	else
+	    showPage( 0 );
+    }
 
     QDialog::show();
 }
 
 
-/*! \reimp */
+/*!
+    \reimp
+*/
 
 void QWizard::setFont( const QFont & font )
 {
@@ -244,7 +250,9 @@ void QWizard::setFont( const QFont & font )
 }
 
 
-/*!  Adds \a page to the end of the page sequence, with the title, \a title.
+/*!
+    Adds \a page to the end of the page sequence, with the title, \a
+    title.
 */
 
 void QWizard::addPage( QWidget * page, const QString & title )
@@ -271,8 +279,8 @@ void QWizard::addPage( QWidget * page, const QString & title )
 }
 
 /*!
-    Inserts \a page at index \a index into the page sequence, with
-    title \a title. If index is -1, the page will be appended to
+    Inserts \a page at position \a index into the page sequence, with
+    title \a title. If \a index is -1, the page will be appended to
     the end of the wizard's page sequence.
 */
 
@@ -304,14 +312,21 @@ void QWizard::insertPage( QWidget * page, const QString & title, int index )
 }
 
 /*!
-  \fn void QWizard::selected(const QString&)
+    \fn void QWizard::selected(const QString&)
 
-  This signal is emitted when the current page changes. The parameter
-  contains the title of the page.
+    This signal is emitted when the current page changes. The
+    parameter contains the title of the selected page.
 */
 
 
-/*!  Makes \a page the current page and emits the selected() signal. */
+/*!
+    Makes \a page the current page and emits the selected() signal.
+
+    This virtual function is called whenever a different page is to
+    be shown, including the first time the QWizard is shown.
+    By reimplementing it (and calling QWizard::showPage()),
+    you can prepare each page prior to it being shown.
+*/
 
 void QWizard::showPage( QWidget * page )
 {
@@ -340,7 +355,9 @@ void QWizard::showPage( QWidget * page )
 }
 
 
-/*!  Returns the number of pages in the wizard. */
+/*!
+    Returns the number of pages in the wizard.
+*/
 
 int QWizard::pageCount() const
 {
@@ -348,8 +365,8 @@ int QWizard::pageCount() const
 }
 
 /*!
-  Returns the sequence index of page \a page.
-  If the page is not part of the wizard -1 is returned.
+    Returns the position of page \a page. If the page is not part of
+    the wizard -1 is returned.
 */
 
 int QWizard::indexOf( QWidget* page ) const
@@ -361,10 +378,10 @@ int QWizard::indexOf( QWidget* page ) const
 }
 
 /*!
-  Called when the user clicks the Back button; this function shows
-  the preceding relevant page in the sequence.
+    Called when the user clicks the Back button; this function shows
+    the preceding relevant page in the sequence.
 
-  \sa appropriate()
+    \sa appropriate()
 */
 void QWizard::back()
 {
@@ -385,10 +402,10 @@ void QWizard::back()
 
 
 /*!
-  Called when the user clicks the Next button, this function shows
-  the next relevant page in the sequence.
+    Called when the user clicks the Next button, this function shows
+    the next relevant page in the sequence.
 
-  \sa appropriate()
+    \sa appropriate()
 */
 void QWizard::next()
 {
@@ -409,13 +426,14 @@ void QWizard::next()
 
 
 /*!
-  \fn void QWizard::helpClicked()
+    \fn void QWizard::helpClicked()
 
-  This signal is emitted when the user clicks on the Help button.
+    This signal is emitted when the user clicks on the Help button.
 */
 
-/*!  Called when the user clicks the Help button, this function emits the
-  helpClicked() signal.
+/*!
+    Called when the user clicks the Help button, this function emits
+    the helpClicked() signal.
 */
 
 void QWizard::help()
@@ -425,8 +443,9 @@ void QWizard::help()
 	return;
 
 #if 0
-    if ( page->inherits( "QWizardPage" ) )
-	emit ((QWizardPage *)page)->helpClicked();
+    QWizardPage *wpage = ::qt_cast<QWizardPage*>(page);
+    if ( wpage )
+	emit wpage->helpClicked();
 #endif
     emit helpClicked();
 }
@@ -465,8 +484,7 @@ void QWizard::setHelpEnabled( bool enable )
 
 /*!
     If \a enable is TRUE, page \a page has a Back button; otherwise \a
-    page has no Back button.
-  By default all pages have this button.
+    page has no Back button. By default all pages have this button.
 */
 void QWizard::setBackEnabled( QWidget * page, bool enable )
 {
@@ -497,9 +515,9 @@ void QWizard::setNextEnabled( QWidget * page, bool enable )
 
 
 /*!
-    If \a enable is TRUE, page \a page has a Finish button; otherwise \a
-    page has no Finish button.
-  By default \e no pages have this button.
+    If \a enable is TRUE, page \a page has a Finish button; otherwise
+    \a page has no Finish button. By default \e no page has this
+    button.
 */
 void QWizard::setFinishEnabled( QWidget * page, bool enable )
 {
@@ -514,8 +532,7 @@ void QWizard::setFinishEnabled( QWidget * page, bool enable )
 
 /*!
     If \a enable is TRUE, page \a page has a Help button; otherwise \a
-    page has no Help button.
-  By default all pages have this button.
+    page has no Help button. By default all pages have this button.
 */
 void QWizard::setHelpEnabled( QWidget * page, bool enable )
 {
@@ -535,8 +552,8 @@ void QWizard::setHelpEnabled( QWidget * page, bool enable )
     default implementation returns the value set using
     setAppropriate(). The ultimate default is TRUE.
 
-  \warning The last page of the wizard will be displayed if no page is relevant
-  in the current context.
+    \warning The last page of the wizard will be displayed if no page
+    is relevant in the current context.
 */
 
 bool QWizard::appropriate( QWidget * page ) const
@@ -548,10 +565,11 @@ bool QWizard::appropriate( QWidget * page ) const
 
 /*!
     If \a appropriate is TRUE then page \a page is considered relevant
-    in the current context and should be displayed in the page sequence;
-    otherwise \a page should not be displayed in the page sequence.
+    in the current context and should be displayed in the page
+    sequence; otherwise \a page should not be displayed in the page
+    sequence.
 
-  \sa appropriate()
+    \sa appropriate()
 */
 void QWizard::setAppropriate( QWidget * page, bool appropriate )
 {
@@ -589,9 +607,10 @@ void QWizard::updateButtons()
 }
 
 
-/*!  Returns a pointer to the current page in the sequence.
-  Although the wizard does its best to make sure that this value is
-  never 0, it can be if you try hard enough.
+/*!
+    Returns a pointer to the current page in the sequence. Although
+    the wizard does its best to make sure that this value is never 0,
+    it can be if you try hard enough.
 */
 
 QWidget * QWizard::currentPage() const
@@ -600,7 +619,8 @@ QWidget * QWizard::currentPage() const
 }
 
 
-/*!  Returns the title of page \a page.
+/*!
+    Returns the title of page \a page.
 */
 
 QString QWizard::title( QWidget * page ) const
@@ -609,7 +629,8 @@ QString QWizard::title( QWidget * page ) const
     return p ? p->t : QString::null;
 }
 
-/*!  Sets the title for page \a page to \a title.
+/*!
+    Sets the title for page \a page to \a title.
 */
 
 void QWizard::setTitle( QWidget *page, const QString &title )
@@ -622,10 +643,10 @@ void QWizard::setTitle( QWidget *page, const QString &title )
 }
 
 /*!
-  \property QWizard::titleFont
-  \brief the font used for page titles
+    \property QWizard::titleFont
+    \brief the font used for page titles
 
-  The default is QApplication::font().
+    The default is QApplication::font().
 */
 QFont QWizard::titleFont() const
 {
@@ -639,10 +660,11 @@ void QWizard::setTitleFont( const QFont & font )
 
 
 /*!
-  Returns a pointer to the Back button of the dialog.
+    Returns a pointer to the dialog's Back button
 
-  By default, this button is connected to the back() slot,
-  which is virtual so you can reimplement it in a QWizard subclass.
+    By default, this button is connected to the back() slot, which is
+    virtual so you can reimplement it in a QWizard subclass. Use
+    setBackEnabled() to enable/disable this button.
 */
 QPushButton * QWizard::backButton() const
 {
@@ -651,10 +673,11 @@ QPushButton * QWizard::backButton() const
 
 
 /*!
-  Returns a pointer to the Next button of the dialog.
+    Returns a pointer to the dialog's Next button
 
-  By default, this button is connected to the next() slot,
-  which is virtual so you can reimplement it in a QWizard subclass.
+    By default, this button is connected to the next() slot, which is
+    virtual so you can reimplement it in a QWizard subclass. Use
+    setNextEnabled() to enable/disable this button.
 */
 QPushButton * QWizard::nextButton() const
 {
@@ -663,10 +686,11 @@ QPushButton * QWizard::nextButton() const
 
 
 /*!
-  Returns a pointer to the Finish button of the dialog.
+    Returns a pointer to the dialog's Finish button
 
-  By default, this button is connected to the QDialog::accept() slot,
-  which is virtual so you can reimplement it in a QWizard subclass.
+    By default, this button is connected to the QDialog::accept()
+    slot, which is virtual so you can reimplement it in a QWizard
+    subclass. Use setFinishEnabled() to enable/disable this button.
 */
 QPushButton * QWizard::finishButton() const
 {
@@ -675,10 +699,11 @@ QPushButton * QWizard::finishButton() const
 
 
 /*!
-  Returns a pointer to the Cancel button of the dialog.
+    Returns a pointer to the dialog's Cancel button
 
-  By default, this button is connected to the QDialog::reject() slot,
-  which is virtual so you can reimplement it in a QWizard subclass.
+    By default, this button is connected to the QDialog::reject()
+    slot, which is virtual so you can reimplement it in a QWizard
+    subclass.
 */
 QPushButton * QWizard::cancelButton() const
 {
@@ -687,10 +712,11 @@ QPushButton * QWizard::cancelButton() const
 
 
 /*!
-  Returns a pointer to the Help button of the dialog.
+    Returns a pointer to the dialog's Help button
 
-  By default, this button is connected to the help() slot,
-  which is virtual so you can reimplement it in a QWizard subclass.
+    By default, this button is connected to the help() slot, which is
+    virtual so you can reimplement it in a QWizard subclass. Use
+    setHelpEnabled() to enable/disable this button.
 */
 QPushButton * QWizard::helpButton() const
 {
@@ -698,10 +724,11 @@ QPushButton * QWizard::helpButton() const
 }
 
 
-/*!  This virtual function is responsible for adding the bottom
-divider and buttons below it.
+/*!
+    This virtual function is responsible for adding the bottom divider
+    and the buttons below it.
 
-\a layout is the vertical layout of the entire wizard.
+    \a layout is the horizontal layout of the entire wizard.
 */
 
 void QWizard::layOutButtonRow( QHBoxLayout * layout )
@@ -725,7 +752,10 @@ void QWizard::layOutButtonRow( QHBoxLayout * layout )
     QBoxLayout * h = new QBoxLayout( QBoxLayout::LeftToRight );
     layout->addLayout( h );
 
-    h->addWidget( d->cancelButton );
+    if ( hasHelp )
+	h->addWidget( d->helpButton );
+    else
+	d->helpButton->hide();
 
     h->addStretch( 42 );
 
@@ -760,19 +790,15 @@ void QWizard::layOutButtonRow( QHBoxLayout * layout )
 	h->addWidget( d->finishButton );
     }
 
-    if ( hasHelp ) {
-	h->addSpacing( 12 );
-	h->addWidget( d->helpButton );
-    } else {
-	d->helpButton->hide();
-    }
+    h->addSpacing( 12 );
+    h->addWidget( d->cancelButton );
 }
 
 
 /*!
     This virtual function is responsible for laying out the title row
-    and adding the vertical divider between the title and the wizard
-    page. \a layout is the vertical layout for the wizard, and \a
+    and adding the horizontal divider between the title and the wizard
+    page. \a layout is the horizontal layout for the wizard, and \a
     title is the title for this page. This function is called every
     time \a title changes.
 */
@@ -822,7 +848,9 @@ void QWizard::layOut()
 }
 
 
-/*! \reimp */
+/*!
+    \reimp
+*/
 
 bool QWizard::eventFilter( QObject * o, QEvent * e )
 {
@@ -835,9 +863,11 @@ bool QWizard::eventFilter( QObject * o, QEvent * e )
 }
 
 
-/*!  Removes \a page from the page sequence but does not delete the page.
-  If \a page is currently being displayed, QWizard will display the page before
-  it in the wizard, or the first page if this was the first page.
+/*!
+    Removes \a page from the page sequence but does not delete the
+    page. If \a page is currently being displayed, QWizard will
+    display the page that precedes it, or the first page if this was
+    the first page.
 */
 
 void QWizard::removePage( QWidget * page )
@@ -865,8 +895,9 @@ void QWizard::removePage( QWidget * page )
 
 
 /*!
-  Returns a pointer to the page at position \a index in the sequence,
-  or 0 if \a index is out of range. The first page has index 0.
+    Returns a pointer to the page at position \a index in the
+    sequence, or 0 if \a index is out of range. The first page has
+    index 0.
 */
 
 QWidget* QWizard::page( int index ) const
