@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id:  qt/qabstractlayout.cpp   3.0.5   edited Jun 11 23:09 $
+** $Id: qt/qabstractlayout.cpp   3.2.3   edited Jul 4 13:32 $
 **
 ** Implementation of the abstract layout base class
 **
@@ -38,10 +38,20 @@
 #include "qlayout.h"
 
 #ifndef QT_NO_LAYOUT
-#include "qmenubar.h"
 #include "qapplication.h"
+#include "qlayoutengine_p.h"
+#include "qmenubar.h"
+#include "qtoolbar.h"
 
-#include "qlayoutengine_p.h" //remove in 3.1
+static int menuBarHeightForWidth( QMenuBar *menubar, int w )
+{
+#ifndef QT_NO_MENUBAR
+    if ( menubar && !menubar->isHidden() && !menubar->isTopLevel() )
+	return menubar->heightForWidth( QMAX(w, menubar->minimumWidth()) );
+    else
+#endif
+	return 0;
+}
 
 /*!
     \class QLayoutItem
@@ -134,8 +144,8 @@ void QLayoutItem::setAlignment( int a )
 /*!
     \fn QSizePolicy::ExpandData QLayoutItem::expanding() const
 
-    Implemented in subclasses to return whether this item "wants" to
-    expand.
+    Implemented in subclasses to return the direction(s) this item
+    "wants" to expand in (if any).
 */
 
 /*!
@@ -316,67 +326,6 @@ int QLayoutItem::heightForWidth( int /* w */ ) const
     return -1;
 }
 
-static QSize smartMinSize( const QWidgetItem *i )
-{
-    QWidget *w = ((QWidgetItem *)i)->widget();
-
-    QSize s( 0, 0 );
-    if ( w->layout() ) {
-	s = w->layout()->totalMinimumSize();
-    } else {
-	QSize sh;
-
-	if ( w->sizePolicy().mayShrinkHorizontally() ) {
-	    s.setWidth( w->minimumSizeHint().width() );
-	} else {
-	    sh = w->sizeHint();
-	    s.setWidth( sh.width() );
-	}
-
-	if ( w->sizePolicy().mayShrinkVertically() ) {
-	    s.setHeight( w->minimumSizeHint().height() );
-	} else {
-	    s.setHeight( sh.isValid() ? sh.height() : w->sizeHint().height() );
-	}
-    }
-    s = s.boundedTo( w->maximumSize() );
-    QSize min = w->minimumSize();
-    if ( min.width() > 0 )
-	s.setWidth( min.width() );
-    if ( min.height() > 0 )
-	s.setHeight( min.height() );
-
-    if ( i->hasHeightForWidth() && min.height() == 0 && min.width() > 0 )
-	s.setHeight( i->heightForWidth( s.width()) );
-
-    s = s.expandedTo( QSize(1, 1) );
-    return s;
-}
-
-// Returns the max size of a box containing \a w with alignment \a align.
-static QSize smartMaxSize( const QWidgetItem *i, int align = 0 )
-{
-    QWidget *w = ( (QWidgetItem*)i )->widget();
-    if ( align & Qt::AlignHorizontal_Mask && align & Qt::AlignVertical_Mask )
-	return QSize( QLAYOUTSIZE_MAX, QLAYOUTSIZE_MAX );
-    QSize s = w->maximumSize();
-    if ( s.width() == QWIDGETSIZE_MAX && !(align&Qt::AlignHorizontal_Mask) )
-	if ( !w->sizePolicy().mayGrowHorizontally() )
-	    s.setWidth( w->sizeHint().width() );
-
-    if ( s.height() ==  QWIDGETSIZE_MAX && !(align&Qt::AlignVertical_Mask) )
-	if ( !w->sizePolicy().mayGrowVertically() )
-	    s.setHeight( w->sizeHint().height() );
-
-    s = s.expandedTo( w->minimumSize() );
-
-    if (align & Qt::AlignHorizontal_Mask )
-	s.setWidth( QLAYOUTSIZE_MAX );
-    if (align & Qt::AlignVertical_Mask )
-	s.setHeight( QLAYOUTSIZE_MAX );
-    return s;
-}
-
 /*!
     Stores the spacer item's rect \a r so that it can be returned by
     geometry().
@@ -392,7 +341,7 @@ void QSpacerItem::setGeometry( const QRect &r )
 */
 void QWidgetItem::setGeometry( const QRect &r )
 {
-    QSize s = r.size().boundedTo( smartMaxSize( this ) );
+    QSize s = r.size().boundedTo( qSmartMaxSize( this ) );
     int x = r.x();
     int y = r.y();
     if ( align & (Qt::AlignHorizontal_Mask | Qt::AlignVertical_Mask) ) {
@@ -480,8 +429,9 @@ int QWidgetItem::heightForWidth( int w ) const
 }
 
 /*!
-    Returns TRUE if this spacer item is expanding; otherwise returns
-    FALSE.
+    Returns the direction in which this spacer item will expand.
+
+    \sa QSizePolicy::ExpandData
 */
 QSizePolicy::ExpandData QSpacerItem::expanding() const
 {
@@ -489,23 +439,36 @@ QSizePolicy::ExpandData QSpacerItem::expanding() const
 }
 
 /*!
-    Returns TRUE if this item's widget is expanding; otherwise returns
-    FALSE.
+    Returns whether this item's widget can make use of more space than
+    sizeHint(). A value of \c Vertical or \c Horizontal means that it wants
+    to grow in only one dimension, whereas \c BothDirections means that
+    it wants to grow in both dimensions and \c NoDirection means that
+    it doesn't want to grow at all.
 */
 QSizePolicy::ExpandData QWidgetItem::expanding() const
 {
     if ( isEmpty() )
 	return QSizePolicy::NoDirection;
-    if ( (align & Qt::AlignHorizontal_Mask) &&
-	 (align & Qt::AlignVertical_Mask) )
-	return QSizePolicy::NoDirection;
 
-    int e = wid->layout() ? wid->layout()->expanding()
-	     : wid->sizePolicy().expanding();
+    int e = wid->sizePolicy().expanding();
+    /*
+      If the layout is expanding, we make the widget expanding, even if
+      its own size policy isn't expanding. This behavior should be
+      reconsidered in Qt 4.0. (###)
+    */
+    if ( wid->layout() ) {
+	if ( wid->sizePolicy().mayGrowHorizontally()
+		&& (wid->layout()->expanding() & QSizePolicy::Horizontally) )
+	    e |= QSizePolicy::Horizontally;
+	if ( wid->sizePolicy().mayGrowVertically()
+		&& (wid->layout()->expanding() & QSizePolicy::Vertically) )
+	    e |= QSizePolicy::Vertically;
+    }
+
     if ( align & Qt::AlignHorizontal_Mask )
-	e = e & ~QSizePolicy::Horizontally;
-    else if ( align & Qt::AlignVertical_Mask)
-	e = e & ~QSizePolicy::Vertically;
+	e &= ~QSizePolicy::Horizontally;
+    if ( align & Qt::AlignVertical_Mask)
+	e &= ~QSizePolicy::Vertically;
     return (QSizePolicy::ExpandData)e;
 }
 
@@ -525,7 +488,7 @@ QSize QWidgetItem::minimumSize() const
 {
     if ( isEmpty() )
 	return QSize( 0, 0 );
-    return smartMinSize( this );
+    return qSmartMinSize( this );
 }
 
 /*!
@@ -542,9 +505,11 @@ QSize QSpacerItem::maximumSize() const
 */
 QSize QWidgetItem::maximumSize() const
 {
-    if ( isEmpty() )
+    if ( isEmpty() ) {
 	return QSize( 0, 0 );
-    return smartMaxSize( this, align );
+    } else {
+	return qSmartMaxSize( this, align );
+    }
 }
 
 /*!
@@ -604,7 +569,7 @@ bool QWidgetItem::isEmpty() const
 
     For users of QLayout subclasses or of QMainWindow there is seldom
     any need to use the basic functions provided by QLayout, such as
-    \l resizeMode or setMenuBar(). See the \link layout.html layout
+    \l setResizeMode() or setMenuBar(). See the \link layout.html layout
     overview page \endlink for more information.
 
     To make your own layout manager, subclass QGLayoutIterator and
@@ -644,8 +609,6 @@ QLayout::QLayout( QWidget *parent, int margin, int spacing, const char *name )
 	    parent->removeChild( this );
 	} else {
 	    topLevel = TRUE;
-	    if ( parent->isTopLevel() )
-		autoMinimum = TRUE;
 	    parent->installEventFilter( this );
 	    setWidgetLayout( parent, this );
 	}
@@ -662,16 +625,17 @@ void QLayout::init()
     insideSpacing = 0;
     outsideBorder = 0;
     topLevel = FALSE;
-    autoMinimum = FALSE;
+    enabled = TRUE;
     autoNewChild = FALSE;
     frozen = FALSE;
     activated = FALSE;
     marginImpl = FALSE;
+    autoMinimum = FALSE;
+    autoResizeMode = TRUE;
     extraData = 0;
 #ifndef QT_NO_MENUBAR
     menubar = 0;
 #endif
-    enabled = TRUE;
 }
 
 /*!
@@ -784,10 +748,11 @@ void QLayout::setMargin( int margin )
     }
 }
 
-//##### bool recursive = FALSE ????
 void QLayout::setSpacing( int spacing )
 {
     insideSpacing = spacing;
+    if ( spacing >= 0 )
+	propagateSpacing( this );
     invalidate();
     if ( mainWidget() ) {
 	QEvent *lh = new QEvent( QEvent::LayoutHint );
@@ -803,14 +768,14 @@ QWidget * QLayout::mainWidget()
 {
     if ( !topLevel ) {
 	if ( parent() ) {
-	    Q_ASSERT( parent()->inherits( "QLayout" ) );
+	    Q_ASSERT( ::qt_cast<QLayout*>(parent()) );
 	    return ((QLayout*)parent())->mainWidget();
 	} else {
 	    return 0;
 	}
     } else {
 	Q_ASSERT( parent() && parent()->isWidgetType() );
-	return	(QWidget*)parent();
+	return (QWidget*)parent();
     }
 }
 
@@ -850,34 +815,24 @@ void QLayout::invalidate()
     rect = QRect();
 }
 
-// ###
-
-// this is an ugly workaround. for some reason, the layout system
-// would add TWO QWidgetItems for some QWidgets, and can then later
-// delete one and derefence the other. the scope of this bug is
-// unknown. I submit this hack so that we can work while I look for
-// the 'double-add' bug.
-
-// ###
-
-static bool removeWidget( QLayoutItem *lay, QWidget *w )
+static bool removeWidgetRecursively( QLayoutItem *lay, QWidget *w )
 {
-    bool foo = FALSE;
+    bool didSomething = FALSE;
     QLayoutIterator it = lay->iterator();
     QLayoutItem *child;
-    while ( (child = it.current() ) ) {
+    while ( (child = it.current()) != 0 ) {
 	if ( child->widget() == w ) {
 	    it.deleteCurrent();
-	    lay->invalidate();
-	    foo = TRUE;
-	} else if ( removeWidget( child, w ) ) {
-	    lay->invalidate();
-	    foo = TRUE;
+	    lay->invalidate(); // maybe redundant
+	    didSomething = TRUE;
+	} else if ( removeWidgetRecursively(child, w) ) {
+	    lay->invalidate(); // maybe redundant
+	    didSomething = TRUE;
 	} else {
 	    ++it;
 	}
     }
-    return foo;
+    return didSomething;
 }
 
 /*!
@@ -897,29 +852,28 @@ bool QLayout::eventFilter( QObject *o, QEvent *e )
     switch ( e->type() ) {
     case QEvent::Resize:
 	if ( activated ) {
-	    QResizeEvent *r = (QResizeEvent*)e;
+	    QResizeEvent *r = (QResizeEvent *)e;
 	    int mbh = 0;
 #ifndef QT_NO_MENUBAR
-	    if ( menubar && !menubar->isHidden() && !menubar->isTopLevel() )
-		mbh = menubar->heightForWidth( r->size().width() );
+	    mbh = menuBarHeightForWidth( menubar, r->size().width() );
 #endif
 	    int b = marginImpl ? 0 : outsideBorder;
-	    setGeometry( QRect( b, mbh + b, r->size().width() - 2*b,
-				r->size().height() - mbh - 2*b ) );
+	    setGeometry( QRect( b, mbh + b, r->size().width() - 2 * b,
+				r->size().height() - mbh - 2 * b ) );
 	} else {
 	    activate();
 	}
 	break;
     case QEvent::ChildRemoved:
 	{
-	    QChildEvent *c = (QChildEvent*)e;
+	    QChildEvent *c = (QChildEvent *)e;
 	    if ( c->child()->isWidgetType() ) {
-		QWidget *w = (QWidget*)c->child();
+		QWidget *w = (QWidget *)c->child();
 #ifndef QT_NO_MENUBAR
 		if ( w == menubar )
 		    menubar = 0;
 #endif
-		if ( removeWidget( this, w ) ) {
+		if ( removeWidgetRecursively( this, w ) ) {
 		    QEvent *lh = new QEvent( QEvent::LayoutHint );
 		    QApplication::postEvent( o, lh );
 		}
@@ -928,13 +882,13 @@ bool QLayout::eventFilter( QObject *o, QEvent *e )
 	break;
     case QEvent::ChildInserted:
 	if ( topLevel && autoNewChild ) {
-	    QChildEvent *c = (QChildEvent*)e;
+	    QChildEvent *c = (QChildEvent *)e;
 	    if ( c->child()->isWidgetType() ) {
-		QWidget *w = (QWidget*)c->child();
+		QWidget *w = (QWidget *)c->child();
 		if ( !w->isTopLevel() ) {
-#ifndef QT_NO_MENUBAR
-		    if ( w->inherits( "QMenuBar" ) && ( !w->parent() || !w->parent()->inherits( "QToolBar" ) ) )
-			menubar = (QMenuBar*)w;
+#if !defined(QT_NO_MENUBAR) && !defined(QT_NO_TOOLBAR)
+		    if ( ::qt_cast<QMenuBar*>(w) && !::qt_cast<QToolBar*>(w->parentWidget()) )
+			menubar = (QMenuBar *)w;
 		    else
 #endif
 			addItem( new QWidgetItem( w ) );
@@ -969,9 +923,10 @@ void QLayout::childEvent( QChildEvent *e )
 	    if ( item == (QLayout*)c->child() ) {
 		it.takeCurrent();
 		invalidate();
-		return;
+		break;
+	    } else {
+		++it;
 	    }
-	    ++it;
 	}
     }
 }
@@ -991,8 +946,7 @@ int QLayout::totalHeightForWidth( int w ) const
     int b = ( topLevel && !marginImpl ) ? 2 * outsideBorder : 0;
     int h = heightForWidth( w - b ) + b;
 #ifndef QT_NO_MENUBAR
-    if ( menubar && !menubar->isTopLevel() )
-	h += menubar->heightForWidth( w );
+    h += menuBarHeightForWidth( menubar, w );
 #endif
     return h;
 }
@@ -1013,8 +967,7 @@ QSize QLayout::totalMinimumSize() const
     QSize s = minimumSize();
     int h = b;
 #ifndef QT_NO_MENUBAR
-    if ( menubar && !menubar->isTopLevel() )
-	h += menubar->heightForWidth( s.width() );
+    h += menuBarHeightForWidth( menubar, s.width() );
 #endif
     return s + QSize( b, h );
 }
@@ -1027,9 +980,8 @@ QSize QLayout::totalSizeHint() const
 {
     if ( topLevel ) {
 	QWidget *mw = (QWidget*)parent();
-	if ( mw && !mw->testWState(WState_Polished) ) {
+	if ( mw && !mw->testWState(WState_Polished) )
 	    mw->polish();
-	}
     }
     int b = ( topLevel && !marginImpl ) ? 2 * outsideBorder : 0;
 
@@ -1038,8 +990,7 @@ QSize QLayout::totalSizeHint() const
 	s.setHeight( heightForWidth(s.width()) );
     int h = b;
 #ifndef QT_NO_MENUBAR
-    if ( menubar && !menubar->isTopLevel() )
-	h += menubar->heightForWidth( s.width() );
+    h += menuBarHeightForWidth( menubar, s.width() );
 #endif
     return s + QSize( b, h );
 }
@@ -1056,13 +1007,12 @@ QSize QLayout::totalMaximumSize() const
 	    mw->polish();
 	}
     }
-    int b = (topLevel && !marginImpl) ? 2*outsideBorder : 0;
+    int b = ( topLevel && !marginImpl ) ? 2 * outsideBorder : 0;
 
     QSize s = maximumSize();
     int h = b;
 #ifndef QT_NO_MENUBAR
-    if ( menubar && !menubar->isTopLevel() )
-	h += menubar->heightForWidth( s.width() );
+    h += menuBarHeightForWidth( menubar, s.width() );
 #endif
 
     if ( isTopLevel() )
@@ -1097,7 +1047,7 @@ void QLayout::deleteAllItems()
 {
     QLayoutIterator it = iterator();
     QLayoutItem *l;
-    while ( (l=it.takeCurrent()) )
+    while ( (l = it.takeCurrent()) )
 	delete l;
 }
 
@@ -1109,13 +1059,15 @@ void QLayout::addChildLayout( QLayout *l )
 {
     if ( l->parent() ) {
 #if defined(QT_CHECK_NULL)
-	qWarning( "QLayout::addChildLayout(), layout already has a parent." );
+	qWarning( "QLayout::addChildLayout: layout already has a parent" );
 #endif
 	return;
     }
     insertChild( l );
-    if ( l->insideSpacing < 0 )
+    if ( l->insideSpacing < 0 ) {
 	l->insideSpacing = insideSpacing;
+	propagateSpacing( l );
+    }
 }
 
 /*! \fn int QLayout::defaultBorder() const
@@ -1191,23 +1143,23 @@ QSize QLayout::maximumSize() const
 
 /*!
     Returns whether this layout can make use of more space than
-    sizeHint(). A value of Vertical or Horizontal means that it wants
-    to grow in only one dimension, whereas BothDirections means that
+    sizeHint(). A value of \c Vertical or \c Horizontal means that it wants
+    to grow in only one dimension, whereas \c BothDirections means that
     it wants to grow in both dimensions.
 
-    The default implementation returns BothDirections.
+    The default implementation returns \c BothDirections.
 */
 QSizePolicy::ExpandData QLayout::expanding() const
 {
     return QSizePolicy::BothDirections;
 }
 
-static void  invalidateRecursive( QLayoutItem *lay )
+static void invalidateRecursive( QLayoutItem *lay )
 {
     lay->invalidate();
     QLayoutIterator it = lay->iterator();
     QLayoutItem *child;
-    while ( (child = it.current() ) ) {
+    while ( (child = it.current()) != 0 ) {
 	invalidateRecursive( child );
 	++it;
     }
@@ -1226,37 +1178,55 @@ static void  invalidateRecursive( QLayoutItem *lay )
 */
 bool QLayout::activate()
 {
-    // Paul: If adding stuff to a QLayout for a widget causes
-    // postEvent(thatWidget, QEvent::LayoutHint), activate() becomes
-    // unnecessary in that case too.
     invalidateRecursive( this );
     if ( !topLevel )
 	return FALSE;
 
-    QWidget *mainW = mainWidget();
-    if ( !mainW ) {
+    if ( mainWidget() == 0 ) {
 #if defined( QT_CHECK_NULL )
-	qWarning( "QLayout::activate(): %s \"%s\" does not have a main widget.",
+	qWarning( "QLayout::activate: %s \"%s\" does not have a main widget",
 		  QObject::className(), QObject::name() );
 #endif
 	return FALSE;
     }
     activated = TRUE;
     QSize s = mainWidget()->size();
+    QSize ms;
     int mbh = 0;
 #ifndef QT_NO_MENUBAR
-    mbh = menubar && !menubar->isTopLevel() ? menubar->heightForWidth( s.width() ) : 0;
+    mbh = menuBarHeightForWidth( menubar, s.width() );
 #endif
     int b = marginImpl ? 0 : outsideBorder;
-    setGeometry( QRect( b, mbh + b,
-			s.width() - 2 * b,
-			s.height() - mbh - 2 * b ) );
-    if ( frozen )
-	mainWidget()->setFixedSize( totalSizeHint() ); //### will trigger resize
-    else if ( autoMinimum )
-	mainWidget()->setMinimumSize( totalMinimumSize() );
+    setGeometry( QRect(b, mbh + b, s.width() - 2 * b,
+		       s.height() - mbh - 2 * b) );
+    if ( frozen ) {
+	// will trigger resize
+	mainWidget()->setFixedSize( totalSizeHint() );
+    } else if ( autoMinimum ) {
+	ms = totalMinimumSize();
+    } else if ( autoResizeMode && topLevel && mainWidget()
+		&& mainWidget()->isTopLevel() ) {
+	ms = totalMinimumSize();
+	if ( hasHeightForWidth() ) {
+	    int h;
 
-    //###if ( sizeHint or sizePolicy has changed )
+	    // ### 4.0: remove this 'if' when minimumHeightForWidth() is virtual
+	    if ( inherits("QBoxLayout") ) {
+		h = ((QBoxLayout *) this)->minimumHeightForWidth( ms.width() );
+	    } else if ( inherits("QGridLayout") ) {
+		h = ((QGridLayout *) this)->minimumHeightForWidth( ms.width() );
+	    } else {
+		h = heightForWidth( ms.width() );
+	    }
+	    if ( h > ms.height() )
+		ms = QSize( 0, 0 );
+	}
+    }
+
+    if ( ms.isValid() )
+	mainWidget()->setMinimumSize( ms );
+
+    // ideally only if sizeHint() or sizePolicy() has changed
     mainWidget()->updateGeometry();
     return TRUE;
 }
@@ -1282,12 +1252,11 @@ bool QLayout::activate()
     vertical size policy. It also contains a flag to indicate whether the
     height and width of its preferred size are related.
 
-    The per-dimension SizeType objects are set in the usual constructor
-    and can be queried using a variety of functions, none of which are
-    really interesting to application programmers.
+    The horizontal and vertical \l{SizeType}s are set in the usual constructor
+    and can be queried using a variety of functions.
 
     The hasHeightForWidth() flag indicates whether the widget's sizeHint()
-    is width-dependent (such as a word-wrapping label).
+    is width-dependent (such as a word-wrapping label) or not.
 
     \sa QSizePolicy::SizeType
 */
@@ -1326,9 +1295,6 @@ bool QLayout::activate()
 
     \value Ignored the sizeHint() is ignored. The widget will get as
     much space as possible.
-
-    In any case, QLayout never shrinks a widget below the
-    QWidget::minimumSizeHint().
 */
 
 /*!
@@ -1351,9 +1317,9 @@ bool QLayout::activate()
 */
 
 /*!
-    \fn QSizePolicy::QSizePolicy ()
+    \fn QSizePolicy::QSizePolicy()
 
-    Default constructor; produces a minimally initialized QSizePolicy.
+    Constructs a minimally initialized QSizePolicy.
 */
 
 /*!
@@ -1444,9 +1410,10 @@ bool QLayout::activate()
 /*!
     \fn QSizePolicy::ExpandData QSizePolicy::expanding() const
 
-    Returns a value indicating whether the widget can make use of
-    extra space (i.e. if it "wants" to grow) horizontally and/or
-    vertically.
+    Returns whether this layout can make use of more space than
+    sizeHint(). A value of \c Vertical or \c Horizontal means that it wants
+    to grow in only one dimension, whereas \c BothDirections means that
+    it wants to grow in both dimensions.
 
     \sa mayShrinkHorizontally() mayGrowHorizontally()
 	mayShrinkVertically() mayGrowVertically()
@@ -1518,6 +1485,12 @@ bool QLayout::activate()
     \sa verStretch() setHorStretch()
 */
 
+/*!
+    \fn void QSizePolicy::transpose()
+
+    Swaps the horizontal and vertical policies and stretches.
+*/
+
 
 /*!
     \fn bool QSizePolicy::operator==( const QSizePolicy &s ) const
@@ -1617,7 +1590,7 @@ QGLayoutIterator::~QGLayoutIterator()
 	QLayoutItem *child;
 	while ( (child = it.current()) != 0 ) {
 	    paintLayout( p, child );
-	    it.next();
+	    ++it;
 	}
 	p->drawRect( lay->geometry() );
     }
@@ -1702,6 +1675,10 @@ QGLayoutIterator::~QGLayoutIterator()
 
     The possible values are:
 
+    \value Auto  If the main widget is a top-level widget with no
+		 height-for-width (hasHeightForWidth()), this is
+		 the same as \c Minimium; otherwise, this is the
+		 same as \c FreeResize.
     \value Fixed  The main widget's size is set to sizeHint(); it
 		  cannot be resized at all.
     \value Minimum  The main widget's minimum size is set to
@@ -1713,8 +1690,7 @@ QGLayoutIterator::~QGLayoutIterator()
     \property QLayout::resizeMode
     \brief the resize mode of the layout
 
-    The default mode is \c Minimum for top-level widgets and \c
-    FreeResize for all others.
+    The default mode is \c Auto.
 
     \sa QLayout::ResizeMode
 */
@@ -1723,25 +1699,36 @@ void QLayout::setResizeMode( ResizeMode mode )
 {
     if ( mode == resizeMode() )
 	return;
-    switch (mode) {
+
+    switch ( mode ) {
+    case Auto:
+	frozen = FALSE;
+	autoMinimum = FALSE;
+	autoResizeMode = TRUE;
+	break;
     case Fixed:
 	frozen = TRUE;
+	autoMinimum = FALSE;
+	autoResizeMode = FALSE;
 	break;
     case FreeResize:
 	frozen = FALSE;
 	autoMinimum = FALSE;
+	autoResizeMode = FALSE;
 	break;
     case Minimum:
 	frozen = FALSE;
 	autoMinimum = TRUE;
-	break;
+	autoResizeMode = FALSE;
     }
-    activate();
+    if ( mainWidget() && mainWidget()->isVisible() )
+	activate();
 }
 
 QLayout::ResizeMode QLayout::resizeMode() const
 {
-    return frozen ? Fixed : ( autoMinimum ? Minimum : FreeResize );
+    return ( autoResizeMode ? Auto :
+	     (frozen ? Fixed : (autoMinimum ? Minimum : FreeResize)) );
 }
 
 /*!
@@ -1810,31 +1797,93 @@ QRect QLayout::alignmentRect( const QRect &r ) const
 {
     QSize s = sizeHint();
     int a = alignment();
-    if ( expanding() & QSizePolicy::Horizontally || !(a & Qt::AlignHorizontal_Mask ) ) {
-	s.setWidth( r.width() );
+
+    /*
+      This is a hack to obtain the real maximum size, not
+      QSize(QLAYOUTSIZE_MAX, QLAYOUTSIZE_MAX), the value consistently
+      returned by QLayoutItems that have an alignment.
+    */
+    QLayout *that = (QLayout *) this;
+    that->setAlignment( 0 );
+    QSize ms = maximumSize();
+    that->setAlignment( a );
+
+    if ( (expanding() & QSizePolicy::Horizontally) ||
+	 !(a & Qt::AlignHorizontal_Mask ) ) {
+	s.setWidth( QMIN(r.width(), ms.width()) );
     }
-    if ( expanding() & QSizePolicy::Vertically || !(a & Qt::AlignVertical_Mask) ) {
-	s.setHeight( r.height() );
+    if ( (expanding() & QSizePolicy::Vertically) ||
+	 !(a & Qt::AlignVertical_Mask) ) {
+	s.setHeight( QMIN(r.height(), ms.height()) );
     } else if ( hasHeightForWidth() ) {
-	s.setHeight( QMIN( s.height(), heightForWidth(s.width()) ) );
+	int hfw = heightForWidth( s.width() );
+	if ( hfw < s.height() )
+	    s.setHeight( QMIN(hfw, ms.height()) );
     }
 
     int x = r.x();
     int y = r.y();
 
     if ( a & Qt::AlignBottom )
-	y = y + ( r.height() - s.height() );
+	y += ( r.height() - s.height() );
     else if ( !(a & Qt::AlignTop) )
-	y = y + ( r.height() - s.height() ) / 2;
+	y += ( r.height() - s.height() ) / 2;
 
     a = QApplication::horizontalAlignment( a );
     if ( a & Qt::AlignRight )
-	x = x + ( r.width() - s.width() );
+	x += ( r.width() - s.width() );
     else if ( !(a & Qt::AlignLeft) )
-	x = x + ( r.width() - s.width() ) / 2;
+	x += ( r.width() - s.width() ) / 2;
 
     return QRect( x, y, s.width(), s.height() );
+}
 
+/*!
+    Removes the widget \a widget from the layout. After this call, it
+    is the caller's responsibility to give the widget a reasonable
+    geometry or to put the widget back into a layout.
+
+    \sa removeItem(), QWidget::setGeometry(), add()
+*/
+void QLayout::remove( QWidget *widget )
+{
+    QLayoutIterator it = iterator();
+    QLayoutItem *child;
+    while ( (child = it.current()) != 0 ) {
+	if ( child->widget() == widget ) {
+	    it.deleteCurrent();
+	    invalidate(); // maybe redundant
+	    QApplication::postEvent( mainWidget(),
+				     new QEvent(QEvent::LayoutHint) );
+	} else {
+	    ++it;
+	}
+    }
+}
+
+/*!
+    Removes the layout item \a item from the layout. It is the
+    caller's responsibility to delete the item.
+
+    Notice that \a item can be a layout (since QLayout inherits
+    QLayoutItem).
+
+    \sa remove(), addItem()
+*/
+void QLayout::removeItem( QLayoutItem *item )
+{
+    QLayoutIterator it = iterator();
+    QLayoutItem *child;
+    while ( (child = it.current()) != 0 ) {
+	if ( child == item ) {
+	    it.takeCurrent();
+	    invalidate(); // maybe redundant
+	    QApplication::postEvent( mainWidget(),
+				     new QEvent(QEvent::LayoutHint) );
+	} else {
+	    ++it;
+	}
+    }
 }
 
 /*!
@@ -1862,4 +1911,18 @@ bool QLayout::isEnabled() const
     return enabled;
 }
 
+void QLayout::propagateSpacing( QLayout *parent )
+{
+    QLayoutIterator it = parent->iterator();
+    QLayoutItem *child;
+    while ( (child = it.current()) ) {
+	QLayout *childLayout = child->layout();
+	if ( childLayout && childLayout->insideSpacing < 0 ) {
+	    childLayout->insideSpacing = parent->insideSpacing;
+	    propagateSpacing( childLayout );
+	}
+	++it;
+    }
+}
+
 #endif // QT_NO_LAYOUT
