@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/qaccel.cpp   3.1.2   edited Jan 31 11:20 $
+** $Id: qt/qaccel.cpp   3.2.3   edited Jul 4 13:16 $
 **
 ** Implementation of QAccel class
 **
@@ -46,6 +46,7 @@
 #include "qwhatsthis.h"
 #include "qguardedptr.h"
 #include "qstatusbar.h"
+#include "qdockwindow.h"
 #include "qsignalslotimp.h"
 /*!
     \class QAccel qaccel.h
@@ -173,6 +174,7 @@ public:
     void unregisterAccel( QAccelPrivate* a ) { accels.removeRef( a ); if ( accels.isEmpty() ) delete this; }
     bool tryAccelEvent( QWidget* w, QKeyEvent* e );
     bool dispatchAccelEvent( QWidget* w, QKeyEvent* e );
+    bool tryComposeUnicode( QWidget* w, QKeyEvent* e );
 
 private:
     QAccelManager():currentState(Qt::NoMatch), clash(-1) { self_ptr = this; }
@@ -198,24 +200,33 @@ bool Q_EXPORT qt_dispatchAccelEvent( QWidget* w, QKeyEvent*  e){
     return QAccelManager::self()->dispatchAccelEvent( w, e );
 }
 
+bool Q_EXPORT qt_tryComposeUnicode( QWidget* w, QKeyEvent*  e){
+    return QAccelManager::self()->tryComposeUnicode( w, e );
+}
+
 /*
     \internal
     Returns TRUE if the accel is in the current subwindow, else FALSE.
 */
 bool QAccelManager::correctSubWindow( QWidget* w, QAccelPrivate* d ) {
+#if !defined ( Q_OS_MACX )
     if ( !d->watch || !d->watch->isVisible() || !d->watch->isEnabled() )
+#else
+    if ( !d->watch || (!d->watch->isVisible() && !d->watch->inherits( "QMenuBar" )) || !d->watch->isEnabled() )
+#endif
 	return FALSE;
     QWidget* tlw = w->topLevelWidget();
     QWidget* wtlw = d->watch->topLevelWidget();
 
     /* if we live in a floating dock window, keep our parent's
      * accelerators working */
-    if ( tlw->isDialog() && tlw->parentWidget() && tlw->inherits( "QDockWindow" ) )
+#ifndef QT_NO_MAINWINDOW
+    if ( tlw->isDialog() && tlw->parentWidget() && ::qt_cast<QDockWindow*>(tlw) )
 	return tlw->parentWidget()->topLevelWidget() == wtlw;
 
     if ( wtlw  != tlw )
 	return FALSE;
-
+#endif
     /* if we live in a MDI subwindow, ignore the event if we are
        not the active document window */
     QWidget* sw = d->watch;
@@ -316,6 +327,55 @@ bool QAccelManager::tryAccelEvent( QWidget* w, QKeyEvent* e )
     return e->isAccepted();
 }
 
+bool QAccelManager::tryComposeUnicode( QWidget* w, QKeyEvent* e )
+{
+    if ( QApplication::metaComposeUnicode ) {
+	int value = e->key() - Key_0;
+	// Ignore acceloverrides so we don't trigger
+	// accels on keypad when Meta compose is on
+	if ( (e->type() == QEvent::AccelOverride) &&
+	     (e->state() == Qt::Keypad + Qt::MetaButton) ) {
+	    e->accept();
+	// Meta compose start/continue
+	} else if ( (e->type() == QEvent::KeyPress) &&
+	     (e->state() == Qt::Keypad + Qt::MetaButton) ) {
+	    if ( value >= 0 && value <= 9 ) {
+		QApplication::composedUnicode *= 10;
+		QApplication::composedUnicode += value;
+		return TRUE;
+	    } else {
+		// Composing interrupted, dispatch!
+		if ( QApplication::composedUnicode ) {
+		    QChar ch( QApplication::composedUnicode );
+		    QString s( ch );
+		    QKeyEvent kep( QEvent::KeyPress, 0, ch.row() ? 0 : ch.cell(), 0, s );
+		    QKeyEvent ker( QEvent::KeyRelease, 0, ch.row() ? 0 : ch.cell(), 0, s );
+		    QApplication::sendEvent( w, &kep );
+		    QApplication::sendEvent( w, &ker );
+		}
+		QApplication::composedUnicode = 0;
+		return TRUE;
+	    }
+	// Meta compose end, dispatch
+	} else if ( (e->type() == QEvent::KeyRelease) &&
+		    (e->key() == Key_Meta) &&
+		    (QApplication::composedUnicode != 0) ) {
+	    if ( (QApplication::composedUnicode > 0) && 
+		 (QApplication::composedUnicode < 0xFFFE) ) {
+		QChar ch( QApplication::composedUnicode );
+		QString s( ch );
+		QKeyEvent kep( QEvent::KeyPress, 0, ch.row() ? 0 : ch.cell(), 0, s );
+		QKeyEvent ker( QEvent::KeyRelease, 0, ch.row() ? 0 : ch.cell(), 0, s );
+		QApplication::sendEvent( w, &kep );
+		QApplication::sendEvent( w, &ker );
+	    }
+	    QApplication::composedUnicode = 0;
+	    return TRUE;
+	}
+    }
+    return FALSE;
+}
+
 /*
     \internal
     Checks for possible accelerators, if no widget
