@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id$
+** $Id: qt/qcolor.cpp   3.2.3   edited May 13 09:08 $
 **
 ** Implementation of QColor class
 **
@@ -38,8 +38,8 @@
 #include "qcolor.h"
 #include "qnamespace.h"
 #include "qdatastream.h"
-#include <stdlib.h>
-#include <ctype.h>
+
+#include <stdio.h>
 
 
 /*!
@@ -70,7 +70,7 @@
     red, \c darkRed, \c green, \c darkGreen, \c blue, \c darkBlue, \c
     cyan, \c darkCyan, \c magenta, \c darkMagenta, \c yellow, \c
     darkYellow, \c gray, \c darkGray, \c lightGray, \c color0 and \c
-    color1.
+    color1, accessible as members of the Qt namespace (ie. \c Qt::red).
 
     \img qt-colors.png Qt Colors
 
@@ -224,8 +224,16 @@ void QColor::initGlobalColors()
 {
     globals_init = TRUE;
 
+    #ifdef Q_WS_X11
+    // HACK: we need a way to recognize color0 and color1 uniquely, so
+    // that we can use color0 and color1 with fixed pixel values on
+    // all screens
+    stdcol[ 0].d.argb = qRgba(255, 255, 255, 1);
+    stdcol[ 1].d.argb = qRgba(  0,   0,   0, 1);
+    #else
     stdcol[ 0].d.argb = qRgb(255,255,255);
     stdcol[ 1].d.argb = 0;
+    #endif // Q_WS_X11
     stdcol[ 0].setPixel( COLOR0_PIX );
     stdcol[ 1].setPixel( COLOR1_PIX );
 
@@ -436,7 +444,6 @@ bool QColor::isDirty() const
     }
 }
 
-#ifndef QT_NO_SPRINTF
 /*!
     Returns the name of the color in the format "#RRGGBB", i.e. a "#"
     character followed by three two-digit hexadecimal numbers.
@@ -446,11 +453,16 @@ bool QColor::isDirty() const
 
 QString QColor::name() const
 {
+#ifndef QT_NO_SPRINTF
     QString s;
     s.sprintf( "#%02x%02x%02x", red(), green(), blue() );
     return s;
-}
+#else
+    char s[20];
+    sprintf( s, "#%02x%02x%02x", red(), green(), blue() );
+    return QString(s);
 #endif
+}
 
 static int hex2int( QChar hexchar )
 {
@@ -486,7 +498,12 @@ static int hex2int( QChar hexchar )
 void QColor::setNamedColor( const QString& name )
 {
     if ( name.isEmpty() ) {
-	setRgb( 0 );
+	d.argb = 0;
+	if ( colormodel == d8 ) {
+	    d.d8.invalid = TRUE;
+	} else {
+	    d.d32.argb = Invalid;
+	}
     } else if ( name[0] == '#' ) {
 	const QChar *p = name.unicode()+1;
 	int len = name.length()-1;
@@ -525,7 +542,8 @@ void QColor::setNamedColor( const QString& name )
   \obsolete
 */
 
-/*!
+/*!  \fn void QColor::getHsv( int *h, int *s, int *v ) const
+
     Returns the current RGB value as HSV. The contents of the \a h, \a
     s and \a v pointers are set to the HSV values. If any of the three
     pointers are null, the function does nothing.
@@ -536,6 +554,8 @@ void QColor::setNamedColor( const QString& name )
     \sa setHsv(), rgb()
 */
 
+/*! \obsolete Use getHsv() instead.
+ */
 void QColor::hsv( int *h, int *s, int *v ) const
 {
     if ( !h || !s || !v )
@@ -635,7 +655,7 @@ void QColor::setHsv( int h, int s, int v )
 
 
 /*!
-    \overload QRgb QColor::rgb() const
+    \fn QRgb QColor::rgb() const
 
     Returns the RGB value.
 
@@ -647,14 +667,16 @@ void QColor::setHsv( int h, int s, int v )
     \sa setRgb(), hsv(), qRed(), qBlue(), qGreen(), isValid()
 */
 
-/*!
+/*! \fn void QColor::getRgb( int *r, int *g, int *b ) const
+
     Sets the contents pointed to by \a r, \a g and \a b to the red,
     green and blue components of the RGB value respectively. The value
     range for a component is 0..255.
 
-    \sa setRgb(), hsv()
+    \sa rgb(), setRgb(), getHsv()
 */
 
+/*! \obsolete Use getRgb() instead */
 void QColor::rgb( int *r, int *g, int *b ) const
 {
     *r = qRed(d.argb);
@@ -829,12 +851,21 @@ uint QColor::pixel() const
     if ( isDirty() )
 	return ((QColor*)this)->alloc();
     else if ( colormodel == d8 )
+#ifdef Q_WS_WIN
+	// since d.d8.pix is uchar we have to use the PALETTEINDEX
+	// macro to get the respective palette entry index.
+	return (0x01000000 | (int)(short)(d.d8.pix));
+#else
 	return d.d8.pix;
+#endif
     else
 	return d.d32.pix;
 }
 
-
+/*!
+    \fn QStringList QColor::colorNames()
+    Returns a QStringList containing the color names Qt knows about.
+*/
 
 /*****************************************************************************
   QColor stream functions
@@ -946,3 +977,4 @@ QDataStream &operator>>( QDataStream &s, QColor &c )
 
     Returns a gray value 0..255 from the given \a rgb colour.
 */
+
