@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id:  qt/qdnd_x11.cpp   3.0.5   edited Apr 9 19:14 $
+** $Id: qt/qdnd_x11.cpp   3.2.3   edited Sep 22 17:17 $
 **
 ** XDND implementation for Qt.  See http://www.cco.caltech.edu/~jafl/xdnd/
 **
 ** Created : 980320
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2003 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the kernel module of the Qt GUI Toolkit.
 **
@@ -20,7 +20,7 @@
 **
 ** Licensees holding valid Qt Enterprise Edition or Qt Professional Edition
 ** licenses for Unix/X11 may use this file in accordance with the Qt Commercial
-** License Agreement provided with the Software.
+** License Agreement prqovided with the Software.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -35,6 +35,8 @@
 **
 **********************************************************************/
 
+#include "qplatformdefs.h"
+
 #include "qapplication.h"
 
 #ifndef QT_NO_DRAGANDDROP
@@ -43,12 +45,12 @@
 #include "qintdict.h"
 #include "qdatetime.h"
 #include "qdict.h"
+#include "qguardedptr.h"
 #include "qdragobject.h"
 #include "qobjectlist.h"
 #include "qcursor.h"
 
-#include "qt_x11.h"
-
+#include "qt_x11_p.h"
 
 // conflict resolution
 
@@ -108,7 +110,7 @@ Atom qt_xdnd_leave;
 Atom qt_xdnd_drop;
 Atom qt_xdnd_finished;
 Atom qt_xdnd_type_list;
-int qt_xdnd_version = 4;
+const int qt_xdnd_version = 4;
 
 // Actions
 //
@@ -175,7 +177,7 @@ static QIntDict<QCString> * qt_xdnd_drag_types = 0;
 static QDict<Atom> * qt_xdnd_atom_numbers = 0;
 
 // timer used when target wants "continuous" move messages (eg. scroll)
-static int heartbeat = 0;
+static int heartbeat = -1;
 // rectangle in which the answer will be the same
 static QRect qt_xdnd_source_sameanswer;
 //static QRect qt_xdnd_target_sameanswer;
@@ -185,24 +187,26 @@ static Window qt_xdnd_current_target;
 // window to send events to (always valid if qt_xdnd_current_target)
 static Window qt_xdnd_current_proxy_target;
 // widget we forwarded position to last, and local position
-static QWidget * qt_xdnd_current_widget;
+static QGuardedPtr<QWidget> qt_xdnd_current_widget;
 static QPoint qt_xdnd_current_position;
 // time of this drop, as type Atom to save on casts
 static Atom qt_xdnd_source_current_time;
 //NOTUSED static Atom qt_xdnd_target_current_time;
+// screen number containing the pointer... -1 means default
 static int qt_xdnd_current_screen = -1;
 // state of dragging... true if dragging, false if not
 bool qt_xdnd_dragging = FALSE;
 
 // dict of payload data, sorted by type atom
-QIntDict<QByteArray> * qt_xdnd_target_data = 0;
+static QIntDict<QByteArray> * qt_xdnd_target_data = 0;
 
 // first drag object, or 0
-QDragObject * qt_xdnd_source_object = 0;
+static QDragObject * qt_xdnd_source_object = 0;
 
 // Motif dnd
 extern void qt_motifdnd_enable( QWidget *, bool );
-extern QByteArray qt_motifdnd_obtain_data();
+extern QByteArray qt_motifdnd_obtain_data( const char *format );
+extern const char *qt_motifdnd_format( int n );
 
 bool qt_motifdnd_active = FALSE;
 static bool dndCancelled = FALSE;
@@ -243,32 +247,27 @@ static const char* const default_pm[] = {
 };
 
 class QShapedPixmapWidget : public QWidget {
-    QPixmap pixmap;
+
 public:
     QShapedPixmapWidget(int screen = -1) :
 	QWidget(QApplication::desktop()->screen( screen ),
-		0,WStyle_Customize | WStyle_Tool | WStyle_NoBorder | WX11BypassWM )
+		0, WStyle_Customize | WStyle_Tool | WStyle_NoBorder | WX11BypassWM )
     {
     }
 
     void setPixmap(QPixmap pm)
     {
-	pixmap = pm;
-	if ( pixmap.mask() ) {
-	    setMask( *pixmap.mask() );
+	if ( pm.mask() ) {
+	    setMask( *pm.mask() );
 	} else {
 	    clearMask();
 	}
 	resize(pm.width(),pm.height());
-    }
-
-    void paintEvent(QPaintEvent*)
-    {
-	bitBlt(this,0,0,&pixmap);
+	setErasePixmap(pm);
     }
 };
 
-QShapedPixmapWidget * qt_xdnd_deco = 0;
+static QShapedPixmapWidget * qt_xdnd_deco = 0;
 
 static QWidget* desktop_proxy = 0;
 
@@ -365,7 +364,7 @@ const char* qt_xdnd_atom_to_str( Atom a )
     }
     QCString* result;
     if ( !(result=qt_xdnd_drag_types->find( a )) ) {
-	const char* mimeType = XGetAtomName( qt_xdisplay(), a );
+	const char* mimeType = XGetAtomName( QPaintDevice::x11AppDisplay(), a );
 	if ( !mimeType )
 	    return 0; // only happens on protocol error
 	result = new QCString( mimeType );
@@ -456,6 +455,9 @@ static QWidget * find_child( QWidget * tlw, QPoint & p )
     bool done = FALSE;
     while ( !done ) {
 	done = TRUE;
+	if ( ((QExtraWidget*)w)->extraData() &&
+	     ((QExtraWidget*)w)->extraData()->xDndProxy != 0 )
+	    break; // stop searching for widgets under the mouse cursor if found widget is a proxy.
 	if ( w->children() ) {
 	    QObjectListIt it( *w->children() );
 	    it.toLast();
@@ -498,13 +500,13 @@ static bool checkEmbedded(QWidget* w, const XEvent* xe)
 	if (current_embedding_widget != w) {
 
 	    last_enter_event.xany.window = extra->xDndProxy;
-	    XSendEvent( qt_xdisplay(), extra->xDndProxy, False, NoEventMask,
+	    XSendEvent( QPaintDevice::x11AppDisplay(), extra->xDndProxy, False, NoEventMask,
 			&last_enter_event );
 	    current_embedding_widget = w;
 	}
 
 	((XEvent*)xe)->xany.window = extra->xDndProxy;
-	XSendEvent( qt_xdisplay(), extra->xDndProxy, False, NoEventMask,
+	XSendEvent( QPaintDevice::x11AppDisplay(), extra->xDndProxy, False, NoEventMask,
 		    (XEvent*)xe );
 	qt_xdnd_current_widget = w;
 	return TRUE;
@@ -539,7 +541,7 @@ void qt_handle_xdnd_enter( QWidget *, const XEvent * xe, bool /*passive*/ )
 	int f;
 	unsigned long n, a;
 	Atom *data;
-	XGetWindowProperty( qt_xdisplay(), qt_xdnd_dragsource_xid,
+	XGetWindowProperty( QPaintDevice::x11AppDisplay(), qt_xdnd_dragsource_xid,
 		    qt_xdnd_type_list, 0,
 		    qt_xdnd_max_type, False, XA_ATOM, &type, &f,&n,&a,(uchar**)&data );
 	for ( ; j<qt_xdnd_max_type && j < (int)n; j++ ) {
@@ -602,7 +604,7 @@ void qt_handle_xdnd_position( QWidget *w, const XEvent * xe, bool passive )
 	QDropEvent::Action accepted_action = xdndaction_to_qtaction(l[4]);
 	me.setAction(accepted_action);
 
-	if ( qt_xdnd_current_widget != c ) {
+	if ( c != qt_xdnd_current_widget ) {
 	    qt_xdnd_target_answerwas = FALSE;
 	    if ( qt_xdnd_current_widget ) {
 		QDragLeaveEvent e;
@@ -683,7 +685,7 @@ void qt_handle_xdnd_position( QWidget *w, const XEvent * xe, bool passive )
     if ( source )
 	qt_handle_xdnd_status( source, (const XEvent *)&response, passive );
     else
-	XSendEvent( qt_xdisplay(), qt_xdnd_dragsource_xid, False,
+	XSendEvent( QPaintDevice::x11AppDisplay(), qt_xdnd_dragsource_xid, False,
 		    emask, (XEvent*)&response );
 }
 
@@ -769,7 +771,7 @@ void qt_xdnd_send_leave()
     if ( w )
 	qt_handle_xdnd_leave( w, (const XEvent *)&leave, FALSE );
     else
-	XSendEvent( qt_xdisplay(), qt_xdnd_current_proxy_target, False,
+	XSendEvent( QPaintDevice::x11AppDisplay(), qt_xdnd_current_proxy_target, False,
 		    emask, (XEvent*)&leave );
     qt_xdnd_current_target = 0;
     qt_xdnd_current_proxy_target = 0;
@@ -817,7 +819,7 @@ void qt_handle_xdnd_drop( QWidget *, const XEvent * xe, bool passive )
 	finished.message_type = qt_xdnd_finished;
 	finished.data.l[0] = qt_xdnd_current_widget?qt_xdnd_current_widget->topLevelWidget()->winId():0;
 	finished.data.l[1] = 0; // flags
-	XSendEvent( qt_xdisplay(), qt_xdnd_dragsource_xid, False,
+	XSendEvent( QPaintDevice::x11AppDisplay(), qt_xdnd_dragsource_xid, False,
 		    NoEventMask, (XEvent*)&finished );
     } else {
 	QDragLeaveEvent e;
@@ -996,8 +998,10 @@ void QDragManager::updateCursor()
 	} else {
 	    c = moveCursor;
 	}
-	if ( qt_xdnd_deco )
+	if ( qt_xdnd_deco ) {
 	    qt_xdnd_deco->show();
+	    qt_xdnd_deco->raise();
+	}
     } else {
 	c = noDropCursor;
 	//if ( qt_xdnd_deco )
@@ -1012,6 +1016,8 @@ void QDragManager::updateCursor()
 
 void QDragManager::cancel( bool deleteSource )
 {
+    killTimer( heartbeat );
+    heartbeat = -1;
     if ( object ) {
 	beingCancelled = TRUE;
 	object = 0;
@@ -1045,7 +1051,7 @@ Window findRealWindow( const QPoint & pos, Window w, int md )
 
     if ( md ) {
 	XWindowAttributes attr;
-	XGetWindowAttributes( qt_xdisplay(), w, &attr );
+	XGetWindowAttributes( QPaintDevice::x11AppDisplay(), w, &attr );
 
 	if ( attr.map_state != IsUnmapped
 	    && QRect(attr.x,attr.y,attr.width,attr.height)
@@ -1057,7 +1063,7 @@ Window findRealWindow( const QPoint & pos, Window w, int md )
 		unsigned long n, a;
 		unsigned char *data;
 
-		XGetWindowProperty( qt_xdisplay(), w, qt_wm_state, 0,
+		XGetWindowProperty( QPaintDevice::x11AppDisplay(), w, qt_wm_state, 0,
 		    0, False, AnyPropertyType, &type, &f,&n,&a,&data );
 		if ( data ) XFree(data);
 		if ( type ) return w;
@@ -1066,7 +1072,7 @@ Window findRealWindow( const QPoint & pos, Window w, int md )
 	    Window r, p;
 	    Window* c;
 	    uint nc;
-	    if ( XQueryTree( qt_xdisplay(), w, &r, &p, &c, &nc ) ) {
+	    if ( XQueryTree( QPaintDevice::x11AppDisplay(), w, &r, &p, &c, &nc ) ) {
 		r=0;
 		for (uint i=nc; !r && i--; ) {
 		    r = findRealWindow( pos-QPoint(attr.x,attr.y),
@@ -1089,6 +1095,7 @@ Window findRealWindow( const QPoint & pos, Window w, int md )
 
 void QDragManager::move( const QPoint & globalPos )
 {
+    Q_ASSERT( object != 0 ); // ### remove in Qt 4.0 (object should never be 0 here)
     int screen = QCursor::x11Screen();
     if ( ( qt_xdnd_current_screen == -1 && screen != QPaintDevice::x11AppScreen() ) ||
 	 ( screen != qt_xdnd_current_screen ) ) {
@@ -1104,15 +1111,14 @@ void QDragManager::move( const QPoint & globalPos )
     }
 
     qt_xdnd_current_screen = screen;
-    Window rootwin = RootWindow( qt_xdisplay(), qt_xdnd_current_screen );
+    Window rootwin = QPaintDevice::x11AppRootWindow( qt_xdnd_current_screen );
     Window target = 0;
     int lx = 0, ly = 0;
-    if ( !XTranslateCoordinates( qt_xdisplay(), rootwin, rootwin,
+    if ( !XTranslateCoordinates( QPaintDevice::x11AppDisplay(), rootwin, rootwin,
 				 globalPos.x(), globalPos.y(),
-				 &lx, &ly, &target) ) {
-	// somehow got to a different screen?  ignore for now
+				 &lx, &ly, &target) )
+	// some wierd error...
 	return;
-    }
 
     if ( target == rootwin ) {
 	// Ok.
@@ -1144,22 +1150,21 @@ void QDragManager::move( const QPoint & globalPos )
 
     {
 	Atom   type = None;
-	int f;
+	int r, f;
 	unsigned long n, a;
 	WId *proxy_id;
 	qt_ignore_badwindow();
-	XGetWindowProperty( qt_xdisplay(), target, qt_xdnd_proxy, 0,
-			    1, False, XA_WINDOW, &type, &f,&n,&a,(uchar**)&proxy_id );
-	if ( qt_badwindow() ) {
+	r = XGetWindowProperty( qt_xdisplay(), target, qt_xdnd_proxy, 0,
+				1, False, XA_WINDOW, &type, &f,&n,&a,(uchar**)&proxy_id );
+	if ( ( r != Success ) || qt_badwindow() ) {
 	    proxy_target = target = 0;
 	} else if ( type == XA_WINDOW && proxy_id ) {
 	    proxy_target = *proxy_id;
 	    XFree(proxy_id);
 	    proxy_id = 0;
-	    qt_ignore_badwindow();
-	    XGetWindowProperty( qt_xdisplay(), proxy_target, qt_xdnd_proxy, 0,
-				1, False, XA_WINDOW, &type, &f,&n,&a,(uchar**)&proxy_id );
-	    if ( qt_badwindow() || !type || !proxy_id || *proxy_id != proxy_target ) {
+	    r = XGetWindowProperty( qt_xdisplay(), proxy_target, qt_xdnd_proxy, 0,
+				    1, False, XA_WINDOW, &type, &f,&n,&a,(uchar**)&proxy_id );
+	    if ( ( r != Success ) || qt_badwindow() || !type || !proxy_id || *proxy_id != proxy_target ) {
 		// Bogus
 		proxy_target = 0;
 		target = 0;
@@ -1170,14 +1175,19 @@ void QDragManager::move( const QPoint & globalPos )
 	if ( proxy_target ) {
 	    int *tv;
 	    qt_ignore_badwindow();
-	    XGetWindowProperty( qt_xdisplay(), proxy_target, qt_xdnd_aware, 0,
-				1, False, AnyPropertyType, &type, &f,&n,&a,(uchar**)&tv );
-	    target_version = QMIN(qt_xdnd_version,tv ? *tv : 1);
-	    if ( tv ) XFree(tv);
-	    if ( !qt_badwindow() && type )
-		emask = EnterWindowMask;
-	    else
+	    r = XGetWindowProperty( qt_xdisplay(), proxy_target, qt_xdnd_aware, 0,
+				    1, False, AnyPropertyType, &type, &f,&n,&a,(uchar**)&tv );
+	    if ( r != Success ) {
 		target = 0;
+	    } else {
+		target_version = QMIN(qt_xdnd_version,tv ? *tv : 1);
+		if ( tv )
+		    XFree( tv );
+		if ( !qt_badwindow() && type )
+		    emask = EnterWindowMask;
+		else
+		    target = 0;
+	    }
 	}
     }
 
@@ -1187,10 +1197,7 @@ void QDragManager::move( const QPoint & globalPos )
 
 	qt_xdnd_current_target = target;
 	qt_xdnd_current_proxy_target = proxy_target;
-	if ( target && object ) {
-	    // check for object shouldn't be necessary,
-	    // since a drag can't start without an object
-
+	if ( target ) {
 	    QMemArray<Atom> type;
 	    int flags = target_version << 24;
 	    const char* fmt;
@@ -1200,7 +1207,7 @@ void QDragManager::move( const QPoint & globalPos )
 		type[nfmt] = *qt_xdnd_str_to_atom( fmt );
 	    }
 	    if ( nfmt >= 3 ) {
-		XChangeProperty( qt_xdisplay(),
+		XChangeProperty( QPaintDevice::x11AppDisplay(),
 				 object->source()->winId(), qt_xdnd_type_list,
 				 XA_ATOM, 32, PropModeReplace,
 				 (unsigned char *)type.data(),
@@ -1224,7 +1231,7 @@ void QDragManager::move( const QPoint & globalPos )
 	    if ( w ) {
 		qt_handle_xdnd_enter( w, (const XEvent *)&enter, FALSE );
 	    } else if ( target ) {
-		XSendEvent( qt_xdisplay(), proxy_target, False, emask,
+		XSendEvent( QPaintDevice::x11AppDisplay(), proxy_target, False, emask,
 			    (XEvent*)&enter );
 	    }
 	}
@@ -1246,7 +1253,7 @@ void QDragManager::move( const QPoint & globalPos )
 	if ( w )
 	    qt_handle_xdnd_position( w, (const XEvent *)&move, FALSE );
 	else
-	    XSendEvent( qt_xdisplay(), proxy_target, False, emask,
+	    XSendEvent( QPaintDevice::x11AppDisplay(), proxy_target, False, emask,
 			(XEvent*)&move );
     } else {
 	if ( willDrop ) {
@@ -1259,6 +1266,8 @@ void QDragManager::move( const QPoint & globalPos )
 
 void QDragManager::drop()
 {
+    killTimer( heartbeat );
+    heartbeat = -1;
     if ( !qt_xdnd_current_target )
 	return;
 
@@ -1287,7 +1296,7 @@ void QDragManager::drop()
     if ( w )
 	qt_handle_xdnd_drop( w, (const XEvent *)&drop, FALSE );
     else
-	XSendEvent( qt_xdisplay(), qt_xdnd_current_proxy_target, False, emask,
+	XSendEvent( QPaintDevice::x11AppDisplay(), qt_xdnd_current_proxy_target, False, emask,
 		    (XEvent*)&drop );
 
 #ifndef QT_NO_CURSOR
@@ -1350,12 +1359,8 @@ bool qt_xdnd_handle_badwindow()
 
 bool QDropEvent::provides( const char *mimeType ) const
 {
-    if ( qt_motifdnd_active ) {
-	if ( 0 == qstrnicmp( mimeType, "text/", 5 ) )
-	    return TRUE;
-	else
-	    return FALSE;
-    }
+    if ( qt_motifdnd_active && qstrnicmp( mimeType, "text/", 5 ) == 0 )
+	return TRUE;
 
     int n=0;
     const char* f;
@@ -1384,18 +1389,18 @@ void qt_xdnd_handle_selection_request( const XSelectionRequestEvent * req )
     if ( format && qt_xdnd_source_object &&
 	 qt_xdnd_source_object->provides( format ) ) {
 	QByteArray a = qt_xdnd_source_object->encodedData(format);
-	XChangeProperty ( qt_xdisplay(), req->requestor, req->property,
+	XChangeProperty ( QPaintDevice::x11AppDisplay(), req->requestor, req->property,
 			  req->target, 8, PropModeReplace,
 			  (unsigned char *)a.data(), a.size() );
 	evt.xselection.property = req->property;
     }
     // ### this can die if req->requestor crashes at the wrong
     // ### moment
-    XSendEvent( qt_xdisplay(), req->requestor, False, 0, &evt );
+    XSendEvent( QPaintDevice::x11AppDisplay(), req->requestor, False, 0, &evt );
 }
 
 /*
-	XChangeProperty ( qt_xdisplay(), req->requestor, req->property,
+	XChangeProperty ( QPaintDevice::x11AppDisplay(), req->requestor, req->property,
 			  XA_STRING, 8,
 			  PropModeReplace,
 			  (uchar *)d->text(), strlen(d->text()) );
@@ -1427,34 +1432,35 @@ static QByteArray qt_xdnd_obtain_data( const char *format )
     if ( qt_xdnd_target_data->find( (int)*a ) ) {
 	result = *(qt_xdnd_target_data->find( (int)*a ));
     } else {
-	if ( XGetSelectionOwner( qt_xdisplay(),
+	if ( XGetSelectionOwner( QPaintDevice::x11AppDisplay(),
 				 qt_xdnd_selection ) == None )
 	    return result; // should never happen?
 
 	QWidget* tw = qt_xdnd_current_widget;
-	if ( qt_xdnd_current_widget->isDesktop() ) {
+	if ( !qt_xdnd_current_widget ||
+	     qt_xdnd_current_widget->isDesktop() ) {
 	    tw = new QWidget;
 	}
-	XConvertSelection( qt_xdisplay(),
+	XConvertSelection( QPaintDevice::x11AppDisplay(),
 			   qt_xdnd_selection, *a,
 			   qt_xdnd_selection,
 			   tw->winId(), CurrentTime );
-	XFlush( qt_xdisplay() );
+	XFlush( QPaintDevice::x11AppDisplay() );
 
 	XEvent xevent;
-	bool got=qt_xclb_wait_for_event( qt_xdisplay(),
+	bool got=qt_xclb_wait_for_event( QPaintDevice::x11AppDisplay(),
 				      tw->winId(),
 				      SelectionNotify, &xevent, 5000);
 	if ( got ) {
 	    Atom type;
 
-	    if ( qt_xclb_read_property( qt_xdisplay(),
+	    if ( qt_xclb_read_property( QPaintDevice::x11AppDisplay(),
 					tw->winId(),
 					qt_xdnd_selection, TRUE,
 					&result, 0, &type, 0, FALSE ) ) {
 		if ( type == qt_incr_atom ) {
 		    int nbytes = result.size() >= 4 ? *((int*)result.data()) : 0;
-		    result = qt_xclb_read_incremental_property( qt_xdisplay(),
+		    result = qt_xclb_read_incremental_property( QPaintDevice::x11AppDisplay(),
 								tw->winId(),
 								qt_xdnd_selection,
 								nbytes, FALSE );
@@ -1470,7 +1476,8 @@ static QByteArray qt_xdnd_obtain_data( const char *format )
 #endif
 	    }
 	}
-	if ( qt_xdnd_current_widget->isDesktop() ) {
+	if ( !qt_xdnd_current_widget ||
+	     qt_xdnd_current_widget->isDesktop() ) {
 	    delete tw;
 	}
     }
@@ -1538,7 +1545,7 @@ bool qt_dnd_enable( QWidget* w, bool on )
 QByteArray QDropEvent::encodedData( const char *format ) const
 {
     if ( qt_motifdnd_active )
-	return qt_motifdnd_obtain_data();
+	return qt_motifdnd_obtain_data( format );
     return qt_xdnd_obtain_data( format );
 }
 
@@ -1556,14 +1563,8 @@ QByteArray QDropEvent::encodedData( const char *format ) const
 
 const char* QDropEvent::format( int n ) const
 {
-    if ( qt_motifdnd_active ) {
-	if ( n == 0 )
-	    return "text/plain";
-	else if ( n == 1 )
-	    return "text/uri-list";
-	else
-	    return 0;
-    }
+    if ( qt_motifdnd_active )
+	return qt_motifdnd_format( n );
 
     int i = 0;
     while( i<n && qt_xdnd_types[i] )
@@ -1616,6 +1617,7 @@ bool QDragManager::drag( QDragObject * o, QDragObject::DragMode mode )
     }
 
     qt_xdnd_source_object = o;
+    qt_xdnd_source_object->setTarget( 0 );
     qt_xdnd_deco = new QShapedPixmapWidget();
 
     willDrop = FALSE;
@@ -1627,7 +1629,7 @@ bool QDragManager::drag( QDragObject * o, QDragObject::DragMode mode )
 
     qApp->installEventFilter( this );
     qt_xdnd_source_current_time = qt_x_time;
-    XSetSelectionOwner( qt_xdisplay(), qt_xdnd_selection,
+    XSetSelectionOwner( QPaintDevice::x11AppDisplay(), qt_xdnd_selection,
 			dragSource->topLevelWidget()->winId(),
 			qt_xdnd_source_current_time );
     oldstate = ButtonState(-1); // #### Should use state that caused the drag
@@ -1655,8 +1657,8 @@ bool QDragManager::drag( QDragObject * o, QDragObject::DragMode mode )
 
     delete qt_xdnd_deco;
     qt_xdnd_deco = 0;
-    killTimer(heartbeat);
-    heartbeat = 0;
+    killTimer( heartbeat );
+    heartbeat = -1;
     qt_xdnd_current_screen = -1;
     qt_xdnd_dragging = FALSE;
 
