@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id:  qt/qdrawutil.cpp   3.0.5   edited Apr 10 19:11 $
+** $Id: qt/qdrawutil.cpp   3.2.3   edited May 13 09:08 $
 **
 ** Implementation of draw utilities
 **
@@ -206,30 +206,28 @@ void qDrawShadeRect( QPainter *p, int x, int y, int w, int h,
 	return;
     if ( ! ( w > 0 && h > 0 && lineWidth >= 0 && midLineWidth >= 0 ) ) {
 #if defined(QT_CHECK_RANGE)
-	qWarning( "qDrawShadeRect() Invalid parameters." );
+	qWarning( "qDrawShadeRect(): Invalid parameters" );
 #endif
 	return;
     }
-    QPen oldPen = p->pen();			// save pen
+    QPen oldPen = p->pen();
     if ( sunken )
 	p->setPen( g.dark() );
     else
 	p->setPen( g.light() );
     int x1=x, y1=y, x2=x+w-1, y2=y+h-1;
     QPointArray a;
+
     if ( lineWidth == 1 && midLineWidth == 0 ) {// standard shade rectangle
-	a.setPoints( 8, x1,y1, x2,y1, x1,y1+1, x1,y2, x1+2,y2-1,
-		     x2-1,y2-1, x2-1,y1+2,  x2-1,y2-2 );
-	p->drawLineSegments( a );		// draw top/left lines
+	p->drawRect( x1, y1, w-1, h-1 );
 	if ( sunken )
 	    p->setPen( g.light() );
 	else
 	    p->setPen( g.dark() );
-	    a.setPoints( 8, x1+1,y1+1, x2,y1+1, x1+1,y1+2, x1+1,y2-1,
-			 x1+1,y2, x2,y2,  x2,y1+2, x2,y2-1 );
+	a.setPoints( 8, x1+1,y1+1, x2-2,y1+1, x1+1,y1+2, x1+1,y2-2,
+		     x1,y2, x2,y2,  x2,y1, x2,y2-1 );
 	p->drawLineSegments( a );		// draw bottom/right lines
-    }
-    else {					// more complicated
+    } else {					// more complicated
 	int m = lineWidth+midLineWidth;
 	int i, j=0, k=m;
 	for ( i=0; i<lineWidth; i++ ) {		// draw top shadow
@@ -311,12 +309,20 @@ void qDrawShadePanel( QPainter *p, int x, int y, int w, int h,
     	qWarning( "qDrawShadePanel() Invalid parameters." );
 #endif
     }
+    QColor shade = g.dark();
+    QColor light = g.light();
+    if ( fill ) {
+	if ( fill->color() == shade )
+	    shade = g.shadow();
+	if ( fill->color() == light )
+	    light = g.midlight();
+    }
     QPen oldPen = p->pen();			// save pen
     QPointArray a( 4*lineWidth );
     if ( sunken )
-	p->setPen( g.dark() );
+	p->setPen( shade );
     else
-	p->setPen( g.light() );
+	p->setPen( light );
     int x1, y1, x2, y2;
     int i;
     int n = 0;
@@ -336,9 +342,9 @@ void qDrawShadePanel( QPainter *p, int x, int y, int w, int h,
     p->drawLineSegments( a );
     n = 0;
     if ( sunken )
-	p->setPen( g.light() );
+	p->setPen( light );
     else
-	p->setPen( g.dark() );
+	p->setPen( shade );
     x1 = x;
     y1 = y2 = y+h-1;
     x2 = x+w-1;
@@ -451,7 +457,7 @@ void qDrawWinButton( QPainter *p, int x, int y, int w, int h,
 		       g.shadow(), g.light(), g.dark(), g.button(), fill );
     else
 	qDrawWinShades( p, x, y, w, h,
-		       g.light(), g.shadow(), g.midlight(), g.dark(), fill );
+		       g.light(), g.shadow(), g.button(), g.dark(), fill );
 }
 
 /*!
@@ -786,7 +792,7 @@ static void qDrawWinArrow( QPainter *p, Qt::ArrowType type, bool down,
 #pragma warning(disable: 4244)
 #endif
 
-
+#ifndef QT_NO_STYLE_MOTIF
 // motif arrows look the same whether they are used or not
 // is this correct?
 static void qDrawMotifArrow( QPainter *p, Qt::ArrowType type, bool down,
@@ -922,7 +928,7 @@ static void qDrawMotifArrow( QPainter *p, Qt::ArrowType type, bool down,
 #undef CTOP
 #undef CBOT
 }
-
+#endif
 
 void qDrawArrow( QPainter *p, Qt::ArrowType type, Qt::GUIStyle style, bool down,
 		 int x, int y, int w, int h,
@@ -932,9 +938,11 @@ void qDrawArrow( QPainter *p, Qt::ArrowType type, Qt::GUIStyle style, bool down,
 	case Qt::WindowsStyle:
 	    qDrawWinArrow( p, type, down, x, y, w, h, g, enabled );
 	    break;
+#ifndef QT_NO_STYLE_MOTIF
 	case Qt::MotifStyle:
 	    qDrawMotifArrow( p, type, down, x, y, w, h, g, enabled );
 	    break;
+#endif
 	default:
 #if defined(QT_CHECK_RANGE)
 	    qWarning( "qDrawArrow: Requested GUI style not supported" );
