@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id:  qt/qjpegio.cpp   3.0.5   edited Feb 11 06:49 $
+** $Id: qt/qjpegio.cpp   3.2.3   edited Oct 13 13:45 $
 **
 ** Implementation of JPEG QImage IOHandler
 **
 ** Created : 990521
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2003 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the kernel module of the Qt GUI Toolkit.
 **
@@ -53,9 +53,8 @@
 // including jpeglib.h seems to be a little messy
 extern "C" {
 #define XMD_H           // shut JPEGlib up
-#if defined(Q_OS_UNIXWARE7)
+#if defined(Q_OS_UNIXWARE)
 #  define HAVE_BOOLEAN  // libjpeg under Unixware seems to need this
-#  define HAVE_PROTOTYPES
 #endif
 #include <jpeglib.h>
 #ifdef const
@@ -123,7 +122,7 @@ boolean qt_fill_input_buffer(j_decompress_ptr cinfo)
     } else {
 	src->bytes_in_buffer = num_read;
     }
-#if defined(Q_OS_UNIXWARE7)
+#if defined(Q_OS_UNIXWARE)
     return B_TRUE;
 #else
     return TRUE;
@@ -177,6 +176,20 @@ inline my_jpeg_source_mgr::my_jpeg_source_mgr(QImageIO* iioptr)
 }
 
 
+static
+void scaleSize( int &reqW, int &reqH, int imgW, int imgH, QImage::ScaleMode mode )
+{
+    if ( mode == QImage::ScaleFree )
+        return;
+    int t1 = imgW * reqH;
+    int t2 = reqW * imgH;
+    if (( mode == QImage::ScaleMin && (t1 > t2) ) || ( mode == QImage::ScaleMax && (t1 < t2) ))
+	reqH = t2 / imgW;
+    else
+        reqW = t1 / imgH;
+}
+
+
 static
 void read_jpeg_image(QImageIO* iio)
 {
@@ -195,7 +208,7 @@ void read_jpeg_image(QImageIO* iio)
     jerr.error_exit = my_error_exit;
 
     if (!setjmp(jerr.setjmp_buffer)) {
-#if defined(Q_OS_UNIXWARE7)
+#if defined(Q_OS_UNIXWARE)
 	(void) jpeg_read_header(&cinfo, B_TRUE);
 #else
 	(void) jpeg_read_header(&cinfo, TRUE);
@@ -203,37 +216,131 @@ void read_jpeg_image(QImageIO* iio)
 
 	(void) jpeg_start_decompress(&cinfo);
 
-	if ( cinfo.output_components == 3 || cinfo.output_components == 4) {
-	    image.create( cinfo.output_width, cinfo.output_height, 32 );
-	} else if ( cinfo.output_components == 1 ) {
-	    image.create( cinfo.output_width, cinfo.output_height, 8, 256 );
-	    for (int i=0; i<256; i++)
-		image.setColor(i, qRgb(i,i,i));
+	QString params = iio->parameters();
+	params.simplifyWhiteSpace();
+	int sWidth = 0, sHeight = 0;
+	char sModeStr[1024] = "";
+	QImage::ScaleMode sMode;
+
+	if ( params.contains( "GetHeaderInformation" ) ) {
+
+	    // Create QImage's without allocating the data
+	    if ( cinfo.output_components == 3 || cinfo.output_components == 4) {
+		image = QImage( NULL, cinfo.output_width, cinfo.output_height, 32, NULL, 0, QImage::IgnoreEndian );
+	    } else if ( cinfo.output_components == 1 ) {
+		image = QImage( NULL, cinfo.output_width, cinfo.output_height, 8, NULL, 0, QImage::IgnoreEndian );
+	    } else {
+		// Unsupported format
+	    }
+
+
+	} else if ( params.contains( "Scale" ) ) {
+	    sscanf( params.latin1(), "Scale( %i, %i, %1023s )",
+		    &sWidth, &sHeight, sModeStr );
+
+	    QString sModeQStr( sModeStr );
+	    if ( sModeQStr == "ScaleFree" ) {
+		sMode = QImage::ScaleFree;
+	    } else if ( sModeQStr == "ScaleMin" ) {
+		sMode = QImage::ScaleMin;
+	    } else if ( sModeQStr == "ScaleMax" ) {
+		sMode = QImage::ScaleMax;
+	    } else {
+		qDebug("read_jpeg_image: invalid scale mode \"%s\", see QImage::ScaleMode documentation", sModeStr);
+		sMode = QImage::ScaleFree;
+	    }
+
+//	    qDebug( "Parameters ask to scale the image to %i x %i ScaleMode: %s", sWidth, sHeight, sModeStr );
+	    scaleSize( sWidth, sHeight, cinfo.output_width, cinfo.output_height, sMode );
+//	    qDebug( "Scaling the jpeg to %i x %i", sWidth, sHeight, sModeStr );
+
+	    if ( cinfo.output_components == 3 || cinfo.output_components == 4) {
+		image.create( sWidth, sHeight, 32 );
+	    } else if ( cinfo.output_components == 1 ) {
+		image.create( sWidth, sHeight, 8, 256 );
+		for (int i=0; i<256; i++)
+		    image.setColor(i, qRgb(i,i,i));
+	    } else {
+		// Unsupported format
+	    }
+
+	    if (!image.isNull()) {
+		QImage tmpImage( cinfo.output_width, 1, 32 );
+		uchar** inLines = tmpImage.jumpTable();
+		uchar** outLines = image.jumpTable();
+		while (cinfo.output_scanline < cinfo.output_height) {
+		    int outputLine = sHeight * cinfo.output_scanline / cinfo.output_height;
+		    (void) jpeg_read_scanlines(&cinfo, inLines, 1);
+		    if ( cinfo.output_components == 3 ) {
+			uchar *in = inLines[0];
+			QRgb *out = (QRgb*)outLines[outputLine];
+			for (uint i=0; i<cinfo.output_width; i++ ) {
+// ### Only scaling down an image works, I don't think scaling up will work at the moment
+// ### An idea I have to make this a smooth scale is to progressively add the pixel values up
+// When scaling down, multiple values are being over drawn in to the output buffer.
+// Instead, a weighting based on the distance the line or pixel is from the output pixel determines
+// the weight of it when added to the output buffer. At present it is a non-smooth scale which is
+// inefficently implemented, it still uncompresses all the jpeg, an optimization for progressive
+// jpegs could be made if scaling by say 50% or some other special cases
+			    out[sWidth * i / cinfo.output_width] = qRgb( in[0], in[1], in[2] );
+			    in += 3;
+			}
+		    } else {
+// ### Need to test the case where the jpeg is grayscale, need some black and white jpegs to test
+// this code. (also only scales down and probably won't scale to a larger size)
+			uchar *in = inLines[0];
+			uchar *out = outLines[outputLine];
+			for (uint i=0; i<cinfo.output_width; i++ ) {
+			    out[sWidth * i / cinfo.output_width] = in[i];
+			}
+		    }
+		}
+		(void) jpeg_finish_decompress(&cinfo);
+	    }
+
 	} else {
-	    // Unsupported format
-	}
 
-	if (!image.isNull()) {
-	    uchar** lines = image.jumpTable();
-	    while (cinfo.output_scanline < cinfo.output_height)
-		(void) jpeg_read_scanlines(&cinfo,
-			    lines + cinfo.output_scanline,
-			    cinfo.output_height);
-	    (void) jpeg_finish_decompress(&cinfo);
-	}
+	    if ( cinfo.output_components == 3 || cinfo.output_components == 4) {
+		image.create( cinfo.output_width, cinfo.output_height, 32 );
+	    } else if ( cinfo.output_components == 1 ) {
+		image.create( cinfo.output_width, cinfo.output_height, 8, 256 );
+		for (int i=0; i<256; i++)
+		    image.setColor(i, qRgb(i,i,i));
+	    } else {
+		// Unsupported format
+	    }
 
-	if ( cinfo.output_components == 3 ) {
-	    // Expand 24->32 bpp.
-	    for (uint j=0; j<cinfo.output_height; j++) {
-		uchar *in = image.scanLine(j) + cinfo.output_width * 3;
-		QRgb *out = (QRgb*)image.scanLine(j);
+	    if (!image.isNull()) {
+		uchar** lines = image.jumpTable();
+		while (cinfo.output_scanline < cinfo.output_height)
+		    (void) jpeg_read_scanlines(&cinfo,
+				lines + cinfo.output_scanline,
+				cinfo.output_height);
+		(void) jpeg_finish_decompress(&cinfo);
+	    }
 
-		for (uint i=cinfo.output_width; i--; ) {
-		    in-=3;
-		    out[i] = qRgb(in[0], in[1], in[2]);
+	    if ( cinfo.output_components == 3 ) {
+		// Expand 24->32 bpp.
+		for (uint j=0; j<cinfo.output_height; j++) {
+		    uchar *in = image.scanLine(j) + cinfo.output_width * 3;
+		    QRgb *out = (QRgb*)image.scanLine(j);
+
+		    for (uint i=cinfo.output_width; i--; ) {
+			in-=3;
+			out[i] = qRgb(in[0], in[1], in[2]);
+		    }
 		}
 	    }
+        }
+
+	if ( cinfo.density_unit == 1 ) {
+	    image.setDotsPerMeterX( int(100. * cinfo.X_density / 2.54) );
+	    image.setDotsPerMeterY( int(100. * cinfo.Y_density / 2.54) );
+	} else if ( cinfo.density_unit == 2 ) {
+	    image.setDotsPerMeterX( int(100. * cinfo.X_density) );
+	    image.setDotsPerMeterY( int(100. * cinfo.Y_density) );
 	}
+
 	iio->setImage(image);
 	iio->setStatus(0);
     }
@@ -285,7 +392,7 @@ boolean qt_empty_output_buffer(j_compress_ptr cinfo)
     dest->next_output_byte = dest->buffer;
     dest->free_in_buffer = max_buf;
 
-#if defined(Q_OS_UNIXWARE7)
+#if defined(Q_OS_UNIXWARE)
     return B_TRUE;
 #else
     return TRUE;
@@ -370,7 +477,7 @@ void write_jpeg_image(QImageIO* iio)
 
 	jpeg_set_defaults(&cinfo);
 	int quality = iio->quality() >= 0 ? QMIN(iio->quality(),100) : 75;
-#if defined(Q_OS_UNIXWARE7)
+#if defined(Q_OS_UNIXWARE)
 	jpeg_set_quality(&cinfo, quality, B_TRUE /* limit to baseline-JPEG values */);
 	jpeg_start_compress(&cinfo, B_TRUE);
 #else
@@ -453,7 +560,7 @@ void write_jpeg_image(QImageIO* iio)
     }
 
     delete iod_dest;
-    delete row_pointer[0];
+    delete [] row_pointer[0];
 }
 
 void qInitJpegIO()
