@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id:  qt/qlayoutengine.cpp   3.0.5   edited Oct 12 2001 $
+** $Id: qt/qlayoutengine.cpp   3.2.3   edited May 13 09:08 $
 **
 ** Implementation of QLayout functionality
 **
@@ -35,6 +35,7 @@
 **
 **********************************************************************/
 
+#include "qlayout.h"
 #include "private/qlayoutengine_p.h"
 
 #ifndef QT_NO_LAYOUT
@@ -48,19 +49,19 @@ static inline int fRound( int i ) {
   This is the main workhorse of the QGridLayout. It portions out
   available space to the chain's children.
 
-  The calculation is done in fixed point: "fixed" variables are scaled
-  by a factor of 256.
+  The calculation is done in fixed point: "fixed" variables are
+  scaled by a factor of 256.
 
   If the layout runs "backwards" (i.e. RightToLeft or Up) the layout
-  is computed mirror-reversed, and it is the callers responsibility do
-  reverse the values before use.
+  is computed mirror-reversed, and it's the caller's responsibility
+  do reverse the values before use.
 
   chain contains input and output parameters describing the geometry.
-  count is the count of items in the chain,
-  pos and space give the interval (relative to parentWidget topLeft).
+  count is the count of items in the chain; pos and space give the
+  interval (relative to parentWidget topLeft).
 */
-void qGeomCalc( QMemArray<QLayoutStruct> &chain, int start, int count, int pos,
-		int space, int spacer )
+Q_EXPORT void qGeomCalc( QMemArray<QLayoutStruct> &chain, int start, int count,
+			 int pos, int space, int spacer )
 {
     typedef int fixed;
     int cHint = 0;
@@ -75,7 +76,7 @@ void qGeomCalc( QMemArray<QLayoutStruct> &chain, int start, int count, int pos,
     int i;
     for ( i = start; i < start + count; i++ ) {
 	chain[i].done = FALSE;
-	cHint += chain[i].sizeHint;
+	cHint += chain[i].smartSizeHint();
 	cMin += chain[i].minimumSize;
 	cMax += chain[i].maximumSize;
 	sumStretch += chain[i].stretch;
@@ -88,24 +89,28 @@ void qGeomCalc( QMemArray<QLayoutStruct> &chain, int start, int count, int pos,
     if ( spacerCount )
 	spacerCount--; // only spacers between things
     if ( space < cMin + spacerCount * spacer ) {
-	//	qDebug("not enough space");
 	for ( i = start; i < start+count; i++ ) {
 	    chain[i].size = chain[i].minimumSize;
 	    chain[i].done = TRUE;
 	}
     } else if ( space < cHint + spacerCount*spacer ) {
-	// Less space than sizeHint, but more than minimum.
-	// Currently take space equally from each, like in Qt 2.x.
-	// Commented-out lines will give more space to stretchier items.
+	/*
+	  Less space than smartSizeHint(), but more than minimumSize.
+	  Currently take space equally from each, as in Qt 2.x.
+	  Commented-out lines will give more space to stretchier
+	  items.
+	*/
 	int n = count;
 	int space_left = space - spacerCount*spacer;
 	int overdraft = cHint - space_left;
-	//first give to the fixed ones:
-	for ( i = start; i < start+count; i++ ) {
-	    if ( !chain[i].done && chain[i].minimumSize >= chain[i].sizeHint) {
-		chain[i].size = chain[i].sizeHint;
+
+	// first give to the fixed ones:
+	for ( i = start; i < start + count; i++ ) {
+	    if ( !chain[i].done
+		 && chain[i].minimumSize >= chain[i].smartSizeHint() ) {
+		chain[i].size = chain[i].smartSizeHint();
 		chain[i].done = TRUE;
-		space_left -= chain[i].sizeHint;
+		space_left -= chain[i].smartSizeHint();
 		// sumStretch -= chain[i].stretch;
 		n--;
 	    }
@@ -124,43 +129,46 @@ void qGeomCalc( QMemArray<QLayoutStruct> &chain, int start, int count, int pos,
 		// else
 		//    fp_w += (fp_over * chain[i].stretch) / sumStretch;
 		int w = fRound( fp_w );
-		chain[i].size = chain[i].sizeHint - w;
-		fp_w -= toFixed( w ); //give the difference to the next
+		chain[i].size = chain[i].smartSizeHint() - w;
+		fp_w -= toFixed( w ); // give the difference to the next
 		if ( chain[i].size < chain[i].minimumSize ) {
 		    chain[i].done = TRUE;
 		    chain[i].size = chain[i].minimumSize;
 		    finished = FALSE;
-		    overdraft -= chain[i].sizeHint - chain[i].minimumSize;
+		    overdraft -= ( chain[i].smartSizeHint()
+				   - chain[i].minimumSize );
 		    // sumStretch -= chain[i].stretch;
 		    n--;
 		    break;
 		}
 	    }
 	}
-    } else { //extra space
+    } else { // extra space
 	int n = count;
 	int space_left = space - spacerCount*spacer;
 	// first give to the fixed ones, and handle non-expansiveness
 	for ( i = start; i < start + count; i++ ) {
-	    if ( !chain[i].done && (chain[i].maximumSize <= chain[i].sizeHint
-				    || wannaGrow && !chain[i].expansive) ) {
-		chain[i].size = chain[i].sizeHint;
+	    if ( !chain[i].done
+		 && (chain[i].maximumSize <= chain[i].smartSizeHint()
+		     || (wannaGrow && !chain[i].expansive)) ) {
+		chain[i].size = chain[i].smartSizeHint();
 		chain[i].done = TRUE;
-		space_left -= chain[i].sizeHint;
+		space_left -= chain[i].smartSizeHint();
 		sumStretch -= chain[i].stretch;
 		n--;
 	    }
 	}
 	extraspace = space_left;
+
 	/*
 	  Do a trial distribution and calculate how much it is off.
 	  If there are more deficit pixels than surplus pixels, give
 	  the minimum size items what they need, and repeat.
 	  Otherwise give to the maximum size items, and repeat.
 
-	  I have a wonderful mathematical proof for the correctness
-	  of this principle, but unfortunately this comment is too
-	  small to contain it.
+	  Paul Olav Tvete has a wonderful mathematical proof of the
+	  correctness of this principle, but unfortunately this
+	  comment is too small to contain it.
 	*/
 	int surplus, deficit;
 	do {
@@ -178,8 +186,8 @@ void qGeomCalc( QMemArray<QLayoutStruct> &chain, int start, int count, int pos,
 		int w = fRound( fp_w );
 		chain[i].size = w;
 		fp_w -= toFixed( w ); // give the difference to the next
-		if ( w < chain[i].sizeHint ) {
-		    deficit +=  chain[i].sizeHint - w;
+		if ( w < chain[i].smartSizeHint() ) {
+		    deficit +=  chain[i].smartSizeHint() - w;
 		} else if ( w > chain[i].maximumSize ) {
 		    surplus += w - chain[i].maximumSize;
 		}
@@ -188,10 +196,10 @@ void qGeomCalc( QMemArray<QLayoutStruct> &chain, int start, int count, int pos,
 		// give to the ones that have too little
 		for ( i = start; i < start+count; i++ ) {
 		    if ( !chain[i].done &&
-			 chain[i].size < chain[i].sizeHint ) {
-			chain[i].size = chain[i].sizeHint;
+			 chain[i].size < chain[i].smartSizeHint() ) {
+			chain[i].size = chain[i].smartSizeHint();
 			chain[i].done = TRUE;
-			space_left -= chain[i].sizeHint;
+			space_left -= chain[i].smartSizeHint();
 			sumStretch -= chain[i].stretch;
 			n--;
 		    }
@@ -215,10 +223,12 @@ void qGeomCalc( QMemArray<QLayoutStruct> &chain, int start, int count, int pos,
 	    extraspace = space_left;
     }
 
-    // as a last resort, we distribute the unwanted space equally
-    // among the spacers (counting the start and end of the chain).
-
-    //### should do a sub-pixel allocation of extra space
+    /*
+      As a last resort, we distribute the unwanted space equally
+      among the spacers (counting the start and end of the chain). We
+      could, but don't, attempt a sub-pixel allocation of the extra
+      space.
+    */
     int extra = extraspace / ( spacerCount + 2 );
     int p = pos + extra;
     for ( i = start; i < start+count; i++ ) {
@@ -229,4 +239,81 @@ void qGeomCalc( QMemArray<QLayoutStruct> &chain, int start, int count, int pos,
     }
 }
 
+Q_EXPORT QSize qSmartMinSize( const QWidgetItem *i )
+{
+    QWidget *w = ((QWidgetItem *)i)->widget();
+
+    QSize s( 0, 0 );
+    if ( w->layout() ) {
+	s = w->layout()->totalMinimumSize();
+    } else {
+	QSize sh;
+
+	if ( w->sizePolicy().horData() != QSizePolicy::Ignored ) {
+	    if ( w->sizePolicy().mayShrinkHorizontally() ) {
+		s.setWidth( w->minimumSizeHint().width() );
+	    } else {
+		sh = w->sizeHint();
+		s.setWidth( sh.width() );
+	    }
+	}
+
+	if ( w->sizePolicy().verData() != QSizePolicy::Ignored ) {
+	    if ( w->sizePolicy().mayShrinkVertically() ) {
+		s.setHeight( w->minimumSizeHint().height() );
+	    } else {
+		s.setHeight( sh.isValid() ? sh.height()
+			     : w->sizeHint().height() );
+	    }
+	}
+    }
+    s = s.boundedTo( w->maximumSize() );
+    QSize min = w->minimumSize();
+    if ( min.width() > 0 )
+	s.setWidth( min.width() );
+    if ( min.height() > 0 )
+	s.setHeight( min.height() );
+
+    if ( i->hasHeightForWidth() && min.height() == 0 && min.width() > 0 )
+	s.setHeight( i->heightForWidth(s.width()) );
+
+    s = s.expandedTo( QSize(1, 1) );
+    return s;
+}
+
+Q_EXPORT QSize qSmartMinSize( QWidget *w )
+{
+    QWidgetItem item( w );
+    return qSmartMinSize( &item );
+}
+
+Q_EXPORT QSize qSmartMaxSize( const QWidgetItem *i, int align )
+{
+    QWidget *w = ( (QWidgetItem*)i )->widget();
+    if ( align & Qt::AlignHorizontal_Mask && align & Qt::AlignVertical_Mask )
+	return QSize( QLAYOUTSIZE_MAX, QLAYOUTSIZE_MAX );
+    QSize s = w->maximumSize();
+    if ( s.width() == QWIDGETSIZE_MAX && !(align & Qt::AlignHorizontal_Mask) )
+	if ( !w->sizePolicy().mayGrowHorizontally() )
+	    s.setWidth( w->sizeHint().width() );
+
+    if ( s.height() == QWIDGETSIZE_MAX && !(align & Qt::AlignVertical_Mask) )
+	if ( !w->sizePolicy().mayGrowVertically() )
+	    s.setHeight( w->sizeHint().height() );
+
+    s = s.expandedTo( w->minimumSize() );
+
+    if ( align & Qt::AlignHorizontal_Mask )
+	s.setWidth( QLAYOUTSIZE_MAX );
+    if ( align & Qt::AlignVertical_Mask )
+	s.setHeight( QLAYOUTSIZE_MAX );
+    return s;
+}
+
+Q_EXPORT QSize qSmartMaxSize( QWidget *w, int align )
+{
+    QWidgetItem item( w );
+    return qSmartMaxSize( &item, align );
+}
+
 #endif // QT_NO_LAYOUT
