@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id:  qt/qlocalfs.cpp   3.0.5   edited Apr 30 16:14 $
+** $Id: qt/qlocalfs.cpp   3.2.3   edited Aug 12 16:21 $
 **
 ** Implementation of QLocalFs class
 **
@@ -52,6 +52,9 @@
     \class QLocalFs qlocalfs.h
     \brief The QLocalFs class is an implementation of a
     QNetworkProtocol that works on the local file system.
+\if defined(commercial)
+    It is part of the <a href="commercialeditions.html">Qt Enterprise Edition</a>.
+\endif
 
     \module network
 
@@ -84,6 +87,30 @@ QLocalFs::QLocalFs()
 {
 }
 
+static int convertPermissions(QFileInfo *fi)
+{
+    int p = 0;
+    if ( fi->permission( QFileInfo::ReadOwner ) )
+	p |= QUrlInfo::ReadOwner;
+    if ( fi->permission( QFileInfo::WriteOwner ) )
+	p |= QUrlInfo::WriteOwner;
+    if ( fi->permission( QFileInfo::ExeOwner ) )
+	p |= QUrlInfo::ExeOwner;
+    if ( fi->permission( QFileInfo::ReadGroup ) )
+	p |= QUrlInfo::ReadGroup;
+    if ( fi->permission( QFileInfo::WriteGroup ) )
+	p |= QUrlInfo::WriteGroup;
+    if ( fi->permission( QFileInfo::ExeGroup ) )
+	p |= QUrlInfo::ExeGroup;
+    if ( fi->permission( QFileInfo::ReadOther ) )
+	p |= QUrlInfo::ReadOther;
+    if ( fi->permission( QFileInfo::WriteOther ) )
+	p |= QUrlInfo::WriteOther;
+    if ( fi->permission( QFileInfo::ExeOther ) )
+	p |= QUrlInfo::ExeOther;
+    return p;
+}
+
 /*!
     \reimp
 */
@@ -124,7 +151,7 @@ void QLocalFs::operationListChildren( QNetworkOperation *op )
     QValueList<QUrlInfo> infos;
     while ( ( fi = it.current() ) != 0 ) {
 	++it;
-	infos << QUrlInfo( fi->fileName(), 0/*permissions*/, fi->owner(), fi->group(),
+	infos << QUrlInfo( fi->fileName(), convertPermissions(fi), fi->owner(), fi->group(),
 			   fi->size(), fi->lastModified(), fi->lastRead(), fi->isDir(), fi->isFile(),
 			   fi->isSymLink(), fi->isWritable(), fi->isReadable(), fi->isExecutable() );
     }
@@ -148,7 +175,7 @@ void QLocalFs::operationMkDir( QNetworkOperation *op )
     dir = QDir( url()->path() );
     if ( dir.mkdir( dirname ) ) {
 	QFileInfo fi( dir, dirname );
-	QUrlInfo inf( fi.fileName(), 0/*permissions*/, fi.owner(), fi.group(),
+	QUrlInfo inf( fi.fileName(), convertPermissions(&fi), fi.owner(), fi.group(),
 		      fi.size(), fi.lastModified(), fi.lastRead(), fi.isDir(), fi.isFile(),
 		      fi.isSymLink(), fi.isWritable(), fi.isReadable(), fi.isExecutable() );
 	emit newChild( inf, op );
@@ -252,39 +279,41 @@ void QLocalFs::operationGet( QNetworkOperation *op )
 
     QByteArray s;
     emit dataTransferProgress( 0, f.size(), op );
-    int blockSize = calcBlockSize( f.size() );
-    if ( (int)f.size() < blockSize ) {
-	s.resize( f.size() );
-	f.readBlock( s.data(), f.size() );
-	emit data( s, op );
-	emit dataTransferProgress( f.size(), f.size(), op );
+    if ( f.size() != 0 ) {
+	int blockSize = calcBlockSize( f.size() );
+	if ( (int)f.size() < blockSize ) {
+	    s.resize( f.size() );
+	    f.readBlock( s.data(), f.size() );
+	    emit data( s, op );
+	    emit dataTransferProgress( f.size(), f.size(), op );
 #ifdef QLOCALFS_DEBUG
-	qDebug( "QLocalFs: got all %d bytes at once", f.size() );
+	    qDebug( "QLocalFs: got all %d bytes at once", f.size() );
 #endif
-    } else {
-	s.resize( blockSize );
-	int remaining = f.size();
-	while ( remaining > 0 ) {
-	    if ( operationInProgress() != op )
-		return;
-	    if ( remaining >= blockSize ) {
-		f.readBlock( s.data(), blockSize );
-		emit data( s, op );
-		emit dataTransferProgress( f.size() - remaining, f.size(), op );
-		remaining -= blockSize;
-	    } else {
-		s.resize( remaining );
-		f.readBlock( s.data(), remaining );
-		emit data( s, op );
-		emit dataTransferProgress( f.size() - remaining, f.size(), op );
-		remaining -= remaining;
+	} else {
+	    s.resize( blockSize );
+	    int remaining = f.size();
+	    while ( remaining > 0 ) {
+		if ( operationInProgress() != op )
+		    return;
+		if ( remaining >= blockSize ) {
+		    f.readBlock( s.data(), blockSize );
+		    emit data( s, op );
+		    emit dataTransferProgress( f.size() - remaining, f.size(), op );
+		    remaining -= blockSize;
+		} else {
+		    s.resize( remaining );
+		    f.readBlock( s.data(), remaining );
+		    emit data( s, op );
+		    emit dataTransferProgress( f.size() - remaining, f.size(), op );
+		    remaining -= remaining;
+		}
+		qApp->processEvents();
 	    }
-	    qApp->processEvents();
-	}
 #ifdef QLOCALFS_DEBUG
-	qDebug( "QLocalFs: got all %d bytes step by step", f.size() );
+	    qDebug( "QLocalFs: got all %d bytes step by step", f.size() );
 #endif
-	emit dataTransferProgress( f.size(), f.size(), op );
+	    emit dataTransferProgress( f.size(), f.size(), op );
+	}
     }
     op->setState( StDone );
     f.close();
