@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id$
+** $Id: qt/qmetaobject.cpp   3.2.3   edited Jun 10 11:10 $
 **
 ** Implementation of QMetaObject class
 **
@@ -38,8 +38,13 @@
 #include "qmetaobject.h"
 #include "qasciidict.h"
 
+#ifdef QT_THREAD_SUPPORT
+#include <private/qmutexpool_p.h>
+#endif // QT_THREAD_SUPPORT
+
 /*!
   \class QMetaData qmetaobject.h
+  \reentrant
 
   \brief The QMetaData class provides information about a member function that is known to the meta object system.
 
@@ -110,8 +115,25 @@
   The private object.
  *****************************************************************************/
 
+// extra flags from moc.y
+enum Flags  {
+    Invalid		= 0x00000000,
+    Readable		= 0x00000001,
+    Writable		= 0x00000002,
+    EnumOrSet		= 0x00000004,
+    UnresolvedEnum	= 0x00000008,
+    StdSet		= 0x00000100,
+    Override		= 0x00000200,
+    NotDesignable	= 0x00001000,
+    DesignableOverride  = 0x00002000,
+    NotScriptable	= 0x00004000,
+    ScriptableOverride  = 0x00008000,
+    NotStored 		= 0x00010000,
+    StoredOverride 	= 0x00020000
+};
 
-static QAsciiDict<QMetaObject> *qt_metaobjects = 0;
+static QAsciiDict<void> *qt_metaobjects = 0;
+static int qt_metaobjects_count = 0;
 
 class QMetaObjectPrivate
 {
@@ -120,17 +142,18 @@ public:
 #ifndef QT_NO_PROPERTIES
 	enumData(0), numEnumData(0),
 	propData(0),numPropData(0),
+	qt_static_property(0),
 #endif
 	classInfo(0), numClassInfo(0) {}
 #ifndef QT_NO_PROPERTIES
-    const QMetaEnum     *enumData;			// enumeration types
-    int		   numEnumData;
-    const QMetaProperty *propData;                    // property meta data
-    int            numPropData;
+    const QMetaEnum *enumData;
+    int numEnumData;
+    const QMetaProperty *propData;
+    int numPropData;
+    bool (*qt_static_property)(QObject*, int, int, QVariant*);
 #endif
-    const QClassInfo    *classInfo;			// class information
-    int            numClassInfo;
-
+    const QClassInfo *classInfo;
+    int numClassInfo;
 };
 
 
@@ -213,11 +236,42 @@ QMetaObject::QMetaObject( const char *const class_name, QMetaObject *super_class
 #ifndef QT_NO_PROPERTIES
     propertyoffset = superclass ? ( superclass->propertyOffset() + superclass->numProperties() ) : 0;
 #endif
-    if ( !qt_metaobjects )
-	qt_metaobjects = new QAsciiDict<QMetaObject>( 257 );
-    qt_metaobjects->replace( class_name, this );
 }
 
+#ifndef QT_NO_PROPERTIES
+/*!\internal
+ */
+QMetaObject::QMetaObject( const char *const class_name, QMetaObject *super_class,
+			  const QMetaData *const slot_data, int n_slots,
+			  const QMetaData *const signal_data, int n_signals,
+			  const QMetaProperty *const prop_data, int n_props,
+			  const QMetaEnum *const enum_data, int n_enums,
+			  bool (*qt_static_property)(QObject*, int, int, QVariant*),
+			  const QClassInfo *const class_info, int n_info )
+{
+    classname = class_name;			// set meta data
+    superclass = super_class;
+    superclassname = superclass ? superclass->className() : 0;
+    slotDict = init( slotData = slot_data, n_slots );
+    signalDict = init( signalData = signal_data, n_signals );
+
+    d = new QMetaObjectPrivate;
+    reserved = 0;
+
+    d->propData = prop_data;
+    d->numPropData = n_props;
+    d->enumData = enum_data;
+    d->numEnumData = n_enums;
+    d->qt_static_property = qt_static_property;
+    d->classInfo = class_info;
+    d->numClassInfo = n_info;
+
+    signaloffset = superclass ? ( superclass->signalOffset() + superclass->numSignals() ) : 0;
+    slotoffset = superclass ? ( superclass->slotOffset() + superclass->numSlots() ) : 0;
+    propertyoffset = superclass ? ( superclass->propertyOffset() + superclass->numProperties() ) : 0;
+}
+#endif
+
 /*!\internal
  */
 QMetaObject::~QMetaObject()
@@ -225,6 +279,10 @@ QMetaObject::~QMetaObject()
     delete slotDict;				// delete dicts
     delete signalDict;
     delete d;
+#ifdef QT_THREAD_SUPPORT
+    QMutexLocker( qt_global_mutexpool ?
+		  qt_global_mutexpool->get( &qt_metaobjects ) : 0 );
+#endif // QT_THREAD_SUPPORT
     if ( qt_metaobjects ) {
 	qt_metaobjects->remove( classname );
 	if ( qt_metaobjects->isEmpty() ) {
@@ -351,15 +409,30 @@ const QMetaData* QMetaObject::signal( int index, bool super ) const
   Returns the index of the signal with name \n or -1 if no such signal exists.
 
   If  \a super is TRUE, inherited signals are included.
- */
+*/
 int QMetaObject::findSignal( const char* n, bool super ) const
 {
-    const QMetaData *md = signalDict ? signalDict->find( n ) : 0;
-    if ( md )
-	return signalOffset() + ( md - signalData );
-    if ( !super || !superclass)
-	return -1;
-    return superclass->findSignal( n, super );
+    const QMetaObject *mo = this;
+    int offset = -1;
+
+    do {
+	const QMetaData *md = mo->signalDict ? mo->signalDict->find( n ) : 0;
+	if ( md ) {
+#if defined(QT_CHECK_RANGE)
+	    if ( offset != -1 ) {
+		qWarning( "QMetaObject::findSignal:%s: Conflict with %s::%s",
+			  className(), mo->className(), n );
+		return offset;
+	    }
+#endif
+	    offset = mo->signalOffset() + ( md - mo->signalData );
+#if !defined(QT_CHECK_RANGE)
+	    return offset;
+#endif
+	}
+    } while ( super && (mo = mo->superclass) );
+
+    return offset;
 }
 
 /*!
@@ -407,6 +480,26 @@ QMetaObject *QMetaObject::new_metaobject( const char *classname,
 			    class_info, n_info );
 }
 
+#ifndef QT_NO_PROPERTIES
+/*!\internal
+ */
+QMetaObject *QMetaObject::new_metaobject( const char *classname,
+					  QMetaObject *superclassobject,
+					  const QMetaData * const slot_data, int n_slots,
+					  const QMetaData * const signal_data, int n_signals,
+					  const QMetaProperty * const prop_data, int n_props,
+					  const QMetaEnum * const enum_data, int n_enums,
+					  bool (*qt_static_property)(QObject*, int, int, QVariant*),
+					  const QClassInfo * const class_info, int n_info )
+{
+    return new QMetaObject( classname, superclassobject, slot_data, n_slots,
+			    signal_data, n_signals,
+			    prop_data, n_props,
+			    enum_data, n_enums,
+			    qt_static_property,
+			    class_info, n_info );
+}
+#endif
 
 /*!\internal
  */
@@ -564,7 +657,7 @@ const QMetaProperty* QMetaObject::resolveProperty( const QMetaProperty* p ) cons
 
   \overload
 
-  The veresion of resolveProperty that is used by moc generated code
+  The version of resolveProperty that is used by moc generated code
 */
 
 int QMetaObject::resolveProperty( int index ) const
@@ -632,6 +725,39 @@ QStrList QMetaObject::slotNames( bool super ) const
     return l;
 }
 
+/*!\internal
+
+ */
+
+int QMetaObject::numEnumerators( bool super ) const
+{
+    int n = 0;
+    if ( superclass && super )
+	n += superclass->numEnumerators( super );
+    return n + d->numEnumData;
+}
+
+/*!\internal
+
+ */
+QStrList QMetaObject::enumeratorNames( bool super ) const
+{
+    QStrList l( FALSE );
+
+    if ( superclass && super ) {
+	QStrList sl = superclass->enumeratorNames( super );
+	for ( QStrListIterator slit( sl ); slit.current(); ++slit )
+	    l.append( slit.current() );
+    }
+
+    for( int i = 0; i < d->numEnumData; ++i ) {
+	if ( d->enumData[i].items )
+	    l.append( d->enumData[i].name );
+    }
+
+    return l;
+}
+
 /*!\internal
  */
 const QMetaEnum* QMetaObject::enumerator( const char* name, bool super ) const
@@ -670,11 +796,51 @@ QMetaObject *QMetaObject::metaObject( const char *class_name )
 {
     if ( !qt_metaobjects )
 	return 0;
-    return qt_metaobjects->find( class_name );
+#ifdef QT_THREAD_SUPPORT
+    QMutexLocker( qt_global_mutexpool ?
+		  qt_global_mutexpool->get( &qt_metaobjects ) : 0 );
+#endif // QT_THREAD_SUPPORT
+    QtStaticMetaObjectFunction func = (QtStaticMetaObjectFunction)qt_metaobjects->find( class_name );
+    if ( func )
+	return func();
+    return 0;
+}
+
+/*! \internal */
+bool QMetaObject::hasMetaObject( const char *class_name )
+{
+    if ( !qt_metaobjects )
+	return FALSE;
+#ifdef QT_THREAD_SUPPORT
+    QMutexLocker( qt_global_mutexpool ?
+		  qt_global_mutexpool->get( &qt_metaobjects ) : 0 );
+#endif // QT_THREAD_SUPPORT
+    return !!qt_metaobjects->find( class_name );
+}
+
+#ifndef QT_NO_PROPERTIES
+/*! \internal
+
+### this functions will go away. It exists purely for the sake of meta
+### object code generated with Qt 3.1.0
+*/
+bool QMetaObject::qt_static_property( QObject* o, int id, int f, QVariant* v)
+{
+    if ( d->qt_static_property )
+	return d->qt_static_property( o, id, f, v );
+    else if ( o ) // compatibility
+	return o->qt_property( id, f, v );
+    else if ( superclass )
+	return superclass->qt_static_property( o, id, f, v );
+    switch ( f ) {
+    case 3: case 4: case 5:
+	return TRUE;
+    default:
+	return FALSE;
+    }
 }
 
 
-#ifndef QT_NO_PROPERTIES
 /*!
     \class QMetaProperty qmetaobject.h
 
@@ -708,17 +874,23 @@ QMetaObject *QMetaObject::metaObject( const char *class_name )
 */
 QStrList QMetaProperty::enumKeys() const
 {
-     QStrList l( FALSE );
-     const QMetaEnum* ed = enumData;
-     if ( !enumData && meta )
-	 ed = (*meta)->enumerator( t, TRUE );
-     if ( !ed )
-	 return l;
-     if ( ed != 0 ) {
-	 for( uint i = 0; i < ed->count; ++i )
-	     l.append( ed->items[i].key );
-     }
-     return l;
+    QStrList l( FALSE );
+    const QMetaEnum* ed = enumData;
+    if ( !enumData && meta )
+	ed = (*meta)->enumerator( t, TRUE );
+    if ( !ed )
+	return l;
+    if ( ed != 0 ) {
+	for( uint i = 0; i < ed->count; ++i ) {
+	    uint j = 0;
+	    while ( j < i &&
+		    ed->items[j].value != ed->items[i].value )
+		++j;
+	    if ( i == j )
+		l.append( ed->items[i].key );
+	}
+    }
+    return l;
 }
 
 /*!
@@ -731,13 +903,13 @@ QStrList QMetaProperty::enumKeys() const
 int QMetaProperty::keyToValue( const char* key ) const
 {
     const QMetaEnum* ed = enumData;
-     if ( !enumData && meta )
-	 ed = (*meta)->enumerator( t, TRUE );
+    if ( !enumData && meta )
+	ed = (*meta)->enumerator( t, TRUE );
     if ( !ed )
 	return -1;
-    for( uint i = ed->count; i > 0; --i ) {
-	if ( !qstrcmp( key, ed->items[i-1].key) )
-	    return ed->items[i-1].value;
+    for ( uint i = 0; i < ed->count; ++i ) {
+	if ( !qstrcmp( key, ed->items[i].key) )
+	    return ed->items[i].value;
     }
     return -1;
 }
@@ -752,13 +924,13 @@ int QMetaProperty::keyToValue( const char* key ) const
 const char* QMetaProperty::valueToKey( int value ) const
 {
     const QMetaEnum* ed = enumData;
-     if ( !enumData && meta )
-	 ed = (*meta)->enumerator( t, TRUE );
+    if ( !enumData && meta )
+	ed = (*meta)->enumerator( t, TRUE );
     if ( !ed )
 	return 0;
-    for( uint i = ed->count; i > 0; --i ) {
-	if ( value == ed->items[i-1].value )
-	    return ed->items[i-1].key ;
+    for ( uint i = 0; i < ed->count; ++i ) {
+	if ( value == ed->items[i].value )
+	    return ed->items[i].key ;
     }
     return 0;
 }
@@ -772,8 +944,8 @@ const char* QMetaProperty::valueToKey( int value ) const
 int QMetaProperty::keysToValue( const QStrList& keys ) const
 {
     const QMetaEnum* ed = enumData;
-     if ( !enumData && meta )
-	 ed = (*meta)->enumerator( t, TRUE );
+    if ( !enumData && meta )
+	ed = (*meta)->enumerator( t, TRUE );
     if ( !ed )
 	return -1;
     int value = 0;
@@ -800,8 +972,8 @@ QStrList QMetaProperty::valueToKeys( int value ) const
 {
     QStrList keys;
     const QMetaEnum* ed = enumData;
-     if ( !enumData && meta )
-	 ed = (*meta)->enumerator( t, TRUE );
+    if ( !enumData && meta )
+	ed = (*meta)->enumerator( t, TRUE );
     if ( !ed )
 	return keys;
 
@@ -856,6 +1028,10 @@ void QMetaProperty::clear()
 
 bool QMetaProperty::isValid() const
 {
+    if ( testFlags( UnresolvedEnum ) ) {
+	if ( !enumData && (!meta || !(*meta)->enumerator( t, TRUE ) ) )
+	    return FALSE;
+    }
     if ( !testFlags( Override ) || testFlags( Readable ) )
 	return testFlags( Readable );
     const QMetaObject* mo = (*meta);
@@ -866,8 +1042,8 @@ bool QMetaProperty::isValid() const
 bool QMetaProperty::isSetType() const
 {
     const QMetaEnum* ed = enumData;
-     if ( !enumData && meta )
-	 ed = (*meta)->enumerator( t, TRUE );
+    if ( !enumData && meta )
+	ed = (*meta)->enumerator( t, TRUE );
     return ( ed != 0 && ed->set );
 }
 
@@ -925,46 +1101,70 @@ bool QMetaProperty::isEnumType() const
 */
 
 
-/*!
-    Returns TRUE if the property is designable for object \a o;
+/*!  Returns TRUE if the property is designable for object \a o;
     otherwise returns FALSE.
+
+    If no object \a o is given, the function returns a static
+    approximation.
  */
 bool QMetaProperty::designable( QObject* o ) const
 {
-    if ( !o || !isValid() || !writable() )
+    if ( !isValid() || !writable() )
 	return FALSE;
-    int idx = _id >= 0 ? _id : (*meta)->indexOfProperty( this, TRUE );
-    if ( idx < 0 )
-	return 0;
-    return o->qt_property( idx, 3, 0 );
+    if ( o ) {
+	int idx = _id >= 0 ? _id : (*meta)->indexOfProperty( this, TRUE );
+	return idx >= 0 && o->qt_property( idx, 3, 0 );
+    }
+    if ( testFlags( DesignableOverride ) ) {
+	const QMetaObject* mo = (*meta);
+	const QMetaProperty* parent = mo->resolveProperty( this );
+	return parent ? parent->designable() : FALSE;
+    }
+    return !testFlags( NotDesignable );
 }
 
 /*!
     Returns TRUE if the property is scriptable for object \a o;
     otherwise returns FALSE.
+
+    If no object \a o is given, the function returns a static
+    approximation.
  */
 bool QMetaProperty::scriptable( QObject* o ) const
 {
-    if ( !o )
-	return FALSE;
-    int idx = _id >= 0 ? _id : (*meta)->indexOfProperty( this, TRUE );
-    if ( idx < 0 )
-	return 0;
-    return o->qt_property( idx, 4, 0 );
+    if ( o ) {
+	int idx = _id >= 0 ? _id : (*meta)->indexOfProperty( this, TRUE );
+	return idx >= 0 && o->qt_property( idx, 4, 0 );
+    }
+    if ( testFlags( ScriptableOverride ) ) {
+	const QMetaObject* mo = (*meta);
+	const QMetaProperty* parent = mo->resolveProperty( this );
+	return parent ? parent->scriptable() : FALSE;
+    }
+    return !testFlags( NotScriptable );
 }
 
 /*!
     Returns TRUE if the property shall be stored for object \a o;
     otherwise returns FALSE.
+
+    If no object \a o is given, the function returns a static
+    approximation.
  */
 bool QMetaProperty::stored( QObject* o ) const
 {
-    if ( !o || !isValid() || !writable() )
+    if ( !isValid() || !writable() )
 	return FALSE;
-    int idx = _id >= 0 ? _id : (*meta)->indexOfProperty( this, TRUE );
-    if ( idx < 0 )
-	return 0;
-    return o->qt_property( idx, 5, 0 );
+    if ( o ) {
+	int idx = _id >= 0 ? _id : (*meta)->indexOfProperty( this, TRUE );
+	return idx >= 0 && o->qt_property( idx, 5, 0 );
+    }
+    if ( testFlags( StoredOverride ) ) {
+	const QMetaObject* mo = (*meta);
+	const QMetaProperty* parent = mo->resolveProperty( this );
+	return parent ? parent->stored() : FALSE;
+    }
+    return !testFlags( NotStored );
 }
 
 
@@ -998,8 +1198,22 @@ bool QMetaProperty::reset( QObject* o ) const
  * files and deletes the QMetaObject provided with setMetaObject. It sets the
  * QObject reference to the metaObj to NULL when it is destroyed.
  */
+QMetaObjectCleanUp::QMetaObjectCleanUp( const char *mo_name, QtStaticMetaObjectFunction func )
+    : metaObject( 0 )
+{
+#ifdef QT_THREAD_SUPPORT
+    QMutexLocker( qt_global_mutexpool ?
+		  qt_global_mutexpool->get( &qt_metaobjects ) : 0 );
+#endif // QT_THREAD_SUPPORT
+    if ( !qt_metaobjects )
+	qt_metaobjects = new QAsciiDict<void>( 257 );
+    qt_metaobjects->insert( mo_name, (void*)func );
+
+    qt_metaobjects_count++;
+}
+
 QMetaObjectCleanUp::QMetaObjectCleanUp()
-: metaObject( 0 )
+    : metaObject( 0 )
 {
 }
 
@@ -1009,6 +1223,14 @@ QMetaObjectCleanUp::QMetaObjectCleanUp()
 
 QMetaObjectCleanUp::~QMetaObjectCleanUp()
 {
+#ifdef QT_THREAD_SUPPORT
+    QMutexLocker( qt_global_mutexpool ?
+		  qt_global_mutexpool->get( &qt_metaobjects ) : 0 );
+#endif // QT_THREAD_SUPPORT
+    if ( !--qt_metaobjects_count ) {
+	delete qt_metaobjects;
+	qt_metaobjects = 0;
+    }
     if ( metaObject ) {
 	delete *metaObject;
 	*metaObject = 0;
