@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id:  qt/qpainter.h   3.0.3   edited Oct 12 12:18 $
+** $Id: qt/qpainter.h   3.2.3   edited May 13 09:08 $
 **
 ** Definition of QPainter class
 **
@@ -54,6 +54,7 @@ class QGfx;
 class QTextCodec;
 class QTextParag;
 class QPaintDevice;
+class QTextItem;
 #if defined( Q_WS_MAC )
 class QMacSavedPortInfo;
 #endif
@@ -79,6 +80,7 @@ public:
 #endif
 
     static void redirect( QPaintDevice *pdev, QPaintDevice *replacement );
+    static QPaintDevice *redirect( QPaintDevice *pdev );
 
     bool	isActive() const;
 
@@ -259,9 +261,8 @@ public:
 			  const QString&, int len = -1, QRect *br=0,
 			  QTextParag **intern=0 );
 
-    //#####    void	drawText( const QPoint &, const QString &, int flags, int rotation = 0);
-
-  // Text drawing functions
+    void drawTextItem( int x, int y, const QTextItem &ti, int textflags = 0 );
+    void drawTextItem( const QPoint& p, const QTextItem &ti, int textflags = 0 );
 
     QRect	boundingRect( int x, int y, int w, int h, int flags,
 			      const QString&, int len = -1, QTextParag **intern=0 );
@@ -277,7 +278,7 @@ public:
 
 #if defined(Q_WS_WIN)
     HDC		handle() const;
-#elif defined(Q_WS_X11)
+#elif defined(Q_WS_X11) || defined(Q_WS_MAC)
     HANDLE	handle() const;
 #endif
 
@@ -299,7 +300,11 @@ private:
     void	map( int, int, int, int, int *, int *, int *, int * ) const;
     void	mapInv( int, int, int *, int * ) const;
     void	mapInv( int, int, int, int, int *, int *, int *, int * ) const;
+#ifdef Q_WS_MACX
+    void	drawPolyInternal( const QPointArray &, bool close=TRUE, bool inset=TRUE );
+#else
     void	drawPolyInternal( const QPointArray &, bool close=TRUE );
+#endif
     void	drawWinFocusRect( int x, int y, int w, int h, bool xorPaint,
 				  const QColor &penColor );
 
@@ -377,33 +382,53 @@ protected:
 	QPoint	internalCurrentPos;
 #endif
 #if defined(Q_WS_WIN)
+    friend class QFontEngineWin;
+    friend class QFontEngineBox;
     QT_WIN_PAINTER_MEMBERS
 #elif defined(Q_WS_X11)
+    friend class QFontEngineXLFD;
+    friend class QFontEngineXft;
+    friend class QFontEngineBox;
     Display    *dpy;				// current display
     int		scrn;				// current screen
     Qt::HANDLE	hd;				// handle to drawable
-    Qt::HANDLE  rendhd;				// handle to Render Picture
+    Qt::HANDLE  rendhd;				// handle to Xft draw
     GC		gc;				// graphics context (standard)
     GC		gc_brush;			// graphics contect for brush
     QPoint	curPt;				// current point
     uint	clip_serial;			// clipping serial number
 #elif defined(Q_WS_MAC)
-    void initPaintDevice(bool force=FALSE);
+    Qt::HANDLE	hd;				// handle to drawable
+    void initPaintDevice(bool force=FALSE, QPoint *off=NULL, QRegion *rgn=NULL);
+    friend const QRegion &qt_mac_update_painter(QPainter *, bool);
+    friend class QFontEngineMac;
 #elif defined(Q_WS_QWS)
+    friend class QFontEngine;
     QGfx * gfx;
 #endif
     friend class QFontMetrics;
     friend class QFontInfo;
+    friend class QTextLayout;
     friend void qt_format_text( const QFont &, const QRect &r,
 		     int tf, const QString& str, int len, QRect *brect,
 		     int tabstops, int* tabarray, int tabarraylen,
 		     QTextParag **internal, QPainter* painter );
+    friend void qt_draw_background( QPainter *p, int x, int y, int w,  int h );
+    friend void qt_draw_transformed_rect( QPainter *p,  int x, int y, int w,  int h, bool fill );
+    friend class QPrinter;
 
 private:	// Disabled copy constructor and operator=
 #if defined(Q_DISABLE_COPY)
     QPainter( const QPainter & );
     QPainter &operator=( const QPainter & );
 #endif
+
+    enum TransformationCodes {
+	TxNone      = 0,		// transformation codes
+	TxTranslate = 1,		// copy in qpainter_*.cpp
+	TxScale     = 2,
+	TxRotShear  = 3
+    };
 };
 
 
@@ -520,7 +545,7 @@ inline HDC QPainter::handle() const
 {
     return hdc;
 }
-#elif defined(Q_WS_X11)
+#elif defined(Q_WS_X11) || defined(Q_WS_MAC)
 inline Qt::HANDLE QPainter::handle() const
 {
     return hd;
@@ -665,6 +690,11 @@ inline void QPainter::drawText( int x, int y, int w, int h, int tf,
     drawText( r, tf, str, len, br, i );
 }
 
+inline void QPainter::drawTextItem( const QPoint& p, const QTextItem &ti, int textflags )
+{
+    drawTextItem( p.x(), p.y(), ti, textflags );
+}
+
 inline QRect QPainter::boundingRect( int x, int y, int w, int h, int tf,
 				     const QString& str, int len, QTextParag **i )
 {
