@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id:  qt/qpainter_x11.cpp   3.0.5   edited May 27 20:55 $
+** $Id: qt/qpainter_x11.cpp   3.2.3   edited Sep 16 11:20 $
 **
 ** Implementation of QPainter class for X11
 **
 ** Created : 940112
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2002 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the kernel module of the Qt GUI Toolkit.
 **
@@ -37,21 +37,23 @@
 
 #include "qplatformdefs.h"
 
+#include "qfont.h"
 #include "qpainter.h"
 #include "qwidget.h"
 #include "qbitmap.h"
 #include "qpixmapcache.h"
-#include "qintdict.h"
-#include "qfontdata_p.h"
-#include "qcomplextext_p.h"
 #include "qtextcodec.h"
 #include "qpaintdevicemetrics.h"
-#include "qt_x11.h"
+
+#include "qt_x11_p.h"
+
+#include "qtextlayout_p.h"
+#include "qfontdata_p.h"
+#include "qfontengine_p.h"
+#include "qtextengine_p.h"
 
 #include <math.h>
 
-// REVISED: arnt
-
 // paintevent magic to provide Windows semantics on X11
 static QRegion* paintEventClipRegion = 0;
 static QPaintDevice* paintEventDevice = 0;
@@ -87,16 +89,10 @@ void qt_erase_region( QWidget* w, const QRegion& region)
 
     if ( !w->isTopLevel() && w->backgroundPixmap()
          && w->backgroundOrigin() != QWidget::WidgetOrigin ) {
-        int ox = w->x();
-        int oy = w->y();
-        if ( w->backgroundOrigin() == QWidget::WindowOrigin ) {
-	    QWidget *topl = w;
-	    while(!topl->isTopLevel() && !topl->testWFlags(Qt::WSubWindow))
-		topl = topl->parentWidget(TRUE);
-            QPoint p = w->mapTo( topl, QPoint(0, 0) );
-            ox = p.x();
-            oy = p.y();
-        }
+	QPoint offset = w->backgroundOffset();
+	int ox = offset.x();
+	int oy = offset.y();
+
         bool unclipped = w->testWFlags( Qt::WPaintUnclipped );
         if ( unclipped )
             ((QWFlagWidget*)w)->clearWFlags( Qt::WPaintUnclipped );
@@ -130,7 +126,44 @@ void qt_erase_rect( QWidget* w, const QRect& r)
 
 }
 
+#ifdef QT_NO_XRENDER
+typedef unsigned long Picture;
+static const Picture rendhd = 0;
+#endif
 
+// hack, so we don't have to make QRegion::clipRectangles() public or include
+// X11 headers in qregion.h
+inline void *qt_getClipRects( const QRegion &r, int &num )
+{
+    return r.clipRectangles( num );
+}
+
+static inline void x11SetClipRegion( Display *dpy, GC gc, Picture rh, const QRegion &r )
+{
+    int num;
+    XRectangle *rects = (XRectangle *)qt_getClipRects( r, num );
+#ifndef QT_NO_XRENDER
+    if (rh)
+	XRenderSetPictureClipRectangles(dpy, rh, 0, 0, rects, num );
+#else
+    Q_UNUSED( rh );
+#endif // QT_NO_XRENDER
+    XSetClipRectangles( dpy, gc, 0, 0, rects, num, Unsorted );
+}
+
+static inline void x11SetClipRegion( Display *dpy, GC gc, Picture rh, const QRegion &r, GC gc2 )
+{
+    int num;
+    XRectangle *rects = (XRectangle *)qt_getClipRects( r, num );
+#ifndef QT_NO_XRENDER
+    if (rh)
+	XRenderSetPictureClipRectangles(dpy, rh, 0, 0, rects, num );
+#else
+    Q_UNUSED( rh );
+#endif // QT_NO_XRENDER
+    XSetClipRectangles( dpy, gc, 0, 0, rects, num, Unsorted );
+    XSetClipRectangles( dpy, gc2, 0, 0, rects, num, Unsorted );
+}
 
 
 /*****************************************************************************
@@ -504,12 +537,6 @@ static inline void release_gc( void *ref )
   QPainter member functions
  *****************************************************************************/
 
-const int TxNone      = 0;                      // transformation codes
-const int TxTranslate = 1;                      // also in qpainter.cpp
-const int TxScale     = 2;
-const int TxRotShear  = 3;
-
-
 /*!
   \internal
 
@@ -531,7 +558,7 @@ void QPainter::initialize()
 void QPainter::cleanup()
 {
     cleanup_gc_cache();
-    cleanup_gc_array( qt_xdisplay() );
+    cleanup_gc_array( QPaintDevice::x11AppDisplay() );
     QPointArray::cleanBuffers();
 }
 
@@ -546,42 +573,6 @@ void QPainter::destroy()
 
 }
 
-typedef QIntDict<QPaintDevice> QPaintDeviceDict;
-static QPaintDeviceDict *pdev_dict = 0;
-
-/*!
-    Redirects all paint commands for a paint device, \a pdev, to
-    another paint device, \a replacement, unless \a replacement is 0.
-    If \a replacement is 0, the redirection for \a pdev is removed.
-
-    In general, you'll probably find calling QPixmap::grabWidget() or
-    QPixmap::grabWindow() is an easier solution.
-*/
-
-void QPainter::redirect( QPaintDevice *pdev, QPaintDevice *replacement )
-{
-    if ( pdev_dict == 0 ) {
-        if ( replacement == 0 )
-            return;
-        pdev_dict = new QPaintDeviceDict;
-        Q_CHECK_PTR( pdev_dict );
-    }
-#if defined(QT_CHECK_NULL)
-    if ( pdev == 0 )
-        qWarning( "QPainter::redirect: The pdev argument cannot be 0" );
-#endif
-    if ( replacement ) {
-        pdev_dict->insert( (long)pdev, replacement );
-    } else {
-        pdev_dict->remove( (long)pdev );
-        if ( pdev_dict->count() == 0 ) {
-            delete pdev_dict;
-            pdev_dict = 0;
-        }
-    }
-}
-
-
 void QPainter::init()
 {
     d = 0;
@@ -630,6 +621,7 @@ void QPainter::setFont( const QFont &font )
 #endif
     if ( cfont.d != font.d ) {
         cfont = font;
+	cfont.x11SetScreen( scrn );
         setf(DirtyFont);
     }
 }
@@ -675,7 +667,7 @@ void QPainter::updatePen()
             else
                 free_gc( dpy, gc );
         }
-        obtained = obtain_gc(&penRef, &gc, cpen.color().pixel(), dpy, scrn,
+        obtained = obtain_gc(&penRef, &gc, cpen.color().pixel(scrn), dpy, scrn,
 			     hd, clip_serial);
         if ( !obtained && !penRef )
             gc = alloc_gc( dpy, scrn, hd, FALSE );
@@ -696,18 +688,10 @@ void QPainter::updatePen()
     if ( !internclipok ) {
         if ( pdev == paintEventDevice && paintEventClipRegion ) {
             if ( penRef &&((QGCC*)penRef)->clip_serial < gc_cache_clip_serial ) {
-#ifndef QT_NO_XRENDER
-		if (rendhd)
-		    XRenderSetPictureClipRegion(dpy, rendhd, paintEventClipRegion->handle());
-#endif // QT_NO_XRENDER
-                XSetRegion( dpy, gc, paintEventClipRegion->handle() );
+		x11SetClipRegion( dpy, gc, rendhd, *paintEventClipRegion );
                 ((QGCC*)penRef)->clip_serial = gc_cache_clip_serial;
             } else if ( !penRef ) {
-#ifndef QT_NO_XRENDER
-		if (rendhd)
-		    XRenderSetPictureClipRegion(dpy, rendhd, paintEventClipRegion->handle());
-#endif // QT_NO_XRENDER
-                XSetRegion( dpy, gc, paintEventClipRegion->handle() );
+		x11SetClipRegion( dpy, gc, rendhd, *paintEventClipRegion );
             }
         } else if (penRef && ((QGCC*)penRef)->clip_serial ) {
 #ifndef QT_NO_XRENDER
@@ -745,39 +729,39 @@ void QPainter::updatePen()
     }
 
     switch( ps ) {
-        case NoPen:
-        case SolidLine:
-            s = LineSolid;
-            break;
-        case DashLine:
-            dashes[0] = fudge * 3 * dot;
-            dashes[1] = fudge * dot;
-            dash_len = 2;
-	    allow_zero_lw = FALSE;
-            break;
-        case DotLine:
-            dashes[0] = dot;
-            dashes[1] = dot;
-            dash_len = 2;
-	    allow_zero_lw = FALSE;
-            break;
-        case DashDotLine:
-            dashes[0] = 3 * dot;
-            dashes[1] = fudge * dot;
-            dashes[2] = dot;
-            dashes[3] = fudge * dot;
-            dash_len = 4;
-	    allow_zero_lw = FALSE;
-            break;
-        case DashDotDotLine:
-            dashes[0] = 3 * dot;
-            dashes[1] = dot;
-            dashes[2] = dot;
-            dashes[3] = dot;
-            dashes[4] = dot;
-            dashes[5] = dot;
-            dash_len = 6;
-	    allow_zero_lw = FALSE;
+    case NoPen:
+    case SolidLine:
+	s = LineSolid;
+	break;
+    case DashLine:
+	dashes[0] = fudge * 3 * dot;
+	dashes[1] = fudge * dot;
+	dash_len = 2;
+	allow_zero_lw = FALSE;
+	break;
+    case DotLine:
+	dashes[0] = dot;
+	dashes[1] = dot;
+	dash_len = 2;
+	allow_zero_lw = FALSE;
+	break;
+    case DashDotLine:
+	dashes[0] = 3 * dot;
+	dashes[1] = fudge * dot;
+	dashes[2] = dot;
+	dashes[3] = fudge * dot;
+	dash_len = 4;
+	allow_zero_lw = FALSE;
+	break;
+    case DashDotDotLine:
+	dashes[0] = 3 * dot;
+	dashes[1] = dot;
+	dashes[2] = dot;
+	dashes[3] = dot;
+	dashes[4] = dot;
+	dashes[5] = dot;
+	dash_len = 6;
+	allow_zero_lw = FALSE;
     }
     Q_ASSERT( dash_len <= (int) sizeof(dashes) );
 
@@ -806,8 +790,8 @@ void QPainter::updatePen()
         break;
     }
 
-    XSetForeground( dpy, gc, cpen.color().pixel() );
-    XSetBackground( dpy, gc, bg_col.pixel() );
+    XSetForeground( dpy, gc, cpen.color().pixel(scrn) );
+    XSetBackground( dpy, gc, bg_col.pixel(scrn) );
 
     if ( dash_len ) {                           // make dash list
         XSetDashes( dpy, gc, 0, dashes, dash_len );
@@ -821,50 +805,50 @@ void QPainter::updatePen()
 
 void QPainter::updateBrush()
 {
-static const uchar dense1_pat[] = { 0xff, 0xbb, 0xff, 0xff, 0xff, 0xbb, 0xff, 0xff };
-static const uchar dense2_pat[] = { 0x77, 0xff, 0xdd, 0xff, 0x77, 0xff, 0xdd, 0xff };
-static const uchar dense3_pat[] = { 0x55, 0xbb, 0x55, 0xee, 0x55, 0xbb, 0x55, 0xee };
-static const uchar dense4_pat[] = { 0x55, 0xaa, 0x55, 0xaa, 0x55, 0xaa, 0x55, 0xaa };
-static const uchar dense5_pat[] = { 0xaa, 0x44, 0xaa, 0x11, 0xaa, 0x44, 0xaa, 0x11 };
-static const uchar dense6_pat[] = { 0x88, 0x00, 0x22, 0x00, 0x88, 0x00, 0x22, 0x00 };
-static const uchar dense7_pat[] = { 0x00, 0x44, 0x00, 0x00, 0x00, 0x44, 0x00, 0x00 };
-static const uchar hor_pat[] = {                      // horizontal pattern
-    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff,
-    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-    0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff,
-    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-    0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };
-static const uchar ver_pat[] = {                      // vertical pattern
-    0x08, 0x82, 0x20, 0x08, 0x82, 0x20, 0x08, 0x82, 0x20, 0x08, 0x82, 0x20,
-    0x08, 0x82, 0x20, 0x08, 0x82, 0x20, 0x08, 0x82, 0x20, 0x08, 0x82, 0x20,
-    0x08, 0x82, 0x20, 0x08, 0x82, 0x20, 0x08, 0x82, 0x20, 0x08, 0x82, 0x20,
-    0x08, 0x82, 0x20, 0x08, 0x82, 0x20, 0x08, 0x82, 0x20, 0x08, 0x82, 0x20,
-    0x08, 0x82, 0x20, 0x08, 0x82, 0x20, 0x08, 0x82, 0x20, 0x08, 0x82, 0x20,
-    0x08, 0x82, 0x20, 0x08, 0x82, 0x20, 0x08, 0x82, 0x20, 0x08, 0x82, 0x20 };
-static const uchar cross_pat[] = {                    // cross pattern
-    0x08, 0x82, 0x20, 0x08, 0x82, 0x20, 0x08, 0x82, 0x20, 0xff, 0xff, 0xff,
-    0x08, 0x82, 0x20, 0x08, 0x82, 0x20, 0x08, 0x82, 0x20, 0x08, 0x82, 0x20,
-    0x08, 0x82, 0x20, 0xff, 0xff, 0xff, 0x08, 0x82, 0x20, 0x08, 0x82, 0x20,
-    0x08, 0x82, 0x20, 0x08, 0x82, 0x20, 0x08, 0x82, 0x20, 0xff, 0xff, 0xff,
-    0x08, 0x82, 0x20, 0x08, 0x82, 0x20, 0x08, 0x82, 0x20, 0x08, 0x82, 0x20,
-    0x08, 0x82, 0x20, 0xff, 0xff, 0xff, 0x08, 0x82, 0x20, 0x08, 0x82, 0x20 };
-static const uchar bdiag_pat[] = {                    // backward diagonal pattern
-    0x20, 0x20, 0x10, 0x10, 0x08, 0x08, 0x04, 0x04, 0x02, 0x02, 0x01, 0x01,
-    0x80, 0x80, 0x40, 0x40, 0x20, 0x20, 0x10, 0x10, 0x08, 0x08, 0x04, 0x04,
-    0x02, 0x02, 0x01, 0x01, 0x80, 0x80, 0x40, 0x40 };
-static const uchar fdiag_pat[] = {                    // forward diagonal pattern
-    0x02, 0x02, 0x04, 0x04, 0x08, 0x08, 0x10, 0x10, 0x20, 0x20, 0x40, 0x40,
-    0x80, 0x80, 0x01, 0x01, 0x02, 0x02, 0x04, 0x04, 0x08, 0x08, 0x10, 0x10,
-    0x20, 0x20, 0x40, 0x40, 0x80, 0x80, 0x01, 0x01 };
-static const uchar dcross_pat[] = {                   // diagonal cross pattern
-    0x22, 0x22, 0x14, 0x14, 0x08, 0x08, 0x14, 0x14, 0x22, 0x22, 0x41, 0x41,
-    0x80, 0x80, 0x41, 0x41, 0x22, 0x22, 0x14, 0x14, 0x08, 0x08, 0x14, 0x14,
-    0x22, 0x22, 0x41, 0x41, 0x80, 0x80, 0x41, 0x41 };
-static const uchar * const pat_tbl[] = {
-    dense1_pat, dense2_pat, dense3_pat, dense4_pat, dense5_pat,
-    dense6_pat, dense7_pat,
-    hor_pat, ver_pat, cross_pat, bdiag_pat, fdiag_pat, dcross_pat };
+    static const uchar dense1_pat[] = { 0xff, 0xbb, 0xff, 0xff, 0xff, 0xbb, 0xff, 0xff };
+    static const uchar dense2_pat[] = { 0x77, 0xff, 0xdd, 0xff, 0x77, 0xff, 0xdd, 0xff };
+    static const uchar dense3_pat[] = { 0x55, 0xbb, 0x55, 0xee, 0x55, 0xbb, 0x55, 0xee };
+    static const uchar dense4_pat[] = { 0x55, 0xaa, 0x55, 0xaa, 0x55, 0xaa, 0x55, 0xaa };
+    static const uchar dense5_pat[] = { 0xaa, 0x44, 0xaa, 0x11, 0xaa, 0x44, 0xaa, 0x11 };
+    static const uchar dense6_pat[] = { 0x88, 0x00, 0x22, 0x00, 0x88, 0x00, 0x22, 0x00 };
+    static const uchar dense7_pat[] = { 0x00, 0x44, 0x00, 0x00, 0x00, 0x44, 0x00, 0x00 };
+    static const uchar hor_pat[] = {                      // horizontal pattern
+	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff,
+	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+	0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff,
+	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+	0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };
+    static const uchar ver_pat[] = {                      // vertical pattern
+	0x08, 0x82, 0x20, 0x08, 0x82, 0x20, 0x08, 0x82, 0x20, 0x08, 0x82, 0x20,
+	0x08, 0x82, 0x20, 0x08, 0x82, 0x20, 0x08, 0x82, 0x20, 0x08, 0x82, 0x20,
+	0x08, 0x82, 0x20, 0x08, 0x82, 0x20, 0x08, 0x82, 0x20, 0x08, 0x82, 0x20,
+	0x08, 0x82, 0x20, 0x08, 0x82, 0x20, 0x08, 0x82, 0x20, 0x08, 0x82, 0x20,
+	0x08, 0x82, 0x20, 0x08, 0x82, 0x20, 0x08, 0x82, 0x20, 0x08, 0x82, 0x20,
+	0x08, 0x82, 0x20, 0x08, 0x82, 0x20, 0x08, 0x82, 0x20, 0x08, 0x82, 0x20 };
+    static const uchar cross_pat[] = {                    // cross pattern
+	0x08, 0x82, 0x20, 0x08, 0x82, 0x20, 0x08, 0x82, 0x20, 0xff, 0xff, 0xff,
+	0x08, 0x82, 0x20, 0x08, 0x82, 0x20, 0x08, 0x82, 0x20, 0x08, 0x82, 0x20,
+	0x08, 0x82, 0x20, 0xff, 0xff, 0xff, 0x08, 0x82, 0x20, 0x08, 0x82, 0x20,
+	0x08, 0x82, 0x20, 0x08, 0x82, 0x20, 0x08, 0x82, 0x20, 0xff, 0xff, 0xff,
+	0x08, 0x82, 0x20, 0x08, 0x82, 0x20, 0x08, 0x82, 0x20, 0x08, 0x82, 0x20,
+	0x08, 0x82, 0x20, 0xff, 0xff, 0xff, 0x08, 0x82, 0x20, 0x08, 0x82, 0x20 };
+    static const uchar bdiag_pat[] = {                    // backward diagonal pattern
+	0x20, 0x20, 0x10, 0x10, 0x08, 0x08, 0x04, 0x04, 0x02, 0x02, 0x01, 0x01,
+	0x80, 0x80, 0x40, 0x40, 0x20, 0x20, 0x10, 0x10, 0x08, 0x08, 0x04, 0x04,
+	0x02, 0x02, 0x01, 0x01, 0x80, 0x80, 0x40, 0x40 };
+    static const uchar fdiag_pat[] = {                    // forward diagonal pattern
+	0x02, 0x02, 0x04, 0x04, 0x08, 0x08, 0x10, 0x10, 0x20, 0x20, 0x40, 0x40,
+	0x80, 0x80, 0x01, 0x01, 0x02, 0x02, 0x04, 0x04, 0x08, 0x08, 0x10, 0x10,
+	0x20, 0x20, 0x40, 0x40, 0x80, 0x80, 0x01, 0x01 };
+    static const uchar dcross_pat[] = {                   // diagonal cross pattern
+	0x22, 0x22, 0x14, 0x14, 0x08, 0x08, 0x14, 0x14, 0x22, 0x22, 0x41, 0x41,
+	0x80, 0x80, 0x41, 0x41, 0x22, 0x22, 0x14, 0x14, 0x08, 0x08, 0x14, 0x14,
+	0x22, 0x22, 0x41, 0x41, 0x80, 0x80, 0x41, 0x41 };
+    static const uchar * const pat_tbl[] = {
+	dense1_pat, dense2_pat, dense3_pat, dense4_pat, dense5_pat,
+	dense6_pat, dense7_pat,
+	hor_pat, ver_pat, cross_pat, bdiag_pat, fdiag_pat, dcross_pat };
 
     if ( testf(ExtDev) ) {
         QPDevCmdParam param[1];
@@ -876,7 +860,7 @@ static const uchar * const pat_tbl[] = {
     int  bs      = cbrush.style();
     bool cacheIt = !testf(ClipOn|MonoDev|NoCache) &&
                    (bs == NoBrush || bs == SolidPattern) &&
-                   bro.x() == 0 && bro.y() == 0 && rop == CopyROP;
+	           bro.x() == 0 && bro.y() == 0 && rop == CopyROP;
 
     bool obtained = FALSE;
     bool internclipok = hasClipping();
@@ -887,7 +871,7 @@ static const uchar * const pat_tbl[] = {
             else
                 free_gc( dpy, gc_brush );
         }
-        obtained = obtain_gc(&brushRef, &gc_brush, cbrush.color().pixel(), dpy,
+        obtained = obtain_gc(&brushRef, &gc_brush, cbrush.color().pixel(scrn), dpy,
 			     scrn, hd, clip_serial);
         if ( !obtained && !brushRef )
             gc_brush = alloc_gc( dpy, scrn, hd, FALSE );
@@ -908,18 +892,10 @@ static const uchar * const pat_tbl[] = {
     if ( !internclipok ) {
         if ( pdev == paintEventDevice && paintEventClipRegion ) {
             if ( brushRef &&((QGCC*)brushRef)->clip_serial < gc_cache_clip_serial ) {
-#ifndef QT_NO_XRENDER
-            if (rendhd)
-                XRenderSetPictureClipRegion(dpy, rendhd, paintEventClipRegion->handle());
-#endif // QT_NO_XRENDER
-                XSetRegion( dpy, gc_brush, paintEventClipRegion->handle() );
+		x11SetClipRegion( dpy, gc_brush, rendhd, *paintEventClipRegion );
                 ((QGCC*)brushRef)->clip_serial = gc_cache_clip_serial;
             } else if ( !brushRef ){
-#ifndef QT_NO_XRENDER
-            if (rendhd)
-                XRenderSetPictureClipRegion(dpy, rendhd, paintEventClipRegion->handle());
-#endif // QT_NO_XRENDER
-                XSetRegion( dpy, gc_brush, paintEventClipRegion->handle() );
+		x11SetClipRegion( dpy, gc_brush, rendhd, *paintEventClipRegion );
             }
         } else if (brushRef && ((QGCC*)brushRef)->clip_serial ) {
 #ifndef QT_NO_XRENDER
@@ -951,8 +927,8 @@ static const uchar * const pat_tbl[] = {
     }
 
     XSetLineAttributes( dpy, gc_brush, 0, LineSolid, CapButt, JoinMiter );
-    XSetForeground( dpy, gc_brush, cbrush.color().pixel() );
-    XSetBackground( dpy, gc_brush, bg_col.pixel() );
+    XSetForeground( dpy, gc_brush, cbrush.color().pixel(scrn) );
+    XSetBackground( dpy, gc_brush, bg_col.pixel(scrn) );
 
     if ( bs == CustomPattern || pat ) {
         QPixmap *pm;
@@ -1032,16 +1008,12 @@ bool QPainter::begin( const QPaintDevice *pd, bool unclipped )
     QPixmap::x11SetDefaultScreen( pd->x11Screen() );
 
     const QWidget *copyFrom = 0;
-    if ( pdev_dict ) {                          // redirected paint device?
-        pdev = pdev_dict->find( (long)pd );
-        if ( pdev ) {
-            if ( pd->devType() == QInternal::Widget )
-                copyFrom = (const QWidget *)pd; // copy widget settings
-        } else {
-            pdev = (QPaintDevice *)pd;
-        }
+    pdev = redirect( (QPaintDevice*)pd );
+    if ( pdev ) {				    // redirected paint device?
+	if ( pd->devType() == QInternal::Widget )
+	    copyFrom = (const QWidget *)pd;	    // copy widget settings
     } else {
-        pdev = (QPaintDevice *)pd;
+	pdev = (QPaintDevice*)pd;
     }
 
     if ( pdev->isExtDev() && pdev->paintingActive() ) {
@@ -1064,7 +1036,7 @@ bool QPainter::begin( const QPaintDevice *pd, bool unclipped )
         ((QPixmap*)pdev)->detach();             // will modify it
 
     dpy = pdev->x11Display();                   // get display variable
-    scrn = pdev->x11Screen();           // get screen variable
+    scrn = pdev->x11Screen();			// get screen variable
     hd  = pdev->handle();                       // get handle to drawable
     rendhd = pdev->x11RenderHandle();
 
@@ -1084,7 +1056,7 @@ bool QPainter::begin( const QPaintDevice *pd, bool unclipped )
             setTabArray( tabarray );
     }
 
-    if ( pdev->x11Depth() != pdev->x11AppDepth() ) {    // non-standard depth
+    if ( pdev->x11Depth() != pdev->x11AppDepth( scrn ) ) { // non-standard depth
         setf(NoCache);
         setf(UsePrivateCx);
     }
@@ -1187,7 +1159,7 @@ bool QPainter::begin( const QPaintDevice *pd, bool unclipped )
 	// ...
 	p.end(); // stops drawing on myPixmap
 	p.begin( this );
-	p.drawPixmap( myPixmap );
+	p.drawPixmap( 0, 0, myPixmap );
     \endcode
 
     Since you can't draw a QPixmap while it is being painted, it is
@@ -1205,10 +1177,6 @@ bool QPainter::end()                            // end painting
         return FALSE;
     }
     killPStack();
-    if ( testf(FontMet) )                       // remove references to this
-        QFontMetrics::reset( this );
-    if ( testf(FontInf) )                       // remove references to this
-        QFontInfo::reset( this );
 
     //#### This should not be necessary:
     if ( pdev->devType() == QInternal::Widget  &&       // #####
@@ -1488,26 +1456,12 @@ void QPainter::setClipping( bool enable )
             rgn = rgn.intersect( *paintEventClipRegion );
         if ( penRef )
             updatePen();
-        XSetRegion( dpy, gc, rgn.handle() );
         if ( brushRef )
             updateBrush();
-        XSetRegion( dpy, gc_brush, rgn.handle() );
-
-#ifndef QT_NO_XRENDER
-        if (rendhd)
-            XRenderSetPictureClipRegion(dpy, rendhd, rgn.handle());
-#endif // QT_NO_XRENDER
-
+	x11SetClipRegion( dpy, gc, rendhd, rgn, gc_brush );
     } else {
         if ( pdev == paintEventDevice && paintEventClipRegion ) {
-            XSetRegion( dpy, gc, paintEventClipRegion->handle() );
-            XSetRegion( dpy, gc_brush, paintEventClipRegion->handle() );
-
-#ifndef QT_NO_XRENDER
-            if (rendhd)
-                XRenderSetPictureClipRegion(dpy, rendhd, paintEventClipRegion->handle());
-#endif // QT_NO_XRENDER
-
+	    x11SetClipRegion( dpy, gc, rendhd, *paintEventClipRegion, gc_brush );
         } else {
             XSetClipMask( dpy, gc, None );
             XSetClipMask( dpy, gc_brush, None );
@@ -1679,7 +1633,7 @@ void QPainter::drawPoints( const QPointArray& a, int index, int npoints )
 }
 
 
-/*!
+/*! \obsolete
     Sets the current pen position to \a (x, y)
 
     \sa lineTo(), pos()
@@ -1697,12 +1651,13 @@ void QPainter::moveTo( int x, int y )
             if ( !pdev->cmd( QPaintDevice::PdcMoveTo, this, param ) || !hd )
                 return;
         }
-        map( x, y, &x, &y );
     }
     curPt = QPoint( x, y );
 }
 
-/*!
+/*! \obsolete
+  Use drawLine() instead.
+
     Draws a line from the current pen position to \a (x, y) and sets
     \a (x, y) to be the new current pen position.
 
@@ -1713,6 +1668,8 @@ void QPainter::lineTo( int x, int y )
 {
     if ( !isActive() )
         return;
+    int cx = curPt.x(), cy = curPt.y();
+    curPt = QPoint( x, y );
     if ( testf(ExtDev|VxF|WxF) ) {
         if ( testf(ExtDev) ) {
             QPDevCmdParam param[1];
@@ -1722,10 +1679,10 @@ void QPainter::lineTo( int x, int y )
                 return;
         }
         map( x, y, &x, &y );
+	map( cx,  cy,  &cx,  &cy );
     }
     if ( cpen.style() != NoPen )
-        XDrawLine( dpy, hd, gc, curPt.x(), curPt.y(), x, y );
-    curPt = QPoint( x, y );
+        XDrawLine( dpy, hd, gc, cx, cy, x, y );
 }
 
 /*!
@@ -1739,6 +1696,7 @@ void QPainter::drawLine( int x1, int y1, int x2, int y2 )
 {
     if ( !isActive() )
         return;
+    curPt = QPoint( x2, y2 );
     if ( testf(ExtDev|VxF|WxF) ) {
         if ( testf(ExtDev) ) {
             QPDevCmdParam param[2];
@@ -1753,7 +1711,6 @@ void QPainter::drawLine( int x1, int y1, int x2, int y2 )
     }
     if ( cpen.style() != NoPen )
         XDrawLine( dpy, hd, gc, x1, y1, x2, y2 );
-    curPt = QPoint( x2, y2 );
 }
 
 
@@ -1778,8 +1735,10 @@ void QPainter::drawRect( int x, int y, int w, int h )
                 return;
         }
         if ( txop == TxRotShear ) {             // rotate/shear polygon
-            QPointArray a( QRect(x, y, w, h), TRUE );
-            drawPolyInternal( xForm(a) );
+	    QPointArray pa = xmat.mapToPolygon( QRect(x, y, w, h) );
+	    pa.resize( 5 );
+	    pa.setPoint( 4, pa.point( 0 ) );
+            drawPolyInternal( pa );
             return;
         }
         map( x, y, w, h, &x, &y, &w, &h );
@@ -2066,13 +2025,17 @@ void QPainter::drawEllipse( int x, int y, int w, int h )
         }
         map( x, y, w, h, &x, &y, &w, &h );
     }
-    w--;
-    h--;
     if ( w <= 0 || h <= 0 ) {
         if ( w == 0 || h == 0 )
             return;
         fix_neg_rect( &x, &y, &w, &h );
     }
+    if ( w == 1 && h == 1 ) {
+	XDrawPoint( dpy, hd, (cpen.style() == NoPen)?gc_brush:gc, x, y );
+	return;
+    }
+    w--;
+    h--;
     if ( cbrush.style() != NoBrush ) {          // draw filled ellipse
         XFillArc( dpy, hd, gc_brush, x, y, w, h, 0, 360*64 );
         if ( cpen.style() == NoPen ) {
@@ -2322,7 +2285,7 @@ void QPainter::drawLineSegments( const QPointArray &a, int index, int nlines )
     QPointArray pa = a;
     if ( testf(ExtDev|VxF|WxF) ) {
         if ( testf(ExtDev) ) {
-            if ( nlines != (int)pa.size()/2 ) {
+            if ( 2*nlines != (int)pa.size() ) {
                 pa = QPointArray( nlines*2 );
                 for ( int i=0; i<nlines*2; i++ )
                     pa.setPoint( i, a.point(index+i) );
@@ -2381,7 +2344,7 @@ void QPainter::drawPolyline( const QPointArray &a, int index, int npoints )
                 return;
         }
         if ( txop != TxNone ) {
-            pa = xForm( a, index, npoints );
+            pa = xForm( pa, index, npoints );
             if ( pa.size() != a.size() ) {
                 index   = 0;
                 npoints = pa.size();
@@ -2661,9 +2624,8 @@ void QPainter::drawPixmap( int x, int y, const QPixmap &pixmap,
             XSetTSOrigin( dpy, gc, 0, 0 );
             XSetFillStyle( dpy, gc, FillSolid );
             if ( !selfmask ) {
-                XSetClipOrigin( dpy, gc, 0, 0 );
                 if ( pdev == paintEventDevice && paintEventClipRegion ) {
-                    XSetRegion( dpy, gc, paintEventClipRegion->handle() );
+                    x11SetClipRegion( dpy, gc, rendhd, *paintEventClipRegion );
 		} else {
                     XSetClipMask( dpy, gc, None );
 		}
@@ -2694,15 +2656,15 @@ void QPainter::drawPixmap( int x, int y, const QPixmap &pixmap,
         XFillRectangle( dpy, comb->handle(), cgc, 0, 0, sw, sh );
         XSetBackground( dpy, cgc, 0 );
         XSetForeground( dpy, cgc, 1 );
-        XSetRegion( dpy, cgc, rgn.handle() );
-        XSetClipOrigin( dpy, cgc, -x, -y );
+	int num;
+	XRectangle *rects = (XRectangle *)qt_getClipRects( rgn, num );
+        XSetClipRectangles( dpy, cgc, -x, -y, rects, num, Unsorted );
         XSetFillStyle( dpy, cgc, FillOpaqueStippled );
         XSetStipple( dpy, cgc, mask->handle() );
         XSetTSOrigin( dpy, cgc, -sx, -sy );
         XFillRectangle( dpy, comb->handle(), cgc, 0, 0, sw, sh );
         XSetTSOrigin( dpy, cgc, 0, 0 );         // restore cgc
         XSetFillStyle( dpy, cgc, FillSolid );
-        XSetClipOrigin( dpy, cgc, 0, 0 );
         XSetClipMask( dpy, cgc, None );
         mask = comb;                            // it's deleted below
 
@@ -2711,7 +2673,7 @@ void QPainter::drawPixmap( int x, int y, const QPixmap &pixmap,
     }
 
     if ( mono ) {
-        XSetBackground( dpy, gc, bg_col.pixel() );
+        XSetBackground( dpy, gc, bg_col.pixel(scrn) );
         XSetFillStyle( dpy, gc, FillOpaqueStippled );
         XSetStipple( dpy, gc, pixmap.handle() );
         XSetTSOrigin( dpy, gc, x-sx, y-sy );
@@ -2862,7 +2824,7 @@ void QPainter::drawTiledPixmap( int x, int y, int w, int h,
 	    }
 	    return;
 	}
-#endif
+#endif // QT_NO_XRENDER
 
 	XSetTile( dpy, gc, pixmap.handle() );
 	XSetFillStyle( dpy, gc, FillTiled );
@@ -2903,7 +2865,7 @@ void QPainter::drawTiledPixmap( int x, int y, int w, int h,
 #endif
 }
 
-
+#if 0
 //
 // Generate a string that describes a transformed bitmap. This string is used
 // to insert and find bitmaps in the global pixmap cache.
@@ -2945,7 +2907,46 @@ static void ins_text_bitmap( const QString &key, QBitmap *bm )
     if ( !QPixmapCache::insert(key, bm) )       // cannot insert pixmap
         delete bm;
 }
+#endif
 
+void qt_draw_transformed_rect( QPainter *p,  int x, int y, int w,  int h, bool fill )
+{
+    XPoint points[5];
+    int xp = x,  yp = y;
+    p->map( xp, yp, &xp, &yp );
+    points[0].x = xp;
+    points[0].y = yp;
+    xp = x + w; yp = y;
+    p->map( xp, yp, &xp, &yp );
+    points[1].x = xp;
+    points[1].y = yp;
+    xp = x + w; yp = y + h;
+    p->map( xp, yp, &xp, &yp );
+    points[2].x = xp;
+    points[2].y = yp;
+    xp = x; yp = y + h;
+    p->map( xp, yp, &xp, &yp );
+    points[3].x = xp;
+    points[3].y = yp;
+    points[4] = points[0];
+
+    if ( fill )
+	XFillPolygon( p->dpy, p->hd, p->gc, points, 4, Convex, CoordModeOrigin );
+    else
+	XDrawLines( p->dpy, p->hd, p->gc, points, 5, CoordModeOrigin );
+}
+
+void qt_draw_background( QPainter *p, int x, int y, int w,  int h )
+{
+    if (p->testf(QPainter::ExtDev)) {
+	if (p->pdev->devType() == QInternal::Printer)
+	    p->fillRect(x, y, w, h, p->bg_col);
+	return;
+    }
+    XSetForeground( p->dpy, p->gc, p->bg_col.pixel(p->scrn) );
+    qt_draw_transformed_rect( p, x, y, w, h, TRUE);
+    XSetForeground( p->dpy, p->gc, p->cpen.color().pixel(p->scrn) );
+}
 
 /*!
     Draws at most \a len characters of the string \a str at position
@@ -2956,14 +2957,9 @@ static void ins_text_bitmap( const QString &key, QBitmap *bm )
 */
 void QPainter::drawText( int x, int y, const QString &str, int len, QPainter::TextDirection dir )
 {
-    if (len < 0)
-        len = str.length();
-    if (len == 0)
-        return;
     drawText( x, y, str, 0, len, dir );
 }
 
-
 /*!
   Draws at most \a len characters starting at position \a pos from the
   string \a str to position \a (x, y).
@@ -2975,157 +2971,151 @@ void QPainter::drawText( int x, int y, const QString &str, int pos, int len, QPa
 {
     if ( !isActive() )
         return;
-    if ( len == 0 || pos >= (int)str.length() ) // empty string
+    if (len < 0)
+        len = str.length() - pos;
+    if ( len <= 0 || pos >= (int)str.length() ) // empty string
         return;
     if ( pos + len > (int)str.length() )
         len = str.length() - pos;
 
-    QFontMetrics fm( fontMetrics() );
-    QString shaped = QComplexText::shapedString( str,  pos, len, dir, &fm );
-    len = shaped.length();
-
-    if ( testf(DirtyFont|ExtDev|VxF|WxF) ) {
-        if ( testf(DirtyFont) ) {
-            updateFont();
-        }
-
-        if ( testf(ExtDev) ) {
-            QPDevCmdParam param[3];
-            QFontPrivate::TextRun *cache = new QFontPrivate::TextRun();
-            pfont->d->textWidth( shaped, 0, len, cache ); // create cache
-            bool retval = FALSE;
-	    QFontPrivate::TextRun *runs = cache;
-            while ( cache ) {
-                QPoint p( x + cache->xoff, y + cache->yoff );
-                QString s =
-                    QConstString( cache->string, cache->length ).string();
-                param[0].point = &p;
-                param[1].str = &s;
-                param[2].ival = cache->script;
-                retval = pdev->cmd(QPaintDevice::PdcDrawText2, this, param);
-                cache = cache->next;
-            }
-            delete runs;
-            if ( !retval || !hd )
-                return;
-        }
-
-        if ( txop >= TxScale ) {
-	    QConstString csubstr( str.unicode()+pos, len );
-	    QString substr = csubstr.string();
-            QFontMetrics fm = fontMetrics();
-            QRect bbox = fm.boundingRect( substr, len );
-            int w=bbox.width(), h=bbox.height();
-            int aw, ah;
-            int tx=-bbox.x(),  ty=-bbox.y();    // text position
-            QWMatrix mat1( m11(), m12(), m21(), m22(), dx(),  dy() );
-            QFont dfont( cfont );
-	    float det = QABS(m11()*m22() - m12()*m21());
-	    if ( det != 1 ) {
-		// det == 1. means we have only a rotation, so no need to change font size.
-		float pixSize = cfont.pixelSize();
-		if ( pixSize == -1 )
-		    pixSize = cfont.deciPointSize() * QPaintDeviceMetrics( pdev ).logicalDpiY() / 720;
-		int newSize = (int) (sqrt( det ) * pixSize) - 1;
-		newSize = QMAX( 6, QMIN( newSize, 72 )); // empirical values
-		dfont.setPixelSize( newSize );
-	    }
-	    QFontMetrics fm2( dfont );
-	    QRect abbox = fm2.boundingRect( substr, len );
-	    aw = abbox.width();
-	    ah = abbox.height();
-	    tx = -abbox.x();
-	    ty = -abbox.y();        // text position - off-by-one?
-	    if ( aw == 0 || ah == 0 )
-		return;
-	    double rx = (double)w / (double)aw;
-	    double ry = (double)h / (double)ah;
-            QWMatrix mat2 = QPixmap::trueMatrix( QWMatrix( rx, 0, 0, ry, 0, 0 )*mat1, aw, ah );
-            QString bm_key = gen_text_bitmap_key( mat2, dfont, str, pos, len );
-            QBitmap *wx_bm = get_text_bitmap( bm_key );
-            bool create_new_bm = wx_bm == 0;
-            if ( create_new_bm ) { 	        // no such cached bitmap
-                QBitmap bm( aw, ah, TRUE );     // create bitmap
-                QPainter paint;
-                paint.begin( &bm );             // draw text in bitmap
-                paint.setFont( dfont );
-                paint.drawText( tx, ty, str, pos, len, dir );
-                paint.end();
-                wx_bm = new QBitmap( bm.xForm(mat2) ); // transform bitmap
-                if ( wx_bm->isNull() ) {
-                    delete wx_bm;               // nothing to draw
-                    return;
-                }
-            }
-            if ( bg_mode == OpaqueMode ) {      // opaque fill
-                int fx = x;
-                int fy = y - fm.ascent();
-                int fw = fm.width(str, len);
-                int fh = fm.ascent() + fm.descent();
-                int m, n;
-                QPointArray a(5);
-                mat1.map( fx,    fy,    &m, &n );  a.setPoint( 0, m, n );
-                a.setPoint( 4, m, n );
-                mat1.map( fx+fw, fy,    &m, &n );  a.setPoint( 1, m, n );
-                mat1.map( fx+fw, fy+fh, &m, &n );  a.setPoint( 2, m, n );
-                mat1.map( fx,    fy+fh, &m, &n );  a.setPoint( 3, m, n );
-                QBrush oldBrush = cbrush;
-                setBrush( backgroundColor() );
-                updateBrush();
-                XFillPolygon( dpy, hd, gc_brush, (XPoint*)a.shortPoints(), 4,
-                              Nonconvex, CoordModeOrigin );
-                XDrawLines( dpy, hd, gc_brush, (XPoint*)a.shortPoints(), 5,
-                            CoordModeOrigin );
-                setBrush( oldBrush );
-            }
-            double fx=x, fy=y, nfx, nfy;
-            mat1.map( fx,fy, &nfx,&nfy );
-            double tfx=tx, tfy=ty, dx, dy;
-            mat2.map( tfx, tfy, &dx, &dy );     // compute position of bitmap
-            x = qRound(nfx-dx);
-            y = qRound(nfy-dy);
-            XSetFillStyle( dpy, gc, FillStippled );
-            XSetStipple( dpy, gc, wx_bm->handle() );
-            XSetTSOrigin( dpy, gc, x, y );
-            XFillRectangle( dpy, hd, gc, x, y,wx_bm->width(),wx_bm->height() );
-            XSetTSOrigin( dpy, gc, 0, 0 );
-            XSetFillStyle( dpy, gc, FillSolid );
-	    if ( create_new_bm )
-                ins_text_bitmap( bm_key, wx_bm );
-            return;
-        }
-        if ( txop == TxTranslate )
-            map( x, y, &x, &y );
+    if ( testf(DirtyFont) ) {
+	updateFont();
     }
 
-    QFontPrivate::TextRun *cache = new QFontPrivate::TextRun();
-    int width = cfont.d->textWidth( shaped, 0, len, cache );
-    cfont.d->drawText( dpy, scrn, hd, rendhd, gc, cpen.color(), (Qt::BGMode) bg_mode,
-                       bg_col, x, y, cache, pdev->metric( QPaintDeviceMetrics::PdmWidth ) );
-    delete cache;
-
-    if ( cfont.underline() || cfont.strikeOut() ) {
-        QFontMetrics fm = fontMetrics();
-        int lw = fm.lineWidth();
-
-        // draw underline effect
-        if ( cfont.underline() ) {
-            XFillRectangle( dpy, hd, gc, x, y+fm.underlinePos(), width, lw );
-        }
-
-        // draw strikeout effect
-        if ( cfont.strikeOut() ) {
-            XFillRectangle( dpy, hd, gc, x, y-fm.strikeOutPos(), width, lw );
-        }
+    if ( testf(ExtDev) && pdev->devType() != QInternal::Printer ) {
+	QPDevCmdParam param[3];
+	QPoint p(x, y);
+	QString string = str.mid( pos,  len );
+	param[0].point = &p;
+	param[1].str = &string;
+	param[2].ival = QFont::Latin;
+	if ( !pdev->cmd(QPaintDevice::PdcDrawText2, this, param) || !hd )
+	    return;
     }
+
+    bool simple = str.simpleText();
+    // we can't take the complete string here as we would otherwise
+    // get quadratic behaviour when drawing long strings in parts.
+    // we do however need some chars around the part we paint to get arabic shaping correct.
+    // ### maybe possible to remove after cursor restrictions work in QRT
+    int start;
+    int end;
+    if ( simple ) {
+	start = pos;
+	end = pos+len;
+    } else {
+	start = QMAX( 0,  pos - 8 );
+	end = QMIN( (int)str.length(), pos + len + 8 );
+    }
+    QConstString cstr( str.unicode() + start, end - start );
+    pos -= start;
+
+    QTextEngine engine( cstr.string(), pfont ? pfont->d : cfont.d );
+    QTextLayout layout( &engine );
+
+    // this is actually what beginLayout does. Inlined here, so we can
+    // avoid the bidi algorithm if we don't need it.
+    engine.itemize( simple ? QTextEngine::NoBidi|QTextEngine::SingleLine : QTextEngine::Full|QTextEngine::SingleLine );
+    engine.currentItem = 0;
+    engine.firstItemInLine = -1;
+
+    if ( !simple ) {
+	layout.setBoundary( pos );
+	layout.setBoundary( pos + len );
+    }
+
+    if ( dir != Auto ) {
+	int level = dir == RTL ? 1 : 0;
+	for ( int i = engine.items.size(); i >= 0; i-- )
+	    engine.items[i].analysis.bidiLevel = level;
+    }
+
+    // small hack to force skipping of unneeded items
+    start = 0;
+    while ( engine.items[start].position < pos )
+	++start;
+    engine.currentItem = start;
+    layout.beginLine( 0xfffffff );
+    end = start;
+    while ( !layout.atEnd() && layout.currentItem().from() < pos + len ) {
+	layout.addCurrentItem();
+	end++;
+    }
+    QFontMetrics fm(fontMetrics());
+    int ascent = fm.ascent(), descent = fm.descent();
+    int left, right;
+    layout.endLine( 0, 0, Qt::SingleLine|Qt::AlignLeft, &ascent, &descent, &left, &right );
+
+    // do _not_ call endLayout() here, as it would clean up the shaped items and we would do shaping another time
+    // for painting.
+
+    int textFlags = 0;
+    if ( cfont.d->underline ) textFlags |= Qt::Underline;
+    if ( cfont.d->overline ) textFlags |= Qt::Overline;
+    if ( cfont.d->strikeOut ) textFlags |= Qt::StrikeOut;
+
+    if ( bg_mode == OpaqueMode )
+	qt_draw_background( this, x, y-ascent, right-left, ascent+descent+1);
+
+    for ( int i = start; i < end; i++ ) {
+	QTextItem ti;
+	ti.item = i;
+	ti.engine = &engine;
+
+	drawTextItem( x, y - ascent, ti, textFlags );
+    }
+    layout.d = 0;
 }
 
+
+/*! \internal
+    Draws the text item \a ti at position \a (x, y ).
+
+    This method ignores the painters background mode and
+    color. drawText and qt_format_text have to do it themselves, as
+    only they know the extents of the complete string.
+
+    It ignores the font set on the painter as the text item has one of its own.
+
+    The underline and strikeout parameters of the text items font are
+    ignored aswell. You'll need to pass in the correct flags to get
+    underlining and strikeout.
+*/
+void QPainter::drawTextItem( int x,  int y, const QTextItem &ti, int textFlags )
+{
+    if ( testf(ExtDev) ) {
+	QPDevCmdParam param[2];
+	QPoint p(x, y);
+	param[0].point = &p;
+	param[1].textItem = &ti;
+	bool retval = pdev->cmd(QPaintDevice::PdcDrawTextItem, this, param);
+	if ( !retval || !hd )
+	    return;
+    }
+
+    QTextEngine *engine = ti.engine;
+    QScriptItem *si = &engine->items[ti.item];
+
+    engine->shape( ti.item );
+    QFontEngine *fe = si->fontEngine;
+    assert( fe );
+
+    x += si->x;
+    y += si->y;
+
+    fe->draw( this, x,  y, engine, si, textFlags );
+}
+
+#if QT_VERSION >= 0x040000
+#error "remove current position and associated methods"
+#endif
 /*!
+  \obsolete
     Returns the current position of the  pen.
 
     \sa moveTo()
  */
 QPoint QPainter::pos() const
 {
-    return xFormDev( curPt );
+    return curPt;
 }
