@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id:  qt/qpalette.cpp   3.0.2   edited Oct 12 12:18 $
+** $Id: qt/qpalette.cpp   3.2.3   edited Jun 13 13:15 $
 **
 ** Implementation of QColorGroup and QPalette classes
 **
@@ -46,157 +46,183 @@
  *****************************************************************************/
 
 /*!
-  \class QColorGroup qpalette.h
-  \brief The QColorGroup class contains a group of widget colors.
+    \class QColorGroup qpalette.h
+    \brief The QColorGroup class contains a group of widget colors.
 
-  \ingroup appearance
-  \ingroup graphics
-  \ingroup images
+    \ingroup appearance
+    \ingroup graphics
+    \ingroup images
 
-  A color group contains a group of colors used by widgets for drawing
-  themselves. We recommend that widgets use color group roles such as
-  "foreground" and "base" rather than literal colors like "red" or
-  "turqoise". The color roles are enumerated and defined in the
-  ColorRole documentation.
+    A color group contains a group of colors used by widgets for
+    drawing themselves. We recommend that widgets use color group
+    roles such as "foreground" and "base" rather than literal colors
+    like "red" or "turquoise". The color roles are enumerated and
+    defined in the \l ColorRole documentation.
 
-  The most common use of QColorGroup is like this:
+    The most common use of QColorGroup is like this:
 
-  \code
-    QPainter p;
-    ...
-    p.setPen( colorGroup().foreground() );
-    p.drawLine( ... )
-  \endcode
+    \code
+	QPainter p;
+	...
+	p.setPen( colorGroup().foreground() );
+	p.drawLine( ... )
+    \endcode
 
-  See the \l ColorRole documentation below for more details on roles.
+    It is also possible to modify color groups or create new color
+    groups from scratch.
 
-  It is also possible to modify color groups or create new color groups
-  from scratch.
+    The color group class can be created using three different
+    constructors or by modifying one supplied by Qt. The default
+    constructor creates an all-black color group, which can then be
+    modified using set functions; there's also a constructor for
+    specifying all the color group colors. And there is also a copy
+    constructor.
 
-  The color group class can be created using three different
-  constructors or by modifying one supplied by the Qt.  The
-  default constructor creates an all-black color group, which can then
-  be modified using set functions.  There are two functions that take
-  long lists of arguments (slightly different lists - beware!).  And
-  there is the copy constructor.
+    We strongly recommend using a system-supplied color group and
+    modifying that as necessary.
 
-  We strongly recommend using a system-supplied color group and
-  modifying that as necessary.
+    You modify a color group by calling the access functions
+    setColor() and setBrush(), depending on whether you want a pure
+    color or a pixmap pattern.
 
-  You modify a color group by calling the access functions setColor()
-  and setBrush(), depending on whether you want a pure color or a
-  pixmap pattern.
+    There are also corresponding color() and brush() getters, and a
+    commonly used convenience function to get each ColorRole:
+    background(), foreground(), base(), etc.
 
-  There are also corresponding color() and brush() getters, and a
-  commonly used convenience function to get each ColorRole:
-  background(), foreground(), base(), etc.
-
-  \sa QColor QPalette QWidget::colorGroup()
+    \sa QColor QPalette QWidget::colorGroup()
 */
 
 
-/*! \enum QColorGroup::ColorRole
-
-  The ColorRole enum defines the different symbolic color roles used
-  in current GUIs.  The central roles are as follow:
-
-  \value Background  general background color.
-
-  \value Foreground  general foreground color.
-
-  \value Base  used as background color for text entry widgets, for example;
-  usually white or another light color.
-
-  \value Text  the foreground color used with \c Base. Usually this
-  is the same as the \c Foreground, in which case it must provide good
-  contrast with \c Background and \c Base.
-
-  \value Button  general button background color in which buttons need a
-  background different from \c Background, as in the Macintosh style.
-
-  \value ButtonText  a foreground color used with the \c Button color.
-
-  There are some color roles used mostly for 3D bevel and shadow
-  effects:
-
-  \value Light  lighter than \c Button color.
-
-  \value Midlight  between \c Button and \c Light.
-
-  \value Dark  darker than \c Button.
-
-  \value Mid  between \c Button and \c Dark.
-
-  \value Shadow  a very dark color.
-
-  All of these are normally derived from \c Background and used
-  in ways that depend on that relationship.  For example, buttons
-  depend on it to make the bevels look attractive, and Motif scroll bars
-  depend on \c Mid to be slightly different from \c Background.
-
-  Selected (marked) items have two roles:
-
-  \value Highlight   a color to indicate a selected or highlighted item.
-
-  \value HighlightedText  a text color that contrasts with \c Highlight.
-
-  Finally, there is a special role for text that needs to be
-  drawn where \c Text or \c Foreground would give poor contrast,
-  such as on pressed push buttons:
-
-  \value BrightText a text color that is very different from \c
-  Foreground and contrasts well with e.g. \c Dark.
-
-  \value Link a text color used for unvisited hyperlinks.
-
-  \value LinkVisited a text color used for already visited hyperlinks.
-
-  \value NColorRoles Internal.
-
-  Note that text colors can be used for things other than just words;
-  text colors are \e usually used for text, but it's quite common to use
-  the text color roles for lines, icons, etc.
-
-  This image shows most of the color roles in use:
-  <img src="palette.png" width="603" height="166" alt="">
-*/
-
-
-
 /*!
-  Constructs a color group with all colors set to black.
+    \enum QColorGroup::ColorRole
+
+    The ColorRole enum defines the different symbolic color roles used
+    in current GUIs.
+
+    The central roles are:
+
+    \value Background  general background color.
+
+    \value Foreground  general foreground color.
+
+    \value Base  used as background color for text entry widgets, for example;
+    usually white or another light color.
+
+    \value Text  the foreground color used with \c Base. Usually this
+    is the same as the \c Foreground, in which case it must provide good
+    contrast with \c Background and \c Base.
+
+    \value Button  general button background color in which buttons need a
+    background different from \c Background, as in the Macintosh style.
+
+    \value ButtonText  a foreground color used with the \c Button color.
+
+    There are some color roles used mostly for 3D bevel and shadow
+    effects:
+
+    \value Light  lighter than \c Button color.
+
+    \value Midlight  between \c Button and \c Light.
+
+    \value Dark  darker than \c Button.
+
+    \value Mid  between \c Button and \c Dark.
+
+    \value Shadow  a very dark color.
+    By default, the shadow color is \c Qt::black.
+
+    All of these are normally derived from \c Background and used in
+    ways that depend on that relationship. For example, buttons depend
+    on it to make the bevels look attractive, and Motif scroll bars
+    depend on \c Mid to be slightly different from \c Background.
+
+    Selected (marked) items have two roles:
+
+    \value Highlight   a color to indicate a selected item or the
+    current item. By default, the highlight color is \c Qt::darkBlue.
+
+    \value HighlightedText  a text color that contrasts with \c Highlight.
+    By default, the highlighted text color is \c Qt::white.
+
+    Finally, there is a special role for text that needs to be
+    drawn where \c Text or \c Foreground would give poor contrast,
+    such as on pressed push buttons:
+
+    \value BrightText a text color that is very different from \c
+    Foreground and contrasts well with e.g. \c Dark.
+
+    \value Link a text color used for unvisited hyperlinks.
+    By default, the link color is \c Qt::blue.
+
+    \value LinkVisited a text color used for already visited hyperlinks.
+    By default, the linkvisited color is \c Qt::magenta.
+
+    \value NColorRoles Internal.
+
+    Note that text colors can be used for things other than just
+    words; text colors are \e usually used for text, but it's quite
+    common to use the text color roles for lines, icons, etc.
+
+    This image shows most of the color roles in use:
+    \img palette.png Color Roles
+*/
+
+
+class QColorGroupPrivate : public QShared
+{
+public:
+    QBrush br[QColorGroup::NColorRoles];
+    QColorGroupPrivate* detach() {
+	if ( count > 1 ) {
+	    deref();
+	    QColorGroupPrivate* d = new QColorGroupPrivate;
+	    for (int i=0; i<QColorGroup::NColorRoles; i++)
+		d->br[i] = br[i];
+	    return d;
+	}
+	return this;
+    }
+};
+
+/*!
+    Constructs a color group with all colors set to black.
 */
 
 QColorGroup::QColorGroup()
 {
-    br = new QBrush[(uint)NColorRoles];	// all colors become black
-
-    // The d pointer may allow sharing in the future.  The br pointer
-    // then will be a redundant pointer that makes possible the inlines
-    // in the header file.  Don't forget to add delete d in the destructor.
-    // QPalette, the main QColorGroup user, is already shared though,
-    // so perhaps not much is to be gained.
-    d = 0;
+    static QColorGroupPrivate* defColorGroupData = 0;
+    if ( !defColorGroupData ) {
+	static QSharedCleanupHandler<QColorGroupPrivate> defColorGroupCleanup;
+	defColorGroupData = new QColorGroupPrivate;
+	defColorGroupCleanup.set( &defColorGroupData );
+    }
+    d = defColorGroupData;
+    br = d->br;
+    d->ref();
 }
 
 /*!
-  Constructs a color group that is an independent copy of \a other.
+    Constructs a color group that is an independent copy of \a other.
 */
 QColorGroup::QColorGroup( const QColorGroup& other )
 {
-    br = new QBrush[(uint)NColorRoles];
-    for (int i=0; i<NColorRoles; i++)
-	br[i] = other.br[i];
-    d = 0;
+    d = other.d;
+    d->ref();
+    br = d->br;
 }
 
 /*!
-  Copies the colors of \a other to this color group.
+    Copies the colors of \a other to this color group.
 */
 QColorGroup& QColorGroup::operator =(const QColorGroup& other)
 {
-    for (int i=0; i<NColorRoles; i++)
-	br[i] = other.br[i];
+    if ( d != other.d ) {
+	if ( d->deref() )
+	    delete d;
+	d = other.d;
+	br = d->br;
+	d->ref();
+    }
     return *this;
 }
 
@@ -207,14 +233,11 @@ static QColor qt_mix_colors( QColor a, QColor b)
 
 
 /*!
-Constructs a color group. You can pass either brushes, pixmaps or
-plain colors for \a foreground, \a button, \a light, \a dark,
-\a mid, \a text, \a bright_text, \a base and \a background.
+    Constructs a color group. You can pass either brushes, pixmaps or
+    plain colors for \a foreground, \a button, \a light, \a dark, \a
+    mid, \a text, \a bright_text, \a base and \a background.
 
-This constructor can be very handy sometimes, but don't overuse it:
-such long lists of arguments are rather error-prone.
-
-\sa QBrush
+    \sa QBrush
 */
  QColorGroup::QColorGroup( const QBrush &foreground, const QBrush &button,
 			   const QBrush &light, const QBrush &dark,
@@ -222,7 +245,8 @@ such long lists of arguments are rather error-prone.
 			   const QBrush &bright_text, const QBrush &base,
 			   const QBrush &background)
 {
-    br = new QBrush[(uint)NColorRoles];
+    d = new QColorGroupPrivate;
+    br = d->br;
     br[Foreground]      = foreground;
     br[Button]		= button;
     br[Light]		= light;
@@ -253,7 +277,8 @@ QColorGroup::QColorGroup( const QColor &foreground, const QColor &background,
 			  const QColor &mid,
 			  const QColor &text, const QColor &base )
 {
-    br = new QBrush[(uint)NColorRoles];
+    d = new QColorGroupPrivate;
+    br = d->br;
     br[Foreground]      = QBrush(foreground);
     br[Button]          = QBrush(background);
     br[Light]           = QBrush(light);
@@ -273,17 +298,19 @@ QColorGroup::QColorGroup( const QColor &foreground, const QColor &background,
 }
 
 /*!
-  Destroys the color group.
+    Destroys the color group.
 */
 
 QColorGroup::~QColorGroup()
 {
-    delete [] br;
+    if ( d->deref() )
+	delete d;
 }
 
 /*!
-  Returns the color that has been set for color role \a r.
-  \sa brush() ColorRole
+    Returns the color that has been set for color role \a r.
+
+    \sa brush() ColorRole
  */
 const QColor &QColorGroup::color( ColorRole r ) const
 {
@@ -291,9 +318,9 @@ const QColor &QColorGroup::color( ColorRole r ) const
 }
 
 /*!
-  Returns the brush that has been set for color role \a r.
+    Returns the brush that has been set for color role \a r.
 
-  \sa color() setBrush() ColorRole
+    \sa color() setBrush() ColorRole
 */
 const QBrush &QColorGroup::brush( ColorRole r ) const
 {
@@ -301,9 +328,9 @@ const QBrush &QColorGroup::brush( ColorRole r ) const
 }
 
 /*!
-  Sets the brush used for color role \a r to a solid color \a c.
+    Sets the brush used for color role \a r to a solid color \a c.
 
-  \sa brush() setColor() ColorRole
+    \sa brush() setColor() ColorRole
 */
 void QColorGroup::setColor( ColorRole r, const QColor &c )
 {
@@ -311,143 +338,166 @@ void QColorGroup::setColor( ColorRole r, const QColor &c )
 }
 
 /*!
-  Sets the brush used for color role \a r to \a b.
+    Sets the brush used for color role \a r to \a b.
 
-  \sa brush() setColor() ColorRole
+    \sa brush() setColor() ColorRole
 */
 void QColorGroup::setBrush( ColorRole r, const QBrush &b )
 {
+    d = d->detach();
+    br = d->br;
     br[r] = b;
 }
 
 
 /*!
-  \fn const QColor & QColorGroup::foreground() const
+    \fn const QColor & QColorGroup::foreground() const
 
-  Returns the foreground color of the color group.
+    Returns the foreground color of the color group.
 
-  \sa ColorRole
+    \sa ColorRole
 */
 
 /*!
-  \fn const QColor & QColorGroup::button() const
-  Returns the button color of the color group.
+    \fn const QColor & QColorGroup::button() const
 
-  \sa ColorRole
+    Returns the button color of the color group.
+
+    \sa ColorRole
 */
 
 /*!
-  \fn const QColor & QColorGroup::light() const
-  Returns the light color of the color group.
+    \fn const QColor & QColorGroup::light() const
 
-  \sa ColorRole
+    Returns the light color of the color group.
+
+    \sa ColorRole
 */
 
 /*!
-  \fn const QColor& QColorGroup::midlight() const
-  Returns the midlight color of the color group.
+    \fn const QColor& QColorGroup::midlight() const
 
-  \sa ColorRole
+    Returns the midlight color of the color group.
+
+    \sa ColorRole
 */
 
 /*!
-  \fn const QColor & QColorGroup::dark() const
-  Returns the dark color of the color group.
+    \fn const QColor & QColorGroup::dark() const
 
-  \sa ColorRole
+    Returns the dark color of the color group.
+
+    \sa ColorRole
 */
 
 /*!
-  \fn const QColor & QColorGroup::mid() const
-  Returns the mid color of the color group.
+    \fn const QColor & QColorGroup::mid() const
 
-  \sa ColorRole
+    Returns the mid color of the color group.
+
+    \sa ColorRole
 */
 
 /*!
-  \fn const QColor & QColorGroup::text() const
-  Returns the text foreground color of the color group.
+    \fn const QColor & QColorGroup::text() const
 
-  \sa ColorRole
+    Returns the text foreground color of the color group.
+
+    \sa ColorRole
 */
 
 /*!
-  \fn const QColor & QColorGroup::brightText() const
-  Returns the bright text foreground color of the color group.
+    \fn const QColor & QColorGroup::brightText() const
 
-  \sa ColorRole
+    Returns the bright text foreground color of the color group.
+
+    \sa ColorRole
 */
 
 /*!
-  \fn const QColor & QColorGroup::buttonText() const
-  Returns the button text foreground color of the color group.
+    \fn const QColor & QColorGroup::buttonText() const
 
-  \sa ColorRole
+    Returns the button text foreground color of the color group.
+
+    \sa ColorRole
 */
 
 /*!
-  \fn const QColor & QColorGroup::base() const
-  Returns the base color of the color group.
+    \fn const QColor & QColorGroup::base() const
 
-  \sa ColorRole
+    Returns the base color of the color group.
+
+    \sa ColorRole
 */
 
 /*!
-  \fn const QColor & QColorGroup::background() const
-  Returns the background color of the color group.
+    \fn const QColor & QColorGroup::background() const
 
-  \sa ColorRole
+    Returns the background color of the color group.
+
+    \sa ColorRole
 */
 
 /*!
-  \fn const QColor & QColorGroup::shadow() const
-  Returns the shadow color of the color group.
+    \fn const QColor & QColorGroup::shadow() const
 
-  \sa ColorRole
+    Returns the shadow color of the color group.
+
+    \sa ColorRole
 */
 
 /*!
-  \fn const QColor & QColorGroup::highlight() const
-  Returns the highlight color of the color group.
+    \fn const QColor & QColorGroup::highlight() const
 
-  \sa ColorRole
+    Returns the highlight color of the color group.
+
+    \sa ColorRole
 */
 
 /*!
-  \fn const QColor & QColorGroup::highlightedText() const
-  Returns the highlighted text color of the color group.
+    \fn const QColor & QColorGroup::highlightedText() const
 
-  \sa ColorRole
+    Returns the highlighted text color of the color group.
+
+    \sa ColorRole
 */
 
 /*!
-  \fn const QColor & QColorGroup::link() const
-  Returns the unvisited link text color of the color group.
+    \fn const QColor & QColorGroup::link() const
 
-  \sa ColorRole
+    Returns the unvisited link text color of the color group.
+
+    \sa ColorRole
 */
 
 /*!
-  \fn const QColor & QColorGroup::linkVisited() const
-  Returns the visited link text color of the color group.
+    \fn const QColor & QColorGroup::linkVisited() const
 
-  \sa ColorRole
+    Returns the visited link text color of the color group.
+
+    \sa ColorRole
 */
 
 /*!
-  \fn bool QColorGroup::operator!=( const QColorGroup &g ) const
-  Returns TRUE if this color group is different from \a g; otherwise
-  returns  FALSE.
-  \sa operator!=()
+    \fn bool QColorGroup::operator!=( const QColorGroup &g ) const
+
+    Returns TRUE if this color group is different from \a g; otherwise
+    returns  FALSE.
+
+    \sa operator!=()
 */
 
 /*!
-  Returns TRUE if this color group is equal to \a g; otherwise returns FALSE.
-  \sa operator==()
+    Returns TRUE if this color group is equal to \a g; otherwise
+    returns FALSE.
+
+    \sa operator==()
 */
 
 bool QColorGroup::operator==( const QColorGroup &g ) const
 {
+    if ( d == g.d )
+	return TRUE;
     for( int r = 0 ; r < NColorRoles ; r++ )
 	if ( br[r] != g.br[r] )
 	    return FALSE;
@@ -460,54 +510,57 @@ bool QColorGroup::operator==( const QColorGroup &g ) const
  *****************************************************************************/
 
 /*!
-  \class QPalette qpalette.h
+    \class QPalette qpalette.h
 
-  \brief The QPalette class contains color groups for each widget state.
+    \brief The QPalette class contains color groups for each widget state.
 
-  \ingroup appearance
-  \ingroup shared
-  \ingroup graphics
-  \ingroup images
-  \mainclass
+    \ingroup appearance
+    \ingroup shared
+    \ingroup graphics
+    \ingroup images
+    \mainclass
 
-  A palette consists of three color groups: \e active, \e disabled,
-  and \e inactive.  All widgets contain a palette, and all widgets in
-  Qt use their palette to draw themselves.  This makes the user
-  interface easily configurable and easier to keep consistent.
+    A palette consists of three color groups: \e active, \e disabled,
+    and \e inactive. All widgets contain a palette, and all widgets in
+    Qt use their palette to draw themselves. This makes the user
+    interface easily configurable and easier to keep consistent.
 
-  If you create a new widget we strongly recommend that you use the
-  colors in the palette rather than hard-coding specific colors.
+    If you create a new widget we strongly recommend that you use the
+    colors in the palette rather than hard-coding specific colors.
 
-  The color groups:
-  \list
-  \i The active() group is used for the window that has keyboard focus.
-  \i The inactive() group is used for other windows.
-  \i The disabled() group is used for widgets (not windows) that are
-  disabled for some reason.
-  \endlist
+    The color groups:
+    \list
+    \i The active() group is used for the window that has keyboard focus.
+    \i The inactive() group is used for other windows.
+    \i The disabled() group is used for widgets (not windows) that are
+    disabled for some reason.
+    \endlist
 
-  Both active and inactive windows can contain disabled
-  widgets.  (Disabled widgets are often called \e inaccessible or \e
-  grayed \e out.)
+    Both active and inactive windows can contain disabled widgets.
+    (Disabled widgets are often called \e inaccessible or \e{grayed
+    out}.)
 
-  In Motif style, active() and inactive() look precisely the same.  In
-  Windows 2000 style and Macintosh Platinum style, the two styles look
-  slightly different.
+    In Motif style, active() and inactive() look the same. In Windows
+    2000 style and Macintosh Platinum style, the two styles look
+    slightly different.
 
-  There are setActive(), setInactive(), and setDisabled() functions to
-  modify the palette.  (Qt also supports a normal() group; this is an
-  obsolete alias for active(), supported for backwards compatibility.)
+    There are setActive(), setInactive(), and setDisabled() functions
+    to modify the palette. (Qt also supports a normal() group; this is
+    an obsolete alias for active(), supported for backwards
+    compatibility.)
 
-  Colors and brushes can be set for particular roles in any of a
-  palette's color groups with setColor() and setBrush().
+    Colors and brushes can be set for particular roles in any of a
+    palette's color groups with setColor() and setBrush().
 
-  You can copy a palette using the copy constructor and test to see if
-  two palettes are \e identical using isCopyOf().
+    You can copy a palette using the copy constructor and test to see
+    if two palettes are \e identical using isCopyOf().
 
-  \sa QApplication::setPalette(), QWidget::setPalette(), QColorGroup, QColor
+    \sa QApplication::setPalette(), QWidget::setPalette(), QColorGroup, QColor
 */
 
-/*! \enum QPalette::ColorGroup
+/*!
+    \enum QPalette::ColorGroup
+
     \value Disabled
     \value Active
     \value Inactive
@@ -516,14 +569,22 @@ bool QColorGroup::operator==( const QColorGroup &g ) const
 */
 
 /*!
+    \obsolete
+
     \fn const QColorGroup &QPalette::normal() const
-    Returns the active color group.
+
+    Returns the active color group. Use active() instead.
+
     \sa setActive() active()
 */
 
 /*!
+    \obsolete
+
     \fn void QPalette::setNormal( const QColorGroup & cg )
-    Sets the active color group to \a cg.
+
+    Sets the active color group to \a cg. Use setActive() instead.
+
     \sa setActive() active()
 */
 
@@ -531,18 +592,18 @@ bool QColorGroup::operator==( const QColorGroup &g ) const
 static int palette_count = 1;
 
 /*!
-  Constructs a palette that consists of color groups with only black colors.
+    Constructs a palette that consists of color groups with only black
+    colors.
 */
 
 QPalette::QPalette()
 {
     static QPalData *defPalData = 0;
     if ( !defPalData ) {                // create common palette data
-        defPalData = new QPalData;      //   for the default palette
-        Q_CHECK_PTR( defPalData );
-	static QCleanupHandler<QPalData> defPalCleanup;
-	defPalCleanup.add( &defPalData );
-        defPalData->ser_no = palette_count++;
+	defPalData = new QPalData;      //   for the default palette
+	static QSharedCleanupHandler<QPalData> defPalCleanup;
+	defPalCleanup.set( &defPalData );
+	defPalData->ser_no = palette_count++;
     }
     data = defPalData;
     data->ref();
@@ -579,8 +640,9 @@ QPalette::QPalette( const QColor &button )
 }
 
 /*!
-  Constructs a palette from a \a button color and a \a background. The
-  other colors are automatically calculated, based on these colors.
+    Constructs a palette from a \a button color and a \a background.
+    The other colors are automatically calculated, based on these
+    colors.
 */
 
 QPalette::QPalette( const QColor &button, const QColor &background )
@@ -608,12 +670,13 @@ QPalette::QPalette( const QColor &button, const QColor &background )
 }
 
 /*!
-  Constructs a palette that consists of the three color groups \a
-  active, \a disabled and \a inactive.  See QPalette for definitions
-  of the color groups and QColorGroup::ColorRole for definitions of
-  each color role in the three groups.
+    Constructs a palette that consists of the three color groups \a
+    active, \a disabled and \a inactive. See the \link #details
+    Detailed Description\endlink for definitions of the color groups
+    and \l QColorGroup::ColorRole for definitions of each color role
+    in the three groups.
 
-  \sa QColorGroup QColorGroup::ColorRole QPalette
+    \sa QColorGroup QColorGroup::ColorRole QPalette
 */
 
 QPalette::QPalette( const QColorGroup &active, const QColorGroup &disabled,
@@ -628,9 +691,9 @@ QPalette::QPalette( const QColorGroup &active, const QColorGroup &disabled,
 }
 
 /*!
-  Constructs a copy of \a p.
+    Constructs a copy of \a p.
 
-  This constructor is fast (it uses copy-on-write).
+    This constructor is fast (it uses copy-on-write).
 */
 
 QPalette::QPalette( const QPalette &p )
@@ -640,7 +703,7 @@ QPalette::QPalette( const QPalette &p )
 }
 
 /*!
-  Destroys the palette.
+    Destroys the palette.
 */
 
 QPalette::~QPalette()
@@ -650,12 +713,12 @@ QPalette::~QPalette()
 }
 
 /*!
-  Assigns \a p to this palette and returns a reference to this
-  palette.
+    Assigns \a p to this palette and returns a reference to this
+    palette.
 
-  This is fast (it uses copy-on-write).
+    This is fast (it uses copy-on-write).
 
-  \sa copy()
+    \sa copy()
 */
 
 QPalette &QPalette::operator=( const QPalette &p )
@@ -669,9 +732,9 @@ QPalette &QPalette::operator=( const QPalette &p )
 
 
 /*!
-  Returns the color in color group \a gr, used for color role \a r.
+    Returns the color in color group \a gr, used for color role \a r.
 
-  \sa brush() setColor() QColorGroup::ColorRole
+    \sa brush() setColor() QColorGroup::ColorRole
 */
 const QColor &QPalette::color( ColorGroup gr, QColorGroup::ColorRole r ) const
 {
@@ -679,9 +742,9 @@ const QColor &QPalette::color( ColorGroup gr, QColorGroup::ColorRole r ) const
 }
 
 /*!
-  Returns the brush in color group \a gr, used for color role \a r.
+    Returns the brush in color group \a gr, used for color role \a r.
 
-  \sa color() setBrush() QColorGroup::ColorRole
+    \sa color() setBrush() QColorGroup::ColorRole
 */
 const QBrush &QPalette::brush( ColorGroup gr, QColorGroup::ColorRole r ) const
 {
@@ -689,10 +752,10 @@ const QBrush &QPalette::brush( ColorGroup gr, QColorGroup::ColorRole r ) const
 }
 
 /*!
-  Sets the brush in color group \a gr, used for color role \a r, to the
-  solid color \a c.
+    Sets the brush in color group \a gr, used for color role \a r, to
+    the solid color \a c.
 
-  \sa setBrush() color() QColorGroup::ColorRole
+    \sa setBrush() color() QColorGroup::ColorRole
 */
 void QPalette::setColor( ColorGroup gr, QColorGroup::ColorRole r,
 			 const QColor &c)
@@ -701,24 +764,26 @@ void QPalette::setColor( ColorGroup gr, QColorGroup::ColorRole r,
 }
 
 /*!
-  Sets the brush in color group \a gr, used for color role \a r, to \a b.
+    Sets the brush in color group \a gr, used for color role \a r, to
+    \a b.
 
-  \sa brush() setColor() QColorGroup::ColorRole
+    \sa brush() setColor() QColorGroup::ColorRole
 */
 void QPalette::setBrush( ColorGroup gr, QColorGroup::ColorRole r,
 			 const QBrush &b)
 {
     detach();
     data->ser_no = palette_count++;
-    directBrush( gr, r ) = b;
+    directSetBrush( gr, r, b);
 }
 
 /*!
     \overload
-  Sets the brush color used for color role \a r to color \a c in all
-  three color groups.
 
-  \sa color() setBrush() QColorGroup::ColorRole
+    Sets the brush color used for color role \a r to color \a c in all
+    three color groups.
+
+    \sa color() setBrush() QColorGroup::ColorRole
 */
 void QPalette::setColor( QColorGroup::ColorRole r, const QColor &c )
 {
@@ -727,22 +792,27 @@ void QPalette::setColor( QColorGroup::ColorRole r, const QColor &c )
 
 /*!
     \overload
-  Sets the brush in for color role \a r in all three color groups to \a b.
 
-  \sa brush() setColor() QColorGroup::ColorRole active() inactive() disabled()
+    Sets the brush in for color role \a r in all three color groups to
+    \a b.
+
+    \sa brush() setColor() QColorGroup::ColorRole active() inactive() disabled()
 */
 void QPalette::setBrush( QColorGroup::ColorRole r, const QBrush &b )
 {
     detach();
     data->ser_no = palette_count++;
-    directBrush( Active,   r ) = b;
-    directBrush( Disabled, r ) = b;
-    directBrush( Inactive,   r ) = b;
+    directSetBrush( Active, r, b );
+    directSetBrush( Disabled, r, b );
+    directSetBrush( Inactive, r, b );
 }
 
 
-/*! Returns a deep copy of this palette.  This is slower than the copy
-constructor and assignment operator and no longer offers any advantages.
+/*!
+    Returns a deep copy of this palette.
+
+    \warning This is slower than the copy constructor and assignment
+    operator and offers no benefits.
 */
 
 QPalette QPalette::copy() const
@@ -753,12 +823,12 @@ QPalette QPalette::copy() const
 
 
 /*!
-  Detaches this palette from any other QPalette objects with which it
-  might implicitly share QColorGroup objects.  In essence, does the
-  copying part of copy-on-write.
+    Detaches this palette from any other QPalette objects with which
+    it might implicitly share QColorGroup objects. In essence, does
+    the copying part of copy-on-write.
 
-  Calling this should generally not be necessary; QPalette calls it
-  itself when necessary.
+    Calling this should generally not be necessary; QPalette calls it
+    itself when necessary.
 */
 
 void QPalette::detach()
@@ -768,16 +838,17 @@ void QPalette::detach()
 }
 
 /*!
-  \fn const QColorGroup & QPalette::disabled() const
+    \fn const QColorGroup & QPalette::disabled() const
 
-  Returns the disabled color group of this palette.
+    Returns the disabled color group of this palette.
 
-  \sa QColorGroup, setDisabled(), active(), inactive()
+    \sa QColorGroup, setDisabled(), active(), inactive()
 */
 
 /*!
-  Sets the \c Disabled color group to \a g.
-  \sa disabled() setActive() setInactive()
+    Sets the \c Disabled color group to \a g.
+
+    \sa disabled() setActive() setInactive()
 */
 
 void QPalette::setDisabled( const QColorGroup &g )
@@ -788,14 +859,17 @@ void QPalette::setDisabled( const QColorGroup &g )
 }
 
 /*!
-  \fn const QColorGroup & QPalette::active() const
-  Returns the active color group of this palette.
-  \sa QColorGroup, setActive(), inactive(), disabled()
+    \fn const QColorGroup & QPalette::active() const
+
+    Returns the active color group of this palette.
+
+    \sa QColorGroup, setActive(), inactive(), disabled()
 */
 
 /*!
-  Sets the \c Active color group to \a g.
-  \sa active() setDisabled() setInactive() QColorGroup
+    Sets the \c Active color group to \a g.
+
+    \sa active() setDisabled() setInactive() QColorGroup
 */
 
 void QPalette::setActive( const QColorGroup &g )
@@ -806,14 +880,17 @@ void QPalette::setActive( const QColorGroup &g )
 }
 
 /*!
-  \fn const QColorGroup & QPalette::inactive() const
-  Returns the inactive color group of this palette.
-  \sa QColorGroup,  setInactive(), active(), disabled()
+    \fn const QColorGroup & QPalette::inactive() const
+
+    Returns the inactive color group of this palette.
+
+    \sa QColorGroup,  setInactive(), active(), disabled()
 */
 
 /*!
-  Sets the \c Inactive color group to \a g.
-  \sa active() setDisabled() setActive() QColorGroup
+    Sets the \c Inactive color group to \a g.
+
+    \sa active() setDisabled() setActive() QColorGroup
 */
 
 void QPalette::setInactive( const QColorGroup &g )
@@ -825,14 +902,15 @@ void QPalette::setInactive( const QColorGroup &g )
 
 
 /*!
-  \fn bool QPalette::operator!=( const QPalette &p ) const
+    \fn bool QPalette::operator!=( const QPalette &p ) const
 
-  Returns TRUE (slowly) if this palette is different from \a p;
-  otherwise returns FALSE (usually quickly).
+    Returns TRUE (slowly) if this palette is different from \a p;
+    otherwise returns FALSE (usually quickly).
 */
 
-/*!  Returns TRUE (usually quickly) if this palette is equal to \a p;
-otherwise returns FALSE (slowly).
+/*!
+    Returns TRUE (usually quickly) if this palette is equal to \a p;
+    otherwise returns FALSE (slowly).
 */
 
 bool QPalette::operator==( const QPalette &p ) const
@@ -844,17 +922,17 @@ bool QPalette::operator==( const QPalette &p ) const
 
 
 /*!
-  \fn int QPalette::serialNumber() const
+    \fn int QPalette::serialNumber() const
 
-  Returns a number that uniquely identifies this QPalette object.  The
-  serial number is intended for caching.  Its value may not be used
-  for anything other than equality testing.
+    Returns a number that uniquely identifies this QPalette object.
+    The serial number is intended for caching. Its value may not be
+    used for anything other than equality testing.
 
-  Note that QPalette uses copy-on-write, and the serial number changes
-  during the lazy copy operation (detach()), not during a shallow
-  copy (copy constructor or assignment).
+    Note that QPalette uses copy-on-write, and the serial number
+    changes during the lazy copy operation (detach()), not during a
+    shallow copy (copy constructor or assignment).
 
-  \sa QPixmap QPixmapCache QCache
+    \sa QPixmap QPixmapCache QCache
 */
 
 
@@ -864,10 +942,11 @@ bool QPalette::operator==( const QPalette &p ) const
 
 #ifndef QT_NO_DATASTREAM
 /*!
-  \relates QColorGroup
-  Writes color group, \a g to the stream \a s.
+    \relates QColorGroup
 
-  \sa \link datastreamformat.html Format of the QDataStream operators \endlink
+    Writes color group, \a g to the stream \a s.
+
+    \sa \link datastreamformat.html Format of the QDataStream operators \endlink
 */
 
 QDataStream &operator<<( QDataStream &s, const QColorGroup &g )
@@ -893,10 +972,11 @@ QDataStream &operator<<( QDataStream &s, const QColorGroup &g )
 }
 
 /*!
-  \related QColorGroup
-  Reads a color group from the stream.
+    \related QColorGroup
 
-  \sa \link datastreamformat.html Format of the QDataStream operators \endlink
+    Reads a color group from the stream.
+
+    \sa \link datastreamformat.html Format of the QDataStream operators \endlink
 */
 
 QDataStream &operator>>( QDataStream &s, QColorGroup &g )
@@ -930,11 +1010,12 @@ QDataStream &operator>>( QDataStream &s, QColorGroup &g )
 
 
 /*!
-  \relates QPalette
-  Writes the palette, \a p to the stream \a s and returns a reference
-  to the stream.
+    \relates QPalette
 
-  \sa \link datastreamformat.html Format of the QDataStream operators \endlink
+    Writes the palette, \a p to the stream \a s and returns a
+    reference to the stream.
+
+    \sa \link datastreamformat.html Format of the QDataStream operators \endlink
 */
 
 QDataStream &operator<<( QDataStream &s, const QPalette &p )
@@ -974,11 +1055,12 @@ static void readV1ColorGroup( QDataStream &s, QColorGroup &g,
 
 
 /*!
-  \relates QPalette
-  Reads a palette from the stream, \a s into the palette \a p, and
-  returns a reference to the stream.
+    \relates QPalette
 
-  \sa \link datastreamformat.html Format of the QDataStream operators \endlink
+    Reads a palette from the stream, \a s into the palette \a p, and
+    returns a reference to the stream.
+
+    \sa \link datastreamformat.html Format of the QDataStream operators \endlink
 */
 
 QDataStream &operator>>( QDataStream &s, QPalette &p )
@@ -997,11 +1079,13 @@ QDataStream &operator>>( QDataStream &s, QPalette &p )
 }
 #endif //QT_NO_DATASTREAM
 
-/*!  Returns TRUE if this palette and \a p are copies of each other,
-  ie. one of them was created as a copy of the other and neither was
-  subsequently modified.  This is much stricter than equality.
+/*!
+    Returns TRUE if this palette and \a p are copies of each other,
+    i.e. one of them was created as a copy of the other and neither
+    was subsequently modified; otherwise returns FALSE. This is much
+    stricter than equality.
 
-  \sa operator=() operator==()
+    \sa operator=() operator==()
 */
 
 bool QPalette::isCopyOf( const QPalette & p )
@@ -1009,7 +1093,7 @@ bool QPalette::isCopyOf( const QPalette & p )
     return data && data == p.data;
 }
 
-QBrush &QPalette::directBrush( ColorGroup gr, QColorGroup::ColorRole r ) const
+const QBrush &QPalette::directBrush( ColorGroup gr, QColorGroup::ColorRole r ) const
 {
     if ( (uint)gr > (uint)QPalette::NColorGroups ) {
 #if defined(QT_CHECK_RANGE)
@@ -1042,6 +1126,38 @@ QBrush &QPalette::directBrush( ColorGroup gr, QColorGroup::ColorRole r ) const
     return data->active.br[QColorGroup::Foreground]; // Satisfy compiler
 }
 
+void QPalette::directSetBrush( ColorGroup gr, QColorGroup::ColorRole r, const QBrush& b)
+{
+    if ( (uint)gr > (uint)QPalette::NColorGroups ) {
+#if defined(QT_CHECK_RANGE)
+	qWarning( "QPalette::directBrush: colorGroup(%i) out of range", gr );
+#endif
+	return;
+    }
+    if ( (uint)r >= (uint)QColorGroup::NColorRoles ) {
+#if defined(QT_CHECK_RANGE)
+	qWarning( "QPalette::directBrush: colorRole(%i) out of range", r );
+#endif
+	return;
+    }
+    switch( gr ) {
+    case Active:
+	data->active.setBrush(r,b);
+	break;
+    case Disabled:
+	data->disabled.setBrush(r,b);
+	break;
+    case Inactive:
+	data->inactive.setBrush(r,b);
+	break;
+    default:
+#if defined(QT_CHECK_RANGE)
+	qWarning( "QPalette::directBrush: colorGroup(%i) internal error", gr );
+#endif
+	break;
+    }
+}
+
 
 /*!\internal*/
 QColorGroup::ColorRole QPalette::foregroundRoleFromMode( Qt::BackgroundMode mode )
@@ -1051,6 +1167,11 @@ QColorGroup::ColorRole QPalette::foregroundRoleFromMode( Qt::BackgroundMode mode
 	return QColorGroup::ButtonText;
     case Qt::PaletteBase:
 	return QColorGroup::Text;
+    case Qt::PaletteDark:
+    case Qt::PaletteShadow:
+	return QColorGroup::Light;
+    case Qt::PaletteHighlight:
+	return QColorGroup::HighlightedText;
     case Qt::PaletteBackground:
     default:
 	return QColorGroup::Foreground;
