@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/qpixmapcache.cpp   3.1.2   edited Nov 7 16:55 $
+** $Id: qt/qpixmapcache.cpp   3.2.3   edited May 13 09:08 $
 **
 ** Implementation of QPixmapCache class
 **
@@ -312,6 +312,16 @@ void QPixmapCache::setCacheLimit( int n )
 }
 
 
+/*!
+  Removes the pixmap associated with \a key from the cache.
+*/
+void QPixmapCache::remove( const QString &key )
+{
+    if ( pm_cache )
+	pm_cache->remove( key );
+}
+
+
 /*!
     Removes all pixmaps from the cache.
 */
