@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id:  qt/qpngio.cpp   3.0.5   edited Jun 13 04:41 $
+** $Id: qt/qpngio.cpp   3.2.3   edited May 13 09:08 $
 **
 ** Implementation of PNG QImage IOHandler
 **
 ** Created : 970521
 **
-** Copyright (C) 1992-1998 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2003 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the kernel module of the Qt GUI Toolkit.
 **
@@ -44,11 +44,19 @@
 
 #include <png.h>
 
+
+#ifdef Q_OS_TEMP
+#define CALLBACK_CALL_TYPE	__cdecl
+#else
+#define CALLBACK_CALL_TYPE
+#endif
+
+
 /*
   All PNG files load to the minimal QImage equivalent.
 
   All QImage formats output to reasonably efficient PNG equivalents.
-  Never to greyscale.
+  Never to grayscale.
 */
 
 #if defined(Q_C_CALLBACKS)
@@ -56,7 +64,7 @@ extern "C" {
 #endif
 
 static
-void iod_read_fn(png_structp png_ptr, png_bytep data, png_size_t length)
+void CALLBACK_CALL_TYPE iod_read_fn(png_structp png_ptr, png_bytep data, png_size_t length)
 {
     QImageIO* iio = (QImageIO*)png_get_io_ptr(png_ptr);
     QIODevice* in = iio->ioDevice();
@@ -73,7 +81,7 @@ void iod_read_fn(png_structp png_ptr, png_bytep data, png_size_t length)
 
 
 static
-void qpiw_write_fn( png_structp png_ptr, png_bytep data, png_size_t length )
+void CALLBACK_CALL_TYPE qpiw_write_fn( png_structp png_ptr, png_bytep data, png_size_t length )
 {
     QPNGImageWriter* qpiw = (QPNGImageWriter*)png_get_io_ptr( png_ptr );
     QIODevice* out = qpiw->device();
@@ -87,7 +95,7 @@ void qpiw_write_fn( png_structp png_ptr, png_bytep data, png_size_t length )
 
 
 static
-void qpiw_flush_fn( png_structp png_ptr )
+void CALLBACK_CALL_TYPE qpiw_flush_fn( png_structp png_ptr )
 {
     QPNGImageWriter* qpiw = (QPNGImageWriter*)png_get_io_ptr( png_ptr );
     QIODevice* out = qpiw->device();
@@ -116,7 +124,7 @@ void setup_qt( QImage& image, png_structp png_ptr, png_infop info_ptr, float scr
 	0, 0, 0);
 
     if ( color_type == PNG_COLOR_TYPE_GRAY ) {
-	// Black & White or 8-bit greyscale
+	// Black & White or 8-bit grayscale
 	if ( bit_depth == 1 && info_ptr->channels == 1 ) {
 	    png_set_invert_mono( png_ptr );
 	    png_read_update_info( png_ptr, info_ptr );
@@ -221,7 +229,7 @@ void setup_qt( QImage& image, png_structp png_ptr, png_infop info_ptr, float scr
 #if defined(Q_C_CALLBACKS)
 extern "C" {
 #endif
-static void qt_png_warning(png_structp /*png_ptr*/, png_const_charp message)
+static void CALLBACK_CALL_TYPE qt_png_warning(png_structp /*png_ptr*/, png_const_charp message)
 {
     qWarning("libpng warning: %s", message);
 }
@@ -245,7 +253,7 @@ void read_png_image(QImageIO* iio)
 	return;
     }
 
-    png_set_error_fn(png_ptr, 0, 0, &qt_png_warning);
+    png_set_error_fn(png_ptr, 0, 0, qt_png_warning);
 
     info_ptr = png_create_info_struct(png_ptr);
     if (!info_ptr) {
@@ -311,8 +319,36 @@ png_get_valid(png_ptr, info_ptr, PNG_INFO_tRNS)
     image.setDotsPerMeterX(png_get_x_pixels_per_meter(png_ptr,info_ptr));
     image.setDotsPerMeterY(png_get_y_pixels_per_meter(png_ptr,info_ptr));
 
+#ifndef QT_NO_IMAGE_TEXT
+    png_textp text_ptr;
+    int num_text=0;
+    png_get_text(png_ptr,info_ptr,&text_ptr,&num_text);
+    while (num_text--) {
+	image.setText(text_ptr->key,0,text_ptr->text);
+	text_ptr++;
+    }
+#endif
+
     delete [] row_pointers;
 
+    if ( image.hasAlphaBuffer() ) {
+	// Many PNG files lie (eg. from PhotoShop). Fortunately this loop will
+	// usually be quick to find those that tell the truth.
+	QRgb* c;
+	int n;
+	if (image.depth()==32) {
+	    c = (QRgb*)image.bits();
+	    n = image.bytesPerLine() * image.height() / 4;
+	} else {
+	    c = image.colorTable();
+	    n = image.numColors();
+	}
+	while ( n-- && qAlpha(*c++)==0xff )
+	    ;
+	if ( n<0 ) // LIAR!
+	    image.setAlphaBuffer(FALSE);
+    }
+
     iio->setImage(image);
 
     png_read_end(png_ptr, end_info);
@@ -406,7 +442,7 @@ bool QPNGImageWriter::writeImage(const QImage& image, int quality_in, int off_x_
 	return FALSE;
     }
 
-    png_set_error_fn(png_ptr, 0, 0, &qt_png_warning);
+    png_set_error_fn(png_ptr, 0, 0, qt_png_warning);
 
     info_ptr = png_create_info_struct(png_ptr);
     if (!info_ptr) {
@@ -879,20 +915,19 @@ const char* QPNGFormatType::formatName() const
     return "PNG";
 }
 
-
 #if defined(Q_C_CALLBACKS)
 extern "C" {
 #endif
 
 static void
-info_callback(png_structp png_ptr, png_infop info)
+CALLBACK_CALL_TYPE info_callback(png_structp png_ptr, png_infop info)
 {
     QPNGFormat* that = (QPNGFormat*)png_get_progressive_ptr(png_ptr);
     that->info(png_ptr,info);
 }
 
 static void
-row_callback(png_structp png_ptr, png_bytep new_row,
+CALLBACK_CALL_TYPE row_callback(png_structp png_ptr, png_bytep new_row,
    png_uint_32 row_num, int pass)
 {
     QPNGFormat* that = (QPNGFormat*)png_get_progressive_ptr(png_ptr);
@@ -900,21 +935,23 @@ row_callback(png_structp png_ptr, png_bytep new_row,
 }
 
 static void
-end_callback(png_structp png_ptr, png_infop info)
+CALLBACK_CALL_TYPE end_callback(png_structp png_ptr, png_infop info)
 {
     QPNGFormat* that = (QPNGFormat*)png_get_progressive_ptr(png_ptr);
     that->end(png_ptr,info);
 }
 
+#if 0
 #ifdef PNG_USER_CHUNKS_SUPPORTED
 static int
-user_chunk_callback(png_structp png_ptr,
+CALLBACK_CALL_TYPE user_chunk_callback(png_structp png_ptr,
          png_unknown_chunkp chunk)
 {
     QPNGFormat* that = (QPNGFormat*)png_get_progressive_ptr(png_ptr);
     return that->user_chunk(png_ptr,chunk->data,chunk->size);
 }
 #endif
+#endif
 
 #if defined(Q_C_CALLBACKS)
 }
@@ -964,7 +1001,7 @@ int QPNGFormat::decode(QImage& img, QImageConsumer* cons,
 		return -1;
 	}
 
-	png_set_error_fn(png_ptr, 0, 0, &qt_png_warning);
+	png_set_error_fn(png_ptr, 0, 0, qt_png_warning);
 	png_set_compression_level(png_ptr, 9);
 
 	info_ptr = png_create_info_struct(png_ptr);
@@ -1105,6 +1142,10 @@ int QPNGFormat::user_chunk(png_structp png,
 	    return 1;
 	}
     }
+#else
+    Q_UNUSED( png )
+    Q_UNUSED( data )
+    Q_UNUSED( length )
 #endif
 
 #ifndef QT_NO_IMAGE_TEXT
@@ -1145,6 +1186,7 @@ static QPNGFormatType* globalPngFormatTypeObject = 0;
 
 #endif // QT_NO_ASYNC_IMAGE_IO
 
+static bool done = FALSE;
 void qCleanupPngIO()
 {
 #ifndef QT_NO_ASYNC_IMAGE_IO
@@ -1153,11 +1195,11 @@ void qCleanupPngIO()
 	globalPngFormatTypeObject = 0;
     }
 #endif
+    done = FALSE;
 }
 
 void qInitPngIO()
 {
-    static bool done = FALSE;
     if ( !done ) {
 	done = TRUE;
 	QImageIO::defineIOHandler( "PNG", "^.PNG\r", 0, read_png_image,
