@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/qprinter_unix.cpp   3.2.1   edited May 13 09:08 $
+** $Id: qt/qprinter_unix.cpp   3.2.3   edited Oct 1 14:22 $
 **
 ** Implementation of QPrinter class for Unix
 **
@@ -118,6 +118,7 @@ QPrinter::QPrinter( PrinterMode m )
     state = PST_IDLE;
     output_file = FALSE;
     to_edge     = FALSE;
+    paper_source = OnlyOne;
     switch ( m ) {
 	case ScreenResolution:
 #ifdef Q_WS_QWS
@@ -196,7 +197,7 @@ bool QPrinter::abort()
 }
 
 /*!
-    Returns TRUE is the printer job was aborted; otherwise returns
+    Returns TRUE if the print job was aborted; otherwise returns
     FALSE.
 
     \sa abort()
@@ -242,7 +243,7 @@ static void deleteGlobalPrinterDefaults()
     it should have.
 
     Returns TRUE if the user pressed "OK" to print, or FALSE if the
-    user cancelled the operation.
+    user canceled the operation.
 */
 
 bool QPrinter::setup( QWidget * parent )
