@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id:  qt/qprocess.cpp   3.0.5   edited Apr 26 16:39 $
+** $Id: qt/qprocess.cpp   3.2.3   edited May 13 09:08 $
 **
 ** Implementation of QProcess class
 **
@@ -43,6 +43,7 @@
 #ifndef QT_NO_PROCESS
 
 #include "qapplication.h"
+#include "private/qinternal_p.h"
 
 
 //#define QT_QPROCESS_DEBUG
@@ -144,12 +145,35 @@
 
     The readyReadStdout() signal is emitted when there is new data on
     standard output. This happens asynchronously: you don't know if
-    more data will arrive later. In the above example you could
-    connect the processExited() signal to the slot
-    UicManager::readFromStdout() instead. If you do so, you will be
-    certain that all the data is available when the slot is called. On
-    the other hand, you must wait until the process has finished
-    before doing any processing.
+    more data will arrive later.
+
+    In the above example you could connect the processExited() signal
+    to the slot UicManager::readFromStdout() instead. If you do so,
+    you will be certain that all the data is available when the slot
+    is called. On the other hand, you must wait until the process has
+    finished before doing any processing.
+
+    Note that if you are expecting a lot of output from the process,
+    you may hit platform-dependent limits to the pipe buffer size. The
+    solution is to make sure you connect to the output, e.g. the
+    readyReadStdout() and readyReadStderr() signals and read the data
+    as soon as it becomes available.
+
+    \section1 Notes for Windows users
+
+    Some Windows commands, for example, \c dir, are not provided by
+    separate applications, but by the command interpreter.
+    If you attempt to use QProcess to execute these commands directly
+    it won't work. One possible solution is to execute the command
+    interpreter itself (\c cmd.exe on some Windows systems), and ask
+    the interpreter to execute the desired command.
+
+    Under Windows there are certain problems starting 16-bit applications
+    and capturing their output. Microsoft recommends using an intermediate
+    application to start 16-bit applications. See Knowledge Base article \link
+    http://support.microsoft.com/default.aspx?scid=KB;en-us;q150956 Q150956
+    \endlink for details on this and example code for an intermediate
+    application.
 
     \sa QSocket
 */
@@ -277,7 +301,7 @@ void QProcess::clearArguments()
     the list are the arguments to the command. Any previous arguments
     are deleted.
 
-    QProcess does perform argument substitutions; for example, if you
+    QProcess does not perform argument substitutions; for example, if you
     specify "*" or "$DISPLAY", these values are passed to the process
     literally. If you want to have the same behavior as the shell
     provides, you must do the substitutions yourself; i.e. instead of
@@ -422,12 +446,10 @@ QByteArray QProcess::readStdout()
 	return QByteArray();
     }
     readStdoutCalled = TRUE;
-
-    QByteArray buf = bufStdout()->copy();
-    consumeBufStdout( -1 ); // consume everything
-
+    QMembuf *buf = membufStdout();
     readStdoutCalled = FALSE;
-    return buf;
+
+    return buf->readAll();
 }
 
 /*!
@@ -446,36 +468,10 @@ QByteArray QProcess::readStderr()
 	return QByteArray();
     }
     readStderrCalled = TRUE;
-
-    QByteArray buf = bufStderr()->copy();
-    consumeBufStderr( -1 ); // consume everything
-
+    QMembuf *buf = membufStderr();
     readStderrCalled = FALSE;
-    return buf;
-}
 
-/*!
-    Returns TRUE if it's possible to read an entire line of text from
-    standard output at this time; otherwise returns FALSE.
-
-    \sa readLineStdout() canReadLineStderr()
-*/
-bool QProcess::canReadLineStdout() const
-{
-    QProcess *that = (QProcess*)this;
-    return that->scanNewline( TRUE, 0 );
-}
-
-/*!
-    Returns TRUE if it's possible to read an entire line of text from
-    standard error at this time; otherwise returns FALSE.
-
-    \sa readLineStderr() canReadLineStdout()
-*/
-bool QProcess::canReadLineStderr() const
-{
-    QProcess *that = (QProcess*)this;
-    return that->scanNewline( FALSE, 0 );
+    return buf->readAll();
 }
 
 /*!
@@ -487,15 +483,24 @@ bool QProcess::canReadLineStderr() const
 */
 QString QProcess::readLineStdout()
 {
-    QByteArray a;
-    QString s;
-    if ( scanNewline( TRUE, &a ) ) {
-	if ( a.isEmpty() )
-	    s = "";
-	else
-	    s = QString( a );
+    QByteArray a( 256 );
+    QMembuf *buf = membufStdout();
+    if ( buf->scanNewline( &a ) ) {
+	uint size = a.size();
+	buf->consumeBytes( size, 0 );
+
+	// get rid of terminating \n or \r\n
+	if ( size>0 && a.at( size - 1 ) == '\n' ) {
+	    if ( size>1 && a.at( size - 2 ) == '\r' )
+		a.at( size - 2 ) = '\0';
+	    else
+		a.at( size - 1 ) = '\0';
+	}
+	return QString( a );
+    } else if ( canReadLineStdout() ) {
+	return QString( buf->readAll() );
     }
-    return s;
+    return QString::null;
 }
 
 /*!
@@ -507,51 +512,24 @@ QString QProcess::readLineStdout()
 */
 QString QProcess::readLineStderr()
 {
-    QByteArray a;
-    QString s;
-    if ( scanNewline( FALSE, &a ) ) {
-	if ( a.isEmpty() )
-	    s = "";
-	else
-	    s = QString( a );
-    }
-    return s;
-}
+    QByteArray a( 256 );
+    QMembuf *buf = membufStderr();
+    if ( buf->scanNewline( &a ) ) {
+	uint size = a.size();
+	buf->consumeBytes( size, 0 );
 
-/*
-  This private function scans for any occurrence of \n or \r\n in the
-  buffer \e buf. It stores the text in the byte array \a store if it is
-  non-null.
-*/
-bool QProcess::scanNewline( bool stdOut, QByteArray *store )
-{
-    QByteArray *buf;
-    if ( stdOut )
-	buf = bufStdout();
-    else
-	buf = bufStderr();
-    uint n = buf->size();
-    uint i;
-    for ( i=0; i<n; i++ ) {
-	if ( buf->at(i) == '\n' ) {
-	    break;
+	// get rid of terminating \n or \r\n
+	if ( size>0 && a.at( size - 1 ) == '\n' ) {
+	    if ( size>1 && a.at( size - 2 ) == '\r' )
+		a.at( size - 2 ) = '\0';
+	    else
+		a.at( size - 1 ) = '\0';
 	}
+	return QString( a );
+    } else if ( canReadLineStderr() ) {
+	return QString( buf->readAll() );
     }
-    if ( i >= n )
-	return FALSE;
-
-    if ( store ) {
-	uint lineLength = i;
-	if ( lineLength>0 && buf->at(lineLength-1) == '\r' )
-	    lineLength--; // (if there are two \r, let one stay)
-	store->resize( lineLength );
-	memcpy( store->data(), buf->data(), lineLength );
-	if ( stdOut )
-	    consumeBufStdout( i+1 );
-	else
-	    consumeBufStderr( i+1 );
-    }
-    return TRUE;
+    return QString::null;
 }
 
 /*!
