@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id$
+** $Id: qt/qprocess_unix.cpp   3.2.3   edited Sep 19 17:29 $
 **
 ** Implementation of QProcess class for Unix
 **
 ** Created : 20000905
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2003 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the kernel module of the Qt GUI Toolkit.
 **
@@ -19,8 +19,8 @@
 ** packaging of this file.
 **
 ** Licensees holding valid Qt Enterprise Edition or Qt Professional Edition
-** licenses may use this file in accordance with the Qt Commercial License
-** Agreement provided with the Software.
+** licenses for Unix/X11 or for Qt/Embedded may use this file in accordance
+** with the Qt Commercial License Agreement provided with the Software.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -53,6 +53,7 @@
 #include "qtimer.h"
 #include "qcleanuphandler.h"
 #include "qregexp.h"
+#include "private/qinternal_p.h"
 
 #include <stdlib.h>
 #include <errno.h>
@@ -91,8 +92,8 @@ public:
     void closeOpenSocketsForChild();
     void newProc( pid_t pid, QProcess *process );
 
-    QByteArray bufStdout;
-    QByteArray bufStderr;
+    QMembuf bufStdout;
+    QMembuf bufStderr;
 
     QPtrQueue<QByteArray> stdinBuf;
 
@@ -140,7 +141,7 @@ public:
 #if defined(QT_QPROCESS_DEBUG)
 	qDebug( "QProc: Destructor for pid %d and QProcess %p", pid, process );
 #endif
-	if ( process != 0 ) {
+	if ( process ) {
 	    if ( process->d->notifierStdin )
 		process->d->notifierStdin->setEnabled( FALSE );
 	    if ( process->d->notifierStdout )
@@ -149,13 +150,11 @@ public:
 		process->d->notifierStderr->setEnabled( FALSE );
 	    process->d->proc = 0;
 	}
-	if( socketStdin != 0 )
+	if( socketStdin )
 	    ::close( socketStdin );
-	// ### close these sockets even on parent exit or is it better only on
-	// sigchld (but what do I have to do with them on exit then)?
-	if( socketStdout != 0 )
+	if( socketStdout )
 	    ::close( socketStdout );
-	if( socketStderr != 0 )
+	if( socketStderr )
 	    ::close( socketStderr );
     }
 
@@ -365,8 +364,11 @@ void QProcessManager::sigchldHnd( int fd )
     // the slot connected to the processExited() signal (e.g. by showing a
     // modal dialog) and there are more than one process which exited in the
     // meantime.
-    if ( sn )
+    if ( sn ) {
+	if ( !sn->isEnabled() )
+	    return;
 	sn->setEnabled( FALSE );
+    }
 
     char tmp;
     ::read( fd, &tmp, sizeof(tmp) );
@@ -385,21 +387,46 @@ void QProcessManager::sigchldHnd( int fd )
 #if defined(QT_QPROCESS_DEBUG)
 		qDebug( "QProcessManager::sigchldHnd() (PID: %d): process exited (QProcess available)", proc->pid );
 #endif
+		/*
+		  Apparently, there is not consistency among different
+		  operating systems on how to use FIONREAD.
+
+		  FreeBSD, Linux and Solaris all expect the 3rd
+		  argument to ioctl() to be an int, which is normally
+		  32-bit even on 64-bit machines.
+
+		  IRIX, on the other hand, expects a size_t, which is
+		  64-bit on 64-bit machines.
+
+		  So, the solution is to use size_t initialized to
+		  zero to make sure all bits are set to zero,
+		  preventing underflow with the FreeBSD/Linux/Solaris
+		  ioctls.
+		*/
+		size_t nbytes = 0;
 		// read pending data
-		int nbytes = 0;
-		if ( ::ioctl(proc->socketStdout, FIONREAD, (char*)&nbytes)==0 && nbytes>0 ) {
+		if ( proc->socketStdout && ::ioctl(proc->socketStdout, FIONREAD, (char*)&nbytes)==0 && nbytes>0 ) {
 #if defined(QT_QPROCESS_DEBUG)
 		    qDebug( "QProcessManager::sigchldHnd() (PID: %d): reading %d bytes of pending data on stdout", proc->pid, nbytes );
 #endif
 		    process->socketRead( proc->socketStdout );
 		}
 		nbytes = 0;
-		if ( ::ioctl(proc->socketStderr, FIONREAD, (char*)&nbytes)==0 && nbytes>0 ) {
+		if ( proc->socketStderr && ::ioctl(proc->socketStderr, FIONREAD, (char*)&nbytes)==0 && nbytes>0 ) {
 #if defined(QT_QPROCESS_DEBUG)
 		    qDebug( "QProcessManager::sigchldHnd() (PID: %d): reading %d bytes of pending data on stderr", proc->pid, nbytes );
 #endif
 		    process->socketRead( proc->socketStderr );
 		}
+		// close filedescriptors if open
+		if ( proc->socketStdout ) {
+		    ::close( proc->socketStdout );
+		    proc->socketStdout = 0;
+		}
+		if ( proc->socketStderr ) {
+		    ::close( proc->socketStderr );
+		    proc->socketStderr = 0;
+		}
 
 		if ( process->notifyOnExit )
 		    emit process->processExited();
@@ -490,11 +517,10 @@ void QProcessPrivate::closeOpenSocketsForChild()
 	    ::close( procManager->sigchldFd[1] );
 
 	// close also the sockets from other QProcess instances
-	QProc *proc;
-	for ( proc=procManager->procList->first(); proc!=0; proc=procManager->procList->next() ) {
-	    ::close( proc->socketStdin );
-	    ::close( proc->socketStdout );
-	    ::close( proc->socketStderr );
+	for ( QProc *p=procManager->procList->first(); p!=0; p=procManager->procList->next() ) {
+	    ::close( p->socketStdin );
+	    ::close( p->socketStdout );
+	    ::close( p->socketStderr );
 	}
     }
 }
@@ -552,54 +578,60 @@ void QProcess::reset()
     d = new QProcessPrivate();
     exitStat = 0;
     exitNormal = FALSE;
-    d->bufStdout.resize( 0 );
-    d->bufStderr.resize( 0 );
+    d->bufStdout.clear();
+    d->bufStderr.clear();
 }
 
-QByteArray* QProcess::bufStdout()
+QMembuf* QProcess::membufStdout()
 {
     if ( d->proc && d->proc->socketStdout ) {
-	int nbytes = 0;
+	/*
+	  Apparently, there is not consistency among different
+	  operating systems on how to use FIONREAD.
+
+	  FreeBSD, Linux and Solaris all expect the 3rd argument to
+	  ioctl() to be an int, which is normally 32-bit even on
+	  64-bit machines.
+
+	  IRIX, on the other hand, expects a size_t, which is 64-bit
+	  on 64-bit machines.
+
+	  So, the solution is to use size_t initialized to zero to
+	  make sure all bits are set to zero, preventing underflow
+	  with the FreeBSD/Linux/Solaris ioctls.
+	*/
+	size_t nbytes = 0;
 	if ( ::ioctl(d->proc->socketStdout, FIONREAD, (char*)&nbytes)==0 && nbytes>0 )
 	    socketRead( d->proc->socketStdout );
     }
     return &d->bufStdout;
 }
 
-QByteArray* QProcess::bufStderr()
+QMembuf* QProcess::membufStderr()
 {
     if ( d->proc && d->proc->socketStderr ) {
-	int nbytes = 0;
+	/*
+	  Apparently, there is not consistency among different
+	  operating systems on how to use FIONREAD.
+
+	  FreeBSD, Linux and Solaris all expect the 3rd argument to
+	  ioctl() to be an int, which is normally 32-bit even on
+	  64-bit machines.
+
+	  IRIX, on the other hand, expects a size_t, which is 64-bit
+	  on 64-bit machines.
+
+	  So, the solution is to use size_t initialized to zero to
+	  make sure all bits are set to zero, preventing underflow
+	  with the FreeBSD/Linux/Solaris ioctls.
+	*/
+	size_t nbytes = 0;
 	if ( ::ioctl(d->proc->socketStderr, FIONREAD, (char*)&nbytes)==0 && nbytes>0 )
 	    socketRead( d->proc->socketStderr );
     }
     return &d->bufStderr;
 }
 
-void QProcess::consumeBufStdout( int consume )
-{
-    uint n = d->bufStdout.size();
-    if ( consume==-1 || (uint)consume >= n ) {
-	d->bufStdout.resize( 0 );
-    } else {
-	QByteArray tmp( n - consume );
-	memcpy( tmp.data(), d->bufStdout.data()+consume, n-consume );
-	d->bufStdout = tmp;
-    }
-}
-
-void QProcess::consumeBufStderr( int consume )
-{
-    uint n = d->bufStderr.size();
-    if ( consume==-1 || (uint)consume >= n ) {
-	d->bufStderr.resize( 0 );
-    } else {
-	QByteArray tmp( n - consume );
-	memcpy( tmp.data(), d->bufStderr.data()+consume, n-consume );
-	d->bufStderr = tmp;
-    }
-}
-
 /*!
     Destroys the instance.
 
@@ -674,7 +706,7 @@ bool QProcess::start( QStringList *env )
 #ifndef Q_OS_QNX6
     if ( (comms & Stderr) && ::socketpair( AF_UNIX, SOCK_STREAM, 0, sStderr ) == -1 ) {
 #else
-    if ( (comms & Stdout) && qnx6SocketPairReplacement(sStderr) == -1 ) {
+    if ( (comms & Stderr) && qnx6SocketPairReplacement(sStderr) == -1 ) {
 #endif
 	return FALSE;
     }
@@ -747,15 +779,15 @@ bool QProcess::start( QStringList *env )
 
 	if ( env == 0 ) { // inherit environment and start process
 	    QString command = _arguments[0];
-#ifdef Q_OS_MACX //look in a bundle
+#if defined(Q_OS_MACX) //look in a bundle
 	    const QString mac_bundle_suffix = ".app/Contents/MacOS/";
-	    if(!QFile::exists(command) && QFile::exists(command + mac_bundle_suffix)) { 
+	    if(!QFile::exists(command) && QFile::exists(command + mac_bundle_suffix)) {
 		QString exec = command;
 		int lslash = command.findRev('/');
-		if(lslash != -1) 
+		if(lslash != -1)
 		    exec = command.mid(lslash+1);
 		QFileInfo fileInfo( command + mac_bundle_suffix + exec );
-		if ( fileInfo.isExecutable() ) 
+		if ( fileInfo.isExecutable() )
 		    command = fileInfo.absFilePath().local8Bit();
 	    }
 #endif
@@ -767,7 +799,7 @@ bool QProcess::start( QStringList *env )
 	} else { // start process with environment settins as specified in env
 	    // construct the environment for exec
 	    int numEntries = env->count();
-#ifdef Q_OS_MACX
+#if defined(Q_OS_MACX)
 	    QString ld_library_path("DYLD_LIBRARY_PATH");
 #else
 	    QString ld_library_path("LD_LIBRARY_PATH");
@@ -799,8 +831,8 @@ bool QProcess::start( QStringList *env )
 		    QStringList pathList = QStringList::split( ':', getenv( "PATH" ) );
 		    for (QStringList::Iterator it = pathList.begin(); it != pathList.end(); ++it ) {
 			QString dir = *it;
-#ifdef Q_OS_MACX //look in a bundle
-			if(!QFile::exists(dir + "/" + command) && QFile::exists(dir + "/" + command + ".app")) 
+#if defined(Q_OS_MACX) //look in a bundle
+			if(!QFile::exists(dir + "/" + command) && QFile::exists(dir + "/" + command + ".app"))
 			    dir += "/" + command + ".app/Contents/MacOS";
 #endif
 #ifndef QT_NO_DIR
@@ -809,7 +841,7 @@ bool QProcess::start( QStringList *env )
 			QFileInfo fileInfo( dir + "/" + command );
 #endif
 			if ( fileInfo.isExecutable() ) {
-#ifdef Q_OS_MACX
+#if defined(Q_OS_MACX)
 			    arglistQ[0] = fileInfo.absFilePath().local8Bit();
 #else
 			    arglistQ[0] = fileInfo.filePath().local8Bit();
@@ -818,16 +850,16 @@ bool QProcess::start( QStringList *env )
 			    break;
 			}
 		    }
-		} 
+		}
 	    }
-#ifdef Q_OS_MACX
+#if defined(Q_OS_MACX)
 	    if(!QFile::exists(arglist[0])) {
 		QString command = arglist[0];
 		const QString mac_bundle_suffix = ".app/Contents/MacOS/";
 		if(QFile::exists(command + mac_bundle_suffix)) {
 		    QString exec = command;
 		    int lslash = command.findRev('/');
-		    if(lslash != -1) 
+		    if(lslash != -1)
 			exec = command.mid(lslash+1);
 		    QFileInfo fileInfo( command + mac_bundle_suffix + exec );
 		    if ( fileInfo.isExecutable() ) {
@@ -863,6 +895,11 @@ bool QProcess::start( QStringList *env )
 	    int n = ::read( fd[0], &buf, 1 );
 	    if ( n==1 ) {
 		// socket was not closed => error
+		if ( ::waitpid( pid, 0, WNOHANG ) != pid ) {
+		    // The wait did not succeed yet, so try again when we get
+		    // the sigchild (to avoid zombies).
+		    d->newProc( pid, 0 );
+		}
 		d->proc = 0;
 		goto error;
 	    } else if ( n==-1 ) {
@@ -880,6 +917,11 @@ bool QProcess::start( QStringList *env )
     if ( comms & Stdin ) {
 	::close( sStdin[0] );
 	d->proc->socketStdin = sStdin[1];
+
+	// Select non-blocking mode
+	int originalFlags = fcntl(d->proc->socketStdin, F_GETFL, 0);
+	fcntl(d->proc->socketStdin, F_SETFL, originalFlags | O_NONBLOCK);
+
 	d->notifierStdin = new QSocketNotifier( sStdin[1], QSocketNotifier::Write );
 	connect( d->notifierStdin, SIGNAL(activated(int)),
 		this, SLOT(socketWrite(int)) );
@@ -1002,8 +1044,7 @@ bool QProcess::isRunning() const
     if ( d->proc == 0 )
 	return FALSE;
     int status;
-    if ( ::waitpid( d->proc->pid, &status, WNOHANG ) == d->proc->pid )
-    {
+    if ( ::waitpid( d->proc->pid, &status, WNOHANG ) == d->proc->pid ) {
 	// compute the exit values
 	QProcess *that = (QProcess*)this; // mutable
 	that->exitNormal = WIFEXITED( status ) != 0;
@@ -1011,6 +1052,20 @@ bool QProcess::isRunning() const
 	    that->exitStat = (char)WEXITSTATUS( status );
 	}
 	d->exitValuesCalculated = TRUE;
+
+	// On heavy processing, the socket notifier for the sigchild might not
+	// have found time to fire yet.
+	if ( d->procManager ) {
+	    fd_set fds;
+	    struct timeval tv;
+	    FD_ZERO( &fds );
+	    FD_SET( d->procManager->sigchldFd[1], &fds );
+	    tv.tv_sec = 0;
+	    tv.tv_usec = 0;
+	    if ( ::select( d->procManager->sigchldFd[1]+1, &fds, 0, 0, &tv ) > 0 )
+		d->procManager->sigchldHnd( d->procManager->sigchldFd[1] );
+	}
+
 #if defined(QT_QPROCESS_DEBUG)
 	qDebug( "QProcess::isRunning() (PID: %d): FALSE", d->proc->pid );
 #endif
@@ -1022,6 +1077,36 @@ bool QProcess::isRunning() const
     return TRUE;
 }
 
+/*!
+    Returns TRUE if it's possible to read an entire line of text from
+    standard output at this time; otherwise returns FALSE.
+
+    \sa readLineStdout() canReadLineStderr()
+*/
+bool QProcess::canReadLineStdout() const
+{
+    if ( !d->proc || !d->proc->socketStdout )
+	return d->bufStdout.size() != 0;
+
+    QProcess *that = (QProcess*)this;
+    return that->membufStdout()->scanNewline( 0 );
+}
+
+/*!
+    Returns TRUE if it's possible to read an entire line of text from
+    standard error at this time; otherwise returns FALSE.
+
+    \sa readLineStderr() canReadLineStdout()
+*/
+bool QProcess::canReadLineStderr() const
+{
+    if ( !d->proc || !d->proc->socketStderr )
+	return d->bufStderr.size() != 0;
+
+    QProcess *that = (QProcess*)this;
+    return that->membufStderr()->scanNewline( 0 );
+}
+
 /*!
     Writes the data \a buf to the process's standard input. The
     process may or may not read this data.
@@ -1093,9 +1178,9 @@ void QProcess::socketRead( int fd )
 #endif
     if ( fd == 0 )
 	return;
-    const int bufsize = 4096;
-    QByteArray *buffer = 0;
-    uint oldSize;
+    if ( !d->proc )
+	return;
+    QMembuf *buffer = 0;
     int n;
     if ( fd == d->proc->socketStdout ) {
 	buffer = &d->bufStdout;
@@ -1105,15 +1190,22 @@ void QProcess::socketRead( int fd )
 	// this case should never happen, but just to be safe
 	return;
     }
+#if defined(QT_QPROCESS_DEBUG)
+    uint oldSize = buffer->size();
+#endif
 
     // try to read data first (if it fails, the filedescriptor was closed)
-    oldSize = buffer->size();
-    buffer->resize( oldSize + bufsize );
-    n = ::read( fd, buffer->data()+oldSize, bufsize );
-    if ( n > 0 )
-	buffer->resize( oldSize + n );
-    else
-	buffer->resize( oldSize );
+    const int basize = 4096;
+    QByteArray *ba = new QByteArray( basize );
+    n = ::read( fd, ba->data(), basize );
+    if ( n > 0 ) {
+	ba->resize( n );
+	buffer->append( ba );
+	ba = 0;
+    } else {
+	delete ba;
+	ba = 0;
+    }
     // eof or error?
     if ( n == 0 || n == -1 ) {
 	if ( fd == d->proc->socketStdout ) {
@@ -1150,13 +1242,15 @@ void QProcess::socketRead( int fd )
 	FD_ZERO( &fds );
 	FD_SET( fd, &fds );
 	// read data
-	oldSize = buffer->size();
-	buffer->resize( oldSize + bufsize );
-	n = ::read( fd, buffer->data()+oldSize, bufsize );
+	ba = new QByteArray( basize );
+	n = ::read( fd, ba->data(), basize );
 	if ( n > 0 ) {
-	    buffer->resize( oldSize + n );
+	    ba->resize( n );
+	    buffer->append( ba );
+	    ba = 0;
 	} else {
-	    buffer->resize( oldSize );
+	    delete ba;
+	    ba = 0;
 	    break;
 	}
     }
@@ -1185,26 +1279,26 @@ void QProcess::socketRead( int fd )
 */
 void QProcess::socketWrite( int fd )
 {
-    if ( fd != d->proc->socketStdin || d->proc->socketStdin == 0 )
-	return;
-    if ( d->stdinBuf.isEmpty() ) {
-	d->notifierStdin->setEnabled( FALSE );
-	return;
-    }
+    while ( fd == d->proc->socketStdin && d->proc->socketStdin != 0 ) {
+	if ( d->stdinBuf.isEmpty() ) {
+	    d->notifierStdin->setEnabled( FALSE );
+	    return;
+	}
+	ssize_t ret = ::write( fd,
+		d->stdinBuf.head()->data() + d->stdinBufRead,
+		d->stdinBuf.head()->size() - d->stdinBufRead );
 #if defined(QT_QPROCESS_DEBUG)
-    qDebug( "QProcess::socketWrite(): write to stdin (%d)", fd );
+	qDebug( "QProcess::socketWrite(): wrote %d bytes to stdin (%d)", ret, fd );
 #endif
-    ssize_t ret = ::write( fd,
-	    d->stdinBuf.head()->data() + d->stdinBufRead,
-	    d->stdinBuf.head()->size() - d->stdinBufRead );
-    if ( ret > 0 )
+	if ( ret == -1 )
+	    return;
 	d->stdinBufRead += ret;
-    if ( d->stdinBufRead == (ssize_t)d->stdinBuf.head()->size() ) {
-	d->stdinBufRead = 0;
-	delete d->stdinBuf.dequeue();
-	if ( wroteToStdinConnected && d->stdinBuf.isEmpty() )
-	    emit wroteToStdin();
-	socketWrite( fd );
+	if ( d->stdinBufRead == (ssize_t)d->stdinBuf.head()->size() ) {
+	    d->stdinBufRead = 0;
+	    delete d->stdinBuf.dequeue();
+	    if ( wroteToStdinConnected && d->stdinBuf.isEmpty() )
+		emit wroteToStdin();
+	}
     }
 }
 
