@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id:  qt/qregion.cpp   3.0.5   edited Apr 26 11:52 $
+** $Id: qt/qregion.cpp   3.2.3   edited May 13 09:08 $
 **
 ** Implementation of QRegion class
 **
@@ -83,8 +83,9 @@
 
     QRegion is an \link shclass.html implicitly shared\endlink class.
 
-    Due to window system limitations, the width and height of a region
-    is limited to 65535 on Unix/X11.
+    \warning Due to window system limitations, the whole coordinate
+    space for a region is limited to the points between -32767 and
+    32767 on Windows 95/98/ME.
 
     \sa QPainter::setClipRegion(), QPainter::setClipRect()
 */
