@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id:  qt/qsizegrip.cpp   3.0.5   edited Apr 29 09:56 $
+** $Id: qt/qsizegrip.cpp   3.2.3   edited May 13 09:08 $
 **
 ** Implementation of QSizeGrip class
 **
@@ -44,11 +44,13 @@
 #include "qstyle.h"
 
 #if defined(Q_WS_X11)
-#include "qt_x11.h"
+#include "qt_x11_p.h"
 extern Atom qt_sizegrip;			// defined in qapplication_x11.cpp
 #elif defined (Q_WS_WIN )
 #include "qobjectlist.h"
 #include "qt_windows.h"
+#elif defined(Q_WS_MAC)
+bool qt_mac_update_sizer(QWidget *, int); //qwidget_mac.cpp
 #endif
 
 
@@ -107,10 +109,12 @@ QSizeGrip::QSizeGrip( QWidget * parent, const char* name )
     : QWidget( parent, name )
 {
 #ifndef QT_NO_CURSOR
+#ifndef Q_WS_MAC
     if ( QApplication::reverseLayout() )
 	setCursor( sizeBDiagCursor );
     else
 	setCursor( sizeFDiagCursor );
+#endif
 #endif
     setSizePolicy( QSizePolicy( QSizePolicy::Minimum, QSizePolicy::Fixed ) );
 #if defined(Q_WS_X11)
@@ -124,6 +128,7 @@ QSizeGrip::QSizeGrip( QWidget * parent, const char* name )
     tlw = qt_sizegrip_topLevelWidget( this );
     if ( tlw )
 	tlw->installEventFilter( this );
+    installEventFilter( this ); //for binary compatibility fix in 4.0 with an event() ###
 }
 
 
@@ -147,7 +152,8 @@ QSizeGrip::~QSizeGrip()
 */
 QSize QSizeGrip::sizeHint() const
 {
-    return QSize( 13, 13 ).expandedTo( QApplication::globalStrut() );
+    return (style().sizeFromContents(QStyle::CT_SizeGrip, this, QSize(13, 13)).
+	    expandedTo(QApplication::globalStrut()));
 }
 
 /*!
@@ -236,7 +242,10 @@ bool QSizeGrip::eventFilter( QObject *o, QEvent *e )
 {
     if ( o == tlw ) {
 	switch ( e->type() ) {
+#ifndef Q_WS_MAC
+	/* The size grip goes no where on Mac OS X when you maximize!  --Sam */
 	case QEvent::ShowMaximized:
+#endif
 	case QEvent::ShowFullScreen:
 	    hide();
 	    break;
@@ -246,6 +255,22 @@ bool QSizeGrip::eventFilter( QObject *o, QEvent *e )
 	default:
 	    break;
 	}
+    } else if(o == this) {
+#if defined(Q_WS_MAC)
+	switch(e->type()) {
+	case QEvent::Hide:
+	case QEvent::Show:
+	    if(!QApplication::closingDown() && parentWidget() && !qt_sizegrip_workspace(this)) {
+		if(QWidget *w = qt_sizegrip_topLevelWidget(this)) {
+		    if(w->isTopLevel()) 
+			qt_mac_update_sizer(w, e->type() == QEvent::Hide ? -1 : 1);
+		}
+	    }
+	    break;
+	default:
+	    break;
+	}
+ #endif	    
     }
     return FALSE;
 }
