@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id:  qt/qsocketnotifier.cpp   3.0.5   edited Apr 29 10:05 $
+** $Id: qt/qsocketnotifier.cpp   3.2.3   edited Nov 7 16:23 $
 **
 ** Implementation of QSocketNotifier class
 **
@@ -36,10 +36,9 @@
 **********************************************************************/
 
 #include "qsocketnotifier.h"
+#include "qapplication.h"
 #include "qevent.h"
-
-
-extern bool qt_set_socket_handler( int, int, QObject *, bool );
+#include "qeventloop.h"
 
 
 /*!
@@ -59,6 +58,9 @@ extern bool qt_set_socket_handler( int, int, QObject *, bool );
     socket. Then you connect the activated() signal to the slot you
     want to be called when a socket event occurs.
 
+    Note for Windows users: the socket passed to QSocketNotifier will
+    become non-blocking, even if it was created as a blocking socket.
+
     There are three types of socket notifiers (read, write and
     exception); you must specify one of these in the constructor.
 
@@ -163,7 +165,7 @@ QSocketNotifier::QSocketNotifier( int socket, Type type, QObject *parent,
     sockfd = socket;
     sntype = type;
     snenabled = TRUE;
-    qt_set_socket_handler( sockfd, sntype, this, TRUE );
+    QApplication::eventLoop()->registerSocketNotifier( this );
 }
 
 /*!
@@ -246,7 +248,15 @@ void QSocketNotifier::setEnabled( bool enable )
     if ( snenabled == enable )			// no change
 	return;
     snenabled = enable;
-    qt_set_socket_handler( sockfd, sntype, this, snenabled );
+
+    QEventLoop *eventloop = QApplication::eventLoop();
+    if ( ! eventloop ) // perhaps application is shutting down
+	return;
+
+    if ( snenabled )
+	eventloop->registerSocketNotifier( this );
+    else
+	eventloop->unregisterSocketNotifier( this );
 }
 
 
