@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id:  qt/qsound_x11.cpp   3.0.5   edited Oct 12 2001 $
+** $Id: qt/qsound_x11.cpp   3.2.3   edited Oct 30 16:55 $
 **
 ** Implementation of QSound class and QAuServer internal class
 **
@@ -55,15 +55,18 @@ static AuServer *nas=0;
 
 class QAuBucketNAS : public QAuBucket {
 public:
-    QAuBucketNAS(int i) : id(i) {}
+    QAuBucketNAS(AuBucketID b, AuFlowID f) : id(b), flow(f) { }
 
     ~QAuBucketNAS()
     {
-	if ( nas )
-	    AuDestroyBucket( nas, (AuBucketID)id, NULL );
+	if ( nas ) {
+            AuDestroyFlow(nas, flow, NULL);
+	    AuDestroyBucket(nas, id, NULL);
+        }
     }
 
     AuBucketID id;
+    AuFlowID flow;
 };
 
 class QAuServerNAS : public QAuServer {
@@ -80,9 +83,11 @@ public:
     void play(QSound*);
     void stop(QSound*);
     bool okay();
+    void setDone(QSound*);
 
 public slots:
     void dataReceived();
+    void soundDestroyed(QObject *o);
 
 private:
     QAuBucketNAS* bucket(QSound* s)
@@ -114,6 +119,17 @@ QAuServerNAS::~QAuServerNAS()
     nas = 0;
 }
 
+static QPtrDict<void> *inprogress=0;
+
+void QAuServerNAS::soundDestroyed(QObject *o)
+{
+    if (inprogress) {
+	QSound *so = static_cast<QSound *>(o);
+	while (inprogress->remove(so))
+	    ; // Loop while remove returns TRUE
+    }
+}
+
 void QAuServerNAS::play(const QString& filename)
 {
     if (nas) {
@@ -127,30 +143,40 @@ void QAuServerNAS::play(const QString& filename)
     }
 }
 
-static QPtrDict<void> *inprogress=0;
-
 static void callback( AuServer*, AuEventHandlerRec*, AuEvent* e, AuPointer p)
 {
-qDebug("Event %d",e->type);
     if ( inprogress->find(p) && e ) {
 	if (e->type==AuEventTypeElementNotify &&
 		    e->auelementnotify.kind==AuElementNotifyKindState) {
 	    if ( e->auelementnotify.cur_state == AuStateStop )
-		qDebug("DONE");
+                ((QAuServerNAS*)inprogress->find(p))->setDone((QSound*)p);
 	}
     }
 }
 
+void QAuServerNAS::setDone(QSound* s)
+{
+    if (nas) {
+        decLoop(s);
+        if (s->loopsRemaining()) {
+            play(s);
+        } else {
+            inprogress->remove(s);
+        }
+    }
+}
+
 void QAuServerNAS::play(QSound* s)
 {
     if (nas) {
 	if ( !inprogress )
 	    inprogress = new QPtrDict<void>;
-	inprogress->insert(s,(void*)1);
+	inprogress->insert(s,(void*)this);
 	int iv=100;
 	AuFixedPoint volume=AuFixedPointFromFraction(iv,100);
-	AuSoundPlayFromBucket(nas, bucket(s)->id, AuNone, volume,
-		callback, s, 0, NULL, NULL, NULL, NULL);
+        QAuBucketNAS *b = bucket(s);
+        AuSoundPlayFromBucket(nas, b->id, AuNone, volume,
+                              callback, s, 0, &b->flow, NULL, NULL, NULL);
 	AuFlush(nas);
 	dataReceived();
 	AuFlush(nas);
@@ -161,18 +187,26 @@ void QAuServerNAS::play(QSound* s)
 void QAuServerNAS::stop(QSound* s)
 {
     if (nas) {
-	inprogress->remove(s);
-
-	// #######
+        AuStopFlow(nas, bucket(s)->flow, NULL);
+        AuFlush(nas);
+	dataReceived();
+	AuFlush(nas);
+	qApp->flushX();
     }
 }
 
 void QAuServerNAS::init(QSound* s)
 {
-    if ( nas )
-	setBucket(s,
-	    new QAuBucketNAS(AuSoundCreateBucketFromFile(nas, s->fileName(),
-		0 /*AuAccessAllMasks*/, NULL, NULL)));
+    connect(s, SIGNAL(destroyed(QObject *)),
+	    this, SLOT(soundDestroyed(QObject *)));
+
+    if ( nas ) {
+        AuBucketID b_id =
+            AuSoundCreateBucketFromFile(nas, s->fileName(),
+                                        0 /*AuAccessAllMasks*/, NULL, NULL);
+        AuFlowID f_id = AuCreateFlow(nas, NULL);
+	setBucket(s, new QAuBucketNAS(b_id, f_id));
+    }
 }
 
 bool QAuServerNAS::okay()
@@ -195,7 +229,7 @@ public:
     QAuServerNull(QObject* parent);
 
     void play(const QString&) { }
-    void play(QSound*) { }
+    void play(QSound*s) { while(decLoop(s) > 0) /* nothing */ ; }
     void stop(QSound*) { }
     bool okay() { return FALSE; }
 };
