@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id$
+** $Id: qt/qtranslator.cpp   3.2.3   edited May 13 09:08 $
 **
 ** Localization database support.
 **
 ** Created : 980906
 **
-** Copyright (C) 1998-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1998-2003 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the kernel module of the Qt GUI Toolkit.
 **
@@ -35,42 +35,26 @@
 **
 **********************************************************************/
 
-// needed for bsearch (at least for Borland)
 #include "qplatformdefs.h"
 
+// POSIX Large File Support redefines open -> open64
+static inline int qt_open(const char *pathname, int flags, mode_t mode)
+{ return ::open(pathname, flags, mode); }
+#if defined(open)
+# undef open
+#endif
+
+// POSIX Large File Support redefines truncate -> truncate64
+#if defined(truncate)
+# undef truncate
+#endif
+
 #include "qtranslator.h"
 
 #ifndef QT_NO_TRANSLATION
 
 #include "qfileinfo.h"
-
-#if defined(Q_OS_UNIX)
-#define QT_USE_MMAP
-#endif
-
-#if defined(QT_USE_MMAP)
-
-// for mmap
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <sys/mman.h>
-#include <fcntl.h>
-#include <errno.h>
-
-// for htonl
-#include <netinet/in.h>
-
-// for close
-#include <unistd.h>
-
-#else
-// appropriate stuff here
-#endif
-
-// for qsort
-#include <stdlib.h>
-
-// other qt stuff necessary for the implementation
+#include "qwidgetlist.h"
 #include "qintdict.h"
 #include "qstring.h"
 #include "qapplication.h"
@@ -80,25 +64,42 @@
 #include "qmap.h"
 #include "qtl.h"
 
+#if defined(Q_OS_UNIX)
+#define QT_USE_MMAP
+#endif
+
+// most of the headers below are already included in qplatformdefs.h
+// also this lacks Large File support but that's probably irrelevant
+#if defined(QT_USE_MMAP)
+// for mmap
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <sys/mman.h>
+#include <fcntl.h>
+#include <errno.h>
+// for htonl
+#include <netinet/in.h>
+#endif
+
+#include <stdlib.h>
+
 /*
 $ mcookie
 3cb86418caef9c95cd211cbf60a1bddd
 $
 */
 
-static const int magic_length = 16;
-static const uchar magic[magic_length] = { // magic number for the file
+// magic number for the file
+static const int MagicLength = 16;
+static const uchar magic[MagicLength] = {
     0x3c, 0xb8, 0x64, 0x18, 0xca, 0xef, 0x9c, 0x95,
     0xcd, 0x21, 0x1c, 0xbf, 0x60, 0xa1, 0xbd, 0xdd
 };
 
 static bool match( const char* found, const char* target )
 {
-    /*
-      Here arises the subtle distinction between a null string (means
-      "any") and an empty string (means empty).
-    */
-    return found == 0 || qstrcmp(found, target) == 0;
+    // 0 means anything, "" means empty
+    return found == 0 || qstrcmp( found, target ) == 0;
 }
 
 #if defined(Q_C_CALLBACKS)
@@ -106,9 +107,9 @@ extern "C" {
 #endif
 
 /*
-  Yes, unfortunately, we have code here that depends on endianness.  The
-  candidate is big endian (it comes from a .qm file) whereas the target
-  endianness depends on the system Qt is running on.
+  Yes, unfortunately, we have code here that depends on endianness.
+  The candidate is big endian (it comes from a .qm file) whereas the
+  target endianness depends on the system Qt is running on.
 */
 #ifdef Q_OS_TEMP
 static int __cdecl cmp_uint32_little( const void* target, const void* candidate )
@@ -152,53 +153,52 @@ static uint elfHash( const char * name )
     uint g;
 
     if ( name ) {
-	k = (const uchar*)name;
+	k = (const uchar *) name;
 	while ( *k ) {
-	    h = (h << 4) + *k++;
+	    h = ( h << 4 ) + *k++;
 	    if ( (g = (h & 0xf0000000)) != 0 )
 		h ^= g >> 24;
 	    h &= ~g;
 	}
     }
-
     if ( !h )
 	h = 1;
-
     return h;
 }
 
+extern bool qt_detectRTLLanguage();
 
 class QTranslatorPrivate {
 public:
     struct Offset {
-	Offset() { h=0; o=0; }
+	Offset()
+	    : h( 0 ), o( 0 ) { }
 	Offset( const QTranslatorMessage& m, int offset )
-	{ h = m.hash(); o = offset; } // ### qChecksum
-
-	bool operator<( const Offset&k ) const { return ( h != k.h )
-							     ? h < k.h
-							     : o < k.o; }
+	    : h( m.hash() ), o( offset ) { }
 
+	bool operator<( const Offset&k ) const {
+	    return ( h != k.h ) ? h < k.h : o < k.o;
+	}
+	Q_DUMMY_COMPARISON_OPERATOR(QTranslatorPrivate::Offset)
 	uint h;
 	uint o;
     };
 
-    enum { Contexts = 0x2f, Hashes = 0x42, Messages = 0x69 } Tag;
+    enum { Contexts = 0x2f, Hashes = 0x42, Messages = 0x69 };
 
     QTranslatorPrivate() :
 	unmapPointer( 0 ), unmapLength( 0 ),
 	messageArray( 0 ), offsetArray( 0 ), contextArray( 0 )
 #ifndef QT_NO_TRANSLATION_BUILDER
-	,messages( 0 )
+	, messages( 0 )
 #endif
     { }
-    // QTranslator must finalize this before deallocating it.
+    // QTranslator must finalize this before deallocating it
 
     // for mmap'ed files, this is what needs to be unmapped.
     char * unmapPointer;
     unsigned int unmapLength;
 
-
     // for squeezed but non-file data, this is what needs to be deleted
     QByteArray * messageArray;
     QByteArray * offsetArray;
@@ -247,6 +247,8 @@ public:
 	return app.exec();
     }
     \endcode
+    Note that the translator must be created \e before the
+    application's main window.
 
     Most applications will never need to do anything else with this
     class. The other functions provided by this class are useful for
@@ -452,7 +454,6 @@ bool QTranslator::load( const QString & filename, const QString & directory,
     // realname is now the fully qualified name of a readable file.
 
 #if defined(QT_USE_MMAP)
-    // unix (if mmap supported)
 
 #ifndef MAP_FILE
 #define MAP_FILE 0
@@ -461,11 +462,9 @@ bool QTranslator::load( const QString & filename, const QString & directory,
 #define MAP_FAILED -1
 #endif
 
-    //const char * lang = getenv( "LANG" );
-
     int f;
 
-    f = ::open( QFile::encodeName(realname), O_RDONLY );
+    f = qt_open( QFile::encodeName(realname), O_RDONLY, 0666 );
     if ( f < 0 ) {
 	// qDebug( "can't open %s: %s", realname.ascii(), strerror( errno ) );
 	return FALSE;
@@ -483,7 +482,6 @@ bool QTranslator::load( const QString & filename, const QString & directory,
 		       f, 0 ); // from offset 0 of f
     if ( !tmp || tmp == (char*)MAP_FAILED ) {
 	// qDebug( "can't mmap %s: %s", filename.ascii(), strerror( errno ) );
-	// #### could revert to file io?
 	return FALSE;
     }
 
@@ -492,12 +490,11 @@ bool QTranslator::load( const QString & filename, const QString & directory,
     d->unmapPointer = tmp;
     d->unmapLength = st.st_size;
 #else
-    // windows, or unix without mmap
-    QFile f(realname);
+    QFile f( realname );
     if ( !f.exists() )
 	return FALSE;
     d->unmapLength = f.size();
-    d->unmapPointer = new char[d->unmapLength]; // ### really not
+    d->unmapPointer = new char[d->unmapLength];
     bool ok = FALSE;
     if ( f.open(IO_ReadOnly) ) {
 	ok = d->unmapLength ==
@@ -511,48 +508,71 @@ bool QTranslator::load( const QString & filename, const QString & directory,
     }
 #endif
 
-    // now that we've read it and all, check that it has the right
-    // magic number, and forget all about it if it doesn't.
-    if ( memcmp( (const void *)(d->unmapPointer), magic, magic_length ) ) {
+    return do_load( (const uchar *) d->unmapPointer, d->unmapLength );
+}
+
+/*!
+  \overload
+  \fn bool QTranslator::load( const uchar *data, int len )
+
+  Loads the .qm file data \a data of length \a len into the
+  translator.
+
+  The data is not copied. The caller must be able to guarantee that \a data
+  will not be deleted or modified.
+*/
+
+bool QTranslator::do_load( const uchar *data, int len )
+{
+    if ( len < MagicLength || memcmp( data, magic, MagicLength ) != 0 ) {
 	clear();
 	return FALSE;
     }
-    // prepare to read.
-    QByteArray tmpArray;
-    tmpArray.setRawData( d->unmapPointer, d->unmapLength );
-    QDataStream s( tmpArray, IO_ReadOnly );
-    if ( !s.device()->at(magic_length) )
-	return FALSE;
 
-    // read.
+    QByteArray array;
+    array.setRawData( (const char *) data, len );
+    QDataStream s( array, IO_ReadOnly );
+    bool ok = TRUE;
+
+    s.device()->at( MagicLength );
+
     Q_UINT8 tag = 0;
-    Q_UINT32 length = 0;
-    s >> tag >> length;
-    while ( tag && length ) {
+    Q_UINT32 blockLen = 0;
+    s >> tag >> blockLen;
+    while ( tag && blockLen ) {
+	if ( (Q_UINT32) s.device()->at() + blockLen > (Q_UINT32) len ) {
+	    ok = FALSE;
+	    break;
+	}
+
 	if ( tag == QTranslatorPrivate::Contexts && !d->contextArray ) {
 	    d->contextArray = new QByteArray;
-	    d->contextArray->setRawData( tmpArray.data()+s.device()->at(),
-					 length );
+	    d->contextArray->setRawData( array.data() + s.device()->at(),
+					 blockLen );
 	} else if ( tag == QTranslatorPrivate::Hashes && !d->offsetArray ) {
 	    d->offsetArray = new QByteArray;
-	    d->offsetArray->setRawData( tmpArray.data()+s.device()->at(),
-					length );
+	    d->offsetArray->setRawData( array.data() + s.device()->at(),
+					blockLen );
 	} else if ( tag == QTranslatorPrivate::Messages && !d->messageArray ) {
 	    d->messageArray = new QByteArray;
-	    d->messageArray->setRawData( tmpArray.data()+s.device()->at(),
-					 length );
+	    d->messageArray->setRawData( array.data() + s.device()->at(),
+					 blockLen );
 	}
-	if ( !s.device()->at(s.device()->at() + length) )
-	    return FALSE;
-	tag = 0;
-	length = 0;
-	if ( !s.atEnd() )
-	    s >> tag >> length;
-    }
 
-    tmpArray.resetRawData( d->unmapPointer, d->unmapLength );
-    QApplication::setReverseLayout( qt_detectRTLLanguage() );
-    return TRUE;
+	if ( !s.device()->at(s.device()->at() + blockLen) ) {
+	    ok = FALSE;
+	    break;
+	}
+	tag = 0;
+	blockLen = 0;
+	if ( !s.atEnd() )
+	    s >> tag >> blockLen;
+    }
+    array.resetRawData( (const char *) data, len );
+
+    if ( qApp && qApp->translators && qApp->translators->contains(this) )
+	qApp->setReverseLayout( qt_detectRTLLanguage() );
+    return ok;
 }
 
 #ifndef QT_NO_TRANSLATION_BUILDER
@@ -574,7 +594,7 @@ bool QTranslator::save( const QString & filename, SaveMode mode )
 	squeeze( mode );
 
 	QDataStream s( &f );
-	s.writeRawBytes( (const char *)magic, magic_length );
+	s.writeRawBytes( (const char *)magic, MagicLength );
 	Q_UINT8 tag;
 
 	if ( d->offsetArray != 0 ) {
@@ -610,12 +630,6 @@ bool QTranslator::save( const QString & filename, SaveMode mode )
 
 void QTranslator::clear()
 {
-    bool wasUsed = d->unmapPointer ||
-		   d->unmapLength ||
-		   d->messageArray ||
-		   d->offsetArray ||
-		   d->contextArray;
-
     if ( d->unmapPointer && d->unmapLength ) {
 #if defined(QT_USE_MMAP)
 	munmap( d->unmapPointer, d->unmapLength );
@@ -624,29 +638,43 @@ void QTranslator::clear()
 #endif
 	d->unmapPointer = 0;
 	d->unmapLength = 0;
-	if ( d->messageArray )
-	    d->messageArray->resetRawData( d->messageArray->data(),
-					   d->messageArray->size() );
-	if ( d->offsetArray )
-	    d->offsetArray->resetRawData( d->offsetArray->data(),
-					  d->offsetArray->size() );
-	if ( d->contextArray )
-	    d->contextArray->resetRawData( d->contextArray->data(),
-					   d->contextArray->size() );
     }
-    delete d->messageArray;
-    d->messageArray = 0;
-    delete d->offsetArray;
-    d->offsetArray = 0;
-    delete d->contextArray;
-    d->contextArray = 0;
+
+    if ( d->messageArray ) {
+	d->messageArray->resetRawData( d->messageArray->data(),
+				       d->messageArray->size() );
+	delete d->messageArray;
+	d->messageArray = 0;
+    }
+    if ( d->offsetArray ) {
+	d->offsetArray->resetRawData( d->offsetArray->data(),
+				      d->offsetArray->size() );
+	delete d->offsetArray;
+	d->offsetArray = 0;
+    }
+    if ( d->contextArray ) {
+	d->contextArray->resetRawData( d->contextArray->data(),
+				       d->contextArray->size() );
+	delete d->contextArray;
+	d->contextArray = 0;
+    }
 #ifndef QT_NO_TRANSLATION_BUILDER
     delete d->messages;
     d->messages = 0;
 #endif
 
-    if ( wasUsed )
-	QApplication::setReverseLayout( qt_detectRTLLanguage() );
+    if ( qApp && qApp->loopLevel() ) {
+	qApp->setReverseLayout( qt_detectRTLLanguage() );
+
+	QWidgetList *list = QApplication::topLevelWidgets();
+	QWidgetListIt it( *list );
+	QWidget *w;
+	while ( ( w=it.current() ) != 0 ) {
+	    ++it;
+	    qApp->postEvent( w, new QEvent( QEvent::LanguageChange ) );
+	}
+	delete list;
+    }
 }
 
 #ifndef QT_NO_TRANSLATION_BUILDER
@@ -731,8 +759,9 @@ void QTranslator::squeeze( SaveMode mode )
 	}
 
 	/*
-	  The contexts found in this translator are stored in a hash table to
-	  provide fast look-up.  The context array has the following format:
+	  The contexts found in this translator are stored in a hash
+	  table to provide fast lookup. The context array has the
+	  following format:
 
 	      Q_UINT16 hTableSize;
 	      Q_UINT16 hTable[hTableSize];
@@ -743,12 +772,13 @@ void QTranslator::squeeze( SaveMode mode )
 	      Q_UINT8  len;
 	      Q_UINT8  data[len];
 
-	  Let's consider the look-up of context "FunnyDialog".  A hash value
-	  between 0 and hTableSize - 1 is computed, say h.  If hTable[h] is 0,
-	  "FunnyDialog" is not covered by this translator.  Else, we check in
-	  the contextPool at offset 2 * hTable[h] to see if "FunnyDialog" is one
-	  of the contexts stored there, until we find it or we meet the empty
-	  string.
+	  Let's consider the look-up of context "FunnyDialog".  A
+	  hash value between 0 and hTableSize - 1 is computed, say h.
+	  If hTable[h] is 0, "FunnyDialog" is not covered by this
+	  translator. Else, we check in the contextPool at offset
+	  2 * hTable[h] to see if "FunnyDialog" is one of the
+	  contexts stored there, until we find it or we meet the
+	  empty string.
 	*/
 	d->contextArray = new QByteArray;
 	d->contextArray->resize( 2 + (hTableSize << 1) );
@@ -831,13 +861,13 @@ void QTranslator::unsqueeze()
 
     This function works with stripped translator files.
 
-    (This is is a one-liner that calls find().)
+    (This is is a one-liner that calls findMessage().)
 */
 
 bool QTranslator::contains( const char* context, const char* sourceText,
 			    const char* comment ) const
 {
-    return findMessage( context, sourceText, comment ).translation() != QString::null;
+    return !findMessage( context, sourceText, comment ).translation().isNull();
 }
 
 
@@ -899,9 +929,9 @@ void QTranslator::remove( const QTranslatorMessage& message )
   \a comment) or QString::null if there is none in this translator.
 */
 
-/*!
-    Returns the QTranslatorMessage for the key (\a context, \a
-    sourceText, \a comment).
+/*!  Returns the QTranslatorMessage for the key
+     (\a context, \a sourceText, \a comment). If none is found,
+     also tries (\a context, \a sourceText, "").
 */
 
 QTranslatorMessage QTranslator::findMessage( const char* context,
@@ -917,12 +947,20 @@ QTranslatorMessage QTranslator::findMessage( const char* context,
 
 #ifndef QT_NO_TRANSLATION_BUILDER
     if ( d->messages ) {
-	QMap<QTranslatorMessage, void *>::ConstIterator it
-	    = d->messages->find( QTranslatorMessage(context, sourceText,
-				 comment) );
-	if ( it == d->messages->end() )
-	    return QTranslatorMessage();
-	return it.key();
+	QMap<QTranslatorMessage, void *>::ConstIterator it;
+
+	it = d->messages->find( QTranslatorMessage(context, sourceText,
+						   comment) );
+	if ( it != d->messages->end() )
+	    return it.key();
+
+	if ( comment[0] ) {
+	    it = d->messages->find( QTranslatorMessage(context, sourceText,
+						       "") );
+	    if ( it != d->messages->end() )
+		return it.key();
+	}
+	return QTranslatorMessage();
     }
 #endif
 
@@ -958,49 +996,63 @@ QTranslatorMessage QTranslator::findMessage( const char* context,
 	}
     }
 
-    Q_UINT32 h = elfHash( QCString(sourceText) + comment );
-
-    Q_UINT32 rh;
-    Q_UINT32 ro;
-
     size_t numItems = d->offsetArray->size() / ( 2 * sizeof(Q_UINT32) );
     if ( !numItems )
 	return QTranslatorMessage();
 
     if ( systemWordSize == 0 )
 	qSysInfo( &systemWordSize, &systemBigEndian );
-    char *r = (char *) bsearch( &h, d->offsetArray->data(), numItems,
-				2 * sizeof(Q_UINT32),
-				systemBigEndian ? cmp_uint32_big
-				: cmp_uint32_little );
-    if ( r == 0 )
-	return QTranslatorMessage();
 
-    // go back on equal key
-    while ( r != d->offsetArray->data() && cmp_uint32_big(r - 8, r) == 0 )
-	r -= 8;
+    for ( ;; ) {
+	Q_UINT32 h = elfHash( QCString(sourceText) + comment );
 
-    QDataStream s( *d->offsetArray, IO_ReadOnly );
-    s.device()->at( r - d->offsetArray->data() );
+	char *r = (char *) bsearch( &h, d->offsetArray->data(), numItems,
+				    2 * sizeof(Q_UINT32),
+				    systemBigEndian ? cmp_uint32_big
+				    : cmp_uint32_little );
+	if ( r != 0 ) {
+	    // go back on equal key
+	    while ( r != d->offsetArray->data() &&
+		    cmp_uint32_big(r - 8, r) == 0 )
+		r -= 8;
 
-    s >> rh >> ro;
+	    QDataStream s( *d->offsetArray, IO_ReadOnly );
+	    s.device()->at( r - d->offsetArray->data() );
 
-    QDataStream ms( *d->messageArray, IO_ReadOnly );
-    while ( rh == h ) {
-	ms.device()->at( ro );
-	QTranslatorMessage m( ms );
-	if ( match(m.context(), context)
-		&& match(m.sourceText(), sourceText)
-		&& match(m.comment(), comment) )
-	    return m;
-	if ( s.atEnd() )
-	    return QTranslatorMessage();
-	s >> rh >> ro;
+	    Q_UINT32 rh, ro;
+	    s >> rh >> ro;
+
+	    QDataStream ms( *d->messageArray, IO_ReadOnly );
+	    while ( rh == h ) {
+		ms.device()->at( ro );
+		QTranslatorMessage m( ms );
+		if ( match(m.context(), context)
+			&& match(m.sourceText(), sourceText)
+			&& match(m.comment(), comment) )
+		    return m;
+		if ( s.atEnd() )
+		    break;
+		s >> rh >> ro;
+	    }
+	}
+	if ( !comment[0] )
+	    break;
+	comment = "";
     }
-
     return QTranslatorMessage();
 }
 
+/*!
+    Returns TRUE if this translator is empty, otherwise returns FALSE.
+    This function works with stripped and unstripped translation files.
+*/
+bool QTranslator::isEmpty() const
+{
+    return !( d->unmapPointer || d->unmapLength || d->messageArray ||
+	      d->offsetArray || d->contextArray );
+}
+
+
 #ifndef QT_NO_TRANSLATION_BUILDER
 
 /*!
@@ -1013,29 +1065,23 @@ QTranslatorMessage QTranslator::findMessage( const char* context,
     \code
     QValueList<QTranslatorMessage> list = myTranslator.messages();
     QValueList<QTranslatorMessage>::Iterator it = list.begin();
-    while( it != list.end() ) {
-	myProcessing( *it );
+    while ( it != list.end() ) {
+	process_message( *it );
 	++it;
     }
-    \endcode
+  \endcode
 */
 
 QValueList<QTranslatorMessage> QTranslator::messages() const
 {
     ((QTranslator *) this)->unsqueeze();
-    QValueList<QTranslatorMessage> result;
-    QMap<QTranslatorMessage, void *>::ConstIterator it;
-    for ( it = d->messages->begin(); it != d->messages->end(); ++it )
-	result.append( it.key() );
-    return result;
+    return d->messages->keys();
 }
 
 #endif
 
 /*!
-    \class QTranslatorMessage qtranslator.h
-
-    \preliminary
+    \class QTranslatorMessage
 
     \brief The QTranslatorMessage class contains a translator message and its
     properties.
@@ -1084,9 +1130,7 @@ QTranslatorMessage::QTranslatorMessage( const char * context,
 					const QString& translation )
     : cx( context ), st( sourceText ), cm( comment ), tn( translation )
 {
-    /*
-      0 means we don't know, "" means we know it's empty.
-    */
+    // 0 means we don't know, "" means empty
     if ( cx == (const char*)0 )
 	cx = "";
     if ( st == (const char*)0 )
