@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id:  qt/qurlinfo.cpp   3.0.5   edited Jun 13 14:24 $
+** $Id: qt/qurlinfo.cpp   3.2.3   edited May 13 09:08 $
 **
 ** Implementation of QUrlInfo class
 **
 ** Created : 950429
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2002 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the kernel module of the Qt GUI Toolkit.
 **
@@ -41,6 +41,7 @@
 
 #include "qurloperator.h"
 #include "qdir.h"
+#include <limits.h>
 
 class QUrlInfoPrivate
 {
@@ -60,7 +61,11 @@ public:
     int permissions;
     QString owner;
     QString group;
+#if defined(QT_LARGEFILE_SUPPORT)
+    QIODevice::Offset size;
+#else
     uint size;
+#endif
     QDateTime lastModified;
     QDateTime lastRead;
     bool isDir;
@@ -92,6 +97,23 @@ public:
     isReadable() and isExecutable().
 */
 
+/*!
+    \enum QUrlInfo::PermissionSpec
+
+    This enum is used by the permissions() function to report the
+    permissions of a file.
+
+    \value ReadOwner The file is readable by the owner of the file.
+    \value WriteOwner The file is writable by the owner of the file.
+    \value ExeOwner The file is executable by the owner of the file.
+    \value ReadGroup The file is readable by the group.
+    \value WriteGroup The file is writable by the group.
+    \value ExeGroup The file is executable by the group.
+    \value ReadOther The file is readable by anyone.
+    \value WriteOther The file is writable by anyone.
+    \value ExeOther The file is executable by anyone.
+*/
+
 /*!
     Constructs an invalid QUrlInfo object with default values.
 
@@ -160,10 +182,17 @@ QUrlInfo::QUrlInfo( const QUrlInfo &ui )
     isExecutable.
 */
 
+#if defined(QT_ABI_QT4)
+QUrlInfo::QUrlInfo( const QString &name, int permissions, const QString &owner,
+		    const QString &group, QIODevice::Offset size, const QDateTime &lastModified,
+		    const QDateTime &lastRead, bool isDir, bool isFile, bool isSymLink,
+		    bool isWritable, bool isReadable, bool isExecutable )
+#else
 QUrlInfo::QUrlInfo( const QString &name, int permissions, const QString &owner,
 		    const QString &group, uint size, const QDateTime &lastModified,
 		    const QDateTime &lastRead, bool isDir, bool isFile, bool isSymLink,
 		    bool isWritable, bool isReadable, bool isExecutable )
+#endif
 {
     d = new QUrlInfoPrivate;
     d->name = name;
@@ -194,10 +223,17 @@ QUrlInfo::QUrlInfo( const QString &name, int permissions, const QString &owner,
     isExecutable.
 */
 
+#if defined(QT_ABI_QT4)
+QUrlInfo::QUrlInfo( const QUrl &url, int permissions, const QString &owner,
+		    const QString &group, QIODevice::Offset size, const QDateTime &lastModified,
+		    const QDateTime &lastRead, bool isDir, bool isFile, bool isSymLink,
+		    bool isWritable, bool isReadable, bool isExecutable )
+#else
 QUrlInfo::QUrlInfo( const QUrl &url, int permissions, const QString &owner,
 		    const QString &group, uint size, const QDateTime &lastModified,
 		    const QDateTime &lastRead, bool isDir, bool isFile, bool isSymLink,
 		    bool isWritable, bool isReadable, bool isExecutable )
+#endif
 {
     d = new QUrlInfoPrivate;
     d->name = QFileInfo( url.path() ).fileName();
@@ -360,7 +396,7 @@ void QUrlInfo::setGroup( const QString &s )
 }
 
 /*!
-    Specifies that the URL has size \a s.
+    Specifies the \a size of the URL.
 
     If you call this function for an invalid URL info, this function
     turns it into a valid one.
@@ -368,11 +404,15 @@ void QUrlInfo::setGroup( const QString &s )
     \sa isValid()
 */
 
-void QUrlInfo::setSize( uint s )
+#if defined(QT_ABI_QT4)
+void QUrlInfo::setSize( QIODevice::Offset size )
+#else
+void QUrlInfo::setSize( uint size )
+#endif
 {
     if ( !d )
 	d = new QUrlInfoPrivate;
-    d->size = s;
+    d->size = size;
 }
 
 
@@ -454,7 +494,8 @@ QString QUrlInfo::name() const
 }
 
 /*!
-    Returns the permissions of the URL.
+    Returns the permissions of the URL. You can use the \c PermissionSpec flags
+    to test for certain permissions.
 
     \sa isValid()
 */
@@ -498,11 +539,19 @@ QString QUrlInfo::group() const
     \sa isValid()
 */
 
+#if defined(QT_ABI_QT4)
+QIODevice::Offset QUrlInfo::size() const
+#else
 uint QUrlInfo::size() const
+#endif
 {
     if ( !d )
 	return 0;
+#if defined(QT_LARGEFILE_SUPPORT) && !defined(QT_ABI_QT4)
+    return d->size > UINT_MAX ? UINT_MAX : (uint)d->size;
+#else
     return d->size;
+#endif
 }
 
 /*!
