@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id:  qt/qurloperator.cpp   3.0.5   edited Jul 1 11:52 $
+** $Id: qt/qurloperator.cpp   3.2.3   edited Aug 12 16:22 $
 **
 ** Implementation of QUrlOperator class
 **
@@ -89,6 +89,9 @@ public:
     \class QUrlOperator qurloperator.h
 
     \brief The QUrlOperator class provides common operations on URLs.
+\if defined(commercial)
+    It is part of the <a href="commercialeditions.html">Qt Enterprise Edition</a>.
+\endif
 
     \ingroup io
     \ingroup misc
@@ -595,6 +598,8 @@ QPtrList<QNetworkOperation> QUrlOperator::copy( const QString &from, const QStri
 		 this, SLOT( continueCopy( QNetworkOperation * ) ) );
 	connect( gProt, SIGNAL( finished( QNetworkOperation * ) ),
 		 this, SIGNAL( finished( QNetworkOperation * ) ) );
+	connect( gProt, SIGNAL( connectionStateChanged( int, const QString & ) ),
+		 this, SIGNAL( connectionStateChanged( int, const QString & ) ) );
 
 	connect( pProt, SIGNAL( dataTransferProgress( int, int, QNetworkOperation * ) ),
 		 this, SIGNAL( dataTransferProgress( int, int, QNetworkOperation * ) ) );
@@ -880,7 +885,8 @@ void QUrlOperator::addEntry( const QValueList<QUrlInfo> &i )
 /*!
     Returns the URL information for the child \a entry, or returns an
     empty QUrlInfo object if there is no information available about
-    \a entry.
+    \a entry. Information about \a entry is only available after a successfully
+    finished listChildren() operation.
 */
 
 QUrlInfo QUrlOperator::info( const QString &entry ) const
@@ -1119,7 +1125,6 @@ void QUrlOperator::finishedCopy()
     qDebug( "QUrlOperator: finished copy (finished putting)" );
 #endif
 
-    d->currPut = 0;
     if ( d->waitingCopies.isEmpty() )
 	return;
 
