@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/qwmatrix.cpp   3.2.1   edited May 13 09:08 $
+** $Id: qt/qwmatrix.cpp   3.2.3   edited Oct 13 17:17 $
 **
 ** Implementation of QWMatrix class
 **
@@ -137,7 +137,7 @@ double qsincos( double, bool calcCos );		// defined in qpainter_x11.cpp
 	double a    = pi/180 * 25;         // convert 25 to radians
 	double sina = sin(a);
 	double cosa = cos(a);
-	QWMatrix m1(0, 0, 0, 0, 10, -20);  // translation matrix
+	QWMatrix m1(1, 0, 0, 1, 10, -20);  // translation matrix
 	QWMatrix m2( cosa, sina,           // rotation matrix
 		    -sina, cosa, 0, 0 );
 	QWMatrix m3(1.2, 0, 0, 0.7, 0, 0); // scaling matrix
@@ -727,7 +727,7 @@ QRegion QWMatrix::operator * (const QRegion &r ) const
 	if ( _m12 == 0.0F && _m21 == 0.0F ) {
 	    // simple case, no rotation
 	    while ( i ) {
-		*rect = QRect( map(rect->topLeft()), map(rect->bottomRight()) );
+		*rect = QRect( map(rect->topLeft()), map(rect->bottomRight()) ).normalize();
 		rect++;
 		i--;
 	    }
