@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id:  qt/qnetwork.cpp   3.0.5   edited Oct 12 2001 $
+** $Id: qt/qnetwork.cpp   3.2.3   edited May 13 09:08 $
 **
 ** Implementation of qInitNetworkProtocols function.
 **
@@ -47,12 +47,13 @@
 
 /*! \file qnetwork.h */
 /*!
-  \relates QUrlOperator
+    \relates QUrlOperator
 
-  This function registers the network protocols for FTP and HTTP. You have to
-  call this function before you use QUrlOperator for these protocols.
+    This function registers the network protocols for FTP and HTTP.
+    You must call this function before you use QUrlOperator for
+    these protocols.
 
-  This function is declared in \l qnetwork.h.
+    This function is declared in \l qnetwork.h.
 */
 void qInitNetworkProtocols()
 {
@@ -64,4 +65,4 @@ void qInitNetworkProtocols()
 #endif
 }
 
-#endif // QT_NO_NETWORKPROTOCOL
+#endif // QT_NO_NETWORK
