@@ -99,7 +99,8 @@ public:
     QStringList fld;
     QStringList fldLabel;
     QValueList<int> fldWidth;
-    QValueList< QIconSet > fldIcon;
+    QValueList<QIconSet> fldIcon;
+    QValueList<bool> fldHidden;
     QSqlCursorManager cur;
     QDataManager dat;
 };
@@ -163,7 +164,7 @@ void qt_debug_buffer( const QString& msg, QSqlRecord* cursor )
     the edit buffer to the database until the user moves to a
     different record in the grid or presses Enter. Cell editing is
     initiated by pressing F2 (or right clicking and then clicking the
-    appropriate popup menu item) and cancelled by pressing Esc. If
+    appropriate popup menu item) and canceled by pressing Esc. If
     there is a problem updating or adding data, errors are handled
     automatically (see handleError() to change this behavior). Note
     that if autoEdit() is FALSE navigating to another record will
@@ -273,8 +274,8 @@ void QDataTable::init()
     d->falseTxt = tr( "False" );
     d->datefmt = Qt::LocalDate;
     reset();
-    connect( this, SIGNAL( currentChanged( int, int ) ),
-	     SLOT( setCurrentSelection( int, int )));
+    connect( this, SIGNAL( selectionChanged() ),
+	     SLOT( updateCurrentSelection()));
 }
 
 /*!
@@ -308,6 +309,7 @@ void QDataTable::addColumn( const QString& fieldName,
     d->fldLabel += label;
     d->fldIcon += iconset;
     d->fldWidth += width;
+    d->fldHidden += FALSE;
 }
 
 /*!
@@ -331,6 +333,7 @@ void QDataTable::setColumn( uint col, const QString& fieldName,
     d->fldLabel[col] = label;
     d->fldIcon[col] = iconset;
     d->fldWidth[col] = width;
+    d->fldHidden[col] = FALSE;
 }
 
 /*!
@@ -347,6 +350,7 @@ void QDataTable::removeColumn( uint col )
 	d->fldLabel.remove( d->fldLabel.at( col ) );
 	d->fldIcon.remove( d->fldIcon.at( col ) );
 	d->fldWidth.remove( d->fldWidth.at( col ) );
+	d->fldHidden.remove( d->fldHidden.at( col ) );
     }
 }
 
@@ -1149,7 +1153,6 @@ bool QDataTable::updateCurrent()
 	    emit cursorChanged( QSql::Update );
 	    refresh();
 	    QSqlIndex idx = sqlCursor()->primaryIndex();
-	    findBuffer( idx, d->lastAt );
 	    endUpdate();
 	}
 	break;
@@ -1377,6 +1380,7 @@ void QDataTable::reset()
     d->fldLabel.clear();
     d->fldWidth.clear();
     d->fldIcon.clear();
+    d->fldHidden.clear();
     if ( sorting() )
 	horizontalHeader()->setSortIndicator( -1 );
 }
@@ -1844,6 +1848,7 @@ void QDataTable::setSqlCursor( QSqlCursor* cursor, bool autoPopulate, bool autoD
 	    d->fldLabel.clear();
 	    d->fldWidth.clear();
 	    d->fldIcon.clear();
+	    d->fldHidden.clear();
 	    for ( uint i = 0; i < sqlCursor()->count(); ++i ) {
 		addColumn( sqlCursor()->field( i )->name(), sqlCursor()->field( i )->name() );
 		setColumnReadOnly( i, sqlCursor()->field( i )->isReadOnly() );
@@ -1872,6 +1877,29 @@ void QDataTable::handleError( const QSqlError& e )
     d->dat.handleError( this, e );
 }
 
+/*! \reimp
+  */
+
+void QDataTable::keyPressEvent( QKeyEvent* e )
+{
+    switch( e->key() ) {
+    case Key_Left:
+    case Key_Right:
+    case Key_Up:
+    case Key_Down:
+    case Key_Prior:
+    case Key_Next:
+    case Key_Home:
+    case Key_End:
+    case Key_F2:
+    case Key_Enter: case Key_Return:
+    case Key_Tab: case Key_BackTab:
+	QTable::keyPressEvent( e );
+    default:
+	return;
+    }
+}
+
 /*!  \reimp
 */
 
@@ -1971,6 +1999,11 @@ void QDataTable::setCurrentSelection( int row, int )
     emit currentChanged( sqlCursor() );
 }
 
+void QDataTable::updateCurrentSelection()
+{
+    setCurrentSelection( currentRow(), -1 );
+}
+
 /*!
     Returns the currently selected record, or 0 if there is no current
     selection. The table owns the pointer, so do \e not delete it or
@@ -2061,10 +2094,13 @@ void QDataTable::refresh( QDataTable::Refresh mode )
 		    setNumCols( numCols() + 1 );
 		    d->colIndex.append( fpos );
 		    setColumnReadOnly( numCols()-1, field->isReadOnly() || isColumnReadOnly( numCols()-1 ) );
-		    QHeader* h = horizontalHeader();
-		    QString label = d->fldLabel[ i ];
-		    QIconSet icons = d->fldIcon[ i ];
-		    h->setLabel( numCols()-1, icons, label );
+		    horizontalHeader()->setLabel( numCols()-1, d->fldIcon[ i ], d->fldLabel[ i ] );
+		    if ( d->fldHidden[ i ] ) {
+			QTable::showColumn( i ); // ugly but necessary
+			QTable::hideColumn( i );
+		    } else {
+			QTable::showColumn( i );
+		    }
 		    if ( d->fldWidth[ i ] > -1 )
 			QTable::setColumnWidth( i, d->fldWidth[i] );
 		}
@@ -2074,13 +2110,15 @@ void QDataTable::refresh( QDataTable::Refresh mode )
     viewport()->setUpdatesEnabled( TRUE );
     viewport()->repaint( FALSE );
     horizontalHeader()->repaint();
+    verticalHeader()->repaint();
     setSize( cur );
     // keep others aware
-    if ( d->lastAt == -1 ) {
-	setCurrentSelection( -1, 0 );
-    } else if ( d->lastAt != currentRow() ) {
+    if ( d->lastAt == -1 )
+ 	setCurrentSelection( -1, -1 );
+    else if ( d->lastAt != currentRow() )
 	setCurrentSelection( currentRow(), currentColumn() );
-    }
+    if ( cur->isValid() )
+	emit currentChanged( sqlCursor() );
 }
 
 /*!
@@ -2182,6 +2220,26 @@ void QDataTable::drawContents( QPainter * p, int cx, int cy, int cw, int ch )
 	sqlCursor()->seek( currentRow() );
 }
 
+/*!
+    \reimp
+*/
+
+void QDataTable::hideColumn( int col )
+{
+    d->fldHidden[col] = TRUE;
+    refresh( RefreshColumns );
+}
+
+/*!
+    \reimp
+*/
+
+void QDataTable::showColumn( int col )
+{
+    d->fldHidden[col] = FALSE;
+    refresh( RefreshColumns );
+}
+
 /*!
     \fn void QDataTable::currentChanged( QSqlRecord* record )
 
