@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id:  qt/qcdestyle.cpp   3.0.5   edited Oct 12 2001 $
+** $Id: qt/qcdestyle.cpp   3.2.3   edited May 27 08:43 $
 **
 ** Implementation of CDE-like style class
 **
@@ -37,46 +37,45 @@
 
 #include "qcdestyle.h"
 
-#ifndef QT_NO_STYLE_CDE
+#if !defined(QT_NO_STYLE_CDE) || defined(QT_PLUGIN)
 
 #include "qpainter.h"
 #include "qdrawutil.h"
 #include "qbutton.h"
 #include <limits.h>
 
-// NOT REVISED
 /*!
-  \class QCDEStyle qcdestyle.h
-  \brief The QCDEStyle class provides a CDE look and feel.
+    \class QCDEStyle qcdestyle.h
+    \brief The QCDEStyle class provides a CDE look and feel.
 
-  \ingroup appearance
+    \ingroup appearance
 
-  This style provides a slightly improved Motif look similar to some
-  versions of the Common Desktop Environment (CDE). The main
-  differences are thinner frames and more modern radio buttons and
-  check boxes. Together with a dark background and a bright
-  text/foreground color, the style looks quite attractive (at least
-  for Motif fans).
-
-  Note that the functions provided by QCDEStyle are reimplementations
-  of QStyle functions; see QStyle for their documentation.
+    This style provides a slightly improved Motif look similar to some
+    versions of the Common Desktop Environment (CDE). The main
+    differences are thinner frames and more modern radio buttons and
+    checkboxes. Together with a dark background and a bright
+    text/foreground color, the style looks quite attractive (at least
+    for Motif fans).
 
+    Note that the functions provided by QCDEStyle are
+    reimplementations of QStyle functions; see QStyle for their
+    documentation.
 */
 
 /*!
     Constructs a QCDEStyle.
 
-    If \a useHighlightCols is FALSE (the default), then the style
-    will polish the application's color palette to emulate the Motif
-    way of highlighting, which is a simple inversion between the base
-    and the text color.
+    If \a useHighlightCols is FALSE (the default), then the style will
+    polish the application's color palette to emulate the Motif way of
+    highlighting, which is a simple inversion between the base and the
+    text color.
 */
 QCDEStyle::QCDEStyle( bool useHighlightCols ) : QMotifStyle( useHighlightCols )
 {
 }
 
 /*!
-  Destroys the style.
+    Destroys the style.
 */
 QCDEStyle::~QCDEStyle()
 {
