@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id:  qt/qcompactstyle.cpp   3.0.5   edited Mar 27 11:14 $
+** $Id: qt/qcompactstyle.cpp   3.2.3   edited May 27 08:44 $
 **
 ** Implementation of compact style class
 **
@@ -37,7 +37,7 @@
 
 #include "qcompactstyle.h"
 
-#ifndef QT_NO_STYLE_COMPACT
+#if !defined(QT_NO_STYLE_COMPACT) || defined(QT_PLUGIN)
 
 #include "qfontmetrics.h"
 #include "qpalette.h"
@@ -156,7 +156,7 @@ void QCompactStyle::drawControl( ControlElement element, QPainter *p, const QWid
 
 	    int tab = opt.tabWidth();
 	    int maxpmw = opt.maxIconWidth();
-	    bool dis = ! mi->isEnabled();
+	    bool dis = !(flags & Style_Enabled);
 	    bool checkable = popupmenu->isCheckable();
 	    bool act = flags & Style_Active;
 	    int x, y, w, h;
@@ -303,7 +303,7 @@ void QCompactStyle::drawControl( ControlElement element, QPainter *p, const QWid
 				  g2, Style_Enabled);
 		} else {
 		    drawPrimitive(PE_ArrowRight, p, QRect(x+w - motifArrowHMargin - motifItemFrame - dim, y + h / 2 - dim / 2, dim, dim),
-				  g, mi->isEnabled() ? Style_Enabled : Style_Default);
+				  g, !dis ? Style_Enabled : Style_Default);
 		}
 	    }
 	}
