@@ -1,9 +1,9 @@
 /****************************************************************************
-** $Id:  qt/qmotifplusstyle.cpp   3.0.5   edited May 27 22:19 $
+** $Id: qt/qmotifplusstyle.cpp   3.2.3   edited May 27 08:45 $
 **
 ** Implementation of QMotifPlusStyle class
 **
-** Created : 2000.07.27
+** Created : 000727
 **
 ** Copyright (C) 2000 Trolltech AS.  All rights reserved.
 **
@@ -37,7 +37,7 @@
 
 #include "qmotifplusstyle.h"
 
-#ifndef QT_NO_STYLE_MOTIFPLUS
+#if !defined(QT_NO_STYLE_MOTIFPLUS) || defined(QT_PLUGIN)
 
 #include "qmenubar.h"
 #include "qapplication.h"
@@ -54,6 +54,7 @@
 #include "qdrawutil.h"
 #include "qscrollbar.h"
 #include "qtabbar.h"
+#include "qtoolbar.h"
 #include "qguardedptr.h"
 #include "qlayout.h"
 
@@ -128,22 +129,23 @@ static void drawMotifPlusShade(QPainter *p,
 
 
 /*!
-  \class QMotifPlusStyle qmotifplusstyle.h
-  \brief The QMotifPlusStyle class provides a more sophisticated Motif-ish look and feel.
-  \ingroup appearance
+    \class QMotifPlusStyle qmotifplusstyle.h
+    \brief The QMotifPlusStyle class provides a more sophisticated Motif-ish look and feel.
 
- This class implements a Motif-ish look and feel with more
- sophisticated bevelling as used by the GIMP Toolkit (GTK+) for
- Unix/X11.
+    \ingroup appearance
+
+    This class implements a Motif-ish look and feel with the more
+    sophisticated bevelling as used by the GIMP Toolkit (GTK+) for
+    Unix/X11.
 */
 
 /*!
-  Constructs a QMotifPlusStyle
+    Constructs a QMotifPlusStyle
 
-  If \a hoveringHighlight is TRUE (the default), then the style will not
-  highlight push buttons, checkboxes, radiobuttons, comboboxes, scrollbars
-  or sliders.
- */
+    If \a hoveringHighlight is TRUE (the default), then the style will
+    not highlight push buttons, checkboxes, radiobuttons, comboboxes,
+    scrollbars or sliders.
+*/
 QMotifPlusStyle::QMotifPlusStyle(bool hoveringHighlight) : QMotifStyle(TRUE)
 {
     if ( !singleton )
@@ -174,28 +176,24 @@ void QMotifPlusStyle::polish(QPalette &)
 void QMotifPlusStyle::polish(QWidget *widget)
 {
 #ifndef QT_NO_FRAME
-    if (widget->inherits("QFrame") &&
-        ((QFrame *) widget)->frameStyle() == QFrame::Panel)
+    if (::qt_cast<QFrame*>(widget) && ((QFrame *) widget)->frameStyle() == QFrame::Panel)
         ((QFrame *) widget)->setFrameStyle(QFrame::WinPanel);
 #endif
 
 #ifndef QT_NO_MENUBAR
-    if (widget->inherits("QMenuBar") &&
-        ((QMenuBar *) widget)->frameStyle() != QFrame::NoFrame)
+    if (::qt_cast<QMenuBar*>(widget) && ((QMenuBar *) widget)->frameStyle() != QFrame::NoFrame)
         ((QMenuBar *) widget)->setFrameStyle(QFrame::StyledPanel | QFrame::Raised);
 #endif
 
 #ifndef QT_NO_TOOLBAR
-    if (widget->inherits("QToolBar"))
+    if (::qt_cast<QToolBar*>(widget))
         widget->layout()->setMargin(2);
 #endif
     if (useHoveringHighlight) {
-	if (widget->inherits("QButton") ||
-	    widget->inherits("QComboBox"))
+	if (::qt_cast<QButton*>(widget) || ::qt_cast<QComboBox*>(widget))
 	    widget->installEventFilter(this);
 
-	if (widget->inherits("QScrollBar") ||
-	    widget->inherits("QSlider")) {
+	if (::qt_cast<QScrollBar*>(widget) || ::qt_cast<QSlider*>(widget)) {
 	    widget->setMouseTracking(TRUE);
 	    widget->installEventFilter(this);
 	}
@@ -671,7 +669,11 @@ void QMotifPlusStyle::drawControl( ControlElement element,
 
 	    if (flags & Style_HasFocus)
 		br.addCoords(1, 1, -1, -1);
+	    p->save();
+	    p->setBrushOrigin( -button->backgroundOffset().x(),
+			       -button->backgroundOffset().y() );
 	    drawPrimitive(PE_ButtonCommand, p, br, cg, flags);
+	    p->restore();
 #endif
 	    break;
 	}
@@ -879,9 +881,9 @@ void QMotifPlusStyle::drawControl( ControlElement element,
 		vrect = visualRect( QRect(x + w - hh - 6, y + (hh / 2), hh, hh), r );
 		drawPrimitive(arrow, p,
 			      vrect, cg,
-			      ((act && mi->isEnabled()) ?
+			      ((act && !dis) ?
 			       Style_Down : Style_Default) |
-			      ((mi->isEnabled()) ? Style_Enabled : Style_Default));
+			      ((!dis) ? Style_Enabled : Style_Default));
 	    }
 	    break;
 	}
@@ -1339,7 +1341,7 @@ void QMotifPlusStyle::drawComplexControl(ComplexControl control,
 		drawMotifPlusShade(p, groove, cg, TRUE, FALSE,
 				   &cg.brush(QColorGroup::Mid));
 
-		if ( slider->hasFocus() ) {
+		if ( flags & Style_HasFocus ) {
 		    QRect fr = subRect( SR_SliderFocusRect, widget );
 		    drawPrimitive( PE_FocusRect, p, fr, cg, flags );
 		}
@@ -1387,11 +1389,8 @@ QRect QMotifPlusStyle::querySubControlMetrics(ComplexControl control,
 					      SubControl subcontrol,
 					      const QStyleOption& opt) const
 {
-    QRect rect;
-
     switch (control) {
-    case CC_SpinWidget:
-	{
+    case CC_SpinWidget: {
 	    int fw = pixelMetric( PM_SpinBoxFrameWidth, 0 );
 	    QSize bs;
 	    bs.setHeight( (widget->height() + 1)/2 );
@@ -1408,109 +1407,78 @@ QRect QMotifPlusStyle::querySubControlMetrics(ComplexControl control,
 
 	    switch ( subcontrol ) {
 	    case SC_SpinWidgetUp:
-		rect.setRect(x, y, bs.width(), bs.height());
-		rect.addCoords(1, 0, 0, -1);
-		break;
+		return QRect(x + 1, y, bs.width(), bs.height() - 1);
 	    case SC_SpinWidgetDown:
-		rect.setRect(x, y + bs.height(), bs.width(), bs.height());
-		rect.addCoords(1, 1, 0, 0);
-		break;
+		return QRect(x + 1, y + bs.height() + 1, bs.width(), bs.height());
 	    case SC_SpinWidgetButtonField:
-		rect.setRect(x, y, bs.width(), h - 2*fw);
-		break;
+		return QRect(x, y, bs.width(), h - 2*fw);
 	    case SC_SpinWidgetEditField:
-		rect.setRect(lx, fw, rx, h - 2*fw);
-		break;
+		return QRect(lx, fw, rx, h - 2*fw);
 	    case SC_SpinWidgetFrame:
-		rect.setRect( widget->x(), widget->y(),
-			      widget->width() - bs.width(), h);
+		return QRect( 0, 0, widget->width() - bs.width(), h);
 	    default:
 		break;
 	    }
-	    break;
-	}
+	    break; }
 
-    case CC_ComboBox:
-	{
 #ifndef QT_NO_COMBOBOX
-	    const QComboBox *combobox = (const QComboBox *) widget;
-
-	    if (combobox->editable()) {
-		int space = (combobox->height() - 13) / 2;
-
-		switch (subcontrol) {
-		case SC_ComboBoxEditField:
-		    rect = widget->rect();
-		    rect.setWidth(rect.width() - 13 - space * 2);
-		    rect.addCoords(3, 3, -3, -3);
-		    break;
-
-		case SC_ComboBoxArrow:
-		    rect.setRect(combobox->width() - 13 - space * 2, 0,
-				 13 + space * 2, combobox->height());
-		    break;
-
-		default:
-		    // shouldn't get here
-		    break;
-		}
-
-	    } else {
-		int space = (combobox->height() - 7) / 2;
-
-		switch (subcontrol) {
-		case SC_ComboBoxEditField:
-		    rect = widget->rect();
-		    rect.addCoords(3, 3, -3, -3);
-		    break;
-
-		case SC_ComboBoxArrow:
-		    // 12 wide, 7 tall
-		    rect.setRect(combobox->width() - 12 - space,
-				 space, 12, 7);
-		    break;
-
-		default:
-		    // shouldn't get here
-		    break;
-		}
+    case CC_ComboBox: {
+	const QComboBox *combobox = (const QComboBox *) widget;
+	if (combobox->editable()) {
+	    int space = (combobox->height() - 13) / 2;
+	    switch (subcontrol) {
+	    case SC_ComboBoxFrame:
+		return QRect();
+	    case SC_ComboBoxEditField: {
+		QRect rect = widget->rect();
+		rect.setWidth(rect.width() - 13 - space * 2);
+		rect.addCoords(3, 3, -3, -3);
+		return rect; }
+	    case SC_ComboBoxArrow:
+		return QRect(combobox->width() - 13 - space * 2, 0,
+			     13 + space * 2, combobox->height());
+	    default: break;		// shouldn't get here
 	    }
-#endif
-	    break;
-	}
 
-    case CC_Slider:
-	{
+	} else {
+	    int space = (combobox->height() - 7) / 2;
+	    switch (subcontrol) {
+	    case SC_ComboBoxFrame:
+		return QRect();
+	    case SC_ComboBoxEditField: {
+		QRect rect = widget->rect();
+		rect.addCoords(3, 3, -3, -3);
+		return rect; }
+	    case SC_ComboBoxArrow:		// 12 wide, 7 tall
+		return QRect(combobox->width() - 12 - space, space, 12, 7);
+	    default: break;		// shouldn't get here
+	    }
+	}
+	break; }
+#endif
+
 #ifndef QT_NO_SLIDER
-	    if (subcontrol == SC_SliderHandle) {
-		const QSlider *slider = (const QSlider *) widget;
-		int tickOffset  = pixelMetric( PM_SliderTickmarkOffset, widget );
-		int thickness   = pixelMetric( PM_SliderControlThickness, widget );
-		int len         = pixelMetric( PM_SliderLength, widget ) + 2;
-		int sliderPos   = slider->sliderStart();
-		int motifBorder = 2;
+    case CC_Slider: {
 
-		if ( slider->orientation() == Horizontal )
-		    rect.setRect( sliderPos + motifBorder,
-				  tickOffset + motifBorder, len,
-				  thickness - 2*motifBorder );
-		else
-		    rect.setRect( tickOffset + motifBorder,
-				  sliderPos + motifBorder,
-				  thickness - 2*motifBorder, len);
-	    } else
-		rect = QMotifStyle::querySubControlMetrics(control, widget,
-							   subcontrol, opt);
-#endif
-	    break;
+	if (subcontrol == SC_SliderHandle) {
+	    const QSlider *slider = (const QSlider *) widget;
+	    int tickOffset  = pixelMetric( PM_SliderTickmarkOffset, widget );
+	    int thickness   = pixelMetric( PM_SliderControlThickness, widget );
+	    int len         = pixelMetric( PM_SliderLength, widget ) + 2;
+	    int sliderPos   = slider->sliderStart();
+	    int motifBorder = 2;
+
+	    if ( slider->orientation() == Horizontal )
+		return QRect( sliderPos + motifBorder, tickOffset + motifBorder, len,
+			      thickness - 2*motifBorder );
+	    return QRect( tickOffset + motifBorder, sliderPos + motifBorder,
+			  thickness - 2*motifBorder, len);
 	}
-
-    default:
-	rect = QMotifStyle::querySubControlMetrics(control, widget, subcontrol, opt);
-	break;
+	break; }
+#endif
+    default: break;
     }
-
-    return rect;
+    return QMotifStyle::querySubControlMetrics(control, widget, subcontrol, opt);
 }
 
 
@@ -1522,7 +1490,7 @@ bool QMotifPlusStyle::eventFilter(QObject *object, QEvent *event)
         {
 	    singleton->mousePressed = TRUE;
 
-            if (! object->inherits("QSlider"))
+            if (!::qt_cast<QSlider*>(object))
 		break;
 
 	    singleton->sliderActive = TRUE;
@@ -1533,7 +1501,7 @@ bool QMotifPlusStyle::eventFilter(QObject *object, QEvent *event)
         {
 	    singleton->mousePressed = FALSE;
 
-            if (! object->inherits("QSlider"))
+            if (!::qt_cast<QSlider*>(object))
 		break;
 
 	    singleton->sliderActive = FALSE;
@@ -1570,7 +1538,7 @@ bool QMotifPlusStyle::eventFilter(QObject *object, QEvent *event)
 	    if (! object->isWidgetType() || object != singleton->hoverWidget)
 		break;
 
-	    if (! object->inherits("QScrollBar") && ! object->inherits("QSlider"))
+	    if (!::qt_cast<QScrollBar*>(object) && ! ::qt_cast<QSlider*>(object))
 		break;
 
 	    singleton->mousePos = ((QMouseEvent *) event)->pos();
