@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id:  qt/qmotifstyle.cpp   3.0.5   edited Jun 12 16:42 $
+** $Id: qt/qmotifstyle.cpp   3.2.3   edited May 27 08:45 $
 **
 ** Implementation of Motif-like style class
 **
@@ -37,7 +37,7 @@
 
 #include "qmotifstyle.h"
 
-#ifndef QT_NO_STYLE_MOTIF
+#if !defined(QT_NO_STYLE_MOTIF) || defined(QT_PLUGIN)
 
 #include "qpopupmenu.h"
 #include "qapplication.h"
@@ -74,14 +74,15 @@ static const int motifCheckMarkSpace    = 12;
 
 
 /*!
-  \class QMotifStyle qmotifstyle.h
-  \brief The QMotifStyle class provides Motif look and feel.
-  \ingroup appearance
+    \class QMotifStyle qmotifstyle.h
+    \brief The QMotifStyle class provides Motif look and feel.
 
-  This class implements the Motif look and feel. It closely resembles
-  the original Motif look as defined by the Open Group, with the
-  addition of some minor improvements. The Motif style is Qt's default
-  GUI style on UNIX platforms.
+    \ingroup appearance
+
+    This class implements the Motif look and feel. It closely
+    resembles the original Motif look as defined by the Open Group,
+    but with some minor improvements. The Motif style is Qt's default
+    GUI style on UNIX platforms.
 */
 
 /*!
@@ -104,27 +105,27 @@ QMotifStyle::~QMotifStyle()
 }
 
 /*!
-  If \a arg is FALSE, the style will polish the
-  application's color palette to emulate the
-  Motif way of highlighting, which is a simple inversion between the
-  base and the text color.
+    If \a arg is FALSE, the style will polish the application's color
+    palette to emulate the Motif way of highlighting, which is a
+    simple inversion between the base and the text color.
 
-  The effect will show up the next time an application palette is set
-  via QApplication::setPalette(). The current color palette of the
-  application remains unchanged.
+    The effect will show up the next time an application palette is
+    set via QApplication::setPalette(). The current color palette of
+    the application remains unchanged.
 
-  \sa QStyle::polish()
- */
+    \sa QStyle::polish()
+*/
 void QMotifStyle::setUseHighlightColors( bool arg )
 {
     highlightCols = arg;
 }
 
 /*!
-  Returns TRUE if the style treats the highlight colors of the palette
-  in a Motif-like manner, which is a simple inversion between the base and the
-  text color; otherwise returns FALSE. The default is FALSE.
- */
+    Returns TRUE if the style treats the highlight colors of the
+    palette in a Motif-like manner, which is a simple inversion
+    between the base and the text color; otherwise returns FALSE. The
+    default is FALSE.
+*/
 bool QMotifStyle::useHighlightColors() const
 {
     return highlightCols;
@@ -206,6 +207,42 @@ void QMotifStyle::drawPrimitive( PrimitiveElement pe,
 				 const QStyleOption& opt ) const
 {
     switch( pe ) {
+#ifndef QT_NO_LISTVIEW
+    case PE_CheckListExclusiveIndicator: {
+	QCheckListItem *item = opt.checkListItem();
+	QListView *lv = item->listView();
+	if(!item)
+	    return;
+
+	if ( item->isEnabled() )
+	    p->setPen( QPen( cg.text() ) );
+	else
+	    p->setPen( QPen( lv->palette().color( QPalette::Disabled, QColorGroup::Text ) ) );
+	QPointArray a;
+
+	int cx = r.width()/2 - 1;
+	int cy = r.height()/2;
+	int e = r.width()/2 - 1;
+	for ( int i = 0; i < 3; i++ ) { //penWidth 2 doesn't quite work
+	    a.setPoints( 4, cx-e, cy, cx, cy-e,  cx+e, cy,  cx, cy+e );
+	    p->drawPolygon( a );
+	    e--;
+	}
+	if ( item->isOn() ) {
+	    if ( item->isEnabled() )
+		p->setPen( QPen( cg.text()) );
+	    else
+		p->setPen( QPen( item->listView()->palette().color( QPalette::Disabled,
+								    QColorGroup::Text ) ) );
+	    QBrush saveBrush = p->brush();
+	    p->setBrush( cg.text() );
+	    e = e - 2;
+	    a.setPoints( 4, cx-e, cy, cx, cy-e,  cx+e, cy,  cx, cy+e );
+	    p->drawPolygon( a );
+	    p->setBrush( saveBrush );
+	}
+	break; }
+#endif
     case PE_ButtonCommand:
     case PE_ButtonBevel:
     case PE_ButtonTool:
@@ -786,9 +823,13 @@ void QMotifStyle::drawControl( ControlElement element,
 		    flags |= Style_On;
 		if (btn->isDown())
 		    flags |= Style_Down;
+		p->save();
+		p->setBrushOrigin( -widget->backgroundOffset().x(),
+				   -widget->backgroundOffset().y() );
 		drawPrimitive( PE_ButtonCommand, p,
 			       tmp, newCg,
  			       flags );
+		p->restore();
 	    }
  	    if ( p->brush().style() != NoBrush )
  		p->setBrush( NoBrush );
@@ -804,7 +845,7 @@ void QMotifStyle::drawControl( ControlElement element,
 
 	    const QTabBar * tb = (const QTabBar *) widget;
 	    const QTab * t = opt.tab();
-	
+
 	    int dfw = pixelMetric( PM_DefaultFrameWidth, tb );
 	    bool selected = flags & Style_Selected;
 	    int o =  dfw > 1 ? 1 : 0;
@@ -1098,10 +1139,10 @@ void QMotifStyle::drawControl( ControlElement element,
 		if ( act )
 		    drawPrimitive(arrow, p, vr, cg,
 				  (Style_Down |
-				   (mi->isEnabled() ? Style_Enabled : Style_Default)));
+				  (dis ? Style_Default : Style_Enabled)) );
 		else
 		    drawPrimitive(arrow, p, vr, cg,
-				  (mi->isEnabled() ? Style_Enabled : Style_Default));
+				  (dis ? Style_Default : Style_Enabled));
 	    }
 
 	    break;
@@ -1217,7 +1258,7 @@ void QMotifStyle::drawComplexControl( ComplexControl control,
 				 &cg.brush( QColorGroup::Mid ) );
 
 
-		if ( slider->hasFocus() ) {
+		if ( flags & Style_HasFocus ) {
 		    QRect fr = subRect( SR_SliderFocusRect, widget );
 		    drawPrimitive( PE_FocusRect, p, fr, cg );
 		}
@@ -1386,7 +1427,7 @@ void QMotifStyle::drawComplexControl( ComplexControl control,
 
 		    // Expand line height to edge of rectangle if there's a
 		    // child, and it's visible
-		    if ( child && (child->height() > 0) ) { 
+		    if ( child && (child->height() > 0) ) {
 			linebot = r.height();
 		    }
 
@@ -1398,7 +1439,7 @@ void QMotifStyle::drawComplexControl( ComplexControl control,
 
 		int line; // index into dotlines
 		p->setPen( cg.text() );
-		for( line = 0; line < c; line += 2 ) {
+		if ( sub & SC_ListViewBranch ) for( line = 0; line < c; line += 2 ) {
 		    p->drawLine( dotlines[line].x(), dotlines[line].y(),
 				 dotlines[line+1].x(), dotlines[line+1].y() );
 		}
@@ -1508,194 +1549,170 @@ QRect QMotifStyle::querySubControlMetrics( ComplexControl control,
 					   SubControl sc,
 					   const QStyleOption& opt ) const
 {
-    QRect rect;
-
     switch ( control ) {
-    case CC_SpinWidget:
-	{
-	    if ( !widget )
-		break;
-	    int fw = pixelMetric( PM_SpinBoxFrameWidth, 0 );
-	    QSize bs;
-	    bs.setHeight( widget->height()/2 );
-	    if ( bs.height() < 8 )
-		bs.setHeight( 8 );
-	    bs.setWidth( QMIN( bs.height() * 8 / 5, widget->width() / 4 ) ); // 1.6 -approximate golden mean
-	    bs = bs.expandedTo( QApplication::globalStrut() );
-	    int y = 0;
-	    int x, lx, rx;
-	    x = widget->width() - y - bs.width();
-	    lx = fw;
-	    rx = x - fw * 2;
-
-	    switch ( sc ) {
-	    case SC_SpinWidgetUp:
-		rect.setRect(x, y, bs.width(), bs.height());
-		break;
-	    case SC_SpinWidgetDown:
-		rect.setRect(x, y + bs.height(), bs.width(), bs.height());
-		break;
-	    case SC_SpinWidgetButtonField:
-		rect.setRect(x, y, bs.width(), widget->height() - 2*fw);
-		break;
-	    case SC_SpinWidgetEditField:
-		rect.setRect(lx, fw, rx, widget->height() - 2*fw);
-		break;
-	    case SC_SpinWidgetFrame:
-		rect.setRect( widget->x(), widget->y(),
-			      widget->width() - bs.width(), widget->height() );
-	    default:
-		break;
-	    }
-	    break;
-	}
-
-    case CC_Slider:
-	{
-#ifndef QT_NO_SLIDER
-	    if (sc == SC_SliderHandle) {
-		const QSlider * sl = (const QSlider *) widget;
-		int tickOffset  = pixelMetric( PM_SliderTickmarkOffset, sl );
-		int thickness   = pixelMetric( PM_SliderControlThickness, sl );
-		int sliderPos   = sl->sliderStart();
-		int len         = pixelMetric( PM_SliderLength, sl );
-		int motifBorder = 3;
-
-		if ( sl->orientation() == Horizontal )
-		    rect.setRect( sliderPos + motifBorder,
-				  tickOffset + motifBorder, len,
-				  thickness - 2*motifBorder );
-		else
-		    rect.setRect( tickOffset + motifBorder,
-				  sliderPos + motifBorder,
-				  thickness - 2*motifBorder, len );
-	    } else
-		rect = QCommonStyle::querySubControlMetrics(control, widget, sc, opt);
-#endif
-	    break;
-	}
-
-    case CC_ScrollBar:
-	{
-#ifndef QT_NO_SCROLLBAR
-	    if (! widget)
-		break;
-
-	    const QScrollBar *scrollbar = (const QScrollBar *) widget;
-	    int sliderstart = scrollbar->sliderStart();
-	    int sbextent = pixelMetric(PM_ScrollBarExtent, widget);
-	    int fw = pixelMetric(PM_DefaultFrameWidth, widget);
-	    int buttonw = sbextent - (fw * 2);
-	    int maxlen = ((scrollbar->orientation() == Qt::Horizontal) ?
-			  scrollbar->width() : scrollbar->height()) -
-			 (buttonw * 2) - (fw * 2);
-	    int sliderlen;
-
-	    // calculate slider length
-	    if (scrollbar->maxValue() != scrollbar->minValue()) {
-		uint range = scrollbar->maxValue() - scrollbar->minValue();
-		sliderlen = (scrollbar->pageStep() * maxlen) /
-			    (range + scrollbar->pageStep());
-
-		if ( sliderlen < 9 || range > INT_MAX/2 )
-		    sliderlen = 9;
-		if ( sliderlen > maxlen )
-		    sliderlen = maxlen;
-	    } else
-		sliderlen = maxlen;
-
-	    switch (sc) {
-	    case SC_ScrollBarSubLine:
-		// top/left button
-		rect.setRect(fw, fw, buttonw, buttonw);
-		break;
-
-	    case SC_ScrollBarAddLine:
-		// bottom/right button
-		if (scrollbar->orientation() == Qt::Horizontal)
-		    rect.setRect(scrollbar->width() - sbextent + fw, fw,
-				 buttonw, buttonw);
-		else
-		    rect.setRect(fw, scrollbar->height() - sbextent + fw,
-				 buttonw, buttonw);
-		break;
-
-	    case SC_ScrollBarSubPage:
-		if (scrollbar->orientation() == Qt::Horizontal)
-		    rect.setRect(buttonw + fw, fw, sliderstart - buttonw - fw, buttonw);
-		else
-		    rect.setRect(fw, buttonw + fw, buttonw, sliderstart - buttonw - fw);
-		break;
-
- 	    case SC_ScrollBarAddPage:
- 		if (scrollbar->orientation() == Qt::Horizontal)
- 		    rect.setRect(sliderstart + sliderlen, fw,
-				 maxlen - sliderstart - sliderlen + buttonw + fw,
-				 buttonw);
- 		else
- 		    rect.setRect(fw, sliderstart + sliderlen, buttonw,
-				 maxlen - sliderstart - sliderlen + buttonw + fw);
- 		break;
-
-	    case SC_ScrollBarGroove:
-		if (scrollbar->orientation() == Qt::Horizontal)
-		    rect.setRect(buttonw + fw, fw, maxlen, buttonw);
-		else
-		    rect.setRect(fw, buttonw + fw, buttonw, maxlen);
- 		break;
-
- 	    case SC_ScrollBarSlider:
- 		if (scrollbar->orientation() == Qt::Horizontal)
- 		    rect.setRect(sliderstart, fw, sliderlen, buttonw);
- 		else
- 		    rect.setRect(fw, sliderstart, buttonw, sliderlen);
- 		break;
-
-	    default:
-		break;
-	    }
-#endif
-	    break;
-	}
-
-    case CC_ComboBox:
-#ifndef QT_NO_COMBOBOX
+    case CC_SpinWidget: {
+	if ( !widget )
+	    return QRect();
+	int fw = pixelMetric( PM_SpinBoxFrameWidth, 0 );
+	QSize bs;
+	bs.setHeight( widget->height()/2 );
+	if ( bs.height() < 8 )
+	    bs.setHeight( 8 );
+	bs.setWidth( QMIN( bs.height() * 8 / 5, widget->width() / 4 ) ); // 1.6 -approximate golden mean
+	bs = bs.expandedTo( QApplication::globalStrut() );
+	int y = 0;
+	int x, lx, rx;
+	x = widget->width() - y - bs.width();
+	lx = fw;
+	rx = x - fw * 2;
 	switch ( sc ) {
-	case SC_ComboBoxArrow:
-	    {
-		const QComboBox * cb = (const QComboBox *) widget;
-		int ew, awh, sh, dh, ax, ay, sy;
-		int fw = pixelMetric( PM_DefaultFrameWidth, cb );
-		QRect cr = cb->rect();
-		cr.addCoords( fw, fw, -fw, -fw );
-		get_combo_parameters( cr, ew, awh, ax, ay, sh, dh, sy );
-		rect.setRect( ax, ay, awh, awh );
-		break;
-	    }
+	case SC_SpinWidgetUp:
+	    return QRect(x, y, bs.width(), bs.height());
+	case SC_SpinWidgetDown:
+	    return QRect(x, y + bs.height(), bs.width(), bs.height());
+	case SC_SpinWidgetButtonField:
+	    return QRect(x, y, bs.width(), widget->height() - 2*fw);
+	case SC_SpinWidgetEditField:
+	    return QRect(lx, fw, rx, widget->height() - 2*fw);
+	case SC_SpinWidgetFrame:
+	    return QRect( 0, 0,
+			  widget->width() - bs.width(), widget->height() );
+	default:
+	    break;
+	}
+	break; }
 
-	case SC_ComboBoxEditField:
-	    {
-		const QComboBox * cb = (const QComboBox *) widget;
-		int fw = pixelMetric( PM_DefaultFrameWidth, cb );
-		rect = cb->rect();
-		rect.addCoords( fw, fw, -fw, -fw );
-		int ew = get_combo_extra_width( rect.height(), rect.width() );
+#ifndef QT_NO_SLIDER
+    case CC_Slider: {
+	if (sc == SC_SliderHandle) {
+	    const QSlider * sl = (const QSlider *) widget;
+	    int tickOffset  = pixelMetric( PM_SliderTickmarkOffset, sl );
+	    int thickness   = pixelMetric( PM_SliderControlThickness, sl );
+	    int sliderPos   = sl->sliderStart();
+	    int len         = pixelMetric( PM_SliderLength, sl );
+	    int motifBorder = 3;
 
- 		rect.addCoords( 1, 1, -1-ew, -1 );
-		break;
+	    if ( sl->orientation() == Horizontal )
+		return QRect( sliderPos + motifBorder, tickOffset + motifBorder, len,
+			      thickness - 2*motifBorder );
+	    return QRect( tickOffset + motifBorder, sliderPos + motifBorder,
+			  thickness - 2*motifBorder, len );
+	}
+	break; }
+#endif
+
+#ifndef QT_NO_SCROLLBAR
+    case CC_ScrollBar: {
+	if (! widget)
+	    return QRect();
+
+	const QScrollBar *scrollbar = (const QScrollBar *) widget;
+	int sliderstart = scrollbar->sliderStart();
+	int sbextent = pixelMetric(PM_ScrollBarExtent, widget);
+	int fw = pixelMetric(PM_DefaultFrameWidth, widget);
+	int buttonw = sbextent - (fw * 2);
+	int buttonh = sbextent - (fw * 2);
+	int maxlen = ((scrollbar->orientation() == Qt::Horizontal) ?
+		      scrollbar->width() : scrollbar->height()) -
+		     (buttonw * 2) - (fw * 2);
+	int sliderlen;
+
+	// calculate slider length
+	if (scrollbar->maxValue() != scrollbar->minValue()) {
+	    uint range = scrollbar->maxValue() - scrollbar->minValue();
+	    sliderlen = (scrollbar->pageStep() * maxlen) /
+			(range + scrollbar->pageStep());
+
+	    if ( sliderlen < 9 || range > INT_MAX/2 )
+		sliderlen = 9;
+	    if ( sliderlen > maxlen )
+		sliderlen = maxlen;
+	} else
+	    sliderlen = maxlen;
+
+	switch (sc) {
+	case SC_ScrollBarSubLine:
+	    // top/left button
+	    if (scrollbar->orientation() == Qt::Horizontal) {
+		if ( scrollbar->width()/2 < sbextent )
+		    buttonw = scrollbar->width()/2 - (fw*2);
+		return QRect(fw, fw, buttonw, buttonh);
+	    } else {
+		if ( scrollbar->height()/2 < sbextent )
+		    buttonh = scrollbar->height()/2 - (fw*2);
+		return QRect(fw, fw, buttonw, buttonh);
 	    }
+	case SC_ScrollBarAddLine:
+	    // bottom/right button
+	    if (scrollbar->orientation() == Qt::Horizontal) {
+		if ( scrollbar->width()/2 < sbextent )
+		    buttonw = scrollbar->width()/2 - (fw*2);
+		return QRect(scrollbar->width() - buttonw - fw, fw,
+			     buttonw, buttonh);
+	    } else {
+		if ( scrollbar->height()/2 < sbextent )
+		    buttonh = scrollbar->height()/2 - (fw*2);
+		return QRect(fw, scrollbar->height() - buttonh - fw,
+			     buttonw, buttonh);
+	    }
+	case SC_ScrollBarSubPage:
+	    if (scrollbar->orientation() == Qt::Horizontal)
+		return QRect(buttonw + fw, fw, sliderstart - buttonw - fw, buttonw);
+	    return QRect(fw, buttonw + fw, buttonw, sliderstart - buttonw - fw);
+
+	case SC_ScrollBarAddPage:
+	    if (scrollbar->orientation() == Qt::Horizontal)
+		return QRect(sliderstart + sliderlen, fw,
+			     maxlen - sliderstart - sliderlen + buttonw + fw, buttonw);
+	    return QRect(fw, sliderstart + sliderlen, buttonw,
+			 maxlen - sliderstart - sliderlen + buttonw + fw);
+
+	case SC_ScrollBarGroove:
+	    if (scrollbar->orientation() == Qt::Horizontal)
+		return QRect(buttonw + fw, fw, maxlen, buttonw);
+	    return QRect(fw, buttonw + fw, buttonw, maxlen);
+
+	case SC_ScrollBarSlider:
+	    if (scrollbar->orientation() == Qt::Horizontal)
+		return QRect(sliderstart, fw, sliderlen, buttonw);
+	    return QRect(fw, sliderstart, buttonw, sliderlen);
 
 	default:
 	    break;
 	}
+	break; }
 #endif
+
+#ifndef QT_NO_COMBOBOX
+    case CC_ComboBox:
+
+	switch ( sc ) {
+	case SC_ComboBoxArrow: {
+	    const QComboBox * cb = (const QComboBox *) widget;
+	    int ew, awh, sh, dh, ax, ay, sy;
+	    int fw = pixelMetric( PM_DefaultFrameWidth, cb );
+	    QRect cr = cb->rect();
+	    cr.addCoords( fw, fw, -fw, -fw );
+	    get_combo_parameters( cr, ew, awh, ax, ay, sh, dh, sy );
+	    return QRect( ax, ay, awh, awh ); }
+
+	case SC_ComboBoxEditField: {
+	    const QComboBox * cb = (const QComboBox *) widget;
+	    int fw = pixelMetric( PM_DefaultFrameWidth, cb );
+	    QRect rect = cb->rect();
+	    rect.addCoords( fw, fw, -fw, -fw );
+	    int ew = get_combo_extra_width( rect.height(), rect.width() );
+	    rect.addCoords( 1, 1, -1-ew, -1 );
+	    return rect; }
+
+	default:
+	    break;
+	}
 	break;
-
-    default:
-	return QCommonStyle::querySubControlMetrics( control, widget, sc, opt );
+#endif
+    default: break;
     }
-
-    return rect;
+    return QCommonStyle::querySubControlMetrics( control, widget, sc, opt );
 }
 
 /*!\reimp
@@ -1784,6 +1801,7 @@ QSize QMotifStyle::sizeFromContents( ContentsType contents,
 QRect QMotifStyle::subRect( SubRect r, const QWidget *widget ) const
 {
     QRect rect;
+    QRect wrect = widget->rect();
 
     switch ( r ) {
     case SR_SliderFocusRect:
@@ -1795,7 +1813,7 @@ QRect QMotifStyle::subRect( SubRect r, const QWidget *widget ) const
 	{
 	    int awh, ax, ay, sh, sy, dh, ew;
 	    int fw = pixelMetric( PM_DefaultFrameWidth, widget );
-	    QRect tr = widget->rect();
+	    QRect tr = wrect;
 
 	    tr.addCoords( fw, fw, -fw, -fw );
 	    get_combo_parameters( tr, ew, awh, ax, ay, sh, dh, sy );
@@ -1833,7 +1851,6 @@ QRect QMotifStyle::subRect( SubRect r, const QWidget *widget ) const
 	    if (progressbar->percentageVisible())
 		textw = fm.width("100%") + 6;
 
-	    QRect wrect = widget->rect();
 	    if (progressbar->indicatorFollowsStyle() ||
 		progressbar->centerIndicator())
 		rect = wrect;
@@ -1854,10 +1871,9 @@ QRect QMotifStyle::subRect( SubRect r, const QWidget *widget ) const
 	    if (progressbar->percentageVisible())
 		textw = fm.width("100%") + 6;
 
-	    QRect wrect = widget->rect();
 	    if (progressbar->indicatorFollowsStyle() ||
 		progressbar->centerIndicator())
-		rect = widget->rect();
+		rect = wrect;
 	    else
 		rect.setCoords(wrect.right() - textw, wrect.top(),
 			       wrect.right(), wrect.bottom());
@@ -1865,6 +1881,24 @@ QRect QMotifStyle::subRect( SubRect r, const QWidget *widget ) const
 	    break;
 	}
 
+    case SR_CheckBoxContents:
+	{
+#ifndef QT_NO_CHECKBOX
+	    QRect ir = subRect(SR_CheckBoxIndicator, widget);
+	    rect.setRect(ir.right() + 10, wrect.y(),
+			 wrect.width() - ir.width() - 10, wrect.height());
+#endif
+	    break;
+	}
+
+    case SR_RadioButtonContents:
+	{
+	    QRect ir = subRect(SR_RadioButtonIndicator, widget);
+	    rect.setRect(ir.right() + 10, wrect.y(),
+			 wrect.width() - ir.width() - 10, wrect.height());
+	    break;
+	}
+
     default:
 	rect = QCommonStyle::subRect( r, widget );
     }
@@ -2021,10 +2055,9 @@ static const char * dock_window_close_xpm[] = {
 
 // Message box icons, from page 210 of the Windows style guide.
 
-// Hand-drawn to resemble Microsoft's icons, but in the Mac/Netscape
-// palette.  The "question mark" icon, which Microsoft recommends not
-// using but a lot of people still use, is left out.
-
+// Hand-drawn to resemble Microsoft's icons, but in the Mac/Netscape palette.
+// Thanks to TrueColor displays, it is slightly more efficient to have
+// them duplicated.
 /* XPM */
 static const char * const information_xpm[]={
 "32 32 5 1",
@@ -2143,6 +2176,47 @@ static const char* const critical_xpm[]={
 ".........aa********aaaaa........",
 "...........aaaaaaaaaaa..........",
 ".............aaaaaaa............"};
+/* XPM */
+static const char *const question_xpm[] = {
+"32 32 5 1",
+". c None",
+"c c #000000",
+"* c #999999",
+"a c #ffffff",
+"b c #0000ff",
+"...........********.............",
+"........***aaaaaaaa***..........",
+"......**aaaaaaaaaaaaaa**........",
+".....*aaaaaaaaaaaaaaaaaa*.......",
+"....*aaaaaaaaaaaaaaaaaaaac......",
+"...*aaaaaaaabbbbbbaaaaaaaac.....",
+"..*aaaaaaaabaaabbbbaaaaaaaac....",
+".*aaaaaaaabbaaaabbbbaaaaaaaac...",
+".*aaaaaaaabbbbaabbbbaaaaaaaac*..",
+"*aaaaaaaaabbbbaabbbbaaaaaaaaac*.",
+"*aaaaaaaaaabbaabbbbaaaaaaaaaac*.",
+"*aaaaaaaaaaaaabbbbaaaaaaaaaaac**",
+"*aaaaaaaaaaaaabbbaaaaaaaaaaaac**",
+"*aaaaaaaaaaaaabbaaaaaaaaaaaaac**",
+"*aaaaaaaaaaaaabbaaaaaaaaaaaaac**",
+"*aaaaaaaaaaaaaaaaaaaaaaaaaaaac**",
+".*aaaaaaaaaaaabbaaaaaaaaaaaac***",
+".*aaaaaaaaaaabbbbaaaaaaaaaaac***",
+"..*aaaaaaaaaabbbbaaaaaaaaaac***.",
+"...caaaaaaaaaabbaaaaaaaaaac****.",
+"....caaaaaaaaaaaaaaaaaaaac****..",
+".....caaaaaaaaaaaaaaaaaac****...",
+"......ccaaaaaaaaaaaaaacc****....",
+".......*cccaaaaaaaaccc*****.....",
+"........***cccaaaac*******......",
+"..........****caaac*****........",
+".............*caaac**...........",
+"...............caac**...........",
+"................cac**...........",
+".................cc**...........",
+"..................***...........",
+"...................**...........",
+};
 #endif
 
 /*!
@@ -2172,6 +2246,7 @@ QPixmap QMotifStyle::stylePixmap(StylePixmap sp,
     case SP_MessageBoxInformation:
     case SP_MessageBoxWarning:
     case SP_MessageBoxCritical:
+    case SP_MessageBoxQuestion:
 	{
 	    const char * const * xpm_data;
 	    switch ( sp ) {
@@ -2184,6 +2259,9 @@ QPixmap QMotifStyle::stylePixmap(StylePixmap sp,
 	    case SP_MessageBoxCritical:
 		xpm_data = critical_xpm;
 		break;
+	    case SP_MessageBoxQuestion:
+		xpm_data = question_xpm;
+		break;
 	    default:
 		xpm_data = 0;
 		break;
@@ -2195,6 +2273,7 @@ QPixmap QMotifStyle::stylePixmap(StylePixmap sp,
 		QColorGroup g = QApplication::palette().active();
 		switch ( sp ) {
 		case SP_MessageBoxInformation:
+		case SP_MessageBoxQuestion:
 		    image.setColor( 2, 0xff000000 | g.dark().rgb() );
 		    image.setColor( 3, 0xff000000 | g.base().rgb() );
 		    image.setColor( 4, 0xff000000 | g.text().rgb() );
@@ -2210,7 +2289,7 @@ QPixmap QMotifStyle::stylePixmap(StylePixmap sp,
 		    image.setColor( 3, 0xff000000 | g.base().rgb() );
 		    break;
 		default:
-		    break; // Can't happen
+		    break;
 		}
 		pm.convertFromImage(image);
 	    }
