@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id:  qt/qplatinumstyle.h   3.0.3   edited Oct 12 12:18 $
+** $Id: qt/qplatinumstyle.h   3.2.3   edited May 27 08:35 $
 **
 ** Definition of Platinum-like style class
 **
@@ -42,7 +42,7 @@
 #include "qwindowsstyle.h"
 #endif // QT_H
 
-#ifndef QT_NO_STYLE_PLATINUM
+#if !defined(QT_NO_STYLE_PLATINUM) || defined(QT_PLUGIN)
 
 class QPalette;
 
