@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id$
+** $Id: qt/qsgistyle.cpp   3.2.3   edited May 27 08:45 $
 **
 ** Implementation of Motif-like style class
 **
@@ -37,7 +37,7 @@
 
 #include "qsgistyle.h"
 
-#ifndef QT_NO_STYLE_SGI
+#if !defined(QT_NO_STYLE_SGI) || defined(QT_PLUGIN)
 
 #include "qpopupmenu.h"
 #include "qapplication.h"
@@ -51,21 +51,29 @@
 #include "qscrollbar.h"
 #include "qcombobox.h"
 #include "qslider.h"
+#include "qtextedit.h"
+#include "qtoolbar.h"
+#include "qlineedit.h"
+#include "qmenubar.h"
 #include <limits.h>
 
+#ifndef QT_NO_SLIDER
 struct SliderLastPosition
 {
     SliderLastPosition() : rect(0,-1,0,-1), slider(0) {}
     QRect rect;
     const QSlider* slider;
 };
+#endif
 
+#ifndef QT_NO_SCROLLBAR
 struct ScrollbarLastPosition
 {
     ScrollbarLastPosition() : rect( 0,-1, 0,-1 ), scrollbar(0) {}
     QRect rect;
     const QScrollBar *scrollbar;
 };
+#endif
 
 class QSGIStylePrivate
 {
@@ -77,28 +85,33 @@ public:
 
     const QWidget *hotWidget;
     QPoint mousePos;
+#ifndef QT_NO_SCROLLBAR
     ScrollbarLastPosition lastScrollbarRect;
+#endif
+#ifndef QT_NO_SLIDER
     SliderLastPosition lastSliderRect;
+#endif
 };
 
 /*!
-  \class QSGIStyle qsgistyle.h
-  \brief The QSGIStyle class provides SGI/Irix look and feel.
-  \ingroup appearance
+    \class QSGIStyle qsgistyle.h
+    \brief The QSGIStyle class provides SGI/Irix look and feel.
 
-  This class implements the SGI look and feel. It resembles the
-  SGI/Irix Motif GUI style as closely as QStyle allows.
+    \ingroup appearance
+
+    This class implements the SGI look and feel. It resembles the
+    SGI/Irix Motif GUI style as closely as QStyle allows.
 */
 
 /*!
-  Constructs a QSGIStyle.
+    Constructs a QSGIStyle.
 
-  If \a useHighlightCols is FALSE (default value), the style will
-  polish the application's color palette to emulate the Motif way of
-  highlighting, which is a simple inversion between the base and the
-  text color.
+    If \a useHighlightCols is FALSE (default value), the style will
+    polish the application's color palette to emulate the Motif way of
+    highlighting, which is a simple inversion between the base and the
+    text color.
 
-  \sa QMotifStyle::useHighlightColors()
+    \sa QMotifStyle::useHighlightColors()
 */
 QSGIStyle::QSGIStyle( bool useHighlightCols ) : QMotifStyle( useHighlightCols ), isApplicationStyle( 0 )
 {
@@ -106,7 +119,7 @@ QSGIStyle::QSGIStyle( bool useHighlightCols ) : QMotifStyle( useHighlightCols ),
 }
 
 /*!
-  Destroys the style.
+    Destroys the style.
 */
 QSGIStyle::~QSGIStyle()
 {
@@ -114,11 +127,10 @@ QSGIStyle::~QSGIStyle()
 }
 
 /*!
-  \reimp
+    \reimp
 
-  Changes some application-wide settings to be
-  SGI-like, e.g., sets bold/italic font for
-  the menu system.
+    Changes some application-wide settings to be SGI-like, e.g. sets a
+    bold italic font for menu options.
 */
 void
 QSGIStyle::polish( QApplication* app)
@@ -172,10 +184,11 @@ QSGIStyle::unPolish( QApplication* /* app */ )
     QApplication::setFont( f, TRUE ); // get rid of the special fonts for special widget classes
 }
 
-/*! \reimp
+/*!
+    \reimp
 
-  Installs an event filter for several widget classes
-  to enable hovering.
+    Installs an event filter for several widget classes to enable
+    hovering.
 */
 void
 QSGIStyle::polish( QWidget* w )
@@ -198,7 +211,7 @@ QSGIStyle::polish( QWidget* w )
             sgiPal.setColor( QPalette::Disabled, QColorGroup::HighlightedText, sgiPal.disabled().base() );
         }
 
-        if ( w->inherits("QLineEdit") || w->inherits("QMultiLineEdit") ) {
+        if ( ::qt_cast<QLineEdit*>(w) || ::qt_cast<QTextEdit*>(w) ) {
             // different basecolor and highlighting in Q(Multi)LineEdit
             sgiPal.setColor( QColorGroup::Base, QColor(211,181,181) );
             sgiPal.setColor( QPalette::Active, QColorGroup::Highlight, sgiPal.active().midlight() );
@@ -208,27 +221,27 @@ QSGIStyle::polish( QWidget* w )
             sgiPal.setColor( QPalette::Disabled, QColorGroup::Highlight, sgiPal.disabled().midlight() );
             sgiPal.setColor( QPalette::Disabled, QColorGroup::HighlightedText, sgiPal.disabled().text() );
 
-        } else if ( w->inherits("QMenuBar") || w->inherits("QToolBar") ) {
+        } else if ( ::qt_cast<QMenuBar*>(w) || ::qt_cast<QToolBar*>(w) ) {
             sgiPal.setColor( QColorGroup::Button, sgiPal.active().midlight() );
         }
 
         w->setPalette( sgiPal );
     }
 
-    if ( w->inherits("QButton") || w->inherits("QSlider") || w->inherits("QScrollBar") ) {
+    if ( ::qt_cast<QButton*>(w) || ::qt_cast<QSlider*>(w) || ::qt_cast<QScrollBar*>(w) ) {
         w->installEventFilter( this );
         w->setMouseTracking( TRUE );
-        if ( w->inherits("QToolButton") )
-            w->setBackgroundMode( QWidget::PaletteBackground );
-        if ( w->inherits("QScrollBar") )
+#ifndef QT_NO_SCROLLBAR
+        if ( ::qt_cast<QScrollBar*>(w) )
             w->setBackgroundMode( QWidget::NoBackground );
-    } else if ( w->inherits( "QComboBox" ) ) {
+#endif
+    } else if ( ::qt_cast<QComboBox*>(w) ) {
 	QFont f = QApplication::font();
 	f.setBold( TRUE );
 	f.setItalic( TRUE );
 	w->setFont( f );
 #ifndef QT_NO_MENUBAR
-    } else if ( w->inherits("QMenuBar") ) {
+    } else if ( ::qt_cast<QMenuBar*>(w) ) {
         ((QFrame*) w)->setFrameStyle(QFrame::StyledPanel | QFrame::Raised);
         w->setBackgroundMode( QWidget::PaletteBackground );
 	QFont f = QApplication::font();
@@ -237,16 +250,14 @@ QSGIStyle::polish( QWidget* w )
 	w->setFont( f );
 #endif
 #ifndef QT_NO_POPUPMENU
-    } else if ( w->inherits("QPopupMenu") ) {
+    } else if ( ::qt_cast<QPopupMenu*>(w) ) {
         ((QFrame*) w)->setLineWidth( pixelMetric( PM_DefaultFrameWidth ) + 1 );
 	QFont f = QApplication::font();
 	f.setBold( TRUE );
 	f.setItalic( TRUE );
 	w->setFont( f );
 #endif
-    } else if ( w->inherits("QToolBar") ) {
-        w->setBackgroundMode( QWidget::PaletteBackground );
-    } else if ( w->inherits("QToolBarSeparator") ) {
+    } else if ( ::qt_cast<QToolBar*>(w) || w->inherits("QToolBarSeparator") ) {
         w->setBackgroundMode( QWidget::PaletteBackground );
     }
 }
@@ -255,15 +266,15 @@ QSGIStyle::polish( QWidget* w )
 void
 QSGIStyle::unPolish( QWidget* w )
 {
-    if ( w->inherits("QButton") || w->inherits("QSlider") || w->inherits("QScrollBar") ) {
+    if ( ::qt_cast<QButton*>(w) || ::qt_cast<QSlider*>(w) || ::qt_cast<QScrollBar*>(w) ) {
         w->removeEventFilter( this );
 #ifndef QT_NO_POPUPMENU
-    } else if ( w->inherits( "QPopupMenu" ) ) {
+    } else if ( ::qt_cast<QPopupMenu*>(w) ) {
 	((QFrame*)w)->setLineWidth( pixelMetric( PM_DefaultFrameWidth ) );
 	w->setFont( QApplication::font() );
 #endif
 #if !defined(QT_NO_MENUBAR) || !defined(QT_NO_COMBOBOX)
-    } else if ( w->inherits( "QMenuBar" ) || w->inherits( "QComboBox" ) ) {
+    } else if ( ::qt_cast<QMenuBar*>(w) || ::qt_cast<QComboBox*>(w) ) {
 	w->setFont( QApplication::font() );
 #endif
     }
@@ -281,15 +292,15 @@ bool QSGIStyle::eventFilter( QObject* o, QEvent* e )
     case QEvent::MouseButtonPress:
         {
 #ifndef QT_NO_SCROLLBAR
-	    if ( widget->inherits( "QScrollBar" ) ) {
+	    if ( ::qt_cast<QScrollBar*>(widget) ) {
 		d->lastScrollbarRect.rect = ((QScrollBar*)widget)->sliderRect();
 		d->lastScrollbarRect.scrollbar = ((QScrollBar*)widget);
 		widget->repaint( FALSE );
-	    } else 
+	    } else
 #endif
 	    {
 #ifndef QT_NO_SLIDER
-		if ( widget->inherits("QSlider") ) {
+		if ( ::qt_cast<QSlider*>(widget) ) {
 		    d->lastSliderRect.rect = ((QSlider*)widget)->sliderRect();
 		    d->lastSliderRect.slider = ((QSlider*)widget);
 		    widget->repaint( FALSE );
@@ -301,14 +312,19 @@ bool QSGIStyle::eventFilter( QObject* o, QEvent* e )
 
     case QEvent::MouseButtonRelease:
         {
-	    if ( widget->inherits( "QScrollBar" ) ) {
+	    if ( 0 ) {
+#ifndef QT_NO_SCROLLBAR
+	    } else if ( ::qt_cast<QScrollBar*>(widget) ) {
 		QRect oldRect = d->lastScrollbarRect.rect;
 		d->lastScrollbarRect.rect = QRect( 0, -1, 0, -1 );
 		widget->repaint( oldRect, FALSE );
-	    } else if ( widget->inherits("QSlider") ) {
+#endif
+#ifndef QT_NO_SLIDER
+	    } else if ( ::qt_cast<QSlider*>(widget) ) {
 		QRect oldRect = d->lastSliderRect.rect;
 		d->lastSliderRect.rect = QRect( 0, -1, 0, -1 );
 		widget->repaint( oldRect, FALSE );
+#endif
             }
         }
         break;
@@ -609,12 +625,12 @@ void QSGIStyle::drawPrimitive( PrimitiveElement pe,
 		return;
 	    }
 
-	    QPen savePen = p->pen();			// save current pen
+	    p->save();
 	    p->setPen( Qt::NoPen );
 	    a.translate( x+w/2, y+h/2 );
 	    p->setBrush( flags & Style_Enabled ? cg.dark() : cg.light() );
 	    p->drawPolygon( a );			// draw arrow
-	    p->setPen( savePen );			// restore pen
+	    p->restore();
 	}
 	break;
 
@@ -885,6 +901,8 @@ void QSGIStyle::drawControl( ControlElement element,
 
 	    p->setPen( cg.foreground() );
 	    p->setBrush( QBrush( cg.button(),Qt::NoBrush ) );
+	    p->setBrushOrigin( -widget->backgroundOffset().x(),
+			       -widget->backgroundOffset().y() );
 
 	    int diw = pixelMetric( QStyle::PM_ButtonDefaultIndicator );
 	    if ( btn->isDefault() || btn->autoDefault() ) {
@@ -1002,8 +1020,8 @@ void QSGIStyle::drawControl( ControlElement element,
 			cflags |= Style_On;
 
 		    if ( mi->isChecked() ) {
-			QRect er( x+sgiItemFrame+1, y+sgiItemFrame+3, 
-				  pixelMetric(PM_IndicatorWidth), 
+			QRect er( x+sgiItemFrame+1, y+sgiItemFrame+3,
+				  pixelMetric(PM_IndicatorWidth),
 				  pixelMetric(PM_IndicatorHeight) );
 			er.addCoords( 1, 1, -1, -1 );
 			drawPrimitive( PE_ButtonBevel, p, er, cg, cflags, opt );
@@ -1153,7 +1171,7 @@ void QSGIStyle::drawComplexControl( ComplexControl control,
 		grooveTop.addCoords( 1, 1, -1, -1 );
 		drawPrimitive( PE_ButtonBevel, p, grooveTop, cg, flags & ~Style_MouseOver, opt );
 
-		if ( slider->hasFocus() ) {
+		if ( flags & Style_HasFocus ) {
 		    QRect fr = subRect( SR_SliderFocusRect, widget );
 		    drawPrimitive( PE_FocusRect, p, fr, cg, flags & ~Style_MouseOver );
 		}
@@ -1210,7 +1228,7 @@ void QSGIStyle::drawComplexControl( ComplexControl control,
 
 	    if ( sub & SC_ComboBoxArrow ) {
 		p->save();
-		QRect er = 
+		QRect er =
 		    QStyle::visualRect( querySubControlMetrics( CC_ComboBox, cb, SC_ComboBoxArrow ), cb );
 
 		er.addCoords( 0, 3, 0, 0 );
@@ -1268,8 +1286,8 @@ void QSGIStyle::drawComplexControl( ComplexControl control,
 	    if ( sub & SC_ScrollBarAddPage ) {
 		QRect er = QStyle::visualRect( querySubControlMetrics( CC_ScrollBar, widget, SC_ScrollBarAddPage, opt ), widget );
 		QRegion region( er );
-		if ( d->lastScrollbarRect.scrollbar == scrollbar && 
-		     d->lastScrollbarRect.rect.isValid() && 
+		if ( d->lastScrollbarRect.scrollbar == scrollbar &&
+		     d->lastScrollbarRect.rect.isValid() &&
 		     er.intersects( d->lastScrollbarRect.rect ) ) {
 		    region = region.subtract( d->lastScrollbarRect.rect );
 		    p->setClipRegion( region );
@@ -1282,7 +1300,7 @@ void QSGIStyle::drawComplexControl( ComplexControl control,
 		drawPrimitive( PE_ScrollBarAddPage, p, er, cg, flags & ~Style_MouseOver, opt );
 
 		if ( d->lastScrollbarRect.scrollbar == scrollbar &&
-		     d->lastScrollbarRect.rect.isValid() && 
+		     d->lastScrollbarRect.rect.isValid() &&
 		     er.intersects( d->lastScrollbarRect.rect ) ) {
 		    if ( sub & SC_ScrollBarSlider && handle.isValid() ) {
 			region = er;
@@ -1298,8 +1316,8 @@ void QSGIStyle::drawComplexControl( ComplexControl control,
 	    if ( sub & SC_ScrollBarSubPage ) {
 		QRect er = QStyle::visualRect( querySubControlMetrics( CC_ScrollBar, widget, SC_ScrollBarSubPage, opt ), widget );
 		QRegion region( er );
-		if ( d->lastScrollbarRect.scrollbar == scrollbar && 
-		     d->lastScrollbarRect.rect.isValid() && 
+		if ( d->lastScrollbarRect.scrollbar == scrollbar &&
+		     d->lastScrollbarRect.rect.isValid() &&
 		     er.intersects( d->lastScrollbarRect.rect ) ) {
 		    region = region.subtract( d->lastScrollbarRect.rect );
 		    p->setClipRegion( region );
@@ -1310,7 +1328,7 @@ void QSGIStyle::drawComplexControl( ComplexControl control,
 		}
 		drawPrimitive( PE_ScrollBarSubPage, p, er, cg, flags & ~Style_MouseOver, opt );
 		if ( d->lastScrollbarRect.scrollbar == scrollbar &&
-		     d->lastScrollbarRect.rect.isValid() && 
+		     d->lastScrollbarRect.rect.isValid() &&
 		     er.intersects( d->lastScrollbarRect.rect ) ) {
 		    if ( sub & SC_ScrollBarSlider && handle.isValid() ) {
 			region = er;
@@ -1359,6 +1377,10 @@ QSize QSGIStyle::sizeFromContents( ContentsType contents,
 	    QMenuItem *mi = opt.menuItem();
 	    sz = QMotifStyle::sizeFromContents( contents, widget, contentsSize,
 						opt );
+	    // SGI checkmark items needs a bit more room
+	    const QPopupMenu *popup = (QPopupMenu *) widget;
+	    if ( popup && popup->isCheckable() )
+		sz.setWidth( sz.width() + 8 );
 	    // submenu indicator needs a bit more room
 	    if (mi->popup())
 		sz.setWidth( sz.width() + sgiTabSpacing );
@@ -1404,50 +1426,37 @@ QRect QSGIStyle::querySubControlMetrics( ComplexControl control,
 					   SubControl sub,
 					   const QStyleOption& opt ) const
 {
-    QRect rect;
-
     switch ( control ) {
     case CC_ComboBox:
 	switch ( sub ) {
 	case SC_ComboBoxFrame:
-	    rect = widget->rect();
-	    break;
+	    return widget->rect();
 
-	case SC_ComboBoxArrow:
-	    {
-		int ew, awh, sh, dh, ax, ay, sy;
-		int fw = pixelMetric( PM_DefaultFrameWidth, widget );
-		QRect cr = widget->rect();
-		cr.addCoords( fw, fw, -fw, -fw );
-		get_combo_parameters( cr, ew, awh, ax, ay, sh, dh, sy );
-		rect.setRect( ax, ay, awh, awh );
-	    }
-	    break;
+	case SC_ComboBoxArrow: {
+	    int ew, awh, sh, dh, ax, ay, sy;
+	    int fw = pixelMetric( PM_DefaultFrameWidth, widget );
+	    QRect cr = widget->rect();
+	    cr.addCoords( fw, fw, -fw, -fw );
+	    get_combo_parameters( cr, ew, awh, ax, ay, sh, dh, sy );
+	    return QRect( ax, ay, awh, awh ); }
 
-	case SC_ComboBoxEditField:
-	    {
-		int fw = pixelMetric( PM_DefaultFrameWidth, widget );
-		rect = widget->rect();
-		rect.addCoords( fw, fw, -fw, -fw );
-		int ew = get_combo_extra_width( rect.height() );
-
- 		rect.addCoords( 1, 1, -1-ew, -1 );
-	    }
-	    break;
+	case SC_ComboBoxEditField: {
+	    int fw = pixelMetric( PM_DefaultFrameWidth, widget );
+	    QRect rect = widget->rect();
+	    rect.addCoords( fw, fw, -fw, -fw );
+	    int ew = get_combo_extra_width( rect.height() );
+	    rect.addCoords( 1, 1, -1-ew, -1 );
+	    return rect; }
 
 	default:
 	    break;
 	}
 	break;
-
-    case CC_ScrollBar:
-	return QCommonStyle::querySubControlMetrics( control, widget, sub, opt );
-
-    default:
-	return QMotifStyle::querySubControlMetrics( control, widget, sub, opt );
+	case CC_ScrollBar:
+	    return QCommonStyle::querySubControlMetrics( control, widget, sub, opt );
+	default: break;
     }
-
-    return rect;
+    return QMotifStyle::querySubControlMetrics( control, widget, sub, opt );
 }
 
 #endif // QT_NO_STYLE_SGI
