@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id$
+** $Id: qt/qstylefactory.cpp   3.2.3   edited Oct 10 13:32 $
 **
 ** Implementation of QStyleFactory class
 **
@@ -48,9 +48,17 @@
 #include "qplatinumstyle.h"
 #include "qsgistyle.h"
 #include "qcompactstyle.h"
+#ifndef QT_NO_STYLE_WINDOWSXP
+#include "qwindowsxpstyle.h"
+#endif
 #ifndef QT_NO_STYLE_AQUA
 #include "qaquastyle.h"
 #endif
+#if !defined( QT_NO_STYLE_MAC ) && defined( Q_WS_MAC )
+QCString p2qstring(const unsigned char *c); //qglobal.cpp
+#include "qt_mac.h"
+#include "qmacstyle_mac.h"
+#endif
 #include <stdlib.h>
 
 #include <private/qpluginmanager_p.h>
@@ -84,26 +92,26 @@ QStyleFactoryPrivate::~QStyleFactoryPrivate()
 #endif //QT_NO_COMPONENT
 
 /*!
-  \class QStyleFactory qstylefactory.h
-  \brief The QStyleFactory class creates QStyle objects.
+    \class QStyleFactory qstylefactory.h
+    \brief The QStyleFactory class creates QStyle objects.
 
-  The style factory creates a QStyle object for a given key with
-  QStyleFactory::create(key).
+    The style factory creates a QStyle object for a given key with
+    QStyleFactory::create(key).
 
-  The styles are either built-in or dynamically loaded from a style
-  plugin (see \l QStylePlugin).
-
-  QStyleFactory::keys() returns a list of valid keys, typically
-  including "windows", "motif", "cde", "motifplus", "platinum", "sgi"
-  and "compact". Depending on the platform, "windowsxp", "aqua" or
-  "macintosh" may be available.
+    The styles are either built-in or dynamically loaded from a style
+    plugin (see \l QStylePlugin).
 
+    QStyleFactory::keys() returns a list of valid keys, typically
+    including "windows", "motif", "cde", "motifplus", "platinum",
+    "sgi" and "compact". Depending on the platform, "windowsxp",
+    "aqua" or "macintosh" may be available.
 */
 
-/*!  Creates a QStyle object that matches \a key. This is either a
-  built-in style, or a style from a style plugin.
+/*!
+    Creates a QStyle object that matches \a key. This is either a
+    built-in style, or a style from a style plugin.
 
-  \sa keys()
+    \sa keys()
 */
 QStyle *QStyleFactory::create( const QString& key )
 {
@@ -114,6 +122,11 @@ QStyle *QStyleFactory::create( const QString& key )
         ret = new QWindowsStyle;
     else
 #endif
+#ifndef QT_NO_STYLE_WINDOWSXP
+    if ( style == "windowsxp" )
+	ret = new QWindowsXPStyle;
+    else
+#endif
 #ifndef QT_NO_STYLE_MOTIF
     if ( style == "motif" )
         ret = new QMotifStyle;
@@ -148,6 +161,11 @@ QStyle *QStyleFactory::create( const QString& key )
     if ( style == "aqua" )
         ret = new QAquaStyle;
 #endif
+#if !defined( QT_NO_STYLE_MAC ) && defined( Q_WS_MAC )
+    if( style.left(9) == "macintosh" )
+	ret = new QMacStyle;
+#endif
+    { } // Keep these here - they make the #ifdefery above work
 
 #ifndef QT_NO_COMPONENT
     if(!ret) {
@@ -168,10 +186,9 @@ QStyle *QStyleFactory::create( const QString& key )
 
 #ifndef QT_NO_STRINGLIST
 /*!
-  Returns the list of keys  this factory can create
-  styles for.
+    Returns the list of keys this factory can create styles for.
 
-  \sa create()
+    \sa create()
 */
 QStringList QStyleFactory::keys()
 {
@@ -187,6 +204,10 @@ QStringList QStyleFactory::keys()
     if ( !list.contains( "Windows" ) )
 	list << "Windows";
 #endif
+#ifndef QT_NO_STYLE_WINDOWSXP
+    if ( !list.contains( "WindowsXP" ) && QWindowsXPStyle::resolveSymbols() )
+	list << "WindowsXP";
+#endif
 #ifndef QT_NO_STYLE_MOTIF
     if ( !list.contains( "Motif" ) )
 	list << "Motif";
@@ -215,6 +236,20 @@ QStringList QStyleFactory::keys()
     if ( !list.contains( "Aqua" ) )
 	list << "Aqua";
 #endif
+#if !defined( QT_NO_STYLE_MAC ) && defined( Q_WS_MAC )
+    QString mstyle = "Macintosh";
+    Collection c = NewCollection();
+    if (c) {
+	GetTheme(c);
+	Str255 str;
+	long int s = 256;
+	if(!GetCollectionItem(c, kThemeNameTag, 0, &s, &str))
+	    mstyle += " (" + p2qstring(str) + ")";
+    }
+    if (!list.contains(mstyle))
+	list << mstyle;
+    DisposeCollection(c);
+#endif
 
     return list;
 }
