@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id:  qt/qwindowsstyle.cpp   3.0.5   edited Jun 27 11:34 $
+** $Id: qt/qwindowsstyle.cpp   3.2.3   edited Jun 20 15:29 $
 **
 ** Implementation of Windows-like style class
 **
@@ -37,7 +37,7 @@
 
 #include "qwindowsstyle.h"
 
-#ifndef QT_NO_STYLE_WINDOWS
+#if !defined(QT_NO_STYLE_WINDOWS) || defined(QT_PLUGIN)
 
 #include "qpopupmenu.h"
 #include "qapplication.h"
@@ -59,6 +59,7 @@
 #include "qlistview.h"
 #include "qbitmap.h"
 #include "qcleanuphandler.h"
+#include "qdockwindow.h"
 
 #if defined(Q_WS_WIN)
 #include "qt_windows.h"
@@ -82,11 +83,12 @@ static bool use2000style = TRUE;
 enum QSliderDirection { SlUp, SlDown, SlLeft, SlRight };
 
 /*!
-  \class QWindowsStyle qwindowsstyle.h
-  \brief The QWindowsStyle class provides a Microsoft Windows-like look and feel.
-  \ingroup appearance
+    \class QWindowsStyle qwindowsstyle.h
+    \brief The QWindowsStyle class provides a Microsoft Windows-like look and feel.
 
-  This style is Qt's default GUI style on Windows.
+    \ingroup appearance
+
+    This style is Qt's default GUI style on Windows.
 */
 
 /*!
@@ -95,13 +97,7 @@ enum QSliderDirection { SlUp, SlDown, SlLeft, SlRight };
 QWindowsStyle::QWindowsStyle() : QCommonStyle()
 {
 #if defined(Q_OS_WIN32)
-    if ( qWinVersion() != Qt::WV_NT &&
-	 qWinVersion() != Qt::WV_95 )
-	use2000style = TRUE;
-    else
-	use2000style = FALSE;
-#else
-    use2000style = TRUE;
+    use2000style = qWinVersion() != Qt::WV_NT && qWinVersion() != Qt::WV_95;
 #endif
 }
 
@@ -118,6 +114,7 @@ void QWindowsStyle::drawPrimitive( PrimitiveElement pe,
 				   SFlags flags,
 				   const QStyleOption& opt ) const
 {
+    QRect rr( r );
     switch (pe) {
     case PE_ButtonCommand:
 	{
@@ -156,6 +153,31 @@ void QWindowsStyle::drawPrimitive( PrimitiveElement pe,
 		p->fillRect(r, fill);
 	    break;
 	}
+#if defined(Q_WS_WIN)
+    case PE_HeaderArrow:
+	p->save();
+	if ( flags & Style_Up ) { // invert logic to follow Windows style guide
+	    QPointArray pa( 3 );
+	    p->setPen( cg.light() );
+	    p->drawLine( r.x() + r.width(), r.y(), r.x() + r.width() / 2, r.height() );
+	    p->setPen( cg.dark() );
+	    pa.setPoint( 0, r.x() + r.width() / 2, r.height() );
+	    pa.setPoint( 1, r.x(), r.y() );
+	    pa.setPoint( 2, r.x() + r.width(), r.y() );
+	    p->drawPolyline( pa );
+	} else {
+	    QPointArray pa( 3 );
+	    p->setPen( cg.light() );
+	    pa.setPoint( 0, r.x(), r.height() );
+	    pa.setPoint( 1, r.x() + r.width(), r.height() );
+	    pa.setPoint( 2, r.x() + r.width() / 2, r.y() );
+	    p->drawPolyline( pa );
+	    p->setPen( cg.dark() );
+	    p->drawLine( r.x(), r.height(), r.x() + r.width() / 2, r.y() );
+	}
+	p->restore();
+	break;
+#endif
 
     case PE_ButtonDefault:
 	p->setPen(cg.shadow());
@@ -202,7 +224,6 @@ void QWindowsStyle::drawPrimitive( PrimitiveElement pe,
 
     case PE_Indicator:
 	{
-#ifndef QT_NO_BUTTON
 	    QBrush fill;
 	    if (flags & Style_NoChange) {
 		QBrush b = p->brush();
@@ -220,35 +241,44 @@ void QWindowsStyle::drawPrimitive( PrimitiveElement pe,
 		fill = cg.brush( QColorGroup::Background );
 
 	    qDrawWinPanel( p, r, cg, TRUE, &fill );
-	    if (! (flags & Style_Off)) {
-		QPointArray a( 7*2 );
-		int i, xx, yy;
-		xx = r.x() + 3;
-		yy = r.y() + 5;
 
-		for ( i=0; i<3; i++ ) {
-		    a.setPoint( 2*i,   xx, yy );
-		    a.setPoint( 2*i+1, xx, yy+2 );
-		    xx++; yy++;
-		}
-
-		yy -= 2;
-		for ( i=3; i<7; i++ ) {
-		    a.setPoint( 2*i,   xx, yy );
-		    a.setPoint( 2*i+1, xx, yy+2 );
-		    xx++; yy--;
-		}
-
-		if (flags & Style_NoChange)
-		    p->setPen( cg.dark() );
-		else
-		    p->setPen( cg.text() );
-
-		p->drawLineSegments( a );
-	    }
-#endif
-	    break;
+	    if (flags & Style_NoChange )
+		p->setPen( cg.dark() );
+	    else
+		p->setPen( cg.text() );
+	} // FALLTHROUGH
+    case PE_CheckListIndicator:
+	if ( pe == PE_CheckListIndicator ) { //since we fall through from PE_Indicator
+	    if ( flags & Style_Enabled )
+		p->setPen( QPen( cg.text(), 1 ) );
+	    else
+		p->setPen( QPen( cg.dark(), 1 ) );
+	    if ( flags & Style_NoChange )
+		p->setBrush( cg.brush( QColorGroup::Button ) );
+	    p->drawRect( r.x()+1, r.y()+1, 11, 11 );
 	}
+	if (! (flags & Style_Off)) {
+	    QPointArray a( 7*2 );
+	    int i, xx, yy;
+	    xx = rr.x() + 3;
+	    yy = rr.y() + 5;
+
+	    for ( i=0; i<3; i++ ) {
+		a.setPoint( 2*i,   xx, yy );
+		a.setPoint( 2*i+1, xx, yy+2 );
+		xx++; yy++;
+	    }
+
+	    yy -= 2;
+	    for ( i=3; i<7; i++ ) {
+		a.setPoint( 2*i,   xx, yy );
+		a.setPoint( 2*i+1, xx, yy+2 );
+		xx++; yy--;
+	    }
+
+	    p->drawLineSegments( a );
+	}
+	break;
 
     case PE_ExclusiveIndicator:
 	{
@@ -318,17 +348,60 @@ void QWindowsStyle::drawPrimitive( PrimitiveElement pe,
 	    int lw = opt.isDefault() ? pixelMetric(PM_DefaultFrameWidth)
 			: opt.lineWidth();
 
-	    if (lw == 2)
-		qDrawWinPanel(p, r, cg, flags & Style_Sunken);
-	    else
+	    if (lw == 2) {
+		QColorGroup popupCG = cg;
+		if ( pe == PE_PanelPopup ) {
+		    popupCG.setColor( QColorGroup::Light, cg.background() );
+		    popupCG.setColor( QColorGroup::Midlight, cg.light() );
+		}
+		qDrawWinPanel(p, r, popupCG, flags & Style_Sunken);
+	    } else {
 		QCommonStyle::drawPrimitive(pe, p, r, cg, flags, opt);
+	    }
 	    break;
 	}
 
     case PE_Splitter:
+	{
+	    QPen oldPen = p->pen();
+	    p->setPen( cg.light() );
+	    if ( flags & Style_Horizontal ) {
+		p->drawLine( r.x() + 1, r.y(), r.x() + 1, r.height() );
+		p->setPen( cg.dark() );
+		p->drawLine( r.x(), r.y(), r.x(), r.height() );
+		p->drawLine( r.right()-1, r.y(), r.right()-1, r.height() );
+		p->setPen( cg.shadow() );
+		p->drawLine( r.right(), r.y(), r.right(), r.height() );
+	    } else {
+		p->drawLine( r.x(), r.y() + 1, r.width(), r.y() + 1 );
+		p->setPen( cg.dark() );
+		p->drawLine( r.x(), r.bottom() - 1, r.width(), r.bottom() - 1 );
+		p->setPen( cg.shadow() );
+		p->drawLine( r.x(), r.bottom(), r.width(), r.bottom() );
+	    }
+	    p->setPen( oldPen );
+	    break;
+	}
     case PE_DockWindowResizeHandle:
-	qDrawWinPanel( p, r.x(), r.y(), r.width(), r.height(), cg );
-	break;
+	{
+	    QPen oldPen = p->pen();
+	    p->setPen( cg.light() );
+	    if ( flags & Style_Horizontal ) {
+		p->drawLine( r.x(), r.y(), r.width(), r.y() );
+		p->setPen( cg.dark() );
+		p->drawLine( r.x(), r.bottom() - 1, r.width(), r.bottom() - 1 );
+		p->setPen( cg.shadow() );
+		p->drawLine( r.x(), r.bottom(), r.width(), r.bottom() );
+	    } else {
+		p->drawLine( r.x(), r.y(), r.x(), r.height() );
+		p->setPen( cg.dark() );
+		p->drawLine( r.right()-1, r.y(), r.right()-1, r.height() );
+		p->setPen( cg.shadow() );
+		p->drawLine( r.right(), r.y(), r.right(), r.height() );
+	    }
+	    p->setPen( oldPen );
+	    break;
+	}
 
     case PE_ScrollBarSubLine:
 	if (use2000style) {
@@ -400,6 +473,15 @@ void QWindowsStyle::drawPrimitive( PrimitiveElement pe,
 	    drawPrimitive(PE_ButtonBevel, p, r, cg, Style_Enabled | Style_Raised);
 	break;
 
+    case PE_WindowFrame:
+	{
+	    QColorGroup popupCG = cg;
+	    popupCG.setColor( QColorGroup::Light, cg.background() );
+	    popupCG.setColor( QColorGroup::Midlight, cg.light() );
+	    qDrawWinPanel(p, r, popupCG, flags & Style_Sunken);
+	}
+	break;
+
     default:
 	if (pe >= PE_ArrowUp && pe <= PE_ArrowLeft) {
 	    QPointArray a;
@@ -540,7 +622,7 @@ void QWindowsStyle::drawControl( ControlElement element,
 		    p->drawLine( r2.left()+1, r2.top(), r2.left()+1, r2.bottom()-2 );
 		    p->setPen( cg.dark() );
 		} else {
- 		    p->setPen( cg.shadow() );
+		    p->setPen( cg.shadow() );
 		    p->drawLine( r2.left() +
 				 (rightAligned && firstTab ? 0 : 1),
 				 r2.top() + 1,
@@ -563,8 +645,8 @@ void QWindowsStyle::drawControl( ControlElement element,
 			     r2.left() + 1, r2.bottom() - 1 );
 
 		p->setPen( cg.midlight() );
- 		p->drawLine( r2.left() + 1, r2.bottom() - 2,
- 			     r2.left() + 1, r2.top() + (selected ? 0 : 2) );
+		p->drawLine( r2.left() + 1, r2.bottom() - 2,
+			     r2.left() + 1, r2.top() + (selected ? 0 : 2) );
 
 		p->setPen( cg.shadow() );
 		p->drawLine( r2.right(),
@@ -584,6 +666,12 @@ void QWindowsStyle::drawControl( ControlElement element,
 	    break;
 	}
 #endif // QT_NO_TABBAR
+    case CE_ToolBoxTab:
+	{
+	    qDrawShadePanel( p, r, cg, flags & (Style_Sunken | Style_Down | Style_On) , 1,
+			     &cg.brush(QColorGroup::Button));
+	    break;
+	}
 
 #ifndef QT_NO_POPUPMENU
     case CE_PopupMenuItem:
@@ -598,7 +686,7 @@ void QWindowsStyle::drawControl( ControlElement element,
 
 	    int tab = opt.tabWidth();
 	    int maxpmw = opt.maxIconWidth();
-	    bool dis = ! mi->isEnabled();
+	    bool dis = !(flags&Style_Enabled);
 	    bool checkable = popupmenu->isCheckable();
 	    bool act = flags & Style_Active;
 	    int x, y, w, h;
@@ -725,9 +813,11 @@ void QWindowsStyle::drawControl( ControlElement element,
 		int text_flags = AlignVCenter|ShowPrefix | DontClip | SingleLine;
 		text_flags |= (QApplication::reverseLayout() ? AlignRight : AlignLeft );
 		if ( t >= 0 ) {                         // draw tab text
-		    int xp;
-		    xp = x + w - tab - ((use2000style) ? 20 : windowsRightBorder) -
-			 windowsItemHMargin - windowsItemFrame + 1;
+		    int xp = x + w - tab - windowsItemHMargin - windowsItemFrame + 1;
+		    if ( use2000style )
+			xp -= 20;
+		    else
+			xp -= windowsRightBorder;
 		    int xoff = visualRect( QRect( xp, y+windowsItemVMargin, tab, h-2*windowsItemVMargin ), r ).x();
 		    if ( dis && !act ) {
 			p->setPen( cg.light() );
@@ -758,18 +848,13 @@ void QWindowsStyle::drawControl( ControlElement element,
 		xpos = x+w - windowsArrowHMargin - windowsItemFrame - dim;
 		vrect = visualRect( QRect(xpos, y + h / 2 - dim / 2, dim, dim), r );
 		if ( act ) {
-		    if ( !dis )
-			discol = white;
-		    QColorGroup g2( discol, cg.highlight(),
-				    white, white,
-				    dis ? discol : white,
-				    discol, white );
-
+		    QColorGroup g2 = cg;
+		    g2.setColor( QColorGroup::ButtonText, g2.highlightedText() );
 		    drawPrimitive(arrow, p, vrect,
-				  g2, Style_Enabled);
+				  g2, dis ? Style_Default : Style_Enabled);
 		} else {
 		    drawPrimitive(arrow, p, vrect,
-				  cg, mi->isEnabled() ? Style_Enabled : Style_Default);
+				  cg, dis ? Style_Default : Style_Enabled );
 		}
 	    }
 
@@ -793,11 +878,8 @@ void QWindowsStyle::drawControl( ControlElement element,
 		    qDrawShadeRect( p, r.x(), r.y(), r.width(), r.height(),
 				    cg, active && down, 1, 0, &b );
 		if ( active && down ) {
-		    pr.setRect( r.x() + 2 + pixelMetric(PM_ButtonShiftHorizontal,
-							widget),
-				r.y() + 2 + pixelMetric(PM_ButtonShiftVertical,
-							widget),
-				r.width()-4, r.height()-4 );
+		    pr.moveBy( pixelMetric(PM_ButtonShiftHorizontal, widget),
+			       pixelMetric(PM_ButtonShiftVertical, widget) );
 		    p->setBrushOrigin(p->brushOrigin() - QPoint(1,1));
 		}
 	    }
@@ -826,7 +908,7 @@ int QWindowsStyle::pixelMetric(PixelMetric metric, const QWidget *widget) const
 	break;
 
     case PM_MaximumDragDistance:
-	ret = 20;
+	ret = 60;
 	break;
 
 #ifndef QT_NO_SLIDER
@@ -870,18 +952,40 @@ int QWindowsStyle::pixelMetric(PixelMetric metric, const QWidget *widget) const
 
 #if defined(Q_WS_WIN)
     case PM_TitleBarHeight:
-	if ( widget && ( widget->testWFlags( WStyle_Tool ) || widget->inherits( "QDockWindow" ) ) ) {
-	    ret = GetSystemMetrics( SM_CYSMCAPTION ) - 1; // MS always use one less than they say
+	if ( widget && ( widget->testWFlags( WStyle_Tool ) || ::qt_cast<QDockWindow*>(widget) ) ) {
+	    // MS always use one less than they say
+#if defined(Q_OS_TEMP)
+	    ret = GetSystemMetrics( SM_CYCAPTION ) - 1;
+#else
+	    ret = GetSystemMetrics( SM_CYSMCAPTION ) - 1;
+#endif
 	} else {
 	    ret = GetSystemMetrics( SM_CYCAPTION ) - 1;
 	}
 	break;
+
+    case PM_ScrollBarExtent:
+	{
+#ifndef Q_OS_TEMP
+	    NONCLIENTMETRICS ncm;
+	    ncm.cbSize = sizeof(NONCLIENTMETRICS);
+	    if ( SystemParametersInfo( SPI_GETNONCLIENTMETRICS, sizeof(NONCLIENTMETRICS), &ncm, 0 ) )
+		ret = QMAX( ncm.iScrollHeight, ncm.iScrollWidth );
+	    else
+#endif
+		ret = QCommonStyle::pixelMetric( metric, widget );
+	}
+	break;
 #endif
 
     case PM_SplitterWidth:
 	ret = QMAX( 6, QApplication::globalStrut().width() );
 	break;
 
+    case PM_MDIFrameWidth:
+	ret = 3;
+	break;
+
     default:
 	ret = QCommonStyle::pixelMetric(metric, widget);
 	break;
@@ -1121,12 +1225,6 @@ static const char * dock_window_close_xpm[] = {
 "........",
 "........"};
 
-// Message box icons, from page 210 of the Windows style guide.
-
-// Hand-drawn to resemble Microsoft's icons, but in the Mac/Netscape
-// palette.  The "question mark" icon, which Microsoft recommends not
-// using but a lot of people still use, is left out.
-
 /* XPM */
 static const char * const information_xpm[]={
 "32 32 5 1",
@@ -1245,7 +1343,47 @@ static const char* const critical_xpm[]={
 ".........aa********aaaaa........",
 "...........aaaaaaaaaaa..........",
 ".............aaaaaaa............"};
-
+/* XPM */
+static const char *const question_xpm[] = {
+"32 32 5 1",
+". c None",
+"c c #000000",
+"* c #999999",
+"a c #ffffff",
+"b c #0000ff",
+"...........********.............",
+"........***aaaaaaaa***..........",
+"......**aaaaaaaaaaaaaa**........",
+".....*aaaaaaaaaaaaaaaaaa*.......",
+"....*aaaaaaaaaaaaaaaaaaaac......",
+"...*aaaaaaaabbbbbbaaaaaaaac.....",
+"..*aaaaaaaabaaabbbbaaaaaaaac....",
+".*aaaaaaaabbaaaabbbbaaaaaaaac...",
+".*aaaaaaaabbbbaabbbbaaaaaaaac*..",
+"*aaaaaaaaabbbbaabbbbaaaaaaaaac*.",
+"*aaaaaaaaaabbaabbbbaaaaaaaaaac*.",
+"*aaaaaaaaaaaaabbbbaaaaaaaaaaac**",
+"*aaaaaaaaaaaaabbbaaaaaaaaaaaac**",
+"*aaaaaaaaaaaaabbaaaaaaaaaaaaac**",
+"*aaaaaaaaaaaaabbaaaaaaaaaaaaac**",
+"*aaaaaaaaaaaaaaaaaaaaaaaaaaaac**",
+".*aaaaaaaaaaaabbaaaaaaaaaaaac***",
+".*aaaaaaaaaaabbbbaaaaaaaaaaac***",
+"..*aaaaaaaaaabbbbaaaaaaaaaac***.",
+"...caaaaaaaaaabbaaaaaaaaaac****.",
+"....caaaaaaaaaaaaaaaaaaaac****..",
+".....caaaaaaaaaaaaaaaaaac****...",
+"......ccaaaaaaaaaaaaaacc****....",
+".......*cccaaaaaaaaccc*****.....",
+"........***cccaaaac*******......",
+"..........****caaac*****........",
+".............*caaac**...........",
+"...............caac**...........",
+"................cac**...........",
+".................cc**...........",
+"..................***...........",
+"...................**...........",
+};
 #endif //QT_NO_IMAGEIO_XPM
 
 /*!
@@ -1258,25 +1396,27 @@ QPixmap QWindowsStyle::stylePixmap(StylePixmap stylepixmap,
 #ifndef QT_NO_IMAGEIO_XPM
     switch (stylepixmap) {
     case SP_TitleBarShadeButton:
-	return QPixmap((const char **)qt_shade_xpm);
+	return QPixmap( (const char **)qt_shade_xpm );
     case SP_TitleBarUnshadeButton:
-	return QPixmap((const char **)qt_unshade_xpm);
+	return QPixmap( (const char **)qt_unshade_xpm );
     case SP_TitleBarNormalButton:
-	return QPixmap((const char **)qt_normalizeup_xpm);
+	return QPixmap( (const char **)qt_normalizeup_xpm );
     case SP_TitleBarMinButton:
-	return QPixmap((const char **)qt_minimize_xpm);
+	return QPixmap( (const char **)qt_minimize_xpm );
     case SP_TitleBarMaxButton:
-	return QPixmap((const char **)qt_maximize_xpm);
+	return QPixmap( (const char **)qt_maximize_xpm );
     case SP_TitleBarCloseButton:
-	return QPixmap((const char **)qt_close_xpm);
+	return QPixmap( (const char **)qt_close_xpm );
     case SP_DockWindowCloseButton:
-	return QPixmap((const char **)dock_window_close_xpm );
+	return QPixmap( (const char **)dock_window_close_xpm );
     case SP_MessageBoxInformation:
-	return QPixmap((const char **)information_xpm);
+	return QPixmap( (const char **)information_xpm);
     case SP_MessageBoxWarning:
-	return QPixmap((const char **)warning_xpm);
+	return QPixmap( (const char **)warning_xpm );
     case SP_MessageBoxCritical:
-	return QPixmap((const char **)critical_xpm);
+	return QPixmap( (const char **)critical_xpm );
+    case SP_MessageBoxQuestion:
+	return QPixmap( (const char **)question_xpm );
     default:
 	break;
     }
@@ -1374,7 +1514,7 @@ void QWindowsStyle::drawComplexControl( ComplexControl ctrl, QPainter *p,
 
 		    // Expand line height to edge of rectangle if there's a
 		    // child, and it's visible
-		    if ( child && (child->height() > 0) ) { 
+		    if ( child && (child->height() > 0) ) {
 			linebot = r.height();
 		    }
 
@@ -1416,7 +1556,7 @@ void QWindowsStyle::drawComplexControl( ComplexControl ctrl, QPainter *p,
 		}
 
 		int line; // index into dotlines
-		for( line = 0; line < c; line += 2 ) {
+		if ( sub & SC_ListViewBranch ) for( line = 0; line < c; line += 2 ) {
 		    // assumptions here: lines are horizontal or vertical.
 		    // lines always start with the numerically lowest
 		    // coordinate.
@@ -1480,7 +1620,7 @@ void QWindowsStyle::drawComplexControl( ComplexControl ctrl, QPainter *p,
 	    if ( widget->isEnabled() )
 		flags |= Style_Enabled;
 
-	    if ( subActive & Style_Sunken ) {
+	    if ( subActive == SC_ComboBoxArrow ) {
 		flags |= Style_Sunken;
 	    }
 	    drawPrimitive( PE_ArrowDown, p, ar, cg, flags );
@@ -1585,7 +1725,7 @@ void QWindowsStyle::drawComplexControl( ComplexControl ctrl, QPainter *p,
 
 		p->fillRect( x, y, wi, he, cg.brush( QColorGroup::Background ) );
 
-		if ( sl->hasFocus() ) {
+		if ( flags & Style_HasFocus ) {
 		    QRect re = subRect( SR_SliderFocusRect, sl );
 		    drawPrimitive( PE_FocusRect, p, re, cg );
 		}
@@ -1762,6 +1902,10 @@ int QWindowsStyle::styleHint( StyleHint hint,
 	    ret = 0;
 	break;
 
+    case SH_ToolBox_SelectedPageTitleBold:
+	ret = 0;
+	break;
+
     default:
 	ret = QCommonStyle::styleHint(hint, widget, opt, returnData);
 	break;
@@ -1770,4 +1914,28 @@ int QWindowsStyle::styleHint( StyleHint hint,
     return ret;
 }
 
+/*! \reimp */
+QRect QWindowsStyle::subRect(SubRect r, const QWidget *widget) const
+{
+    QRect rect;
+
+    switch (r) {
+#ifndef QT_NO_SLIDER
+    case SR_SliderFocusRect:
+	{
+	    rect = widget->rect();
+	    break;
+	}
+#endif // QT_NO_SLIDER
+    case SR_ToolBoxTabContents:
+	rect = widget->rect();
+	break;
+    default:
+	rect = QCommonStyle::subRect( r, widget );
+	break;
+    }
+
+    return rect;
+}
+
 #endif
