@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id$
+** $Id: qt/qdir_unix.cpp   3.2.3   edited May 13 09:08 $
 **
 ** Implementation of QDir class
 **
 ** Created : 950628
 **
-** Copyright (C) 1992-2002 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2003 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the tools module of the Qt GUI Toolkit.
 **
@@ -45,8 +45,13 @@
 #include "qregexp.h"
 #include "qstringlist.h"
 
+#ifdef QT_THREAD_SUPPORT
+#  include <private/qmutexpool_p.h>
+#endif // QT_THREAD_SUPPORT
+
 #include <stdlib.h>
 #include <limits.h>
+#include <errno.h>
 
 
 void QDir::slashify( QString& )
@@ -66,42 +71,44 @@ QString QDir::homeDirPath()
 QString QDir::canonicalPath() const
 {
     QString r;
-
     char cur[PATH_MAX+1];
-    char tmp[PATH_MAX+1];
-    if ( QT_GETCWD( cur, PATH_MAX ) ) {
-	if ( QT_CHDIR(QFile::encodeName(dPath)) >= 0 )
-	    if ( QT_GETCWD( tmp, PATH_MAX ) )
-		r = QFile::decodeName(tmp);
-	QT_CHDIR( cur );
-    }
+    if ( ::getcwd( cur, PATH_MAX ) ) {
+	char tmp[PATH_MAX+1];
+	// need the cast for old solaris versions of realpath that doesn't take
+	// a const char*.
+	if( ::realpath( (char*)QFile::encodeName( dPath ).data(), tmp ) )
+	    r = QFile::decodeName( tmp );
+	slashify( r );
 
-    slashify( r );
+    	// always make sure we go back to the current dir
+	::chdir( cur );
+    }
     return r;
 }
 
 bool QDir::mkdir( const QString &dirName, bool acceptAbsPath ) const
 {
-#ifdef Q_OS_MACX  // Mac X doesn't support trailing /'s
+#if defined(Q_OS_MACX)  // Mac X doesn't support trailing /'s
     QString name = dirName;
     if (dirName[dirName.length() - 1] == "/")
 	name = dirName.left( dirName.length() - 1 );
-    return QT_MKDIR( QFile::encodeName(filePath(name,acceptAbsPath)), 0777 )
-	== 0;
+    int status =
+	::mkdir( QFile::encodeName(filePath(name,acceptAbsPath)), 0777 );
 #else
-    return QT_MKDIR( QFile::encodeName(filePath(dirName,acceptAbsPath)), 0777 )
-	== 0;
+    int status =
+	::mkdir( QFile::encodeName(filePath(dirName,acceptAbsPath)), 0777 );
 #endif
+    return status == 0;
 }
 
 bool QDir::rmdir( const QString &dirName, bool acceptAbsPath ) const
 {
-    return QT_RMDIR( QFile::encodeName(filePath(dirName,acceptAbsPath)) ) == 0;
+    return ::rmdir( QFile::encodeName(filePath(dirName,acceptAbsPath)) ) == 0;
 }
 
 bool QDir::isReadable() const
 {
-    return QT_ACCESS( QFile::encodeName(dPath), R_OK | X_OK ) == 0;
+    return ::access( QFile::encodeName(dPath), R_OK | X_OK ) == 0;
 }
 
 bool QDir::isRoot() const
@@ -127,7 +134,7 @@ bool QDir::rename( const QString &name, const QString &newName,
 bool QDir::setCurrent( const QString &path )
 {
     int r;
-    r = QT_CHDIR( QFile::encodeName(path) );
+    r = ::chdir( QFile::encodeName(path) );
     return r >= 0;
 }
 
@@ -135,10 +142,10 @@ QString QDir::currentDirPath()
 {
     QString result;
 
-    QT_STATBUF st;
-    if ( QT_STAT( ".", &st ) == 0 ) {
+    struct stat st;
+    if ( ::stat( ".", &st ) == 0 ) {
 	char currentName[PATH_MAX+1];
-	if ( QT_GETCWD( currentName, PATH_MAX ) != 0 )
+	if ( ::getcwd( currentName, PATH_MAX ) )
 	    result = QFile::decodeName(currentName);
 #if defined(QT_DEBUG)
 	if ( result.isNull() )
@@ -182,7 +189,7 @@ bool QDir::readDirEntries( const QString &nameFilter,
 	fiList->clear();
     }
 
-    QStringList filters = qt_makeFilterList( nameFilter );
+    QValueList<QRegExp> filters = qt_makeFilterList( nameFilter );
 
     bool doDirs	    = (filterSpec & Dirs)	!= 0;
     bool doFiles    = (filterSpec & Files)	!= 0;
@@ -193,11 +200,6 @@ bool QDir::readDirEntries( const QString &nameFilter,
     bool doHidden   = (filterSpec & Hidden)	!= 0;
     bool doSystem   = (filterSpec & System)     != 0;
 
-#if defined(Q_OS_OS2EMX)
-    //QRegExp   wc( nameFilter, FALSE, TRUE );	// wild card, case insensitive
-#else
-    //QRegExp   wc( nameFilter, TRUE, TRUE );	// wild card, case sensitive
-#endif
     QFileInfo fi;
     DIR	     *dir;
     dirent   *file;
@@ -206,10 +208,19 @@ bool QDir::readDirEntries( const QString &nameFilter,
     if ( !dir )
 	return FALSE; // cannot read the directory
 
-    while ( (file = readdir(dir)) ) {
+#if defined(QT_THREAD_SUPPORT) && defined(_POSIX_THREAD_SAFE_FUNCTIONS) && !defined(Q_OS_CYGWIN)
+    union {
+	struct dirent mt_file;
+	char b[sizeof(struct dirent) + MAXNAMLEN + 1];
+    } u;
+    while ( readdir_r(dir, &u.mt_file, &file ) == 0 && file )
+#else
+    while ( (file = readdir(dir)) )
+#endif // QT_THREAD_SUPPORT && _POSIX_THREAD_SAFE_FUNCTIONS
+    {
 	QString fn = QFile::decodeName(file->d_name);
 	fi.setFile( *this, fn );
-	if ( !match( filters, fn ) && !(allDirs && fi.isDir()) )
+	if ( !qt_matchFilterList(filters, fn) && !(allDirs && fi.isDir()) )
 	     continue;
 	if  ( (doDirs && fi.isDir()) || (doFiles && fi.isFile()) ||
 	      (doSystem && (!fi.isFile() && !fi.isDir())) ) {
@@ -270,9 +281,17 @@ const QFileInfoList * QDir::drives()
     static QFileInfoList * knownMemoryLeak = 0;
 
     if ( !knownMemoryLeak ) {
-	knownMemoryLeak = new QFileInfoList;
-	// non-win32 versions both use just one root directory
-	knownMemoryLeak->append( new QFileInfo( rootDirPath() ) );
+
+#ifdef QT_THREAD_SUPPORT
+	QMutexLocker locker( qt_global_mutexpool ?
+			     qt_global_mutexpool->get( &knownMemoryLeak ) : 0 );
+#endif // QT_THREAD_SUPPORT
+
+	if ( !knownMemoryLeak ) {
+	    knownMemoryLeak = new QFileInfoList;
+	    // non-win32 versions both use just one root directory
+	    knownMemoryLeak->append( new QFileInfo( rootDirPath() ) );
+	}
     }
 
     return knownMemoryLeak;
