@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id$
+** $Id: qt/qfile_unix.cpp   3.2.3   edited Oct 3 09:32 $
 **
-** Implementation of QFileInfo class
+** Implementation of QFile class
 **
 ** Created : 950628
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2002 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the tools module of the Qt GUI Toolkit.
 **
@@ -36,17 +36,30 @@
 **********************************************************************/
 
 #include "qplatformdefs.h"
-#include "qfile.h"
 
+// POSIX Large File Support redefines open -> open64
+static inline int qt_open(const char *pathname, int flags, mode_t mode)
+{ return ::open(pathname, flags, mode); }
+#if defined(open)
+# undef open
+#endif
+
+// POSIX Large File Support redefines truncate -> truncate64
+#if defined(truncate)
+# undef truncate
+#endif
+
+#include "qfile.h"
 #include <errno.h>
 #include <limits.h>
 
+extern const char* qt_fileerr_read;
 
 bool qt_file_access( const QString& fn, int t )
 {
     if ( fn.isEmpty() )
 	return FALSE;
-    return QT_ACCESS( QFile::encodeName(fn), t ) == 0;
+    return ::access( QFile::encodeName(fn), t ) == 0;
 }
 
 /*!
@@ -64,7 +77,6 @@ bool QFile::remove( const QString &fileName )
 	return FALSE;
     }
     return unlink( QFile::encodeName(fileName) ) == 0;
-    // unlink more common in UNIX
 }
 
 #if defined(O_NONBLOCK)
@@ -76,60 +88,77 @@ bool QFile::remove( const QString &fileName )
 #endif
 
 /*!
-  Opens the file specified by the file name currently set, using the mode \a m.
-  Returns TRUE if successful, otherwise FALSE.
+    Opens the file specified by the file name currently set, using the
+    mode \a m. Returns TRUE if successful, otherwise FALSE.
 
-  \keyword IO_Raw
-  \keyword IO_ReadOnly
-  \keyword IO_WriteOnly
-  \keyword IO_ReadWrite
-  \keyword IO_Append
-  \keyword IO_Truncate
-  \keyword IO_Translate
+    \keyword IO_Raw
+    \keyword IO_ReadOnly
+    \keyword IO_WriteOnly
+    \keyword IO_ReadWrite
+    \keyword IO_Append
+    \keyword IO_Truncate
+    \keyword IO_Translate
 
-  The mode parameter \a m must be a combination of the following flags:
-  \list
-  \i \c IO_Raw specified raw (non-buffered) file access.
-  \i \c IO_ReadOnly opens the file in read-only mode.
-  \i \c IO_WriteOnly opens the file in write-only mode (and truncates).
-  \i \c IO_ReadWrite opens the file in read/write mode, equivalent to
-     \c (IO_ReadOnly | IO_WriteOnly).
-  \i \c IO_Append opens the file in append mode. This mode is very useful
-  when you want to write something to a log file. The file index is set to
-  the end of the file. Note that the result is undefined if you position the
-  file index manually using at() in append mode.
-  \i \c IO_Truncate truncates the file.
-  \i \c IO_Translate enables carriage returns and linefeed translation
-  for text files under MS-DOS, Windows and OS/2.
-  \endlist
+    The mode parameter \a m must be a combination of the following flags:
+    \table
+    \header \i Flag \i Meaning
+    \row \i IO_Raw
+	 \i Raw (non-buffered) file access.
+    \row \i IO_ReadOnly
+	 \i Opens the file in read-only mode.
+    \row \i IO_WriteOnly
+	 \i Opens the file in write-only mode. If this flag is used
+	    with another flag, e.g. \c IO_ReadOnly or \c IO_Raw or \c
+	    IO_Append, the file is \e not truncated; but if used on
+	    its own (or with \c IO_Truncate), the file is truncated.
+    \row \i IO_ReadWrite
+	 \i Opens the file in read/write mode, equivalent to \c
+	    (IO_ReadOnly | IO_WriteOnly).
+    \row \i IO_Append
+	 \i Opens the file in append mode. (You must actually use \c
+	    (IO_WriteOnly | IO_Append) to make the file writable and
+	    to go into append mode.) This mode is very useful when you
+	    want to write something to a log file. The file index is
+	    set to the end of the file. Note that the result is
+	    undefined if you position the file index manually using
+	    at() in append mode.
+    \row \i IO_Truncate
+	 \i Truncates the file.
+    \row \i IO_Translate
+	 \i Enables carriage returns and linefeed translation for text
+	    files under Windows.
+    \endtable
 
-  The raw access mode is best when I/O is block-operated using 4kB block size
-  or greater. Buffered access works better when reading small portions of
-  data at a time.
-
-  <strong>Important:</strong> When working with buffered files, data may
-  not be written to the file at once. Call flush()
-  to make sure that the data is really written.
+    The raw access mode is best when I/O is block-operated using a 4KB
+    block size or greater. Buffered access works better when reading
+    small portions of data at a time.
 
+    \warning When working with buffered files, data may not be written
+    to the file at once. Call flush() to make sure that the data is
+    really written.
 
     \warning If you have a buffered file opened for both reading and
     writing you must not perform an input operation immediately after
     an output operation or vice versa. You should always call flush()
-    or a file positioning operation, e.g. seek(), between input and
+    or a file positioning operation, e.g. at(), between input and
     output operations, otherwise the buffer may contain garbage.
 
-  If the file does not exist and \c IO_WriteOnly or \c IO_ReadWrite is
-  specified, it is created.
+    If the file does not exist and \c IO_WriteOnly or \c IO_ReadWrite
+    is specified, it is created.
 
-  Example:
-  \code
-    QFile f1( "/tmp/data.bin" );
-    QFile f2( "readme.txt" );
-    f1.open( IO_Raw | IO_ReadWrite | IO_Append );
-    f2.open( IO_ReadOnly | IO_Translate );
-  \endcode
+    Example:
+    \code
+	QFile f1( "/tmp/data.bin" );
+	f1.open( IO_Raw | IO_ReadWrite );
 
-  \sa name(), close(), isOpen(), flush()
+	QFile f2( "readme.txt" );
+	f2.open( IO_ReadOnly | IO_Translate );
+
+	QFile f3( "audit.log" );
+	f3.open( IO_WriteOnly | IO_Append );
+    \endcode
+
+    \sa name(), close(), isOpen(), flush()
 */
 
 bool QFile::open( int m )
@@ -140,7 +169,7 @@ bool QFile::open( int m )
 #endif
 	return FALSE;
     }
-    if ( fn.isNull() ) {			// no file name defined
+    if ( fn.isEmpty() ) {			// no file name defined
 #if defined(QT_CHECK_NULL)
 	qWarning( "QFile::open: No file name specified" );
 #endif
@@ -155,24 +184,24 @@ bool QFile::open( int m )
 	return FALSE;
     }
     bool ok = TRUE;
-    QT_STATBUF st;
-    if ( isRaw() ) {				// raw file I/O
-	int oflags = QT_OPEN_RDONLY;
+    struct stat st;
+    if ( isRaw() ) {
+	int oflags = O_RDONLY;
 	if ( isReadable() && isWritable() )
-	    oflags = QT_OPEN_RDWR;
+	    oflags = O_RDWR;
 	else if ( isWritable() )
-	    oflags = QT_OPEN_WRONLY;
+	    oflags = O_WRONLY;
 	if ( flags() & IO_Append ) {		// append to end of file?
 	    if ( flags() & IO_Truncate )
-		oflags |= (QT_OPEN_CREAT | QT_OPEN_TRUNC);
+		oflags |= (O_CREAT | O_TRUNC);
 	    else
-		oflags |= (QT_OPEN_APPEND | QT_OPEN_CREAT);
+		oflags |= (O_APPEND | O_CREAT);
 	    setFlags( flags() | IO_WriteOnly ); // append implies write
 	} else if ( isWritable() ) {		// create/trunc if writable
 	    if ( flags() & IO_Truncate )
-		oflags |= (QT_OPEN_CREAT | QT_OPEN_TRUNC);
+		oflags |= (O_CREAT | O_TRUNC);
 	    else
-		oflags |= QT_OPEN_CREAT;
+		oflags |= O_CREAT;
 	}
 #if defined(HAS_TEXT_FILEMODE)
 	if ( isTranslated() )
@@ -184,10 +213,10 @@ bool QFile::open( int m )
 	if ( isAsynchronous() )
 	    oflags |= OPEN_ASYNC;
 #endif
-	fd = QT_OPEN( QFile::encodeName(fn), oflags, 0666 );
+	fd = qt_open( QFile::encodeName(fn), oflags, 0666 );
 
 	if ( fd != -1 ) {			// open successful
-	    QT_FSTAT( fd, &st ); // get the stat for later usage
+	    ::fstat( fd, &st );			// get the stat for later usage
 	} else {
 	    ok = FALSE;
 	}
@@ -231,7 +260,7 @@ bool QFile::open( int m )
 	    }
 	}
 	if ( fh ) {
-	    QT_FSTAT( QT_FILENO(fh), &st ); // get the stat for later usage
+	    ::fstat( fileno(fh), &st );		// get the stat for later usage
 	} else {
 	    ok = FALSE;
 	}
@@ -240,14 +269,14 @@ bool QFile::open( int m )
 	setState( IO_Open );
 	// on successful open the file stat was got; now test what type
 	// of file we have
-	if ( (st.st_mode & QT_STAT_MASK) != QT_STAT_REG ) {
+	if ( (st.st_mode & S_IFMT) != S_IFREG ) {
 	    // non-seekable
 	    setType( IO_Sequential );
 	    length = INT_MAX;
-	    ioIndex  = 0;
+	    ioIndex = 0;
 	} else {
-	    length = (int)st.st_size;
-	    ioIndex  = (flags() & IO_Append) == 0 ? 0 : length;
+	    length = (Offset)st.st_size;
+	    ioIndex = (flags() & IO_Append) == 0 ? 0 : length;
 	    if ( !(flags()&IO_Truncate) && length == 0 && isReadable() ) {
 		// try if you can read from it (if you can, it's a sequential
 		// device; e.g. a file in the /proc filesystem)
@@ -256,8 +285,9 @@ bool QFile::open( int m )
 		    ungetch(c);
 		    setType( IO_Sequential );
 		    length = INT_MAX;
-		    ioIndex  = 0;
+		    ioIndex = 0;
 		}
+		resetStatus();
 	    }
 	}
     } else {
@@ -266,6 +296,7 @@ bool QFile::open( int m )
 	    setStatus( IO_ResourceError );
 	else
 	    setStatus( IO_OpenError );
+	setErrorStringErrno( errno );
     }
     return ok;
 }
@@ -311,16 +342,20 @@ bool QFile::open( int m, FILE *f )
     setState( IO_Open );
     fh = f;
     ext_f = TRUE;
-    QT_STATBUF st;
-    QT_FSTAT( QT_FILENO(fh), &st );
-    ioIndex = (int)ftell( fh );
-    if ( (st.st_mode & QT_STAT_MASK) != QT_STAT_REG || f == stdin ) { //stdin is non seekable
+    struct stat st;
+    ::fstat( fileno(fh), &st );
+#if defined(QT_LARGEFILE_SUPPORT)
+    ioIndex = (Offset)ftello( fh );
+#else
+    ioIndex = (Offset)ftell( fh );
+#endif
+    if ( (st.st_mode & S_IFMT) != S_IFREG || f == stdin ) { //stdin is non seekable
 	// non-seekable
 	setType( IO_Sequential );
 	length = INT_MAX;
-	ioIndex  = 0;
+	ioIndex = 0;
     } else {
-	length = (int)st.st_size;
+	length = (Offset)st.st_size;
 	if ( !(flags()&IO_Truncate) && length == 0 && isReadable() ) {
 	    // try if you can read from it (if you can, it's a sequential
 	    // device; e.g. a file in the /proc filesystem)
@@ -329,8 +364,9 @@ bool QFile::open( int m, FILE *f )
 		ungetch(c);
 		setType( IO_Sequential );
 		length = INT_MAX;
-		ioIndex  = 0;
+		ioIndex = 0;
 	    }
+	    resetStatus();
 	}
     }
     return TRUE;
@@ -369,16 +405,16 @@ bool QFile::open( int m, int f )
     setState( IO_Open );
     fd = f;
     ext_f = TRUE;
-    QT_STATBUF st;
-    QT_FSTAT( fd, &st );
-    ioIndex  = (int)QT_LSEEK(fd, 0, SEEK_CUR);
-    if ( (st.st_mode & QT_STAT_MASK) != QT_STAT_REG || f == 0 ) { // stdin is not seekable...
+    struct stat st;
+    ::fstat( fd, &st );
+    ioIndex = (Offset)::lseek(fd, 0, SEEK_CUR);
+    if ( (st.st_mode & S_IFMT) != S_IFREG || f == 0 ) { // stdin is not seekable...
 	// non-seekable
 	setType( IO_Sequential );
 	length = INT_MAX;
-	ioIndex  = 0;
+	ioIndex = 0;
     } else {
-	length = (int)st.st_size;
+	length = (Offset)st.st_size;
 	if ( length == 0 && isReadable() ) {
 	    // try if you can read from it (if you can, it's a sequential
 	    // device; e.g. a file in the /proc filesystem)
@@ -387,7 +423,7 @@ bool QFile::open( int m, int f )
 		ungetch(c);
 		setType( IO_Sequential );
 		length = INT_MAX;
-		ioIndex  = 0;
+		ioIndex = 0;
 	    }
 	    resetStatus();
 	}
@@ -402,36 +438,45 @@ bool QFile::open( int m, int f )
 
 QIODevice::Offset QFile::size() const
 {
-    QT_STATBUF st;
+    struct stat st;
+    int ret = 0;
     if ( isOpen() ) {
-	QT_FSTAT( fh ? QT_FILENO(fh) : fd, &st );
+	ret = ::fstat( fh ? fileno(fh) : fd, &st );
     } else {
-	QT_STAT( QFile::encodeName(fn), &st );
+	ret = ::stat( QFile::encodeName(fn), &st );
     }
+    if ( ret == -1 )
+	return 0;
+#if defined(QT_LARGEFILE_SUPPORT) && !defined(QT_ABI_64BITOFFSET)
+    return (uint)st.st_size > UINT_MAX ? UINT_MAX : (QIODevice::Offset)st.st_size;
+#else
     return st.st_size;
+#endif
 }
 
 
 /*!
-  Sets the file index to \a pos. Returns TRUE if successful; otherwise
-  returns FALSE.
+    \overload
 
-  Example:
-  \code
+    Sets the file index to \a pos. Returns TRUE if successful;
+    otherwise returns FALSE.
+
+    Example:
+    \code
     QFile f( "data.bin" );
-    f.open( IO_ReadOnly );			// index set to 0
-    f.at( 100 );				// set index to 100
-    f.at( f.at()+50 );				// set index to 150
-    f.at( f.size()-80 );			// set index to 80 before EOF
+    f.open( IO_ReadOnly );  // index set to 0
+    f.at( 100 );	    // set index to 100
+    f.at( f.at()+50 );	    // set index to 150
+    f.at( f.size()-80 );    // set index to 80 before EOF
     f.close();
-  \endcode
+    \endcode
 
-  Use \c at() without arguments to retrieve the file offset.
+    Use \c at() without arguments to retrieve the file offset.
 
-  \warning The result is undefined if the file was open()'ed
-  using the \c IO_Append specifier.
+    \warning The result is undefined if the file was open()'ed using
+    the \c IO_Append specifier.
 
-  \sa size(), open()
+    \sa size(), open()
 */
 
 bool QFile::at( Offset pos )
@@ -445,76 +490,86 @@ bool QFile::at( Offset pos )
     if ( isSequentialAccess() )
 	return FALSE;
     bool ok;
-    if ( isRaw() ) {				// raw file
-	pos = (long int) QT_LSEEK( fd, pos, SEEK_SET );
-	ok = ( (long int) pos != -1 );
+    if ( isRaw() ) {
+	off_t l = ::lseek( fd, pos, SEEK_SET );
+	ok = ( l != -1 );
+	pos = (Offset)l;
     } else {					// buffered file
-	ok = ( fseek(fh, pos, SEEK_SET) == 0 );
+#if defined(QT_LARGEFILE_SUPPORT)
+	ok = ( ::fseeko(fh, pos, SEEK_SET) == 0 );
+#else
+	ok = ( ::fseek(fh, pos, SEEK_SET) == 0 );
+#endif
     }
     if ( ok )
 	ioIndex = pos;
 #if defined(QT_CHECK_RANGE)
     else
-	qWarning( "QFile::at: Cannot set file position %ld", pos );
+#if defined(QT_LARGEFILE_SUPPORT) && defined(QT_ABI_64BITOFFSET)
+	qWarning( "QFile::at: Cannot set file position %llu", pos );
+#else
+	qWarning( "QFile::at: Cannot set file position %lu", pos );
+#endif
 #endif
     return ok;
 }
 
 /*!
-  Reads at most \a len bytes from the file into \a p and returns the
-  number of bytes actually read.
-
-  Returns -1 if a serious error occurred.
+  \reimp
 
   \warning We have experienced problems with some C libraries when a buffered
   file is opened for both reading and writing. If a read operation takes place
   immediately after a write operation, the read buffer contains garbage data.
   Worse, the same garbage is written to the file. Calling flush() before
   readBlock() solved this problem.
-
-  \sa writeBlock()
 */
 
 Q_LONG QFile::readBlock( char *p, Q_ULONG len )
 {
+    if ( !len ) // nothing to do
+	return 0;
+
 #if defined(QT_CHECK_NULL)
     if ( !p )
 	qWarning( "QFile::readBlock: Null pointer error" );
 #endif
 #if defined(QT_CHECK_STATE)
-    if ( !isOpen() ) {				// file not open
+    if ( !isOpen() ) {
 	qWarning( "QFile::readBlock: File not open" );
-	return 0;
+	return -1;
     }
-    if ( !isReadable() ) {			// reading not permitted
+    if ( !isReadable() ) {
 	qWarning( "QFile::readBlock: Read operation not permitted" );
-	return 0;
+	return -1;
     }
 #endif
-    int nread = 0;					// number of bytes read
+    Q_ULONG nread = 0;					// number of bytes read
     if ( !ungetchBuffer.isEmpty() ) {
 	// need to add these to the returned string.
-	int l = ungetchBuffer.length();
+	uint l = ungetchBuffer.length();
 	while( nread < l ) {
-	    *p = ungetchBuffer[ l - nread - 1 ];
+	    *p = ungetchBuffer.at( l - nread - 1 );
 	    p++;
 	    nread++;
 	}
 	ungetchBuffer.truncate( l - nread );
     }
 
-    if ( nread < (int)len ) {
+    if ( nread < len ) {
 	if ( isRaw() ) {				// raw file
-	    nread += QT_READ( fd, p, len-nread );
+	    nread += ::read( fd, p, len-nread );
 	    if ( len && nread <= 0 ) {
 		nread = 0;
 		setStatus(IO_ReadError);
+		setErrorStringErrno( errno );
 	    }
 	} else {					// buffered file
 	    nread += fread( p, 1, len-nread, fh );
 	    if ( (uint)nread != len ) {
-		if ( ferror( fh ) || nread==0 )
+		if ( ferror( fh ) || nread==0 ) {
 		    setStatus(IO_ReadError);
+		    setErrorString( qt_fileerr_read );
+		}
 	    }
 	}
     }
@@ -523,8 +578,6 @@ Q_LONG QFile::readBlock( char *p, Q_ULONG len )
     return nread;
 }
 
-/*! \overload Q_ULONG QFile::writeBlock( const QByteArray& data )
-*/
 
 /*! \reimp
 
@@ -542,6 +595,9 @@ Q_LONG QFile::readBlock( char *p, Q_ULONG len )
 
 Q_LONG QFile::writeBlock( const char *p, Q_ULONG len )
 {
+    if ( !len ) // nothing to do
+	return 0;
+
 #if defined(QT_CHECK_NULL)
     if ( p == 0 && len != 0 )
 	qWarning( "QFile::writeBlock: Null pointer error" );
@@ -556,21 +612,26 @@ Q_LONG QFile::writeBlock( const char *p, Q_ULONG len )
 	return -1;
     }
 #endif
-    int nwritten;				// number of bytes written
+    Q_ULONG nwritten;				// number of bytes written
     if ( isRaw() )				// raw file
-	nwritten = QT_WRITE( fd, (void *)p, len );
+	nwritten = ::write( fd, (void *)p, len );
     else					// buffered file
 	nwritten = fwrite( p, 1, len, fh );
-    if ( nwritten != (int)len ) {		// write error
+    if ( nwritten != len ) {		// write error
 	if ( errno == ENOSPC )			// disk is full
 	    setStatus( IO_ResourceError );
 	else
 	    setStatus( IO_WriteError );
+	setErrorStringErrno( errno );
 	if ( !isSequentialAccess() ) {
-	    if ( isRaw() )				// recalc file position
-		ioIndex = (int)QT_LSEEK( fd, 0, SEEK_CUR );
+	    if ( isRaw() )			// recalc file position
+		ioIndex = (Offset)::lseek( fd, 0, SEEK_CUR );
 	    else
-		ioIndex = fseek( fh, 0, SEEK_CUR );
+#if defined(QT_LARGEFILE_SUPPORT)
+		ioIndex = (Offset)::fseeko( fh, 0, SEEK_CUR );
+#else
+		ioIndex = (Offset)::fseek( fh, 0, SEEK_CUR );
+#endif
 	}
     } else {
 	if ( !isSequentialAccess() )
@@ -585,7 +646,9 @@ Q_LONG QFile::writeBlock( const char *p, Q_ULONG len )
   Returns the file handle of the file.
 
   This is a small positive integer, suitable for use with C library
-  functions such as fdopen() and fcntl(), as well as with QSocketNotifier.
+  functions such as fdopen() and fcntl(). On systems that use file
+  descriptors for sockets (ie. Unix systems, but not Windows) the handle
+  can be used with QSocketNotifier as well.
 
   If the file is not open or there is an error, handle() returns -1.
 
@@ -597,7 +660,7 @@ int QFile::handle() const
     if ( !isOpen() )
 	return -1;
     else if ( fh )
-	return QT_FILENO( fh );
+	return fileno( fh );
     else
 	return fd;
 }
@@ -632,12 +695,12 @@ void QFile::close()
 	    if ( ext_f )
 		ok = TRUE;			// cannot close
 	    else
-		ok = QT_CLOSE( fd ) != -1;
+		ok = ::close( fd ) != -1;
 	}
 	init();					// restore internal state
     }
-    if (!ok)
-	setStatus (IO_UnspecifiedError);
-
-    return;
+    if (!ok) {
+	setStatus( IO_UnspecifiedError );
+	setErrorStringErrno( errno );
+    }
 }
