@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id$
+** $Id: qt/qgdict.cpp   3.2.3   edited May 13 09:08 $
 **
 ** Implementation of QGDict and QGDictIterator classes
 **
@@ -43,6 +43,7 @@
 
 /*!
   \class QGDict
+  \reentrant
   \ingroup collection
   \brief The QGDict class is an internal class for implementing QDict template classes.
 
@@ -203,7 +204,9 @@ QGDict::QGDict( uint len, KeyType kt, bool caseSensitive, bool copyKeys )
 
 void QGDict::init( uint len, KeyType kt, bool caseSensitive, bool copyKeys )
 {
-    vec = new QBaseBucket *[vlen = len];		// allocate hash table
+    vlen = len ? len : 17;
+    vec = new QBaseBucket *[ vlen ];
+
     Q_CHECK_PTR( vec );
     memset( (char*)vec, 0, vlen*sizeof(QBaseBucket*) );
     numItems  = 0;
@@ -992,6 +995,7 @@ QDataStream& QGDict::write( QDataStream &s ) const
 
 /*!
   \class QGDictIterator qgdict.h
+  \reentrant
   \ingroup collection
   \brief The QGDictIterator class is an internal class for implementing QDictIterator and QIntDictIterator.
 
