@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id$
+** $Id: qt/qglobal.cpp   3.2.3   edited Oct 27 20:12 $
 **
 ** Global functions
 **
 ** Created : 920604
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2003 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the tools module of the Qt GUI Toolkit.
 **
@@ -38,15 +38,26 @@
 #include "qplatformdefs.h"
 
 #include "qasciidict.h"
+#include <limits.h>
 #include <stdio.h>
+#include <limits.h>
 #include <stdarg.h>
 #include <stdlib.h>
 
-// NOT REVISED
+#if defined(Q_CC_MSVC) && !defined(Q_OS_TEMP)
+#include <crtdbg.h>
+#endif
+
 
 /*!
-  \relates QApplication
-  Returns the Qt version number for the library, typically "1.44" or "2.3.0".
+    \relates QApplication
+
+    Returns the Qt version number as a string, for example, "2.3.0" or
+    "3.0.5".
+
+    The \c QT_VERSION define has the numeric value in the form:
+    0xmmiibb (m = major, i = minor, b = bugfix). For example, Qt
+    3.0.5's \c QT_VERSION is 0x030005.
 */
 
 const char *qVersion()
@@ -54,6 +65,14 @@ const char *qVersion()
     return QT_VERSION_STR;
 }
 
+bool qSharedBuild()
+{
+#ifdef QT_SHARED
+    return TRUE;
+#else
+    return FALSE;
+#endif
+}
 
 /*****************************************************************************
   System detection routines
@@ -64,16 +83,17 @@ static int  si_wordSize;
 static bool si_bigEndian;
 
 /*!
-  \relates QApplication
-  Obtains information about the system.
+    \relates QApplication
 
-  The system's word size in bits (typically 32) is returned in \a wordSize.
-  The \a bigEndian is set to TRUE if this is a big-endian machine,
-  or to FALSE if this is a little-endian machine.
+    Obtains information about the system.
 
-  In debug mode, this function calls qFatal() with a message if the computer is
-  truly weird (i.e. different endianness for 16 bit and 32 bit integers), in
-  release mode it returns FALSE.
+    The system's word size in bits (typically 32) is returned in \a
+    *wordSize. The \a *bigEndian is set to TRUE if this is a big-endian
+    machine, or to FALSE if this is a little-endian machine.
+
+    In debug mode, this function calls qFatal() with a message if the
+    computer is truly weird (i.e. different endianness for 16 bit and
+    32 bit integers); in release mode it returns FALSE.
 */
 
 bool qSysInfo( int *wordSize, bool *bigEndian )
@@ -88,7 +108,6 @@ bool qSysInfo( int *wordSize, bool *bigEndian )
 	*bigEndian = si_bigEndian;
 	return TRUE;
     }
-    si_alreadyDone = TRUE;
 
     si_wordSize = 0;
     Q_ULONG n = (Q_ULONG)(~0);
@@ -138,11 +157,41 @@ bool qSysInfo( int *wordSize, bool *bigEndian )
     }
 
     *bigEndian = si_bigEndian = be32;
+    si_alreadyDone = TRUE;
     return TRUE;
 }
 
-#if defined(Q_OS_WIN32)
+#if !defined(QWS) && defined(Q_OS_MAC)
+
+#include "qt_mac.h"
+
+int qMacVersion()
+{
+    static int macver = Qt::MV_Unknown;
+    static bool first = TRUE;
+    if(first) {
+	first = FALSE;
+	long gestalt_version;
+	if(Gestalt(gestaltSystemVersion, &gestalt_version) == noErr) {
+	    if(gestalt_version >= 0x1030 && gestalt_version < 0x1040)
+		macver = Qt::MV_10_DOT_3;
+	    else if(gestalt_version >= 0x1020 && gestalt_version < 0x1030)
+		macver = Qt::MV_10_DOT_2;
+	    else if(gestalt_version >= 0x1010 && gestalt_version < 0x1020)
+		macver = Qt::MV_10_DOT_1;
+	    else if(gestalt_version >= 0x1000 && gestalt_version < 0x1010)
+		macver = Qt::MV_10_DOT_0;
+
+	}
+    }
+    return macver;
+}
+Qt::MacintoshVersion qt_macver = (Qt::MacintoshVersion)qMacVersion();
+#elif defined(Q_OS_WIN32) || defined(Q_OS_CYGWIN)
+bool qt_winunicode;
+
 #include "qt_windows.h"
+
 int qWinVersion()
 {
 #ifndef VER_PLATFORM_WIN32s
@@ -189,8 +238,18 @@ int qWinVersion()
 	    }
 	}
     }
+
+#if defined(UNICODE)
+    if ( winver & Qt::WV_NT_based )
+	qt_winunicode = TRUE;
+    else
+#endif
+	qt_winunicode = FALSE;
+
     return winver;
 }
+
+Qt::WindowsVersion qt_winver = (Qt::WindowsVersion)qWinVersion();
 #endif
 
 
@@ -199,87 +258,99 @@ int qWinVersion()
  *****************************************************************************/
 
 /*!
-  \fn void qDebug( const char *msg, ... )
+    \fn void qDebug( const char *msg, ... )
 
-  \relates QApplication
-  Prints a debug message \a msg, or calls the message handler (if it has been
-  installed).
+    \relates QApplication
 
-  This function takes a format string and a list of arguments, similar to
-  the C printf() function.
+    Prints a debug message \a msg, or calls the message handler (if it
+    has been installed).
 
-  Example:
-  \code
-    qDebug( "my window handle = %x", myWidget->id() );
-  \endcode
+    This function takes a format string and a list of arguments,
+    similar to the C printf() function.
 
-  Under X11, the text is printed to stderr.  Under Windows, the text is
-  sent to the debugger.
+    Example:
+    \code
+	qDebug( "my window handle = %x", myWidget->id() );
+    \endcode
 
-  \warning The internal buffer is limited to 8196 bytes (including the
-  0-terminator).
+    Under X11, the text is printed to stderr. Under Windows, the text
+    is sent to the debugger.
 
-  \sa qWarning(), qFatal(), qInstallMsgHandler(),
-  \link debug.html Debugging\endlink
+    \warning The internal buffer is limited to 8196 bytes (including
+    the '\0'-terminator).
+
+    \warning Passing (const char *)0 as argument to qDebug might lead
+    to crashes on certain platforms due to the platforms printf implementation.
+
+    \sa qWarning(), qFatal(), qInstallMsgHandler(),
+	\link debug.html Debugging\endlink
 */
 
 /*!
-  \fn void qWarning( const char *msg, ... )
+    \fn void qWarning( const char *msg, ... )
 
-  \relates QApplication
-  Prints a warning message \a msg, or calls the message handler (if it has been
-  installed).
+    \relates QApplication
 
-  This function takes a format string and a list of arguments, similar to
-  the C printf() function.
+    Prints a warning message \a msg, or calls the message handler (if
+    it has been installed).
 
-  Example:
-  \code
-    void f( int c )
-    {
-	if ( c > 200 )
-	    qWarning( "f: bad argument, c == %d", c );
-    }
-  \endcode
+    This function takes a format string and a list of arguments,
+    similar to the C printf() function.
 
-  Under X11, the text is printed to stderr.  Under Windows, the text is
-  sent to the debugger.
+    Example:
+    \code
+	void f( int c )
+	{
+	    if ( c > 200 )
+		qWarning( "f: bad argument, c == %d", c );
+	}
+    \endcode
 
-  \warning The internal buffer is limited to 8196 bytes (including the
-  0-terminator).
+    Under X11, the text is printed to stderr. Under Windows, the text
+    is sent to the debugger.
 
-  \sa qDebug(), qFatal(), qInstallMsgHandler(),
-  \link debug.html Debugging\endlink
+    \warning The internal buffer is limited to 8196 bytes (including
+    the '\0'-terminator).
+
+    \warning Passing (const char *)0 as argument to qWarning might lead
+    to crashes on certain platforms due to the platforms printf implementation.
+
+    \sa qDebug(), qFatal(), qInstallMsgHandler(),
+    \link debug.html Debugging\endlink
 */
 
 /*!
-  \fn void qFatal( const char *msg, ... )
+    \fn void qFatal( const char *msg, ... )
 
-  \relates QApplication
-  Prints a fatal error message \a msg and exits, or calls the message handler (if it
-  has been installed).
+    \relates QApplication
 
-  This function takes a format string and a list of arguments, similar to
-  the C printf() function.
+    Prints a fatal error message \a msg and exits, or calls the
+    message handler (if it has been installed).
 
-  Example:
-  \code
-    int divide( int a, int b )
-    {
-	if ( b == 0 )				// program error
-	    qFatal( "divide: cannot divide by zero" );
-	return a/b;
-    }
-  \endcode
+    This function takes a format string and a list of arguments,
+    similar to the C printf() function.
 
-  Under X11, the text is printed to stderr.  Under Windows, the text is
-  sent to the debugger.
+    Example:
+    \code
+	int divide( int a, int b )
+	{
+	    if ( b == 0 )				// program error
+		qFatal( "divide: cannot divide by zero" );
+	    return a/b;
+	}
+    \endcode
 
-  \warning The internal buffer is limited to 8196 bytes (including the
-  0-terminator).
+    Under X11, the text is printed to stderr. Under Windows, the text
+    is sent to the debugger.
 
-  \sa qDebug(), qWarning(), qInstallMsgHandler(),
-  \link debug.html Debugging\endlink
+    \warning The internal buffer is limited to 8196 bytes (including
+    the '\0'-terminator).
+
+    \warning Passing (const char *)0 as argument to qFatal might lead
+    to crashes on certain platforms due to the platforms printf implementation.
+
+    \sa qDebug(), qWarning(), qInstallMsgHandler(),
+    \link debug.html Debugging\endlink
 */
 
 
@@ -288,22 +359,43 @@ static const int QT_BUFFER_LENGTH = 8196;	// internal buffer length
 
 
 #ifdef Q_OS_MAC
-const unsigned char * p_str(const char * c, int len=-1)
+CFStringRef qstring2cfstring(const QString &str)
 {
-    const int maxlen = 255;
-    static unsigned char ret[maxlen+2];
+    return CFStringCreateWithCharacters(0, (UniChar *)str.unicode(), str.length());
+}
 
-    if(len == -1)
-	len = qstrlen(c);
-    if(len > maxlen)
-	qWarning( "p_str len must never exceed %d", maxlen );
-    ret[0]=len;
-    memcpy(((char *)ret)+1,c,len);
-    ret[len+1] = '\0';
+QString cfstring2qstring(CFStringRef str)
+{
+    if(!str)
+	return QString();
+
+    CFIndex length = CFStringGetLength(str);
+    if(const UniChar *chars = CFStringGetCharactersPtr(str))
+	return QString((QChar *)chars, length);
+    UniChar *buffer = (UniChar*)malloc(length * sizeof(UniChar));
+    CFStringGetCharacters(str, CFRangeMake(0, length), buffer);
+    QString ret((QChar *)buffer, length);
+    free(buffer);
     return ret;
 }
 
-const unsigned char * p_str(const QString &s)
+unsigned char * p_str(const char * c, int len=-1)
+{
+    const int maxlen = 255;
+    if(len == -1)
+	len = qstrlen(c);
+    if(len > maxlen) {
+	qWarning( "p_str len must never exceed %d", maxlen );
+	len = maxlen;
+    }
+    unsigned char *ret = (unsigned char*)malloc(len+2);
+    *ret=len;
+    memcpy(((char *)ret)+1,c,len);
+    *(ret+len+1) = '\0';
+    return ret;
+}
+
+unsigned char * p_str(const QString &s)
 {
     return p_str(s, s.length());
 }
@@ -326,10 +418,13 @@ QCString p2qstring(const unsigned char *c) {
 extern bool qt_is_gui_used;
 static void mac_default_handler( const char *msg )
 {
-    if ( qt_is_gui_used )
-	DebugStr(p_str(msg));
-    else
+    if ( qt_is_gui_used ) {
+	const unsigned char *p = p_str(msg);
+	DebugStr(p);
+	free((void*)p);
+    } else {
 	fprintf( stderr, msg );
+    }
 }
 
 #endif
@@ -461,12 +556,18 @@ void qFatal( const char *msg, ... )
 	va_end( ap );
         mac_default_handler(buf);
 #else
-	vfprintf( stderr, msg, ap );
+	vsprintf( buf, msg, ap );
 	va_end( ap );
-	fprintf( stderr, "\n" );		// add newline
+	fprintf( stderr, "%s\n", buf );		// add newline
 #endif
 #if defined(Q_OS_UNIX) && defined(QT_DEBUG)
 	abort();				// trap; generates core dump
+#elif defined(Q_OS_TEMP) && defined(_DEBUG)
+	QString fstr;
+	fstr.sprintf( "%s:%s %s %s", __FILE__, __LINE__, QT_VERSION_STR, buf );
+	OutputDebugString( fstr.ucs2() );
+#elif defined(Q_CC_MSVC) && defined(_DEBUG)
+	_CrtDbgReport( _CRT_ERROR, __FILE__, __LINE__, QT_VERSION_STR, buf );
 #else
 	exit( 1 );				// goodbye cruel world
 #endif
@@ -499,6 +600,12 @@ void fatal( const char *msg, ... )
 #endif
 #if defined(Q_OS_UNIX) && defined(QT_DEBUG)
 	abort();				// trap; generates core dump
+#elif defined(Q_OS_TEMP) && defined(_DEBUG)
+	QString fstr;
+	fstr.sprintf( "%s:%s %s %s", __FILE__, __LINE__, QT_VERSION_STR, buf );
+	OutputDebugString( fstr.ucs2() );
+#elif defined(Q_CC_MSVC) && defined(_DEBUG)
+	_CrtDbgReport( _CRT_ERROR, __FILE__, __LINE__, QT_VERSION_STR, buf );
 #else
 	exit( 1 );				// goodbye cruel world
 #endif
@@ -508,12 +615,12 @@ void fatal( const char *msg, ... )
 /*!
   \relates QApplication
 
-  Prints the message \a msg and uses \a code to get a system
-  specific error message. When \a code is -1 (default), the system's last
-  error code will be used if possible.
-  Use this method to handle failures in platform specific API calls.
+  Prints the message \a msg and uses \a code to get a system specific
+  error message. When \a code is -1 (the default), the system's last
+  error code will be used if possible. Use this method to handle
+  failures in platform specific API calls.
 
-  This function does nothing when Qt is built with Q_NO_DEBUG
+  This function does nothing when Qt is built with \c QT_NO_DEBUG
   defined.
 */
 void qSystemWarning( const char* msg, int code )
@@ -526,32 +633,28 @@ void qSystemWarning( const char* msg, int code )
     if ( !code )
 	return;
 
-#ifdef Q_OS_TEMP
     unsigned short *string;
+    QT_WA( {
+	FormatMessage( FORMAT_MESSAGE_ALLOCATE_BUFFER|FORMAT_MESSAGE_FROM_SYSTEM,
+		       NULL,
+		       code,
+		       MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
+		       (LPTSTR)&string,
+		       0,
+		       NULL );
 
-    FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER|FORMAT_MESSAGE_FROM_SYSTEM,
-			  NULL,
-			  code,
-			  MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
-			  (LPTSTR)&string,
-			  0,
-			  NULL );
-
-    qWarning( "%s\n\tError code %d - %s (###may need fixing in qglobal.h)", msg, code, (const char *)string );
-#else
-    char* string;
-
-    FormatMessageA(FORMAT_MESSAGE_ALLOCATE_BUFFER|FORMAT_MESSAGE_FROM_SYSTEM,
-			  NULL,
-			  code,
-			  MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
-			  (char*)&string,
-			  0,
-			  NULL );
-
-    qWarning( "%s\n\tError code %d - %s", msg, code, (const char*)string );
-#endif
+	qWarning( "%s\n\tError code %d - %s", msg, code, QString::fromUcs2(string).latin1() );
+    }, {
+	FormatMessageA( FORMAT_MESSAGE_ALLOCATE_BUFFER|FORMAT_MESSAGE_FROM_SYSTEM,
+			NULL,
+			code,
+			MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
+			(char*)&string,
+			0,
+			NULL );
 
+	qWarning( "%s\n\tError code %d - %s", msg, code, (const char*)string );
+    } );
     LocalFree( (HLOCAL)string );
 #else
     if ( code != -1 )
@@ -559,73 +662,86 @@ void qSystemWarning( const char* msg, int code )
     else
 	qWarning( msg );
 #endif
+#else
+    Q_UNUSED( msg );
+    Q_UNUSED( code );
 #endif
 }
 
 /*!
-  \fn void Q_ASSERT( bool test )
-  \relates QApplication
-  Prints a warning message containing the source code file name and line number
-  if \a test is FALSE.
+    \fn void Q_ASSERT( bool test )
 
-  This is really a macro defined in qglobal.h.
+    \relates QApplication
 
-  Q_ASSERT is useful for testing required conditions in your program.
+    Prints a warning message containing the source code file name and
+    line number if \a test is FALSE.
 
-  Example:
-  \code
-    //
-    // File: div.cpp
-    //
+    This is really a macro defined in \c qglobal.h.
 
-    #include <qglobal.h>
+    Q_ASSERT is useful for testing pre- and post-conditions.
 
-    int divide( int a, int b )
-    {
-	Q_ASSERT( b != 0 );			// this is line 9
-	return a/b;
-    }
-  \endcode
+    Example:
+    \code
+	//
+	// File: div.cpp
+	//
 
-  If \c b is zero, the Q_ASSERT statement will output the following message
-  using the qWarning() function:
-  \code
-    ASSERT: "b == 0" in div.cpp (9)
-  \endcode
+	#include <qglobal.h>
 
-  \sa qWarning(), \link debug.html Debugging\endlink
+	int divide( int a, int b )
+	{
+	    Q_ASSERT( b != 0 );			// this is line 9
+	    return a/b;
+	}
+    \endcode
+
+    If \c b is zero, the Q_ASSERT statement will output the following
+    message using the qWarning() function:
+    \code
+	ASSERT: "b == 0" in div.cpp (9)
+    \endcode
+
+    \sa qWarning(), \link debug.html Debugging\endlink
 */
 
 
 /*!
-  \fn void Q_CHECK_PTR( void *p )
-  \relates QApplication
-  If \a p is null, a fatal messages says that the program ran out of memory
-  and exits.  If \e p is not null, nothing happens.
+    \fn void Q_CHECK_PTR( void *p )
 
-  This is really a macro defined in qglobal.h.
+    \relates QApplication
 
-  Example:
-  \code
-    int *a;
-    Q_CHECK_PTR( a = new int[80] );	// never do this!
-      // do this instead:
-    a = new int[80];
-    Q_CHECK_PTR( a );			// this is fine
-  \endcode
+    If \a p is 0, prints a warning message containing the source code file
+    name and line number, saying that the program ran out of memory.
 
-  \sa qFatal(), \link debug.html Debugging\endlink
+    This is really a macro defined in \c qglobal.h.
+
+    Example:
+    \code
+	int *a;
+
+	Q_CHECK_PTR( a = new int[80] );  // WRONG!
+
+	a = new (nothrow) int[80];       // Right
+	Q_CHECK_PTR( a );
+    \endcode
+
+    \sa qWarning(), \link debug.html Debugging\endlink
 */
 
 
 //
 // The Q_CHECK_PTR macro calls this function to check if an allocation went ok.
 //
-
+#if (QT_VERSION-0 >= 0x040000)
+#if defined(Q_CC_GNU)
+#warning "Change Q_CHECK_PTR to '{if ((p)==0) qt_check_pointer(__FILE__,__LINE__);}'"
+#warning "No need for qt_check_pointer() to return a value - make it void!"
+#endif
+#endif
 bool qt_check_pointer( bool c, const char *n, int l )
 {
     if ( c )
-	qFatal( "In file %s, line %d: Out of memory", n, l );
+	qWarning( "In file %s, line %d: Out of memory", n, l );
     return TRUE;
 }
 
@@ -696,56 +812,58 @@ void qObsolete(	 const char *message )
 
 
 /*!
-  \relates QApplication
-  Installs a Qt message handler \a h.  Returns a pointer to the message handler
-  previously defined.
+    \relates QApplication
 
-  The message handler is a function that prints out debug messages,
-  warnings and fatal error messages.  The Qt library (debug version)
-  contains hundreds of warning messages that are printed when internal
-  errors (usually invalid function arguments) occur.  If you implement
-  your own message handler, you get total control of these messages.
+    Installs a Qt message handler \a h. Returns a pointer to the
+    message handler previously defined.
 
-  The default message handler prints the message to the standard output
-  under X11 or to the debugger under Windows.  If it is a fatal message,
-  the application aborts immediately.
+    The message handler is a function that prints out debug messages,
+    warnings and fatal error messages. The Qt library (debug version)
+    contains hundreds of warning messages that are printed when
+    internal errors (usually invalid function arguments) occur. If you
+    implement your own message handler, you get total control of these
+    messages.
 
-  Only one message handler can be defined, since this is usually done on
-  an application-wide basis to control debug output.
+    The default message handler prints the message to the standard
+    output under X11 or to the debugger under Windows. If it is a
+    fatal message, the application aborts immediately.
 
-  To restore the message handler, call \c qInstallMsgHandler(0).
+    Only one message handler can be defined, since this is usually
+    done on an application-wide basis to control debug output.
 
-  Example:
-  \code
-    #include <qapplication.h>
-    #include <stdio.h>
-    #include <stdlib.h>
+    To restore the message handler, call \c qInstallMsgHandler(0).
 
-    void myMessageOutput( QtMsgType type, const char *msg )
-    {
-	switch ( type ) {
-	    case QtDebugMsg:
-		fprintf( stderr, "Debug: %s\n", msg );
-		break;
-	    case QtWarningMsg:
-		fprintf( stderr, "Warning: %s\n", msg );
-		break;
-	    case QtFatalMsg:
-		fprintf( stderr, "Fatal: %s\n", msg );
-		abort();			// dump core on purpose
+    Example:
+    \code
+	#include <qapplication.h>
+	#include <stdio.h>
+	#include <stdlib.h>
+
+	void myMessageOutput( QtMsgType type, const char *msg )
+	{
+	    switch ( type ) {
+		case QtDebugMsg:
+		    fprintf( stderr, "Debug: %s\n", msg );
+		    break;
+		case QtWarningMsg:
+		    fprintf( stderr, "Warning: %s\n", msg );
+		    break;
+		case QtFatalMsg:
+		    fprintf( stderr, "Fatal: %s\n", msg );
+		    abort();			// deliberately core dump
+	    }
 	}
-    }
 
-    int main( int argc, char **argv )
-    {
-	qInstallMsgHandler( myMessageOutput );
-	QApplication a( argc, argv );
-	...
-	return a.exec();
-    }
-  \endcode
+	int main( int argc, char **argv )
+	{
+	    qInstallMsgHandler( myMessageOutput );
+	    QApplication a( argc, argv );
+	    ...
+	    return a.exec();
+	}
+    \endcode
 
-  \sa qDebug(), qWarning(), qFatal(), \link debug.html Debugging\endlink
+    \sa qDebug(), qWarning(), qFatal(), \link debug.html Debugging\endlink
 */
 
 QtMsgHandler qInstallMsgHandler( QtMsgHandler h )
@@ -756,6 +874,31 @@ QtMsgHandler qInstallMsgHandler( QtMsgHandler h )
 }
 
 
-#ifdef Q_WS_WIN
-bool qt_winunicode=FALSE;
-#endif
+/*
+    Dijkstra's bisection algorithm to find the square root as an integer.
+    Deliberately not exported as part of the Qt API, but used in both
+    qsimplerichtext.cpp and qgfxraster_qws.cpp
+*/
+unsigned int qt_int_sqrt( unsigned int n )
+{
+    // n must be in the range 0...UINT_MAX/2-1
+    if ( n >= ( UINT_MAX>>2 ) ) {
+	unsigned int r = 2 * qt_int_sqrt( n / 4 );
+	unsigned int r2 = r + 1;
+	return ( n >= r2 * r2 ) ? r2 : r;
+    }
+    uint h, p= 0, q= 1, r= n;
+    while ( q <= n )
+        q <<= 2;
+    while ( q != 1 ) {
+        q >>= 2;
+        h= p + q;
+        p >>= 1;
+        if ( r >= h ) {
+            p += q;
+            r -= h;
+        }
+    }
+    return p;
+}
+
