@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id$
+** $Id: qt/qglobal.h   3.2.3   edited Nov 10 11:21 $
 **
 ** Global type declarations and definitions
 **
 ** Created : 920529
 **
-** Copyright (C) 1992-2002 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2003 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the tools module of the Qt GUI Toolkit.
 **
@@ -38,55 +38,60 @@
 #ifndef QGLOBAL_H
 #define QGLOBAL_H
 
-#define QT_VERSION_STR   "3.0.5"
+#define QT_VERSION_STR   "3.2.3"
 /*
-   QT_VERSION is ( MAJOR<<16+MINOR<<8+_PATCH)
+   QT_VERSION is (major << 16) + (minor << 8) + patch.
  */
-#define QT_VERSION 0x030005
+#define QT_VERSION 0x030203
 
 /*
    The operating system, must be one of: (Q_OS_x)
 
-     MACX	  - Mac OS X
-     MAC9	  - Mac OS 9
-     MSDOS  - MS-DOS and Windows
-     OS2	  - OS/2
-     OS2EMX - XFree86 on OS/2 (not PM)
-     WIN32  - Win32 (Windows 95/98/ME and Windows NT/2000/XP)
-     SUN	  - SunOS
-     SOLARIS	 - Sun Solaris
-     HPUX	  - HP-UX
-     ULTRIX - DEC Ultrix
-     LINUX  - Linux
-     FREEBSD	 - FreeBSD
-     NETBSD - NetBSD
-     OPENBSD    - OpenBSD
-     BSDI	  - BSD/OS
-     IRIX	  - SGI Irix
-     OSF	  - Compaq Tru64
-     UNIXWARE	 - SCO UnixWare 2
-     SCO	  - SCO OpenServer, UnixWare 7, Open UNIX
-     AIX	  - AIX
-     HURD	  - GNU Hurd
-     DGUX	  - DG/UX
-     DYNIX  - DYNIX/ptx
-     RELIANT	 - Reliant UNIX
-     QNX6   - QNX RTP 6.1 
-     QNX4   - QNX 4.24
-     LYNX	  - LynxOS
-     BSD4	  - Any BSD 4.4 system
-     UNIX	  - Any UNIX BSD/SYSV system
+     MACX	- Mac OS X
+     MAC9	- Mac OS 9
+     DARWIN     - Darwin OS (Without Mac OS X)
+     MSDOS	- MS-DOS and Windows
+     OS2	- OS/2
+     OS2EMX	- XFree86 on OS/2 (not PM)
+     WIN32	- Win32 (Windows 95/98/ME and Windows NT/2000/XP)
+     CYGWIN	- Cygwin
+     SOLARIS	- Sun Solaris
+     HPUX	- HP-UX
+     ULTRIX	- DEC Ultrix
+     LINUX	- Linux
+     FREEBSD	- FreeBSD
+     NETBSD	- NetBSD
+     OPENBSD	- OpenBSD
+     BSDI	- BSD/OS
+     IRIX	- SGI Irix
+     OSF	- HP Tru64 UNIX
+     SCO	- SCO OpenServer 5
+     UNIXWARE	- UnixWare 7, Open UNIX 8
+     AIX	- AIX
+     HURD	- GNU Hurd
+     DGUX	- DG/UX
+     RELIANT	- Reliant UNIX
+     DYNIX	- DYNIX/ptx
+     QNX	- QNX
+     QNX6	- QNX RTP 6.1
+     LYNX	- LynxOS
+     BSD4	- Any BSD 4.4 system
+     UNIX	- Any UNIX BSD/SYSV system
 */
 
-#if defined(__APPLE__) && defined(__GNUC__)
+#if defined(__DARWIN_X11__)
+#  define Q_OS_DARWIN
+#elif defined(__APPLE__) && defined(__GNUC__)
 #  define Q_OS_MACX
 #elif defined(__MACOSX__)
 #  define Q_OS_MACX
 #elif defined(macintosh)
 #  define Q_OS_MAC9
-#elif defined(MSDOS) || defined(_MSDOS) || defined(__MSDOS__)
+#elif defined(__CYGWIN__)
+#  define Q_OS_CYGWIN
+#elif defined(MSDOS) || defined(_MSDOS)
 #  define Q_OS_MSDOS
-#elif defined(OS2) || defined(_OS2) || defined(__OS2__)
+#elif defined(__OS2__)
 #  if defined(__EMX__)
 #    define Q_OS_OS2EMX
 #  else
@@ -99,20 +104,15 @@
 #  define Q_OS_WIN32
 #elif defined(__MWERKS__) && defined(__INTEL__)
 #  define Q_OS_WIN32
-#elif defined(sun) || defined(__sun) || defined(__sun__)
-#  if defined(__SVR4)
-#    define Q_OS_SOLARIS
-#  else
-#    define Q_OS_SUN
-#    define Q_OS_BSD4
-#  endif
-#elif defined(hpux) || defined(__hpux) || defined(__hpux__)
+#elif defined(__sun) || defined(sun)
+#  define Q_OS_SOLARIS
+#elif defined(hpux) || defined(__hpux)
 #  define Q_OS_HPUX
-#elif defined(ultrix) || defined(__ultrix) || defined(__ultrix__)
+#elif defined(__ultrix) || defined(ultrix)
 #  define Q_OS_ULTRIX
 #elif defined(sinix)
 #  define Q_OS_RELIANT
-#elif defined(linux) || defined(__linux) || defined(__linux__)
+#elif defined(__linux__) || defined(__linux)
 #  define Q_OS_LINUX
 #elif defined(__FreeBSD__)
 #  define Q_OS_FREEBSD
@@ -123,10 +123,10 @@
 #elif defined(__OpenBSD__)
 #  define Q_OS_OPENBSD
 #  define Q_OS_BSD4
-#elif defined(bsdi) || defined(__bsdi__)
+#elif defined(__bsdi__)
 #  define Q_OS_BSDI
 #  define Q_OS_BSD4
-#elif defined(sgi) || defined(__sgi)
+#elif defined(__sgi)
 #  define Q_OS_IRIX
 #elif defined(__osf__)
 #  define Q_OS_OSF
@@ -134,36 +134,38 @@
 #  define Q_OS_AIX
 #elif defined(__Lynx__)
 #  define Q_OS_LYNX
-#elif defined(_UNIXWARE)
-#  define Q_OS_UNIXWARE
-#elif defined(__GNU__)
+#elif defined(__GNU_HURD__)
 #  define Q_OS_HURD
-#elif defined(DGUX)
+#elif defined(__DGUX__)
 #  define Q_OS_DGUX
 #elif defined(__QNXNTO__)
 #  define Q_OS_QNX6
 #elif defined(__QNX__)
-#  define Q_OS_QNX4
-#elif defined(_SCO_DS) || defined(M_UNIX) || defined(M_XENIX)
-#  define Q_OS_SCO
-#elif defined(sco) || defined(_UNIXWARE7)
-#  define Q_OS_UNIXWARE7
-#elif !defined(_SCO_DS) && defined(__USLC__) && defined(__SCO_VERSION__)
-#  define Q_OS_UNIXWARE7
+#  define Q_OS_QNX
 #elif defined(_SEQUENT_)
 #  define Q_OS_DYNIX
-#elif defined(__svr4__)
-/*   generic fallback for the rest of svr4 systems, e.g. g++ on UnixWare7.*/
-#  define Q_OS_SVR4
+#elif defined(_SCO_DS)                   /* SCO OpenServer 5 + GCC */
+#  define Q_OS_SCO
+#elif defined(__USLC__)                  /* all SCO platforms + UDK or OUDK */
+#  define Q_OS_UNIXWARE
+#  define Q_OS_UNIXWARE7
+#elif defined(__svr4__) && defined(i386) /* Open UNIX 8 + GCC */
+#  define Q_OS_UNIXWARE
+#  define Q_OS_UNIXWARE7
+#elif defined(__MAKEDEPEND__)
 #else
 #  error "Qt has not been ported to this OS - talk to qt-bugs@trolltech.com"
 #endif
 
+#if defined(Q_OS_WIN32) || defined(Q_OS_WIN64)
+#  define Q_OS_WIN
+#endif
+
 #if defined(Q_OS_MAC9) || defined(Q_OS_MACX)
 #  define Q_OS_MAC
 #endif
 
-#if defined(Q_OS_MAC9) || defined(Q_OS_MSDOS) || defined(Q_OS_OS2) || defined(Q_OS_WIN32) || defined(Q_OS_WIN64)
+#if defined(Q_OS_MAC9) || defined(Q_OS_MSDOS) || defined(Q_OS_OS2) || defined(Q_OS_WIN)
 #  undef Q_OS_UNIX
 #elif !defined(Q_OS_UNIX)
 #  define Q_OS_UNIX
@@ -173,83 +175,101 @@
 /*
    The compiler, must be one of: (Q_CC_x)
 
-     SYM - Symantec C++ for both PC and Macintosh
-     MPW - MPW C++
-     MWERKS - Metrowerks CodeWarrior
-     MSVC - Microsoft Visual C/C++
-     BOR - Borland/Turbo C++
-     WAT - Watcom C++
-     GNU - GNU C++
-     COMEAU - Comeau C++
-     EDG - Edison Design Group C++
-     OC - CenterLine C++
-     SUN - Sun C++
-     MIPS - MIPSpro C++
-     DEC - DEC C++
-     HP - HPUX C++
-     HPACC - HPUX ANSI C++
-     USLC - SCO UnixWare C++
-     CDS - Reliant C++
-     KAI - KAI C++
-     INTEL - Intel C++
-     HIGHC - MetaWare High C/C++
-     PGI - Portland Group C++
+     SYM	- Symantec C++ for both PC and Macintosh
+     MPW	- MPW C++
+     MWERKS	- Metrowerks CodeWarrior
+     MSVC	- Microsoft Visual C/C++
+     BOR	- Borland/Turbo C++
+     WAT	- Watcom C++
+     GNU	- GNU C++
+     COMEAU	- Comeau C++
+     EDG	- Edison Design Group C++
+     OC		- CenterLine C++
+     SUN	- Sun WorkShop, Forte Developer, or Sun ONE Studio C++
+     MIPS	- MIPSpro C++
+     DEC	- DEC C++
+     HP		- HPUX C++
+     HPACC	- HPUX ANSI C++
+     USLC	- SCO OUDK, UDK, and UnixWare 2.X C++
+     CDS	- Reliant C++
+     KAI	- KAI C++
+     INTEL	- Intel C++
+     HIGHC	- MetaWare High C/C++
+     PGI	- Portland Group C++
+     GHS	- Green Hills Optimizing C++ Compilers
 
    Should be sorted most to least authoritative.
 */
 
-/* Symantec C++ is now Digital Mars? */
-#if defined(__SC__)
+/* Symantec C++ is now Digital Mars */
+#if defined(__DMC__) || defined(__SC__)
 #  define Q_CC_SYM
+/* "explicit" semantics implemented in 8.1e but keyword recognized since 7.5 */
+#  if defined(__SC__) && __SC__ < 0x750
+#    define Q_NO_EXPLICIT_KEYWORD
+#  endif
+#  define Q_NO_USING_KEYWORD
+#  if !defined(_CPPUNWIND)
+#    define Q_NO_EXCEPTIONS
+#  endif
 
 #elif defined(applec)
 #  define Q_CC_MPW
 #  define Q_NO_BOOL_TYPE
+#  define Q_NO_EXPLICIT_KEYWORD
+#  define Q_NO_USING_KEYWORD
 
 #elif defined(__MWERKS__)
 #  define Q_CC_MWERKS
+/* "explicit" recognized since 4.0d1 */
 #  define QMAC_PASCAL pascal
+#  define Q_NO_USING_KEYWORD /* ### check "using" status */
 
 #elif defined(_MSC_VER)
 #  define Q_CC_MSVC
 /* proper support of bool for _MSC_VER >= 1100 */
 #  define Q_CANNOT_DELETE_CONSTANT
 #  define Q_INLINE_TEMPLATES inline
+/* Visual C++.Net issues for _MSC_VER >= 1300 */
+#  if _MSC_VER >= 1300
+#    define Q_CC_MSVC_NET
+#    if _MSC_VER < 1310
+#      define Q_TYPENAME
+#    endif
+#  endif
+#  define Q_NO_USING_KEYWORD /* ### check "using" status */
+
 #elif defined(__BORLANDC__) || defined(__TURBOC__)
 #  define Q_CC_BOR
-#  if __BORLANDC__ < 0x500
+#  if __BORLANDC__ < 0x502
 #    define Q_NO_BOOL_TYPE
+#    define Q_NO_EXPLICIT_KEYWORD
 #  endif
+#  define Q_NO_USING_KEYWORD /* ### check "using" status */
 
 #elif defined(__WATCOMC__)
 #  define Q_CC_WAT
-#ifdef Q_OS_QNX4
-//compiler flags
-#define Q_TYPENAME
-#define Q_NO_BOOL_TYPE
-#define Q_CANNOT_DELETE_CONSTANT
-#define mutable
-
-// XXX
-#define Q_BROKEN_TEMPLATE_SPECIALIZATION
-// No Template classes in QVariant
-#define QT_NO_TEMPLATE_VARIANT
-// Wcc does not fill in functions needed by valuelists
-// maps, and valuestacks implicitly
-#define Q_FULL_TEMPLATE_INSTANTIATION
-// can we just compare the structures?
-#define Q_FULL_TEMPLATE_INSTANTIATION_MEMCMP
-
-//these are not useful to our customers
-#define QT_QWS_NO_SHM
-#define QT_NO_QWS_MULTIPROCESS
-#define QT_NO_SQL
-#define QT_NO_QWS_CURSOR
-#endif
-
-/* Never tested! */
-#elif defined(__HIGHC__)
-#  define Q_CC_HIGHC
+#  if defined(Q_OS_QNX4)
+/* compiler flags */
+#    define Q_TYPENAME
+#    define Q_NO_BOOL_TYPE
+#    define Q_CANNOT_DELETE_CONSTANT
+#    define mutable
+/* ??? */
+#    define Q_BROKEN_TEMPLATE_SPECIALIZATION
+/* no template classes in QVariant */
+#    define QT_NO_TEMPLATE_VARIANT
+/* Wcc does not fill in functions needed by valuelists, maps, and
+   valuestacks implicitly */
+#    define Q_FULL_TEMPLATE_INSTANTIATION
+/* can we just compare the structures? */
+#    define Q_FULL_TEMPLATE_INSTANTIATION_MEMCMP
+/* these are not useful to our customers */
+#    define QT_QWS_NO_SHM
+#    define QT_NO_QWS_MULTIPROCESS
+#    define QT_NO_SQL
+#    define QT_NO_QWS_CURSOR
+#  endif
 
 #elif defined(__GNUC__)
 #  define Q_CC_GNU
@@ -257,99 +277,188 @@
 #  if __GNUC__ == 2 && __GNUC_MINOR__ <= 7
 #    define Q_FULL_TEMPLATE_INSTANTIATION
 #  endif
+/* GCC 2.95 knows "using" but does not support it correctly */
+#  if __GNUC__ == 2 && __GNUC_MINOR__ <= 95
+#    define Q_NO_USING_KEYWORD
+#  endif
+/* GCC 3.1 and GCC 3.2 wrongly define _SB_CTYPE_MACROS on HP-UX */
+#  if defined(Q_OS_HPUX) && __GNUC__ == 3 && __GNUC_MINOR__ >= 1
+#    define Q_WRONG_SB_CTYPE_MACROS
+#  endif
 #  if (defined(__arm__) || defined(__ARMEL__)) && !defined(QT_MOC_CPP)
 #    define Q_PACKED __attribute__ ((packed))
 #  endif
+#  if !defined(__EXCEPTIONS)
+#    define Q_NO_EXCEPTIONS
+#  endif
 
 /* IBM compiler versions are a bit messy. There are actually two products:
    the C product, and the C++ product. The C++ compiler is always packaged
    with the latest version of the C compiler. Version numbers do not always
    match. This little table (I'm not sure it's accurate) should be helpful:
 
-	   C++ product               C product
+   C++ product                C product
 
-	    C Set 3.1              C Compiler 3.0
-		 ...                       ...
-	C++ Compiler 3.6.6         C Compiler 4.3
-		 ...                       ...
-	Visual Age C++ 4.0              ...
-		 ...                       ...
-	Visual Age C++ 5.0         C Compiler 5.0
+            C Set 3.1         C Compiler 3.0
+                  ...         ...
+   C++ Compiler 3.6.6         C Compiler 4.3
+                  ...         ...
+   Visual Age C++ 4.0         ...
+                  ...         ...
+   Visual Age C++ 5.0         C Compiler 5.0
+                  ...         ...
+   Visual Age C++ 6.0         C Compiler 6.0
 
    Now:
    __xlC__    is the version of the C compiler in hexadecimal notation
-              - it's only an approximation of the C++ compiler version
+              is only an approximation of the C++ compiler version
    __IBMCPP__ is the version of the C++ compiler in decimal notation
-              - but it's not defined on older compilers like C Set 3.1 */
+              but it is not defined on older compilers like C Set 3.1 */
 #elif defined(__xlC__)
 #  define Q_CC_XLC
 #  define Q_FULL_TEMPLATE_INSTANTIATION
 #  if __xlC__ < 0x400
 #    define Q_NO_BOOL_TYPE
+#    define Q_NO_EXPLICIT_KEYWORD
+#    define Q_NO_USING_KEYWORD
+#    define Q_TYPENAME
+#    define Q_INLINE_TEMPLATES inline
 #    define Q_BROKEN_TEMPLATE_SPECIALIZATION
 #    define Q_CANNOT_DELETE_CONSTANT
 #  endif
 
+/* Older versions of DEC C++ do not define __EDG__ or __EDG - observed
+   on DEC C++ V5.5-004. New versions do define  __EDG__ - observed on
+   Compaq C++ V6.3-002.
+   This compiler is different enough from other EDG compilers to handle
+   it separately anyway. */
+#elif defined(__DECCXX)
+#  define Q_CC_DEC
+/* Compaq C++ V6 compilers are EDG-based but I'm not sure about older
+   DEC C++ V5 compilers. */
+#  if defined(__EDG__)
+#    define Q_CC_EDG
+#  endif
+/* Compaq have disabled EDG's _BOOL macro and use _BOOL_EXISTS instead
+   - observed on Compaq C++ V6.3-002.
+   In any case versions prior to Compaq C++ V6.0-005 do not have bool. */
+#  if !defined(_BOOL_EXISTS)
+#    define Q_NO_BOOL_TYPE
+#  endif
+/* Spurious (?) error messages observed on Compaq C++ V6.5-014. */
+#  define Q_NO_USING_KEYWORD
+/* Apply to all versions prior to Compaq C++ V6.0-000 - observed on
+   DEC C++ V5.5-004. */
+#  if __DECCXX_VER < 60060000
+#    define Q_TYPENAME
+#    define Q_BROKEN_TEMPLATE_SPECIALIZATION
+#    define Q_CANNOT_DELETE_CONSTANT
+#  endif
+/* avoid undefined symbol problems with out-of-line template members */
+#  define Q_INLINE_TEMPLATES inline
+
 /* Compilers with EDG front end are similar. To detect them we test:
    __EDG documented by SGI, observed on MIPSpro 7.3.1.1 and KAI C++ 4.0b
    __EDG__ documented in EDG online docs, observed on Compaq C++ V6.3-002 */
-#elif defined(__EDG) || defined(__EDG__) || defined(Q_CC_EDG)
-#  if !defined(Q_CC_EDG)
-#    define Q_CC_EDG
-#  endif
-/* Compaq have disabled EDG's _BOOL macro - observed on Compaq C++ V6.3-002. */
-#  if defined(__DECCXX)
-#    define Q_CC_DEC
-/* Compaq use _BOOL_EXISTS instead of _BOOL. */
-#    if defined(_BOOL_EXISTS)
-/* Well, at least macro _BOOL_EXISTS is documented for Compaq C++ V6.3.
-   In any case versions prior to Compaq C++ V6.0-005 do not have bool. */
-#    elif __DECCXX_VER < 60060005
-#      define Q_NO_BOOL_TYPE
-#    endif
-/* Apart from Compaq, from the EDG documentation:
+#elif defined(__EDG) || defined(__EDG__)
+#  define Q_CC_EDG
+/* From the EDG documentation (does not seem to apply to Compaq C++):
    _BOOL
-    Defined in C++ mode when bool is a keyword. The name of this predefined
-    macro is specified by a configuration flag. _BOOL is the default.
+    	Defined in C++ mode when bool is a keyword. The name of this
+	predefined macro is specified by a configuration flag. _BOOL
+	is the default.
    __BOOL_DEFINED
-    Defined in Microsoft C++ mode when bool is a keyword. */
-#  else
-#    if !defined(_BOOL) && !defined(__BOOL_DEFINED)
-#      define Q_NO_BOOL_TYPE
-#    endif
-#    if defined(__COMO__)
-#      define Q_CC_COMEAU
-#      define Q_C_CALLBACKS
-#    elif defined(__KCC)
-#      define Q_CC_KAI
-#    elif defined(__INTEL_COMPILER)
-#      define Q_CC_INTEL
-/* The Portland Group compiler is based on EDG and does define __EDG__ */
-#    elif defined(__PGI)
-#      define Q_CC_PGI
-/* The new UnixWare 7 compiler is based on EDG and does define __EDG__ */
-#    elif defined(__USLC__)
-#      define Q_CC_EDG
-#      define Q_CC_USLC
-/* Never tested! */
-#    elif defined(CENTERLINE_CLPP) || defined(OBJECTCENTER)
-#      define Q_CC_OC
-/* The MIPSpro compiler in o32 mode is based on EDG but disables features
-   such as template specialization nevertheless */
-#    elif defined(sgi) || defined(__sgi)
-#      define Q_CC_MIPS
-#      if defined(_MIPS_SIM) && (_MIPS_SIM == _ABIO32) /* o32 ABI */
-#        define Q_BROKEN_TEMPLATE_SPECIALIZATION
-#      elif defined(_COMPILER_VERSION) && (_COMPILER_VERSION < 730) /* 7.2 */
-#        define Q_BROKEN_TEMPLATE_SPECIALIZATION
-#      endif
-#    endif
+   	Defined in Microsoft C++ mode when bool is a keyword. */
+#  if !defined(_BOOL) && !defined(__BOOL_DEFINED)
+#    define Q_NO_BOOL_TYPE
 #  endif
 
-/* the older UnixWare compiler is not based on EDG */
+/* The Portland Group compiler is based on EDG and does define __EDG__ */
+#  if defined(__COMO__)
+#    define Q_CC_COMEAU
+#    define Q_C_CALLBACKS
+
+/* The `using' keyword was introduced to avoid KAI C++ warnings
+   but it's now causing KAI C++ errors instead. The standard is
+   unclear about the use of this keyword, and in practice every
+   compiler is using its own set of rules. Forget it. */
+#  elif defined(__KCC)
+#    define Q_CC_KAI
+#    if !defined(_EXCEPTIONS)
+#      define Q_NO_EXCEPTIONS
+#    endif
+#    define Q_NO_USING_KEYWORD
+
+/* Using the `using' keyword avoids Intel C++ warnings */
+#  elif defined(__INTEL_COMPILER)
+#    define Q_CC_INTEL
+#    if !defined(__EXCEPTIONS)
+#      define Q_NO_EXCEPTIONS
+#    endif
+
+/* The Portland Group compiler is based on EDG and does define __EDG__ */
+#  elif defined(__PGI)
+#    define Q_CC_PGI
+#    if !defined(__EXCEPTIONS)
+#      define Q_NO_EXCEPTIONS
+#    endif
+
+/* Never tested! */
+#  elif defined(__ghs)
+#    define Q_CC_GHS
+
+/* The UnixWare 7 UDK compiler is based on EDG and does define __EDG__ */
+#  elif defined(__USLC__) && defined(__SCO_VERSION__)
+#    define Q_CC_USLC
+/* The latest UDK 7.1.1b does not need this, but previous versions do */
+#    if !defined(__SCO_VERSION__) || (__SCO_VERSION__ < 302200010)
+#      define Q_INLINE_TEMPLATES inline
+#    endif
+#    define Q_NO_USING_KEYWORD /* ### check "using" status */
+
+/* Never tested! */
+#  elif defined(CENTERLINE_CLPP) || defined(OBJECTCENTER)
+#    define Q_CC_OC
+#    define Q_NO_USING_KEYWORD
+
+/* CDS++ defines __EDG__ although this is not documented in the Reliant
+   documentation. It also follows conventions like _BOOL and this documented */
+#  elif defined(sinix)
+#    define Q_CC_CDS
+#    define Q_NO_USING_KEYWORD
+#    if defined(__cplusplus) && (__cplusplus < 2) /* Cfront C++ mode */
+#      define Q_NO_EXCEPTIONS
+#    endif
+
+/* The MIPSpro compiler in o32 mode is based on EDG but disables features
+   such as template specialization nevertheless */
+#  elif defined(__sgi)
+#    define Q_CC_MIPS
+#    define Q_STRICT_INLINING_RULES
+#    if defined(_MIPS_SIM) && (_MIPS_SIM == _ABIO32) /* o32 ABI */
+#      define Q_TYPENAME
+#      define Q_BROKEN_TEMPLATE_SPECIALIZATION
+#      define Q_NO_EXPLICIT_KEYWORD
+#      define Q_INLINE_TEMPLATES inline
+#    elif defined(_COMPILER_VERSION) && (_COMPILER_VERSION < 730) /* 7.2 */
+#      define Q_TYPENAME
+#      define Q_BROKEN_TEMPLATE_SPECIALIZATION
+#    endif
+#    define Q_NO_USING_KEYWORD /* ### check "using" status */
+#  endif
+
+/* The older UnixWare 2.X compiler? */
 #elif defined(__USLC__)
 #  define Q_CC_USLC
+#  define Q_TYPENAME
 #  define Q_NO_BOOL_TYPE
+#  define Q_NO_EXPLICIT_KEYWORD
+#  define Q_NO_USING_KEYWORD
+#  define Q_INLINE_TEMPLATES inline
+
+/* Never tested! */
+#elif defined(__HIGHC__)
+#  define Q_CC_HIGHC
 
 #elif defined(__SUNPRO_CC)
 #  define Q_CC_SUN
@@ -361,10 +470,15 @@
 #    if !defined(_BOOL)
 #      define Q_NO_BOOL_TYPE
 #    endif
+#    if defined(__SUNPRO_CC_COMPAT) && (__SUNPRO_CC_COMPAT <= 4)
+#      define Q_NO_USING_KEYWORD
+#    endif
 #    define Q_C_CALLBACKS
 /* 4.2 compiler or older */
 #  else
 #    define Q_NO_BOOL_TYPE
+#    define Q_NO_EXPLICIT_KEYWORD
+#    define Q_NO_USING_KEYWORD
 #  endif
 
 /* CDS++ does not seem to define __EDG__ or __EDG according to Reliant
@@ -375,6 +489,7 @@
 #  if !defined(_BOOL)
 #    define Q_NO_BOOL_TYPE
 #  endif
+#  define Q_BROKEN_TEMPLATE_SPECIALIZATION
 
 #elif defined(Q_OS_HPUX)
 /* __HP_aCC was not defined in first aCC releases */
@@ -385,7 +500,9 @@
 #    define Q_NO_BOOL_TYPE
 #    define Q_FULL_TEMPLATE_INSTANTIATION
 #    define Q_BROKEN_TEMPLATE_SPECIALIZATION
+#    define Q_NO_EXPLICIT_KEYWORD
 #  endif
+#  define Q_NO_USING_KEYWORD /* ### check "using" status */
 
 #else
 #  error "Qt has not been tested with this compiler - talk to qt-bugs@trolltech.com"
@@ -399,18 +516,16 @@
 /*
    The window system, must be one of: (Q_WS_x)
 
-     MACX	  - Mac OS X
-     MAC9	  - Mac OS 9
-     QWS	  - Qt/Embedded
-     WIN32  - Windows
-     X11	  - X Window System
-     PM	  - unsupported
-     WIN16  - unsupported
+     MACX	- Mac OS X
+     MAC9	- Mac OS 9
+     QWS	- Qt/Embedded
+     WIN32	- Windows
+     X11	- X Window System
+     PM		- unsupported
+     WIN16	- unsupported
 */
 
-#if defined(Q_OS_MACX)
-#  define Q_WS_MACX
-#elif defined(Q_OS_MAC9)
+#if defined(Q_OS_MAC9)
 #  define Q_WS_MAC9
 #elif defined(Q_OS_MSDOS)
 #  define Q_WS_WIN16
@@ -426,8 +541,11 @@
 #  define Q_WS_PM
 #  error "Qt does not work with OS/2 Presentation Manager or Workplace Shell"
 #elif defined(Q_OS_UNIX)
-#  ifdef QWS
+#  if defined(QWS)
 #    define Q_WS_QWS
+#    define QT_NO_QWS_IM
+#  elif defined(Q_OS_MACX)
+#    define Q_WS_MACX
 #  else
 #    define Q_WS_X11
 #  endif
@@ -440,7 +558,7 @@
 #  define Q_WS_WIN
 #endif
 
-#if defined(Q_WS_MAC9) || defined(Q_WS_MACX)
+#if (defined(Q_WS_MAC9) || defined(Q_WS_MACX)) && !defined(Q_WS_QWS) && !defined(Q_WS_X11)
 #  define Q_WS_MAC
 #endif
 
@@ -456,6 +574,7 @@
 
 #if defined(__cplusplus)
 
+
 //
 // Useful type definitions for Qt
 //
@@ -473,8 +592,8 @@ typedef unsigned char   uchar;
 typedef unsigned short  ushort;
 typedef unsigned	uint;
 typedef unsigned long   ulong;
-typedef char		   *pchar;
-typedef uchar		   *puchar;
+typedef char	       *pchar;
+typedef uchar	       *puchar;
 typedef const char     *pcchar;
 
 
@@ -487,12 +606,31 @@ const bool FALSE = 0;
 const bool TRUE = !0;
 #endif
 #if defined(__WATCOMC__)
-#ifdef Q_OS_QNX4
+#  if defined(Q_OS_QNX4)
 const bool false = FALSE;
 const bool true = TRUE;
+#  endif
 #endif
+
+//
+// Proper for-scoping
+// ### turn on in 4.0
+
+#if 0 && defined(Q_CC_MSVC) && !defined(Q_CC_MSVC_NET)
+#  define for if(0){}else for
 #endif
 
+//
+// Use the "explicit" keyword on platforms that support it.
+//
+
+#if !defined(Q_NO_EXPLICIT_KEYWORD)
+#  define Q_EXPLICIT explicit
+#else
+#  define Q_EXPLICIT
+#endif
+
+
 //
 // Workaround for static const members on MSVC++.
 //
@@ -516,7 +654,7 @@ const bool true = TRUE;
 
 inline int qRound( double d )
 {
-    return int( d >= 0.0 ? d + 0.5 : d - 0.5 );
+    return d >= 0.0 ? int(d + 0.5) : int( d - ((int)d-1) + 0.5 ) + ((int)d-1);
 }
 
 
@@ -526,34 +664,48 @@ inline int qRound( double d )
 
 #if !defined(QT_CLEAN_NAMESPACE)
 // source compatibility with Qt 1.x
-typedef signed char	    INT8;		 // 8 bit signed
-typedef unsigned char	   UINT8;		// 8 bit unsigned
-typedef short		    INT16;	 // 16 bit signed
-typedef unsigned short	   UINT16;	// 16 bit unsigned
-typedef int		INT32;	  // 32 bit signed
-typedef unsigned int	   UINT32;	// 32 bit unsigned
+typedef signed char		INT8;		// 8 bit signed
+typedef unsigned char		UINT8;		// 8 bit unsigned
+typedef short			INT16;		// 16 bit signed
+typedef unsigned short		UINT16;		// 16 bit unsigned
+typedef int			INT32;		// 32 bit signed
+typedef unsigned int		UINT32;		// 32 bit unsigned
 #endif
 
-typedef signed char	    Q_INT8;	 // 8 bit signed
-typedef unsigned char	   Q_UINT8;	// 8 bit unsigned
-typedef short		    Q_INT16;	 // 16 bit signed
-typedef unsigned short	   Q_UINT16;	// 16 bit unsigned
-typedef int		Q_INT32;	  // 32 bit signed
-typedef unsigned int	   Q_UINT32;	// 32 bit unsigned
+typedef signed char		Q_INT8;		// 8 bit signed
+typedef unsigned char		Q_UINT8;	// 8 bit unsigned
+typedef short			Q_INT16;	// 16 bit signed
+typedef unsigned short		Q_UINT16;	// 16 bit unsigned
+typedef int			Q_INT32;	// 32 bit signed
+typedef unsigned int		Q_UINT32;	// 32 bit unsigned
 #if defined(Q_OS_WIN64)
-// LLP64 64-bit model on Windows
-typedef __int64	    Q_LONG;	 // word up to 64 bit signed
-typedef unsigned __int64	   Q_ULONG;	// word up to 64 bit unsigned
+typedef __int64			Q_LONG;		// word up to 64 bit signed
+typedef unsigned __int64	Q_ULONG;	// word up to 64 bit unsigned
 #else
-// LP64 64-bit model on Linux
-typedef long		    Q_LONG;
-typedef unsigned long	   Q_ULONG;
+typedef long			Q_LONG;		// word up to 64 bit signed
+typedef unsigned long		Q_ULONG;	// word up to 64 bit unsigned
 #endif
+#if defined(Q_OS_WIN) && !defined(Q_CC_GNU)
+#  define Q_INT64_C(c) 		c ## i64	// signed 64 bit constant
+#  define Q_UINT64_C(c)		c ## ui64	// unsigned 64 bit constant
+typedef __int64			Q_INT64;	// 64 bit signed
+typedef unsigned __int64	Q_UINT64;	// 64 bit unsigned
+#else
+#  define Q_INT64_C(c) 		c ## LL		// signed 64 bit constant
+#  define Q_UINT64_C(c)		c ## ULL	// unsigned 64 bit constant
+typedef long long		Q_INT64;	// 64 bit signed
+typedef unsigned long long	Q_UINT64;	// 64 bit unsigned
+#endif
+typedef Q_INT64			Q_LLONG;	// signed long long
+typedef Q_UINT64		Q_ULLONG;	// unsigned long long
 
-#if !defined(QT_CLEAN_NAMESPACE)
-// mininum size of 64 bits is not guaranteed
-#define Q_INT64	    Q_LONG
-#define Q_UINT64	    Q_ULONG
+#if defined(Q_OS_MACX) && !defined(QT_LARGEFILE_SUPPORT)
+#  define QT_LARGEFILE_SUPPORT 64
+#endif
+#if defined(QT_LARGEFILE_SUPPORT)
+    typedef Q_ULLONG QtOffset;
+#else
+    typedef Q_ULONG QtOffset;
 #endif
 
 
@@ -565,34 +717,27 @@ class QDataStream;
 
 
 //
-// Some platform specific stuff
-//
-
-#ifdef Q_WS_WIN
-extern bool qt_winunicode;
-#endif
-
-//
-// feature seubsetting
+// Feature subsetting
 //
 // Note that disabling some features will produce a libqt that is not
 // compatible with other libqt builds. Such modifications are only
 // supported on Qt/Embedded where reducing the library size is important
 // and where the application-suite is often a fixed set.
 //
+
 #if !defined(QT_MOC)
 #if defined(QCONFIG_LOCAL)
-#include <qconfig-local.h>
+#include "qconfig-local.h"
 #elif defined(QCONFIG_MINIMAL)
-#include <qconfig-minimal.h>
+#include "qconfig-minimal.h"
 #elif defined(QCONFIG_SMALL)
-#include <qconfig-small.h>
+#include "qconfig-small.h"
 #elif defined(QCONFIG_MEDIUM)
-#include <qconfig-medium.h>
+#include "qconfig-medium.h"
 #elif defined(QCONFIG_LARGE)
-#include <qconfig-large.h>
+#include "qconfig-large.h"
 #else // everything...
-#include <qconfig.h>
+#include "qconfig.h"
 #endif
 #endif
 
@@ -603,6 +748,9 @@ extern bool qt_winunicode;
 
 // prune to local config
 #include "qmodules.h"
+#ifndef QT_MODULE_DIALOGS
+# define QT_NO_DIALOG
+#endif
 #ifndef QT_MODULE_ICONVIEW
 # define QT_NO_ICONVIEW
 #endif
@@ -628,7 +776,7 @@ extern bool qt_winunicode;
 # define QT_NO_SQL
 #endif
 
-#ifdef Q_WS_MAC9
+#if defined(Q_WS_MAC9)
 //No need for menu merging
 #  ifndef QMAC_QMENUBAR_NO_MERGE
 #    define QMAC_QMENUBAR_NO_MERGE
@@ -641,7 +789,7 @@ extern bool qt_winunicode;
 #    define QMAC_QMENUBAR_NO_EVENT
 #  endif
 #endif
-#ifdef Q_WS_MACX //for no nobody uses quartz, just putting in first level hooks
+#if defined(Q_WS_MACX) //for no nobody uses quartz, just putting in first level hooks
 #  ifndef QMAC_NO_QUARTZ
 #    define QMAC_NO_QUARTZ
 #  endif
@@ -650,39 +798,57 @@ extern bool qt_winunicode;
 #  endif
 #endif
 
+#if !defined(Q_WS_QWS) && !defined(QT_NO_COP)
+#  define QT_NO_COP
+#endif
+
 #ifndef QT_H
-#include <qfeatures.h>
+#include "qfeatures.h"
 #endif // QT_H
 
 
 //
 // Create Qt DLL if QT_DLL is defined (Windows only)
+// or QT_SHARED is defined (Kylix only)
 //
 
-#if defined(Q_OS_WIN32) || defined(Q_OS_WIN64)
+#if defined(Q_OS_WIN)
 #  if defined(QT_NODLL)
 #    undef QT_MAKEDLL
 #    undef QT_DLL
-#  elif defined(QT_MAKEDLL) /* create a Qt DLL library */
+#  elif defined(QT_MAKEDLL)	/* create a Qt DLL library */
 #    if defined(QT_DLL)
 #      undef QT_DLL
 #    endif
 #    define Q_EXPORT  __declspec(dllexport)
 #    define Q_TEMPLATEDLL
 #    define Q_TEMPLATE_EXTERN
-#    undef  Q_DISABLE_COPY  /* avoid unresolved externals */
-#  elif defined(QT_DLL)	   /* use a Qt DLL library */
+#    undef  Q_DISABLE_COPY	/* avoid unresolved externals */
+#  elif defined(QT_DLL)		/* use a Qt DLL library */
 #    define Q_EXPORT  __declspec(dllimport)
 #    define Q_TEMPLATEDLL
-#    if defined(Q_CC_MSVC)
-#      define Q_TEMPLATE_EXTERN /*extern*/ //### too many warnings, even though disabled
-#    else
-#      define Q_TEMPLATE_EXTERN
+#    ifndef Q_TEMPLATE_EXTERN
+#      if defined(Q_CC_MSVC_NET)
+#        define Q_TEMPLATE_EXTERN extern
+#      else
+#        define Q_TEMPLATE_EXTERN
+#      endif
 #    endif
-#    undef  Q_DISABLE_COPY  /* avoid unresolved externals */
+#    undef  Q_DISABLE_COPY	/* avoid unresolved externals */
+#  endif
+#elif defined(Q_OS_LINUX) && defined(Q_CC_BOR)
+#  if defined(QT_SHARED)	/* create a Qt shared library */
+#    define Q_EXPORT  __declspec(dllexport)
+#    define Q_TEMPLATEDLL
+#    define Q_TEMPLATE_EXTERN
+#    undef  Q_DISABLE_COPY	/* avoid unresolved externals */
+#  else
+#    define Q_TEMPLATEDLL
+#    define Q_TEMPLATE_EXTERN
+#    undef  Q_DISABLE_COPY 	/* avoid unresolved externals */
 #  endif
 #else
-#  undef QT_MAKEDLL	    /* ignore these for other platforms */
+#  undef QT_MAKEDLL		/* ignore these for other platforms */
 #  undef QT_DLL
 #endif
 
@@ -691,16 +857,209 @@ extern bool qt_winunicode;
 #endif
 
 
+//
+// Some platform specific stuff
+//
+
+#if defined(Q_WS_WIN)
+extern Q_EXPORT bool qt_winunicode;
+#endif
+
+
 //
 // System information
 //
 
 Q_EXPORT const char *qVersion();
 Q_EXPORT bool qSysInfo( int *wordSize, bool *bigEndian );
-#if defined(Q_WS_WIN)
+Q_EXPORT bool qSharedBuild();
+#if defined(Q_OS_MAC)
+int qMacVersion();
+#elif defined(Q_WS_WIN)
 Q_EXPORT int qWinVersion();
+#if defined(UNICODE)
+#define QT_WA( uni, ansi ) if ( qt_winunicode ) { uni } else { ansi }
+#define QT_WA_INLINE( uni, ansi ) ( qt_winunicode ? uni : ansi )
+#else
+#define QT_WA( uni, ansi ) ansi
+#define QT_WA_INLINE( uni, ansi ) ansi
+#endif
 #endif
 
+#ifdef Q_OS_TEMP
+#ifdef QT_WA
+#undef QT_WA
+#undef QT_WA_INLINE
+#endif
+#define QT_WA( uni, ansi ) uni
+#define QT_WA_INLINE( uni, ansi ) ( uni )
+#endif
+
+#ifndef Q_INLINE_TEMPLATES
+#  define Q_INLINE_TEMPLATES
+#endif
+
+#ifndef Q_TYPENAME
+#  define Q_TYPENAME typename
+#endif
+
+//
+// Use to avoid "unused parameter" warnings
+//
+
+#define Q_UNUSED(x) (void)x;
+
+//
+// Debugging and error handling
+//
+
+#if !defined(QT_NO_CHECK)
+#  define QT_CHECK_STATE			// check state of objects etc.
+#  define QT_CHECK_RANGE			// check range of indexes etc.
+#  define QT_CHECK_NULL				// check null pointers
+#  define QT_CHECK_MATH				// check math functions
+#endif
+
+#if !defined(QT_NO_DEBUG) && !defined(QT_DEBUG)
+#  define QT_DEBUG				// display debug messages
+#  if !defined(QT_NO_COMPAT)			// compatibility with Qt 2
+#    if !defined(NO_DEBUG) && !defined(DEBUG)
+#      if !defined(Q_OS_MACX)			// clash with MacOS X headers
+#        define DEBUG
+#      endif
+#    endif
+#  endif
+#endif
+
+
+Q_EXPORT void qDebug( const char *, ... )	// print debug message
+#if defined(Q_CC_GNU) && !defined(__INSURE__)
+    __attribute__ ((format (printf, 1, 2)))
+#endif
+;
+
+Q_EXPORT void qWarning( const char *, ... )	// print warning message
+#if defined(Q_CC_GNU) && !defined(__INSURE__)
+    __attribute__ ((format (printf, 1, 2)))
+#endif
+;
+
+Q_EXPORT void qFatal( const char *, ... )	// print fatal message and exit
+#if defined(Q_CC_GNU)
+    __attribute__ ((format (printf, 1, 2)))
+#endif
+;
+
+Q_EXPORT void qSystemWarning( const char *, int code = -1 );
+
+#if !defined(QT_CLEAN_NAMESPACE) 		// compatibility with Qt 1
+
+Q_EXPORT void debug( const char *, ... )	// print debug message
+#if defined(Q_CC_GNU) && !defined(__INSURE__)
+    __attribute__ ((format (printf, 1, 2)))
+#endif
+;
+
+Q_EXPORT void warning( const char *, ... )	// print warning message
+#if defined(Q_CC_GNU) && !defined(__INSURE__)
+    __attribute__ ((format (printf, 1, 2)))
+#endif
+;
+
+Q_EXPORT void fatal( const char *, ... )	// print fatal message and exit
+#if defined(Q_CC_GNU) && !defined(__INSURE__)
+    __attribute__ ((format (printf, 1, 2)))
+#endif
+;
+
+#endif // QT_CLEAN_NAMESPACE
+
+
+#if !defined(Q_ASSERT)
+#  if defined(QT_CHECK_STATE)
+#    if defined(QT_FATAL_ASSERT)
+#      define Q_ASSERT(x)  ((x) ? (void)0 : qFatal("ASSERT: \"%s\" in %s (%d)",#x,__FILE__,__LINE__))
+#    else
+#      define Q_ASSERT(x)  ((x) ? (void)0 : qWarning("ASSERT: \"%s\" in %s (%d)",#x,__FILE__,__LINE__))
+#    endif
+#  else
+#    define Q_ASSERT(x)
+#  endif
+#endif
+
+#if !defined(QT_NO_COMPAT)			// compatibility with Qt 2
+#  if !defined(ASSERT)
+#    if !defined(Q_OS_TEMP)
+#      define ASSERT(x) Q_ASSERT(x)
+#    endif
+#  endif
+#endif // QT_NO_COMPAT
+
+
+Q_EXPORT bool qt_check_pointer( bool c, const char *, int );
+
+#if defined(QT_CHECK_NULL)
+#  define Q_CHECK_PTR(p) (qt_check_pointer((p)==0,__FILE__,__LINE__))
+#else
+#  define Q_CHECK_PTR(p)
+#endif
+
+#if !defined(QT_NO_COMPAT)			// compatibility with Qt 2
+#  if !defined(CHECK_PTR)
+#    define CHECK_PTR(x) Q_CHECK_PTR(x)
+#  endif
+#endif // QT_NO_COMPAT
+
+enum QtMsgType { QtDebugMsg, QtWarningMsg, QtFatalMsg };
+
+typedef void (*QtMsgHandler)(QtMsgType, const char *);
+Q_EXPORT QtMsgHandler qInstallMsgHandler( QtMsgHandler );
+
+#if !defined(QT_NO_COMPAT)			// compatibility with Qt 2
+typedef QtMsgHandler msg_handler;
+#endif // QT_NO_COMPAT
+
+Q_EXPORT void qSuppressObsoleteWarnings( bool = TRUE );
+
+Q_EXPORT void qObsolete( const char *obj, const char *oldfunc,
+		   const char *newfunc );
+Q_EXPORT void qObsolete( const char *obj, const char *oldfunc );
+Q_EXPORT void qObsolete( const char *message );
+
+
+//
+// Install paths from configure
+//
+
+Q_EXPORT const char *qInstallPath();
+Q_EXPORT const char *qInstallPathDocs();
+Q_EXPORT const char *qInstallPathHeaders();
+Q_EXPORT const char *qInstallPathLibs();
+Q_EXPORT const char *qInstallPathBins();
+Q_EXPORT const char *qInstallPathPlugins();
+Q_EXPORT const char *qInstallPathData();
+Q_EXPORT const char *qInstallPathTranslations();
+Q_EXPORT const char *qInstallPathSysconf();
+
+#endif // __cplusplus
+
+// compilers which follow outdated template instantiation rules
+// require a class to have a comparison operator to exist when
+// a QValueList of this type is instantiated. It's not actually
+// used in the list, though. Hence the dummy implementation.
+// Just in case other code relies on it we better trigger a warning
+// mandating a real implementation.
+#ifdef Q_FULL_TEMPLATE_INSTANTIATION
+#  define Q_DUMMY_COMPARISON_OPERATOR(C) \
+    bool operator==( const C& ) const { \
+        qWarning( #C"::operator==( const "#C"& ) got called." ); \
+        return FALSE; \
+    }
+#else
+#  define Q_DUMMY_COMPARISON_OPERATOR(C)
+#endif
+
+#endif // QGLOBAL_H
 
 //
 // Avoid some particularly useless warnings from some stupid compilers.
@@ -723,6 +1082,7 @@ Q_EXPORT int qWinVersion();
 #    pragma warning(disable: 4660) // template-class specialization 'identifier' is already instantiated
 #    pragma warning(disable: 4355) // 'this' : used in base member initializer list
 #    pragma warning(disable: 4231) // nonstandard extension used : 'extern' before template explicit instantiation
+#    pragma warning(disable: 4710) // function not inlined
 #  elif defined(Q_CC_BOR)
 #    pragma option -w-inl
 #    pragma option -w-aus
@@ -731,149 +1091,5 @@ Q_EXPORT int qWinVersion();
 #    pragma warn -ccc
 #    pragma warn -rch
 #    pragma warn -sig
-#  elif defined(Q_CC_MWERKS)
-#    pragma warn_possunwant off
 #  endif
 #endif
-
-
-#ifndef Q_INLINE_TEMPLATES
-#  define Q_INLINE_TEMPLATES 
-#endif
-
-#ifndef Q_TYPENAME
-#  define Q_TYPENAME typename
-#endif
-
-//
-// Use to avoid "unused parameter" warnings
-//
-
-#define Q_UNUSED(x) (void)x;
-
-//
-// Debugging and error handling
-//
-
-#if !defined(QT_NO_CHECK)
-#  define QT_CHECK_STATE			 // check state of objects etc.
-#  define QT_CHECK_RANGE			 // check range of indexes etc.
-#  define QT_CHECK_NULL			 // check null pointers
-#  define QT_CHECK_MATH			 // check math functions
-#endif
-
-#if !defined(QT_NO_DEBUG) && !defined(QT_DEBUG)
-#  define QT_DEBUG			  // display debug messages
-#  if !defined(QT_NO_COMPAT)
-// source compatibility with Qt 2.x
-#    if !defined(NO_DEBUG) && !defined(DEBUG)
-#      if !defined(Q_OS_MACX)			// clash with MacOS X headers
-#        define DEBUG
-#      endif
-#    endif
-#  endif
-#endif
-
-
-Q_EXPORT void qDebug( const char *, ... )   // print debug message
-#if defined(Q_CC_GNU) && !defined(__INSURE__)
-    __attribute__ ((format (printf, 1, 2)))
-#endif
-;
-
-Q_EXPORT void qWarning( const char *, ... ) // print warning message
-#if defined(Q_CC_GNU) && !defined(__INSURE__)
-    __attribute__ ((format (printf, 1, 2)))
-#endif
-;
-
-Q_EXPORT void qFatal( const char *, ... )   // print fatal message and exit
-#if defined(Q_CC_GNU)
-    __attribute__ ((format (printf, 1, 2)))
-#endif
-;
-
-Q_EXPORT void qSystemWarning( const char *, int code = -1 );
-
-#if !defined(QT_CLEAN_NAMESPACE)
-// source compatibility with Qt 1.x
-
-Q_EXPORT void debug( const char *, ... )    // print debug message
-#if defined(Q_CC_GNU) && !defined(__INSURE__)
-    __attribute__ ((format (printf, 1, 2)))
-#endif
-;
-
-Q_EXPORT void warning( const char *, ... )  // print warning message
-#if defined(Q_CC_GNU) && !defined(__INSURE__)
-    __attribute__ ((format (printf, 1, 2)))
-#endif
-;
-
-Q_EXPORT void fatal( const char *, ... )    // print fatal message and exit
-#if defined(Q_CC_GNU) && !defined(__INSURE__)
-    __attribute__ ((format (printf, 1, 2)))
-#endif
-;
-
-#endif // QT_CLEAN_NAMESPACE
-
-
-#if !defined(Q_ASSERT)
-#if defined(QT_CHECK_STATE)
-#if defined(QT_FATAL_ASSERT)
-#define Q_ASSERT(x)  ((x) ? (void)0 : qFatal("ASSERT: \"%s\" in %s (%d)",#x,__FILE__,__LINE__))
-#else
-#define Q_ASSERT(x)  ((x) ? (void)0 : qWarning("ASSERT: \"%s\" in %s (%d)",#x,__FILE__,__LINE__))
-#endif
-#else
-#define Q_ASSERT(x)
-#endif
-#endif
-
-#if !defined(QT_NO_COMPAT)
-// source compatibility with Qt 2.x
-#ifndef Q_OS_TEMP
-#  if !defined(ASSERT)
-#    define ASSERT(x) Q_ASSERT(x)
-#  endif
-#endif // Q_OS_TEMP
-#endif // QT_NO_COMPAT
-
-
-Q_EXPORT bool qt_check_pointer( bool c, const char *, int );
-
-#if defined(QT_CHECK_NULL)
-#  define Q_CHECK_PTR(p) (qt_check_pointer((p)==0,__FILE__,__LINE__))
-#else
-#  define Q_CHECK_PTR(p)
-#endif
-
-#if !defined(QT_NO_COMPAT)
-// source compatibility with Qt 2.x
-#  if !defined(CHECK_PTR)
-#    define CHECK_PTR(x) Q_CHECK_PTR(x)
-#  endif
-#endif // QT_NO_COMPAT
-
-
-enum QtMsgType { QtDebugMsg, QtWarningMsg, QtFatalMsg };
-
-typedef void (*QtMsgHandler)(QtMsgType, const char *);
-Q_EXPORT QtMsgHandler qInstallMsgHandler( QtMsgHandler );
-
-#if !defined(QT_NO_COMPAT)
-// source compatibility with Qt 2.x
-typedef QtMsgHandler msg_handler;
-#endif
-
-Q_EXPORT void qSuppressObsoleteWarnings( bool = TRUE );
-
-Q_EXPORT void qObsolete( const char *obj, const char *oldfunc,
-		   const char *newfunc );
-Q_EXPORT void qObsolete( const char *obj, const char *oldfunc );
-Q_EXPORT void qObsolete( const char *message );
-
-#endif // __cplusplus
-
-#endif // QGLOBAL_H
