@@ -1,9 +1,9 @@
 /****************************************************************************
-** $Id$
+** $Id: qt/qgpluginmanager.cpp   3.2.3   edited Sep 1 05:03 $
 **
 ** Implementation of QGPluginManager class
 **
-** Copyright (C) 2000-2001 Trolltech AS.  All rights reserved.
+** Copyright (C) 2000-2003 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the tools module of the Qt GUI Toolkit.
 **
@@ -203,6 +203,7 @@ static int similarity( const QString& s1, const QString& s2 )
 
 /*!
   \class QPluginManager qpluginmanager.h
+  \reentrant
   \brief The QPluginManager class provides basic functions to access a certain kind of functionality in libraries.
   \ingroup componentmodel
 
@@ -314,21 +315,21 @@ void QGPluginManager::addLibraryPath( const QString& path )
 	return;
 
 #if defined(Q_OS_WIN32)
-    QString filter = "dll";
+    QString filter = "*.dll";
 #elif defined(Q_OS_MACX)
-    QString filter = "dylib";
+    QString filter = "*.dylib; *.so; *.bundle";
+#elif defined(Q_OS_HPUX)
+    QString filter = "*.sl";
 #elif defined(Q_OS_UNIX)
-    QString filter = "so";
+    QString filter = "*.so";
 #endif
 
-    QStringList plugins = QDir(path).entryList( "*." + filter );
+    QStringList plugins = QDir(path).entryList( filter );
     for ( QStringList::Iterator p = plugins.begin(); p != plugins.end(); ++p ) {
-	QString lib = path + "/" + *p;
+	QString lib = QDir::cleanDirPath( path + "/" + *p );
 	if ( libList.contains( lib ) )
 	    continue;
-
-	// use QFileInfo::filePath() to correct backslashes to slashes
-	libList.append( QFileInfo( lib ).filePath() );
+	libList.append( lib );
     }
 }
 
@@ -476,7 +477,7 @@ bool QGPluginManager::addLibrary( QLibrary* lib )
 	else if ( cpiFace )
 	    fl << cpiFace->name();
 
-	for ( QStringList::Iterator f = fl.begin(); f != fl.end(); f++ ) {
+	for ( QStringList::Iterator f = fl.begin(); f != fl.end(); ++f ) {
 	    QLibrary *old = plugDict[*f];
 	    if ( !old ) {
 		useful = TRUE;
@@ -530,8 +531,9 @@ bool QGPluginManager::enabled() const
 {
 #ifdef QT_SHARED
     return TRUE;
-#endif
+#else
     return FALSE;
+#endif
 }
 
 QRESULT QGPluginManager::queryUnknownInterface(const QString& feature, QUnknownInterface** iface) const
