@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id$
+** $Id: qt/qgvector.cpp   3.2.3   edited May 13 09:08 $
 **
 ** Implementation of QGVector class
 **
@@ -35,6 +35,12 @@
 **
 **********************************************************************/
 
+#include "qglobal.h"
+#if defined(Q_CC_BOR)
+// needed for qsort() because of a std namespace problem on Borland
+#include "qplatformdefs.h"
+#endif
+
 #define	 QGVECTOR_CPP
 #include "qgvector.h"
 #include "qglist.h"
@@ -42,6 +48,10 @@
 #include "qdatastream.h"
 #include <stdlib.h>
 
+#ifdef QT_THREAD_SUPPORT
+#  include <private/qmutexpool_p.h>
+#endif // QT_THREAD_SUPPORT
+
 #define USE_MALLOC				// comment to use new/delete
 
 #undef NEW
@@ -58,6 +68,7 @@
 
 /*!
   \class QGVector
+  \reentrant
   \ingroup collection
 
   \brief The QGVector class is an internal class for implementing Qt
@@ -386,6 +397,12 @@ void QGVector::sort()				// sort vector
 	    break;
 	}
     }
+
+#ifdef QT_THREAD_SUPPORT
+    QMutexLocker locker( qt_global_mutexpool ?
+			 qt_global_mutexpool->get( &sort_vec ) : 0 );
+#endif // QT_THREAD_SUPPORT
+
     sort_vec = (QGVector*)this;
     qsort( vec, count(), sizeof(Item), cmp_vec );
     sort_vec = 0;
