@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/qiodevice.cpp   3.1.2   edited Nov 7 16:55 $
+** $Id: qt/qiodevice.cpp   3.2.3   edited Jun 1 18:37 $
 **
 ** Implementation of QIODevice class
 **
@@ -563,7 +563,7 @@ bool QIODevice::at( Offset pos )
 {
 #if defined(QT_CHECK_RANGE)
     if ( pos > size() ) {
-#if defined(QT_LARGEFILE_SUPPORT) && defined(QT_ABI_64BITOFFSET)
+#if defined(QT_LARGEFILE_SUPPORT) && defined(QT_ABI_QT4)
 	qWarning( "QIODevice::at: Index %llu out of range", pos );
 #else
 	qWarning( "QIODevice::at: Index %lu out of range", pos );
