@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id:  qt/qstringlist.cpp   3.0.5   edited Nov 8 2001 $
+** $Id: qt/qstringlist.cpp   3.2.3   edited Jun 25 14:57 $
 **
 ** Implementation of QStringList
 **
@@ -44,23 +44,24 @@
 #include "qtl.h"
 
 /*!
-  \class QStringList qstringlist.h
-  \brief The QStringList class provides a list of strings.
+    \class QStringList qstringlist.h
+    \reentrant
+    \brief The QStringList class provides a list of strings.
 
-  \ingroup tools
-  \ingroup shared
-  \ingroup text
-  \mainclass
+    \ingroup tools
+    \ingroup shared
+    \ingroup text
+    \mainclass
 
-  It is used to store and manipulate strings that logically belong
-  together.  Basically QStringList is a QValueList of QString
-  objects. As opposed to QStrList, which stores pointers to
-  characters, QStringList deals with real QString objects.  It is the
-  class of choice whenever you work with Unicode strings. QStringList
-  is part of the <a href="qtl.html"> Qt Template Library</a>.
+    It is used to store and manipulate strings that logically belong
+    together. Essentially QStringList is a QValueList of QString
+    objects. Unlike QStrList, which stores pointers to characters,
+    QStringList holds real QString objects. It is the class of choice
+    whenever you work with Unicode strings. QStringList is part of the
+    \link qtl.html Qt Template Library\endlink.
 
-  Like QString itself, QStringList objects are implicitly shared.
-  Passing them around as value-parameters is both fast and safe.
+    Like QString itself, QStringList objects are implicitly shared, so
+    passing them around as value-parameters is both fast and safe.
 
     Strings can be added to a list using append(), operator+=() or
     operator<<(), e.g.
@@ -96,8 +97,8 @@
 
     You can sort the list with sort(), and extract a new list which
     contains only those strings which contain a particular substring
-    (or match a particular regular expression) using the
-    grep() functions, e.g.
+    (or match a particular regular expression) using the grep()
+    functions, e.g.
     \code
     fonts.sort();
     cout << fonts.join( ", " ) << endl;
@@ -119,28 +120,32 @@
     // Output:
     //	Red, Green, Blue
     \endcode
-
 */
 
 /*!
- \fn QStringList::QStringList()
-  Creates an empty string list.
-*/
+    \fn QStringList::QStringList()
 
-/*! \fn QStringList::QStringList( const QStringList& l )
-
-  Creates a copy of the list \a l. This function is very fast because
-  QStringList is implicitly shared. However, for the programmer this
-  is the same as a deep copy. If this list or the original one or some
-  other list referencing the same shared data is modified, the
-  modifying list first makes a copy, i.e. copy-on-write.
+    Creates an empty string list.
 */
 
 /*!
-  \fn QStringList::QStringList (const QString & i)
+    \fn QStringList::QStringList( const QStringList& l )
 
-  Constructs a string list consisting of the single string \a i.
-  Longer lists are easily created as follows:
+    Creates a copy of the list \a l. This function is very fast
+    because QStringList is implicitly shared. In most situations this
+    acts like a deep copy, for example, if this list or the original
+    one or some other list referencing the same shared data is
+    modified, the modifying list first makes a copy, i.e.
+    copy-on-write.
+    In a threaded environment you may require a real deep copy
+    \omit see \l QDeepCopy\endomit.
+*/
+
+/*!
+    \fn QStringList::QStringList (const QString & i)
+
+    Constructs a string list consisting of the single string \a i.
+    Longer lists are easily created as follows:
 
     \code
     QStringList items;
@@ -149,38 +154,42 @@
 */
 
 /*!
-  \fn QStringList::QStringList (const char* i)
-  Constructs a string list consisting of the single latin-1 string \a i.
-*/
+    \fn QStringList::QStringList (const char* i)
 
-/*! \fn QStringList::QStringList( const QValueList<QString>& l )
-
-  Constructs a new string list that is a copy of \a l.
+    Constructs a string list consisting of the single Latin-1 string \a i.
 */
 
 /*!
-  Sorts the list of strings in ascending case-sensitive order.
+    \fn QStringList::QStringList( const QValueList<QString>& l )
 
-  Sorting is very fast. It uses the Qt Template Library's efficient
-  HeapSort implementation that has a time complexity of O(n*log n).
+    Constructs a new string list that is a copy of \a l.
+*/
 
-  If you want to sort your strings in an arbitrary order consider
-  using a QMap. For example you could use a QMap\<QString,QString\> to
-  create a case-insensitive ordering (e.g. mapping the lowercase text
-  to the text), or a QMap\<int,QString\> to sort the strings by some
-  integer index, etc.
+/*!
+    Sorts the list of strings in ascending case-sensitive order.
+
+    Sorting is very fast. It uses the \link qtl.html Qt Template
+    Library's\endlink efficient HeapSort implementation that has a
+    time complexity of O(n*log n).
+
+    If you want to sort your strings in an arbitrary order consider
+    using a QMap. For example you could use a QMap\<QString,QString\>
+    to create a case-insensitive ordering (e.g. mapping the lowercase
+    text to the text), or a QMap\<int,QString\> to sort the strings by
+    some integer index, etc.
 */
 void QStringList::sort()
 {
     qHeapSort( *this );
 }
 
-/*! \overload
+/*!
+    \overload
 
-  This version of the function uses a QChar as separator, rather than
-  a regular expression.
+    This version of the function uses a QChar as separator, rather
+    than a regular expression.
 
-  \sa join() QString::section()
+    \sa join() QString::section()
 */
 
 QStringList QStringList::split( const QChar &sep, const QString &str,
@@ -189,20 +198,21 @@ QStringList QStringList::split( const QChar &sep, const QString &str,
     return split( QString(sep), str, allowEmptyEntries );
 }
 
-/*! \overload
+/*!
+    \overload
 
-  This version of the function uses a QString as separator, rather than
-  a regular expression.
+    This version of the function uses a QString as separator, rather
+    than a regular expression.
 
-  If \a sep is an empty string, the return value is a list of
-  one-character strings: split( QString( "" ), "mfc" ) returns the
-  three-item list, "m", "f", "c".
+    If \a sep is an empty string, the return value is a list of
+    one-character strings: split( QString( "" ), "four" ) returns the
+    four-item list, "f", "o", "u", "r".
 
-  If \a allowEmptyEntries is TRUE, an empty string is inserted in the
-  list wherever the separator matches twice without intervening
-  text.
+    If \a allowEmptyEntries is TRUE, an empty string is inserted in
+    the list wherever the separator matches twice without intervening
+    text.
 
-  \sa join() QString::section()
+    \sa join() QString::section()
 */
 
 QStringList QStringList::split( const QString &sep, const QString &str,
@@ -231,23 +241,25 @@ QStringList QStringList::split( const QString &sep, const QString &str,
     return lst;
 }
 
-/*! Splits the string \a str into strings wherever the
-    regular expression \a sep occurs, and
-  returns the list of those strings.
+#ifndef QT_NO_REGEXP
+/*!
+    Splits the string \a str into strings wherever the regular
+    expression \a sep occurs, and returns the list of those strings.
 
-  If \a allowEmptyEntries is TRUE, an empty string is inserted in the
-  list wherever the separator matches twice without intervening
-  text.
+    If \a allowEmptyEntries is TRUE, an empty string is inserted in
+    the list wherever the separator matches twice without intervening
+    text.
 
-  For example, if you split the string "a,,b,c" on commas, split()
-  returns the three-item list "a", "b", "c" if \a allowEmptyEntries is
-  FALSE (the default), and the four-item list "a", "", "b", "c" if \a
-  allowEmptyEntries is TRUE.
+    For example, if you split the string "a,,b,c" on commas, split()
+    returns the three-item list "a", "b", "c" if \a allowEmptyEntries
+    is FALSE (the default), and the four-item list "a", "", "b", "c"
+    if \a allowEmptyEntries is TRUE.
 
-  If \a sep does not match anywhere in \a str, split() returns a list
-  consisting of the single string \a str.
+    If \a sep does not match anywhere in \a str, split() returns a
+    single element list with the element containing the single string
+    \a str.
 
-  \sa join() QString::section()
+    \sa join() QString::section()
 */
 
 QStringList QStringList::split( const QRegExp &sep, const QString &str,
@@ -280,12 +292,22 @@ QStringList QStringList::split( const QRegExp &sep, const QString &str,
 
     return lst;
 }
+#endif
 
 /*!
-  Returns a list of all strings containing the substring \a str.
+    Returns a list of all the strings containing the substring \a str.
 
-  If \a cs is TRUE, the grep is done case-sensitively; otherwise case
-  is ignored.
+    If \a cs is TRUE, the grep is done case-sensitively; otherwise
+    case is ignored.
+
+    \code
+    QStringList list;
+    list << "Bill Gates" << "John Doe" << "Bill Clinton";
+    list = list.grep( "Bill" );
+    // list == ["Bill Gates", "Bill Clinton"]
+    \endcode
+
+    \sa QString::find()
 */
 
 QStringList QStringList::grep( const QString &str, bool cs ) const
@@ -298,27 +320,103 @@ QStringList QStringList::grep( const QString &str, bool cs ) const
     return res;
 }
 
-/*! \overload
+#ifndef QT_NO_REGEXP
+/*!
+    \overload
 
-  Returns a list of all the strings that contain a substring that matches
-  the regular expression \a expr.
+    Returns a list of all the strings that match the regular
+    expression \a rx.
+
+    \sa QString::find()
 */
 
-QStringList QStringList::grep( const QRegExp &expr ) const
+QStringList QStringList::grep( const QRegExp &rx ) const
 {
     QStringList res;
     for ( QStringList::ConstIterator it = begin(); it != end(); ++it )
-	if ( (*it).contains(expr) )
+	if ( (*it).find(rx) != -1 )
 	    res << *it;
 
     return res;
 }
+#endif
 
 /*!
-  Joins the string list into a single string with each element
-  separated by the string \a sep.
+    Replaces every occurrence of the string \a before in the strings
+    that constitute the string list with the string \a after. Returns
+    a reference to the string list.
 
-  \sa split()
+    If \a cs is TRUE, the search is case sensitive; otherwise the
+    search is case insensitive.
+
+    Example:
+    \code
+    QStringList list;
+    list << "alpha" << "beta" << "gamma" << "epsilon";
+    list.gres( "a", "o" );
+    // list == ["olpho", "beto", "gommo", "epsilon"]
+    \endcode
+
+    \sa QString::replace()
+*/
+QStringList& QStringList::gres( const QString &before, const QString &after,
+				bool cs )
+{
+    QStringList::Iterator it = begin();
+    while ( it != end() ) {
+	(*it).replace( before, after, cs );
+	++it;
+    }
+    return *this;
+}
+
+#ifndef QT_NO_REGEXP_CAPTURE
+/*!
+    \overload
+
+    Replaces every occurrence of the regexp \a rx in the string
+    with \a after. Returns a reference to the string list.
+
+    Example:
+    \code
+    QStringList list;
+    list << "alpha" << "beta" << "gamma" << "epsilon";
+    list.gres( QRegExp("^a"), "o" );
+    // list == ["olpha", "beta", "gamma", "epsilon"]
+    \endcode
+
+    For regexps containing \link qregexp.html#capturing-text
+    capturing parentheses \endlink, occurrences of <b>\\1</b>,
+    <b>\\2</b>, ..., in \a after are replaced with \a{rx}.cap(1),
+    cap(2), ...
+
+    Example:
+    \code
+    QStringList list;
+    list << "Bill Clinton" << "Gates, Bill";
+    list.gres( QRegExp("^(.*), (.*)$"), "\\2 \\1" );
+    // list == ["Bill Clinton", "Bill Gates"]
+    \endcode
+
+    \sa QString::replace()
+*/
+QStringList& QStringList::gres( const QRegExp &rx, const QString &after )
+{
+    QStringList::Iterator it = begin();
+    while ( it != end() ) {
+	(*it).replace( rx, after );
+	++it;
+    }
+    return *this;
+}
+
+#endif
+
+/*!
+    Joins the string list into a single string with each element
+    separated by the string \a sep (which can be empty).
+
+    \sa split()
 */
 QString QStringList::join( const QString &sep ) const
 {
@@ -346,7 +444,8 @@ Q_EXPORT QDataStream &operator<<( QDataStream & s, const QStringList& l )
 }
 #endif
 
-/*! Converts from an ASCII-QStrList \a ascii to a QStringList (Unicode).
+/*!
+    Converts from an ASCII-QStrList \a ascii to a QStringList (Unicode).
 */
 QStringList QStringList::fromStrList(const QStrList& ascii)
 {
@@ -357,4 +456,9 @@ QStringList QStringList::fromStrList(const QStrList& ascii)
     return res;
 }
 
+/*! \fn void QStringList::detach()
+    \reimp
+*/
+
+
 #endif //QT_NO_STRINGLIST
