@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id:  qt/qbuttongroup.cpp   3.0.5   edited Oct 12 2001 $
+** $Id: qt/qbuttongroup.cpp   3.2.3   edited May 24 15:31 $
 **
 ** Implementation of QButtonGroup class
 **
@@ -40,68 +40,72 @@
 #include "qbutton.h"
 #include "qptrlist.h"
 #include "qapplication.h"
+#include "qradiobutton.h"
 
 
 
 /*!
-  \class QButtonGroup qbuttongroup.h
-  \brief The QButtonGroup widget organizes QButton widgets in a group.
+    \class QButtonGroup qbuttongroup.h
+    \brief The QButtonGroup widget organizes QButton widgets in a group.
 
-  \ingroup organizers
-  \ingroup geomanagement
-  \ingroup appearance
-  \mainclass
+    \ingroup organizers
+    \ingroup geomanagement
+    \ingroup appearance
+    \mainclass
 
-  A button group widget makes it easier to deal with groups of
-  buttons.  Each button in a button group has a unique
-  identifier. The button group emits a clicked() signal with this
-  identifier when the button is clicked. This makes a button group
-  particularly useful when you have several similar buttons and want to
-  connect all their clicked() signals to one slot.
+    A button group widget makes it easier to deal with groups of
+    buttons. Each button in a button group has a unique identifier.
+    The button group emits a clicked() signal with this identifier
+    when a button in the group is clicked. This makes a button group
+    particularly useful when you have several similar buttons and want
+    to connect all their clicked() signals to a single slot.
 
-  An \link setExclusive() exclusive\endlink button group switches off
-  all toggle buttons except the one that was clicked. A button group
-  is by default non-exclusive. By default, all radio
-  buttons that are inserted will be
-  mutually exclusive even if the button group is non-exclusive. (See
-  setRadioButtonExclusive().)
+    An \link setExclusive() exclusive\endlink button group switches
+    off all toggle buttons except the one that was clicked. A button
+    group is, by default, non-exclusive. Note that all radio buttons
+    that are inserted into a button group are mutually exclusive even
+    if the button group is non-exclusive. (See
+    setRadioButtonExclusive().)
 
-  There are two ways of using a button group:
-  \list
-  \i The button group is a parent widget of a number of buttons,
-  i.e., the button group is the parent argument in the button constructor.
-  The buttons are assigned identifiers 0, 1, 2, etc. in the order they are
-  created. A QButtonGroup can display a frame and a title because it inherits
-  QGroupBox.
-  \i The button group is an invisible widget and the contained buttons
-  have some other parent widget.  A button must then be manually inserted
-  using the insert() function with an identifier.
-  \endlist
+    There are two ways of using a button group:
+    \list
+    \i The button group is the parent widget of a number of buttons,
+    i.e. the button group is the parent argument in the button
+    constructor. The buttons are assigned identifiers 0, 1, 2, etc.,
+    in the order they are created. A QButtonGroup can display a frame
+    and a title because it inherits QGroupBox.
+    \i The button group is an invisible widget and the contained
+    buttons have some other parent widget. In this usage, each button
+    must be manually inserted, using insert(), into the button group
+    and given an identifier.
+    \endlist
 
-  A button can be removed from the group with remove(). A pointer to a
-  button with a given id can be obtained using find(). The id of a
-  button is available using id(). A button can be set on with
-  setButton(). The number of buttons in the group is returned by
-  count().
+    A button can be removed from the group with remove(). A pointer to
+    a button with a given id can be obtained using find(). The id of a
+    button is available using id(). A button can be set \e on with
+    setButton(). The number of buttons in the group is returned by
+    count().
 
-  <img src=qbttngrp-m.png> <img src=qbttngrp-w.png>
+    <img src=qbttngrp-m.png> <img src=qbttngrp-w.png>
 
-  \sa QButton, QPushButton, QCheckBox, QRadioButton
+    \sa QPushButton, QCheckBox, QRadioButton
 */
 
-/*! \property QButtonGroup::exclusive
+/*!
+    \property QButtonGroup::exclusive
     \brief whether the button group is exclusive
 
-  If this property is TRUE, then the buttons in the group are
-  toggled, and to untoggle a button you must click on another button in
-  the group. The default value is FALSE.
+    If this property is TRUE, then the buttons in the group are
+    toggled, and to untoggle a button you must click on another button
+    in the group. The default value is FALSE.
 */
 
-/*! \property QButtonGroup::radioButtonExclusive
-    \brief whether the radiobuttons in the group are exclusive
+/*!
+    \property QButtonGroup::radioButtonExclusive
+    \brief whether the radio buttons in the group are exclusive
 
-  If this property is TRUE (the default), the radiobuttons in the
-  group are treated exclusively.
+    If this property is TRUE (the default), the \link QRadioButton
+    radiobuttons\endlink in the group are treated exclusively.
 */
 
 struct QButtonItem
@@ -123,9 +127,10 @@ typedef QPtrListIterator<QButtonItem> QButtonListIt;
 
 
 /*!
-  Constructs a button group with no title.
+    Constructs a button group with no title.
 
-  The \a parent and \a name arguments are passed to the QWidget constructor.
+    The \a parent and \a name arguments are passed to the QWidget
+    constructor.
 */
 
 QButtonGroup::QButtonGroup( QWidget *parent, const char *name )
@@ -135,9 +140,10 @@ QButtonGroup::QButtonGroup( QWidget *parent, const char *name )
 }
 
 /*!
-  Constructs a button group with the title \a title.
+    Constructs a button group with the title \a title.
 
-  The \a parent and \a name arguments are passed to the QWidget constructor.
+    The \a parent and \a name arguments are passed to the QWidget
+    constructor.
 */
 
 QButtonGroup::QButtonGroup( const QString &title, QWidget *parent,
@@ -148,10 +154,12 @@ QButtonGroup::QButtonGroup( const QString &title, QWidget *parent,
 }
 
 /*!
-  Constructs a button group with no title. Child widgets will be arranged
-  in \a strips rows or columns (depending on \a orientation).
+    Constructs a button group with no title. Child widgets will be
+    arranged in \a strips rows or columns (depending on \a
+    orientation).
 
-  The \a parent and \a name arguments are passed to the QWidget constructor.
+    The \a parent and \a name arguments are passed to the QWidget
+    constructor.
 */
 
 QButtonGroup::QButtonGroup( int strips, Orientation orientation,
@@ -162,10 +170,12 @@ QButtonGroup::QButtonGroup( int strips, Orientation orientation,
 }
 
 /*!
-  Constructs a button group with title \a title. Child widgets will be
-  arranged in \a strips rows or columns (depending on \a orientation).
+    Constructs a button group with title \a title. Child widgets will
+    be arranged in \a strips rows or columns (depending on \a
+    orientation).
 
-  The \a parent and \a name arguments are passed to the QWidget constructor.
+    The \a parent and \a name arguments are passed to the QWidget
+    constructor.
 */
 
 QButtonGroup::QButtonGroup( int strips, Orientation orientation,
@@ -177,7 +187,7 @@ QButtonGroup::QButtonGroup( int strips, Orientation orientation,
 }
 
 /*!
-  Initializes the button group.
+    Initializes the button group.
 */
 
 void QButtonGroup::init()
@@ -215,23 +225,24 @@ void QButtonGroup::setExclusive( bool enable )
 
 
 /*!
-  Inserts the \a button with the identifier \a id into the button group.
-  Returns the button identifier.
+    Inserts the \a button with the identifier \a id into the button
+    group. Returns the button identifier.
 
     Buttons are normally inserted into a button group automatically by
-    giving the button group as the parent when the button is
-    constructed. So it is not necessary to manually insert buttons that
-    have this button group as their parent widget. An exception is when
-    you want custom identifiers instead of the default 0, 1, 2, etc.
+    passing the button group as the parent when the button is
+    constructed. So it is not necessary to manually insert buttons
+    that have this button group as their parent widget. An exception
+    is when you want custom identifiers instead of the default 0, 1,
+    2, etc., or if you want the buttons to have some other parent.
 
-  The button is assigned the identifier \a id or an automatically
-  generated identifier.	 It works as follows: If \a id >= 0, this
-  identifier is assigned.  If \a id == -1 (default), the identifier is
-  equal to the number of buttons in the group.	If \a id is any other
-  negative integer, for instance -2, a unique identifier (negative
-  integer \<= -2) is generated. No button has an id of -1.
+    The button is assigned the identifier \a id or an automatically
+    generated identifier. It works as follows: If \a id >= 0, this
+    identifier is assigned. If \a id == -1 (default), the identifier
+    is equal to the number of buttons in the group. If \a id is any
+    other negative integer, for instance -2, a unique identifier
+    (negative integer \<= -2) is generated. No button has an id of -1.
 
-  \sa find(), remove(), setExclusive()
+    \sa find(), remove(), setExclusive()
 */
 
 int QButtonGroup::insert( QButton *button, int id )
@@ -268,7 +279,7 @@ int QButtonGroup::insert( QButton *button, int id )
 }
 
 /*!
-  Returns the number of buttons in the group.
+    Returns the number of buttons in the group.
 */
 int QButtonGroup::count() const
 {
@@ -276,8 +287,9 @@ int QButtonGroup::count() const
 }
 
 /*!
-  Removes the \a button from the button group.
-  \sa insert()
+    Removes the \a button from the button group.
+
+    \sa insert()
 */
 
 void QButtonGroup::remove( QButton *button )
@@ -300,10 +312,8 @@ void QButtonGroup::remove( QButton *button )
 
 
 /*!
-  Finds and returns a pointer to the button with the specified identifier
-  \a id.
-
-  Returns null if the button was not found.
+    Returns the button with the specified identifier \a id, or 0 if
+    the button was not found.
 */
 
 QButton *QButtonGroup::find( int id ) const
@@ -317,26 +327,33 @@ QButton *QButtonGroup::find( int id ) const
 
 
 /*!
-  \fn void QButtonGroup::pressed( int id )
-  This signal is emitted when a button in the group is
-  \link QButton::pressed() pressed\endlink.
-  The \a id argument is the button's identifier.
+    \fn void QButtonGroup::pressed( int id )
+
+    This signal is emitted when a button in the group is \link
+    QButton::pressed() pressed\endlink. The \a id argument is the
+    button's identifier.
+
+    \sa insert()
 */
 
 /*!
-  \fn void QButtonGroup::released( int id )
-  This signal is emitted when a button in the group is
-  \link QButton::released() released\endlink.
-  The \a id argument is the button's identifier.
+    \fn void QButtonGroup::released( int id )
+
+    This signal is emitted when a button in the group is \link
+    QButton::released() released\endlink. The \a id argument is the
+    button's identifier.
+
+    \sa insert()
 */
 
 /*!
-  \fn void QButtonGroup::clicked( int id )
+    \fn void QButtonGroup::clicked( int id )
 
-  This signal is emitted when a button in the group is clicked.  The
-  \a id argument is the button's identifier.
+    This signal is emitted when a button in the group is \link
+    QButton::clicked() clicked\endlink. The \a id argument is the
+    button's identifier.
 
-  \sa QButton::clicked() insert()
+    \sa insert()
 */
 
 
@@ -390,9 +407,9 @@ void QButtonGroup::buttonClicked()
 {
     // introduce a QButtonListIt if calling anything
     int id = -1;
-    QButton *bt = (QButton *)sender();		// object that sent the signal
+    QButton *bt = ::qt_cast<QButton*>(sender());		// object that sent the signal
 #if defined(QT_CHECK_NULL)
-    Q_ASSERT( bt->inherits("QButton") );
+    Q_ASSERT( bt );
 #endif
     for ( register QButtonItem *i=buttons->first(); i; i=buttons->next() ) {
 	if ( bt == i->button ) {			// button was clicked
@@ -416,19 +433,19 @@ void QButtonGroup::buttonToggled( bool on )
     // introduce a QButtonListIt if calling anything
     if ( !on || !excl_grp && !radio_excl )
 	return;
-    QButton *bt = (QButton *)sender();		// object that sent the signal
+    QButton *bt = ::qt_cast<QButton*>(sender());		// object that sent the signal
 #if defined(QT_CHECK_NULL)
-    Q_ASSERT( bt->inherits("QButton") );
+    Q_ASSERT( bt );
     Q_ASSERT( bt->isToggleButton() );
 #endif
 
-    if ( !excl_grp && !bt->inherits("QRadioButton") )
+    if ( !excl_grp && !::qt_cast<QRadioButton*>(bt) )
 	return;
     QButtonItem * i = buttons->first();
     bool hasTabFocus = FALSE;
 
     while( i != 0 && hasTabFocus == FALSE ) {
-	if ( ( excl_grp || i->button->inherits("QRadioButton") ) &&
+	if ( ( excl_grp || ::qt_cast<QRadioButton*>(i->button) ) &&
 	     (i->button->focusPolicy() & TabFocus) )
 	    hasTabFocus = TRUE;
 	i = buttons->next();
@@ -439,9 +456,9 @@ void QButtonGroup::buttonToggled( bool on )
 	if ( bt != i->button &&
 	     i->button->isToggleButton() &&
 	     i->button->isOn() &&
-	     ( excl_grp || i->button->inherits( "QRadioButton" ) ) )
+	     ( excl_grp || ::qt_cast<QRadioButton*>(i->button) ) )
 	    i->button->setOn( FALSE );
-	if ( ( excl_grp || i->button->inherits( "QRadioButton" ) ) &&
+	if ( ( excl_grp || ::qt_cast<QRadioButton*>(i->button) ) &&
 	     i->button->isToggleButton() &&
 	     hasTabFocus )
 	    i->button->setFocusPolicy( (FocusPolicy)(i->button->focusPolicy() &
@@ -455,8 +472,9 @@ void QButtonGroup::buttonToggled( bool on )
 
 
 
-/*!  Sets the button with id \a id to be on; if this is an
-  exclusive group, all other buttons in the group will be set to off.
+/*!
+    Sets the button with id \a id to be on; if this is an exclusive
+    group, all the other buttons in the group will be set to off.
 */
 
 void QButtonGroup::setButton( int id )
@@ -472,12 +490,13 @@ void QButtonGroup::setRadioButtonExclusive( bool on)
 }
 
 
-/*!  Moves the keyboard focus according to \a key, and if appropriate
-  checks the new focus item.
+/*!
+    Moves the keyboard focus according to \a key, and if appropriate
+    checks the new focus item.
 
-  This function does nothing unless the keyboard focus points to one
-  of the button group members and \a key is one of \c Key_Up, \c
-  Key_Down, \c Key_Left and \c Key_Right.
+    This function does nothing unless the keyboard focus points to one
+    of the button group members and \a key is one of \c Key_Up, \c
+    Key_Down, \c Key_Left and \c Key_Right.
 */
 
 void QButtonGroup::moveFocus( int key )
@@ -504,37 +523,50 @@ void QButtonGroup::moveFocus( int key )
 	    QPoint p(i->button->mapToGlobal(i->button->geometry().center()));
 	    int score = (p.y() - goal.y())*(p.y() - goal.y()) +
 			(p.x() - goal.x())*(p.x() - goal.x());
+	    bool betterScore = score < bestScore || !candidate;
 	    switch( key ) {
 	    case Key_Up:
-		if ( p.y() < goal.y() &&
-		     QABS( p.x() - goal.x() ) < QABS( p.y() - goal.y() ) &&
-		     ( score < bestScore || !candidate ) ) {
-		    candidate = i->button;
-		    bestScore = score;
+		if ( p.y() < goal.y() && betterScore ) {
+		    if ( QABS( p.x() - goal.x() ) < QABS( p.y() - goal.y() ) ) {
+			candidate = i->button;
+			bestScore = score;
+		    } else if ( i->button->x() == f->x() ) {
+			candidate = i->button;
+			bestScore = score/2;
+		    }
 		}
 		break;
 	    case Key_Down:
-		if ( p.y() > goal.y() &&
-		     QABS( p.x() - goal.x() ) < QABS( p.y() - goal.y() ) &&
-		     ( score < bestScore || !candidate ) ) {
-		    candidate = i->button;
-		    bestScore = score;
+		if ( p.y() > goal.y() && betterScore ) {
+		    if ( QABS( p.x() - goal.x() ) < QABS( p.y() - goal.y() ) ) {
+			candidate = i->button;
+			bestScore = score;
+		    } else if ( i->button->x() == f->x() ) {
+			candidate = i->button;
+			bestScore = score/2;
+		    }
 		}
 		break;
 	    case Key_Left:
-		if ( p.x() < goal.x() &&
-		     QABS( p.y() - goal.y() ) < QABS( p.x() - goal.x() ) &&
-		     ( score < bestScore || !candidate ) ) {
-		    candidate = i->button;
-		    bestScore = score;
+		if ( p.x() < goal.x() && betterScore ) {
+		    if ( QABS( p.y() - goal.y() ) < QABS( p.x() - goal.x() ) ) {
+			candidate = i->button;
+			bestScore = score;
+		    } else if ( i->button->y() == f->y() ) {
+			candidate = i->button;
+			bestScore = score/2;
+		    }
 		}
 		break;
 	    case Key_Right:
-		if ( p.x() > goal.x() &&
-		     QABS( p.y() - goal.y() ) < QABS( p.x() - goal.x() ) &&
-		     ( score < bestScore || !candidate ) ) {
-		    candidate = i->button;
-		    bestScore = score;
+		if ( p.x() > goal.x() && betterScore ) {
+		    if ( QABS( p.y() - goal.y() ) < QABS( p.x() - goal.x() ) ) {
+			candidate = i->button;
+			bestScore = score;
+		    } else if ( i->button->y() == f->y() ) {
+			candidate = i->button;
+			bestScore = score/2;
+		    }
 		}
 		break;
 	    }
@@ -542,20 +574,20 @@ void QButtonGroup::moveFocus( int key )
 	i = buttons->next();
     }
 
-    if ( candidate && f && f->inherits( "QButton" ) &&
+    QButton *buttoncand = ::qt_cast<QButton*>(candidate);
+    if ( buttoncand && ::qt_cast<QButton*>(f) &&
 	 ((QButton*)f)->isOn() &&
-	 candidate->inherits( "QButton" ) &&
-	 ((QButton*)candidate)->isToggleButton() &&
-	 ( isExclusive() || ( f->inherits( "QRadioButton" ) &&
-			      candidate->inherits( "QRadioButton" )))) {
+	 buttoncand->isToggleButton() &&
+	 ( isExclusive() || ( ::qt_cast<QRadioButton*>(f) &&
+			      ::qt_cast<QRadioButton*>(candidate)))) {
 	if ( f->focusPolicy() & TabFocus ) {
 	    f->setFocusPolicy( (FocusPolicy)(f->focusPolicy() & ~TabFocus) );
 	    candidate->setFocusPolicy( (FocusPolicy)(candidate->focusPolicy()|
 						     TabFocus) );
 	}
-	((QButton*)candidate)->setOn( TRUE );
-	((QButton*)candidate)->animateClick();
-	((QButton*)candidate)->animateTimeout(); // ### crude l&f hack
+	buttoncand->setOn( TRUE );
+	buttoncand->animateClick();
+	buttoncand->animateTimeout(); // ### crude l&f hack
     }
 
     if ( candidate )
@@ -564,8 +596,10 @@ void QButtonGroup::moveFocus( int key )
 
 
 /*!
-  Returns a pointer to the selected toggle button if exactly one
-  is selected; returns 0 otherwise.
+    Returns the selected toggle button if exactly one is selected;
+    otherwise returns 0.
+
+    \sa selectedId()
 */
 
 QButton * QButtonGroup::selected() const
@@ -587,9 +621,24 @@ QButton * QButtonGroup::selected() const
     return candidate;
 }
 
+/*!
+    Returns the id of the selected toggle button if exactly one is selected;
+    otherwise returns -1.
 
-/*! Returns the id of \a button, or -1 if \a button is not a member of
-  this group.
+    \sa selected()
+*/
+
+int QButtonGroup::selectedId() const
+{
+    return id( selected() );
+}
+
+
+/*!
+    Returns the id of \a button, or -1 if \a button is not a member of
+    this group.
+
+    \sa selectedId();
 */
 
 int QButtonGroup::id( QButton * button ) const
@@ -599,4 +648,23 @@ int QButtonGroup::id( QButton * button ) const
 	i = buttons->next();
     return i ? i->id : -1;
 }
+
+
+/*!
+    \reimp
+*/
+bool QButtonGroup::event( QEvent * e )
+{
+    if ( e->type() == QEvent::ChildInserted ) {
+	QChildEvent * ce = (QChildEvent *) e;
+	if ( radio_excl && ::qt_cast<QRadioButton*>(ce->child()) ) {
+	    QButton * button = (QButton *) ce->child();
+	    if ( button->isToggleButton() && !button->isOn() &&
+		 selected() && (selected()->focusPolicy() & TabFocus) != 0 )
+		button->setFocusPolicy( (FocusPolicy)(button->focusPolicy() &
+					      ~TabFocus) );
+	}
+    }
+    return QGroupBox::event( e );
+}
 #endif
