@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/qcheckbox.cpp   3.1.2   edited Feb 5 11:17 $
+** $Id: qt/qcheckbox.cpp   3.2.3   edited May 13 09:08 $
 **
 ** Implementation of QCheckBox class
 **
@@ -122,7 +122,7 @@
 */
 
 QCheckBox::QCheckBox( QWidget *parent, const char *name )
-	: QButton( parent, name, WRepaintNoErase | WResizeNoErase | WMouseNoMask )
+	: QButton( parent, name, WNoAutoErase | WMouseNoMask )
 {
     setToggleButton( TRUE );
     setSizePolicy( QSizePolicy( QSizePolicy::Minimum, QSizePolicy::Fixed ) );
@@ -136,7 +136,7 @@ QCheckBox::QCheckBox( QWidget *parent, const char *name )
 */
 
 QCheckBox::QCheckBox( const QString &text, QWidget *parent, const char *name )
-	: QButton( parent, name, WRepaintNoErase | WResizeNoErase | WMouseNoMask )
+	: QButton( parent, name, WNoAutoErase | WMouseNoMask )
 {
     setText( text );
     setToggleButton( TRUE );
@@ -345,4 +345,17 @@ void QCheckBox::updateMask()
 
     setMask(bm);
 }
+
+/*!\reimp*/
+bool QCheckBox::hitButton( const QPoint &pos ) const
+{
+    QRect r = QStyle::visualRect( style().subRect( QStyle::SR_CheckBoxFocusRect, this ), this );
+    if ( qApp->reverseLayout() ) {
+	r.setRight( width() );
+    } else {
+	r.setLeft( 0 );
+    }
+    return r.contains( pos );
+}
+
 #endif
