@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id:  qt/qmenubar.cpp   3.0.5   edited Jun 11 14:06 $
+** $Id: qt/qmenubar.cpp   3.2.3   edited Aug 19 08:33 $
 **
 ** Implementation of QMenuBar class
 **
 ** Created : 941209
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2002 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the widgets module of the Qt GUI Toolkit.
 **
@@ -49,7 +49,6 @@
 #include "qguardedptr.h"
 #include "qlayout.h"
 #include "qcleanuphandler.h"
-#include <ctype.h>
 #include "../kernel/qinternal_p.h"
 #include "qstyle.h"
 #include "qtimer.h"
@@ -69,34 +68,38 @@ public:
 static bool inMenu = FALSE;
 #endif
 
+#if defined(Q_WS_X11)
+extern int qt_xfocusout_grab_counter; // defined in qapplication_x11.cpp
+#endif
+
 /*!
-  \class QMenuBar qmenubar.h
-  \brief The QMenuBar class provides a horizontal menu bar.
+    \class QMenuBar qmenubar.h
+    \brief The QMenuBar class provides a horizontal menu bar.
 
-  \ingroup application
-  \mainclass
+    \ingroup application
+    \mainclass
 
-  A menu bar consists of a list of pull-down menu items.
-  You add menu items with \link QMenuData::insertItem()
-  insertItem()\endlink. For example, asuming that \c menubar is a
-  pointer to a QMenuBar and \c filemenu is a pointer to a
-  QPopupMenu, the following statement inserts the menu into
-  the menu bar:
-  \code
-  menubar->insertItem( "&File", filemenu );
-  \endcode
-  The ampersand in the menu
-  item's text sets Alt+F as a shortcut for this menu. (You can use
-  "&&" to get a real ampersand in the menu bar.)
+    A menu bar consists of a list of pull-down menu items. You add
+    menu items with \link QMenuData::insertItem()
+    insertItem()\endlink. For example, asuming that \c menubar is a
+    pointer to a QMenuBar and \c filemenu is a pointer to a
+    QPopupMenu, the following statement inserts the menu into the menu
+    bar:
+    \code
+    menubar->insertItem( "&File", filemenu );
+    \endcode
+    The ampersand in the menu item's text sets Alt+F as a shortcut for
+    this menu. (You can use "\&\&" to get a real ampersand in the menu
+    bar.)
 
-  Items are either enabled or disabled. You toggle their state with
-  setItemEnabled().
+    Items are either enabled or disabled. You toggle their state with
+    setItemEnabled().
 
-  There is no need to lay out a menu bar. It automatically sets its
-  own geometry to the top of the parent widget and changes it
-  appropriately whenever the parent is resized.
+    There is no need to lay out a menu bar. It automatically sets its
+    own geometry to the top of the parent widget and changes it
+    appropriately whenever the parent is resized.
 
-  \important insertItem removeItem clear insertSeparator setItemEnabled isItemEnabled
+    \important insertItem removeItem clear insertSeparator setItemEnabled isItemEnabled setItemVisible isItemVisible
 
     Example of creating a menu bar with menu items (from \l menu/menu.cpp):
     \quotefile menu/menu.cpp
@@ -126,83 +129,94 @@ static bool inMenu = FALSE;
     also add custom menu items that are derived from
     \l{QCustomMenuItem}.
 
-    Menu items may be removed with removeItem() and enabled or disabled
-    with \link QMenuData::setItemEnabled() setItemEnabled()\endlink.
+    Menu items may be removed with removeItem() and enabled or
+    disabled with \link QMenuData::setItemEnabled()
+    setItemEnabled()\endlink.
 
-  <img src=qmenubar-m.png> <img src=qmenubar-w.png>
+    <img src=qmenubar-m.png> <img src=qmenubar-w.png>
 
+    \section1 QMenuBar on Qt/Mac
 
-    QMenuBar on Qt/Mac is a wrapper for using the system-wide
-    menubar. However, if you have multiple menubars in one dialog the
-    outermost menubar (normally inside a widget with \l
-    WType_TopLevel) will be used for the global menubar.
+    QMenuBar on Qt/Mac is a wrapper for using the system-wide menubar.
+    If you have multiple menubars in one dialog the outermost menubar
+    (normally inside a widget with widget flag \c WType_TopLevel) will
+    be used for the system-wide menubar.
 
-    Qt/Mac also provides a menubar merging feature, with this your
-    QMenubar will be brought closer to conforming to accepted Mac OS X
-    menubar layout. The merging functionality is based on string
-    matching the title of a QPopupMenu entry, these strings are
-    translated (using QObject::tr()) in the "QMenuBar" context. If an
-    entry is moved the regularly connected slots will still fire as if
-    it was in the original place. The table below outlines the strings
-    looked for and where the entry is placed if matched:
+    Note that arbitrary Qt widgets \e cannot be inserted into a
+    QMenuBar on the Mac because Qt uses Mac's native menus which don't
+    support this functionality. This limitation does not apply to
+    stand-alone QPopupMenus.
+
+    Qt/Mac also provides a menubar merging feature to make QMenuBar
+    conform more closely to accepted Mac OS X menubar layout. The
+    merging functionality is based on string matching the title of a
+    QPopupMenu entry. These strings are translated (using
+    QObject::tr()) in the "QMenuBar" context. If an entry is moved its
+    slots will still fire as if it was in the original place. The
+    table below outlines the strings looked for and where the entry is
+    placed if matched:
 
     \table
     \header \i String matches \i Placement \i Notes
-    \row \i about.* \i Application Menu | About <application name>
-         \i If this entry is not found no About item will appear in the Application Menu
-    \row \i config, options, setup, settings, preferences \i Application Menu | Settings
-         \i If this entry is not found the Settings item will be disabled
-    \row \i quit, exit \i Application Menu | Quit <application name>
-         \i If this entry is not found a default Quit item will be created to call
-	    QApplication::quit()
+    \row \i about.*
+	 \i Application Menu | About <application name>
+	 \i If this entry is not found no About item will appear in
+	    the Application Menu
+    \row \i config, options, setup, settings or preferences
+	 \i Application Menu | Preferences
+	 \i If this entry is not found the Settings item will be disabled
+    \row \i quit or exit
+	 \i Application Menu | Quit <application name>
+	 \i If this entry is not found a default Quit item will be
+	    created to call QApplication::quit()
     \endtable
 
-  \sa QPopupMenu QAccel QAction
-      \link http://developer.apple.com/techpubs/macosx/Carbon/HumanInterfaceToolbox/Aqua/aqua.html
-            Aqua Style Guidelines \endlink
-      \link guibooks.html#fowler GUI Design Handbook: Menu Bar \endlink
+    \link menu-example.html menu/menu.cpp\endlink is an example of
+    QMenuBar and QPopupMenu use.
+
+    \sa QPopupMenu QAccel QAction \link http://developer.apple.com/techpubs/macosx/Carbon/HumanInterfaceToolbox/Aqua/aqua.html Aqua Style Guidelines \endlink \link guibooks.html#fowler GUI Design Handbook: Menu Bar \endlink
 */
 
 
-/*! \enum QMenuBar::Separator
-
-  This enum type is used to decide whether QMenuBar should draw a
-  separator line at its bottom.	 The possible values are:
-
-  \value Never	In many applications there is already a separator,
-  and having two looks wrong.
-
-  \value InWindowsStyle	 In some other applications a separator
-  looks good in Windows style, but nowhere else.
-
- */
-
 /*!
-  \fn void QMenuBar::activated( int id )
+    \enum QMenuBar::Separator
 
-  This signal is emitted when a menu item is selected; \a id is the id
-  of the selected item.
+    This enum type is used to decide whether QMenuBar should draw a
+    separator line at its bottom.
 
-  Normally you will connect each menu item to a single slot using
-  QMenuData::insertItem(), but sometimes you will want to connect
-  several items to a single slot (most often if the user selects from
-  an array).  This signal is useful in such cases.
+    \value Never In many applications there is already a separator,
+    and having two looks wrong.
 
-  \sa highlighted(), QMenuData::insertItem()
+    \value InWindowsStyle In some other applications a separator looks
+    good in Windows style, but nowhere else.
 */
 
 /*!
-  \fn void QMenuBar::highlighted( int id )
+    \fn void QMenuBar::activated( int id )
 
-  This signal is emitted when a menu item is highlighted; \a id is the
-  id of the highlighted item.
+    This signal is emitted when a menu item is selected; \a id is the
+    id of the selected item.
 
-  Normally, you will connect each menu item to a single slot using
-  QMenuData::insertItem(), but sometimes you will want to connect
-  several items to a single slot (most often if the user selects from
-  an array).  This signal is useful in such cases.
+    Normally you will connect each menu item to a single slot using
+    QMenuData::insertItem(), but sometimes you will want to connect
+    several items to a single slot (most often if the user selects
+    from an array). This signal is useful in such cases.
 
-  \sa activated(), QMenuData::insertItem()
+    \sa highlighted(), QMenuData::insertItem()
+*/
+
+/*!
+    \fn void QMenuBar::highlighted( int id )
+
+    This signal is emitted when a menu item is highlighted; \a id is
+    the id of the highlighted item.
+
+    Normally, you will connect each menu item to a single slot using
+    QMenuData::insertItem(), but sometimes you will want to connect
+    several items to a single slot (most often if the user selects
+    from an array). This signal is useful in such cases.
+
+    \sa activated(), QMenuData::insertItem()
 */
 
 
@@ -239,14 +253,14 @@ static const int motifItemVMargin	= 4;	// menu item ver text margin
 
 
 /*!
-  Constructs a menu bar with a \a parent and a \a name.
+    Constructs a menu bar called \a name with parent \a parent.
 */
 QMenuBar::QMenuBar( QWidget *parent, const char *name )
-    : QFrame( parent, name, WResizeNoErase | WRepaintNoErase )
+    : QFrame( parent, name, WNoAutoErase )
 {
 #if defined( Q_WS_MAC ) && !defined(QMAC_QMENUBAR_NO_NATIVE)
     mac_eaten_menubar = FALSE;
-    mac_d = NULL;
+    mac_d = 0;
     macCreateNativeMenubar();
 #endif
     isMenuBar = TRUE;
@@ -272,10 +286,10 @@ QMenuBar::QMenuBar( QWidget *parent, const char *name )
 	if ( tlw != parent )
 	    tlw->installEventFilter( this );
     }
+    installEventFilter( this );
 
     setBackgroundMode( PaletteButton );
     setFrameStyle( QFrame::MenuBarPanel | QFrame::Raised );
-    setLineWidth( style().pixelMetric(QStyle::PM_MenuBarFrameWidth, this) );
 
     QFontMetrics fm = fontMetrics();
     int h = 2*motifBarVMargin + fm.height() + motifItemVMargin + 2*frameWidth() + 2*motifItemFrame;
@@ -292,15 +306,13 @@ QMenuBar::QMenuBar( QWidget *parent, const char *name )
 void QMenuBar::styleChange( QStyle& old )
 {
     setMouseTracking( style().styleHint(QStyle::SH_GUIStyle) == WindowsStyle );
-    setFrameStyle( QFrame::MenuBarPanel | QFrame::Raised );
-    setLineWidth( style().pixelMetric( QStyle::PM_MenuBarFrameWidth, this ) );
     QFrame::styleChange( old );
 }
 
 
 
 /*!
-  Destroys the menu bar.
+    Destroys the menu bar.
 */
 
 QMenuBar::~QMenuBar()
@@ -316,8 +328,10 @@ QMenuBar::~QMenuBar()
 }
 
 /*!
-  \internal
-  Needs documentation.
+    \internal
+
+    Repaints the menu item with id \a id; does nothing if there is no
+    such menu item.
 */
 void QMenuBar::updateItem( int id )
 {
@@ -326,14 +340,16 @@ void QMenuBar::updateItem( int id )
 	repaint( irects[i], FALSE );
 }
 
+#if defined(Q_WS_MAC) && !defined(QMAC_QMENUBAR_NO_NATIVE)
 static bool fromFrameChange = FALSE;
+#endif
 
 /*!
-  Recomputes the menu bar's display data according to the new
-  contents.
+    Recomputes the menu bar's display data according to the new
+    contents.
 
-  You should never need to call this; it is called automatically by
-  QMenuData whenever it needs to be called.
+    You should never need to call this; it is called automatically by
+    QMenuData whenever it needs to be called.
 */
 
 void QMenuBar::menuContentsChanged()
@@ -345,7 +361,7 @@ void QMenuBar::menuContentsChanged()
         return;
     pendingDelayedContentsChanges = 1;
     if( !pendingDelayedStateChanges ) // if the timer hasn't been started yet
-        QTimer::singleShot( 0, this, SLOT(performDelayedChanges()));
+	QTimer::singleShot( 0, this, SLOT(performDelayedChanges()));
 }
 
 void QMenuBar::performDelayedContentsChanged()
@@ -362,9 +378,10 @@ void QMenuBar::performDelayedContentsChanged()
     if ( isVisible() ) {
 	update();
 #ifndef QT_NO_MAINWINDOW
-	if ( parent() && parent()->inherits( "QMainWindow" ) ) {
-	    ( (QMainWindow*)parent() )->triggerLayout();
-	    ( (QMainWindow*)parent() )->update();
+	QMainWindow *mw = ::qt_cast<QMainWindow*>(parent());
+	if ( mw ) {
+	    mw->triggerLayout();
+	    mw->update();
 	}
 #endif
 #ifndef QT_NO_LAYOUT
@@ -373,30 +390,30 @@ void QMenuBar::performDelayedContentsChanged()
 #endif
     }
 #if defined(Q_WS_MAC) && !defined(QMAC_QMENUBAR_NO_NATIVE)
-    if(mac_eaten_menubar)
+    if(mac_eaten_menubar) {
 	macDirtyNativeMenubar();
 
-    bool all_hidden = TRUE;
-    if(irects) {
-	for(int i = 0; all_hidden && i < (int)mitems->count(); i++)
-	    all_hidden = irects[i].isEmpty();
-    }
-    if( all_hidden ) {
-	if( !isHidden())
-	    hide();
-    } else {
-	if( isHidden() && !fromFrameChange )
-	    show();
+	bool all_hidden = TRUE;
+	if(irects) {
+	    for(int i = 0; all_hidden && i < (int)mitems->count(); i++)
+		all_hidden = irects[i].isEmpty();
+	}
+	if( all_hidden ) {
+	    if( !isHidden())
+		hide();
+	} else {
+	    if( !isShown() && !fromFrameChange )
+		show();
+	}
     }
 #endif
 }
 
 /*!
-  Recomputes the menu bar's display data according to the new
-  state.
+    Recomputes the menu bar's display data according to the new state.
 
-  You should never need to call this; it is called automatically by
-  QMenuData whenever it needs to be called.
+    You should never need to call this; it is called automatically by
+    QMenuData whenever it needs to be called.
 */
 
 void QMenuBar::menuStateChanged()
@@ -405,7 +422,7 @@ void QMenuBar::menuStateChanged()
         return;
     pendingDelayedStateChanges = 1;
     if( !pendingDelayedContentsChanges ) // if the timer hasn't been started yet
-        QTimer::singleShot( 0, this, SLOT(performDelayedChanges()));
+	QTimer::singleShot( 0, this, SLOT(performDelayedChanges()));
 }
 
 void QMenuBar::performDelayedStateChanged()
@@ -413,7 +430,8 @@ void QMenuBar::performDelayedStateChanged()
     pendingDelayedStateChanges = 0;
     // here the part that can be delayed
 #ifndef QT_NO_ACCEL
-    setupAccelerators(); // ### when we have a good solution for the accel vs. focus widget problem, remove that. That is only a workaround
+    setupAccelerators(); // ### when we have a good solution for the accel vs. focus
+			 // widget problem, remove that. That is only a workaround
                          // if you remove this, see performDelayedContentsChanged()
 #endif
     update();
@@ -449,39 +467,55 @@ void QMenuBar::menuDelPopup( QPopupMenu *popup )
 
 void QMenuBar::frameChanged()
 {
+#if defined(Q_WS_MAC) && !defined(QMAC_QMENUBAR_NO_NATIVE)
     fromFrameChange = TRUE;
+#endif
     menuContentsChanged();
+#if defined(Q_WS_MAC) && !defined(QMAC_QMENUBAR_NO_NATIVE)
     fromFrameChange = FALSE;
+#endif
 }
 
+void QMenuBar::languageChange()
+{
+    menuContentsChanged(); 
+}
 
 /*!
-  This function is used to adjust the menu bar's geometry to the
-  parent widget's geometry.  Note that this is \e not part of the public
-  interface - the function is \c public only because
-  QObject::eventFilter() is.
+    \internal
 
-  \internal
-  Resizes the menu bar to fit in the parent widget when the parent receives
-  a resize event.
+    This function is used to adjust the menu bar's geometry to the
+    parent widget's geometry. Note that this is \e not part of the
+    public interface - the function is \c public only because
+    QObject::eventFilter() is.
+
+    Resizes the menu bar to fit in the parent widget when the parent
+    receives a resize event.
 */
 
 bool QMenuBar::eventFilter( QObject *object, QEvent *event )
 {
-    if ( object == parent() && object && !object->inherits( "QToolBar" ) &&
-	 event->type() == QEvent::Resize ) {
+    if ( object == parent() && object
+#ifndef QT_NO_TOOLBAR
+	 && !::qt_cast<QToolBar*>(object)
+#endif
+	 && event->type() == QEvent::Resize ) {
 	QResizeEvent *e = (QResizeEvent *)event;
 	int w = e->size().width();
 	setGeometry( 0, y(), w, heightForWidth(w) );
 	return FALSE;
     }
 
-    if ( ! isVisible() || ! object->isWidgetType() )
+    if ( !isVisible() || !object->isWidgetType() )
 	return FALSE;
 
-    if ( event->type() == QEvent::MouseButtonPress ||
-	event->type() == QEvent::MouseButtonRelease ) {
-	waitforalt = FALSE;
+    if ( object == this && event->type() == QEvent::LanguageChange ) {
+	badSize = TRUE;
+	calculateRects();
+	return FALSE;
+    } else if ( event->type() == QEvent::MouseButtonPress ||
+		event->type() == QEvent::MouseButtonRelease ) {
+	waitforalt = 0;
 	return FALSE;
     } else if ( waitforalt && event->type() == QEvent::FocusOut ) {
 	// some window systems/managers use alt/meta as accelerator keys
@@ -491,37 +525,47 @@ bool QMenuBar::eventFilter( QObject *object, QEvent *event )
 	// need to touch the event filters
 	waitforalt = 0;
 	return FALSE;
-    } else if ( ! ( event->type() == QEvent::Accel ||
-	event->type() == QEvent::KeyPress ||
-	event->type() == QEvent::KeyRelease ) ||
-	! style().styleHint(QStyle::SH_MenuBar_AltKeyNavigation, this) ) {
+    } else if ( !( event->type() == QEvent::Accel ||
+		event->type() == QEvent::AccelOverride ||
+		event->type() == QEvent::KeyPress ||
+		event->type() == QEvent::KeyRelease ) ||
+		!style().styleHint(QStyle::SH_MenuBar_AltKeyNavigation, this) ) {
 	return FALSE;
     }
 
+    QKeyEvent * ke = (QKeyEvent *) event;
 #ifndef QT_NO_ACCEL
     // look for Alt press and Alt-anything press
     if ( event->type() == QEvent::Accel ) {
 	QWidget * f = ((QWidget *)object)->focusWidget();
-	QKeyEvent * ke = (QKeyEvent *) event;
 	// ### this thinks alt and meta are the same
 	if ( ke->key() == Key_Alt || ke->key() == Key_Meta ) {
+	    // A new Alt press and we wait for release, eat
+	    // this key and don't wait for Alt on this widget
 	    if ( waitforalt ) {
 		waitforalt = 0;
 		if ( object->parent() )
 		    object->removeEventFilter( this );
 		ke->accept();
 		return TRUE;
+	    // Menu has focus, send focus back
 	    } else if ( hasFocus() ) {
 		setAltMode( FALSE );
 		ke->accept();
 		return TRUE;
-	    } else {
+	    // Start waiting for Alt release on focus widget
+	    } else if ( ke->stateAfter() == AltButton ) {
 		waitforalt = 1;
+#if defined(Q_WS_X11)
+		QMenuData::d->aInt = qt_xfocusout_grab_counter;
+#endif
 		if ( f && f != object )
 		    f->installEventFilter( this );
 	    }
+	// Other modifiers kills focus on menubar
 	} else if ( ke->key() == Key_Control || ke->key() == Key_Shift) {
 	    setAltMode( FALSE );
+	// Got other key, no need to wait for Alt release
 	} else {
 	    waitforalt = 0;
 	}
@@ -536,10 +580,12 @@ bool QMenuBar::eventFilter( QObject *object, QEvent *event )
     // look for Alt release
     if ( ((QWidget*)object)->focusWidget() == object ||
 	 (object->parent() == 0 && ((QWidget*)object)->focusWidget() == 0) ) {
-	if ( waitforalt &&
-	     event->type() == QEvent::KeyRelease &&
-	     (((QKeyEvent *)event)->key() == Key_Alt ||
-	      ((QKeyEvent *)event)->key() == Key_Meta) ) {
+	if ( waitforalt && event->type() == QEvent::KeyRelease &&
+	    ( ke->key() == Key_Alt || ke->key() == Key_Meta )
+#if defined(Q_WS_X11)
+		&& QMenuData::d->aInt == qt_xfocusout_grab_counter
+#endif
+	    ) {
 	    setAltMode( TRUE );
 	    if ( object->parent() )
 		object->removeEventFilter( this );
@@ -552,7 +598,8 @@ bool QMenuBar::eventFilter( QObject *object, QEvent *event )
 		tlw->installEventFilter( this );
 	    }
 	    return TRUE;
-	} else if ( (event->type() == QEvent::KeyPress ) &&
+	// Cancel if next keypress is NOT Alt/Meta,
+	} else if ( !hasFocus() && (event->type() == QEvent::AccelOverride ) &&
 		    !(((QKeyEvent *)event)->key() == Key_Alt ||
 		      ((QKeyEvent *)event)->key() == Key_Meta) ) {
 	    if ( object->parent() )
@@ -593,8 +640,13 @@ void QMenuBar::subHighlighted( int id )
 #ifndef QT_NO_ACCEL
 void QMenuBar::accelActivated( int id )
 {
+#if defined(Q_WS_MAC) && !defined(QMAC_QMENUBAR_NO_NATIVE)
+    if(mac_eaten_menubar)
+	return;
+#endif
     if ( !isEnabled() )				// the menu bar is disabled
 	return;
+    setAltMode( TRUE );
     setActiveItem( indexOf( id ) );
 }
 #endif
@@ -644,10 +696,7 @@ void QMenuBar::goodbye( bool cancelled )
 {
     mouseBtDn = FALSE;
     popupvisible = 0;
-    if ( cancelled && style().styleHint(QStyle::SH_MenuBar_AltKeyNavigation, this) )
-	setAltMode( TRUE );
-    else
-	setAltMode( FALSE );
+    setAltMode( cancelled && style().styleHint(QStyle::SH_MenuBar_AltKeyNavigation, this) );
 }
 
 
@@ -732,8 +781,9 @@ void QMenuBar::hidePopups()
 
 
 /*!
-  Reimplements QWidget::show() in order to set up the correct keyboard
-  accelerators and to raise itself to the top of the widget stack.
+    Reimplements QWidget::show() in order to set up the correct
+    keyboard accelerators and to raise itself to the top of the widget
+    stack.
 */
 
 void QMenuBar::show()
@@ -746,33 +796,39 @@ void QMenuBar::show()
 	resize( parentWidget()->width(), height() );
 
     QApplication::sendPostedEvents( this, QEvent::Resize );
+    performDelayedChanges();
     calculateRects();
 
 #if defined(Q_WS_MAC) && !defined(QMAC_QMENUBAR_NO_NATIVE)
-    //If all elements are invisible no reason for me to be visible either
-    bool all_hidden = TRUE;
-    if(irects) {
-	for(int i = 0; all_hidden && i < (int)mitems->count(); i++)
-	    all_hidden = irects[i].isEmpty();
-    }
-    if(all_hidden)
-	QWidget::hide();
-    else
+    if(mac_eaten_menubar) {
+	//If all elements are invisible no reason for me to be visible either
+	bool all_hidden = TRUE;
+	if(irects) {
+	    for(int i = 0; all_hidden && i < (int)mitems->count(); i++)
+		all_hidden = irects[i].isEmpty();
+	}
+	if(all_hidden)
+	    QWidget::hide();
+	else
+	    QWidget::show();
+    } else {
 	QWidget::show();
+    }
 #else
     QWidget::show();
 #endif
 
 #ifndef QT_NO_MAINWINDOW
-    if ( parent() && parent()->inherits( "QMainWindow" ) ) //### ugly workaround
-	( (QMainWindow*)parent() )->triggerLayout();
+    QMainWindow *mw = ::qt_cast<QMainWindow*>(parent());
+    if ( mw ) //### ugly workaround
+	mw->triggerLayout();
 #endif
     raise();
 }
 
 /*!
-  Reimplements QWidget::hide() in order to deselect any selected item, and
-  calls setUpLayout() for the main window.
+    Reimplements QWidget::hide() in order to deselect any selected
+    item, and calls setUpLayout() for the main window.
 */
 
 void QMenuBar::hide()
@@ -781,8 +837,9 @@ void QMenuBar::hide()
     setAltMode( FALSE );
     hidePopups();
 #ifndef QT_NO_MAINWINDOW
-    if ( parent() && parent()->inherits( "QMainWindow" ) ) //### ugly workaround
-		( (QMainWindow*)parent() )->triggerLayout();
+    QMainWindow *mw = ::qt_cast<QMainWindow*>(parent());
+    if ( mw ) //### ugly workaround
+	mw->triggerLayout();
 #endif
 }
 
@@ -805,12 +862,12 @@ void QMenuBar::fontChange( const QFont & f )
   Item geometry functions
  *****************************************************************************/
 
-/*!
-  This function serves two different purposes.	If the parameter is negative,
-  it updates the irects member for the current width and resizes.  Otherwise,
-  it does the same calculations for the GIVEN width and returns the height
-  to which it WOULD have resized.  A bit tricky, but both operations require
-  almost identical steps.
+/*
+    This function serves two different purposes. If the parameter is
+    negative, it updates the irects member for the current width and
+    resizes. Otherwise, it does the same calculations for the GIVEN
+    width and returns the height to which it WOULD have resized. A bit
+    tricky, but both operations require almost identical steps.
 */
 int QMenuBar::calculateRects( int max_width )
 {
@@ -821,15 +878,14 @@ int QMenuBar::calculateRects( int max_width )
 	rightSide = 0;
 	if ( !badSize )				// size was not changed
 	    return 0;
-	if ( irects )				// Avoid purify complaint.
-	    delete [] irects;
-	if ( mitems->isEmpty() ) {
+	delete [] irects;
+	int i = mitems->count();
+	if ( i == 0 ) {
 	    irects = 0;
-	    return 0;
+	} else {
+	    irects = new QRect[ i ];
+	    Q_CHECK_PTR( irects );
 	}
-	int i = mitems->count();		// workaround for gcc 2.95.2
-	irects = new QRect[ i ];		// create rectangle array
-	Q_CHECK_PTR( irects );
 	max_width = width();
     }
     QFontMetrics fm = fontMetrics();
@@ -857,13 +913,13 @@ int QMenuBar::calculateRects( int max_width )
 	QMenuItem *mi = mitems->at(i);
 
 	int w=0, h=0;
+	if ( !mi->isVisible()
 #if defined(Q_WS_MAC) && !defined(QMAC_QMENUBAR_NO_NATIVE)
-	if(mac_eaten_menubar && !mi->custom() && !mi->widget()) {
-	    w = 0;
-	    h = 0;
-	} else
+				  ||  (mac_eaten_menubar && !mi->custom() && !mi->widget() )
 #endif
-	if ( mi->widget() ) {
+	     ) {
+	    ; // empty rectangle
+	} else if ( mi->widget() ) {
 	    if ( mi->widget()->parentWidget() != this ) {
 		mi->widget()->reparent( this, QPoint(0,0) );
 	    }
@@ -963,11 +1019,11 @@ int QMenuBar::calculateRects( int max_width )
 }
 
 /*!
-  Returns the height that the menu would resize itself to if its parent
-  (and hence itself) resized to the given \a max_width.  This can be
-  useful for simple layout tasks in which the height of the menu bar
-  is needed after items have been inserted.  See
-  \l showimg/showimg.cpp for an example of the usage.
+    Returns the height that the menu would resize itself to if its
+    parent (and hence itself) resized to the given \a max_width. This
+    can be useful for simple layout tasks in which the height of the
+    menu bar is needed after items have been inserted. See \l
+    showimg/showimg.cpp for an example of the usage.
 */
 int QMenuBar::heightForWidth(int max_width) const
 {
@@ -1031,17 +1087,18 @@ QMenuBar::Separator QMenuBar::separator() const
  *****************************************************************************/
 
 /*!
-  Called from QFrame::paintEvent().
-  Draws the menu bar contents using painter \a p.
+    Called from QFrame::paintEvent(). Draws the menu bar contents
+    using painter \a p.
 */
 
 void QMenuBar::drawContents( QPainter *p )
 {
+    performDelayedChanges();
     QRegion reg( contentsRect() );
     QColorGroup g = colorGroup();
     bool e;
 
-    // ### this shouldn't happen.
+    // this shouldn't happen
     if ( !irects )
 	return;
 
@@ -1049,9 +1106,9 @@ void QMenuBar::drawContents( QPainter *p )
 	QMenuItem *mi = mitems->at( i );
 	if ( !mi->text().isNull() || mi->pixmap() ) {
 	    QRect r = irects[i];
-	    if(r.isEmpty())
+	    if(r.isEmpty() || !mi->isVisible())
 		continue;
-	    e = mi->isEnabled();
+	    e = mi->isEnabledAndVisible();
 	    if ( e )
 		g = isEnabled() ? ( isActiveWindow() ? palette().active() :
 				    palette().inactive() ) : palette().disabled();
@@ -1064,7 +1121,7 @@ void QMenuBar::drawContents( QPainter *p )
 	    buffer.painter()->setBrush( p->brush() );
 
 	    QStyle::SFlags flags = QStyle::Style_Default;
-	    if (isEnabled() && mi->isEnabled())
+	    if (isEnabled() && e)
 		flags |= QStyle::Style_Enabled;
 	    if ( i == actItem )
 		flags |= QStyle::Style_Active;
@@ -1077,11 +1134,15 @@ void QMenuBar::drawContents( QPainter *p )
 	}
     }
 
-    erase( reg );
+    p->save();
+    p->setClipRegion(reg);
+    style().drawControl(QStyle::CE_MenuBarEmptyArea, p, this, contentsRect(), g);
+    p->restore();
 
 #if defined(Q_WS_MAC) && !defined(QMAC_QMENUBAR_NO_NATIVE)
-    if ( !mac_eaten_menubar ) {
+    if ( !mac_eaten_menubar )
 #endif
+    {
 	Qt::GUIStyle gs = (Qt::GUIStyle) style().styleHint(QStyle::SH_GUIStyle);
 	if ( mseparator == InWindowsStyle && gs == WindowsStyle ) {
 	    p->setPen( g.light() );
@@ -1089,13 +1150,12 @@ void QMenuBar::drawContents( QPainter *p )
 	    p->setPen( g.dark() );
 	    p->drawLine( 0, height()-2, width()-1, height()-2 );
 	}
-#if defined(Q_WS_MAC) && !defined(QMAC_QMENUBAR_NO_NATIVE)
     }
-#endif
 }
 
 
-/*!\reimp
+/*!
+    \reimp
 */
 void QMenuBar::mousePressEvent( QMouseEvent *e )
 {
@@ -1105,11 +1165,22 @@ void QMenuBar::mousePressEvent( QMouseEvent *e )
     int item = itemAtPos( e->pos() );
     if ( item == actItem && popupvisible )
 	toggleclose = 1;
+    if ( item >= 0 ) {
+	QFocusEvent::Reason oldReason = QFocusEvent::reason();
+	QMenuItem *mi = findItem( idAt( item ) );
+	// we know that a popup will open, so set the reason to avoid
+	// itemviews to redraw their selections
+	if ( mi && mi->popup() )
+	    QFocusEvent::setReason( QFocusEvent::Popup );
+	setAltMode( TRUE );
+	QFocusEvent::setReason( oldReason );
+    }
     setActiveItem( item, TRUE, FALSE );
 }
 
 
-/*!\reimp
+/*!
+    \reimp
 */
 void QMenuBar::mouseReleaseEvent( QMouseEvent *e )
 {
@@ -1119,9 +1190,10 @@ void QMenuBar::mouseReleaseEvent( QMouseEvent *e )
 	return;
     mouseBtDn = FALSE;				// mouse button up
     int item = itemAtPos( e->pos() );
-    if ( item >= 0 && !mitems->at(item)->isEnabled() ||
-	 actItem >= 0 && !mitems->at(actItem)->isEnabled() ) {
+    if ( item >= 0 && !mitems->at(item)->isEnabledAndVisible() ||
+	 actItem >= 0 && !mitems->at(actItem)->isEnabledAndVisible() ) {
 	hidePopups();
+	setActiveItem( -1 );
 	return;
     }
     bool showMenu = TRUE;
@@ -1135,7 +1207,8 @@ void QMenuBar::mouseReleaseEvent( QMouseEvent *e )
 }
 
 
-/*!\reimp
+/*!
+    \reimp
 */
 void QMenuBar::mouseMoveEvent( QMouseEvent *e )
 {
@@ -1156,7 +1229,8 @@ void QMenuBar::mouseMoveEvent( QMouseEvent *e )
 }
 
 
-/*!\reimp
+/*!
+    \reimp
 */
 void QMenuBar::leaveEvent( QEvent * e )
 {
@@ -1169,7 +1243,8 @@ void QMenuBar::leaveEvent( QEvent * e )
 }
 
 
-/*!\reimp
+/*!
+    \reimp
 */
 void QMenuBar::keyPressEvent( QKeyEvent *e )
 {
@@ -1179,6 +1254,13 @@ void QMenuBar::keyPressEvent( QKeyEvent *e )
     QMenuItem  *mi = 0;
     int dx = 0;
 
+    if ( e->state() & Qt::ControlButton &&
+	 ( e->key() == Qt::Key_Tab || e->key() == Qt::Key_Backtab ) )
+    {
+	e->ignore();
+	return;
+    }
+
     switch ( e->key() ) {
      case Key_Left:
 	dx = QApplication::reverseLayout() ? 1 : -1;
@@ -1215,7 +1297,7 @@ void QMenuBar::keyPressEvent( QKeyEvent *e )
 	    mi = mitems->at( i );
 	    // ### fix windows-style traversal - currently broken due to
 	    // QMenuBar's reliance on QPopupMenu
-	    if ( /* (style() == WindowsStyle || */ mi->isEnabled() /* ) */
+	    if ( /* (style() == WindowsStyle || */ mi->isEnabledAndVisible() /* ) */
 		 && !mi->isSeparator() )
 		break;
 	}
@@ -1224,8 +1306,13 @@ void QMenuBar::keyPressEvent( QKeyEvent *e )
 	QChar c = e->text()[0].upper();
 
 	QMenuItemListIt it(*mitems);
+	QMenuItem* first = 0;
+	QMenuItem* currentSelected = 0;
+	QMenuItem* firstAfterCurrent = 0;
+
 	register QMenuItem *m;
 	int indx = 0;
+	int clashCount = 0;
 	while ( (m=it.current()) ) {
 	    ++it;
 	    QString s = m->text();
@@ -1234,21 +1321,38 @@ void QMenuBar::keyPressEvent( QKeyEvent *e )
 		if ( i >= 0 )
 		{
 		    if ( s[i+1].upper() == c ) {
-			mi = m;
-			break;
+			clashCount++;
+			if ( !first )
+			    first = m;
+			if ( indx == actItem )
+			    currentSelected = m;
+			else if ( !firstAfterCurrent && currentSelected )
+			    firstAfterCurrent = m;
 		    }
 		}
 	    }
 	    indx++;
 	}
-	if ( mi ) {
-	    setActiveItem( indx );
+	if ( 0 == clashCount ) {
+	    return;
+	} else if ( 1 == clashCount ) {
+ 	    indx = indexOf( first->id() );
+	} else {
+	    // If there's clashes and no one is selected, use first one
+	    // or if there is no clashes _after_ current, use first one
+	    if ( !currentSelected || (currentSelected && !firstAfterCurrent))
+		indx = indexOf( first->id() );
+	    else
+		indx = indexOf( firstAfterCurrent->id() );
 	}
+
+	setActiveItem( indx );
     }
 }
 
 
-/*!\reimp
+/*!
+    \reimp
 */
 void QMenuBar::resizeEvent( QResizeEvent *e )
 {
@@ -1259,11 +1363,11 @@ void QMenuBar::resizeEvent( QResizeEvent *e )
     calculateRects();
 }
 
-/*!  Sets actItem to \a i and calls repaint for the
-  changed things.
+/*
+    Sets actItem to \a i and calls repaint for the changed things.
 
-  Takes care to optimize the repainting.  Assumes that
-  calculateRects() has been called as appropriate.
+    Takes care to optimize the repainting. Assumes that
+    calculateRects() has been called as appropriate.
 */
 
 void QMenuBar::setActiveItem( int i, bool show, bool activate_first_item )
@@ -1274,7 +1378,7 @@ void QMenuBar::setActiveItem( int i, bool show, bool activate_first_item )
     QMenuItem* mi = 0;
     if ( i >= 0 )
 	mi = mitems->at( i );
-    if ( mi && !mi->isEnabled() )
+    if ( mi && !mi->isEnabledAndVisible() )
 	return;
 
     popupvisible = i >= 0 ? (show) : 0;
@@ -1304,11 +1408,16 @@ void QMenuBar::setActiveItem( int i, bool show, bool activate_first_item )
 
     hidePopups();
 
-    if ( !popupvisible && actItem >= 0 ) {
+    if ( !popupvisible && actItem >= 0 && irects ) {
 	QRect mfrect = irects[actItem];
 	setMicroFocusHint( mfrect.x(), mfrect.y(), mfrect.width(), mfrect.height(), FALSE );
     }
 
+#if defined(QT_ACCESSIBILITY_SUPPORT)
+    if ( mi )
+	QAccessible::updateAccessibility( this, indexOf( mi->id() )+1, QAccessible::Focus );
+#endif
+
     if ( actItem < 0 || !popupvisible || !mi  )
 	return;
 
@@ -1347,10 +1456,14 @@ void QMenuBar::setAltMode( bool enable )
 	setFocus();
 	updateItem( idAt( actItem ) );
     } else {
-	if ( QMenuData::d->aWidget )
-	    QMenuData::d->aWidget->setFocus();
-	else
-	    clearFocus();
+	// set the focus back to the previous widget if
+	// we still have the focus.
+	if ( qApp->focusWidget() == this ) {
+	    if ( QMenuData::d->aWidget )
+		QMenuData::d->aWidget->setFocus();
+	    else
+		clearFocus();
+	}
 	int actId = idAt( actItem );
 	actItem = -1;
 	updateItem( actId );
@@ -1358,7 +1471,9 @@ void QMenuBar::setAltMode( bool enable )
     }
 }
 
-/*!  Sets up keyboard accelerators for the menu bar. */
+/*!
+    Sets up keyboard accelerators for the menu bar.
+*/
 #ifndef QT_NO_ACCEL
 
 void QMenuBar::setupAccelerators()
@@ -1370,7 +1485,7 @@ void QMenuBar::setupAccelerators()
     register QMenuItem *mi;
     while ( (mi=it.current()) ) {
 	++it;
-	if ( !mi->isEnabled() ) // ### when we have a good solution for the accel vs. focus widget problem, remove that. That is only a workaround
+	if ( !mi->isEnabledAndVisible() ) // ### when we have a good solution for the accel vs. focus widget problem, remove that. That is only a workaround
 	    continue;
 	QString s = mi->text();
 	if ( !s.isEmpty() ) {
@@ -1382,11 +1497,12 @@ void QMenuBar::setupAccelerators()
 		    autoaccel->setIgnoreWhatsThis( TRUE );
 		    connect( autoaccel, SIGNAL(activated(int)),
 			     SLOT(accelActivated(int)) );
+		    connect( autoaccel, SIGNAL(activatedAmbiguously(int)),
+			     SLOT(accelActivated(int)) );
 		    connect( autoaccel, SIGNAL(destroyed()),
 			     SLOT(accelDestroyed()) );
 		}
 		autoaccel->insertItem( i, mi->id() );
-		autoaccel->insertItem( Key_Shift + i, mi->id() );
 	    }
 	}
 	if ( mi->popup() ) {
@@ -1399,7 +1515,8 @@ void QMenuBar::setupAccelerators()
 }
 #endif
 
-/*!\reimp
+/*!
+    \reimp
  */
 bool QMenuBar::customWhatsThis() const
 {
@@ -1408,7 +1525,8 @@ bool QMenuBar::customWhatsThis() const
 
 
 
-/*!\reimp
+/*!
+    \reimp
  */
 void QMenuBar::focusInEvent( QFocusEvent * )
 {
@@ -1416,7 +1534,7 @@ void QMenuBar::focusInEvent( QFocusEvent * )
 	int i = -1;
 	while ( actItem < 0 && ++i < (int) mitems->count() ) {
 	    QMenuItem* mi = mitems->at( i );
-	    if ( mi && mi->isEnabled() && !mi->isSeparator() )
+	    if ( mi && mi->isEnabledAndVisible() && !mi->isSeparator() )
 		setActiveItem( i, FALSE );
 	}
     } else if ( !popupvisible ) {
@@ -1424,7 +1542,8 @@ void QMenuBar::focusInEvent( QFocusEvent * )
     }
 }
 
-/*!\reimp
+/*!
+    \reimp
  */
 void QMenuBar::focusOutEvent( QFocusEvent * )
 {
@@ -1434,7 +1553,7 @@ void QMenuBar::focusOutEvent( QFocusEvent * )
 }
 
 /*!
-  \reimp
+    \reimp
 */
 
 QSize QMenuBar::sizeHint() const
@@ -1448,22 +1567,26 @@ QSize QMenuBar::sizeHint() const
 	    s.setWidth( s.width() + irects[ i ].width() + 2 );
     }
     s.setHeight( h );
-    return s.expandedTo( QApplication::globalStrut() );
+    return (style().sizeFromContents(QStyle::CT_MenuBar, this, s.
+				     expandedTo(QApplication::globalStrut())));
 }
 
 /*!
-  \reimp
+    \reimp
 */
 
 QSize QMenuBar::minimumSize() const
 {
-    if ( parent() && parent()->inherits( "QToolBar" ) )
+#ifndef QT_NO_TOOLBAR
+    QToolBar *tb = ::qt_cast<QToolBar*>(parent());
+    if ( tb )
 	return sizeHint();
+#endif
     return QFrame::minimumSize();
 }
 
 /*!
-  \reimp
+    \reimp
 */
 
 QSize QMenuBar::minimumSizeHint() const
@@ -1472,16 +1595,16 @@ QSize QMenuBar::minimumSizeHint() const
 }
 
 /*!
-  \property QMenuBar::defaultUp
-  \brief the popup orientation
+    \property QMenuBar::defaultUp
+    \brief the popup orientation
 
-  The default popup orientation. By default, menus pop "down" the
-  screen.  By setting the property to TRUE, the menu will pop "up".  You
-  might call this for menus that are \e below the document to which
-  they refer.
+    The default popup orientation. By default, menus pop "down" the
+    screen. By setting the property to TRUE, the menu will pop "up".
+    You might call this for menus that are \e below the document to
+    which they refer.
 
-  If the menu would not fit on the screen, the other direction is used
-  rather than the default.
+    If the menu would not fit on the screen, the other direction is
+    used automatically.
 */
 void QMenuBar::setDefaultUp( bool on )
 {
@@ -1494,7 +1617,8 @@ bool QMenuBar::isDefaultUp() const
 }
 
 
-/*!\reimp
+/*!
+    \reimp
  */
 void QMenuBar::activateItemAt( int index )
 {
