@@ -1,5 +1,5 @@
 /**********************************************************************
-** $Id:  qt/qmultilineedit.cpp   3.0.5   edited Jun 10 09:17 $
+** $Id: qt/qmultilineedit.cpp   3.2.3   edited May 13 09:08 $
 **
 ** Implementation of QMultiLineEdit widget class
 **
@@ -50,8 +50,6 @@
 #include "qdict.h"
 #include "../kernel/qrichtext_p.h"
 
-#include <ctype.h>
-
 
 /*!
   \class QMultiLineEdit qmultilineedit.h
@@ -348,6 +346,12 @@ void QMultiLineEdit::end( bool mark )
     moveCursor( MoveLineEnd, mark );
 }
 
+
+/*!
+  \fn void QMultiLineEdit::setCursorPosition( int line, int col )
+  \reimp
+*/
+
 /*!  Sets the cursor position to character number \a col in paragraph
   number \a line.  The parameters are adjusted to lie within the legal
   range.
@@ -370,7 +374,7 @@ void QMultiLineEdit::setCursorPosition( int line, int col, bool mark )
 
 QPoint QMultiLineEdit::cursorPoint() const
 {
-    return QPoint( textCursor()->x(), textCursor()->y() );
+    return QPoint( textCursor()->x(), textCursor()->y() + textCursor()->paragraph()->rect().y() );
 }
 
 /*!  \property QMultiLineEdit::alignment
@@ -439,6 +443,11 @@ void QMultiLineEdit::cursorWordBackward( bool mark )
     moveCursor( MoveWordBackward, mark );
 }
 
+/*!
+  \fn QMultiLineEdit::insertAt( const QString &s, int line, int col )
+  \reimp
+*/
+
 /*!  Inserts string \a s at paragraph number \a line, after character
   number \a col in the paragraph.  If \a s contains newline
   characters, new lines are inserted.
