@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id:  qt/qscrollbar.h   3.0.5   edited Apr 2 21:19 $
+** $Id: qt/qscrollbar.h   3.2.3   edited May 13 09:08 $
 **
 ** Definition of QScrollBar class
 **
@@ -47,7 +47,6 @@ class QTimer;
 
 #ifndef QT_NO_SCROLLBAR
 
-
 class Q_EXPORT QScrollBar : public QWidget, public QRangeControl
 {
     Q_OBJECT
@@ -61,11 +60,11 @@ class Q_EXPORT QScrollBar : public QWidget, public QRangeControl
     Q_PROPERTY( Orientation orientation READ orientation WRITE setOrientation )
 
 public:
-    QScrollBar( QWidget *parent, const char* name=0 );
-    QScrollBar( Orientation, QWidget *parent, const char* name=0 );
-    QScrollBar( int minValue, int maxValue, int LineStep, int PageStep,
-		int value, Orientation,
-		QWidget *parent, const char* name=0 );
+    QScrollBar( QWidget *parent, const char* name = 0 );
+    QScrollBar( Orientation, QWidget *parent, const char* name = 0 );
+    QScrollBar( int minValue, int maxValue, int lineStep, int pageStep,
+		int value, Orientation, QWidget *parent, const char* name = 0 );
+    ~QScrollBar();
 
     virtual void setOrientation( Orientation );
     Orientation orientation() const;
@@ -74,7 +73,9 @@ public:
     bool	draggingSlider() const;
 
     virtual void setPalette( const QPalette & );
-    QSize	sizeHint() const;
+    virtual QSize sizeHint() const;
+    virtual void setSizePolicy( QSizePolicy sp );
+    void setSizePolicy( QSizePolicy::SizeType hor, QSizePolicy::SizeType ver, bool hfw = FALSE );
 
     int	 minValue() const;
     int	 maxValue() const;
@@ -134,13 +135,13 @@ private:
     void stopAutoRepeat();
 
     int rangeValueToSliderPos( int val ) const;
-    int sliderPosToRangeValue( int  val ) const;
+    int sliderPosToRangeValue( int val ) const;
 
     void action( int control );
 
     void drawControls( uint controls, uint activeControl ) const;
     void drawControls( uint controls, uint activeControl,
-				QPainter *p ) const;
+		       QPainter *p ) const;
 
     uint pressedControl;
     bool track;
@@ -182,6 +183,11 @@ inline int QScrollBar::sliderStart() const
     return sliderPos;
 }
 
+inline void QScrollBar::setSizePolicy( QSizePolicy::SizeType hor, QSizePolicy::SizeType ver, bool hfw )
+{ 
+    QWidget::setSizePolicy( hor, ver, hfw ); 
+}
+
 
 #endif // QT_NO_SCROLLBAR
 
