@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id$
+** $Id: qt/qslider.cpp   3.2.3   edited May 13 09:08 $
 **
 ** Implementation of QSlider class
 **
@@ -47,65 +47,73 @@
 #include "qaccessible.h"
 #endif
 
-static const int thresholdTime = 500;
+static const int thresholdTime = 300;
 static const int repeatTime = 100;
 
-static int sliderStartVal = 0; //##### class member?
+struct QSliderPrivate
+{
+    // ### move these to QSlider in Qt 4.0
+    int sliderStartVal;
+    QSliderPrivate() : sliderStartVal( 0 ) { }
+};
 
 
 /*!
-  \class QSlider qslider.h
-  \brief The QSlider widget provides a vertical or horizontal slider.
-  \ingroup basic
-  \mainclass
+    \class QSlider
+    \brief The QSlider widget provides a vertical or horizontal slider.
 
-  The slider is the classic widget for controlling a bounded value.
-  It lets the user move a slider along a horizontal or vertical
-  groove and translates the slider's position into an integer value
-  within the legal range.
+    \ingroup basic
+    \mainclass
 
-  QSlider inherits QRangeControl, which provides the "integer" side of
-  the slider.  setRange() and value() are likely to be used by
-  practically all slider users; see the \l QRangeControl documentation
-  for information about the many other functions that class provides.
+    The slider is the classic widget for controlling a bounded value.
+    It lets the user move a slider along a horizontal or vertical
+    groove and translates the slider's position into an integer value
+    within the legal range.
 
-  The main functions offered by the slider itself are tickmark and
-  orientation control; you can use setTickmarks() to indicate where
-  you want the tickmarks to be, setTickInterval() to indicate how many
-  of them you want and setOrientation() to indicate whether the
-  slider is to be horizontal or vertical.
+    QSlider inherits QRangeControl, which provides the "integer" side
+    of the slider. setRange() and value() are likely to be used by
+    practically all slider users; see the \l QRangeControl
+    documentation for information about the many other functions that
+    class provides.
 
-  A slider has a default focusPolicy() of \c WeakWheelFocus, i.e. it
-  accepts focus on Tab and uses the mouse wheel and a
-  suitable keyboard interface.
+    The main functions offered by the slider itself are tickmark and
+    orientation control; you can use setTickmarks() to indicate where
+    you want the tickmarks to be, setTickInterval() to indicate how
+    many of them you want and setOrientation() to indicate whether the
+    slider is to be horizontal or vertical.
 
-  <img src=qslider-m.png> <img src=qslider-w.png>
+    A slider accepts focus on Tab and uses the mouse wheel and a
+    suitable keyboard interface.
 
-  \important setRange
+    <img src=qslider-m.png> <img src=qslider-w.png>
 
-  \sa QScrollBar QSpinBox
-  \link guibooks.html#fowler GUI Design Handbook: Slider\endlink
-*/
+    \important setRange
 
-
-/*! \enum QSlider::TickSetting
-
-  This enum specifies where the tickmarks are to be drawn relative
-  to the slider's groove and the handle the user moves.  The possible
-  values are:
-  \value NoMarks do not draw any tickmarks.
-  \value Both draw tickmarks on both sides of the groove.
-  \value Above draw tickmarks above the (horizontal) slider
-  \value Below draw tickmarks below the (horizontal) slider
-  \value Left draw tickmarks to the left of the (vertical) slider
-  \value Right draw tickmarks to the right of the (vertical) slider
+    \sa QScrollBar QSpinBox
+    \link guibooks.html#fowler GUI Design Handbook: Slider\endlink
 */
 
 
 /*!
-  Constructs a vertical slider.
+    \enum QSlider::TickSetting
 
-  The \a parent and \a name arguments are sent to the QWidget constructor.
+    This enum specifies where the tickmarks are to be drawn relative
+    to the slider's groove and the handle the user moves.
+
+    \value NoMarks do not draw any tickmarks.
+    \value Both draw tickmarks on both sides of the groove.
+    \value Above draw tickmarks above the (horizontal) slider
+    \value Below draw tickmarks below the (horizontal) slider
+    \value Left draw tickmarks to the left of the (vertical) slider
+    \value Right draw tickmarks to the right of the (vertical) slider
+*/
+
+
+/*!
+    Constructs a vertical slider.
+
+    The \a parent and \a name arguments are sent on to the QWidget
+    constructor.
 */
 
 QSlider::QSlider( QWidget *parent, const char *name )
@@ -116,11 +124,12 @@ QSlider::QSlider( QWidget *parent, const char *name )
 }
 
 /*!
-  Constructs a slider.
+    Constructs a slider.
 
-  The \a orientation must be \l Qt::Vertical or \l Qt::Horizontal.
+    The \a orientation must be \l Qt::Vertical or \l Qt::Horizontal.
 
-  The \a parent and \a name arguments are sent to the QWidget constructor.
+    The \a parent and \a name arguments are sent on to the QWidget
+    constructor.
 */
 
 QSlider::QSlider( Orientation orientation, QWidget *parent, const char *name )
@@ -130,15 +139,17 @@ QSlider::QSlider( Orientation orientation, QWidget *parent, const char *name )
     init();
 }
 
-/*!  Constructs a slider whose value can never be smaller than \a
-  minValue or greater than \a maxValue, whose page step size is
-  \a pageStep and whose value is initially \a value (which is
-  guaranteed to be in range using bound()).
+/*!
+    Constructs a slider whose value can never be smaller than \a
+    minValue or greater than \a maxValue, whose page step size is \a
+    pageStep and whose value is initially \a value (which is
+    guaranteed to be in range using bound()).
 
-  If \a orientation is Qt::Vertical the slider is vertical and if it is
-  Qt::Horizontal the slider is horizontal.
+    If \a orientation is \c Qt::Vertical the slider is vertical and if it
+    is \c Qt::Horizontal the slider is horizontal.
 
-  The \a parent and \a name arguments are sent to the QWidget constructor.
+    The \a parent and \a name arguments are sent on to the QWidget
+    constructor.
 */
 
 QSlider::QSlider( int minValue, int maxValue, int pageStep,
@@ -152,10 +163,17 @@ QSlider::QSlider( int minValue, int maxValue, int pageStep,
     sliderVal = value;
 }
 
+/*!
+    Destructor.
+*/
+QSlider::~QSlider()
+{
+    delete d;
+}
 
 void QSlider::init()
 {
-    extra = 0;
+    d = new QSliderPrivate;
     timer = 0;
     sliderPos = 0;
     sliderVal = 0;
@@ -167,18 +185,16 @@ void QSlider::init()
     setFocusPolicy( TabFocus  );
     initTicks();
 
-    if ( orient == Horizontal )
-	setSizePolicy( QSizePolicy( QSizePolicy::Expanding,
-				    QSizePolicy::Fixed ) );
-    else
-	setSizePolicy( QSizePolicy(  QSizePolicy::Fixed,
-				     QSizePolicy::Expanding ) );
-
+    QSizePolicy sp( QSizePolicy::Expanding, QSizePolicy::Fixed );
+    if ( orient == Vertical )
+	sp.transpose();
+    setSizePolicy( sp );
+    clearWState( WState_OwnSizePolicy );
 }
 
 
-/*!
-  Does what's needed when someone changes the tickmark status
+/*
+    Does what's needed when someone changes the tickmark status.
 */
 
 void QSlider::initTicks()
@@ -188,14 +204,14 @@ void QSlider::initTicks()
 
 
 /*!
-  \property QSlider::tracking
-  \brief whether slider tracking is enabled
+    \property QSlider::tracking
+    \brief whether slider tracking is enabled
 
-  If tracking is enabled (the default), the slider emits the
-  valueChanged() signal whenever the slider is being dragged.  If
-  tracking is disabled, the slider emits the valueChanged() signal
-  when the user releases the mouse button (unless the value happens to
-  be the same as before).
+    If tracking is enabled (the default), the slider emits the
+    valueChanged() signal whenever the slider is being dragged. If
+    tracking is disabled, the slider emits the valueChanged() signal
+    when the user releases the mouse button (unless the value happens
+    to be the same as before).
 */
 
 void QSlider::setTracking( bool enable )
@@ -205,29 +221,34 @@ void QSlider::setTracking( bool enable )
 
 
 /*!
-  \fn void QSlider::valueChanged( int value )
-  This signal is emitted when the slider value is changed, with the
-  new slider \a value as an argument.
+    \fn void QSlider::valueChanged( int value )
+
+    This signal is emitted when the slider value is changed, with the
+    new slider \a value as its argument.
 */
 
 /*!
-  \fn void QSlider::sliderPressed()
-  This signal is emitted when the user presses the slider with the mouse.
+    \fn void QSlider::sliderPressed()
+
+    This signal is emitted when the user presses the slider with the
+    mouse.
 */
 
 /*!
-  \fn void QSlider::sliderMoved( int value )
-  This signal is emitted when the slider is dragged, with the
-  new slider \a value as an argument.
+    \fn void QSlider::sliderMoved( int value )
+
+    This signal is emitted when the slider is dragged, with the new
+    slider \a value as its argument.
 */
 
 /*!
-  \fn void QSlider::sliderReleased()
-  This signal is emitted when the user releases the slider with the mouse.
+    \fn void QSlider::sliderReleased()
+
+    This signal is emitted when the user releases the slider with the mouse.
 */
 
-/*!
-  Calculates slider position corresponding to value \a v.
+/*
+    Calculates slider position corresponding to value \a v.
 */
 
 int QSlider::positionFromValue( int v ) const
@@ -239,8 +260,8 @@ int QSlider::positionFromValue( int v ) const
     return x;
 }
 
-/*!
-  Returns the available space in which the slider can move.
+/*
+    Returns the available space in which the slider can move.
 */
 
 int QSlider::available() const
@@ -248,8 +269,8 @@ int QSlider::available() const
     return style().pixelMetric( QStyle::PM_SliderSpaceAvailable, this );
 }
 
-/*!
-  Calculates a value corresponding to slider position \a p.
+/*
+    Calculates a value corresponding to slider position \a p.
 */
 
 int QSlider::valueFromPosition( int p ) const
@@ -262,7 +283,7 @@ int QSlider::valueFromPosition( int p ) const
 }
 
 /*!
-  Implements the virtual QRangeControl function.
+    Implements the virtual QRangeControl function.
 */
 
 void QSlider::rangeChange()
@@ -274,7 +295,7 @@ void QSlider::rangeChange()
 }
 
 /*!
-  Implements the virtual QRangeControl function.
+    Implements the virtual QRangeControl function.
 */
 
 void QSlider::valueChange()
@@ -291,7 +312,8 @@ void QSlider::valueChange()
 }
 
 
-/*!\reimp
+/*!
+    \reimp
 */
 void QSlider::resizeEvent( QResizeEvent * )
 {
@@ -301,10 +323,10 @@ void QSlider::resizeEvent( QResizeEvent * )
 
 
 /*!
-  Reimplements the virtual function QWidget::setPalette().
+    Reimplements the virtual function QWidget::setPalette().
 
-  Sets the background color to the mid color for Motif style sliders
-  using palette \a p.
+    Sets the background color to the mid color for Motif style sliders
+    using palette \a p.
 */
 
 void QSlider::setPalette( const QPalette &p )
@@ -315,24 +337,27 @@ void QSlider::setPalette( const QPalette &p )
 
 
 /*!
-  \property QSlider::orientation
-  \brief the orientation of the slider
+    \property QSlider::orientation
+    \brief the slider's orientation
 
-  The orientation must be \l Qt::Vertical (the default) or \l Qt::Horizontal.
+    The orientation must be \l Qt::Vertical (the default) or \l
+    Qt::Horizontal.
 */
 
 void QSlider::setOrientation( Orientation orientation )
 {
     if ( orientation == orient )
 	return;
+
+    if ( !testWState( WState_OwnSizePolicy ) ) {
+	QSizePolicy sp = sizePolicy();
+	sp.transpose();
+	setSizePolicy( sp );
+	clearWState( WState_OwnSizePolicy );
+    }
+
     orient = orientation;
 
-    if ( orient == Horizontal )
-	setSizePolicy( QSizePolicy( QSizePolicy::Expanding,
-				    QSizePolicy::Fixed ) );
-    else
-	setSizePolicy( QSizePolicy(  QSizePolicy::Fixed,
-				     QSizePolicy::Expanding ) );
     rangeChange();
     update();
 }
@@ -345,8 +370,8 @@ void QSlider::setOrientation( Orientation orientation )
 
 
 /*!
-  Returns the slider handle rectangle. (This is the visual marker that
-  the user can move.)
+    Returns the slider handle rectangle. (This is the visual marker
+    that the user can move.)
 */
 
 QRect QSlider::sliderRect() const
@@ -355,34 +380,28 @@ QRect QSlider::sliderRect() const
 					   QStyle::SC_SliderHandle );
 }
 
-/*!
-  Performs the actual moving of the slider.
+/*
+    Performs the actual moving of the slider.
 */
 
 void QSlider::reallyMoveSlider( int newPos )
 {
-    QRect oldR = sliderRect();
+    QRegion oldR(sliderRect());
     sliderPos = newPos;
-    QRect newR = sliderRect();
-    //since sliderRect isn't virtual, I know that oldR and newR
-    // are the same size.
-    if ( orient == Horizontal ) {
-	if ( oldR.left() < newR.left() )
-	    oldR.setRight( QMIN ( oldR.right(), newR.left()));
-	else           //oldR.right() >= newR.right()
-	    oldR.setLeft( QMAX ( oldR.left(), newR.right()));
-    } else {
-	if ( oldR.top() < newR.top() )
-	    oldR.setBottom( QMIN ( oldR.bottom(), newR.top()));
-	else           //oldR.bottom() >= newR.bottom()
-	    oldR.setTop( QMAX ( oldR.top(), newR.bottom()));
+    QRegion newR(sliderRect());
+
+    /* just the one repaint if no background */
+    if (backgroundMode() == NoBackground)
+	repaint(newR | oldR, FALSE);
+    else {
+	repaint(oldR.subtract(newR));
+	repaint(newR, FALSE);
     }
-    repaint( oldR );
-    repaint( newR, FALSE );
 }
 
 
-/*!\reimp
+/*!
+    \reimp
 */
 void QSlider::paintEvent( QPaintEvent * )
 {
@@ -403,13 +422,14 @@ void QSlider::paintEvent( QPaintEvent * )
 }
 
 
-/*!\reimp
+/*!
+    \reimp
 */
 void QSlider::mousePressEvent( QMouseEvent *e )
 {
     int slideLength = style().pixelMetric( QStyle::PM_SliderLength, this );
     resetState();
-    sliderStartVal = sliderVal;
+    d->sliderStartVal = sliderVal;
     QRect r = sliderRect();
 
     if ( e->button() == RightButton )
@@ -454,7 +474,8 @@ void QSlider::mousePressEvent( QMouseEvent *e )
     update( sliderRect() );
 }
 
-/*!\reimp
+/*!
+    \reimp
 */
 void QSlider::mouseMoveEvent( QMouseEvent *e )
 {
@@ -472,7 +493,7 @@ void QSlider::mouseMoveEvent( QMouseEvent *e )
 	    r.setRect( r.x() - 2*m/3, r.y() - m,
 		       r.width() + 3*m, r.height() + 2*m );
 	if ( !r.contains( e->pos() ) ) {
-	    moveSlider( positionFromValue( sliderStartVal) );
+	    moveSlider( positionFromValue(d->sliderStartVal) );
 	    return;
 	}
     }
@@ -481,10 +502,15 @@ void QSlider::mouseMoveEvent( QMouseEvent *e )
     moveSlider( pos - clickOffset );
 }
 
-/*!\reimp
+/*!
+    \reimp
 */
 #ifndef QT_NO_WHEELEVENT
-void QSlider::wheelEvent( QWheelEvent * e){
+void QSlider::wheelEvent( QWheelEvent * e )
+{
+    if ( e->orientation() != orientation() && !rect().contains(e->pos()) )
+	return;
+
     static float offset = 0;
     static QSlider* offset_owner = 0;
     if (offset_owner != this){
@@ -500,7 +526,8 @@ void QSlider::wheelEvent( QWheelEvent * e){
 }
 #endif
 
-/*!\reimp
+/*!
+    \reimp
 */
 void QSlider::mouseReleaseEvent( QMouseEvent * )
 {
@@ -508,14 +535,16 @@ void QSlider::mouseReleaseEvent( QMouseEvent * )
     update( sliderRect() );
 }
 
-/*!\reimp
+/*!
+    \reimp
 */
 void QSlider::focusInEvent( QFocusEvent * e)
 {
     QWidget::focusInEvent( e );
 }
 
-/*!\reimp
+/*!
+    \reimp
 */
 void QSlider::focusOutEvent( QFocusEvent * e )
 {
@@ -523,9 +552,9 @@ void QSlider::focusOutEvent( QFocusEvent * e )
 }
 
 /*!
-  Moves the left (or top) edge of the slider to position
-  \a pos. The slider is actually moved to the step position nearest
-  the given \a pos.
+    Moves the left (or top) edge of the slider to position \a pos. The
+    slider is actually moved to the step position nearest the given \a
+    pos.
 */
 
 void QSlider::moveSlider( int pos )
@@ -533,21 +562,22 @@ void QSlider::moveSlider( int pos )
     int  a = available();
     int newPos = QMIN( a, QMAX( 0, pos ) );
     int newVal = valueFromPosition( newPos );
+    if (style().styleHint(QStyle::SH_Slider_SnapToValue, this))
+	newPos = positionFromValue( newVal );
+    if ( sliderPos != newPos )
+	reallyMoveSlider( newPos );
     if ( sliderVal != newVal ) {
 	sliderVal = newVal;
 	emit sliderMoved( sliderVal );
     }
     if ( tracking() && sliderVal != value() )
 	setValue( sliderVal );
-    if (style().styleHint(QStyle::SH_Slider_SnapToValue, this))
-	newPos = positionFromValue( newVal );
-    if ( sliderPos != newPos )
-	reallyMoveSlider( newPos );
+
 }
 
 
-/*!
-  Resets all state information and stops the timer.
+/*
+    Resets all state information and stops the timer.
 */
 
 void QSlider::resetState()
@@ -574,7 +604,8 @@ void QSlider::resetState()
 }
 
 
-/*!\reimp
+/*!
+    \reimp
 */
 void QSlider::keyPressEvent( QKeyEvent *e )
 {
@@ -623,8 +654,24 @@ void QSlider::setValue( int value )
 }
 
 
+/*! \reimp
+*/
+
+void QSlider::addLine()
+{
+    QRangeControl::addLine();
+}
+
+/*! \reimp
+*/
+
+void QSlider::subtractLine()
+{
+    QRangeControl::subtractLine();
+}
+
 /*!
-  Moves the slider one pageStep() up or right.
+    Moves the slider one pageStep() up or right.
 */
 
 void QSlider::addStep()
@@ -634,7 +681,7 @@ void QSlider::addStep()
 
 
 /*!
-  Moves the slider one pageStep() down or left.
+    Moves the slider one pageStep() down or left.
 */
 
 void QSlider::subtractStep()
@@ -643,7 +690,7 @@ void QSlider::subtractStep()
 }
 
 
-/*!
+/*
   Waits for autorepeat.
 */
 
@@ -659,7 +706,7 @@ void QSlider::repeatTimeout()
 }
 
 
-/*!
+/*
   Returns the relevant dimension of \a p.
 */
 
@@ -668,32 +715,31 @@ int QSlider::goodPart( const QPoint &p ) const
     return (orient == Horizontal) ?  p.x() : p.y();
 }
 
-/*!\reimp
+/*!
+    \reimp
 */
 QSize QSlider::sizeHint() const
 {
     constPolish();
-    const int length = 84;
-    //    int thick = style() == MotifStyle ? 24 : 16;
+    const int length = 84, tickSpace = 5;
     int thick = style().pixelMetric( QStyle::PM_SliderThickness, this );
-    const int tickSpace = 5;
-
     if ( ticks & Above )
 	thick += tickSpace;
     if ( ticks & Below )
 	thick += tickSpace;
-    // if ( style() == WindowsStyle && ticks != Both && ticks != NoMarks )
-    // thick += style().pixelMetric( QStyle::PM_SliderLength, this ) / 4; // pointed slider
-    if ( orient == Horizontal )
-	return QSize( length, thick ).expandedTo( QApplication::globalStrut() );
-    else
-	return QSize( thick, length ).expandedTo( QApplication::globalStrut() );
+    int w = thick, h = length;
+    if ( orient == Horizontal ) {
+	w = length;
+	h = thick;
+    }
+    return (style().sizeFromContents(QStyle::CT_Slider, this,
+				     QSize(w, h)).expandedTo(QApplication::globalStrut()));
 }
 
 
 
 /*!
-  \reimp
+    \reimp
 */
 
 QSize QSlider::minimumSizeHint() const
@@ -708,64 +754,32 @@ QSize QSlider::minimumSizeHint() const
     return s;
 }
 
-
-
-/*!\reimp
+/*! \fn void QSlider::setSizePolicy( QSizePolicy::SizeType, QSizePolicy::SizeType, bool )
+    \reimp
 */
-QSizePolicy QSlider::sizePolicy() const
-{
-    return QWidget::sizePolicy();
 
-    /*
-    if ( orient == Horizontal )
-	return QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Fixed );
-    else
-	return QSizePolicy(  QSizePolicy::Fixed, QSizePolicy::Expanding );
-    */
+/*! \reimp */
+void QSlider::setSizePolicy( QSizePolicy sp )
+{
+    // ## remove 4.0
+    QWidget::setSizePolicy( sp );
 }
 
-
-/*
-  Returns the number of pixels to use for the business part of the
-  slider (i.e., the non-tickmark portion). The remaining space is shared
-  equally between the tickmark regions. This function and  sizeHint()
-  are closely related; if you change one you almost certainly
-  have to change the other.
-*/
-
-// style dependent stuff here! Remove when done!
-// int QSlider::thickness() const
-// {
-//     return style().pixelMetric( QStyle::PM_SliderControlThickness, this );
-//     int space = (orient == Horizontal) ? height() : width();
-//     int n = 0;
-//     if ( ticks & Above )
-// 	n++;
-//     if ( ticks & Below )
-// 	n++;
-//     if ( !n )
-// 	return space;
-
-//     int thick = 6;	// Magic constant to get 5 + 16 + 5
-//     if ( style() == WindowsStyle && ticks != Both && ticks != NoMarks ) {
-// 	thick += style().sliderLength() / 4;
-//     }
-//     space -= thick;
-//     //### the two sides may be unequal in size
-//     if ( space > 0 )
-// 	thick += ( space * 2 ) / ( n + 2 );
-//     return thick;
-// }
-
+/*! \reimp */
+QSizePolicy QSlider::sizePolicy() const
+{
+    // ### 4.0 remove this reimplementation
+    return QWidget::sizePolicy();
+}
 
 /*!
-  \property QSlider::tickmarks
-  \brief the tickmark settings for this slider
+    \property QSlider::tickmarks
+    \brief the tickmark settings for this slider
 
     The valid values are in \l{QSlider::TickSetting}. The default is
     \c NoMarks.
 
-  \sa tickInterval
+    \sa tickInterval
 */
 
 void QSlider::setTickmarks( TickSetting s )
@@ -777,14 +791,14 @@ void QSlider::setTickmarks( TickSetting s )
 
 
 /*!
-  \property QSlider::tickInterval
-  \brief the interval between tickmarks
+    \property QSlider::tickInterval
+    \brief the interval between tickmarks
 
-  This is a value interval, not a pixel interval. If it is 0, the slider
-  will choose between lineStep() and pageStep(). The initial value of
-  tickInterval is 0.
+    This is a value interval, not a pixel interval. If it is 0, the
+    slider will choose between lineStep() and pageStep(). The initial
+    value of tickInterval is 0.
 
-  \sa QRangeControl::lineStep(), QRangeControl::pageStep()
+    \sa QRangeControl::lineStep(), QRangeControl::pageStep()
 */
 
 void QSlider::setTickInterval( int i )
@@ -795,21 +809,21 @@ void QSlider::setTickInterval( int i )
 
 
 /*!
-  \reimp
- */
+    \reimp
+*/
 void QSlider::styleChange( QStyle& old )
 {
     QWidget::styleChange( old );
 }
 
 /*!
-  \property QSlider::minValue
-  \brief the current minimum value of the slider
+    \property QSlider::minValue
+    \brief the current minimum value of the slider
 
-  When setting this property, the \l QSlider::maxValue is adjusted, if
-  necessary, to ensure that the range remains valid.
+    When setting this property, the \l QSlider::maxValue is adjusted,
+    if necessary, to ensure that the range remains valid.
 
-  \sa setRange()
+    \sa setRange()
 */
 int QSlider::minValue() const
 {
@@ -817,13 +831,13 @@ int QSlider::minValue() const
 }
 
 /*!
-  \property QSlider::maxValue
-  \brief the current maximum value of the slider
+    \property QSlider::maxValue
+    \brief the current maximum value of the slider
 
-  When setting this property, the \l QSlider::minValue is adjusted, if
-  necessary, to ensure that the range remains valid.
+    When setting this property, the \l QSlider::minValue is adjusted,
+    if necessary, to ensure that the range remains valid.
 
-  \sa setRange()
+    \sa setRange()
 */
 int QSlider::maxValue() const
 {
@@ -841,13 +855,14 @@ void QSlider::setMaxValue( int maxVal )
 }
 
 /*!
-  \property QSlider::lineStep
-  \brief the current line step
+    \property QSlider::lineStep
+    \brief the current line step
 
-  When setting lineStep, the virtual stepChange() function will be called
-  if the new line step is different from the previous setting.
+    When setting lineStep, the virtual stepChange() function will be
+    called if the new line step is different from the previous
+    setting.
 
-  \sa setSteps() QRangeControl::pageStep() setRange()
+    \sa setSteps() QRangeControl::pageStep() setRange()
 */
 int QSlider::lineStep() const
 {
@@ -855,13 +870,14 @@ int QSlider::lineStep() const
 }
 
 /*!
-  \property QSlider::pageStep
-  \brief the current page step
+    \property QSlider::pageStep
+    \brief the current page step
 
-  When setting pageStep, the virtual stepChange() function will be called
-  if the new page step is different from the previous setting.
+    When setting pageStep, the virtual stepChange() function will be
+    called if the new page step is different from the previous
+    setting.
 
-  \sa QRangeControl::setSteps() setLineStep() setRange()
+    \sa QRangeControl::setSteps() setLineStep() setRange()
 */
 
 int QSlider::pageStep() const
@@ -880,10 +896,10 @@ void QSlider::setPageStep( int i )
 }
 
 /*!
-  \property QSlider::value
-  \brief the current slider value
+    \property QSlider::value
+    \brief the current slider value
 
-  \sa QRangeControl::value() prevValue()
+    \sa QRangeControl::value() prevValue()
 */
 
 int QSlider::value() const
