@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id:  qt/qtabwidget.cpp   3.0.5   edited Jun 28 11:02 $
+** $Id: qt/qtabwidget.cpp   3.2.3   edited Jun 27 09:25 $
 **
 ** Implementation of QTabWidget class
 **
 ** Created : 990318
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2003 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the widgets module of the Qt GUI Toolkit.
 **
@@ -45,90 +45,103 @@
 #include "qaccel.h"
 #include "qstyle.h"
 #include "qpainter.h"
+#include "qtoolbutton.h"
 
 /*!
-  \class QTabWidget qtabwidget.h
+    \class QTabWidget qtabwidget.h
+    \brief The QTabWidget class provides a stack of tabbed widgets.
 
-  \brief The QTabWidget class provides a stack of tabbed widgets.
+    \ingroup organizers
+    \ingroup advanced
+    \mainclass
 
-  \ingroup organizers
-  \mainclass
-  \ingroup advanced
+    A tab widget provides a tab bar of tabs and a `page area' below
+    (or above, see \l{TabPosition}) the tabs. Each tab is associated
+    with a different widget (called a `page'). Only the current tab's
+    page is shown in the page area; all the other tabs' pages are
+    hidden. The user can show a different page by clicking on its tab
+    or by pressing its Alt+\e{letter} accelerator if it has one.
 
-  A tabbed widget is a widget that has a tab bar of tabs, and for each
-  tab a "page" which is a widget. The user selects which page to see
-  and use by clicking on its tab or by pressing the indicated
-  Alt+\e{letter} key combination.
+    The normal way to use QTabWidget is to do the following in the
+    constructor:
+    \list 1
+    \i Create a QTabWidget.
+    \i Create a QWidget for each of the pages in the tab dialog,
+    insert children into it, set up geometry management for it and use
+    addTab() (or insertTab()) to set up a tab and keyboard accelerator
+    for it.
+    \i Connect to the signals and slots.
+    \endlist
 
-  QTabWidget provides a single row of tabs along the top or bottom of
-  the pages (see \l{TabPosition}).
+    The position of the tabs is set with setTabPosition(), their shape
+    with setTabShape(), and their margin with setMargin().
 
-  The normal way to use QTabWidget is to do the following in the
-  constructor:
-  \list 1
-  \i Create a QTabWidget.
-  \i Create a QWidget for each of the pages in the tab dialog, insert
-  children into it, set up geometry management for it and use addTab()
-  (or insertTab()) to set up a tab and keyboard accelerator for it.
-  \i Connect to the signals and slots.
-  \endlist
+    If you don't call addTab() and the QTabWidget is already visible,
+    then the page you have created will not be visible. Don't
+    confuse the object name you supply to the QWidget constructor and
+    the tab label you supply to addTab(). addTab() takes a name which
+    indicates an accelerator and is meaningful and descriptive to the
+    user, whereas the widget name is used primarily for debugging.
 
-  The position of the tabs is set with setTabPosition(), their
-  shape with setTabShape(), and their margin with setMargin().
+    The signal currentChanged() is emitted when the user selects a
+    page.
 
-  If you don't call addTab() the page you have created will not be
-  visible.  Don't confuse the object name you supply to the
-  QWidget constructor and the tab label you supply to addTab().
-  addTab() takes a name which indicates an accelerator and is
-  meaningful and descriptive to the user, whereas the widget name is
-  used primarily for debugging.
+    The current page is available as an index position with
+    currentPageIndex() or as a wiget pointer with currentPage(). You
+    can retrieve a pointer to a page with a given index using page(),
+    and can find the index position of a page with indexOf(). Use
+    setCurrentPage() to show a particular page by index, or showPage()
+    to show a page by widget pointer.
 
-  The signal currentChanged() is emitted when the user selects a page.
+    You can change a tab's label and iconset using changeTab() or
+    setTabLabel() and setTabIconSet(). A tab page can be removed with
+    removePage().
 
-  The current page is available as an index position with
-  currentPageIndex() or as a wiget pointer with currentPage(). You can
-  retrieve a pointer to a page with a given index using page(), and
-  can find the index position of a page with indexOf(). Use
-  setCurrentPage() to show a particular page by index, or showPage()
-  to show a page by widget pointer.
+    Each tab is either enabled or disabled at any given time (see
+    setTabEnabled()). If a tab is enabled, the tab text is drawn
+    normally and the user can select that tab. If it is disabled, the
+    tab is drawn in a different way and the user cannot select that
+    tab. Note that even if a tab is disabled, the page can still be
+    visible, for example if all of the tabs happen to be disabled.
 
-  You can change a tab's label and iconset using changeTab() or
-  setTabLabel() and setTabIconSet(). A tab page can be removed with
-  removePage().
+    Although tab widgets can be a very good way to split up a complex
+    dialog, it's also very easy to get into a mess. See QTabDialog for
+    some design hints. An alternative is to use a QWidgetStack for
+    which you provide some means of navigating between pages, for
+    example, a QToolBar or a QListBox.
 
-  Each tab is either enabled or disabled at any given time (see
-  setTabEnabled()).  If a tab is enabled, the tab text is drawn in
-  black and the user can select that tab.  If it is disabled, the tab
-  is drawn in a different way and the user cannot select that tab.
-  Note that even if a tab is disabled, the page can still be visible,
-  for example if all of the tabs happen to be disabled.
+    Most of the functionality in QTabWidget is provided by a QTabBar
+    (at the top, providing the tabs) and a QWidgetStack (most of the
+    area, organizing the individual pages).
 
-  Although tab widgets can be a very good way to split up a complex
-  dialog, it's also very easy to get into a mess. See QTabDialog for
-  some design hints.
+    <img src=qtabwidget-m.png> <img src=qtabwidget-w.png>
 
-  Most of the functionality in QTabWidget is provided by a QTabBar (at
-  the top, providing the tabs) and a QWidgetStack (most of the area,
-  organizing the individual pages).
-
-  <img src=qtabwidget-m.png> <img src=qtabwidget-w.png>
-
-  \sa QTabDialog
+    \sa QTabDialog, QToolBox
 */
 
-
-/*! \enum QTabWidget::TabPosition
-
-  This enum type defines where QTabWidget can draw the tab row:
-  \value Top  above the pages
-  \value Bottom  below the pages
+/*!
+  \enum Qt::Corner
+  This enum type specifies a corner in a rectangle:
+  \value TopLeft top left corner
+  \value TopRight top right corner
+  \value BottomLeft bottom left corner
+  \value BottomRight bottom right corner
 */
 
-/*! \enum QTabWidget::TabShape
+/*!
+    \enum QTabWidget::TabPosition
 
-  This enum type defines the shape of the tabs:
-  \value Rounded  rounded look (normal)
-  \value Triangular  triangular look (very unusual, included for completeness)
+    This enum type defines where QTabWidget draws the tab row:
+    \value Top  above the pages
+    \value Bottom  below the pages
+*/
+
+/*!
+    \enum QTabWidget::TabShape
+
+    This enum type defines the shape of the tabs:
+    \value Rounded  rounded look (normal)
+    \value Triangular  triangular look (very unusual, included for completeness)
 */
 
 /* undocumented now
@@ -143,12 +156,13 @@
 */
 
 
-/*! \fn void QTabWidget::currentChanged( QWidget* );
+/*!
+    \fn void QTabWidget::currentChanged( QWidget* );
 
-  This signal is emitted whenever the current page changes. The
-  parameter is the new current page.
+    This signal is emitted whenever the current page changes. The
+    parameter is the new current page.
 
-  \sa currentPage(), showPage(), tabLabel()
+    \sa currentPage(), showPage(), tabLabel()
 */
 
 class QTabBarBase : public QWidget
@@ -169,6 +183,8 @@ protected:
 		flags |= QStyle::Style_Top;
 	    if ( t->tabPosition() == QTabWidget::Bottom )
 		flags |= QStyle::Style_Bottom;
+	    if(parentWidget()->isEnabled())
+		flags |= QStyle::Style_Enabled;
 
 	    style().drawPrimitive( QStyle::PE_TabBarBase, &p, rect(),
 				   colorGroup(), flags );
@@ -181,7 +197,8 @@ class QTabWidgetData
 public:
     QTabWidgetData()
         : tabs(0), tabBase(0), stack(0), dirty( TRUE ),
-          pos( QTabWidget::Top ), shape( QTabWidget::Rounded ) {};
+          pos( QTabWidget::Top ), shape( QTabWidget::Rounded ),
+	  leftCornerWidget(0), rightCornerWidget(0) {};
     ~QTabWidgetData(){};
     QTabBar* tabs;
     QTabBarBase* tabBase;
@@ -190,17 +207,18 @@ public:
     QTabWidget::TabPosition pos;
     QTabWidget::TabShape shape;
     int alignment;
+    QWidget* leftCornerWidget;
+    QWidget* rightCornerWidget;
 };
 
 /*!
-  Constructs a tabbed widget with parent \a parent, name \a name,
-  and widget flags \a f.
+    Constructs a tabbed widget called \a name with parent \a parent,
+    and widget flags \a f.
 */
 QTabWidget::QTabWidget( QWidget *parent, const char *name, WFlags f )
     : QWidget( parent, name, f )
 {
     d = new QTabWidgetData;
-
     d->stack = new QWidgetStack( this, "tab pages" );
     d->stack->installEventFilter( this );
     d->tabBase = new QTabBarBase( this, "tab base" );
@@ -208,14 +226,16 @@ QTabWidget::QTabWidget( QWidget *parent, const char *name, WFlags f )
     setTabBar( new QTabBar( this, "tab control" ) );
 
     d->stack->setFrameStyle( QFrame::TabWidgetPanel | QFrame::Raised );
-    d->stack->setLineWidth( style().pixelMetric(QStyle::PM_DefaultFrameWidth, this) );
 
     setSizePolicy( QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding ) );
     setFocusPolicy( TabFocus );
     setFocusProxy( d->tabs );
+
+    installEventFilter( this );
 }
 
-/*!\reimp
+/*!
+    \reimp
 */
 QTabWidget::~QTabWidget()
 {
@@ -223,91 +243,81 @@ QTabWidget::~QTabWidget()
 }
 
 /*!
-  Adds another tab and page to the tab view.
+    Adds another tab and page to the tab view.
 
-    The new page is \a child; the tab's label is \a label.
-  Note the difference between the widget name (which you supply to
-  widget constructors and to setTabEnabled(), for example) and the tab
-  label. The name is internal to the program and invariant, whereas
-  the label is shown on-screen and may vary according to language and
-  other factors.
+    The new page is \a child; the tab's label is \a label. Note the
+    difference between the widget name (which you supply to widget
+    constructors and to setTabEnabled(), for example) and the tab
+    label. The name is internal to the program and invariant, whereas
+    the label is shown on-screen and may vary according to language
+    and other factors.
 
     If the tab's \a label contains an ampersand, the letter following
     the ampersand is used as an accelerator for the tab, e.g. if the
-    label is "Bro&wse" then Alt+W becomes an accelerator which will
+    label is "Bro\&wse" then Alt+W becomes an accelerator which will
     move the focus to this tab.
 
-  If you call addTab() after show() the screen will flicker and the
-  user may be confused.
+    If you call addTab() after show() the screen will flicker and the
+    user may be confused.
 
-  \sa insertTab()
+    \sa insertTab()
 */
 void QTabWidget::addTab( QWidget *child, const QString &label)
 {
-    QTab * t = new QTab();
-    Q_CHECK_PTR( t );
-    t->label = label;
-    addTab( child, t );
+    insertTab( child, label );
 }
 
 
 /*!
     \overload
-  Adds another tab and page to the tab view.
 
-  This function is the same as addTab(), but with an additional
-  \a iconset.
- */
+    Adds another tab and page to the tab view.
+
+    This function is the same as addTab(), but with an additional \a
+    iconset.
+*/
 void QTabWidget::addTab( QWidget *child, const QIconSet& iconset, const QString &label )
 {
-    QTab * t = new QTab();
-    Q_CHECK_PTR( t );
-    t->label = label;
-    t->iconset = new QIconSet( iconset );
-    addTab( child, t );
+    insertTab( child, iconset, label );
 }
 
-/*! \overload
+/*!
+    \overload
 
-  This is a low-level function for adding tabs. It is useful if you
-  are using setTabBar() to set a QTabBar subclass with an overridden
-  QTabBar::paint() routine for a subclass of QTab. The \a child is
-  the new page and \a tab is the tab to put the \a child on.
+    This is a low-level function for adding tabs. It is useful if you
+    are using setTabBar() to set a QTabBar subclass with an overridden
+    QTabBar::paint() function for a subclass of QTab. The \a child is
+    the new page and \a tab is the tab to put the \a child on.
 */
 void QTabWidget::addTab( QWidget *child, QTab* tab )
 {
-    tab->enabled = TRUE;
-    int id = d->tabs->addTab( tab );
-    d->stack->addWidget( child, id );
-    if ( d->stack->frameStyle() != ( QFrame::TabWidgetPanel | QFrame::Raised ) )
-        d->stack->setFrameStyle( QFrame::TabWidgetPanel | QFrame::Raised );
-    setUpLayout();
+    insertTab( child, tab );
 }
 
 
 
 /*!
-  Inserts another tab and page to the tab view.
+    Inserts another tab and page to the tab view.
 
-    The new page is \a child; the tab's label is \a label.
-  Note the difference between the widget name (which you supply to
-  widget constructors and to setTabEnabled(), for example) and the tab
-  label. The name is internal to the program and invariant, whereas
-  the label is shown on-screen and may vary according to language and
-  other factors.
+    The new page is \a child; the tab's label is \a label. Note the
+    difference between the widget name (which you supply to widget
+    constructors and to setTabEnabled(), for example) and the tab
+    label. The name is internal to the program and invariant, whereas
+    the label is shown on-screen and may vary according to language
+    and other factors.
 
     If the tab's \a label contains an ampersand, the letter following
     the ampersand is used as an accelerator for the tab, e.g. if the
-    label is "Bro&wse" then Alt+W becomes an accelerator which will
+    label is "Bro\&wse" then Alt+W becomes an accelerator which will
     move the focus to this tab.
 
-  If \a index is not specified, the tab is simply added. Otherwise
-  it is inserted at the specified position.
+    If \a index is not specified, the tab is simply appended.
+    Otherwise it is inserted at the specified position.
 
-  If you call insertTab() after show(), the screen will flicker and the
-  user may be confused.
+    If you call insertTab() after show(), the screen will flicker and
+    the user may be confused.
 
-  \sa addTab()
+    \sa addTab()
 */
 void QTabWidget::insertTab( QWidget *child, const QString &label, int index)
 {
@@ -320,11 +330,12 @@ void QTabWidget::insertTab( QWidget *child, const QString &label, int index)
 
 /*!
     \overload
-  Inserts another tab and page to the tab view.
 
-  This function is the same as insertTab(), but with an additional
-  \a iconset.
- */
+    Inserts another tab and page to the tab view.
+
+    This function is the same as insertTab(), but with an additional
+    \a iconset.
+*/
 void QTabWidget::insertTab( QWidget *child, const QIconSet& iconset, const QString &label, int index )
 {
     QTab * t = new QTab();
@@ -336,13 +347,13 @@ void QTabWidget::insertTab( QWidget *child, const QIconSet& iconset, const QStri
 
 /*!
     \overload
-  This is a lower-level method for inserting tabs, similar to the other
-  insertTab() method.  It is useful if you are using setTabBar() to set a
-  QTabBar subclass with an overridden QTabBar::paint() routine for a
-  subclass of QTab.
-  The \a child is the new page, \a tab is the tab to put the \a child
-  on and \a index is the position in the tab bar that this page should
-  occupy.
+
+    This is a lower-level method for inserting tabs, similar to the
+    other insertTab() method. It is useful if you are using
+    setTabBar() to set a QTabBar subclass with an overridden
+    QTabBar::paint() function for a subclass of QTab. The \a child is
+    the new page, \a tab is the tab to put the \a child on and \a
+    index is the position in the tab bar that this page should occupy.
 */
 void QTabWidget::insertTab( QWidget *child, QTab* tab, int index)
 {
@@ -356,8 +367,8 @@ void QTabWidget::insertTab( QWidget *child, QTab* tab, int index)
 
 
 /*!
-  Defines a new \a label for page \a{w}'s tab.
- */
+    Defines a new \a label for page \a{w}'s tab.
+*/
 void QTabWidget::changeTab( QWidget *w, const QString &label)
 {
     int id = d->stack->id( w );
@@ -372,7 +383,9 @@ void QTabWidget::changeTab( QWidget *w, const QString &label)
     int ct = d->tabs->currentTab();
     bool block = d->tabs->signalsBlocked();
     d->tabs->blockSignals( TRUE );
-    d->tabs->setCurrentTab( 0 );
+    QTab* ft = d->tabs->tabAt( 0 );
+    if ( ft )
+	d->tabs->setCurrentTab( ft->identifier() );
     d->tabs->setCurrentTab( ct );
     d->tabs->blockSignals( block );
 
@@ -382,8 +395,9 @@ void QTabWidget::changeTab( QWidget *w, const QString &label)
 
 /*!
     \overload
-  Defines a new \a iconset and a new \a label for page \a{w}'s tab.
- */
+
+    Defines a new \a iconset and a new \a label for page \a{w}'s tab.
+*/
 void QTabWidget::changeTab( QWidget *w, const QIconSet& iconset, const QString &label)
 {
     int id = d->stack->id( w );
@@ -403,7 +417,9 @@ void QTabWidget::changeTab( QWidget *w, const QIconSet& iconset, const QString &
     int ct = d->tabs->currentTab();
     bool block = d->tabs->signalsBlocked();
     d->tabs->blockSignals( TRUE );
-    d->tabs->setCurrentTab( 0 );
+    QTab* ft = d->tabs->tabAt( 0 );
+    if ( ft )
+	d->tabs->setCurrentTab( ft->identifier() );
     d->tabs->setCurrentTab( ct );
     d->tabs->blockSignals( block );
 
@@ -412,9 +428,9 @@ void QTabWidget::changeTab( QWidget *w, const QIconSet& iconset, const QString &
 }
 
 /*!
-  Returns TRUE if the page \a w is enabled; otherwise returns FALSE.
+    Returns TRUE if the page \a w is enabled; otherwise returns FALSE.
 
-  \sa setTabEnabled(), QWidget::isEnabled()
+    \sa setTabEnabled(), QWidget::isEnabled()
 */
 
 bool QTabWidget::isTabEnabled( QWidget* w ) const
@@ -430,14 +446,14 @@ bool QTabWidget::isTabEnabled( QWidget* w ) const
     If \a enable is TRUE, page \a w is enabled; otherwise page \a w is
     disabled. The page's tab is redrawn appropriately.
 
-  QTabWidget uses QWidget::setEnabled() internally, rather than keeping a
-  separate flag.
+    QTabWidget uses QWidget::setEnabled() internally, rather than
+    keeping a separate flag.
 
-  Note that even a disabled tab/page may be visible.  If the page is
-  visible already, QTabWidget will not hide it; if all the pages
-  are disabled, QTabWidget will show one of them.
+    Note that even a disabled tab/page may be visible. If the page is
+    visible already, QTabWidget will not hide it; if all the pages are
+    disabled, QTabWidget will show one of them.
 
-  \sa isTabEnabled(), QWidget::setEnabled()
+    \sa isTabEnabled(), QWidget::setEnabled()
 */
 
 void QTabWidget::setTabEnabled( QWidget* w, bool enable)
@@ -449,12 +465,42 @@ void QTabWidget::setTabEnabled( QWidget* w, bool enable)
     }
 }
 
-/*!  Ensures that page \a w is shown.  This is useful mainly for accelerators.
+/*!
+  Sets widget \a w to be the shown in the specified \a corner of the
+  tab widget.
 
-  \warning Used carelessly, this function can easily surprise or
-  confuse the user.
+  Only the horizontal element of the \a corner will be used.
 
-  \sa QTabBar::setCurrentTab()
+  \sa cornerWidget(), setTabPosition()
+*/
+void QTabWidget::setCornerWidget( QWidget * w, Qt::Corner corner )
+{
+    if ( !w )
+	return;
+    if ( (uint)corner & 1 )
+	d->rightCornerWidget = w;
+    else
+	d->leftCornerWidget = w;
+}
+
+/*!
+    Returns the widget shown in the \a corner of the tab widget or 0.
+*/
+QWidget * QTabWidget::cornerWidget( Qt::Corner corner ) const
+{
+    if ( (uint)corner & 1 )
+	return d->rightCornerWidget;
+    return d->leftCornerWidget;
+}
+
+/*!
+    Ensures that page \a w is shown. This is useful mainly for
+    accelerators.
+
+    \warning Used carelessly, this function can easily surprise or
+    confuse the user.
+
+    \sa QTabBar::setCurrentTab()
 */
 void QTabWidget::showPage( QWidget * w)
 {
@@ -468,17 +514,27 @@ void QTabWidget::showPage( QWidget * w)
     }
 }
 
-/*! Removes page \a w from this stack of widgets.  Does not
-  delete \a w.
-  \sa showPage(), QWidgetStack::removeWidget()
+/*!
+    Removes page \a w from this stack of widgets. Does not delete \a
+    w.
+
+    \sa addTab(), showPage(), QWidgetStack::removeWidget()
 */
 void QTabWidget::removePage( QWidget * w )
 {
     int id = d->stack->id( w );
     if ( id >= 0 ) {
+	int oldId = d->stack->id(currentPage());
+	bool fixCurrentTab = oldId == id;
+	//switches to the next enabled tab
 	d->tabs->setTabEnabled( id, FALSE );
+	//if no next enabled page we fix the current page
+	fixCurrentTab = fixCurrentTab && oldId == d->stack->id(currentPage());
+
 	d->stack->removeWidget( w );
 	d->tabs->removeTab( d->tabs->tab(id) );
+	if ( fixCurrentTab )
+	    showTab( d->tabs->currentTab() );
 	setUpLayout();
 
 	if ( d->tabs->count() == 0 )
@@ -486,7 +542,8 @@ void QTabWidget::removePage( QWidget * w )
     }
 }
 
-/*!  Returns the label text for the tab on page \a w.
+/*!
+    Returns the label text for the tab on page \a w.
 */
 
 QString QTabWidget::tabLabel( QWidget * w ) const
@@ -495,8 +552,9 @@ QString QTabWidget::tabLabel( QWidget * w ) const
     return t ? t->label : QString::null;
 }
 
-/*!  Sets the tab label for page \a w to \a l
- */
+/*!
+    Sets the tab label for page \a w to \a l
+*/
 
 void QTabWidget::setTabLabel( QWidget * w, const QString &l )
 {
@@ -508,8 +566,9 @@ void QTabWidget::setTabLabel( QWidget * w, const QString &l )
     setUpLayout();
 }
 
-/*! Returns a pointer to the page currently being displayed by the
-    tab dialog.  The tab dialog does its best to make sure that this value
+/*!
+    Returns a pointer to the page currently being displayed by the tab
+    dialog. The tab dialog does its best to make sure that this value
     is never 0 (but if you try hard enough, it can be).
 */
 
@@ -518,13 +577,15 @@ QWidget * QTabWidget::currentPage() const
     return page( currentPageIndex() );
 }
 
-/*! \property QTabWidget::autoMask
+/*!
+    \property QTabWidget::autoMask
     \brief whether the tab widget is automatically masked
 
     \sa QWidget::setAutoMask()
 */
 
-/*! \property QTabWidget::currentPage
+/*!
+    \property QTabWidget::currentPage
     \brief the index position of the current tab page
 
   \sa QTabBar::currentTab()
@@ -543,9 +604,9 @@ void QTabWidget::setCurrentPage( int index )
 
 
 /*!
-  Returns the index position of page \a w, or -1 if the widget cannot
-  be found.
- */
+    Returns the index position of page \a w, or -1 if the widget
+    cannot be found.
+*/
 int QTabWidget::indexOf( QWidget* w ) const
 {
     return d->tabs->indexOf( d->stack->id( w ) );
@@ -553,8 +614,8 @@ int QTabWidget::indexOf( QWidget* w ) const
 
 
 /*!
-  \reimp
- */
+    \reimp
+*/
 void QTabWidget::resizeEvent( QResizeEvent *e )
 {
     QWidget::resizeEvent( e );
@@ -562,10 +623,11 @@ void QTabWidget::resizeEvent( QResizeEvent *e )
 }
 
 /*!
-  Replaces the QTabBar heading the dialog by the tab bar \a tb.
-  Note that this must be called \e before any tabs have been added,
-  or the behavior is undefined.
-  \sa tabBar()
+    Replaces the dialog's QTabBar heading with the tab bar \a tb. Note
+    that this must be called \e before any tabs have been added, or
+    the behavior is undefined.
+
+    \sa tabBar()
 */
 void QTabWidget::setTabBar( QTabBar* tb)
 {
@@ -581,8 +643,9 @@ void QTabWidget::setTabBar( QTabBar* tb)
 
 
 /*!
-  Returns the currently set QTabBar.
-  \sa setTabBar()
+    Returns the current QTabBar.
+
+    \sa setTabBar()
 */
 QTabBar* QTabWidget::tabBar() const
 {
@@ -590,7 +653,8 @@ QTabBar* QTabWidget::tabBar() const
 }
 
 /*!
-  Ensures that the selected tab's page is visible and appropriately sized.
+    Ensures that the selected tab's page is visible and appropriately
+    sized.
 */
 
 void QTabWidget::showTab( int i )
@@ -602,9 +666,9 @@ void QTabWidget::showTab( int i )
     }
 }
 
-/*!
-  Set up the layout.
- */
+/*
+    Set up the layout.
+*/
 void QTabWidget::setUpLayout( bool onlyCheck )
 {
     if ( onlyCheck && !d->dirty )
@@ -615,9 +679,24 @@ void QTabWidget::setUpLayout( bool onlyCheck )
 	return; // we'll do it later
     }
 
-    QSize t( d->tabs->sizeHint() );
-    if ( t.width() > width() )
-	t.setWidth( width() );
+    QSize t( 0, d->stack->frameWidth() );
+    if ( d->tabs->isVisibleTo(this) )
+	t = d->tabs->sizeHint();
+    int lcw = 0;
+    if ( d->leftCornerWidget && d->leftCornerWidget->isVisible()  ) {
+	lcw = d->leftCornerWidget->width();
+	if ( t.height() > lcw )
+	    lcw = t.height();
+     }
+    int rcw = 0;
+    if ( d->rightCornerWidget && d->rightCornerWidget->isVisible() ) {
+	rcw = d->rightCornerWidget->width();
+	if ( t.height() > rcw )
+	    rcw = t.height();
+    }
+    int tw = width() - lcw - rcw;
+    if ( t.width() > tw )
+	t.setWidth( tw );
     int lw = d->stack->lineWidth();
     bool reverse = QApplication::reverseLayout();
     int tabx, taby, stacky, exty, exth, overlap;
@@ -625,10 +704,10 @@ void QTabWidget::setUpLayout( bool onlyCheck )
     exth = style().pixelMetric( QStyle::PM_TabBarBaseHeight, this );
     overlap = style().pixelMetric( QStyle::PM_TabBarBaseOverlap, this );
 
-    if( reverse ) {
-	tabx = QMIN( width() - t.width(), width() - t.width() - lw + 2 );
+    if ( reverse ) {
+	tabx = QMIN( width() - t.width(), width() - t.width() - lw + 2 ) - lcw;
     } else {
-	tabx = QMAX( 0, lw - 2 );
+	tabx = QMAX( 0, lw - 2 ) + lcw;
     }
     if ( d->pos == Bottom ) {
 	taby = height() - t.height() - lw;
@@ -664,9 +743,22 @@ void QTabWidget::setUpLayout( bool onlyCheck )
 	update();
     if ( autoMask() )
 	updateMask();
+
+    // move cornerwidgets
+    if ( d->leftCornerWidget ) {
+	int y = ( t.height() / 2 ) - ( d->leftCornerWidget->height() / 2 );
+	int x = ( reverse ? width() - lcw + y : y );
+	d->leftCornerWidget->move( x, y + taby );
+    }
+    if ( d->rightCornerWidget ) {
+	int y = ( t.height() / 2 ) - ( d->rightCornerWidget->height() / 2 );
+	int x = ( reverse ? y : width() - rcw + y );
+	d->rightCornerWidget->move( x, y + taby );
+    }
 }
 
-/*!\reimp
+/*!
+    \reimp
 */
 QSize QTabWidget::sizeHint() const
 {
@@ -676,13 +768,17 @@ QSize QTabWidget::sizeHint() const
     }
     QSize s( d->stack->sizeHint() );
     QSize t( d->tabs->sizeHint().boundedTo( QSize(200,200) ) );
-    return QSize( QMAX( s.width(), t.width() ),
-		  s.height() + t.height() + d->tabBase->height() );
+
+    QSize sz( QMAX( s.width(), t.width() ), s.height() + t.height() +
+	      ( d->tabBase->isVisible() ? d->tabBase->height() : 0 ) );
+    return style().sizeFromContents(QStyle::CT_TabWidget, this, sz).expandedTo(QApplication::globalStrut());
 }
 
 
-/*! \reimp
-  Returns a suitable minimum size for the tab widget.
+/*!
+    \reimp
+
+    Returns a suitable minimum size for the tab widget.
 */
 QSize QTabWidget::minimumSizeHint() const
 {
@@ -692,11 +788,13 @@ QSize QTabWidget::minimumSizeHint() const
     }
     QSize s( d->stack->minimumSizeHint() );
     QSize t( d->tabs->minimumSizeHint() );
-    return QSize( QMAX( s.width(), t.width() ),
-		  s.height() + t.height() + d->tabBase->height() );
+    QSize sz( QMAX( s.width(), t.width() ), s.height() + t.height() +
+	      ( d->tabBase->isVisible() ? d->tabBase->height() : 0 ) );
+    return style().sizeFromContents(QStyle::CT_TabWidget, this, sz).expandedTo(QApplication::globalStrut());
 }
 
-/*! \reimp
+/*!
+    \reimp
  */
 void QTabWidget::showEvent( QShowEvent * )
 {
@@ -704,14 +802,15 @@ void QTabWidget::showEvent( QShowEvent * )
 }
 
 
-/*! \property QTabWidget::tabPosition
+/*!
+    \property QTabWidget::tabPosition
     \brief the position of the tabs in this tab widget
 
-  Possible values for this property are QTabWidget::Top and
-  QTabWidget::Bottom.
+    Possible values for this property are \c QTabWidget::Top and \c
+    QTabWidget::Bottom.
 
-  \sa TabPosition
- */
+    \sa TabPosition
+*/
 QTabWidget::TabPosition QTabWidget::tabPosition() const
 {
     return d->pos;
@@ -738,11 +837,12 @@ void QTabWidget::setTabPosition( TabPosition pos)
     setUpLayout();
 }
 
-/*! \property QTabWidget::tabShape
+/*!
+    \property QTabWidget::tabShape
     \brief the shape of the tabs in this tab widget
 
-    Possible values for this property are QTabWidget::Rounded (default) or
-    QTabWidget::Triangular.
+    Possible values for this property are \c QTabWidget::Rounded
+    (default) or \c QTabWidget::Triangular.
 
     \sa TabShape
 */
@@ -773,11 +873,12 @@ void QTabWidget::setTabShape( TabShape s )
 }
 
 
-/*! \property QTabWidget::margin
+/*!
+    \property QTabWidget::margin
     \brief the margin in this tab widget
 
-  The margin is the distance between the innermost pixel of the frame
-  and the outermost pixel of the pages.
+    The margin is the distance between the innermost pixel of the
+    frame and the outermost pixel of the pages.
 */
 int QTabWidget::margin() const
 {
@@ -791,17 +892,18 @@ void QTabWidget::setMargin( int w )
 }
 
 
-/*! \reimp
+/*!
+    \reimp
  */
 void QTabWidget::styleChange( QStyle& old )
 {
-    d->stack->setLineWidth( style().pixelMetric(QStyle::PM_DefaultFrameWidth, this));
-    setUpLayout();
     QWidget::styleChange( old );
+    setUpLayout();
 }
 
 
-/*! \reimp
+/*!
+    \reimp
  */
 void QTabWidget::updateMask()
 {
@@ -816,20 +918,53 @@ void QTabWidget::updateMask()
 }
 
 
-/*!\reimp
+/*!
+    \reimp
  */
 bool QTabWidget::eventFilter( QObject *o, QEvent * e)
 {
-    if ( o == d->stack && e->type() == QEvent::ChildRemoved
-         && ( (QChildEvent*)e )->child()->isWidgetType() ) {
-        removePage( (QWidget*)  ( (QChildEvent*)e )->child() );
-        return TRUE;
+    if ( o == this ) {
+	if ( e->type() == QEvent::LanguageChange || e->type() == QEvent::LayoutHint ) {
+	    d->dirty = TRUE;
+	    setUpLayout();
+	    updateGeometry();
+	} else if ( e->type() == QEvent::KeyPress ) {
+	    QKeyEvent *ke = (QKeyEvent*) e;
+	    if ( ( ke->key() == Qt::Key_Tab || ke->key() == Qt::Key_Backtab ) &&
+		 count() > 1 &&
+		 ke->state() & Qt::ControlButton ) {
+		int page = currentPageIndex();
+		if ( ke->key() == Qt::Key_Backtab || ke->state() & Qt::ShiftButton ) {
+		    page--;
+		    if ( page < 0 )
+			page = count() - 1;
+		} else {
+		    page++;
+		    if ( page >= count() )
+			page = 0;
+		}
+		setCurrentPage( page );
+		if ( !qApp->focusWidget() )
+		    d->tabs->setFocus();
+		return TRUE;
+	    }
+	}
+
+    } else if ( o == d->stack ) {
+	if ( e->type() == QEvent::ChildRemoved
+	     && ( (QChildEvent*)e )->child()->isWidgetType() ) {
+	    removePage( (QWidget*)  ( (QChildEvent*)e )->child() );
+	    return TRUE;
+	} else if ( e->type() == QEvent::LayoutHint ) {
+	    updateGeometry();
+	}
     }
     return FALSE;
 }
 
 /*!
-  Returns the tab page at index position \a index.
+    Returns the tab page at index position \a index or 0 if the \a
+    index is out of range.
 */
 QWidget *QTabWidget::page( int index ) const
 {
@@ -841,7 +976,8 @@ QWidget *QTabWidget::page( int index ) const
 }
 
 /*!
-  Returns the label of the tab at index position \a index.
+    Returns the label of the tab at index position \a index or
+    QString::null if the \a index is out of range.
 */
 QString QTabWidget::label( int index ) const
 {
@@ -852,8 +988,9 @@ QString QTabWidget::label( int index ) const
     return QString::null;
 }
 
-/*! \property QTabWidget::count
-     \brief the number of tabs in the tab bar
+/*!
+    \property QTabWidget::count
+    \brief the number of tabs in the tab bar
 */
 int QTabWidget::count() const
 {
@@ -861,7 +998,9 @@ int QTabWidget::count() const
 }
 
 /*!
-  Returns the iconset of page \a w.
+    Returns the iconset of page \a w or a \link QIconSet::QIconSet()
+    null iconset\endlink if \a w is not a tab page or does not have an
+    iconset.
 */
 QIconSet QTabWidget::tabIconSet( QWidget * w ) const
 {
@@ -878,7 +1017,7 @@ QIconSet QTabWidget::tabIconSet( QWidget * w ) const
 }
 
 /*!
-  Sets the iconset for page \a w to \a iconset.
+    Sets the iconset for page \a w to \a iconset.
 */
 void QTabWidget::setTabIconSet( QWidget * w, const QIconSet & iconset )
 {
@@ -897,7 +1036,9 @@ void QTabWidget::setTabIconSet( QWidget * w, const QIconSet & iconset )
     int ct = d->tabs->currentTab();
     bool block = d->tabs->signalsBlocked();
     d->tabs->blockSignals( TRUE );
-    d->tabs->setCurrentTab( 0 );
+    QTab* ft = d->tabs->tabAt( 0 );
+    if ( ft )
+	d->tabs->setCurrentTab( ft->identifier() );
     d->tabs->setCurrentTab( ct );
     d->tabs->blockSignals( block );
 
@@ -906,9 +1047,10 @@ void QTabWidget::setTabIconSet( QWidget * w, const QIconSet & iconset )
 }
 
 /*!
-  Sets the tab tool tip for page \a w to \a tip.
-  \sa removeTabToolTip(), tabToolTip()
- */
+    Sets the tab tool tip for page \a w to \a tip.
+
+    \sa removeTabToolTip(), tabToolTip()
+*/
 void QTabWidget::setTabToolTip( QWidget * w, const QString & tip )
 {
     int index = d->tabs->indexOf( d->stack->id( w ) );
@@ -918,9 +1060,11 @@ void QTabWidget::setTabToolTip( QWidget * w, const QString & tip )
 }
 
 /*!
-  Returns the tab tool tip for page \a w.
-  \sa setTabToolTip(), removeTabToolTip()
- */
+    Returns the tab tool tip for page \a w or QString::null if no tool
+    tip has been set.
+
+    \sa setTabToolTip(), removeTabToolTip()
+*/
 QString QTabWidget::tabToolTip( QWidget * w ) const
 {
     int index = d->tabs->indexOf( d->stack->id( w ) );
@@ -929,10 +1073,12 @@ QString QTabWidget::tabToolTip( QWidget * w ) const
     return d->tabs->toolTip( index );
 }
 
-/*! Removes the tab tool tip for page \a w. If the page does not have
-  a tip, nothing happens.
-  \sa setTabToolTip(), tabToolTip()
- */
+/*!
+    Removes the tab tool tip for page \a w. If the page does not have
+    a tip, nothing happens.
+
+    \sa setTabToolTip(), tabToolTip()
+*/
 void QTabWidget::removeTabToolTip( QWidget * w )
 {
     int index = d->tabs->indexOf( d->stack->id( w ) );
