@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id:  qt/qtextbrowser.cpp   3.0.5   edited May 3 10:54 $
+** $Id: qt/qtextbrowser.cpp   3.2.3   edited May 13 09:08 $
 **
 ** Implementation of the QTextBrowser class
 **
@@ -57,42 +57,49 @@
 #include "qcursor.h"
 
 /*!
-  \class QTextBrowser qtextbrowser.h
-  \brief The QTextBrowser class provides a rich text browser with hypertext navigation.
-  \ingroup advanced
-  \ingroup helpsystem
-  \ingroup text
-  \mainclass
+    \class QTextBrowser qtextbrowser.h
+    \brief The QTextBrowser class provides a rich text browser with hypertext navigation.
 
-  This class extends QTextEdit (in read-only mode), adding some
-  navigation functionality so that users can follow links in hypertext
-  documents. The contents of QTextEdit is set with setText(), but
-  QTextBrowser has an additional function, setSource(), which makes it
-  possible to set the text to a named document. The name is looked up
-  in the text view's mime source factory. If a document name ends with
-  an anchor (for example, "\c #anchor"), the text browser
-  automatically scrolls to that position (using
-  scrollToAnchor()). When the user clicks on a hyperlink, the browser
-  will call setSource() itself, with the link's \c href value as
-  argument.
+    \ingroup advanced
+    \ingroup helpsystem
+    \ingroup text
+    \mainclass
 
-  QTextBrowser provides backward() and forward() slots which you can use
-  to implement Back and Forward buttons. The home() slot sets the text
-  to the very first document displayed. The linkClicked() signal is
-  emitted when the user clicks a link.
+    This class extends QTextEdit (in read-only mode), adding some
+    navigation functionality so that users can follow links in
+    hypertext documents. The contents of QTextEdit is set with
+    setText(), but QTextBrowser has an additional function,
+    setSource(), which makes it possible to set the text to a named
+    document. The name is looked up in the text view's mime source
+    factory. If a document name ends with an anchor (for example, "\c
+    #anchor"), the text browser automatically scrolls to that position
+    (using scrollToAnchor()). When the user clicks on a hyperlink, the
+    browser will call setSource() itself, with the link's \c href
+    value as argument. You can track the current source by connetion
+    to the sourceChanged() signal.
 
-  By using QTextEdit::setMimeSourceFactory() you can provide your own
-  subclass of QMimeSourceFactory. This makes it possible to access
-  data from anywhere, for example from a network or from a
-  database. See QMimeSourceFactory::data() for details.
+    QTextBrowser provides backward() and forward() slots which you can
+    use to implement Back and Forward buttons. The home() slot sets
+    the text to the very first document displayed. The linkClicked()
+    signal is emitted when the user clicks a link.
 
-  If you intend using the mime factory to read the data directly from
-  the file system, you may have to specify the encoding for the file
-  extension you are using. For example:
-  \code
-  mimeSourceFactory()->setExtensionType("qml", "text/utf8");
-  \endcode
-  This is to ensure that the factory is able to resolve the document names.
+    By using QTextEdit::setMimeSourceFactory() you can provide your
+    own subclass of QMimeSourceFactory. This makes it possible to
+    access data from anywhere, for example from a network or from a
+    database. See QMimeSourceFactory::data() for details.
+
+    If you intend using the mime factory to read the data directly
+    from the file system, you may have to specify the encoding for the
+    file extension you are using. For example:
+    \code
+    mimeSourceFactory()->setExtensionType("qml", "text/utf8");
+    \endcode
+    This is to ensure that the factory is able to resolve the document
+    names.
+
+    QTextBrowser interprets the tags it processes in accordance with
+    the default style sheet. Change the style sheet with
+    \l{setStyleSheet()}; see QStyleSheet for details.
 
     If you want to provide your users with editable rich text use
     QTextEdit. If you want a text browser without hypertext navigation
@@ -100,24 +107,30 @@
     editing. If you just need to display a small piece of rich text
     use QSimpleRichText or QLabel.
 
-  <img src=qtextbrowser-m.png> <img src=qtextbrowser-w.png>
+    <img src=qtextbrowser-m.png> <img src=qtextbrowser-w.png>
 */
 
 class QTextBrowserData
 {
 public:
-    QTextBrowserData() {}
+    QTextBrowserData():textOrSourceChanged(FALSE) {}
 
     QValueStack<QString> stack;
     QValueStack<QString> forwardStack;
     QString home;
     QString curmain;
     QString curmark;
+    
+    /*flag necessary to give the linkClicked() signal some meaningful
+      semantics when somebody connected to it calls setText() or
+      setSource() */
+    bool textOrSourceChanged;
 };
 
 
-/*!  Constructs an empty QTextBrowser with parent \a parent called \a
-    name.
+/*!
+    Constructs an empty QTextBrowser called \a name, with parent \a
+    parent.
 */
 QTextBrowser::QTextBrowser(QWidget *parent, const char *name)
     : QTextEdit( parent, name )
@@ -126,11 +139,10 @@ QTextBrowser::QTextBrowser(QWidget *parent, const char *name)
     d = new QTextBrowserData;
 
     viewport()->setMouseTracking( TRUE );
-    connect( this, SIGNAL( linkClicked( const QString & ) ),
-	     this, SLOT( setSource( const QString & ) ) );
 }
 
-/*!  \reimp
+/*!
+    \reimp
 */
 QTextBrowser::~QTextBrowser()
 {
@@ -139,28 +151,28 @@ QTextBrowser::~QTextBrowser()
 
 
 /*!
-  \property QTextBrowser::source
-  \brief the name of the currently displayed document.
+    \property QTextBrowser::source
+    \brief the name of the displayed document.
 
-  This is a null string if no document is displayed or
-  the source is unknown.
+    This is a QString::null if no document is displayed or if the
+    source is unknown.
 
-  Setting this property uses the mimeSourceFactory to lookup the
-  named document. It also checks for optional anchors and scrolls the
-  document accordingly.
+    Setting this property uses the mimeSourceFactory() to lookup the
+    named document. It also checks for optional anchors and scrolls
+    the document accordingly.
 
-  If the first tag in the document is \c{<qt type=detail>}, the document
-  is displayed as a popup rather than as new document in the browser
-  window itself. Otherwise, the document is displayed normally in the
-  text browser with the text set to the contents of the named document
-  with setText().
+    If the first tag in the document is \c{<qt type=detail>}, the
+    document is displayed as a popup rather than as new document in
+    the browser window itself. Otherwise, the document is displayed
+    normally in the text browser with the text set to the contents of
+    the named document with setText().
 
-  If you are using the filesystem access capabilities of the mime
-  source factory, you must ensure that the factory knows about the
-  encoding of specified files; otherwise no data will be
-  available. The default factory handles a couple of common file
-  extensions such as \c *.html and \c *.txt with reasonable defaults. See
-  QMimeSourceFactory::data() for details.
+    If you are using the filesystem access capabilities of the mime
+    source factory, you must ensure that the factory knows about the
+    encoding of specified files; otherwise no data will be available.
+    The default factory handles a couple of common file extensions
+    such as \c *.html and \c *.txt with reasonable defaults. See
+    QMimeSourceFactory::data() for details.
 */
 
 QString QTextBrowser::source() const
@@ -171,7 +183,36 @@ QString QTextBrowser::source() const
 	return d->stack.top();
 }
 
-/*! Reloads the current set source */
+/*!
+    \property QTextBrowser::undoDepth
+    \brief This text browser's undo depth.
+*/
+
+/*!
+    \property QTextBrowser::overwriteMode
+    \brief This text browser's overwrite mode.
+*/
+
+/*!
+    \property QTextBrowser::modified
+    \brief Whether the contents have been modified.
+*/
+
+/*!
+    \property QTextBrowser::readOnly
+    \brief Whether the contents are read only.
+*/
+
+/*!
+    \property QTextBrowser::undoRedoEnabled
+    \brief Whether undo and redo are enabled.
+*/
+
+
+
+/*!
+    Reloads the current set source.
+*/
 
 void QTextBrowser::reload()
 {
@@ -187,6 +228,7 @@ void QTextBrowser::setSource(const QString& name)
     if ( isVisible() )
 	qApp->setOverrideCursor( waitCursor );
 #endif
+    d->textOrSourceChanged = TRUE;
     QString source = name;
     QString mark;
     int hash = name.find('#');
@@ -237,12 +279,11 @@ void QTextBrowser::setSource(const QString& name)
     if ( !d->home )
 	d->home = url;
 
-    if ( d->stack.isEmpty() || d->stack.top() != url) {
+    if ( d->stack.isEmpty() || d->stack.top() != url)
 	d->stack.push( url );
-    }
 
     int stackCount = (int)d->stack.count();
-    if ( d->stack.isEmpty() || d->stack.top() == url )
+    if ( d->stack.top() == url )
 	stackCount--;
     emit backwardAvailable( stackCount > 0 );
     stackCount = (int)d->forwardStack.count();
@@ -262,27 +303,42 @@ void QTextBrowser::setSource(const QString& name)
     if ( isVisible() )
 	qApp->restoreOverrideCursor();
 #endif
+    
+    emit sourceChanged( url );
 }
 
-/*!  \fn void QTextBrowser::backwardAvailable(bool available)
+/*!
+    \fn void QTextBrowser::backwardAvailable(bool available)
 
-  This signal is emitted when the availability of the backward()
-  changes. \a available is FALSE when the user is at home(); otherwise
-  it is TRUE.
+    This signal is emitted when the availability of backward()
+    changes. \a available is FALSE when the user is at home();
+    otherwise it is TRUE.
 */
 
-/*!  \fn void QTextBrowser::forwardAvailable(bool available)
+/*!
+    \fn void QTextBrowser::forwardAvailable(bool available)
 
-  This signal is emitted when the availability of the forward() changes.
-  \a available is TRUE after the user navigates backward() and
-  FALSE when the user navigates or goes forward().
+    This signal is emitted when the availability of forward() changes.
+    \a available is TRUE after the user navigates backward() and FALSE
+    when the user navigates or goes forward().
 */
 
-/*!  \fn void QTextBrowser::highlighted (const QString &href)
+/*!
+    \fn void QTextBrowser::sourceChanged( const QString& src)
 
-  This signal is emitted when the user has selected but not activated
-  a link in the document.  \a href is the value of the href tag in the
-  link.
+    This signal is emitted when the mime source has changed, \a src
+    being the new source.
+    
+    Source changes happen both programmatically when calling
+    setSource(), forward(), backword() or home() or when the user
+    clicks on links or presses the equivalent key sequences.
+*/
+
+/*!  \fn void QTextBrowser::highlighted (const QString &link)
+
+    This signal is emitted when the user has selected but not
+    activated a link in the document. \a link is the value of the \c
+    href i.e. the name of the target document.
 */
 
 /*!
@@ -291,14 +347,29 @@ void QTextBrowser::setSource(const QString& name)
     This signal is emitted when the user clicks a link. The \a link is
     the value of the \c href i.e. the name of the target document.
 
+    The \a link will be the absolute location of the document, based
+    on the value of the anchor's href tag and the current context of
+    the document.
+
+    \sa anchorClicked(), context()
 */
 
+/*!
+    \fn void QTextBrowser::anchorClicked( const QString& name, const QString &link)
 
-/*!  Changes the document displayed to the previous document in the
-  list of documents built by navigating links. Does nothing if there is
-  no previous document.
+    This signal is emitted when the user clicks a an anchor. The \a link is
+    the value of the \c href i.e. the name of the target document.  The \a name
+    is the name of the anchor.
 
-  \sa forward(), backwardAvailable()
+    \sa linkClicked()
+*/
+
+/*!
+    Changes the document displayed to the previous document in the
+    list of documents built by navigating links. Does nothing if there
+    is no previous document.
+
+    \sa forward(), backwardAvailable()
 */
 void QTextBrowser::backward()
 {
@@ -309,11 +380,12 @@ void QTextBrowser::backward()
     emit forwardAvailable( TRUE );
 }
 
-/*!  Changes the document displayed to the next document in the list of
-  documents built by navigating links. Does nothing if there is no next
-  document.
+/*!
+    Changes the document displayed to the next document in the list of
+    documents built by navigating links. Does nothing if there is no
+    next document.
 
-  \sa backward(), forwardAvailable()
+    \sa backward(), forwardAvailable()
 */
 void QTextBrowser::forward()
 {
@@ -323,8 +395,9 @@ void QTextBrowser::forward()
     emit forwardAvailable( !d->forwardStack.isEmpty() );
 }
 
-/*!  Changes the document displayed to be the first document the
-  browser displayed.
+/*!
+    Changes the document displayed to be the first document the
+    browser displayed.
 */
 void QTextBrowser::home()
 {
@@ -334,11 +407,12 @@ void QTextBrowser::home()
 
 /*!
     The event \a e is used to provide the following keyboard shortcuts:
-    <ul>
-    <li><i>Alt+Left Arrow</i> - backward()
-    <li><i>Alt+Right Arrow</i> - forward()
-    <li><i>Alt+Up Arrow</i> - home()
-    </ul>
+    \table
+    \header \i Keypress	    \i Action
+    \row \i Alt+Left Arrow  \i \l backward()
+    \row \i Alt+Right Arrow \i \l forward()
+    \row \i Alt+Up Arrow    \i \l home()
+    \endtable
 */
 void QTextBrowser::keyPressEvent( QKeyEvent * e )
 {
@@ -443,17 +517,36 @@ void QTextBrowser::popupDetail( const QString& contents, const QPoint& pos )
 }
 
 /*!
-    \overload void QTextBrowser::setText( const QString &txt )
+    \fn void QTextBrowser::setText( const QString &txt )
+
+    \overload
+
     Sets the text to \a txt.
 */
 
-/*! \reimp */
+/*!
+    \reimp
+*/
 
 void QTextBrowser::setText( const QString &txt, const QString &context )
 {
+    d->textOrSourceChanged = TRUE;
     d->curmark = "";
     d->curmain = "";
     QTextEdit::setText( txt, context );
 }
 
+void QTextBrowser::emitHighlighted( const QString &s ) 
+{ 
+    emit highlighted( s ); 
+}
+
+void QTextBrowser::emitLinkClicked( const QString &s ) 
+{ 
+    d->textOrSourceChanged = FALSE;
+    emit linkClicked( s );
+    if ( !d->textOrSourceChanged )
+	setSource( s );
+}
+
 #endif  // QT_NO_TEXTBROWSER
