@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id:  qt/qtitlebar.cpp   3.0.5   edited Jun 3 15:38 $
+** $Id: qt/qtitlebar.cpp   3.2.3   edited May 24 15:31 $
 **
 ** Implementation of some Qt private functions.
 **
 ** Created : 001101
 **
-** Copyright (C) 2000-2001 Trolltech AS.  All rights reserved.
+** Copyright (C) 2000-2003 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the widgets module of the Qt GUI Toolkit.
 **
@@ -35,8 +35,8 @@
 **
 **********************************************************************/
 
-// Get the system specific includes and defines
 #include "qplatformdefs.h"
+
 #include "qtitlebar_p.h"
 
 #ifndef QT_NO_TITLEBAR
@@ -67,7 +67,7 @@ public:
 
     void maybeTip( const QPoint &pos )
     {
-	if ( !parentWidget()->inherits( "QTitleBar" ) )
+	if ( !::qt_cast<QTitleBar*>(parentWidget()) )
 	    return;
 	QTitleBar *t = (QTitleBar *)parentWidget();
 
@@ -146,10 +146,13 @@ public:
     bool pressed            :1;
     bool autoraise          :1;
     QString cuttext;
+#ifdef QT_NO_WIDGET_TOPEXTRA
+    QString cap;
+#endif
 };
 
 QTitleBar::QTitleBar(QWidget* w, QWidget* parent, const char* name)
-    : QWidget( parent, name, WStyle_Customize | WStyle_NoBorder | WResizeNoErase | WRepaintNoErase )
+    : QWidget( parent, name, WStyle_Customize | WStyle_NoBorder | WNoAutoErase )
 {
     d = new QTitleBarPrivate();
 
@@ -160,12 +163,14 @@ QTitleBar::QTitleBar(QWidget* w, QWidget* parent, const char* name)
     d->buttonDown = QStyle::SC_None;
     d->act = 0;
     if ( w ) {
-	setWFlags( ((QTitleBar*)w)->getWFlags() | WResizeNoErase | WRepaintNoErase );
+	setWFlags( ((QTitleBar*)w)->getWFlags() | WNoAutoErase );
+	if ( w->minimumSize() == w->maximumSize() )
+	    clearWFlags( WStyle_Maximize );
 #ifndef QT_NO_WIDGET_TOPEXTRA
     	setCaption( w->caption() );
 #endif
     } else {
-	setWFlags( WStyle_Customize | WResizeNoErase | WRepaintNoErase );
+	setWFlags( WStyle_Customize | WNoAutoErase );
     }
 
     readColors();
@@ -191,6 +196,8 @@ void QTitleBar::readColors()
 {
     QPalette pal = palette();
 
+    bool colorsInitialized = FALSE;
+
 #ifdef Q_WS_WIN // ask system properties on windows
 #ifndef SPI_GETGRADIENTCAPTIONS
 #define SPI_GETGRADIENTCAPTIONS 0x1008
@@ -207,17 +214,13 @@ void QTitleBar::readColors()
 	pal.setColor( QPalette::Active, QColorGroup::HighlightedText, qt_colorref2qrgb(GetSysColor(COLOR_CAPTIONTEXT)) );
 	pal.setColor( QPalette::Inactive, QColorGroup::HighlightedText, qt_colorref2qrgb(GetSysColor(COLOR_INACTIVECAPTIONTEXT)) );
 	if ( qt_winver != Qt::WV_95 && qt_winver != WV_NT ) {
+	    colorsInitialized = TRUE;
 	    BOOL gradient;
-#ifdef Q_OS_TEMP
-	    SystemParametersInfo( SPI_GETGRADIENTCAPTIONS, 0, &gradient, 0 );
-#else
-#if defined(UNICODE)
-	    if ( qt_winver & Qt::WV_NT_based )
+	    QT_WA( {
 		SystemParametersInfo( SPI_GETGRADIENTCAPTIONS, 0, &gradient, 0 );
-	    else
-#endif
+	    } , {
 		SystemParametersInfoA( SPI_GETGRADIENTCAPTIONS, 0, &gradient, 0 );
-#endif
+	    } );
 	    if ( gradient ) {
 		pal.setColor( QPalette::Active, QColorGroup::Base, qt_colorref2qrgb(GetSysColor(COLOR_GRADIENTACTIVECAPTION)) );
 		pal.setColor( QPalette::Inactive, QColorGroup::Base, qt_colorref2qrgb(GetSysColor(COLOR_GRADIENTINACTIVECAPTION)) );
@@ -226,9 +229,9 @@ void QTitleBar::readColors()
 		pal.setColor( QPalette::Inactive, QColorGroup::Base, palette().inactive().highlight() );
 	    }
 	}
-    } else
+    }
 #endif // Q_WS_WIN
-    {
+    if ( !colorsInitialized ) {
 	pal.setColor( QPalette::Active, QColorGroup::Highlight, palette().active().highlight() );
 	pal.setColor( QPalette::Active, QColorGroup::Base, palette().active().highlight() );
 	pal.setColor( QPalette::Inactive, QColorGroup::Highlight, palette().inactive().dark() );
@@ -242,7 +245,8 @@ void QTitleBar::readColors()
 
 void QTitleBar::mousePressEvent( QMouseEvent * e)
 {
-    emit doActivate();
+    if ( !d->act )
+	emit doActivate();
     if ( e->button() == LeftButton ) {
 	d->pressed = TRUE;
 	QStyle::SCFlags ctrl = style().querySubControl(QStyle::CC_TitleBar, this, e->pos());
@@ -296,7 +300,7 @@ void QTitleBar::mousePressEvent( QMouseEvent * e)
 	    d->buttonDown = ctrl;
 	    d->moveOffset = mapToParent( e->pos() );
 	    break;
-	
+
 	default:
 	    break;
 	}
@@ -309,10 +313,10 @@ void QTitleBar::mousePressEvent( QMouseEvent * e)
 void QTitleBar::contextMenuEvent( QContextMenuEvent *e )
 {
     QStyle::SCFlags ctrl = style().querySubControl(QStyle::CC_TitleBar, this, e->pos());
-    if( ctrl == QStyle::SC_TitleBarLabel || ctrl == QStyle::SC_TitleBarSysMenu ) {
+    if( ctrl == QStyle::SC_TitleBarLabel || ctrl == QStyle::SC_TitleBarSysMenu )
 	emit popupOperationMenu(e->globalPos());
-	e->accept();
-    }
+    else
+	e->ignore();
 }
 
 void QTitleBar::mouseReleaseEvent( QMouseEvent * e)
@@ -327,7 +331,7 @@ void QTitleBar::mouseReleaseEvent( QMouseEvent * e)
 		if( testWFlags( WStyle_MinMax ) && testWFlags( WStyle_Tool ) )
 		    emit doShade();
 		break;
-	
+
 	    case QStyle::SC_TitleBarNormalButton:
 		if( testWFlags( WStyle_MinMax ) && !testWFlags( WStyle_Tool ) )
 		    emit doNormal();
@@ -351,7 +355,7 @@ void QTitleBar::mouseReleaseEvent( QMouseEvent * e)
 		if( testWFlags( WStyle_SysMenu ) ) {
 		    d->buttonDown = QStyle::SC_None;
 		    repaint(FALSE);
-		    emit doClose();		
+		    emit doClose();
 		    return;
 		}
 		break;
@@ -397,9 +401,8 @@ void QTitleBar::mouseMoveEvent( QMouseEvent * e)
 		QPoint p = mapFromGlobal(e->globalPos());
 #ifndef QT_NO_WORKSPACE
 		if(d->window && d->window->parentWidget()->inherits("QWorkspaceChild")) {
-		    QWidget *w = d->window->parentWidget()->parentWidget();
-		    if(w && w->inherits("QWorkspace")) {
-			QWorkspace *workspace = (QWorkspace*)w;
+		    QWorkspace *workspace = ::qt_cast<QWorkspace*>(d->window->parentWidget()->parentWidget());
+		    if(workspace) {
 			p = workspace->mapFromGlobal( e->globalPos() );
 			if ( !workspace->rect().contains(p) ) {
 			    if ( p.x() < 0 )
@@ -465,8 +468,7 @@ void QTitleBar::paintEvent(QPaintEvent *)
 	ctrls ^= under_mouse;
     }
 
-    QSharedDoubleBuffer buffer( (bool)FALSE, (bool)FALSE );
-    buffer.begin( this, rect() );
+    QSharedDoubleBuffer buffer( this, rect() );
     style().drawComplexControl(QStyle::CC_TitleBar, buffer.painter(), this, rect(),
 			       colorGroup(),
 			       isEnabled() ? QStyle::Style_Enabled :
@@ -499,9 +501,16 @@ void QTitleBar::mouseDoubleClickEvent( QMouseEvent *e )
     }
 }
 
+#ifdef QT_NO_WIDGET_TOPEXTRA
+// We provide one, since titlebar is useless otherwise.
+QString QTitleBar::caption() const
+{
+    return d->cap;
+}
+#endif
+
 void QTitleBar::cutText()
 {
-#ifndef QT_NO_WIDGET_TOPEXTRA
     QFontMetrics fm( font() );
 
     int maxw = style().querySubControlMetrics(QStyle::CC_TitleBar, this,
@@ -518,19 +527,20 @@ void QTitleBar::cutText()
 	if(i != (int)txt.length())
 	    d->cuttext = txt.left( i ) + "...";
     }
-#endif
 }
 
 void QTitleBar::setCaption( const QString& title )
 {
-#ifndef QT_NO_WIDGET_TOPEXTRA
     if( caption() == title)
 	return;
+#ifndef QT_NO_WIDGET_TOPEXTRA
     QWidget::setCaption( title );
+#else
+    d->cap = title;
+#endif
     cutText();
 
     update();
-#endif
 }
 
 
