@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id:  qt/qtoolbar.cpp   3.0.5   edited May 13 18:33 $
+** $Id: qt/qtoolbar.cpp   3.2.3   edited Jul 4 11:53 $
 **
 ** Implementation of QToolBar class
 **
 ** Created : 980315
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2002 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the widgets module of the Qt GUI Toolkit.
 **
@@ -45,11 +45,11 @@
 #include "qlayout.h"
 #include "qframe.h"
 #include "qobjectlist.h"
-#include "qintdict.h"
 #include "qpainter.h"
 #include "qdrawutil.h"
 #include "qtoolbutton.h"
 #include "qpopupmenu.h"
+#include "qcombobox.h"
 #include "qtimer.h"
 #include "qwidgetlist.h"
 #include "qstyle.h"
@@ -87,15 +87,14 @@ class QToolBarExtensionWidget;
 class QToolBarPrivate
 {
 public:
-    QToolBarPrivate() : moving( FALSE ), firstShow( TRUE ), button( 0 ) {}
+    QToolBarPrivate() : moving( FALSE ) {
+	extensionSubMenues.setAutoDelete(TRUE);
+    }
 
     bool moving;
-    bool firstShow;
     QToolBarExtensionWidget *extension;
     QPopupMenu *extensionPopup;
-    QIntDict<QButton> hiddenItems;
-    QButton *button;
-
+    QObjectList extensionSubMenues;
 };
 
 
@@ -211,81 +210,84 @@ void QToolBarSeparator::paintEvent( QPaintEvent * )
 #include "qtoolbar.moc"
 
 
-/*! \class QToolBar qtoolbar.h
+/*!
+    \class QToolBar qtoolbar.h
+    \brief The QToolBar class provides a movable panel containing
+    widgets such as tool buttons.
 
-  \brief The QToolBar class provides a movable panel containing
-  widgets such as tool buttons.
+    \ingroup application
+    \mainclass
 
-  \ingroup application
-  \mainclass
+    A toolbar is a panel that contains a set of controls, usually
+    represented by small icons. It's purpose is to provide quick
+    access to frequently used commands or options. Within a
+    QMainWindow the user can drag toolbars within and between the
+    \link QDockArea dock areas\endlink. Toolbars can also be dragged
+    out of any dock area to float freely as top-level windows.
 
-  A toolbar is a panel that contains a set of controls, usually
-  represented by small icons.  It's purpose is to provide quick access
-  to frequently used commands or options. Within a QMainWindow the user
-  can drag toolbars within and between the dock areas. Toolbars can also
-  be dragged out of any dock area to float freely as top level windows.
+    QToolBar is a specialization of QDockWindow, and so provides all
+    the functionality of a QDockWindow.
 
-  QToolBar is a specialization of QDockWindow, and so provides
-  all the functionality of a QDockWindow.
+    To use QToolBar you simply create a QToolBar as a child of a
+    QMainWindow, create a number of QToolButton widgets (or other
+    widgets) in left to right (or top to bottom) order and call
+    addSeparator() when you want a separator. When a toolbar is
+    floated the caption used is the label given in the constructor
+    call. This can be changed with setLabel().
 
-  To use QToolBar you simply create a QToolBar as a child of a
-  QMainWindow, create a number of QToolButton widgets (or other widgets)
-  in left to right (or top to bottom) order and call addSeparator() when
-  you want a separator. When a toolbar is floated the caption used is
-  the label given in the constructor call. This can be changed with
-  setLabel().
+    \quotefile action/application.cpp
+    \skipto new QToolBar
+    \printuntil fileSaveAction
 
-  \quotefile action/application.cpp
-  \skipto new QToolBar
-  \printuntil fileSaveAction
+    This extract from the \l application/application.cpp example shows
+    the creation of a new toolbar as a child of a QMainWindow and
+    adding two QActions.
 
-  This extract from the \l application/application.cpp example shows the
-  creation of a new toolbar as a child of a QMainWindow and adding two
-  QActions.
+    You may use most widgets within a toolbar, with QToolButton and
+    QComboBox being the most common.
 
-  You may use most widgets within a toolbar, with QToolButton
-  and QComboBox being the most common.
+    If you create a new widget on an already visible QToolBar, this
+    widget will automatically become visible without needing a show()
+    call. (This differs from every other Qt widget container. We
+    recommend calling show() anyway since we hope to fix this anomaly
+    in a future release.)
 
-  Note that if you create a new widget on a already visible QToolBar,
-  this widget becomes automatically visible without explicitly calling
-  show() on it (unlike in all other widget containers in Qt). Please
-  call explicitly show() on all widgets you put into an already
-  visible QToolBar anyway, as this behavior might be fixed in the
-  future.
+    QToolBars, like QDockWindows, are located in \l{QDockArea}s or
+    float as top-level windows. QMainWindow provides four QDockAreas
+    (top, left, right and bottom). When you create a new toolbar (as
+    in the example above) as a child of a QMainWindow the toolbar will
+    be added to the top dock area. You can move it to another dock
+    area (or float it) by calling QMainWindow::moveDockWindow(). QDock
+    areas lay out their windows in \link qdockarea.html#lines
+    Lines\endlink.
 
-  QToolBars, like QDockWindows, are located in QDockAreas or float as
-  top level windows. QMainWindow provides four QDockAreas (top, left,
-  right and bottom). When you create a new toolbar (as in the example
-  above) as a child of a QMainWindow the toolbar will be added to the
-  top dock area. You can move it to another dock area (or float it) by
-  calling QMainWindow::moveDockWindow(). QDock areas lay out their
-  windows in \link qdockarea.html#lines Lines\endlink.
+    If the main window is resized so that the area occupied by the
+    toolbar is too small to show all its widgets a little arrow button
+    (which looks like a right-pointing chevron, '&#187;') will appear
+    at the right or bottom of the toolbar depending on its
+    orientation. Clicking this button pops up a menu that shows the
+    'overflowing' items. QToolButtons are represented in the menu using
+    their textLabel property, other QButton subclasses are represented
+    using their text property, and QComboBoxes are represented as submenus,
+    with the caption text being used in the submenu item.
 
-  If the main window is resized so that the area occupied by the toolbar
-  is too small to show all its widgets a little arrow button (which
-  looks like a right-pointing chevron, '&#187;') will appear at the right or
-  bottom of the toolbar depending on its orientation. Clicking this
-  button pops up a menu that shows the 'overflowing' items.
+    Usually a toolbar will get precisely the space it needs. However,
+    with setHorizontalStretchable(), setVerticalStretchable() or
+    setStretchableWidget() you can tell the main window to expand the
+    toolbar to fill all available space in the specified orientation.
 
-  Usually a toolbar will get precisely the space it needs. However, with
-  setHorizontalStretchable(), setVerticalStretchable() or
-  setStretchableWidget() you can tell the main window to expand the
-  toolbar to fill all available space in the specified orientation.
+    The toolbar arranges its buttons either horizontally or vertically
+    (see orientation() for details). Generally, QDockArea will set the
+    orientation correctly for you, but you can set it yourself with
+    setOrientation() and track any changes by connecting to the
+    orientationChanged() signal.
 
-  The toolbar arranges its buttons either horizontally or vertically (see
-  orientation() for details). Generally, QDockArea will set the
-  orientation correctly for you, but you can set it yourself with
-  setOrientation() and track any changes by connecting to the
-  orientationChanged() signal.
+    You can use the clear() method to remove all items from a toolbar.
 
-  You can use the clear() method to remove all items from a toolbar.
+    \img qdockwindow.png Toolbar (dock window)
+    \caption A floating QToolbar (dock window)
 
-  \img qdockwindow.png Toolbar (dock window)
-  \caption A floating QToolbar (dock window)
-
-  \sa QToolButton QMainWindow
-  \link http://www.iarchitect.com/visual.htm Parts of Isys on Visual Design\endlink
-  \link guibooks.html#fowler GUI Design Handbook: Tool Bar\endlink.
+    \sa QToolButton QMainWindow \link http://www.iarchitect.com/visual.htm Parts of Isys on Visual Design\endlink \link guibooks.html#fowler GUI Design Handbook: Tool Bar\endlink.
 */
 
 /*!
@@ -295,18 +297,19 @@ void QToolBarSeparator::paintEvent( QPaintEvent * )
     \obsolete
 */
 
-/*!  Constructs an empty toolbar.
+/*!
+    Constructs an empty toolbar.
 
-    The toolbar is a child of \a parent and is managed by \a parent. It
-    is initially located in dock area \a dock and is labeled \a label.
-    If \a newLine is TRUE the toolbar will be placed on a new line in
-    the dock area.
+    The toolbar is called \a name and is a child of \a parent and is
+    managed by \a parent. It is initially located in dock area \a dock
+    and is labeled \a label. If \a newLine is TRUE the toolbar will be
+    placed on a new line in the dock area.
 */
 
 QToolBar::QToolBar( const QString &label,
 		    QMainWindow * parent, QMainWindow::ToolBarDock dock,
 		    bool newLine, const char * name )
-    : QDockWindow( InDock, parent, name )
+    : QDockWindow( InDock, parent, name, 0, TRUE )
 {
     mw = parent;
     init();
@@ -317,22 +320,22 @@ QToolBar::QToolBar( const QString &label,
 
 
 /*!
-
     Constructs an empty horizontal toolbar.
 
-    The toolbar is a child of \a parent and is managed by \a mainWindow.
-    The \a label and \a newLine parameters are passed straight to
-    QMainWindow::addDockWindow(). \a name is the object name and \a f is
-    the widget flags.
+    The toolbar is called \a name and is a child of \a parent and is
+    managed by \a mainWindow. The \a label and \a newLine parameters
+    are passed straight to QMainWindow::addDockWindow(). \a name and
+    the widget flags \a f are passed on to the QDockWindow constructor.
 
-  Use this constructor if you want to create torn-off (undocked,
-  floating) toolbars or toolbars in the status bar.
+    Use this constructor if you want to create torn-off (undocked,
+    floating) toolbars or toolbars in the \link QStatusBar status
+    bar\endlink.
 */
 
 QToolBar::QToolBar( const QString &label, QMainWindow * mainWindow,
 		    QWidget * parent, bool newLine, const char * name,
 		    WFlags f )
-    : QDockWindow( InDock, parent, name, f )
+    : QDockWindow( InDock, parent, name, f, TRUE )
 {
     mw = mainWindow;
     init();
@@ -348,13 +351,13 @@ QToolBar::QToolBar( const QString &label, QMainWindow * mainWindow,
 /*!
     \overload
 
-    Constructs an empty toolbar, with parent \a parent and name \a name,
+    Constructs an empty toolbar called \a name, with parent \a parent,
     in its \a parent's top dock area, without any label and without
     requiring a newline.
 */
 
 QToolBar::QToolBar( QMainWindow * parent, const char * name )
-    : QDockWindow( InDock, parent, name )
+    : QDockWindow( InDock, parent, name, 0, TRUE )
 {
     mw = parent;
     init();
@@ -375,24 +378,20 @@ void QToolBar::init()
     d->extension = new QToolBarExtensionWidget( this );
     d->extension->hide();
     d->extensionPopup = new QPopupMenu( this, "qt_dockwidget_internal" );
-    connect( d->extensionPopup, SIGNAL( activated( int ) ),
-	     this, SLOT( popupSelected( int ) ) );
     connect( d->extensionPopup, SIGNAL( aboutToShow() ),
-	     this, SLOT( setupArrowMenu() ) );
+	     this, SLOT( createPopup() ) );
     d->extension->button()->setPopup( d->extensionPopup );
     d->extension->button()->setPopupDelay( -1 );
     sw = 0;
 
-#if defined(QT_CHECK_NULL)
-    if ( !mw )
-	qWarning( "QToolBar::QToolBar main window cannot be 0." );
-#endif
     setBackgroundMode( PaletteButton);
     setFocusPolicy( NoFocus );
     setFrameStyle( QFrame::ToolBarPanel | QFrame::Raised);
 }
 
-/*! \reimp */
+/*!
+    \reimp
+*/
 
 QToolBar::~QToolBar()
 {
@@ -400,7 +399,9 @@ QToolBar::~QToolBar()
     d = 0;
 }
 
-/*! \reimp */
+/*!
+    \reimp
+*/
 
 void QToolBar::setOrientation( Orientation o )
 {
@@ -417,14 +418,17 @@ void QToolBar::setOrientation( Orientation o )
     delete childs;
 }
 
-/*!  Adds a separator to the right/bottom of the toolbar. */
+/*!
+    Adds a separator to the right/bottom of the toolbar.
+*/
 
 void QToolBar::addSeparator()
 {
     (void) new QToolBarSeparator( orientation(), this, "toolbar separator" );
 }
 
-/*!\reimp
+/*!
+    \reimp
 */
 
 void QToolBar::styleChange( QStyle& )
@@ -433,30 +437,29 @@ void QToolBar::styleChange( QStyle& )
     if ( childs ) {
         QObject *ob = 0;
 	for ( ob = childs->first(); ob; ob = childs->next() ) {
-            if ( ob->inherits( "QToolButton" ) ) {
-                QToolButton* w = (QToolButton*)ob;
+	    QWidget *w = (QWidget*)ob;
+            if ( ::qt_cast<QToolButton*>(w) || ::qt_cast<QToolBarSeparator*>(w) )
                 w->setStyle( &style() );
-            } else if ( ob->inherits( "QToolBarSeparator" ) ) {
-                QToolBarSeparator* w = (QToolBarSeparator*)ob;
-                w->setStyle( &style() );
-            }
         }
     }
     delete childs;
 }
 
-/*!  \reimp. */
+/*!
+    \reimp.
+*/
 
 void QToolBar::show()
 {
     QDockWindow::show();
     if ( mw )
 	mw->triggerLayout( FALSE );
+    checkForExtension( size() );
 }
 
 
 /*!
-  \reimp
+    \reimp
 */
 
 void QToolBar::hide()
@@ -466,7 +469,8 @@ void QToolBar::hide()
 	mw->triggerLayout( FALSE );
 }
 
-/*!  Returns a pointer to the QMainWindow which manages this toolbar.
+/*!
+    Returns a pointer to the QMainWindow which manages this toolbar.
 */
 
 QMainWindow * QToolBar::mainWindow() const
@@ -475,18 +479,20 @@ QMainWindow * QToolBar::mainWindow() const
 }
 
 
-/*! Sets the widget \a w to be expanded if this toolbar is requested to
- stretch.
+/*!
+    Sets the widget \a w to be expanded if this toolbar is requested
+    to stretch.
 
     The request to stretch might occur because QMainWindow
-    right-justifies the dock it's in, or because this toolbar's
-    isVerticalStretchable() or isHorizontalStretchable() is set to TRUE.
+    right-justifies the dock area the toolbar is in, or because this
+    toolbar's isVerticalStretchable() or isHorizontalStretchable() is
+    set to TRUE.
 
-  If you call this function and the toolbar is not yet
-  stretchable, setStretchable() is called.
+    If you call this function and the toolbar is not yet stretchable,
+    setStretchable() is called.
 
-  \sa QMainWindow::setRightJustification(), setVerticalStretchable(),
-  setHorizontalStretchable()
+    \sa QMainWindow::setRightJustification(), setVerticalStretchable(),
+    setHorizontalStretchable()
 */
 
 void QToolBar::setStretchableWidget( QWidget * w )
@@ -503,29 +509,30 @@ void QToolBar::setStretchableWidget( QWidget * w )
 }
 
 
-/*! \reimp */
+/*!
+    \reimp
+*/
 
 bool QToolBar::event( QEvent * e )
 {
     bool r =  QDockWindow::event( e );
-    //after the event filters have dealt with it:
+    // After the event filters have dealt with it, do our stuff.
     if ( e->type() == QEvent::ChildInserted ) {
 	QObject * child = ((QChildEvent*)e)->child();
-	if ( child && child->isWidgetType() && !((QWidget*)child)->isTopLevel() &&
-	     child->parent() == this && qstrcmp( "qt_dockwidget_internal", child->name() ) != 0 ) {
+	if ( child && child->isWidgetType() && !((QWidget*)child)->isTopLevel()
+	     && child->parent() == this
+	     && qstrcmp("qt_dockwidget_internal", child->name()) != 0 ) {
 	    boxLayout()->addWidget( (QWidget*)child );
-	    if ( isVisibleTo(0) ) {
-		( (QWidget*)child )->show();
-		// We resize the toolbar in case it is too small to show all the icons
-		resize( sizeHint() );
+	    if ( isVisible() ) {
+		if ( ((QWidget*)child)->testWState( WState_CreatedHidden ) )
+		    ((QWidget*)child)->show();
+		checkForExtension( size() );
 	    }
 	}
 	if ( child && child->isWidgetType() && ((QWidget*)child) == sw )
 	    boxLayout()->setStretchFactor( (QWidget*)child, 1 );
-    } else if ( e->type() == QEvent::Show && d->firstShow ) {
-	QWidget::layout()->invalidate();
-	QWidget::layout()->activate();
-	d->firstShow = FALSE;
+    } else if ( e->type() == QEvent::Show ) {
+	layout()->activate();
     } else if ( e->type() == QEvent::LayoutHint && place() == OutsideDock ) {
 	adjustSize();
     }
@@ -534,11 +541,11 @@ bool QToolBar::event( QEvent * e )
 
 
 /*!
-  \property QToolBar::label
-  \brief the label of the toolbar.
+    \property QToolBar::label
+    \brief the toolbar's label.
 
-  If the toolbar is floated the label becomes the toolbar window's
-  caption. There is no default label text.
+    If the toolbar is floated the label becomes the toolbar window's
+    caption. There is no default label text.
 */
 
 void QToolBar::setLabel( const QString & label )
@@ -553,7 +560,9 @@ QString QToolBar::label() const
 }
 
 
-/*! Deletes all the toolbar's child widgets. */
+/*!
+    Deletes all the toolbar's child widgets.
+*/
 
 void QToolBar::clear()
 {
@@ -569,7 +578,9 @@ void QToolBar::clear()
     }
 }
 
-/*!  \reimp */
+/*!
+    \reimp
+*/
 
 QSize QToolBar::minimumSize() const
 {
@@ -579,7 +590,7 @@ QSize QToolBar::minimumSize() const
 }
 
 /*!
-  \reimp
+    \reimp
 */
 
 QSize QToolBar::minimumSizeHint() const
@@ -589,80 +600,131 @@ QSize QToolBar::minimumSizeHint() const
     return QSize( QDockWindow::minimumSizeHint().width(), 0 );
 }
 
+void QToolBar::createPopup()
+{
+    d->extensionPopup->clear();
+    d->extensionSubMenues.clear();
+
+    QObjectList *childlist = queryList( "QWidget", 0, FALSE, TRUE );
+    QObjectListIt it( *childlist );
+    bool hide = FALSE;
+    bool doHide = FALSE;
+    int id;
+    while ( it.current() ) {
+        int j = 2;
+        if ( !it.current()->isWidgetType() || it.current() == d->extension->button() ||
+	    qstrcmp( "qt_dockwidget_internal", it.current()->name() ) == 0 ) {
+	    ++it;
+	    continue;
+	}
+	QWidget *w = (QWidget*)it.current();
+#ifndef QT_NO_COMBOBOX
+	if ( ::qt_cast<QComboBox*>(w) )
+	    j = 1;
+#endif
+	hide = FALSE;
+	QPoint p = w->parentWidget()->mapTo( this, w->geometry().bottomRight() );
+	if ( orientation() == Horizontal ) {
+	    if ( p.x() > ( doHide ? width() - d->extension->width() / j : width() ) )
+	        hide = TRUE;
+	} else {
+	    if ( p.y() > ( doHide ? height()- d->extension->height() / j : height() ) )
+	        hide = TRUE;
+	}
+	if ( hide && w->isVisible() ) {
+	    doHide = TRUE;
+	    if ( ::qt_cast<QToolButton*>(w) ) {
+		QToolButton *b = (QToolButton*)w;
+	        QString s = b->textLabel();
+	        if ( s.isEmpty() )
+		    s = b->text();
+	        if ( b->popup() && b->popupDelay() == 0 )
+		    id = d->extensionPopup->insertItem( b->iconSet(), s, b->popup() );
+		else
+		    id = d->extensionPopup->insertItem( b->iconSet(), s, b, SLOT( emulateClick() ) ) ;
+	        if ( b->isToggleButton() )
+		    d->extensionPopup->setItemChecked( id, b->isOn() );
+		if ( !b->isEnabled() )
+		    d->extensionPopup->setItemEnabled( id, FALSE );
+	    } else if ( ::qt_cast<QButton*>(w) ) {
+		QButton *b = (QButton*)w;
+		QString s = b->text();
+		if ( s.isEmpty() )
+		    s = "";
+		if ( b->pixmap() )
+		    id = d->extensionPopup->insertItem( *b->pixmap(), s, b, SLOT( emulateClick() ) );
+		else
+		    id = d->extensionPopup->insertItem( s, b, SLOT( emulateClick() ) );
+		if ( b->isToggleButton() )
+		    d->extensionPopup->setItemChecked( id, b->isOn() );
+		if ( !b->isEnabled() )
+		    d->extensionPopup->setItemEnabled( id, FALSE );
+#ifndef QT_NO_COMBOBOX
+	    } else if ( ::qt_cast<QComboBox*>(w) ) {
+		QComboBox *c = (QComboBox*)w;
+		if ( c->count() != 0 ) {
+#ifndef QT_NO_WIDGET_TOPEXTRA
+		    QString s = c->caption();
+#else
+		    QString s;
+#endif
+		    if ( s.isEmpty() )
+		        s = c->currentText();
+		    uint maxItems = 0;
+		    QPopupMenu *cp = new QPopupMenu;
+		    d->extensionSubMenues.append( cp );
+		    d->extensionPopup->insertItem( s, cp );
+		    connect( cp, SIGNAL( activated( int ) ),
+		        c, SLOT( internalActivate( int ) ) );
+		    for ( int i = 0; i < c->count(); ++i ) {
+		        QString tmp = c->text( i );
+			cp->insertItem( tmp, i );
+		        if ( c->currentText() == tmp )
+			    cp->setItemChecked( i, TRUE );
+			if ( !maxItems ) {
+			    if ( cp->count() == 10 ) {
+			        int h = cp->sizeHint().height();
+			        maxItems = QApplication::desktop()->height() * 10 / h;
+			    }
+			} else if ( cp->count() >= maxItems - 1 ) {
+			    QPopupMenu* sp = new QPopupMenu;
+			    d->extensionSubMenues.append( sp );
+			    cp->insertItem( tr( "More..." ), sp );
+			    cp = sp;
+			    connect( cp, SIGNAL( activated( int ) ),
+			        c, SLOT( internalActivate( int ) ) );
+			}
+		    }
+		}
+#endif //QT_NO_COMBOBOX
+	    }
+	}
+        ++it;
+    }
+    delete childlist;
+}
+
+
 /*!
-  \reimp
+    \reimp
 */
 
 void QToolBar::resizeEvent( QResizeEvent *e )
 {
-    bool tooSmall;
-    if ( orientation() ==  Horizontal )
-	tooSmall = e->size().width() < sizeHint().width();
-    else
-	tooSmall = e->size().height() < sizeHint().height();
-    if ( tooSmall ) {
-	QObjectListIt it( *children() );
-	bool hide = FALSE;
-	bool doHide = FALSE;
-	bool noButtons = TRUE;
-	d->extensionPopup->clear();
-	d->hiddenItems.clear();
-	while ( it.current() ) {
-	    if ( !it.current()->isWidgetType() ||
-		 qstrcmp( "qt_dockwidget_internal", it.current()->name() ) == 0 ) {
-		++it;
-		continue;
-	    }
-	    QWidget *w = (QWidget*)it.current();
-	    hide = FALSE;
-	    QPoint p = w->parentWidget()->mapTo( this, w->geometry().bottomRight() );
-	    if ( orientation() == Horizontal ) {
-		if ( p.x() > e->size().width()-d->extension->width()/2 )
-		    hide = TRUE;
-	    } else {
-		if ( p.y() > e->size().height()-d->extension->height()/2 )
-		    hide = TRUE;
-	    }
-	    if ( hide && !w->isHidden() ) {
-		doHide = TRUE;
-		if ( w->inherits( "QToolButton" ) ) {
-		    noButtons = FALSE;
-		    QToolButton *b = (QToolButton*)w;
-		    QString s = b->textLabel();
-		    if ( s.isEmpty() )
-			s = b->text();
-		    int id;
-		    if ( b->popup() && b->popupDelay() == 0 )
-			id = d->extensionPopup->insertItem( b->iconSet(), s, b->popup() );
-		    else
-			id = d->extensionPopup->insertItem( b->iconSet(), s );
+    checkForExtension( e->size() );
+}
 
-		    d->hiddenItems.insert( id, b );
-		    if ( b->isToggleButton() )
-			d->extensionPopup->setItemChecked( id, b->isOn() );
-		    if ( !b->isEnabled() )
-			d->extensionPopup->setItemEnabled( id, FALSE );
-		} else if ( w->inherits( "QButton" ) ) {
-		    noButtons = FALSE;
-		    QButton *b = (QButton*)w;
-		    QString s = b->text();
-		    if ( s.isEmpty() )
-			s = "";
-		    int id = -1;
-		    if ( b->pixmap() )
-			id = d->extensionPopup->insertItem( *b->pixmap(), s );
-		    else
-			id = d->extensionPopup->insertItem( s );
-		    d->hiddenItems.insert( id, b );
-		    if ( b->isToggleButton() )
-			d->extensionPopup->setItemChecked( id, b->isOn() );
-		    if ( !b->isEnabled() )
-			d->extensionPopup->setItemEnabled( id, FALSE );
-		}
-	    }
-	    ++it;
-	}
-	if ( doHide && !noButtons ) {
+void QToolBar::checkForExtension( const QSize &sz )
+{
+    bool tooSmall;
+    if ( orientation() == Horizontal )
+	tooSmall = sz.width() < sizeHint().width();
+    else
+	tooSmall = sz.height() < sizeHint().height();
+
+    if ( tooSmall ) {
+	createPopup();
+	if ( d->extensionPopup->count() ) {
 	    if ( orientation() == Horizontal )
 	        d->extension->setGeometry( width() - 20, 1, 20, height() - 2 );
 	    else
@@ -677,55 +739,10 @@ void QToolBar::resizeEvent( QResizeEvent *e )
     }
 }
 
-void QToolBar::popupSelected( int id )
-{
-    QButton *b = d->hiddenItems.find( id );
-    d->button = b;
-    if ( d->button )
-	QTimer::singleShot( 0, this, SLOT( emulateButtonClicked() ) );
-}
 
-void QToolBar::emulateButtonClicked()
-{
-    if ( !d->button )
-	return;
-
-#ifndef QT_NO_PUSHBUTTON
-    if ( d->button->inherits( "QPushButton" ) &&
-	 ( (QPushButton*)d->button )->popup() ) {
-	( (QPushButton*)d->button )->popup()->exec( QCursor::pos() );
-    } else
-#endif
-    if ( d->button->inherits( "QToolButton" ) &&
-		( (QToolButton*)d->button )->popup() ) {
-	( (QToolButton*)d->button )->popup()->exec( QCursor::pos() );
-    } else if ( d->button->isToggleButton() ) {
-	d->button->setOn( !d->button->isOn() );
-	emit d->button->pressed();
-	emit d->button->released();
-	emit d->button->clicked();
-	if ( d->button->inherits( "QWhatsThisButton" ) )
-	    d->button->setOn( FALSE );
-    } else {
-	emit d->button->pressed();
-	emit d->button->released();
-	emit d->button->clicked();
-    }
-    if ( d )
-	d->button = 0;
-}
-
-void QToolBar::setupArrowMenu()
-{
-    QIntDictIterator<QButton> it( d->hiddenItems );
-    while ( it.current() ) {
-	d->extensionPopup->setItemEnabled( it.currentKey(), it.current()->isEnabled() );
-	d->extensionPopup->setItemChecked( it.currentKey(), it.current()->isOn() );
-	++it;
-    }
-}
-
-/*! \reimp */
+/*!
+    \reimp
+*/
 
 void QToolBar::setMinimumSize( int, int )
 {
