@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id:  qt/qtooltip.cpp   3.0.5   edited May 13 14:21 $
+** $Id: qt/qtooltip.cpp   3.2.3   edited Nov 7 21:11 $
 **
 ** Tool Tips (or Balloon Help) for any widget or rectangle
 **
@@ -64,15 +64,16 @@ public:
 	     WStyle_StaysOnTop | WStyle_Customize | WStyle_NoBorder | WStyle_Tool | WX11BypassWM )
     {
 	setMargin(1);
-	setIndent(0);
 	setAutoMask( FALSE );
 	setFrameStyle( QFrame::Plain | QFrame::Box );
 	setLineWidth( 1 );
 	setAlignment( AlignAuto | AlignTop );
+	setIndent(0);
 	polish();
 	setText(text);
 	adjustSize();
     }
+    void setWidth( int w ) { resize( sizeForWidth( w ) ); }
 };
 
 // Internal class - don't touch
@@ -109,6 +110,7 @@ public:
     void    hideTipAndSleep();
 
     QString find( QWidget *, const QPoint& );
+    void setWakeUpDelay(int);
 
 public slots:
     void    hideTip();
@@ -121,10 +123,11 @@ private slots:
 
 private:
     QTimer  wakeUp;
+    int wakeUpDelay;
     QTimer  fallAsleep;
 
     QPtrDict<Tip> *tips;
-    QLabel *label;
+    QTipLabel *label;
     QPoint pos;
     QGuardedPtr<QWidget> widget;
     Tip *currentTip;
@@ -151,6 +154,7 @@ static void initTipManager()
 QTipManager::QTipManager()
     : QObject( qApp, "toolTipManager" )
 {
+    wakeUpDelay = 700;
     tips = new QPtrDict<QTipManager::Tip>( 313 );
     currentTip = 0;
     previousTip = 0;
@@ -285,7 +289,7 @@ void QTipManager::remove( QWidget *w, const QRect & r )
 }
 
 
-/*!
+/*
   The label was destroyed in the program cleanup phase.
 */
 
@@ -295,7 +299,7 @@ void QTipManager::labelDestroyed()
 }
 
 
-/*!
+/*
   Remove sender() from the tool tip data structures.
 */
 
@@ -362,13 +366,16 @@ bool QTipManager::eventFilter( QObject *obj, QEvent *e )
 	label && label->isVisible() )
 	hideTipAndSleep();
 
-    if ( !qApp || !qApp->activeWindow() ||
-	 !obj || !obj->isWidgetType() || // isWidgetType() catches most stuff
-	 !e ||
-	 e->type() == QEvent::Paint ||
-	 e->type() == QEvent::Timer ||
-	 e->type() == QEvent::SockAct ||
-	 !tips )
+    if ( !qApp
+#ifndef Q_WS_X11
+	 || !qApp->activeWindow()
+#endif
+	 || !obj || !obj->isWidgetType() // isWidgetType() catches most stuff
+	 || !e
+	 || e->type() == QEvent::Paint
+	 || e->type() == QEvent::Timer
+	 || e->type() == QEvent::SockAct
+	 || !tips )
 	return FALSE;
     QWidget *w = (QWidget *)obj;
 
@@ -407,9 +414,8 @@ bool QTipManager::eventFilter( QObject *obj, QEvent *e )
 	hideTipAndSleep();
 	break;
     case QEvent::MouseMove:
-	{ // a whole scope just for one variable
+	{
 	    QMouseEvent * m = (QMouseEvent *)e;
-
 	    QPoint mousePos = w->mapFromGlobal( m->globalPos() );
 
 	    if ( currentTip && !currentTip->rect.contains( mousePos ) ) {
@@ -429,7 +435,7 @@ bool QTipManager::eventFilter( QObject *obj, QEvent *e )
 			wakeUp.start( 1, TRUE );
 		    } else {
 			previousTip = 0;
-			wakeUp.start( 700, TRUE );
+			wakeUp.start( wakeUpDelay, TRUE );
 		    }
 		    if ( t->group && t->group->ena &&
 			 !t->group->del && !t->groupText.isEmpty() ) {
@@ -462,7 +468,11 @@ bool QTipManager::eventFilter( QObject *obj, QEvent *e )
 
 void QTipManager::showTip()
 {
-    if ( !widget || !globally_enabled )
+    if ( !widget || !globally_enabled
+#ifndef Q_WS_X11
+	 || !widget->isActiveWindow()
+#endif
+	)
 	return;
 
     QTipManager::Tip *t = (*tips)[ widget ];
@@ -482,12 +492,18 @@ void QTipManager::showTip()
     if ( t->group && !t->group->ena )
 	return;
 
+    int scr;
+    if ( QApplication::desktop()->isVirtualDesktop() )
+	scr = QApplication::desktop()->screenNumber( widget->mapToGlobal( pos ) );
+    else
+	scr = QApplication::desktop()->screenNumber( widget );
+
     if ( label
 #if defined(Q_WS_X11)
 	 && label->x11Screen() == widget->x11Screen()
 #endif
 	 ) {
-	// the next two lines are a workaround for QLabel being to intelligent.
+	// the next two lines are a workaround for QLabel being too intelligent.
 	// QLabel turns on the wordbreak flag once it gets a richtext. The two lines below
 	// ensure we get correct textflags when switching back and forth between a richtext and
 	// non richtext tooltip
@@ -498,7 +514,6 @@ void QTipManager::showTip()
 	if ( t->geometry != QRect( -1, -1, -1, -1 ) )
 	    label->resize( t->geometry.size() );
     } else {
-	int scr = QApplication::desktop()->screenNumber( widget );
 	delete label;
 	label = new QTipLabel( QApplication::desktop()->screen( scr ), t->text);
 	if ( t->geometry != QRect( -1, -1, -1, -1 ) )
@@ -506,11 +521,25 @@ void QTipManager::showTip()
 	Q_CHECK_PTR( label );
 	connect( label, SIGNAL(destroyed()), SLOT(labelDestroyed()) );
     }
-    int scr = QApplication::desktop()->screenNumber( widget );
+    // the above deletion and creation of a QTipLabel causes events to be sent. We had reports that the widget
+    // pointer was 0 after this. This is in principle possible if the wrong kind of events get sent through our event
+    // filter in this time. So better be safe and check widget once again here.
+    if (!widget)
+	return;
+
+#ifdef Q_WS_MAC
+    QRect screen = QApplication::desktop()->availableGeometry( scr );
+#else
     QRect screen = QApplication::desktop()->screenGeometry( scr );
+#endif
     QPoint p;
     if ( t->geometry == QRect( -1, -1, -1, -1 ) ) {
-	p = widget->mapToGlobal( pos ) + QPoint( 2, 16 );
+	p = widget->mapToGlobal( pos ) +
+#ifdef Q_WS_WIN
+	    QPoint( 2, 24 );
+#else
+	    QPoint( 2, 16 );
+#endif
 	if ( p.x() + label->width() > screen.x() + screen.width() )
 	    p.rx() -= 4 + label->width();
 	if ( p.y() + label->height() > screen.y() + screen.height() )
@@ -518,8 +547,7 @@ void QTipManager::showTip()
     } else {
 	p = widget->mapToGlobal( t->geometry.topLeft() );
 	label->setAlignment( WordBreak | AlignCenter );
-	int h = label->heightForWidth( t->geometry.width() - 4 );
-	label->resize( label->width(), h );
+	label->setWidth( t->geometry.width() - 4 );
     }
     if ( p.y() < screen.y() )
 	p.setY( screen.y() );
@@ -606,99 +634,107 @@ QString QTipManager::find( QWidget *w, const QPoint& pos )
     return t ? t->text : QString::null;
 }
 
+void QTipManager::setWakeUpDelay ( int i )
+{
+    wakeUpDelay = i;
+}
+
 /*!
-  \class QToolTip qtooltip.h
+    \class QToolTip qtooltip.h
+    \brief The QToolTip class provides tool tips (balloon help) for
+    any widget or rectangular part of a widget.
 
-  \brief The QToolTip class provides tool tips (balloon help) for any
-  widget or rectangular part of a widget.
+    \ingroup helpsystem
+    \mainclass
 
-  \ingroup helpsystem
-  \mainclass
+    The tip is a short, single line of text reminding the user of the
+    widget's or rectangle's function. It is drawn immediately below
+    the region in a distinctive black-on-yellow combination.
 
-  The tip is a short, single line of text reminding the user of the widget's
-  or rectangle's function.  It is drawn immediately below the region
-  in a distinctive black-on-yellow combination.
+    The tip can be any Rich-Text formatted string.
 
-  QToolTipGroup provides a way for tool tips to display another text
-  elsewhere (most often in a status bar).
+    QToolTipGroup provides a way for tool tips to display another text
+    elsewhere (most often in a \link QStatusBar status bar\endlink).
 
-  At any point in time, QToolTip is either dormant or active.  In
-  dormant mode the tips are not shown and in active mode they are.
-  The mode is global, not particular to any one widget.
+    At any point in time, QToolTip is either dormant or active. In
+    dormant mode the tips are not shown and in active mode they are.
+    The mode is global, not particular to any one widget.
 
-  QToolTip switches from dormant to active mode when the user hovers the
-  mouse on a tip-equipped region for a second or so and remains
-  active until the user either clicks a mouse button, presses
-  a key, lets the mouse hover for five seconds or moves the mouse
-  outside \e all tip-equipped regions for at least a second.
+    QToolTip switches from dormant to active mode when the user hovers
+    the mouse on a tip-equipped region for a second or so and remains
+    active until the user either clicks a mouse button, presses a key,
+    lets the mouse hover for five seconds or moves the mouse outside
+    \e all tip-equipped regions for at least a second.
 
-  The QToolTip class can be used in three different ways:
-  \list 1
-  \i Adding a tip to an entire widget.
-  \i Adding a tip to a fixed rectangle within a widget.
-  \i Adding a tip to a dynamic rectangle within a widget.
-  \endlist
+    The QToolTip class can be used in three different ways:
+    \list 1
+    \i Adding a tip to an entire widget.
+    \i Adding a tip to a fixed rectangle within a widget.
+    \i Adding a tip to a dynamic rectangle within a widget.
+    \endlist
 
-  To add a tip to a widget, call the \e static function QToolTip::add()
-  with the widget and tip as arguments:
+    To add a tip to a widget, call the \e static function
+    QToolTip::add() with the widget and tip as arguments:
 
-  \code
-    QToolTip::add( quitButton, "Leave the application" );
-  \endcode
+    \code
+	QToolTip::add( quitButton, "Leave the application" );
+    \endcode
 
-  This is the simplest and most common use of QToolTip.  The tip will
-  be deleted automatically when \e quitButton is deleted, but you can
-  remove it yourself, too:
+    This is the simplest and most common use of QToolTip. The tip
+    will be deleted automatically when \e quitButton is deleted, but
+    you can remove it yourself, too:
 
-  \code
-    QToolTip::remove( quitButton );
-  \endcode
+    \code
+	QToolTip::remove( quitButton );
+    \endcode
 
-  You can also display another text (typically in a \link QStatusBar
-  status bar),\endlink courtesy of QToolTipGroup.  This example
-  assumes that \e g is a \c{QToolTipGroup *} and is already
-  connected to the appropriate status bar:
+    You can also display another text (typically in a \link QStatusBar
+    status bar),\endlink courtesy of \l{QToolTipGroup}. This example
+    assumes that \e grp is a \c{QToolTipGroup *} and is already
+    connected to the appropriate status bar:
 
-  \code
-    QToolTip::add( quitButton, "Leave the application", g,
-		   "Leave the application, prompting to save if necessary" );
-    QToolTip::add( closeButton, "Close this window", g,
-		   "Close this window, prompting to save if necessary" );
-  \endcode
+    \code
+	QToolTip::add( quitButton, "Leave the application", grp,
+		       "Leave the application, prompting to save if necessary" );
+	QToolTip::add( closeButton, "Close this window", grp,
+		       "Close this window, prompting to save if necessary" );
+    \endcode
 
-  To add a tip to a fixed rectangle within a widget, call the static
-  function QToolTip::add() with the widget, rectangle and tip as
-  arguments.  (See the tooltip/tooltip.cpp example.)  Again, you can supply a
-  \c{QToolTipGroup *} and another text if you want.
+    To add a tip to a fixed rectangle within a widget, call the static
+    function QToolTip::add() with the widget, rectangle and tip as
+    arguments. (See the \c tooltip/tooltip.cpp example.) Again, you
+    can supply a \c{QToolTipGroup *} and another text if you want.
 
-  Both of these are one-liners and cover the majority of
-  cases. The third and most general way to use QToolTip requires you
-  to reimplement a pure virtual function to decide whether to pop up a
-  tool tip. The tooltip/tooltip.cpp example demonstrates this too.
-  This mode can be used to implement tips for text that can move as the
-  user scrolls, for example.
+    Both of these are one-liners and cover the majority of cases. The
+    third and most general way to use QToolTip requires you to
+    reimplement a pure virtual function to decide whether to pop up a
+    tool tip. The \c tooltip/tooltip.cpp example demonstrates this
+    too. This mode can be used to implement tips for text that can
+    move as the user scrolls, for example.
 
-  To use QToolTip like this, you need to subclass QToolTip and reimplement
-  maybeTip(). QToolTip calls maybeTip() when a tip should pop up, and
-  maybeTip decides whether to show a tip.
+    To use QToolTip like this, you must subclass QToolTip and
+    reimplement maybeTip(). QToolTip calls maybeTip() when a tip
+    should pop up, and maybeTip() decides whether to show a tip.
 
-  Tool tips can be globally disabled using QToolTip::setGloballyEnabled() or
-  disabled in groups with QToolTipGroup::setEnabled().
+    Tool tips can be globally disabled using
+    QToolTip::setGloballyEnabled() or disabled in groups with
+    QToolTipGroup::setEnabled().
 
-  You can retreive the text of a tooltip for a given position within a
-  widget using textFor().
+    You can retrieve the text of a tooltip for a given position within
+    a widget using textFor().
 
-  The global tooltip font and palette can be set with the static
-  setFont() and setPalette() functions respectively.
+    The global tooltip font and palette can be set with the static
+    setFont() and setPalette() functions respectively.
 
-  \sa QStatusBar QWhatsThis QToolTipGroup
-  \link guibooks.html#fowler GUI Design Handbook: Tool Tip\endlink
+    \sa QStatusBar QWhatsThis QToolTipGroup
+    \link guibooks.html#fowler GUI Design Handbook: Tool Tip\endlink
 */
 
 
 /*!
-  Returns the font common to all tool tips.
-  \sa setFont()
+    Returns the font common to all tool tips.
+
+    \sa setFont()
 */
 
 QFont QToolTip::font()
@@ -709,8 +745,9 @@ QFont QToolTip::font()
 
 
 /*!
-  Sets the font for all tool tips to \a font.
-  \sa font()
+    Sets the font for all tool tips to \a font.
+
+    \sa font()
 */
 
 void QToolTip::setFont( const QFont &font )
@@ -720,8 +757,9 @@ void QToolTip::setFont( const QFont &font )
 
 
 /*!
-  Returns the palette common to all tool tips.
-  \sa setPalette()
+    Returns the palette common to all tool tips.
+
+    \sa setPalette()
 */
 
 QPalette QToolTip::palette()
@@ -732,8 +770,9 @@ QPalette QToolTip::palette()
 
 
 /*!
-  Sets the palette for all tool tips to \a palette.
-  \sa palette()
+    Sets the palette for all tool tips to \a palette.
+
+    \sa palette()
 */
 
 void QToolTip::setPalette( const QPalette &palette )
@@ -742,17 +781,28 @@ void QToolTip::setPalette( const QPalette &palette )
 }
 
 /*!
-  Constructs a tool tip object.  This is necessary only if you need
-  tool tips on regions that can move within the widget (most often
-  because the widget's contents can scroll).
+    Constructs a tool tip object. This is only necessary if you need
+    tool tips on regions that can move within the widget (most often
+    because the widget's contents can scroll).
 
-  \a widget is the widget you want to add dynamic tool tips to and \a
-  group (optional) is the tool tip group they should belong to.
+    \a widget is the widget you want to add dynamic tool tips to and
+    \a group (optional) is the tool tip group they should belong to.
 
-  \warning QToolTip is not a subclass of QObject, so the instance of
-  QToolTip is not deleted when \a widget is deleted.
+    \warning QToolTip is not a subclass of QObject, so the instance of
+    QToolTip is not deleted when \a widget is deleted.
 
-  \sa maybeTip().
+    \warning If you delete the tool tip before you have deleted
+    \a widget then you need to make sure you call remove() yourself from
+    \a widget in your reimplemented QToolTip destructor.
+
+    \code
+	MyToolTip::~MyToolTip()
+	{
+	    remove( widget );
+	}
+    \endcode
+
+    \sa maybeTip().
 */
 
 QToolTip::QToolTip( QWidget * widget, QToolTipGroup * group )
@@ -766,12 +816,12 @@ QToolTip::QToolTip( QWidget * widget, QToolTipGroup * group )
 
 
 /*!
-  Adds a tool tip to \a widget.  \a text is the text to be shown in
-  the tool tip.
+    Adds a tool tip to \a widget. \a text is the text to be shown in
+    the tool tip.
 
-  This is the most common entry point to the QToolTip class; it is
-  suitable for adding tool tips to buttons, check boxes, combo boxes
-  and so on.
+    This is the most common entry point to the QToolTip class; it is
+    suitable for adding tool tips to buttons, check boxes, combo boxes
+    and so on.
 */
 
 void QToolTip::add( QWidget *widget, const QString &text )
@@ -784,12 +834,14 @@ void QToolTip::add( QWidget *widget, const QString &text )
 
 /*!
     \overload
-  Adds a tool tip to \a widget and to tool tip group \a group.
 
-  \a text is the text shown in the tool tip and \a longText is the
-  text emitted from \a group.
+    Adds a tool tip to \a widget and to tool tip group \a group.
 
-  Normally, \a longText is shown in a status bar or similar.
+    \a text is the text shown in the tool tip and \a longText is the
+    text emitted from \a group.
+
+    Normally, \a longText is shown in a \link QStatusBar status
+    bar\endlink or similar.
 */
 
 void QToolTip::add( QWidget *widget, const QString &text,
@@ -801,10 +853,10 @@ void QToolTip::add( QWidget *widget, const QString &text,
 
 
 /*!
-  Removes the tool tip from \a widget.
+    Removes the tool tip from \a widget.
 
-  If there is more than one tool tip on \a widget, only the one
-  covering the entire widget is removed.
+    If there is more than one tool tip on \a widget, only the one
+    covering the entire widget is removed.
 */
 
 void QToolTip::remove( QWidget * widget )
@@ -815,8 +867,9 @@ void QToolTip::remove( QWidget * widget )
 
 /*!
     \overload
-  Adds a tool tip to a fixed rectangle, \a rect, within \a widget.  \a
-  text is the text shown in the tool tip.
+
+    Adds a tool tip to a fixed rectangle, \a rect, within \a widget.
+    \a text is the text shown in the tool tip.
 */
 
 void QToolTip::add( QWidget * widget, const QRect & rect, const QString &text )
@@ -828,13 +881,16 @@ void QToolTip::add( QWidget * widget, const QRect & rect, const QString &text )
 
 /*!
     \overload
-  Adds a tool tip to an entire \a widget and to tool tip group \a
-  group. The tooltip will disappear when the mouse leaves the \a rect.
 
-  \a text is the text shown in the tool tip and \a groupText is the
-  text emitted from \a group.
+    Adds a tool tip to an entire \a widget and to tool tip group \a
+    group. The tooltip will disappear when the mouse leaves the \a
+    rect.
 
-  Normally, \a groupText is shown in a status bar or similar.
+    \a text is the text shown in the tool tip and \a groupText is the
+    text emitted from \a group.
+
+    Normally, \a groupText is shown in a \link QStatusBar status
+    bar\endlink or similar.
 */
 
 void QToolTip::add( QWidget *widget, const QRect &rect,
@@ -848,10 +904,11 @@ void QToolTip::add( QWidget *widget, const QRect &rect,
 
 /*!
     \overload
-  Removes the tool tip for \a rect from \a widget.
 
-  If there is more than one tool tip on \a widget, only the one
-  covering rectangle \a rect is removed.
+    Removes the tool tip for \a rect from \a widget.
+
+    If there is more than one tool tip on \a widget, only the one
+    covering rectangle \a rect is removed.
 */
 
 void QToolTip::remove( QWidget * widget, const QRect & rect )
@@ -861,8 +918,9 @@ void QToolTip::remove( QWidget * widget, const QRect & rect )
 }
 
 /*!
-  Returns the text for \a widget at position \a pos, or a null string
-  if there is no tool tip for the \a widget.
+    Returns the tool tip text for \a widget at position \a pos, or
+    QString::null if there is no tool tip for the given widget and
+    position.
 */
 
 QString QToolTip::textFor( QWidget *widget, const QPoint& pos )
@@ -873,10 +931,10 @@ QString QToolTip::textFor( QWidget *widget, const QPoint& pos )
 }
 
 /*!
-  Hides any tip that is currently being shown.
+    Hides any tip that is currently being shown.
 
-  Normally, there is no need to call this function; QToolTip takes
-  care of showing and hiding the tips as the user moves the mouse.
+    Normally, there is no need to call this function; QToolTip takes
+    care of showing and hiding the tips as the user moves the mouse.
 */
 
 void QToolTip::hide()
@@ -886,45 +944,46 @@ void QToolTip::hide()
 }
 
 /*!
-  \fn virtual void QToolTip::maybeTip( const QPoint & p);
+    \fn virtual void QToolTip::maybeTip( const QPoint & p);
 
-  This pure virtual function is half of the most versatile interface
-  QToolTip offers.
+    This pure virtual function is half of the most versatile interface
+    QToolTip offers.
 
-  It is called when there is a possibility that a tool tip should be
-  shown and must decide whether there is a tool tip for the point \a p
-  in the widget that this QToolTip object relates to.  If so,
-  maybeTip() must call tip() with the rectangle the tip applies to,
-  the tip's text and optionally the QToolTipGroup details.
+    It is called when there is a possibility that a tool tip should be
+    shown and must decide whether there is a tool tip for the point \a
+    p in the widget that this QToolTip object relates to. If so,
+    maybeTip() must call tip() with the rectangle the tip applies to,
+    the tip's text and optionally the QToolTipGroup details and the
+    geometry in screen coordinates.
 
-  \a p is given in that widget's local coordinates.  Most maybeTip()
-  implementations will be of the form:
+    \a p is given in that widget's local coordinates. Most maybeTip()
+    implementations will be of the form:
 
-  \code
-    if ( <something> ) {
-	tip( <something>, <something> );
-    }
-  \endcode
+    \code
+	if ( <something> ) {
+	    tip( <something>, <something> );
+	}
+    \endcode
 
-  The first argument to tip() (a rectangle) must encompass \a p, i.e.
-  the tip must apply to the current mouse position; otherwise
-  QToolTip's operation is undefined.
+    The first argument to tip() (a rectangle) must encompass \a p,
+    i.e. the tip must apply to the current mouse position; otherwise
+    QToolTip's operation is undefined.
 
-  Note that the tip will disappear once the mouse moves outside the
-  rectangle you give to tip(), and will not reappear if the mouse moves
-  back in - maybeTip() is called again instead.
+    Note that the tip will disappear once the mouse moves outside the
+    rectangle you give to tip(), and will not reappear if the mouse
+    moves back in: maybeTip() is called again instead.
 
-  \sa tip()
+    \sa tip()
 */
 
 
 /*!
-  Immediately pops up a tip saying \a text and removes the tip once
-  the cursor moves out of rectangle \a rect (which is given in the
-  coordinate system of the widget this QToolTip relates to).
+    Immediately pops up a tip saying \a text and removes the tip once
+    the cursor moves out of rectangle \a rect (which is given in the
+    coordinate system of the widget this QToolTip relates to).
 
-  The tip will not reappear if the cursor moves back; your maybeTip()
-  has to reinstate it each time.
+    The tip will not reappear if the cursor moves back; your
+    maybeTip() must reinstate it each time.
 */
 
 void QToolTip::tip( const QRect & rect, const QString &text )
@@ -933,20 +992,16 @@ void QToolTip::tip( const QRect & rect, const QString &text )
     tipManager->add( parentWidget(), rect, text, 0, QString::null, 0, TRUE );
 }
 
-void QToolTip::tip( const QRect &geometry, const QRect &rect, const QString &text )
-{
-    initTipManager();
-    tipManager->add( geometry, parentWidget(), rect, text, 0, QString::null, 0, TRUE );
-}
-
 /*!
     \overload
-  Immediately pops up a tip saying \a text and removes that tip once
-  the cursor moves out of rectangle \a rect. \a groupText is the
-  text emitted from the group.
 
-  The tip will not reappear if the cursor moves back; your maybeTip()
-  has to reinstate it each time.
+    Immediately pops up a tip saying \a text and removes that tip once
+    the cursor moves out of rectangle \a rect (which is given in the
+    coordinate system of the widget this QToolTip relates to). \a
+    groupText is the text emitted from the group.
+
+    The tip will not reappear if the cursor moves back; your
+    maybeTip() must reinstate it each time.
 */
 
 void QToolTip::tip( const QRect & rect, const QString &text,
@@ -956,9 +1011,52 @@ void QToolTip::tip( const QRect & rect, const QString &text,
     tipManager->add( parentWidget(), rect, text, group(), groupText, 0, TRUE );
 }
 
+/*!
+    \overload
+
+    Immediately pops up a tip within the rectangle \a geometry, saying
+    \a text and removes the tip once the cursor moves out of rectangle
+    \a rect. Both rectangles are given in the coordinate system of the
+    widget this QToolTip relates to.
+
+    The tip will not reappear if the cursor moves back; your
+    maybeTip() must reinstate it each time.
+
+    If the tip does not fit inside \a geometry, the tip expands.
+*/
+
+void QToolTip::tip( const QRect &rect, const QString &text, const QRect &geometry )
+{
+    initTipManager();
+    tipManager->add( geometry, parentWidget(), rect, text, 0, QString::null, 0, TRUE );
+}
 
 /*!
-  Immediately removes all tool tips for this tooltip's parent widget.
+    \overload
+
+    mmediately pops up a tip within the rectangle \a geometry, saying
+    \a text and removes the tip once the cursor moves out of rectangle
+    \a rect. \a groupText is the text emitted from the group. Both
+    rectangles are given in the coordinate system of the widget this
+    QToolTip relates to.
+
+    The tip will not reappear if the cursor moves back; your
+    maybeTip() must reinstate it each time.
+
+    If the tip does not fit inside \a geometry, the tip expands.
+*/
+
+void QToolTip::tip( const QRect &rect, const QString &text, const QString& groupText, const QRect &geometry )
+{
+    initTipManager();
+    tipManager->add( geometry, parentWidget(), rect, text, group(), groupText, 0, TRUE );
+}
+
+
+
+/*!
+    Immediately removes all tool tips for this tooltip's parent
+    widget.
 */
 
 void QToolTip::clear()
@@ -969,85 +1067,87 @@ void QToolTip::clear()
 
 
 /*!
-  \fn QWidget * QToolTip::parentWidget() const
+    \fn QWidget * QToolTip::parentWidget() const
 
-  Returns the widget this QToolTip applies to.
+    Returns the widget this QToolTip applies to.
 
-  The tool tip is destroyed automatically when the parent widget is
-  destroyed.
+    The tool tip is destroyed automatically when the parent widget is
+    destroyed.
 
-  \sa group()
+    \sa group()
 */
 
 
 /*!
-  \fn QToolTipGroup * QToolTip::group() const
+    \fn QToolTipGroup * QToolTip::group() const
 
-  Returns the tool tip group this QToolTip is a member of or 0 if it
-  isn't a member of any group.
+    Returns the tool tip group this QToolTip is a member of or 0 if it
+    isn't a member of any group.
 
-  The tool tip group is the object responsible for maintaining contact
-  between tool tips and a status bar or something else which can show
-  the longer help text.
+    The tool tip group is the object responsible for maintaining
+    contact between tool tips and a \link QStatusBar status
+    bar\endlink or something else which can show the longer help text.
 
-  \sa parentWidget(), QToolTipGroup
+    \sa parentWidget(), QToolTipGroup
 */
 
 
 /*!
-  \class QToolTipGroup qtooltip.h
+    \class QToolTipGroup qtooltip.h
+    \brief The QToolTipGroup class collects tool tips into related groups.
 
-  \brief The QToolTipGroup class collects tool tips into related groups.
+    \ingroup helpsystem
 
-  \ingroup helpsystem
+    Tool tips can display \e two texts: one in the tip and
+    (optionally) one that is typically in a \link QStatusBar status
+    bar\endlink. QToolTipGroup provides a way to link tool tips to
+    this status bar.
 
-  Tool tips can display \e two texts: one in the tip and (optionally) one
-  that is typically in a status bar.  QToolTipGroup provides a way to link
-  tool tips to this status bar.
+    QToolTipGroup has practically no API; it is only used as an
+    argument to QToolTip's member functions, for example like this:
 
-  QToolTipGroup has practically no API; it is used only as an argument
-  to QToolTip's member functions, for example like this:
+    \code
+	QToolTipGroup * grp = new QToolTipGroup( this, "tool tip relay" );
+	connect( grp, SIGNAL(showTip(const QString&)),
+		 myLabel, SLOT(setText(const QString&)) );
+	connect( grp, SIGNAL(removeTip()),
+		 myLabel, SLOT(clear()) );
+	QToolTip::add( giraffeButton, "feed giraffe",
+		       grp, "Give the giraffe a meal" );
+	QToolTip::add( gorillaButton, "feed gorilla",
+		       grp, "Give the gorilla a meal" );
+    \endcode
 
-  \code
-    QToolTipGroup * g = new QToolTipGroup( this, "tool tip relay" );
-    connect( g, SIGNAL(showTip(const QString&)),
-	     myLabel, SLOT(setText(const QString&)) );
-    connect( g, SIGNAL(removeTip()),
-	     myLabel, SLOT(clear()) );
-    QToolTip::add( giraffeButton, "feed giraffe",
-		   g, "Give the giraffe a meal" );
-    QToolTip::add( gorillaButton, "feed gorilla",
-		   g, "Give the gorilla a meal" );
-  \endcode
+    This example makes the object myLabel (which you must supply)
+    display (one assumes, though you can make myLabel do anything, of
+    course) the strings "Give the giraffe a meal" and "Give the
+    gorilla a meal" while the relevant tool tips are being displayed.
 
-  This example makes the object myLabel (which you have to supply)
-  display (one assumes, though you can make myLabel do anything, of
-  course) the strings "Give the giraffe a meal" and "Give the gorilla
-  a meal" while the relevant tool tips are being displayed.
-
-  Deleting a tool tip group removes the tool tips in it.
+    Deleting a tool tip group removes the tool tips in it.
 */
 
-/*! \fn void QToolTipGroup::showTip (const QString &longText)
+/*!
+    \fn void QToolTipGroup::showTip (const QString &longText)
 
-  This signal is emitted when one of the tool tips in the group is
-  displayed.  \a longText is the extra text for the displayed
-  tool tip.
+    This signal is emitted when one of the tool tips in the group is
+    displayed. \a longText is the extra text for the displayed tool
+    tip.
 
-  \sa removeTip()
+    \sa removeTip()
 */
 
-/*! \fn void QToolTipGroup::removeTip ()
+/*!
+    \fn void QToolTipGroup::removeTip ()
 
-  This signal is emitted when a tool tip in this group is hidden.  See
-  the QToolTipGroup documentation for an example of use.
+    This signal is emitted when a tool tip in this group is hidden.
+    See the QToolTipGroup documentation for an example of use.
 
-  \sa showTip()
+    \sa showTip()
 */
 
 
 /*!
-  Constructs a tool tip group with parent \a parent and name \a name.
+    Constructs a tool tip group called \a name, with parent \a parent.
 */
 
 QToolTipGroup::QToolTipGroup( QObject *parent, const char *name )
@@ -1059,7 +1159,7 @@ QToolTipGroup::QToolTipGroup( QObject *parent, const char *name )
 
 
 /*!
-  Destroys this tool tip group and all tool tips in it.
+    Destroys this tool tip group and all tool tips in it.
 */
 
 QToolTipGroup::~QToolTipGroup()
@@ -1070,12 +1170,12 @@ QToolTipGroup::~QToolTipGroup()
 
 
 /*!
-  \property QToolTipGroup::delay
-  \brief whether the display of the group text is delayed.
+    \property QToolTipGroup::delay
+    \brief whether the display of the group text is delayed.
 
-  If set to TRUE (the default), the group text is displayed at the time of
-  the tool tip.  Otherwise, the group text is displayed immediately when the
-  cursor enters the widget.
+    If set to TRUE (the default), the group text is displayed at the
+    same time as the tool tip. Otherwise, the group text is displayed
+    immediately when the cursor enters the widget.
 */
 
 bool QToolTipGroup::delay() const
@@ -1104,10 +1204,10 @@ void QToolTipGroup::setDelay( bool enable )
     \obsolete
 */
 /*!
-  \property QToolTipGroup::enabled
-  \brief whether tool tips in the group are enabled.
+    \property QToolTipGroup::enabled
+    \brief whether tool tips in the group are enabled.
 
-  This property's default is TRUE.
+    This property's default is TRUE.
 */
 
 void QToolTipGroup::setEnabled( bool enable )
@@ -1125,10 +1225,10 @@ bool QToolTipGroup::enabled() const
     needed); if \a enable is FALSE sets all tool tips to be disabled
     (never shown).
 
-  By default, tool tips are enabled. Note that this function
-  affects all tool tips in the entire application.
+    By default, tool tips are enabled. Note that this function affects
+    all tool tips in the entire application.
 
-  \sa QToolTipGroup::setEnabled()
+    \sa QToolTipGroup::setEnabled()
 */
 
 void QToolTip::setGloballyEnabled( bool enable )
@@ -1137,13 +1237,24 @@ void QToolTip::setGloballyEnabled( bool enable )
 }
 
 /*!
-  Returns whether tool tips are enabled globally.
-  \sa setGloballyEnabled()
+    Returns whether tool tips are enabled globally.
+
+    \sa setGloballyEnabled()
 */
 bool QToolTip::isGloballyEnabled()
 {
     return globally_enabled;
 }
 
+/*!
+  Sets the wakeup delay for all tooltips to \a i.
+*/
+void QToolTip::setWakeUpDelay ( int i )
+{
+    initTipManager();
+    tipManager->setWakeUpDelay(i);
+}
+
+
 #include "qtooltip.moc"
 #endif
