@@ -1,17 +1,23 @@
 /**********************************************************************
 ** Copyright (C) 2000 Trolltech AS.  All rights reserved.
 **
-** This file is part of Qt Designer.
+**1 This file is part of Qt Designer.
 **
 ** This file may be distributed and/or modified under the terms of the
 ** GNU General Public License version 2 as published by the Free Software
 ** Foundation and appearing in the file LICENSE.GPL included in the
 ** packaging of this file.
 **
+** Licensees holding valid Qt Enterprise Edition or Qt Professional Edition
+** licenses may use this file in accordance with the Qt Commercial License
+** Agreement provided with the Software.
+**
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 **
 ** See http://www.trolltech.com/gpl/ for GPL licensing information.
+** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
+**   information about Qt Commercial License Agreements.
 **
 ** Contact info@trolltech.com if any conditions of this licensing are
 ** not clear to you.
@@ -20,13 +26,15 @@
 
 #include "project.h"
 #include "formwindow.h"
-#include "config.h"
 #include "designerappiface.h"
 #include "../interfaces/languageinterface.h"
 #include "pixmapcollection.h"
 #ifndef QT_NO_SQL
 #include "dbconnectionimpl.h"
 #endif
+#include "resource.h"
+#include <qwidgetfactory.h>
+#include "outputwindow.h"
 
 #include <qfile.h>
 #include <qtextstream.h>
@@ -38,6 +46,7 @@
 #include <qmessagebox.h>
 #include <qapplication.h>
 #include "mainwindow.h"
+#include <qworkspace.h>
 
 #ifndef QT_NO_SQL
 #include <qsqldatabase.h>
@@ -45,6 +54,10 @@
 #include <qdatatable.h>
 #endif
 
+#include <stdlib.h>
+#ifdef Q_OS_UNIX
+# include <unistd.h>
+#endif
 
 #ifndef QT_NO_SQL
 DatabaseConnection::~DatabaseConnection()
@@ -59,7 +72,7 @@ bool DatabaseConnection::refreshCatalog()
 	return TRUE;
     if ( !open() )
 	return FALSE;
-    tbls = conn->tables();
+    tbls = conn->tables( QSql::TableType( QSql::Tables | QSql::Views ) );
     flds.clear();
     for ( QStringList::Iterator it = tbls.begin(); it != tbls.end(); ++it ) {
 	QSqlRecord fil = conn->record( *it );
@@ -130,14 +143,14 @@ bool DatabaseConnection::open( bool suppressDialog )
 	conn->setPort( prt );
 	success = conn->open();
 	if ( !success ) {
-	    switch( QMessageBox::warning( MainWindow::self, QApplication::tr( "Connection" ),
+	    switch( QMessageBox::warning( project->messageBoxParent(), QApplication::tr( "Connection" ),
 					  QApplication::tr( "Could not connect to the database.\n"
 							    "Press 'OK' to continue or 'Cancel' to "
 							    "specify different\nconnection information.\n" )
 					  + QString( "[" + conn->lastError().driverText() + "\n" +
 						     conn->lastError().databaseText() + "]\n" ),
 					  QApplication::tr( "&OK" ),
-					  QApplication::tr( "&Cancel" ), 0, 0, 1 ) ) {
+					  QApplication::tr( "&Cancel" ), QString::null, 0, 1 ) ) {
 	    case 0: // OK or Enter
 		continue;
 	    case 1: // Cancel or Escape
@@ -186,13 +199,16 @@ bool Project::isDummy() const
     return isDummyProject;
 }
 
-Project::Project( const QString &fn, const QString &pName, QPluginManager<ProjectSettingsInterface> *pm, bool isDummy  )
+Project::Project( const QString &fn, const QString &pName,
+		  QPluginManager<ProjectSettingsInterface> *pm, bool isDummy,
+		  const QString &l )
     : proName( pName ), projectSettingsPluginManager( pm ), isDummyProject( isDummy )
 {
     modified = TRUE;
     pixCollection = new PixmapCollection( this );
     iface = 0;
-    lang = "C++";
+    lang = l;
+    is_cpp = lang == "C++";
     cfg.insert( "(all)", "qt warn_on release" );
     templ = "app";
     setFileName( fn );
@@ -200,10 +216,14 @@ Project::Project( const QString &fn, const QString &pName, QPluginManager<Projec
 	proName = pName;
     sourcefiles.setAutoDelete( TRUE );
     modified = FALSE;
+    objs.setAutoDelete( FALSE );
+    fakeFormFiles.setAutoDelete( FALSE );
 }
 
 Project::~Project()
 {
+    if ( singleProjectMode() )
+	removeTempProject();
     delete iface;
     delete pixCollection;
 }
@@ -231,11 +251,37 @@ void Project::setFileName( const QString &fn, bool doClear )
 {
     if ( fn == filename )
 	return;
-    filename = fn;
 
-    if ( !filename.endsWith( ".pro" ) )
-	filename += ".pro";
-    proName = filename;
+    if ( singleProjectMode() ) {
+	QString qsa = QString( getenv( "HOME" ) ) + QString( "/.qsa" );
+	if ( !QFile::exists( qsa ) ) {
+	    QDir d;
+	    d.mkdir( qsa );
+	}
+	if ( fn == singleProFileName )
+	    return;
+	singleProFileName = fn;
+	static int counter = 0;
+	QString str_counter = QString::number( counter++ );
+	str_counter = "/.qsa/" + str_counter;
+	LanguageInterface *iface = MetaDataBase::languageInterface( language() );
+	filename = QString( getenv( "HOME" ) + str_counter + QString( "tmp_" ) +
+			    QFileInfo( fn ).baseName() + "/" + QFileInfo( fn ).baseName() + ".pro" );
+	removeTempProject();
+	if ( iface && iface->supports( LanguageInterface::CompressProject ) ) {
+	    filename = iface->uncompressProject( makeAbsolute( singleProFileName ),
+						 QString( getenv( "HOME" ) +
+							  str_counter + QString( "tmp_" ) +
+							  QFileInfo( fn ).baseName() ) );
+	    proName = makeAbsolute( singleProFileName );
+	}
+    } else {
+	filename = fn;
+	if ( !filename.endsWith( ".pro" ) )
+	    filename += ".pro";
+	proName = filename;
+    }
+
 
     if ( proName.contains( '.' ) )
 	proName = proName.left( proName.find( '.' ) );
@@ -243,12 +289,14 @@ void Project::setFileName( const QString &fn, bool doClear )
     if ( !doClear )
 	return;
     clear();
-    if ( QFile::exists( fn ) )
+    if ( QFile::exists( filename ) )
 	parse();
 }
 
-QString Project::fileName() const
+QString Project::fileName( bool singlePro ) const
 {
+    if ( singlePro )
+	return singleProFileName;
     return filename;
 }
 
@@ -307,7 +355,7 @@ QStringList parse_multiline_part( const QString &contents, const QString &key, i
 	default:
 	    lastWord += c;
 	}
-	
+
 	// ### we should read the 'bla { SOURCES= ... }' stuff as well (braceCount > 0)
 	if ( lastWord == key && braceCount == 0 ) {
 	    if ( start )
@@ -326,7 +374,7 @@ QStringList parse_multiline_part( const QString &contents, const QString &key, i
 		}
 		if ( ( c.isLetter() || c.isDigit() || c == '.' || c == '/' || c == '_' || c == '\\' ||
 		       c == '\"' || c == '\'' || c == '=' ||
-		       c == '$' || c == '-' || c == '(' || c == ')' || c == ':'  || c == '+' || c == ',' ) &&
+		       c == '$' || c == '-' || c == '(' || c == ')' || c == ':'  || c == '+' || c == ',' || c == '~' ) &&
 		     c != ' ' && c != '\t' && c != '\n' ) {
 		    if ( !inName )
 			currName = QString::null;
@@ -367,8 +415,10 @@ void Project::parse()
     int i = contents.find( "LANGUAGE" );
     if ( i != -1 ) {
 	lang = "";
+	is_cpp = FALSE;
 	QString part = contents.mid( i + QString( "LANGUAGE" ).length() );
 	lang = parse_part( part );
+	is_cpp = lang == "C++";
     }
 
     i = contents.find( "DBFILE" );
@@ -380,8 +430,11 @@ void Project::parse()
 
     QStringList uifiles = parse_multiline_part( contents, "FORMS" );
     uifiles += parse_multiline_part( contents, "INTERFACES" ); // compatibility
-    for ( it = uifiles.begin(); it != uifiles.end(); ++it )
+    for ( it = uifiles.begin(); it != uifiles.end(); ++it ) {
+	if ( (*it).startsWith( "__APPOBJ" ) )
+	    continue;
 	(void) new FormFile( *it, FALSE, this );
+    }
 
 
     i = contents.find( "TEMPLATE" );
@@ -531,13 +584,16 @@ static void remove_multiline_contents( QString &contents, const QString &s, int
 
 void Project::save( bool onlyProjectFile )
 {
+    bool anythingModified = FALSE;
     if ( !onlyProjectFile ) {
 	for ( SourceFile *sf = sourcefiles.first(); sf; sf = sourcefiles.next() ) {
+	    anythingModified = anythingModified || sf->isModified();
 	    if ( !sf->save() )
 		return;
 	}
 
 	for ( FormFile *ff = formfiles.first(); ff; ff = formfiles.next() ) {
+	    anythingModified = anythingModified || ff->isModified();
 	    if ( !ff->save() )
 		return;
 	}
@@ -546,8 +602,14 @@ void Project::save( bool onlyProjectFile )
     if ( isDummy() || filename.isEmpty() )
 	return;
 
-    if ( !modified )
+    if ( !modified ) {
+	if ( singleProjectMode() ) {
+	    LanguageInterface *iface = MetaDataBase::languageInterface( language() );
+	    if ( iface && iface->supports( LanguageInterface::CompressProject ) )
+		iface->compressProject( makeAbsolute( filename ), singleProFileName, anythingModified );
+	}
  	return;
+    }
 
     QFile f( filename );
     QString contents;
@@ -570,18 +632,23 @@ void Project::save( bool onlyProjectFile )
 
     if ( !formfiles.isEmpty() ) {
 	contents += "FORMS\t= ";
-	for ( QPtrListIterator<FormFile> fit = formfiles;
-	      fit.current(); ++fit )
-	    contents += fit.current()->fileName() + " ";
+	for ( QPtrListIterator<FormFile> fit = formfiles; fit.current(); ++fit ) {
+	    contents += fit.current()->fileName() +
+		 (fit != formfiles.last() ? " \\\n\t" : "");
+	}
 	contents += "\n";
     }
 
     remove_multiline_contents( contents, "IMAGES" );
-    if ( !pixCollection->isEmpty() ) {
+     if ( !pixCollection->isEmpty() ) {
 	contents += "IMAGES\t= ";
 	QValueList<PixmapCollection::Pixmap> pixmaps = pixCollection->pixmaps();
-	for ( QValueList<PixmapCollection::Pixmap>::Iterator it = pixmaps.begin(); it != pixmaps.end(); ++it )
-	    contents += makeRelative( (*it).absname ) + " ";
+	for ( QValueList<PixmapCollection::Pixmap>::Iterator it = pixmaps.begin();
+	      it != pixmaps.end(); ++it ) {
+		  contents += makeRelative( (*it).absname );
+		  contents += ++it != pixmaps.end() ? " \\\n\t" : "";
+		  --it;
+	}
 	contents += "\n";
     }
 
@@ -622,14 +689,15 @@ void Project::save( bool onlyProjectFile )
 	    lst << makeRelative( f->fileName() );
 	    sourceToKey.replace( key, lst );
 	}
-	
+
 	for ( QMap<QString, QStringList>::Iterator skit = sourceToKey.begin();
 	      skit != sourceToKey.end(); ++skit ) {
 	    QString part = skit.key() + "\t+= ";
 	    QStringList lst = *skit;
 	    for ( QStringList::Iterator sit = lst.begin(); sit != lst.end(); ++sit ) {
 		part += *sit;
-		part += " ";
+		part += ++sit != lst.end() ? " \\\n\t" : "";
+		--sit;
 	    }
 	    contents.insert( 0, part + "\n" );
 	}
@@ -643,7 +711,8 @@ void Project::save( bool onlyProjectFile )
     }
 
     if ( !f.open( IO_WriteOnly | IO_Translate ) ) {
-	QMessageBox::warning( MainWindow::self, "Save Project Failed", "Couldn't write project file " + filename );
+	QMessageBox::warning( messageBoxParent(),
+			      "Save Project Failed", "Couldn't write project file " + filename );
 	return;
     }
 
@@ -655,6 +724,12 @@ void Project::save( bool onlyProjectFile )
     saveConnections();
 
     setModified( FALSE );
+
+    if ( singleProjectMode() ) {
+	LanguageInterface *iface = MetaDataBase::languageInterface( language() );
+	if ( iface && iface->supports( LanguageInterface::CompressProject ) )
+	    iface->compressProject( makeAbsolute( filename ), singleProFileName, TRUE );
+    }
 }
 
 #ifndef QT_NO_SQL
@@ -753,20 +828,25 @@ void Project::saveConnections()
     if ( inSaveConnections )
 	return;
     inSaveConnections = TRUE;
+
     if ( dbFile.isEmpty() ) {
-	QFileInfo fi( fileName() );
-	dbFile = fi.baseName() + ".db";
-	save( TRUE );
-    }
-
-    if ( !QFile::exists( makeAbsolute( dbFile ) ) ) {
 	QFileInfo fi( fileName() );
 	setDatabaseDescription( fi.baseName() + ".db" );
-	save( TRUE );
     }
+    QFile f( makeAbsolute( dbFile ) );
+
+    if ( dbConnections.isEmpty() ) {
+	if ( f.exists() )
+	    f.remove();
+	setDatabaseDescription( "" );
+	modified = TRUE;
+	save( TRUE );
+	inSaveConnections = FALSE;
+	return;
+    }
+    save( TRUE );
 
     /* .db xml */
-    QFile f( makeAbsolute( dbFile ) );
     if ( f.open( IO_WriteOnly | IO_Translate ) ) {
 	QTextStream ts( &f );
 	ts.setCodec( QTextCodec::codecForName( "UTF-8" ) );
@@ -836,7 +916,7 @@ static QDomElement loadSingleProperty( QDomElement e, const QString& name )
 void Project::loadConnections()
 {
 #ifndef QT_NO_SQL
-    if ( !QFile::exists( makeAbsolute( dbFile ) ) )
+    if ( dbFile.isEmpty() || !QFile::exists( makeAbsolute( dbFile ) ) )
 	return;
 
     QFile f( makeAbsolute( dbFile ) );
@@ -942,13 +1022,18 @@ void Project::closeDatabase( const QString &connection )
 //     formWindows.remove( fw );
 // }
 
-QObjectList *Project::formList() const
+QObjectList *Project::formList( bool resolveFakeObjects ) const
 {
     QObjectList *l = new QObjectList;
     for ( QPtrListIterator<FormFile> forms(formfiles);   forms.current(); ++forms ) {
 	FormFile* f = forms.current();
 	if ( f->formWindow() ) {
-	    l->append( f->formWindow()->child( 0, "QWidget" ) );
+	    if ( resolveFakeObjects && f->formWindow()->isFake() )
+		l->append( objectForFakeForm( f->formWindow() ) );
+	    else
+		l->append( f->formWindow()->child( 0, "QWidget" ) );
+	} else if ( f->isFake() ) {
+	    l->append( objectForFakeFormFile( f ) );
 	}
     }
     return l;
@@ -966,6 +1051,7 @@ void Project::setLanguage( const QString &l )
     if ( l == lang )
 	return;
     lang = l;
+    is_cpp = lang == "C++";
     updateCustomSettings();
     modified = TRUE;
 }
@@ -1163,3 +1249,269 @@ bool Project::removeFormFile( FormFile *ff )
     emit formFileRemoved( ff );
     return TRUE;
 }
+
+void Project::addObject( QObject *o )
+{
+    bool wasModified = modified;
+    objs.append( o );
+    FormFile *ff = new FormFile( "", FALSE, this, "qt_fakewindow" );
+    ff->setFileName( "__APPOBJ" + QString( o->name() ) + ".ui" );
+    fakeFormFiles.insert( (void*)o, ff );
+    MetaDataBase::addEntry( o );
+    if ( hasGUI() ) {
+	QWidget *parent = MainWindow::self ? MainWindow::self->qWorkspace() : 0;
+	FormWindow *fw = new FormWindow( ff, MainWindow::self, parent, "qt_fakewindow" );
+	fw->setProject( this );
+	if ( QFile::exists( ff->absFileName() ) )
+	    Resource::loadExtraSource( ff, ff->absFileName(),
+				       MetaDataBase::languageInterface( language() ), FALSE );
+	if ( MainWindow::self )
+	    fw->setMainWindow( MainWindow::self );
+	if ( MainWindow::self ) {
+	    QApplication::sendPostedEvents( MainWindow::self->qWorkspace(), QEvent::ChildInserted );
+	    connect( fw,
+		     SIGNAL( undoRedoChanged( bool, bool, const QString &, const QString & ) ),
+		     MainWindow::self,
+		     SLOT( updateUndoRedo( bool, bool, const QString &, const QString & ) )
+		);
+	}
+	if ( fw->parentWidget() ) {
+	    fw->parentWidget()->setFixedSize( 1, 1 );
+	    fw->show();
+	}
+    } else {
+	if ( QFile::exists( ff->absFileName() ) )
+	    Resource::loadExtraSource( ff, ff->absFileName(),
+				       MetaDataBase::languageInterface( language() ), FALSE );
+    }
+    emit objectAdded( o );
+    modified = wasModified;
+}
+
+void Project::setObjects( const QObjectList &ol )
+{
+    for ( QObjectListIt it( ol ); it.current(); ++it )
+	addObject( it.current() );
+}
+
+void Project::removeObject( QObject *o )
+{
+    bool wasModified = modified;
+    objs.removeRef( o );
+    MetaDataBase::removeEntry( o );
+    fakeFormFiles.remove( (void*)o );
+    emit objectRemoved( o );
+    modified = wasModified;
+}
+
+QObjectList Project::objects() const
+{
+    return objs;
+}
+
+FormFile *Project::fakeFormFileFor( QObject *o ) const
+{
+    return fakeFormFiles.find( (void*)o );
+}
+
+QObject *Project::objectForFakeForm( FormWindow *fw ) const
+{
+    for ( QPtrDictIterator<FormFile> it( fakeFormFiles ); it.current(); ++it ) {
+	if ( it.current()->formWindow() == fw ||
+	    it.current() == fw->formFile() )
+	    return (QObject*)it.currentKey();
+    }
+    return 0;
+}
+
+QObject *Project::objectForFakeFormFile( FormFile *ff ) const
+{
+    for ( QPtrDictIterator<FormFile> it( fakeFormFiles ); it.current(); ++it ) {
+	if ( it.current() == ff )
+	    return (QObject*)it.currentKey();
+    }
+    return 0;
+}
+
+void Project::removeTempProject()
+{
+    if ( !singleProjectMode() )
+	return;
+    QDir d( QFileInfo( filename ).dirPath() );
+    if ( !d.exists( QFileInfo( filename ).dirPath() ) )
+	return;
+    QStringList files = d.entryList( QDir::Files );
+    QStringList::Iterator it;
+    for ( it = files.begin(); it != files.end(); ++it ) {
+	d.remove( *it );
+    }
+    if ( d.exists( QFileInfo( filename ).dirPath() + "/images" ) ) {
+	d = QDir( QFileInfo( filename ).dirPath() + "/images" );
+	files = d.entryList( QDir::Files );
+	for ( it = files.begin(); it != files.end(); ++it )
+	    d.remove( *it );
+	d = QDir( QFileInfo( filename ).dirPath() );
+	d.remove( "images" );
+    }
+    d.remove( QFileInfo( filename ).dirPath() );
+#if defined(Q_OS_UNIX)
+    // ##### implement for all platforms, ideally should be in Qt
+    ::rmdir( d.absPath().latin1() );
+#endif
+}
+
+void Project::addAndEditFunction( const QString &function, const QString &functionBody, bool openDeveloper )
+{
+    for ( SourceFile *f = sourcefiles.first(); f; f = sourcefiles.next() ) {
+	if ( QFileInfo( f->fileName() ).baseName() == "main" ) {
+	    QValueList<LanguageInterface::Function> funcs;
+	    LanguageInterface *iface = MetaDataBase::languageInterface( language() );
+	    if ( !iface )
+		return;
+	    iface->functions( f->text(), &funcs );
+	    QString func = function;
+	    int i = func.find( '(' );
+	    if ( i != -1 )
+		func = func.left( i );
+
+	    bool found = FALSE;
+	    for ( QValueList<LanguageInterface::Function>::Iterator it = funcs.begin();
+		  it != funcs.end(); ++it ) {
+		if ( (*it).name.left( (*it).name.find( '(' ) ) == func ) {
+		    found = TRUE;
+		    break;
+		}
+	    }
+
+	    if ( !found ) {
+		QString code = f->text();
+		if ( functionBody.isEmpty() )
+		    code += "\n\n" + iface->createFunctionStart( "", func, "", "" ) + "()\n{\n\n}\n";
+		else
+		    code += "\n\n" + iface->createFunctionStart( "", func, "", "" ) +
+			    "()\n" + functionBody + "\n";
+		f->setText( code );
+		if ( f->editor() )
+		    f->editor()->refresh( FALSE );
+	    }
+
+	    if ( openDeveloper ) {
+		if ( MainWindow::self )
+		    MainWindow::self->editSource( f );
+		f->editor()->setFunction( func, "" );
+	    }
+
+	    break;
+	}
+    }
+}
+
+bool Project::hasParentObject( QObject *o )
+{
+    for ( QObject *p = objs.first(); p; p = objs.next() ) {
+	QObject *c = p->child( o->name(), o->className() );
+	if ( c )
+	    return TRUE;
+    }
+    return FALSE;
+}
+
+QString Project::qualifiedName( QObject *o )
+{
+    QString name = o->name();
+    QObject *p = o->parent();
+    while ( p ) {
+	name.prepend( QString( p->name() ) + "." );
+	if ( objs.findRef( p ) != -1 )
+	    break;
+	p = p->parent();
+    }
+    return name;
+}
+
+bool Project::singleProjectMode() const
+{
+    return !MainWindow::self || MainWindow::self->singleProjectMode();
+}
+
+QWidget *Project::messageBoxParent() const
+{
+    return MainWindow::self;
+}
+
+void Project::designerCreated()
+{
+    for ( FormFile *ff = formfiles.first(); ff; ff = formfiles.next() ) {
+	FormWindow *fw = ff->formWindow();
+	if ( !fw || fw->mainWindow() )
+	    continue;
+	fw->setMainWindow( MainWindow::self );
+	connect( fw, SIGNAL( undoRedoChanged( bool, bool, const QString &,
+					      const QString & ) ),
+		 MainWindow::self, SLOT( updateUndoRedo( bool, bool,
+					 const QString &, const QString & ) ) );
+	fw->reparent( MainWindow::self->qWorkspace(), QPoint( 0, 0 ), FALSE );
+	QApplication::sendPostedEvents( MainWindow::self->qWorkspace(),
+					QEvent::ChildInserted );
+	fw->parentWidget()->setFixedSize( 1, 1 );
+	fw->show();
+    }
+}
+
+void Project::formOpened( FormWindow *fw )
+{
+    if ( fw->isFake() )
+	return;
+    emit newFormOpened( fw );
+}
+
+QString Project::locationOfObject( QObject *o )
+{
+    if ( !o )
+	return QString::null;
+
+    if ( MainWindow::self ) {
+	QWidgetList windows = MainWindow::self->qWorkspace()->windowList();
+	for ( QWidget *w = windows.first(); w; w = windows.next() ) {
+	    FormWindow *fw = ::qt_cast<FormWindow*>(w);
+	    SourceEditor *se = ::qt_cast<SourceEditor*>(w);
+	    if ( fw ) {
+		if ( fw->isFake() )
+		    return objectForFakeForm( fw )->name() + QString( " [Source]" );
+		else
+		    return fw->name() + QString( " [Source]" );
+	    } else if ( se ) {
+		if ( !se->object() )
+		    continue;
+		if ( se->formWindow() )
+		    return se->formWindow()->name() + QString( " [Source]" );
+		else
+		    return makeRelative( se->sourceFile()->fileName() );
+	    }
+	}
+    }
+
+    if ( ::qt_cast<SourceFile*>(o) ) {
+	for ( QPtrListIterator<SourceFile> sources = sourceFiles();
+	      sources.current(); ++sources ) {
+	    SourceFile* f = sources.current();
+	    if ( f == o )
+		return makeRelative( f->fileName() );
+	}
+    }
+
+    extern QMap<QWidget*, QString> *qwf_forms;
+    if ( !qwf_forms ) {
+	qWarning( "Project::locationOfObject: qwf_forms is NULL!" );
+	return QString::null;
+    }
+
+    QString s = makeRelative( *qwf_forms->find( (QWidget*)o ) );
+    s += " [Source]";
+    return s;
+}
+
+bool Project::hasGUI() const
+{
+    return qApp->type() != QApplication::Tty;
+}
