@@ -1,16 +1,26 @@
 /**********************************************************************
-**   Copyright (C) 2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 2000-2002 Trolltech AS.  All rights reserved.
 **
-**   msgedit.cpp
+** This file is part of Qt Linguist.
 **
-**   This file is part of Qt Linguist.
+** This file may be distributed and/or modified under the terms of the
+** GNU General Public License version 2 as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL included in the
+** packaging of this file.
 **
-**   See the file LICENSE included in the distribution for the usage
-**   and distribution terms.
+** Licensees holding valid Qt Enterprise Edition or Qt Professional Edition
+** licenses may use this file in accordance with the Qt Commercial License
+** Agreement provided with the Software.
 **
-**   The file is provided AS IS with NO WARRANTY OF ANY KIND,
-**   INCLUDING THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR
-**   A PARTICULAR PURPOSE.
+** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
+** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+**
+** See http://www.trolltech.com/gpl/ for GPL licensing information.
+** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
+**   information about Qt Commercial License Agreements.
+**
+** Contact info@trolltech.com if any conditions of this licensing are
+** not clear to you.
 **
 **********************************************************************/
 
@@ -19,12 +29,18 @@
   This is the right panel of the main window.
 */
 
+#include "msgedit.h"
+
+#include "trwindow.h"
+#include "phraselv.h"
+#include "simtexth.h"
+
 #include <qapplication.h>
 #include <qcheckbox.h>
 #include <qclipboard.h>
 #include <qlabel.h>
 #include <qlayout.h>
-#include <qmultilineedit.h>
+#include <qtextedit.h>
 #include <qpalette.h>
 #include <qpushbutton.h>
 #include <qstring.h>
@@ -34,31 +50,21 @@
 #include <qmainwindow.h>
 #include <qheader.h>
 #include <qregexp.h>
-
 #include <qdockarea.h>
 #include <qdockwindow.h>
 #include <qscrollview.h>
 #include <qfont.h>
 #include <qaccel.h>
-
 #include <private/qrichtext_p.h>
 
-#include "trwindow.h"
-
-#include "msgedit.h"
-#include "phraselv.h"
-
-#include "simtexth.h"
-
 static const int MaxCandidates = 5;
 
-class MED : public QMultiLineEdit
+class MED : public QTextEdit
 {
 public:
     MED( QWidget *parent, const char *name = 0 )
-	: QMultiLineEdit( parent, name ) {}
+	: QTextEdit( parent, name ) {}
 
-    void del() { QMultiLineEdit::del(); }
     int cursorX() const { return textCursor()->x(); }
     int cursorY() const { return textCursor()->paragraph()->rect().y() +
 			         textCursor()->y(); }
@@ -195,11 +201,13 @@ EditorPage::EditorPage( QWidget * parent, const char * name )
     QPalette p = palette();
     p.setColor( QPalette::Active, QColorGroup::Base, QColor( white ) );
     p.setColor( QPalette::Inactive, QColorGroup::Base, QColor( white ) );
-
+    p.setColor( QPalette::Disabled, QColorGroup::Base, QColor( white ) );
     p.setColor( QPalette::Active, QColorGroup::Background,
 		p.active().color( QColorGroup::Base ) );
     p.setColor( QPalette::Inactive, QColorGroup::Background,
 		p.inactive().color( QColorGroup::Base ) );
+    p.setColor( QPalette::Disabled, QColorGroup::Background,
+		p.disabled().color( QColorGroup::Base ) );
     setPalette( p );
 
     srcTextLbl = new QLabel( tr("Source text"), this, "source text label" );
@@ -230,10 +238,8 @@ EditorPage::EditorPage( QWidget * parent, const char * name )
     cmtText->setHScrollBarMode( QScrollView::AlwaysOff );
     cmtText->setVScrollBarMode( QScrollView::AlwaysOff );
     p = cmtText->palette();
-    p.setColor( QPalette::Active, QColorGroup::Base,
- 		QColor( 236,245,255 ) );
-    p.setColor( QPalette::Inactive, QColorGroup::Base,
- 		QColor( 236,245,255 ) );
+    p.setColor( QPalette::Active, QColorGroup::Base, QColor( 236,245,255 ) );
+    p.setColor( QPalette::Inactive, QColorGroup::Base, QColor( 236,245,255 ) );
     cmtText->setPalette( p );
     connect( cmtText, SIGNAL(textChanged()), SLOT(handleCommentChanges()) );
 
@@ -415,11 +421,12 @@ MessageEditor::MessageEditor( MetaTranslator * t, QWidget * parent,
     srcTextList->setShowSortIndicator( TRUE );
     srcTextList->setAllColumnsShowFocus( TRUE );
     srcTextList->setSorting( 0 );
-    srcTextList->addColumn( tr("Done"), 40 );
+    QFontMetrics fm( font() );
+    srcTextList->addColumn( tr("Done"), fm.width( tr("Done") ) + 10 );
     srcTextList->addColumn( tr("Source text") );
-    srcTextList->addColumn( tr("Translation"), 200 );
-    srcTextList->setColumnWidthMode( 1, QListView::Maximum );
+    srcTextList->addColumn( tr("Translation"), 300 );
     srcTextList->setColumnAlignment( 0, Qt::AlignCenter );
+    srcTextList->setColumnWidthMode( 1, QListView::Manual );
     srcTextList->header()->setStretchEnabled( TRUE, 1 );
     srcTextList->setMinimumSize( QSize( 50, 50 ) );
     srcTextList->setHScrollBarMode( QScrollView::AlwaysOff );
@@ -531,56 +538,67 @@ void MessageEditor::toggleFinished()
     emit finished( itemFinished );
 }
 
-bool MessageEditor::eventFilter( QObject * o, QEvent * e )
+bool MessageEditor::eventFilter( QObject *o, QEvent *e )
 {
     static uchar doFocusChange = FALSE;
 
     // Handle keypresses in the message editor - scroll the view if the current
     // line is hidden.
-    if ( o->inherits("QMultiLineEdit") ) {
-	MED * ed = (MED *) o;
-	QKeyEvent * ke;
-	int k;
-	if ( e->type() == QEvent::KeyPress ) {
-	    ke = (QKeyEvent *) e;
-	    k  = ke->key();
-	    // Hardcode the Tab key to do focus changes when pressed
-	    // inside the editor
-	    if ( k == Key_BackTab && doFocusChange ) {
-		emit focusSourceList();
-		doFocusChange = FALSE;
-		return TRUE;
-	    } else if ( k == Key_Tab && doFocusChange ) {
-		emit focusPhraseList();
-		doFocusChange = FALSE;
-		return TRUE;
-	    }
-	} else if ( e->type() == QEvent::KeyRelease ) {
-	    ke = (QKeyEvent *) e;
-	    k  = ke->key();
+    if ( e->type() == QEvent::KeyPress || e->type() == QEvent::KeyRelease ) {
+	QKeyEvent * ke = (QKeyEvent*)e;
+	const int k = ke->key();
 
-	    if ( (k == Key_Up) && (ed->cursorY() < 10) )
-		sv->verticalScrollBar()->subtractLine();
-	    else if ( (k == Key_Down) && (ed->cursorY() >= ed->height() - 20) )
-		sv->verticalScrollBar()->addLine();
-	    else if ( (k == Key_PageUp) && (ed->cursorY() < 10) )
-		sv->verticalScrollBar()->subtractPage();
-	    else if ( (k == Key_PageDown) && (ed->cursorY() >=
-					      (ed->height() - 50)) )
-		sv->verticalScrollBar()->addPage();
-	    else
-		sv->ensureVisible( sw->margin() + ed->x() + ed->cursorX(),
-				   sw->margin() + ed->y() + ed->cursorY() );
+	if ( ::qt_cast<QTextEdit*>(o) ) {
+	    if ( e->type() == QEvent::KeyPress ) {
+		// Hardcode the Tab key to do focus changes when pressed
+		// inside the editor
+		if ( doFocusChange ) {
+		    if ( k == Key_BackTab ) {
+			emit focusSourceList();
+			doFocusChange = FALSE;
+			return TRUE;
+		    } else if ( k == Key_Tab ) {
+			emit focusPhraseList();
+			doFocusChange = FALSE;
+			return TRUE;
+		    }
+		}
+	    } else if ( e->type() == QEvent::KeyRelease ) {
+		MED * ed = (MED *) o;
+		switch( k ) {
+		case Key_Up:
+		    if (ed->cursorY() < 10)
+			sv->verticalScrollBar()->subtractLine();
+		    break;
+
+		case Key_Down:
+		    if (ed->cursorY() >= ed->height() - 20)
+			sv->verticalScrollBar()->addLine();
+		    break;
+
+		case Key_PageUp:
+		    if (ed->cursorY() < 10)
+			sv->verticalScrollBar()->subtractPage();
+		    break;
+
+		case Key_PageDown:
+		    if (ed->cursorY() >= ed->height() - 50)
+			sv->verticalScrollBar()->addPage();
+		    break;
+		default:
+		    sv->ensureVisible( sw->margin() + ed->x() + ed->cursorX(),
+				       sw->margin() + ed->y() + ed->cursorY() );
+		    break;
+		}
+	    }
+	    doFocusChange = TRUE;
+	} else if ( ::qt_cast<QListView*>(o) ) {
+	    // handle the ESC key in the list views
+	    if ( e->type() == QEvent::KeyRelease && k == Key_Escape )
+		editorPage->translationMed->setFocus();
 	}
-	doFocusChange = TRUE;
     }
-    if ( o->inherits("QListView") ) {
-	// handle the ESC key in the list views
-	if ( e->type() == QEvent::KeyRelease &&
-	     ((QKeyEvent *) e)->key() == Key_Escape )
-	    editorPage->translationMed->setFocus();
-    }
-    return FALSE;
+    return QWidget::eventFilter( o, e );
 }
 
 void MessageEditor::updatePageHeight( int height )
@@ -660,7 +678,7 @@ void MessageEditor::showMessage( const QString& text,
     for ( p = phrases.begin(); p != phrases.end(); ++p )
  	(void) new PhraseLVI( phraseLv, *p );
 
-    if ( doGuesses ) {
+    if ( doGuesses && !sourceText.isEmpty() ) {
 	CandidateList cl = similarTextHeuristicCandidates( tor,
 							   sourceText.latin1(),
 							   MaxCandidates );
@@ -669,7 +687,7 @@ void MessageEditor::showMessage( const QString& text,
 	while ( it != cl.end() ) {
 	    QString def;
 	    if ( n < 9 )
-		def = tr( "Guess (Ctrl+%1)" ).arg( n + 1 );
+		def = tr( "Guess (%1)" ).arg( QString(QKeySequence(CTRL | (Key_0 + (n + 1)))) );
 	    else
 		def = tr( "Guess" );
 	    (void) new PhraseLVI( phraseLv,
